<?php require_once(dirname(__FILE__).'/../jheader.inc.php'); ?>
var sbColors = new function()
{
  this.ColorFromHSV = function(hue, sat, val)
  {
    var color = new sbColor();
    color.SetHSV(hue,sat,val);
    return color;
  }

  this.ColorFromRGB = function(r, g, b)
  {
    var color = new sbColor();
    color.SetRGB(r,g,b);
    return color;
  }

  this.ColorFromHex = function(hexStr)
  {
    var color = new sbColor();
    color.SetHexString(hexStr);
    return color;
  }

  function sbColor()
  {
    var red = 0;
    var green = 0;
    var blue = 0;
   
    var hue = 0;
    var saturation = 0;
    var value = 0;
     
    this.SetRGB = function(r, g, b)
    {
      if (isNaN(r) || isNaN(g) || isNaN(b))
        return false;
       
      r = r/255.0;
      red = r> 1 ? 1 : r <0 ? 0 : r;
      g = g/255.0;
      green = g> 1 ? 1 : g <0 ? 0 : g;
      b = b/255.0;
      blue = b> 1 ? 1 : b <0 ? 0 : b;
     
      calculateHSV();
      return true;
    }
   
    this.Red = function()
    { return Math.round(red*255); }
   
    this.Green = function()
    { return Math.round(green*255); }
   
    this.Blue = function()
    { return Math.round(blue*255); }
   
    this.SetHSV = function(h, s, v)
    {
      if (isNaN(h) || isNaN(s) || isNaN(v))
        return false;
       
      hue = (h>= 360) ? 359.99 : (h <0) ? 0 : h;
      saturation = (s> 1) ? 1 : (s <0) ? 0 : s;
      value = (v> 1) ? 1 : (v <0) ? 0 : v;
      calculateRGB();
      return true;
    }
     
    this.Hue = function()
    { return hue; }
     
    this.Saturation = function()
    { return saturation; }
     
    this.Value = function()
    { return value; }
     
    this.SetHexString = function(hexString)
    {
      if(hexString == null || typeof(hexString) != "string")
        return false;
       
      if (hexString.substr(0, 1) == '#')
        hexString = hexString.substr(1);
       
      if(hexString.length != 6)
        return false;
         
      var r = parseInt(hexString.substr(0, 2), 16);
      var g = parseInt(hexString.substr(2, 2), 16);
      var b = parseInt(hexString.substr(4, 2), 16);
     
      return this.SetRGB(r,g,b);
    }
     
    this.HexString = function()
    {
      var rStr = this.Red().toString(16);
      if (rStr.length == 1)
        rStr = '0' + rStr;
      var gStr = this.Green().toString(16);
      if (gStr.length == 1)
        gStr = '0' + gStr;
      var bStr = this.Blue().toString(16);
      if (bStr.length == 1)
        bStr = '0' + bStr;
      return ('#' + rStr + gStr + bStr).toUpperCase();
    }
   
    this.Complement = function()
    {
      var newHue = (hue>= 180) ? hue - 180 : hue + 180;
      var newVal = (value * (saturation - 1) + 1);
      var newSat = (value*saturation) / newVal;
      var newColor = new sbColor();
      newColor.SetHSV(newHue, newSat, newVal);
      return newColor;
    }
   
    function calculateHSV()
    {
      var max = Math.max(Math.max(red, green), blue);
      var min = Math.min(Math.min(red, green), blue);
     
      value = max;
     
      saturation = 0;
      if(max != 0)
        saturation = 1 - min/max;
       
      hue = 0;
      if(min == max)
        return;
     
      var delta = (max - min);
      if (red == max)
        hue = (green - blue) / delta;
      else if (green == max)
        hue = 2 + ((blue - red) / delta);
      else
        hue = 4 + ((red - green) / delta);
      hue = hue * 60;
      if(hue <0)
        hue += 360;
    }
   
    function calculateRGB()
    {
      red = value;
      green = value;
      blue = value;
     
      if(value == 0 || saturation == 0)
        return;
     
      var tHue = (hue / 60);
      var i = Math.floor(tHue);
      var f = tHue - i;
      var p = value * (1 - saturation);
      var q = value * (1 - saturation * f);
      var t = value * (1 - saturation * (1 - f));
      switch(i)
      {
        case 0:
          red = value; green = t; blue = p;
          break;
        case 1:
          red = q; green = value; blue = p;
          break;
        case 2:
          red = p; green = value; blue = t;
          break;
        case 3:
          red = p; green = q; blue = value;
          break;
        case 4:
          red = t; green = p; blue = value;
          break;
        default:
          red = value; green = p; blue = q;
          break;
      }
    }
  }
};

function sbColorGetMousePos(eventObj)
{
  eventObj = eventObj ? eventObj : window.event;
  var pos;
  if(isNaN(eventObj.layerX))
    pos = new sbPosition(eventObj.offsetX, eventObj.offsetY);
  else
    pos = new sbPosition(eventObj.layerX, eventObj.layerY);
  return sbColorCorrectOffset(pos, sbColorPointerOffset, true);
}

function sbColorCorrectOffset(pos, offset, neg)
{
  if(neg)
    return pos.Subtract(offset);
  return pos.Add(offset);
}

function sbColorArrowsDown(e, arrows)
{
  var pos = sbColorGetMousePos(e);
 
  if(sbEventTarget(e) == arrows)
    pos.Y += parseInt(arrows.style.top);
 
  pos = sbColorCorrectOffset(pos, sbColorArrowsOffset, true);
  pos = pos.Bound(sbColorArrowsLowBounds, sbColorArrowsUpBounds);
  pos.Apply(arrows);
 
  sbColorArrowsMoved(pos);
}

function sbColorArrowsMoved(pos, element)
{
  pos = sbColorCorrectOffset(pos, sbColorArrowsOffset, false);
  sbColorCurrent.SetHSV((256 - pos.Y)*359.99/255, sbColorCurrent.Saturation(), sbColorCurrent.Value());
  sbColorChanged("sb_arrows_img");
}

function sbColorCircleDown(e, circle)
{
  var pos = sbColorGetMousePos(e);
 
  if(sbEventTarget(e) == circle)
  {
    pos.X += parseInt(circle.style.left);
    pos.Y += parseInt(circle.style.top);
  }
 
  pos = sbColorCorrectOffset(pos, sbColorCircleOffset, true);
  pos = pos.Bound(sbColorCircleLowBounds, sbColorCircleUpBounds);
  pos.Apply(circle);
   
  sbColorCircleMoved(pos);
}

function sbColorCircleMoved(pos, element)
{
  pos = sbColorCorrectOffset(pos, sbColorCircleOffset, false);
  sbColorCurrent.SetHSV(sbColorCurrent.Hue(), 1-pos.Y/255.0, pos.X/255.0);
  sbColorChanged("sb_circle_img");
}

function sbColorChanged(source)
{
  sbGetE("sb_color").value = sbColorCurrent.HexString();
  sbGetE("spin_red_box").value = sbColorCurrent.Red();
  sbGetE("spin_green_box").value = sbColorCurrent.Green();
  sbGetE("spin_blue_box").value = sbColorCurrent.Blue();
  sbGetE("spin_hue_box").value = Math.round(sbColorCurrent.Hue());
  
  var str = (sbColorCurrent.Saturation()*100).toString();
  if(str.length > 4)
    str = str.substr(0,4);
  sbGetE("spin_saturation_box").value = str;
  
  str = (sbColorCurrent.Value()*100).toString();
  if(str.length > 4)
    str = str.substr(0,4);
  sbGetE("spin_value_box").value = str;
 
  if(source == "sb_arrows_img" || source == "box")
    sbGetE("sb_gradient_box").style.backgroundColor = sbColors.ColorFromHSV(sbColorCurrent.Hue(), 1, 1).HexString();
   
  if(source == "box")
  {
    var el = sbGetE("sb_arrows_img");
    el.style.top = (256 - sbColorCurrent.Hue()*255/359.99 - sbColorArrowsOffset.Y) + 'px';
    var pos = new sbPosition(sbColorCurrent.Value()*255, (1 - sbColorCurrent.Saturation())*255);
    pos = sbColorCorrectOffset(pos, sbColorCircleOffset, true);
    pos.Apply("sb_circle_img");
    sbColorEndMovement();
  }
 
  sbGetE("sb_quick_color").style.backgroundColor = sbColorCurrent.HexString();
}

function sbColorEndMovement()
{
  sbGetE("sb_static_color").style.backgroundColor = sbColorCurrent.HexString();
}

function sbHexBoxChanged(e)
{
  sbColorCurrent.SetHexString(sbGetE("sb_color").value);
  sbColorChanged("box");
}

function sbRedBoxChanged(e)
{
  sbColorCurrent.SetRGB(parseInt(sbGetE("spin_red_box").value), sbColorCurrent.Green(), sbColorCurrent.Blue());
  sbColorChanged("box");
}

function sbGreenBoxChanged(e)
{
  sbColorCurrent.SetRGB(sbColorCurrent.Red(), parseInt(sbGetE("spin_green_box").value), sbColorCurrent.Blue());
  sbColorChanged("box");
}

function sbBlueBoxChanged(e)
{
  sbColorCurrent.SetRGB(sbColorCurrent.Red(), sbColorCurrent.Green(), parseInt(sbGetE("spin_blue_box").value));
  sbColorChanged("box");
}

function sbHueBoxChanged(e)
{
  sbColorCurrent.SetHSV(parseFloat(sbGetE("spin_hue_box").value), sbColorCurrent.Saturation(), sbColorCurrent.Value());
  sbColorChanged("box");
}

function sbSaturationBoxChanged(e)
{
  sbColorCurrent.SetHSV(sbColorCurrent.Hue(), parseFloat(sbGetE("spin_saturation_box").value)/100.0, sbColorCurrent.Value());
  sbColorChanged("box");
}

function sbValueBoxChanged(e)
{
  sbColorCurrent.SetHSV(sbColorCurrent.Hue(), sbColorCurrent.Saturation(), parseFloat(sbGetE("spin_value_box").value)/100.0);
  sbColorChanged("box");
}

function sbColorChoose()
{
    sbReturnValue(sbGetE("sb_color").value);
}

function sbColorDrop()
{
	sbReturnValue("");
}

var sbColorArrowsLowBounds = new sbPosition(0, -4);
var sbColorArrowsUpBounds = new sbPosition(0, 251);
var sbColorCircleLowBounds = new sbPosition(-4, -4);
var sbColorCircleUpBounds = new sbPosition(251, 251);

var sbColorPointerOffset = new sbPosition(0, navigator.userAgent.indexOf("Firefox")>= 0 ? 1 : 0);
var sbColorCircleOffset = new sbPosition(4, 4);
var sbColorArrowsOffset = new sbPosition(0, 5);

<?php require_once(dirname(__FILE__).'/../jfooter.inc.php'); ?>