<?php
/**
 * Подключение сжатия страниц системы и инициализация основных библиотек ядра системы
 *
 * В данном файле идет отсылка следующих заголовков браузеру:
 * <code>
 * // Запрещаем кэширование страницы барузером и проксей
 * header('ETag: PUB' . time());
 * header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
 * header('Last-Modified: '.date('r'));
 * header('Cache-Control: no-store, no-cache, must-revalidate, proxy-revalidate, post-check=0, pre-check=0', false);
 * header('Pragma: no-cache');
 *
 * // Cообщаем браузеру кодировку системы
 * header('Content-Language: '.SB_CMS_LANG.', en');
 * header('Content-Type: text/html; charset='.strtolower(SB_CHARSET));
 *
 * // Если браузер поддерживает GZip-компрессию, то сообщаем о том, что контент сжат
 * header('Content-Encoding: gzip');
 * </code>
 *
 * Также здесь мы устанавливаем перехват ошибок PHP нашей функцией, чтобы в дальнейшем
 * записывать ошибки в системный журнал:
 * <code>
 * set_error_handler('sb_php_error_reporting');
 * </code>
 *
 * И, наконец, после подключения всех библиотечных файлов, стартуем сессию:
 * <code>
 * session_start();
 * </code>
 *
 * @see SB_CMS_LANG, SB_CHARSET
 * @see function sbPHPErrorReporting()
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Core
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */

ob_start();

// убираем GET-параметры модулей
// событие вызванное
if (isset($_SERVER['QUERY_STRING']))
{
	$get_str = preg_replace('/[?&]?event=[A-Z0-9_\-]+/i', '', $_SERVER['QUERY_STRING']);
	// пейджирование
	$get_str = preg_replace('/[?&]?page_[0-9A-Z]+=[0-9]+/i', '', $get_str);
	// макеты меню и разделов
	$get_str = preg_replace('/[?&]?add_level=[0-9]+/i', '', $get_str);
	
	$_SERVER['QUERY_STRING'] = trim($get_str, '&?');
}
else
{
	$_SERVER['QUERY_STRING'] = '';
}

// Вспомогательные функции ядра системы
require_once(dirname(__FILE__).'/../lib/sbFunctions.inc.php');
require_once(dirname(__FILE__).'/init.inc.php');

/*
 * Запрещаем кэширование страницы барузером и проксей
 */
header('ETag: PUB' . time());
header('Expires: '.date('r'));
header('Cache-Control: no-store, no-cache, must-revalidate, proxy-revalidate, post-check=0, pre-check=0');
@session_cache_limiter('nocache');

/*
 * Сообщаем браузеру кодировку страницы
 */
header('Content-Language: '.SB_CMS_LANG.', en');
header('Content-Type: text/html; charset='.strtolower(SB_CHARSET));
header('Content-Script-Type: text/javascript; charset='.strtolower(SB_CHARSET));

/*
 * Подключаем обязательные файлы ядра системы
 */
// Основные константы системы
require_once(SB_CMS_KERNEL_PATH.'/data.inc.php');
// Языковые константы ядра системы
require_once(SB_CMS_LANG_PATH.'/kernel.lng.php');
// Определяем браузер
require_once(SB_CMS_LIB_PATH.'/sbCheckBrowser.inc.php');
// Подключение библиотеки работы с файловой системой
require_once(SB_CMS_LIB_PATH.'/sbVfs.inc.php');

/*
 * Устанавливаем перехват ошибок PHP нашей функцией, чтобы в дальнейшем
 * записывать ошибки в системный журнал.
 */

set_error_handler('sb_php_error_reporting');
register_shutdown_function('sb_php_shutdown');

/*
 * Если все-таки включено экранирование GPC-массивов (мы пытаемся отключить его в config.inc.php через ini_set),
 * то убираем экранирование из значений этих массивов
 */
if (get_magic_quotes_gpc())
{
    sb_array_stripslashes($_POST);
    sb_array_stripslashes($_GET);
    sb_array_stripslashes($_COOKIE);
    sb_array_stripslashes($_FILES);
    sb_array_stripslashes($_REQUEST);
}

// Подключение библиотеки работы с СУБД
require_once(SB_CMS_LIB_PATH.'/sbSql.inc.php');

// Подключение библиотеки работы с модулями системы
require_once(SB_CMS_LIB_PATH.'/sbPlugins.inc.php');

if (class_exists('sbProfiler'))
{
	sbProfiler::init();
}

// Считываем настройки системы
sbPlugins::initSettings();

// Подключение библиотеки управления авторизацией пользователей системы и сессиями пользователей системы
require_once(SB_CMS_LIB_PATH.'/sbAuth.inc.php');

// Стартуем сессию
@session_name('SBSESSID');
if (isset($_POST['SBSESSID']))
{
	@session_id($_POST['SBSESSID']);
}
@session_start();

if (sbPlugins::getSetting('sb_debug_mode') == 1)
{
	restore_error_handler();
	@ini_set('display_errors', '1');
	$GLOBALS['sbSql']->mShowErrors = true;	
}

$timeout = sbPlugins::getSetting('sb_session_timeout');
if (is_null($timeout) || $timeout <= 0)
{
    $timeout = 60;
}

@ini_set('session.gc_maxlifetime', (string) ($timeout * 60) + 600);

// Считываем настройки пользователя
sbPlugins::initUserSettings();
?>