<?php
/**
 * Взаимодействие с публикацией из Word через XMLRPC
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Core
 * @copyright Copyright (c) 2009, OOO "СИБИЭС Групп"
 */

#TODO: Сделать возможность задавать дату

require_once(dirname(__FILE__).'/header.inc.php');

header('Connection: close');
header('Content-Type: text/xml; charset='.strtolower(SB_CHARSET));

$_COOKIE = array();

// читаем RAW_POST_DATA
if ( !isset( $HTTP_RAW_POST_DATA ) )
{
	$HTTP_RAW_POST_DATA = @file_get_contents( 'php://input' );
}
$HTTP_RAW_POST_DATA = trim($HTTP_RAW_POST_DATA);

// создаем XMLRPC - сервер
if (!function_exists('xmlrpc_server_create'))
{
	sb_xmlrpc_error(405, XMLRPC_NO_PHP_EXT);
}

$xmlrpc_server = xmlrpc_server_create ();
if (!is_resource($xmlrpc_server))
{
	sb_xmlrpc_error(405, XMLRPC_SERVER_ERROR);
}

// регистрируем методы
$error = false;

// Blogger API
$error = $error || !xmlrpc_server_register_method($xmlrpc_server, 'blogger.getUsersBlogs', 'sb_xmlrpc_init');
$error = $error || !xmlrpc_server_register_method($xmlrpc_server, 'blogger.getUserInfo', 'sb_xmlrpc_user_info');
$error = $error || !xmlrpc_server_register_method($xmlrpc_server, 'blogger.deletePost', 'sb_xmlrpc_delete_post');

// MetaWeblog API
$error = $error || !xmlrpc_server_register_method($xmlrpc_server, 'metaWeblog.getUsersBlogs', 'sb_xmlrpc_init');
$error = $error || !xmlrpc_server_register_method($xmlrpc_server, 'metaWeblog.getCategories', 'sb_xmlrpc_categs');
$error = $error || !xmlrpc_server_register_method($xmlrpc_server, 'metaWeblog.getRecentPosts', 'sb_xmlrpc_get_posts');
$error = $error || !xmlrpc_server_register_method($xmlrpc_server, 'metaWeblog.getPost', 'sb_xmlrpc_get_post');
$error = $error || !xmlrpc_server_register_method($xmlrpc_server, 'metaWeblog.editPost', 'sb_xmlrpc_edit_post');
$error = $error || !xmlrpc_server_register_method($xmlrpc_server, 'metaWeblog.newPost', 'sb_xmlrpc_new_post');
$error = $error || !xmlrpc_server_register_method($xmlrpc_server, 'metaWeblog.newMediaObject', 'sb_xmlrpc_new_media');

if ($error)
{
	xmlrpc_server_destroy ($xmlrpc_server);

	sb_xmlrpc_error(405, XMLRPC_REGISTER_METHOD_ERROR);
}

// вызываем нужный метод
echo xmlrpc_server_call_method ($xmlrpc_server, $HTTP_RAW_POST_DATA, null,
								array (	'output_type' => 'xml',
										'encoding' => SB_CHARSET,
										'version' => 'xmlrpc',
										'escaping' => 'cdata'
									  )
							    );
// убиваем сервер
xmlrpc_server_destroy ($xmlrpc_server);

function sb_xmlrpc_init($method_name, $params, $app_data)
{
	$auth_res = sb_xmlrpc_login($params[1], $params[2]);

	if (!$auth_res)
	{
		sb_xmlrpc_error (401, XMLRPC_LOGIN_ERROR);
		return false;
	}

	$result = array('url' => SB_DOMAIN, 'blogid' => SB_COOKIE_DOMAIN, 'blogName' => sprintf(XMLRPC_NEWS_BLOGNAME, SB_COOKIE_DOMAIN));

	return array($result);
}

function sb_xmlrpc_categs($method_name, $params, $app_data)
{
	$auth_res = sb_xmlrpc_login($params[1], $params[2]);

	if (!$auth_res)
	{
		sb_xmlrpc_error (401, XMLRPC_LOGIN_ERROR);
		return false;
	}

	$res = sql_query('SELECT cat_id, cat_title, cat_level, cat_rights FROM sb_categs WHERE cat_ident=? ORDER BY cat_left', 'pl_news');
	if (!$res)
	{
		sb_xmlrpc_error (403, XMLRPC_NEWS_CATEGS_ERROR);
		return false;
	}

	$result = array();
	foreach ($res as $value)
	{
		list($cat_id, $cat_title, $cat_level, $cat_rights) = $value;

		if ($auth_res['u_admin'] != 1 && !sb_xmlrpc_check_categs_rights($cat_rights, $auth_res))
			continue;

		$cat_title = '['.$cat_id.'] '.str_repeat(' ', $cat_level * 3).trim($cat_title);

		$result[] = array('description' => $cat_title, 'title' => $cat_title);
	}

	return $result;
}

function sb_xmlrpc_user_info($method_name, $params, $app_data)
{
	$auth_res = sb_xmlrpc_login($params[1], $params[2]);

	if (!$auth_res)
	{
		sb_xmlrpc_error (401, XMLRPC_LOGIN_ERROR);
		return false;
	}

	$name = explode(' ', $auth_res['u_id']);
	$firstname = $name[0];
	$lastname = isset($name[1]) ? $name[1] : '';

	$result = array(
	'userid' => $auth_res['u_id'],
	'firstname' => $firstname,
	'lastname' => $lastname,
	'url' => SB_DOMAIN,
	'email' => $auth_res['u_email'],
	'nickname' => $params[1]
	);

	return $result;
}

function sb_xmlrpc_get_posts($method_name, $params, $app_data)
{
	$auth_res = sb_xmlrpc_login($params[1], $params[2]);

	if (!$auth_res)
	{
		sb_xmlrpc_error (401, XMLRPC_LOGIN_ERROR);
		return;
	}

	$res = sql_query('SELECT cat_id, cat_title, cat_rights FROM sb_categs WHERE cat_ident=? ORDER BY cat_left', 'pl_news');
	if (!$res)
	{
		sb_xmlrpc_error (403, XMLRPC_NEWS_CATEGS_ERROR);
		return;
	}

	$cat_titles = array();
	$cat_ids = array();
	foreach ($res as $value)
	{
		list($cat_id, $cat_title, $cat_rights) = $value;

		if ($auth_res['u_admin'] != 1 && !sb_xmlrpc_check_categs_rights($cat_rights, $auth_res, 'elems_edit'))
			continue;

		$cat_titles[$cat_id] = $cat_title;
		$cat_ids[] = $cat_id;
	}

	if (count($cat_ids) == 0)
	{
		sb_xmlrpc_error (403, XMLRPC_NEWS_CATEGS_ERROR);
		return;
	}

	$res = sql_query('SELECT STRAIGHT_JOIN l.link_cat_id, n.n_id, n.n_title, n.n_date, n.n_short, n.n_full, n.n_active
                      	FROM sb_news n, sb_catlinks l
                        WHERE l.link_cat_id IN (?a) AND  l.link_el_id = n.n_id
                        GROUP BY n.n_id ORDER BY n.n_date DESC LIMIT '.intval(sbPlugins::getSetting('sb_xmlrpc_edit_limit')), $cat_ids);

	if (!$res)
	{
		return array();
	}

	$result = array();
	foreach ($res as $value)
	{
		list($cat_id, $n_id, $n_title, $n_date, $n_short, $n_full, $n_active) = $value;

		$n_date = sb_date('Ymd\TH:i:s', $n_date);
		xmlrpc_set_type($n_date, 'datetime');

		$result[] = array(
			'postid' => $n_id,
			'dateCreated' => $n_date,
			'title' => $n_title,
		    'description' => (trim($n_short) != '' ? $n_short.'<p>+++</p>'.$n_full : $n_full),
		    'categories' => array('['.$cat_id.'] '.$cat_titles[$cat_id]),
			'publish' => ($n_active == 1)
			);
	}

	return $result;
}

function sb_xmlrpc_get_post($method_name, $params, $app_data)
{
	$auth_res = sb_xmlrpc_login($params[1], $params[2]);

	if (!$auth_res)
	{
		sb_xmlrpc_error (401, XMLRPC_LOGIN_ERROR);
		return;
	}

	$res = sql_query('SELECT STRAIGHT_JOIN c.cat_id, c.cat_title, c.cat_level, c.cat_rights, n.n_id, n.n_title, n.n_date, n.n_short, n.n_full, n.n_active
                      	FROM sb_news n, sb_catlinks l, sb_categs c
                        WHERE n.n_id=?d
                        	AND l.link_el_id=n.n_id
                        	AND l.link_cat_id = c.cat_id
                        	AND c.cat_ident = ?', $params[0], 'pl_news');

	if (!$res)
	{
		sb_xmlrpc_error (401, XMLRPC_NEWS_WAS_DELETED);
		return;
	}

	$cat_titles = array();
	if ($auth_res['u_admin'] != 1)
	{
		$access = false;
		foreach ($res as $value)
		{
			list($cat_id, $cat_title, $cat_level, $cat_rights, $n_id, $n_title, $n_date, $n_short, $n_full, $n_active) = $value;

			$access = $access || sb_xmlrpc_check_categs_rights($cat_rights, $auth_res, 'elems_edit');

			$cat_titles[] = '['.$cat_id.'] '.str_repeat(' ', $cat_level * 3).trim($cat_title);
		}

		if (!$access)
		{
			sb_xmlrpc_error (403, XMLRPC_NEWS_EDIT_ERROR);
			return;
		}
	}
	else
	{
		foreach ($res as $value)
		{
			list($cat_id, $cat_title, $cat_level, $cat_rights, $n_id, $n_title, $n_date, $n_short, $n_full, $n_active) = $value;

			$cat_titles[] = '['.$cat_id.'] '.str_repeat(' ', $cat_level * 3).trim($cat_title);
		}
	}

	$result = array();

	$n_date = sb_date('Ymd\TH:i:s', $n_date);
	xmlrpc_set_type($n_date, 'datetime');

	$result[] = array(
			'postid' => $n_id,
			'dateCreated' => $n_date,
			'title' => $n_title,
		    'description' => (trim($n_short) != '' ? $n_short.'<p>+++</p>'.$n_full : $n_full),
			'publish' => ($n_active == 1),
			'categories' => $cat_titles
		);

	return $result;
}

function sb_xmlrpc_edit_post($method_name, $params, $app_data)
{
	$auth_res = sb_xmlrpc_login($params[1], $params[2]);

	if (!$auth_res)
	{
		sb_xmlrpc_error (401, XMLRPC_LOGIN_ERROR);
		return;
	}

	$res = sql_param_query('SELECT STRAIGHT_JOIN c.cat_id, c.cat_rights, l.link_id, n.n_title
                      	FROM sb_news n, sb_catlinks l, sb_categs c
                        WHERE n.n_id=?d
                        	AND l.link_el_id=n.n_id
                        	AND l.link_cat_id = c.cat_id
                        	AND c.cat_ident=?', $params[0], 'pl_news');

	if (!$res)
	{
		sb_xmlrpc_error (403, XMLRPC_NEWS_WAS_DELETED);
		return;
	}

	$access = false;
	$cat_accesses = array();
	$link_ids = array();

	foreach ($res as $value)
	{
		list($cat_id, $cat_rights, $link_id, $n_title) = $value;

		$cat_access = ($auth_res['u_admin'] == 1 || sb_xmlrpc_check_categs_rights($cat_rights, $auth_res, 'elems_edit'));
		$cat_accesses[$cat_id] = $cat_access;

		if (!in_array($link_id, $link_ids))
		{
			$link_ids[] = $link_id;
		}

		if (!$access)
		{
			$access = $cat_access;
		}
	}

	if (!$access)
	{
		sb_xmlrpc_error (403, XMLRPC_NEWS_EDIT_ERROR);
		return;
	}

	$content = $params[3];
	$cat_titles = array();

	if (isset($content['categories']) && is_array($content['categories']) && count($content['categories']) > 0)
	{
		$i = -1;
		foreach ($content['categories'] as $value)
		{
			$value = trim($value);
			$cat_id = $i--;

			if (sb_substr($value, 0, 1) == '[')
			{
				$pos = sb_strpos($value, ']');
				if ($pos)
				{
					$cat_id = intval(sb_substr($value, 1, $pos - 1));
				}
			}

			if ($cat_id < 0)
			{
				$cat_titles[$cat_id] = trim($value);
			}
			else
			{
				$cat_titles[$cat_id] = trim(sb_substr($value, $pos + 1));
			}
		}
	}

	if (count($cat_titles) > 0)
	{
		$res = sql_query('SELECT cat_id FROM sb_categs WHERE cat_ident=? ORDER BY cat_left LIMIT 1', 'pl_news');
		if ($res)
			list($root_cat_id) = $res[0];
		else
			$root_cat_id = -1;

		$cat_ids = array();
		foreach ($cat_titles as $cat_id => $cat_title)
		{
			if ($cat_id > 0)
			{
				if ($auth_res['u_admin'] != 1)
				{
					if (isset($cat_accesses[$cat_id]) && $cat_accesses[$cat_id])
					{
						$cat_ids[] = $cat_id;
					}
					else
					{
						$res = sql_query('SELECT cat_rights FROM sb_categs WHERE cat_id=?d', $cat_id);
						if ($res && sb_xmlrpc_check_categs_rights($res[0][0], $auth_res, 'elems_edit'))
						{
							$cat_ids[] = $cat_id;
						}
					}

					if (!sb_xmlrpc_check_plugin_rights($auth_res['u_groups'], 'categs_edit'))
						continue;
				}
				else
				{
					$cat_ids[] = $cat_id;
				}

				// Редактирование раздела
				sql_query('UPDATE sb_categs SET cat_title=? WHERE cat_id=?d', $cat_title, $cat_id, sprintf(XMLRPC_NEWS_CATEGS_EDIT_SYSTEM_LOG, $cat_title));
			}
		}

		if (count($cat_ids) > 0)
		{
			sql_query('DELETE FROM sb_catlinks WHERE link_id IN (?a)', $link_ids);

			$src_cat_id = 0;
			foreach ($cat_ids as $key => $value)
			{
				if ($key == 0)
				{
					sql_query('INSERT INTO sb_catlinks (link_cat_id, link_el_id, link_src_cat_id) VALUES (?d, ?d, 0)', $value, $params[0]);
					$src_cat_id = $value;
				}
				else
				{
					sql_query('INSERT INTO sb_catlinks (link_cat_id, link_el_id, link_src_cat_id) VALUES (?d, ?d, ?d)', $value, $params[0], $src_cat_id);
				}
			}
		}
	}

	$row = array();
	if (isset($content['title']) && trim($content['title']) != '')
	{
		$title = $content['title'];

		if ($auth_res['u_admin'] != 1 && $auth_res['u_strip_php'])
		{
			sb_replace_php($title);
		}

		$row['n_title'] = trim($title);

		if ($row['n_title'] == '')
		{
			sb_xmlrpc_error (401, XMLRPC_NEWS_TITLE_ERROR);
			return;
		}
	}
	else
	{
		sb_xmlrpc_error (401, XMLRPC_NEWS_TITLE_ERROR);
		return;
	}

	if ($params[4] == true)
	{
		$row['n_active'] = 1;
	}
	else
	{
		$row['n_active'] = 0;
	}

	if (isset($content['description']) && trim($content['description']) != '' && trim($content['description']) != '<span/>')
	{
		$text = sb_clean_word($content['description']);
		$text = preg_split('/<p[^>]*>([^<]* | < [^\/]* | <\/ [^p]* | <\/p [^>]*)\+\+\+(.*?)<\/p>/imsx', $text);
        
        //preg_split никогда не возвращает пустой массив
		if (count($text) == 1)
		{
			$text = preg_split('/<!\-\-[\s]*more[\s]*\-\->/im', $text[0]);
		}

		if (count($text) > 1)
		{
			$short = $text[0];
			$full = $text[1];

			if ($auth_res['u_admin'] != 1 && $auth_res['u_strip_php'])
			{
				sb_replace_php($short);
				sb_replace_php($full);
			}

			$row['n_short'] = trim($short);
			$row['n_full'] = trim($full);
		}
		else
		{
			$short = $text[0];

			if ($auth_res['u_admin'] != 1 && $auth_res['u_strip_php'])
			{
				sb_replace_php($short);
			}

			$row['n_short'] = trim($short);
			$row['n_full'] = '';
		}
	}

    sql_query('UPDATE sb_news SET ?a WHERE n_id=?d', $row, $params[0]);
    sql_query('INSERT INTO sb_catchanges (el_id, cat_ident, change_date, change_user_id, action) VALUES (?d, ?, ?d, ?d, ?)', $params[0], 'pl_news', time(), $auth_res['u_id'], 'edit');

    sb_xmlrpc_add_system_message(sprintf(XMLRPC_NEWS_EDIT_SYSTEM_LOG, $n_title), $auth_res);

    return true;
}

function sb_xmlrpc_new_post($method_name, $params, $app_data)
{
	$auth_res = sb_xmlrpc_login($params[1], $params[2]);

	if (!$auth_res)
	{
		sb_xmlrpc_error (401, XMLRPC_LOGIN_ERROR);
		return;
	}

	$content = $params[3];

	if (!isset($content['categories']) || !is_array($content['categories']) || count($content['categories']) <= 0)
	{
		sb_xmlrpc_error (401, XMLRPC_NEWS_NO_CATEGS_ERROR);
		return;
	}

	$cat_titles = array();

	$i = -1;
	foreach ($content['categories'] as $value)
	{
		$value = trim($value);
		$cat_id = $i--;

		if (sb_substr($value, 0, 1) == '[')
		{
			$pos = sb_strpos($value, ']');
			if ($pos)
			{
				$cat_id = intval(sb_substr($value, 1, $pos - 1));
			}
		}

		if ($cat_id < 0)
		{
			$cat_titles[$cat_id] = trim($value);
		}
		else
		{
			$res = sql_query('SELECT cat_rights FROM sb_categs WHERE cat_id=?d', $cat_id);
			if ($res && ($auth_res['u_admin'] == 1 || sb_xmlrpc_check_categs_rights($res[0][0], $auth_res, 'elems_edit')))
			{
				$cat_titles[$cat_id] = trim(sb_substr($value, $pos + 1));
			}
		}
	}

	if (count($cat_titles) == 0)
	{
		sb_xmlrpc_error (403, XMLRPC_NEWS_CATEGS_RIGHTS_ERROR);
		return;
	}

	// Переименование разделов
	if ($auth_res['u_admin'] == 1 || sb_xmlrpc_check_plugin_rights($auth_res['u_groups'], 'categs_edit'))
	{
		foreach ($cat_titles as $cat_id => $cat_title)
		{
			if ($cat_id > 0)
			{
				// Редактирование раздела
				sql_query('UPDATE sb_categs SET cat_title=? WHERE cat_id=?d', $cat_title, $cat_id, sprintf(XMLRPC_NEWS_CATEGS_EDIT_SYSTEM_LOG, $cat_title));
			}
		}
	}

	$row = array();

    if (isset($content['title']) && trim($content['title']) != '')
	{
		$title = $content['title'];

		if ($auth_res['u_admin'] != 1 && $auth_res['u_strip_php'])
		{
			sb_replace_php($title);
		}

		$row['n_title'] = trim($title);

		if ($row['n_title'] == '')
		{
			sb_xmlrpc_error (401, XMLRPC_NEWS_TITLE_ERROR);
			return;
		}

		$row['n_url'] = sb_check_chpu('', '', $row['n_title'], 'sb_news', 'n_url', 'n_id');;
	}
	else
	{
		sb_xmlrpc_error (401, XMLRPC_NEWS_TITLE_ERROR);
		return;
	}

	$row['n_pub_start'] = null;
	$row['n_pub_end'] = null;
	$row['n_date'] = time();
    $row['n_short_foto'] = '';
    $row['n_full_foto'] = '';

	$res = sql_query('SELECT MAX(n_sort) FROM sb_news');
	if ($res)
	{
	    list($n_sort) = $res[0];
	    $n_sort += 10;
	}
	else
	{
	    $n_sort = 0;
	}
	$row['n_sort'] = $n_sort;

	if ($params[4] == true)
	{
		$row['n_active'] = 1;
	}
	else
	{
		$row['n_active'] = 0;
	}

	if (isset($content['description']) && trim($content['description']) != '' && trim($content['description']) != '<span/>')
	{
		$text = sb_clean_word($content['description']);
		$text = preg_split('/<p[^>]*>([^<]* | < [^\/]* | <\/ [^p]* | <\/p [^>]*)\+\+\+(.*?)<\/p>/imsx', $text);
		if (count($text) <= 1)
		{
			$text = preg_split('/<!\-\-[\s]*more[\s]*\-\->/im', $text[0]);
		}

		if (count($text) > 1)
		{
			$short = $text[0];
			$full = $text[1];

			if ($auth_res['u_admin'] != 1 && $auth_res['u_strip_php'])
			{
				sb_replace_php($short);
				sb_replace_php($full);
			}

			$row['n_short'] = trim($short);
			$row['n_full'] = trim($full);
		}
		else
		{
			$short = $text[0];

			if ($auth_res['u_admin'] != 1 && $auth_res['u_strip_php'])
			{
				sb_replace_php($short);
			}

			$row['n_short'] = trim($short);
			$row['n_full'] = '';
		}
	}
	else
	{
		$row['n_short'] = '';
	    $row['n_full'] = '';
	}

	sql_query('LOCK TABLES sb_news WRITE, sb_catlinks WRITE, sb_catchanges WRITE');
	if (!sql_query('INSERT INTO sb_news (?#) VALUES (?a)', array_keys($row), array_values($row)))
	{
		sql_query('UNLOCK TABLES');
		sb_xmlrpc_error (405, sprintf(XMLRPC_NEWS_ADD_ERROR, $row['n_title']));
		return;
	}

	$n_id = sql_insert_id();

	$added = false;
    $src_cat_id = -1;
    foreach ($cat_titles as $cat_id => $cat_title)
    {
    	if ($src_cat_id == -1)
    	{
    		$res = sql_query('INSERT INTO sb_catlinks (link_cat_id, link_el_id, link_src_cat_id) VALUES (?d, ?d, 0)', $cat_id, $n_id);
    		if ($res)
    		{
    			$src_cat_id = $cat_id;
    		}
    	}
    	else
    	{
    		$res = sql_query('INSERT INTO sb_catlinks (link_cat_id, link_el_id, link_src_cat_id) VALUES (?d, ?d, ?d)', $cat_id, $n_id, $src_cat_id);
    	}

   		$added = $added || $res;
    }

    if ($added)
    {
    	sql_query('INSERT INTO sb_catchanges (el_id, cat_ident, change_date, change_user_id, action) VALUES (?d, ?, ?d, ?d, ?)', $n_id, 'pl_news', time(), $auth_res['u_id'], 'add');
    	sql_query('UNLOCK TABLES');
    }
    else
    {
    	sql_query('DELETE FROM sb_news WHERE n_id=?d', $n_id);
    	sql_query('UNLOCK TABLES');
    	sb_xmlrpc_error (405, sprintf(XMLRPC_NEWS_ADD_ERROR, $row['n_title']));
    	return;
    }

    sb_xmlrpc_add_system_message(sprintf(XMLRPC_NEWS_ADD_SYSTEM_LOG, $row['n_title']), $auth_res);

    return $n_id;
}

function sb_xmlrpc_new_media($method_name, $params, $app_data)
{
	$auth_res = sb_xmlrpc_login($params[1], $params[2]);

	if (!$auth_res)
	{
		sb_xmlrpc_error (401, XMLRPC_LOGIN_ERROR);
		return;
	}

	$image = $params[3];
	$ext_pos = strrpos($image['name'], '.');

	if ($ext_pos === false)
	{
		sb_xmlrpc_error (403, XMLRPC_MEDIA_RIGHTS_ERROR);
		return;
	}

	$ext = trim(substr($image['name'], $ext_pos + 1));
	$raw_name = trim(substr($image['name'], 0, $ext_pos));

	if ($auth_res['u_admin'] != 1)
	{
		$allowed_exts = explode(' ', $auth_res['u_uploading_exts']);
		if (!in_array($ext, $allowed_exts))
		{
			sb_xmlrpc_error (403, XMLRPC_MEDIA_RIGHTS_ERROR);
			return;
		}
	}

	if (strpos($image['bits']->scalar, '<script') > -1 ||
 	    strpos($image['bits']->scalar, '<embed') > -1 ||
     	strpos($image['bits']->scalar, '<applet') > -1 ||
     	strpos($image['bits']->scalar, '<object') > -1 ||
     	strpos($image['bits']->scalar, '<html') > -1 ||
     	strpos($image['bits']->scalar, '<body') > -1 ||
     	strpos($image['bits']->scalar, 'javascript') > -1 ||
     	strpos($image['bits']->scalar, '<head') > -1 )
     {
 	    sb_xmlrpc_error (403, XMLRPC_MEDIA_RIGHTS_ERROR);
		return;
     }

	if (!$GLOBALS['sbVfs']->is_dir(SB_SITE_USER_UPLOAD_PATH.'/pl_news'))
	{
		if (!$GLOBALS['sbVfs']->mkdir(SB_SITE_USER_UPLOAD_PATH.'/pl_news'))
		{
			sb_xmlrpc_error (405, sprintf(XMLRPC_MEDIA_MKDIR_ERROR, SB_SITE_USER_UPLOAD_PATH.'/pl_news/'));
			return;
		}
	}

	$image['name'] = sb_strtolat($image['name']);
    //только малые буквы, цифры, знак подчеркивания, точка и тире
	$image['name'] = preg_replace('/[^A-Za-z0-9_\.\-]/', '', str_replace(array(' ', '%20'), array('_', '_'), $image['name']));

	if ($GLOBALS['sbVfs']->exists(SB_SITE_USER_UPLOAD_PATH.'/pl_news/'.$image['name']) &&
	    sb_crc($GLOBALS['sbVfs']->file_get_contents(SB_SITE_USER_UPLOAD_PATH.'/pl_news/'.$image['name'])) != sb_crc($image['bits']->scalar))
	{
		$n = 1;
		$copy = '_'.$n;
	    $image['name'] = $raw_name.$copy.'.'.$ext;

		while($GLOBALS['sbVfs']->exists(SB_SITE_USER_UPLOAD_PATH.'/pl_news/'.$image['name']))
    	{
    		if (sb_crc($GLOBALS['sbVfs']->file_get_contents(SB_SITE_USER_UPLOAD_PATH.'/pl_news/'.$image['name'])) == sb_crc($image['bits']->scalar))
    		{
    			break;
    		}

    	    $n++;
    		$copy = '_'.$n;

    		$image['name'] = $raw_name.$copy.'.'.$ext;
    	}
	}

	if (!$GLOBALS['sbVfs']->file_put_contents(SB_SITE_USER_UPLOAD_PATH.'/pl_news/'.$image['name'], $image['bits']->scalar))
	{
		sb_xmlrpc_error (405, sprintf(XMLRPC_MEDIA_FILE_ERROR, SB_SITE_USER_UPLOAD_PATH.'/pl_news/'.$image['name'], SB_SITE_USER_UPLOAD_PATH.'/pl_news/'));
		return;
	}

	$result = array('url' => SB_SITE_USER_UPLOAD_URL.'/pl_news/'.$image['name']);

	return $result;
}

function sb_xmlrpc_delete_post($method_name, $params, $app_data)
{
	$auth_res = sb_xmlrpc_login($params[2], $params[3]);

	if (!$auth_res)
	{
		sb_xmlrpc_error (401, XMLRPC_LOGIN_ERROR);
		return;
	}

	$res = sql_query('SELECT STRAIGHT_JOIN c.cat_id, c.cat_rights, l.link_id, n.n_title
                      	FROM sb_news n, sb_catlinks l, sb_categs c
                        WHERE n.n_id=?d
                        	AND l.link_el_id=n.n_id
                        	AND l.link_cat_id = c.cat_id
                        	AND c.cat_ident=?', $params[1], 'pl_news');

	if (!$res)
	{
		sb_xmlrpc_error (403, XMLRPC_NEWS_WAS_DELETED);
		return;
	}

	$cat_ids = array();
	$link_ids = array();
	$access = false;

	foreach ($res as $value)
	{
		list($cat_id, $cat_rights, $link_id, $n_title) = $value;

		if (!in_array($cat_id, $cat_ids))
			$cat_ids[] = $cat_id;

		if (!in_array($link_id, $link_ids))
			$link_ids[] = $link_id;

		if (!$access)
		{
			$access = sb_xmlrpc_check_categs_rights($cat_rights, $auth_res, 'elems_delete');
		}
	}

	if ($auth_res['u_admin'] != 1 && !$access)
	{
		sb_xmlrpc_error (403, XMLRPC_NEWS_DELETE_RIGHTS_ERROR);
		return;
	}

	sql_query('LOCK TABLES sb_news WRITE, sb_catlinks WRITE, sb_catchanges WRITE');
	if (sql_query('DELETE FROM sb_news WHERE n_id=?d', $params[1]))
	{
	    sql_query('DELETE FROM sb_catlinks WHERE link_id IN (?a)', $link_ids);
        sql_query('DELETE FROM sb_catchanges WHERE el_id=?d AND cat_ident=?', $params[1], 'pl_news');
        sql_query('UNLOCK TABLES');

        if (trim($n_title) != '')
        {
        	sb_xmlrpc_add_system_message(sprintf(XMLRPC_NEWS_DELETE_SYSTEM_LOG, $n_title), $auth_res);
        }
	}
	else
	{
		sql_query('UNLOCK TABLES');
		sb_xmlrpc_error (405, XMLRPC_NEWS_DELETE_ERROR);
		return;
	}

	return true;
}

function sb_xmlrpc_add_system_message($message, $auth_res, $type = SB_MSG_INFORMATION)
{
	if (empty($message) || !isset($GLOBALS['sbSql']))
        return;

    if (!is_resource($GLOBALS['sbSql']->mLinkId) && !($GLOBALS['sbSql']->mLinkId instanceof mysqli))
        return;

    $system_log_date = intval(sbPlugins::getSetting('sb_system_log_date'));
    if ($system_log_date <= 0)
        return;

	$GLOBALS['sbSql']->paramQuery('INSERT INTO sb_system_log (sl_date, sl_user_id, sl_user_login, sl_user_ip, sl_message, sl_type, sl_domain) VALUES (?d, ?d, ?, ?, ?, ?, ?)', time(), $auth_res['u_id'], $auth_res['u_login'], (class_exists('sbAuth') ? sbAuth::getIP() : '-'), $message, $type, SB_COOKIE_DOMAIN);
}

function sb_xmlrpc_check_plugin_rights($u_groups, $plugin_rights = 'read')
{
	$res = sql_query('SELECT ur_plugin_rights FROM sb_users_rights WHERE ur_plugin=? AND ur_cat_id IN (?a) AND ur_plugin_rights <> ?', 'pl_news', $u_groups, '');

    if (!$res)
    {
    	// информации о правах доступа нет
    	return false;
    }

	list ($user_rights) = $res[0];

    $user_rights = explode('|', $user_rights);
    $plugin_rights = explode('|', $plugin_rights);

    foreach ($plugin_rights as $value)
    {
        if (in_array($value, $user_rights))
        {
            return true;
        }
    }

    return false;
}

function sb_xmlrpc_check_categs_rights($cat_rights, $auth_res, $plugin_rights = 'read')
{
	if ($auth_res['u_admin'] == 1)
	{
		return true;
	}

	if (trim($cat_rights) == '')
	{
		return false;
	}

	static $plugin_avail = null;

	if (is_null($plugin_avail))
	{
		$plugin_avail = sb_xmlrpc_check_plugin_rights($auth_res['u_groups'], $plugin_rights);
	}

	if (!$plugin_avail)
		return false;

	$cat_rights = explode('^', $cat_rights);
	$cat_groups = array();

    foreach ($cat_rights as $right)
    {
    	if ($right[0] == 'g')
    	{
    		$cat_groups[] = intval(substr($right, 1));
    	}
    	elseif ($right[0] == 'u')
    	{
    		if (intval(substr($right, 1)) == $auth_res['u_id'])
    		{
    			return true;
    		}
    	}
    	else
    	{
    		$cat_groups[] = intval($right);
    	}
    }

	if (count(array_intersect($auth_res['u_groups'], $cat_groups)) > 0)
    {
        return true;
    }

	return false;
}

function sb_xmlrpc_login($login, $password)
{
	if (!sbPlugins::getSetting('sb_xmlrpc_calls'))
	{
		sb_xmlrpc_error (405, XMLRPC_OFF_ERROR);
		return false;
	}

	sql_query('UPDATE sb_users SET u_block = 1 WHERE u_active_date != 0 AND u_active_date < '.time());

	$password = md5($password);

	$res = sql_query('SELECT u_id, u_name, u_email, u_admin, u_block, u_domains, u_strip_php, u_uploading_exts FROM sb_users WHERE u_login=? AND u_pass=?', $login, $password);
	if (!$res)
	{
		sb_xmlrpc_error (401, XMLRPC_LOGIN_ERROR);
		return false;
	}

	$result = array();
	$result['u_login'] = $login;

	list($result['u_id'], $result['u_name'], $result['u_email'], $result['u_admin'], $u_block, $u_domains, $result['u_strip_php'], $result['u_uploading_exts']) = $res[0];

	if ($u_block == 1)
	{
		sb_xmlrpc_error (403, XMLRPC_LOGIN_BLOCKED_ERROR);
		return false;
	}

	if ($result['u_admin'] != 1)
    {
        $res = sql_query('SELECT c.cat_id FROM sb_categs c, sb_catlinks l
                         WHERE c.cat_closed = 1 AND c.cat_ident = ?
                         AND l.link_cat_id = c.cat_id AND l.link_el_id = ?d', 'pl_users', $result['u_id']);

        if ($res)
        {
            $result['u_admin'] = 1;
        }
        else
        {
            $u_domains = explode(',', $u_domains);
            if (!in_array(SB_COOKIE_DOMAIN, $u_domains))
            {
            	sb_xmlrpc_error (403, XMLRPC_LOGIN_DOMAIN_ERROR);
                return false;
            }
        }
    }

	$availIPs = sbPlugins::getSetting('sb_admin_ip');
    $u_ip = sbAuth::getIP();

    if ($availIPs != '')
    {
        $error = true;
        $availIPs = explode(' ', $availIPs);
        foreach ($availIPs as $value)
        {
            if (trim($value) != '')
            {
                if (sbAuth::compareIP($value, $u_ip))
                {
                    $error = false;
                    break;
                }
            }
        }

        if ($error)
        {
            //пытаемся зайти не с разрешенного IP-адреса
            sb_xmlrpc_error (403, XMLRPC_LOGIN_IP_ERROR);
            return false;
        }
    }

	$group_ids = array();
    $res = sql_query('SELECT DISTINCT links.link_cat_id
    	FROM sb_categs categs, sb_catlinks links
    	WHERE categs.cat_ident = ? AND categs.cat_rubrik = 1
    	AND links.link_cat_id = categs.cat_id AND links.link_el_id = ?d', 'pl_users', $result['u_id']);

    if ($res)
    {
        foreach ($res as $value)
        {
            $group_ids[] = $value[0];
        }
    }
    else
    {
        sb_xmlrpc_error (403, XMLRPC_LOGIN_GROUP_ERROR);
        return false;
    }

    $result['u_groups'] = $group_ids;

	return $result;
}

/**
 * Генерация ошибки
 *
 * @param int $code Код ошибки.
 * @param string $message Сообщение об ошибке.
 */
function sb_xmlrpc_error($code, $message)
{
	$str = '<methodResponse>
  <fault>
    <value>
      <struct>
        <member>
          <name>faultCode</name>
          <value><int>'.intval($code).'</int></value>
        </member>
        <member>
          <name>faultString</name>
          <value><string><![CDATA['.$message.']]</string></value>
        </member>
      </struct>
    </value>
  </fault>
</methodResponse>';

	echo $str;

	require_once(SB_PROG_KERNEL_PATH.'/footer.inc.php');
	exit(0);
}

require_once(SB_PROG_KERNEL_PATH.'/footer.inc.php');
?>