<?php
/**
 * Класс, отвечающий за вывод текстового поля с возможностью автозаполнения (input + select)
 *
 * @see sbLayout
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2009, OOO "СИБИЭС Групп"
 */
class sbLayoutAutocomplete
{
    /**
     * Текст по умолчанию (значение атрибута <i>value</i>)
     *
     * @var string
     */
    private $mValue = '';

    /**
     * Имя поля (значение атрибута <i>name</i>)
     *
     * @var string
     */
    private $mName = '';

    /**
     * Идентификатор поля (значение атрибута <i>id</i>)
     *
     * @var string
     */
    private $mId = '';

    /**
     * Дополнительные св-ва поля (например, стили)
     *
     * @var string
     */
    private $mDopStr = '';

    /**
     * Поле обязательно для заполнения (TRUE) или нет (FALSE)
     *
     * @var bool
     */
    private $mRequired = false;

    /**
     * Событие, вызываемое для подгрузки значений поля
     *
     * @var int
     */
    public $mAction = '';

    /**
     * Дополнительный HTML-код, выводимый под полем
     *
     * @var string
     */
    public $mHTML = '';

    /**
     * Конструктор класса
     *
     * @param string $type Тип поля (значение атрибута <i>type</i>)
     * @param string $value Текст по умолчанию (значение атрибута <i>value</i>).
     * @param string $name Имя поля (значение атрибута <i>name</i>).
     * @param string $id Идентификатор поля (значение атрибута <i>id</i>).
     * @param string $dop_str Дополнительные св-ва поля (например, стили).
     * @param bool $required Поле обязательно для заполнения (TRUE) или нет (FALSE).
     */
    public function __construct($value='', $name='', $id='', $dop_str='', $required=false)
    {
        if ($id == '') $id = $name;

        $this->mValue = $value;
        $this->mName = $name;
        $this->mId = $id;
        $this->mDopStr = $dop_str;
        $this->mRequired = $required;
    }

    /**
     * Возвращает HTML-код поля
     *
     * @return string HTML-код поля.
     */
    public function getField()
    {
        $res = '<input type="text" autocomplete="off" value="'.sb_htmlspecialchars($this->mValue, ENT_QUOTES).'" onkeyup="sbAutocompleteShowOptions(this, \''.$this->mAction.'\', event);"'.($this->mName != '' ? ' name="'.$this->mName.'"' : '').($this->mId != '' ? ' id="'.$this->mId.'"' : '').' '.$this->mDopStr.' />'.($this->mRequired ? ' <sup class="red">*</sup>' : '');

        return $res.$this->mHTML;
    }

    /**
     * Возвращает JavaScript-код поля
     *
     * @return string JavaScript-код поля.
     */
    public function getJavaScript()
    {
        static $echo = true;

        if ($echo)
        {
            $echo = false;
            return '<script src="'.SB_CMS_JSCRIPT_URL.'/sbAutocomplete.js.php"></script>';
        }
        else
        {
            return '';
        }
    }
}
?>