<?php
/**
 * Класс, отвечающий за вывод полей выбора разделов
 *
 * @see sbLayout
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbLayoutCategs
{
    /**
     * Идентификатор разделов
     *
     * @var string
     */
    private $mIdent = '';

    /**
     * ID выбранного раздела
     *
     * @var string
     */
    private $mValue = '';

    /**
     * Имя поля (значение атрибута <i>name</i>)
     *
     * @var string
     */
    private $mName = '';

    /**
     * Идентификатор поля (значение атрибута <i>id</i>)
     *
     * @var string
     */
    private $mId = '';

    /**
     * Дополнительные св-ва поля (например, стили)
     *
     * @var string
     */
    private $mDopStr = '';

    /**
     * Дополнительный HTML-код, вставляемый после поля
     *
     * @var string
     */
    public $mHTML = '';

    /**
     * Используется множественный выбор у разделов или нет
     *
     * @var bool
     */
    public $mMultiSelect = false;

    /**
     * Уникальный идентификатор раздела, подразделы которого будут выводиться
     *
     * @var string
     */
    private $mCatId = '';

    /**
     * Поле обязательно для заполнения (TRUE) или нет (FALSE)
     *
     * @var bool
     */
    private $mRequired = false;

    /**
     * Отображать или нет кнопку "Сбросить значение"
     *
     * @var bool
     */
    public $mDropButton = true;

    /**
     * Поле доступно только для чтения
     *
     * @var bool
     */
    public $mReadOnly = false;

    /**
     * Конструктор класса
     *
     * @param string $ident Идентификатор разделов
     * @param string $cat_id Идентификаторы раздела, подразделы которого будут выводиться.
     * @param string $value ID выбранного раздела.
     * @param string $name Имя поля (значение атрибута <i>name</i>).
     * @param string $id Идентификатор поля (значение атрибута <i>id</i>).
     * @param string $dop_str Дополнительные св-ва поля (например, стили).
     * @param bool $required Поле обязательно для заполнения (TRUE) или нет (FALSE).
     */
    public function __construct($ident, $cat_id='', $value='', $name='', $id='', $dop_str='', $required=false)
    {
        if ($id == '') $id = $name;

        $this->mIdent = $ident;
        $this->mValue = $value;
        $this->mCatId = $cat_id;
        $this->mName = $name;
        $this->mId = $id;
        $this->mDopStr = $dop_str;
        $this->mRequired = $required;
    }

    /**
     * Возвращает HTML-код поля
     *
     * @return string HTML-код поля.
     */
    public function getField()
    {
        $res = false;
        if ($this->mValue != '')
        {
            if ($this->mMultiSelect == 1)
                $res = sql_query('SELECT cat_title FROM sb_categs WHERE cat_id IN (?a) ORDER BY cat_left LIMIT 0,4', explode('^', $this->mValue));
            else
                $res = sql_query('SELECT cat_title FROM sb_categs WHERE cat_id=?d', $this->mValue);
        }

        $result = '<table><tr style="background-color: transparent;"><td>';
        if (!$res)
        {
            $result .= '<span id="'.$this->mId.'_text"><b style="font-size:11px;">'.SB_LAYOUT_NO_CATEG.'</b>';
        }
        else
        {
            if ($this->mMultiSelect == 1)
            {
                $result .= '<span id="'.$this->mId.'_text">';
                $more_than_three = false;
                $num = count($res);

                if ($num > 3)
                {
                    $more_than_three = true;
                    unset($res[$num - 1]);
                }

                foreach ($res as $value)
                {
                    $result .= $value[0].', ';
                }

                $result = substr($result, 0, -2);
                if ($more_than_three)
                {
                    $result .= ', ...';
                }
            }
            else
            {
                $result .= '<span id="'.$this->mId.'_text">'.$res[0][0];
            }
        }

        $result .= '</span>'.($this->mRequired ? '&nbsp;<sup class="red">*</sup>' : '').'</td><td><nobr>'.
                   ($this->mReadOnly ? '' : '&nbsp;<img class="button" src="'.SB_CMS_IMG_URL.'/categs.png" width="20" height="20" align="absmiddle" id="'.$this->mId.'_button" onmouseup="sbPress(this, false);" onmousedown="sbPress(this, true);" onclick="sbGetCateg(\''.$this->mId.'\')" />').
                   ($this->mDropButton  && !$this->mReadOnly ? '<img class="button" src="'.SB_CMS_IMG_URL.'/categs_drop.png" width="20" height="20" align="absmiddle" id="'.$this->mId.'_drop_button" onmouseup="sbPress(this, false);" onmousedown="sbPress(this, true);" onclick="sbDropCateg(\''.$this->mId.'\')" />' : '').'
                    <input type="hidden" name="'.$this->mName.'" id="'.$this->mId.'" value="'.$this->mValue.'"'.($this->mDopStr != '' ? ' '.$this->mDopStr : '').'>
                    <input type="hidden" id="'.$this->mId.'_multi" value="'.($this->mMultiSelect ? '1' : '0').'">
                    <input type="hidden" id="'.$this->mId.'_cat" value="'.$this->mCatId.'">
                    <input type="hidden" id="'.$this->mId.'_ident" value="'.$this->mIdent.'"></nobr></td></tr></table>'.$this->mHTML;

        return $result;
    }

    /**
     * Возвращает JavaScript-код поля
     *
     * @return string JavaScript-код поля.
     */
    public function getJavaScript()
    {
        static $echo = true;

        if ($echo)
        {
            $echo = false;
            return '<script>
                var sbCategElId = null;
                function sbGetCateg(el_id)
                {
                    var el = sbGetE(el_id);
                    if (el.disabled)
                        return;

                    if (el)
                    {
                        sbCategElId = el_id;
                        var cat_el = sbGetE(el_id + "_cat");
                        var ident_el = sbGetE(el_id + "_ident");
                        var multi_el = sbGetE(el_id + "_multi");

                        var strPage = sb_cms_modal_dialog_file + "?event=pl_categs_get_categs&id="+el.value+"&cat_id="+cat_el.value+"&ident="+ident_el.value+"&multi="+multi_el.value;
                        var strAttr = "resizable=1,width=700,height=500";

                        sbShowModalDialog(strPage, strAttr, sbAfterGetCateg);
                    }
                }
                function sbAfterGetCateg()
                {
                    if (sbModalDialog.returnValue && sbCategElId)
                    {
                        var el = sbGetE(sbCategElId);
                        var el_name = sbGetE(sbCategElId + "_text");

                        el.value = sbModalDialog.returnValue.id;
                        el_name.innerHTML = sbModalDialog.returnValue.text != "" ? sbModalDialog.returnValue.text : "<b style=\'font-size:11px;\'>'.SB_LAYOUT_NO_CATEG.'</b>";
                    }
                }
                function sbDropCateg(el_id)
                {
                    var el = sbGetE(el_id);
                    var el_name = sbGetE(el_id + "_text");

                    el.value = "";
                    el_name.innerHTML = "<b style=\'font-size:11px;\'>'.SB_LAYOUT_NO_CATEG.'</b>";
                }
                </script>';
        }
        else
        {
            return '';
        }
    }
}
?>