<?php
/**
 * Класс, отвечающий за вывод поля с кодом цвета и кнопкой открытия диалогового окна выбора цвета
 *
 * @see sbLayout
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbLayoutColor
{
    /**
     * Цвет по умолчанию (значение атрибута <i>value</i>)
     *
     * @var string
     */
    private $mValue = '';

    /**
     * Имя поля (значение атрибута <i>name</i>)
     *
     * @var string
     */
    private $mName = '';

    /**
     * Идентификатор поля (значение атрибута <i>id</i>)
     *
     * @var string
     */
    private $mId = '';

    /**
     * Дополнительные св-ва поля (например, стили)
     *
     * @var string
     */
    private $mDopStr = '';

    /**
     * Дополнительный HTML-код, вставляемый после поля
     *
     * @var string
     */
    public $mHTML = '';

    /**
     * Поле обязательно для заполнения (TRUE) или нет (FALSE)
     *
     * @var bool
     */
    private $mRequired = false;

    /**
     * Отображать или нет кнопку "Сбросить значение"
     *
     * @var bool
     */
    public $mDropButton = false;

    /**
     * Поле доступно только для чтения
     *
     * @var bool
     */
    public $mReadOnly = false;

    /**
     * Конструктор класса
     *
     * @param string $value Цвет по умолчанию (значение атрибута <i>value</i>).
     * @param string $name Имя поля (значение атрибута <i>name</i>).
     * @param string $id Идентификатор поля (значение атрибута <i>id</i>).
     * @param string $dop_str Дополнительные св-ва поля (например, стили).
     * @param bool $required Поле обязательно для заполнения (TRUE) или нет (FALSE).
     */
    public function __construct($value='', $name='', $id='', $dop_str='', $required=false)
    {
        if ($id == '') $id = $name;

        $this->mValue = $value;
        $this->mName = $name;
        $this->mId = $id;
        $this->mDopStr = $dop_str;
        $this->mRequired = $required;
    }

    /**
     * Возвращает HTML-код поля
     *
     * @return string HTML-код поля.
     */
    public function getField()
    {
        return '<nobr><input type="text" name="'.$this->mName.'" id="'.$this->mId.'" value="'.sb_htmlspecialchars($this->mValue, ENT_QUOTES).'"'.($this->mDopStr != '' ? ' '.$this->mDopStr : '').' readonly="readonly" style="width: 120px;height:25px;'.($this->mValue != '' ? 'background-color:'.$this->mValue.';' : '').'">'.($this->mRequired ? ' <sup class="red">*</sup>' : '').
               ($this->mReadOnly ? '' : '&nbsp;<img class="button" src="'.SB_CMS_IMG_URL.'/color.png" width="20" height="20" align="absmiddle" id="'.$this->mId.'_button" onmouseup="sbPress(this, false);" onmousedown="sbPress(this, true);" onclick="sbCDlg(\''.$this->mId.'\');">').
               ($this->mDropButton && !$this->mReadOnly ? '<img class="button" src="'.SB_CMS_IMG_URL.'/color_drop.png" width="20" height="20" id="'.$this->mId.'_drop_button" align="absmiddle" onmouseup="sbPress(this, false);" onmousedown="sbPress(this, true);" onclick="sbCDlgDrop(\''.$this->mId.'\');"></nobr>' : '</nobr>').$this->mHTML;
    }

    /**
     * Возвращает JavaScript-код поля
     *
     * @return string JavaScript-код поля.
     */
    public function getJavaScript()
    {
        static $echo = true;

        if ($echo)
        {
            $echo = false;
            return '<script>
                var sbCDlgEl = null;
                function sbCDlg(el_id)
                {
                    var el = sbGetE(el_id);
                    if (el.disabled)
                        return;
                    if (el)
                    {
                        sbCDlgEl = el_id;
                        var strPage = sb_cms_modal_dialog_file + "?event=pl_editor_color&color="+sbEscapeEx(el.value);
                        if (/MSIE (5\.5|6\.)/.test(navigator.userAgent))
                            var strAttr = "resizable=0,width=700,height=470";
                        else
                            var strAttr = "resizable=0,width=700,height=450";

                        sbShowModalDialog(strPage, strAttr, sbAfterCDlg);
                    }
                }
                function sbAfterCDlg()
                {
                    if (sbModalDialog.returnValue)
                    {
                        var el = sbGetE(sbCDlgEl);
                        if (el)
                        {
                            el.value = sbModalDialog.returnValue;
                            el.style.background = el.value;
                            if (typeof(el.onchange) != "undefined" && el.onchange)
                                el.onchange();
                        }
                    }
                }
                function sbCDlgDrop(el_id)
                {
                    var el = sbGetE(el_id);
                    if (el.disabled)
                        return;
                    if (el)
                    {
                        el.value = "";
                        el.style.background = "#FFFFFF";
                    }
                }
                </script>';
        }
        else
        {
            return '';
        }
    }
}
?>