<?php
/**
 * Класс, отвечающий за вывод поля с датой и кнопкой календаря
 *
 * @see sbLayout
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbLayoutDate
{
    /**
     * Дата по умолчанию (значение атрибута <i>value</i>)
     *
     * @var string
     */
    private $mValue = '';

    /**
     * Имя поля (значение атрибута <i>name</i>)
     *
     * @var string
     */
    private $mName = '';

    /**
     * Идентификатор поля (значение атрибута <i>id</i>)
     *
     * @var string
     */
    private $mId = '';

    /**
     * Дополнительные св-ва поля (например, стили)
     *
     * @var string
     */
    private $mDopStr = '';

    /**
     * Дополнительный HTML-код, вставляемый после поля
     *
     * @var string
     */
    public $mHTML = '';

    /**
     * Поле обязательно для заполнения (TRUE) или нет (FALSE)
     *
     * @var bool
     */
    private $mRequired = false;

    /**
     * Выводить в календаре время или нет
     *
     * @var bool
     */
    public $mShowTime = true;

    /**
     * Отображать или нет кнопку "Сбросить значение календаря"
     *
     * @var bool
     */
    public $mDropButton = false;

    /**
     * Поле доступно только для чтения
     *
     * @var bool
     */
    public $mReadOnly = false;

    /**
     * Конструктор класса
     *
     * @param string $value Дата по умолчанию (значение атрибута <i>value</i>).
     * @param string $name Имя поля (значение атрибута <i>name</i>).
     * @param string $id Идентификатор поля (значение атрибута <i>id</i>).
     * @param string $dop_str Дополнительные св-ва поля (например, стили).
     * @param bool $required Поле обязательно для заполнения (TRUE) или нет (FALSE).
     */
    public function __construct($value='', $name='', $id='', $dop_str='', $required=false)
    {
        if ($id == '') $id = $name;

        $this->mValue = $value;
        $this->mName = $name;
        $this->mId = $id;
        $this->mDopStr = $dop_str;
        $this->mRequired = $required;
    }

    /**
     * Возвращает HTML-код поля
     *
     * @return string HTML-код поля.
     */
    public function getField()
    {
        if ($this->mReadOnly)
        {
            return '<nobr><input type="text" name="'.$this->mName.'" id="'.$this->mId.'" value="'.sb_htmlspecialchars($this->mValue, ENT_QUOTES).'" readonly="readonly" style="'.($this->mShowTime ? 'width:110px;' : 'width:90px;').'height:25px;"'.($this->mDopStr != '' ? ' '.$this->mDopStr : '').'>'.($this->mRequired ? ' <sup class="red">*</sup>' : '').'</nobr>';
        }
        else
        {
            return '<nobr><input type="text" name="'.$this->mName.'" id="'.$this->mId.'" value="'.sb_htmlspecialchars($this->mValue, ENT_QUOTES).'" readonly="readonly" style="'.($this->mShowTime ? 'width:110px;' : 'width:90px;').'height:25px;"'.($this->mDopStr != '' ? ' '.$this->mDopStr : '').'>'.($this->mRequired ? ' <sup class="red">*</sup>' : '').
              '&nbsp;<img class="button" src="'.SB_CMS_IMG_URL.'/calendar.gif" align="absmiddle" id="'.$this->mId.'_button" onmouseup="sbPress(this, false);" onmousedown="sbPress(this, true);" />'.
              ($this->mDropButton ? '&nbsp;<img class="button" src="'.SB_CMS_IMG_URL.'/calendar_drop.png" id="'.$this->mId.'_drop_button" align="absmiddle" width="20" height="20" onmouseup="sbPress(this, false);" onmousedown="sbPress(this, true);" onclick="var el = sbGetE(\''.$this->mId.'\'); if (!el.disabled) el.value=\'\';" />' : '').'
              <script>
                Calendar.setup({
                '.(!$this->mShowTime ? 'ifFormat : "%d.%m.%Y",' : '').'
                inputField     :    "'.$this->mId.'",
                showsTime      :    '.($this->mShowTime ? 'true' : 'false').',
                button         :    "'.$this->mId.'_button"
                });
              </script></nobr>'.$this->mHTML;
        }
    }

    /**
     * Возвращает JavaScript-код поля
     *
     * @return string JavaScript-код поля.
     */
    public function getJavaScript()
    {
        static $echo = true;

        if ($echo)
        {
            $echo = false;
            return '<link rel="stylesheet" type="text/css" href="'.SB_CMS_CSS_URL.'/sbCalendar.css" />
                <script src="'.SB_CMS_JSCRIPT_URL.'/sbCalendar.js.php"></script>
                <script src="'.SB_CMS_LANG_URL.'/calendar.js.php"></script>
                <script src="'.SB_CMS_JSCRIPT_URL.'/sbCalendarS.js.php"></script>';
        }
        else
        {
            return '';
        }
    }
}
?>