<?php
/**
 * Класс, отвечающий за вывод поля выбора файла с сервера
 *
 * @see sbLayout
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbLayoutElement
{
    /**
     * Идентификатор плагина
     *
     * @var string
     */
    protected $mPlugin = '';

    /**
     * Значение поля по умолчанию (атрибут <i>value</i>)
     *
     * @var string
     */
    protected $mValue = '';

    /**
     * Имя поля (значение атрибута <i>name</i>)
     *
     * @var string
     */
    protected $mName = '';

    /**
     * Идентификатор поля (значение атрибута <i>id</i>)
     *
     * @var string
     */
    protected $mId = '';

    /**
     * Дополнительные св-ва поля (например, стили)
     *
     * @var string
     */
    protected $mDopStr = '';

    /**
     * Дополнительный HTML-код, вставляемый после поля
     *
     * @var string
     */
    public $mHTML = '';

    /**
     * Поле обязательно для заполнения (TRUE) или нет (FALSE)
     *
     * @var bool
     */
    protected $mRequired = false;

    /**
     * Поле только для чтения
     *
     * @var bool
     */
    public $mReadOnly = false;

    /**
     * Пользовательская функция, которая будет вызываться при клике по кнопке "Обзор"
     * @var string
     */
    public $mCustomJS = '';

    /**
     * Массив идентификаторов разделов, которые надо выводить
     * @var array
     */
    public $mOnlyCats = array();

   /**
    * Массив идентификаторов разделов, запрещенных для вывода
    * @var array
    */
    public $mNeverCats = array();

    /**
     * Конструктор класса
     *
     * @param string $value ID элемента по умаолчанию(значение атрибута <i>value</i>).
     * @param string $name Имя поля (значение атрибута <i>name</i>).
     * @param string $id Идентификатор поля (значение атрибута <i>id</i>).
     * @param string $dop_str Дополнительные св-ва поля (например, стили).
     * @param bool $required Поле обязательно для заполнения (TRUE) или нет (FALSE).
     */
    public function __construct($ident, $value='', $name='', $id='', $dop_str='', $required=false)
    {
        if ($id == '') $id = $name;

        $this->mPlugin = $ident;
        $this->mValue = $value;
        $this->mName = $name;
        $this->mId = $id;
        $this->mDopStr = $dop_str;
        $this->mRequired = $required;
    }

    /**
     * Возвращает HTML-код поля
     *
     * @return string HTML-код поля.
     */
    public function getField($read_only = false)
    {
        //echo '<pre>'; print_r($this->mOnlyCats);
        $func_name = ($this->mCustomJS == '')? 'sbGetElement' : $this->mCustomJS;
        $sql_params = getPluginSqlParams($this->mPlugin);
        // Если есть id - достаем название элемента
        $title = '';
        if($this ->mValue != '' && isset($sql_params['title']))
        {
            if($this->mPlugin != 'pl_faq')
            {
                $res = sql_query('SELECT '.$sql_params['title'].' FROM '.$sql_params['table'].' WHERE '.$sql_params['id'].' = ?d', $this->mValue);
                if($res)
                    $title = $res[0][0];
            }
            elseif($this->mPlugin == 'pl_faq')
            {
                    $res = sql_query('SELECT f_author, f_email FROM sb_faq WHERE f_id = ?d', $this->mValue);
                    if($res)
                    {
                        if(!empty($res[0][0]))
                            $title = $res[0][0];
                        elseif(!empty($res[0][1]))
                            $title = $res[0][1];
                    }
            }
        }

        $result = '<nobr><input type="text" name="'.$this->mName.'_title" id="'.$this->mId.'_title" value="'.sb_htmlspecialchars($title, ENT_QUOTES).'"'.($this->mDopStr != '' ? ' '.$this->mDopStr : '').' />
                <input type="hidden" name="'.$this->mName.'" id="'.$this->mId.'" value="'.sb_htmlspecialchars($this->mValue, ENT_QUOTES).'" />
                '.($this->mRequired ? ' <sup class="red">*</sup>' : '').
                ($this->mReadOnly || $read_only ? '' : '&nbsp;<img class="button" src="'.SB_CMS_IMG_URL.'/pages.png" width="20" height="20" align="absmiddle" id="'.$this->mId.'_button" onmouseup="sbPress(this, false);" onmousedown="sbPress(this, true);" onclick="'.$func_name.'(\''.$this->mId.'\', \''.$this->mPlugin.'\', '.(sb_strpos($this->mPlugin,'pl_plugin_') !== false ? '\'&pm_id='.sb_str_replace('pl_plugin_', '', $this->mPlugin).'\'' : '\'\'').')" title="'.KERNEL_BROWSE.'" />').
                '</nobr>';

        $result .= '<input id="'.$this->mId.'_onlycatids" type="hidden" value="'.((is_array($this->mOnlyCats) && !empty($this->mOnlyCats)) ? implode(',', $this->mOnlyCats) : '').'">';
        $result .= '<input id="'.$this->mId.'_nevercatids" type="hidden" value="'.((is_array($this->mNeverCats) && !empty($this->mNeverCats)) ? implode(',', $this->mNeverCats) : '').'">';

        $result .= $this->mHTML;

        return $result;
    }

    /**
     * Возвращает JavaScript-код поля
     *
     * @return string JavaScript-код поля.
     */
    public function getJavaScript()
    {
        static $echo = true;
        if ($echo)
        {
            $echo = false;

            return '<script>
                var sbElementElId = null;
                var sbElementElTitle = null;

                function sbGetElement(el_id, pl_id, pm_id)
                {
                    var el = sbGetE(el_id);
                    if (el.disabled)
                        return;

                    if (el)
                    {
                        sbElementElId = el_id;
                        sbElementElTitle = el_id+"_title";
                        var strPage = sb_cms_modal_dialog_file + "?event=pl_plugin_data_get_elements&ident="+pl_id+"&el_id="+el.value+pm_id
                        +"&cat_only_ids=" + sbGetE(el_id+"_onlycatids").value
                        +"&cat_never_ids=" + sbGetE(el_id+"_nevercatids").value
                        ;
                        var strAttr = "resizable=1,width="+(screen.availWidth - 50)+",height="+(screen.availHeight - 50);

                        sbShowModalDialog(strPage, strAttr, sbAfterGetElement);
                    }
                    //&cat_only_ids= //&cat_never_ids=
                }

                function sbAfterGetElement()
                {
                    if (sbModalDialog.returnValue && sbElementElId)
                    {
                        var el = sbGetE(sbElementElId);
                        var el_title = sbGetE(sbElementElTitle);
                        el.value = sbModalDialog.returnValue.id;
                        el_title.value = sbModalDialog.returnValue.title;
                        if (typeof(el.onchange) != "undefined" && el.onchange)
                            el.onchange();

                        if (typeof(el_title.onchange) != "undefined" && el_title.onchange)
                            el_title.onchange();
                    }
                }

                </script>';
        }
        else
        {
            return '';
        }
    }
}
?>