<?php
/**
 * Класс, отвечающий за вывод поля выбора файла с сервера
 *
 * @see sbLayout
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbLayoutFile
{
    /**
     * Выбор файла только с локального компьютера пользователя (TRUE)
     *
     * @var bool
     */
    public $mLocal = false;

    /**
     * Значение поля по умолчанию (атрибут <i>value</i>)
     *
     * @var string
     */
    protected $mValue = '';

    /**
     * Показывать ссылку на файл.
     *
     * @var bool
     */
    public $mShow = true;

    /**
     * Имя поля (значение атрибута <i>name</i>)
     *
     * @var string
     */
    protected $mName = '';

    /**
     * Идентификатор поля (значение атрибута <i>id</i>)
     *
     * @var string
     */
    protected $mId = '';

    /**
     * Дополнительные св-ва поля (например, стили)
     *
     * @var string
     */
    protected $mDopStr = '';

    /**
     * Дополнительный HTML-код, вставляемый после поля
     *
     * @var string
     */
    public $mHTML = '';

    /**
     * Поле обязательно для заполнения (TRUE) или нет (FALSE)
     *
     * @var bool
     */
    protected $mRequired = false;

    /**
     * Строка, содержащая расширения отображаемых файлов через пробел (например, gif jpg png)
     *
     * @var string
     */
    public $mFileTypes = '';

    /**
     * Поле только для чтения
     *
     * @var bool
     */
    public $mReadOnly = false;

    /**
     * Конструктор класса
     *
     * @param string $value Путь к файлу по умолчанию (значение атрибута <i>value</i>).
     * @param string $name Имя поля (значение атрибута <i>name</i>).
     * @param string $id Идентификатор поля (значение атрибута <i>id</i>).
     * @param string $dop_str Дополнительные св-ва поля (например, стили).
     * @param bool $required Поле обязательно для заполнения (TRUE) или нет (FALSE).
     */
    public function __construct($value='', $name='', $id='', $dop_str='', $required=false)
    {
        if ($id == '') $id = $name;

        $this->mValue = $value;
        $this->mName = $name;
        $this->mId = $id;
        $this->mDopStr = $dop_str;
        $this->mRequired = $required;
    }

    /**
     * Возвращает HTML-код поля
     *
     * @return string HTML-код поля.
     */
    public function getField($read_only = false)
    {
        if($this->mLocal)
        {
            $return = '<nobr><input type="file" name="'.$this->mName.'" id="'.$this->mId.'" '.
            ($this->mDopStr != '' ? ' '.$this->mDopStr : '').
            ($this->mReadOnly || $read_only ? ' disabled="disabled"' : '')
            .' />'.($this->mRequired ? ' <sup class="red">*</sup>' : '').'</nobr>';

            if ($this->mShow && $this->mValue != '')
            {
                if (SB_HTTPS)
                {
                    $href = str_ireplace(array('http://www.'.SB_COOKIE_DOMAIN, 'http://'.SB_COOKIE_DOMAIN), SB_DOMAIN, $this->mValue);
                    $return .= '<br /><br /><div><a target="_blank" href="'.$href.'" id="'.$this->mId.'_file">'.$href.'</a></div>';
                }
                else
                {
                    $return .= '<br /><br /><div><a target="_blank" href="'.$this->mValue.'" id="'.$this->mId.'_file">'.$this->mValue.'</a></div>';
                }
            }
            if (!$this->mRequired && $this->mValue != '' && !$this->mReadOnly)
                $return .= '<br><input type="checkbox" value="1" id="'.$this->mId.'_delete" name="'.$this->mId.'_delete" onclick="var el = sbGetE(\''.$this->mId.'\');el.disabled=this.checked;el.value=\'\';"> <label for="'.$this->mId.'_delete">'.SB_LAYOUT_DELETE_FILE.'</label>';

            return $return.$this->mHTML;
        }
        else
        {
            return '<nobr><input type="text" name="'.$this->mName.'" id="'.$this->mId.'" value="'.sb_htmlspecialchars($this->mValue, ENT_QUOTES).'"'.($this->mDopStr != '' ? ' '.$this->mDopStr : '').' />
                '.($this->mRequired ? ' <sup class="red">*</sup>' : '').
                ($this->mReadOnly || $read_only ? '' : '&nbsp;<img class="button" src="'.SB_CMS_IMG_URL.'/images.png" width="20" height="20" align="absmiddle" id="'.$this->mId.'_button" onmouseup="sbPress(this, false);" onmousedown="sbPress(this, true);" onclick="sbGetFile(\''.$this->mId.'\', \''.$this->mFileTypes.'\')" title="'.KERNEL_BROWSE.'" />').
                ($this->mReadOnly || $read_only ? '' : '&nbsp;<img class="button" src="'.SB_CMS_IMG_URL.'/btn_delete.png" id="'.$this->mId.'_drop_button" align="absmiddle" width="20" height="20" onmouseup="sbPress(this, false);" onmousedown="sbPress(this, true);" onclick="var el = sbGetE(\''.$this->mId.'\'); if (!el.disabled) {el.value=\'\'; el.onchange();}" />').
                '</nobr>'.$this->mHTML;
        }
    }

    /**
     * Возвращает JavaScript-код поля
     *
     * @return string JavaScript-код поля.
     */
    public function getJavaScript()
    {
        static $echo = true;

        if ($echo)
        {
            $echo = false;
            return '<script>
                var sbFileElId = null;
                function sbGetFile(el_id, types)
                {
                    var el = sbGetE(el_id);
                    if (el.disabled)
                        return;

                    if (el)
                    {
                        sbFileElId = el_id;
                        var strPage = sb_cms_modal_dialog_file + "?event=pl_filelist_get_files&path="+sbEscapeEx(el.value)+"&types=" + types + "&fld_id=" + el_id;
                        var strAttr = "resizable=1,width="+(screen.availWidth - 200)+",height="+(screen.availHeight - 200);

                        sbShowModalDialog(strPage, strAttr, sbAfterGetFile);
                    }
                }

                function sbAfterGetFile()
                {
                    if (sbModalDialog.returnValue && sbFileElId)
                    {
                        var el = sbGetE(sbFileElId);
                        el.value = sbModalDialog.returnValue.path;
                        if (typeof(el.onchange) != "undefined" && el.onchange)
                            el.onchange();
                    }
                }
                </script>';
        }
        else
        {
            return '';
        }
    }
}
?>