<?php
/**
 * Класс, отвечающий за вывод дерева выбора папки с сервера
 *
 * @see sbLayout
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbLayoutFolder
{
    /**
     * Значение поля по умолчанию (атрибут <i>value</i>)
     *
     * @var string
     */
    private $mValue = '';

    /**
     * Имя поля (значение атрибута <i>name</i>)
     *
     * @var string
     */
    private $mName = '';

    /**
     * Идентификатор поля (значение атрибута <i>id</i>)
     *
     * @var string
     */
    private $mId = '';

    /**
     * Дополнительные св-ва поля (например, стили)
     *
     * @var string
     */
    private $mDopStr = '';

    /**
     * Отображать или нет кнопку "Сбросить значение"
     *
     * @var bool
     */
    public $mDropButton = false;

    /**
     * Дополнительный HTML-код, вставляемый после поля
     *
     * @var string
     */
    public $mHTML = '';

    /**
     * Поле обязательно для заполнения (TRUE) или нет (FALSE)
     *
     * @var bool
     */
    private $mRequired = false;

    /**
     * Конструктор класса
     *
     * @param string $value Путь к файлу по умолчанию (значение атрибута <i>value</i>).
     * @param string $name Имя поля (значение атрибута <i>name</i>).
     * @param string $id Идентификатор поля (значение атрибута <i>id</i>).
     * @param string $dop_str Дополнительные св-ва поля (например, стили).
     * @param bool $required Поле обязательно для заполнения (TRUE) или нет (FALSE).
     */
    public function __construct($value='', $name='', $id='', $dop_str='', $required=false)
    {
        if ($id == '') $id = $name;

        $this->mValue = $value;
        $this->mName = $name;
        $this->mId = $id;
        $this->mDopStr = $dop_str;
        $this->mRequired = $required;
    }

    /**
     * Возвращает HTML-код поля
     *
     * @return string HTML-код поля.
     */
    public function getField()
    {
        return '<nobr><input type="text" name="'.$this->mName.'" id="'.$this->mId.'" value="'.sb_htmlspecialchars($this->mValue, ENT_QUOTES).'"'.($this->mDopStr != '' ? ' '.$this->mDopStr : '').' />
               '.($this->mRequired ? ' <sup class="red">*</sup>' : '').'&nbsp;
               &nbsp;<img class="button" src="'.SB_CMS_IMG_URL.'/files.png" width="20" height="20" align="absmiddle" id="'.$this->mId.'_button" onmouseup="sbPress(this, false);" onmousedown="sbPress(this, true);" onclick="sbGetFolder(\''.$this->mId.'\');" title="'.KERNEL_BROWSE.'">
               '.($this->mDropButton ? '<img class="button" src="'.SB_CMS_IMG_URL.'/files_drop.png" width="20" height="20" id="'.$this->mId.'_drop_button" align="absmiddle" onmouseup="sbPress(this, false);" onmousedown="sbPress(this, true);" onclick="sbFolderDrop(\''.$this->mId.'\');"></nobr>' : '</nobr>').$this->mHTML;
    }

    /**
     * Возвращает JavaScript-код поля
     *
     * @return string JavaScript-код поля.
     */
    public function getJavaScript()
    {
        static $echo = true;

        if ($echo)
        {
            $echo = false;
            return '<script>
                var sbFolderElId = null;
                function sbGetFolder(el_id)
                {
                    var el = sbGetE(el_id);
                    if (el.disabled)
                        return;

                    if (el)
                    {
                        sbFolderElId = el_id;
                        var strPage = sb_cms_modal_dialog_file + "?event=pl_folders_get_folder&path="+sbEscapeEx(el.value);
                        var strAttr = "resizable=1,width=500,height=700";

                        sbShowModalDialog(strPage, strAttr, sbAfterGetFolder);
                    }
                }
                function sbAfterGetFolder()
                {
                    if (sbModalDialog.returnValue && sbFolderElId)
                    {
                        var el = sbGetE(sbFolderElId);
                        el.value = sbModalDialog.returnValue.path;
                        if (typeof(el.onchange) != "undefined" && el.onchange)
                            el.onchange();
                    }
                }
                function sbFolderDrop(el_id)
                {
                    var el = sbGetE(el_id);
                    if (el.disabled)
                        return;
                    if (el)
                    {
                        el.value = "";
                    }
                }
                </script>';
        }
        else
        {
            return '';
        }
    }
}
?>