<?php
/**
 * Класс, отвечающий за вывод графиков
 *
 * @see sbLayout
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2009, OOO "СИБИЭС Групп"
 */
class sbLayoutGraph
{
    /**
     * Текст по умолчанию (значение атрибута <i>value</i>)
     *
     * @var string
     */
    private $mValue = '';

    /**
     * Идентификатор поля (значение атрибута <i>id</i>)
     *
     * @var string
     */
    private $mId = '';

    /**
     * Ширина графика
     *
     * @var int
     */
    private $mWidth = 600;

    /**
     * Высота графика
     *
     * @var int
     */
    private $mHeight = 300;

    /**
     * Дополнительные св-ва поля (например, стили)
     *
     * @var string
     */
    private $mDopStr = '';

    /**
     * Выводить HTML-код внутри ячейки (false) или полностью строкой без вывода заголовка поля (true)
     *
     * @var bool
     */
    private $mFull = true;

    /**
     * Инкремент для формирования идентификатора поля
     *
     * @var int
     */
    private static $mIncr = 0;

    /**
     * Дополнительный HTML-код, вставляемый после поля
     *
     * @var string
     */
    public $mHTML = '';

    /**
     * Отображать или нет двоеточие в столбце заголовков полей
     *
     * @var bool
     */
    public $mShowColon = true;

    /**
     * URL для подгрузки данных графика
     *
     * @var string
     */
    public $mAutoloadingUrl = '';

    /**
     * Конструктор класса
     *
     * @param string $value Текст по умолчанию (значение атрибута <i>value</i>).
     * @param string $id Идентификатор поля (значение атрибута <i>id</i>).
     * @param string $width Ширина графика.
     * @param string $height Высота графика.
     * @param string $dop_str Дополнительные св-ва поля (например, стили).
     */
    public function __construct($value='', $id='', $width=600, $height=300, $dop_str='', $full=true)
    {
        if (trim($id) == '')
        {
            $id = 'sb_layout_graph_'.self::$mIncr;
            self::$mIncr++;
        }

        $this->mValue = $value;
        $this->mId = $id;
        $this->mWidth = intval($width);
        $this->mHeight = intval($height);
        $this->mDopStr = $dop_str;
        $this->mFull = $full;
    }

    /**
     * Возвращает HTML-код поля
     *
     * @param int      $title_width Ширина столбца заголовка.
     * @param string   $title Наименование заголовка
     * @param bool     $show_title Отображать или нет столбец заголовков полей.
     * @param string   $tr_str Доп. свойства тегов <i>tr</i>.
     * @param string   $th_str Доп. свойства тегов <i>th</i>.
     * @param string   $td_str Доп. свойства тегов <i>td</i>.
     *
     * @return string  HTML-код поля.
     */
    public function getFull($title_width, $title, $show_title, $tr_str, $th_str, $td_str)
    {
        if($this->mFull)
        {
            $colspan = '<td colspan="2"'.($td_str != '' ? ' '.$td_str : '').'>';
        }
        else
        {
            $colspan = ($show_title ? '<th width="'.$title_width.'"'.($th_str != '' ? ' '.$th_str : '').'>
                        <img src="'.SB_CMS_IMG_URL.'/blank.gif" height="1" width="'.$title_width.'" /><br />
                        '.$title.($this->mShowColon && $title != '' ? ':' : '').'</th><td'.($td_str != '' ? ' '.$td_str : '').'>' : '');
        }

        $result = '<tr '.($tr_str != '' ? ' '.$tr_str : '').'>'.$colspan.'<div id="'.$this->mId.'"'.($this->mDopStr != '' ? ' '.$this->mDopStr : '').'>
        <div class="hint_div">
            '.SB_LAYOUT_GET_FLASH_HINT.'
            <br /><br /><div align="center"><a href="http://www.adobe.com/go/getflashplayer" target="_blank"><img src="'.SB_CMS_IMG_URL.'/get_flash.png" border="0" width="158" height="39" alt="'.SB_LAYOUT_GET_FLASH.'" /></a></div>
        </div>
        </div>'.$this->mHTML.'
        <script>
            function sb_lgl_'.$this->mId.'()
            {
                swfobject.embedSWF("'.SB_CMS_IMG_URL.'/swf/sbChart.swf",
                "'.$this->mId.'", "'.$this->mWidth.'", "'.$this->mHeight.'",
                "9.0.0", "'.SB_CMS_IMG_URL.'/swf/sbExpressInstall.swf",';

        if ($this->mAutoloadingUrl != '')
        {
            $result .= '{"data-file":"'.urlencode($this->mAutoloadingUrl).'", "id":"'.$this->mId.'", "loading":"'.KERNEL_LOADING.'"}, {menu:"false"});
            }';
        }
        else
        {
            $result .= '{"get-data":"sb_lgf_'.$this->mId.'", "id":"'.$this->mId.'", "loading":"'.KERNEL_LOADING.'"}, {menu:"false"});
            }

            function sb_lgf_'.$this->mId.'()
            {
                return \''.str_replace('\'', '\\"', $this->mValue).'\';
            }';
        }

        $result .= '
        sbAddEvent(window, "load", sb_lgl_'.$this->mId.');
        </script></td></tr>';

        return $result;
    }

    /**
     * Возвращает JavaScript-код поля
     *
     * @return string JavaScript-код
     */
    public function getJavaScript()
    {
        static $echo = true;
        if($echo)
        {
            $echo = false;
            return '<script src="'.SB_CMS_JSCRIPT_URL.'/sbSWFObject.js.php"></script>';
        }

        return '';
    }
}
?>