<?php
/**
 * Класс, отвечающий за вывод произвольного HTML-кода в форме
 *
 * @see sbLayout
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbLayoutHTML
{
    /**
     * Выводимый HTML-код формы
     *
     * @var string
     */
    private $mHtml = '';

    /**
     * Выводить HTML-код внутри ячейки или полностью строкой без вывода заголовка поля
     *
     * @var bool
     */
    private $mFull = false;

    /**
     * Отображать или нет двоеточие в столбце заголовков полей
     *
     * @var bool
     */
    public $mShowColon = true;

    /**
     * Конструктор класса
     *
     * @param string $html Выводимый HTML-код формы.
     * @param bool $full Выводить HTML-код внутри ячейки (false) или полностью строкой без вывода заголовка поля (true).
     */
    public function __construct($html, $full=false)
    {
        $this->mHtml = $html;
        $this->mFull = $full;
    }

    /**
     * Возвращает HTML-код поля
     *
     * @param int $title_width Ширина столбца заголовка поля.
     * @param string $title Заголовок поля.
     * @param bool $show_title Отображать или нет столбец заголовков полей.
     * @param string $tr_str Доп. свойства тегов <i>tr</i>.
     * @param string $th_str Доп. свойства тегов <i>th</i>.
     * @param string $td_str Доп. свойства тегов <i>td</i>.
     *
     * @return string HTML-код поля.
     */
    public function getFull($title_width, $title, $show_title, $tr_str, $th_str, $td_str)
    {
        if($this->mFull)
        {
            return '<tr'.($tr_str != '' ? ' '.$tr_str : '').'>
                      <td width="100%" colspan="2"'.($td_str != '' ? ' '.$td_str : '').'>'.$this->mHtml.'</td>
                    </tr>';
        }
        else
        {
            return '<tr'.($tr_str != '' ? ' '.$tr_str : '').'>'.
                      ($show_title ? '<th width="'.$title_width.'"'.($th_str != '' ? ' '.$th_str : '').'><img src="'.SB_CMS_IMG_URL.'/blank.gif" height="1" width="'.$title_width.'" /><br />'.$title.($this->mShowColon && $title != '' ? ':' : '').'</th>' : '').
                      '<td width="100%"'.($td_str != '' ? ' '.$td_str : '').'>'.$this->mHtml.'</td>
                    </tr>';
        }
    }
}
?>