<?php
/**
 * Класс, отвечающий за вывод поля выбора изображения с сервера или локально
 *
 * @see sbLayout
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbLayoutImage extends sbLayoutFile
{
    /**
     * Выбор изображения только с локального компьютера пользователя (TRUE)
     *
     * @var bool
     */
    public $mLocal = false;

    /**
     * Показывать изображение
     *
     * @var bool
     */
    public $mShow = true;

    /** Дополнительный HTML-код, вставляемый после поля
     *
     * @var string
     */
    public $mHTML = '';

    /**
     * Поле только для чтения
     *
     * @var bool
     */
    public $mReadOnly = false;

    private $upload;

    private $settings;

    /**
     * Конструктор класса
     *
     * @param string $value Путь к файлу с изображением по умолчанию (значение атрибута <i>value</i>).
     * @param string $name Имя поля (значение атрибута <i>name</i>).
     * @param string $id Идентификатор поля (значение атрибута <i>id</i>).
     * @param string $dop_str Дополнительные св-ва поля (например, стили).
     * @param bool $required Поле обязательно для заполнения (TRUE) или нет (FALSE).
     */
    public function __construct($value='', $name='', $id='', $dop_str='', $required=false, $settings=array())
    {
        sbLayoutFile::__construct($value, $name, $id, $dop_str, $required);
        $this->upload = new sbLayoutUpload('pl_folders_upload_file', $this->mName.'_local', 'local-'.$this->mId, $this->mDopStr.($this->mReadOnly ? ' disabled="disabled"' : ''), $this->mRequired);
        $this->upload->mValue = $value;
        $this->settings = $settings;
    }

    /**
     * Возвращает HTML-код поля
     *
     * @return string HTML-код поля.
     */
    public function getField($read_only = false)
    {
        $f_img = '';
        $f_img_delete = '';
        if ($this->mShow)
        {
            if ($this->mValue == '')
            {
                $f_img = '<img src="'.SB_CMS_IMG_URL.'/noimage.png" width="88" height="88" id="'.$this->mId.'_img" border="0" style="margin: 5px;">';
            }
            else
            {
                $ar = false;
                if (stripos($this->mValue, 'http://') === false && stripos($this->mValue, 'https://') === false)
                {
                    if ($GLOBALS['sbVfs']->exists($this->mValue))
                        $ar = $GLOBALS['sbVfs']->getimagesize($this->mValue);
                }
                else
                {
                    $ar = @getimagesize($this->mValue);
                }

                if (!$ar || !in_array($ar[2], array(1,2,3)))
                {
                    // не картинка
                    $f_img = '<img src="'.SB_CMS_IMG_URL.'/noimage.png" width="88" height="88" id="'.$this->mId.'_img" border="0" style="margin: 5px;">';
                }
                else
                {
                    if (SB_HTTPS)
                        $f_img = '<img src="'.str_ireplace(array('http://www.'.SB_COOKIE_DOMAIN, 'http://'.SB_COOKIE_DOMAIN), SB_DOMAIN, $this->mValue).'" id="'.$this->mId.'_img" width="'.min(150, $ar[0]).'" border="0" style="margin: 5px;">';
                    else
                        $f_img = '<img src="'.$this->mValue.'" id="'.$this->mId.'_img" width="'.min(150, $ar[0]).'" border="0" style="margin: 5px;">';
                }
            }
        }

        $checked = (!isset($this->settings['parent_field']) || $this->settings['parent_field'] == '' || $this->upload->mValue != '')? 'checked="checked"' : '';

        if(!empty($this->settings))
        {
            $fld1 = new sbLayoutInput('radio', 'local', $this->mName.'_type', $this->mName.'_type_1', ' onclick="switchType(this)"');
            $switcher_local = $fld1->getField().'<label for="'.$this->mName.'_type_1">'.SB_LAYOUT_UPLOAD_TYPE_LOCAL.'</label>';
        }
        $fld2 = new sbLayoutInput('radio', 'server', $this->mName.'_type', $this->mName.'_type_2', $checked.' onclick="switchType(this)"');
        $switcher_server = $fld2->getField().'<label for="'.$this->mName.'_type_2">'.SB_LAYOUT_UPLOAD_TYPE_SERVER.'</label>';
        $switcher_generate = '';

        if(isset($this->settings['parent_field']) && $this->settings['parent_field'] != '')
        {
            $fld3 = new sbLayoutInput('radio', 'generate', $this->mName.'_type', $this->mName.'_type_3', ($this->upload->mValue == ''? 'checked="checked"' : '').' onclick="switchType(this)"');
            $switcher_generate = $fld3->getField().'<label for="'.$this->mName.'_type_3">'.SB_LAYOUT_UPLOAD_TYPE_GENERATE.'</label>';
            $tmp1 = new sbLayoutInput('text', $this->settings['img_width'], $this->mName.'_generate_width', 'spin_'.$this->mName.'_generate_width', 'style="width: 50px"');
            $tmp1->mMinValue = 1;
            $tmp2 = new sbLayoutInput('text', $this->settings['img_height'], $this->mName.'_generate_height', 'spin_'.$this->mName.'_generate_height', 'style="width: 50px"');
            $tmp2->mMinValue = 1;
            $fld_generate = '<table>
                <tr>
                    <td>'.SB_LAYOUT_VALUE_WIDTH.'</td>
                    <td>'.$tmp1->getField().'</td>
                </tr>
                <tr>
                    <td>'.SB_LAYOUT_VALUE_HEIGHT.'</td>
                    <td>'.$tmp2->getField().'</td>
                </tr>
            </table>';
        }

        if ($this->mShow)
            {
                if (preg_match('/[\s]*onchange[\s]*=[\s]*[\'|"](.*)[\'|"]/i', $this->mDopStr))
                {
                    $this->mDopStr = preg_replace('/([\s]*onchange[\s]*=[\s]*[\'|"])(.*)([\'|"])/i', '$1sbSetFoto(this); $2$3', $this->mDopStr);
                }
                else
                {
                    $this->mDopStr .= ' onchange="sbSetFoto(this);"';
                }
            }
            $tmp = $this->mHTML;
            $this->mHTML = '';
            $old = $this->mLocal;
            $this->mLocal = false;
            $fld_server = sbLayoutFile::getField($this->mReadOnly);
            $this->mLocal = $old;
            $this->mHTML = $tmp;


            $over_ext = array_diff(explode(' ', $this->mFileTypes), array('png', 'gif', 'jpg', 'jpeg'));

            $fld_local = $this->upload->getField();

            //if (!$this->mRequired && $this->mValue != '' && !$this->mReadOnly)
            //       $f_img_delete = '<br/><input type="checkbox" value="1" id="'.$this->mId.'_delete" name="'.$this->mId.'_delete" onclick="fldDisable(\''.$this->mId.'\');"> <label for="'.$this->mId.'_delete">'.SB_LAYOUT_DELETE_FOTO.'</label>';

        $result = '<div id="'.$this->mName.'_local_fld"  style="display: none">'
            .$fld_local.$f_img_delete
            .'</div><div id="'.$this->mName.'_server_fld" '.((isset($fld_generate) && $this->upload->mValue == '')? 'style="display: none"' : '').'>'.$fld_server.'</div>';
        if(isset($fld_generate))
        {
            $result .= '<div id="'.$this->mName.'_generate_fld"'.($this->upload->mValue != ''? ' style="display: none"' : '').'>'.$fld_generate.'</div>';
        }
        else
        {
            $result .= '<div id="'.$this->mName.'_generate_fld" style="display: none"></div>';
        }
        //$result .= $f_img_delete;
        if($this->mShow)
        {
            $result .= '<div>'.$f_img.'</div>';
        }

        return (isset($switcher_local) ? $switcher_local.'<br/>' : '').(isset($switcher_server) ? $switcher_server.'<br/>' : '').'<br/>'.$switcher_generate.' '.$result.$this->mHTML;
    }

    /**
     * Возвращает JavaScript-код поля
     *
     * @return string JavaScript-код поля.
     */
    public function getJavaScript()
    {
        static $echo = true;

        if ($echo)
        {
            $echo = false;
            $result = sbLayoutFile::getJavaScript();
            $result .= $this->upload->getJavaScript();

            $result .= '<script>
                function sbSetFoto(el)
                {
                    var id = el.id+"_img";
                    var img = sbGetE(id);
                    if (img)
                    {
                        img.onload = function()
                        {
                            this.onload = null;
                            if (this.height > this.width)
                            {
                                if (this.height > 150)
                                    this.height = 150;

                                this.removeAttribute("width", 0);
                            }
                            else
                            {
                                if (this.width > 150)
                                    this.width = 150;

                                this.removeAttribute("height", 0);
                            }
                        }

                        img.src = (el.value != "" ? el.value : "'.SB_CMS_IMG_URL.'/noimage.png");
                        img.removeAttribute("height", 0);
                        img.removeAttribute("width", 0);
                    }
                }

                function switchType(obj)
                {
                    var id = obj.getAttribute("id");
                    var ids = id.split("_");
                    var user_f = id.replace(/_type_\d/, "");

                    if(ids[ids.length-1] == 1)
                    {
                        sbGetE(user_f+"_server_fld").style.display = "none";
                        sbGetE(user_f+"_local_fld").style.display = "block";
                        sbGetE(user_f+"_generate_fld").style.display = "none";
                    }
                    else if(ids[ids.length-1] == 2)
                    {
                        sbGetE(user_f+"_local_fld").style.display = "none";
                        sbGetE(user_f+"_server_fld").style.display = "block";
                        sbGetE(user_f+"_generate_fld").style.display = "none";
                    }
                    else if(ids[ids.length-1] == 3)
                    {
                        sbGetE(user_f+"_local_fld").style.display = "none";
                        sbGetE(user_f+"_server_fld").style.display = "none";
                        sbGetE(user_f+"_generate_fld").style.display = "block";
                    }
                }

                function fldDisable(id)
                {
                    var checked = sbGetE(id+"_delete").checked;
                    var fld_local = sbGetE(id+"_local");
                    var fld_server = sbGetE(id);
                    fld_local.disabled = checked;
                    fld_local.value = "";
                    fld_server.disabled = checked;
                    fld_server.value ="";
                    sbGetE(id+"_type_1").disabled = checked;
                    sbGetE(id+"_type_2").disabled = checked;
                }
                </script>';

            return $result;
        }
        else
        {
            return '';
        }
    }
}
?>