<?php
/**
 * Класс, отвечающий за вывод полей (input) формы
 *
 * @see sbLayout
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbLayoutInput
{
    /**
     * Тип поля (значение атрибута <i>type</i>)
     *
     * @var string
     */
    private $mType = 'text';

    /**
     * Текст по умолчанию (значение атрибута <i>value</i>)
     *
     * @var string
     */
    private $mValue = '';

    /**
     * Имя поля (значение атрибута <i>name</i>)
     *
     * @var string
     */
    private $mName = '';

    /**
     * Идентификатор поля (значение атрибута <i>id</i>)
     *
     * @var string
     */
    private $mId = '';

    /**
     * Дополнительные св-ва поля (например, стили)
     *
     * @var string
     */
    private $mDopStr = '';

    /**
     * Выводить или нет Счетчик символов
     *
     * @var boolean
     */
    public $mShowCharCount = false;

    /**
     * Дополнительный HTML-код, вставляемый после поля
     *
     * @var string
     */
    public $mHTML = '';

    /**
     * Поле обязательно для заполнения (TRUE) или нет (FALSE)
     *
     * @var bool
     */
    private $mRequired = false;

    /**
     * Для цифрового типа данных (spin) - минимальное значение
     *
     * @var integer
     */
    public $mMinValue = '';

    /**
     * Для цифрового типа данных (spin) - максимальное значение
     *
     * @var integer
     */
    public $mMaxValue = '';

    /**
     * Для цифрового типа данных (spin) - значение инкремента
     *
     * Если инкремент - значение с точкой, то в поле разрешено использовать дробные числа. Кол-во цифр после точки в значении инкремента
     * задает количество цифр после точки в значении поля.
     *
     * @var float
     */
    public $mIncrement = '1';

    /**
     * Конструктор класса
     *
     * @param string $type Тип поля (значение атрибута <i>type</i>)
     * @param string $value Текст по умолчанию (значение атрибута <i>value</i>).
     * @param string $name Имя поля (значение атрибута <i>name</i>).
     * @param string $id Идентификатор поля (значение атрибута <i>id</i>).
     * @param string $dop_str Дополнительные св-ва поля (например, стили).
     * @param bool $required Поле обязательно для заполнения (TRUE) или нет (FALSE).
     */
    public function __construct($type, $value='', $name='', $id='', $dop_str='', $required=false)
    {
        if ($id == '') $id = $name;

        $this->mType = $type;
        $this->mValue = $value;
        $this->mName = $name;
        $this->mId = $id;
        $this->mDopStr = $dop_str;
        $this->mRequired = $required;
    }

    /**
     * Возвращает тип поля
     *
     * @return string Тип поля.
     */
    public function getType()
    {
        return $this->mType;
    }

    /**
     * Возвращает HTML-код поля
     *
     * @return string HTML-код поля.
     */
    public function getField()
    {
        $str = '';
        $res = '';
        if (substr($this->mId, 0, 5) == 'spin_')
        {
            $str .= ($this->mMinValue !== '' ? ' min_value="'.sb_htmlspecialchars($this->mMinValue, ENT_QUOTES).'"' : '');
            $str .= ($this->mMaxValue !== '' ? ' max_value="'.sb_htmlspecialchars($this->mMaxValue, ENT_QUOTES).'"' : '');
            $str .= ($this->mIncrement !== '' ? ' incr="'.sb_htmlspecialchars(str_replace(',', '.', $this->mIncrement), ENT_QUOTES).'"' : '1');

            $res = '<table cellpadding="0" cellspacing="0"><tr style="background-color: transparent;"><td><input type="'.$this->mType.'"'.$str.' value="'.sb_htmlspecialchars($this->mValue, ENT_QUOTES).'"'.($this->mName != '' ? ' name="'.$this->mName.'"' : '').($this->mId != '' ? ' id="'.$this->mId.'"' : '').' '.$this->mDopStr.' /></td>'.($this->mRequired ? '<td>&nbsp;<sup class="red">*</sup></td>' : '').'</tr></table>';
        }
        else
        {
            $res = '<input type="'.$this->mType.'"'.$str.' value="'.sb_htmlspecialchars($this->mValue, ENT_QUOTES).'"'.($this->mName != '' ? ' name="'.$this->mName.'"' : '').($this->mId != '' ? ' id="'.$this->mId.'"' : '').' '.$this->mDopStr.' '.($this->mShowCharCount && is_numeric($this->mMaxValue) && $this->mMaxValue > 0 ? ' oninput = "sbLayoutCharCount(this, \''.$this->mId.'\',\''.$this->mMaxValue.'\')" onfocus = "sbLayoutCharCount(this, \''.$this->mId.'\',\''.$this->mMaxValue.'\')" onkeyup="sbLayoutCharCount(this, \''.$this->mId.'\',\''.$this->mMaxValue.'\')"' : '').'  />
            '.($this->mShowCharCount ? '<div style="padding-top:3px; float:left;">'.SB_LAYOUT_VALUE_CHAR_COUNT.' <span id="char_count_'.$this->mId.'" style="font-style:italic;">'.($this->mMaxValue - sb_strlen(sb_htmlspecialchars($this->mValue, ENT_QUOTES))).'</span></div>':'').'
            '.($this->mRequired ? ' <sup class="red">*</sup>' : '');
        }

        if($this->mShowCharCount && is_numeric($this->mMaxValue) && $this->mMaxValue > 0)
        {
            $res = $res.'
            <script>
                sbLayoutCharCount(sbGetE("'.$this->mId.'"), \''.$this->mId.'\',\''.$this->mMaxValue.'\');
            </script>
            ';
        }

        return $res.$this->mHTML;
    }

    /**
     * Возвращает JavaScript-код поля
     *
     * @return string JavaScript-код поля.
     */
    public function getJavaScript()
    {
        static $echo = true;

        if ($echo && substr($this->mId, 0, 5) == 'spin_')
        {
            $echo = false;
            return '<script src="'.SB_CMS_JSCRIPT_URL.'/sbSpinButton.js.php"></script>';
        }
        else
        {
            return '';
        }
    }
}
?>