<?php
/**
 * Класс, отвечающий за вывод простых текстов в форме (span)
 *
 * @see sbLayout
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbLayoutLabel
{
    /**
     * Выводимый текст
     *
     * @var string
     */
    private $mValue = '';

    /**
     * Идентификатор поля (значение атрибута <i>id</i>)
     *
     * @var string
     */
    private $mId = '';

    /**
     * Дополнительные св-ва поля (например, стили)
     *
     * @var string
     */
    private $mDopStr = '';

    /**
     * Применять функцию htmlscpecialchars к выводимому значению или нет
     *
     * @var bool
     */
    private $mUseSpecial = true;

    /**
     * Выводить как подсказку
     *
     * @var bool
     */
    public $mHint = false;

    /**
     * Конструктор класса
     *
     * @param string $value Выводимый текст.
     * @param string $id Идентификатор поля (значение атрибута <i>id</i>).
     * @param string $dop_str Дополнительные св-ва поля (например, стили).
     * @param string $use_special Переводить спец. символы в мнемоники (true) или нет (false).
     */
    public function __construct($value, $id='', $dop_str='', $use_special=true)
    {
        $this->mValue = $value;
        $this->mId = $id;
        $this->mDopStr = $dop_str;
        $this->mUseSpecial = $use_special;
    }

    /**
     * Возвращает HTML-код поля
     *
     * @return string HTML-код поля.
     */
    function getField()
    {
        if ($this->mHint)
            return '<div class="hint_div"'.($this->mId != '' ? ' id="'.$this->mId.'"' : '').' '.$this->mDopStr.'>'.($this->mUseSpecial ? sb_htmlspecialchars($this->mValue) : $this->mValue).'</div>';
        else
            return '<span'.($this->mId != '' ? ' id="'.$this->mId.'"' : '').' '.$this->mDopStr.'>'.($this->mUseSpecial ? sb_htmlspecialchars($this->mValue) : $this->mValue).'</span>';
    }
}
?>