<?php
/**
 * Класс, отвечающий за вывод поля выбора страницы
 *
 * @see sbLayout
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbLayoutPage
{
    /**
     * URL страницы по умолчанию
     *
     * @var string
     */
    private $mValue = '';

    /**
     * Имя поля (значение атрибута <i>name</i>)
     *
     * @var string
     */
    private $mName = '';

    /**
     * Идентификатор поля (значение атрибута <i>id</i>)
     *
     * @var string
     */
    private $mId = '';

    /**
     * Дополнительные св-ва поля (например, стили)
     *
     * @var string
     */
    private $mDopStr = '';

    /**
     * Дополнительный HTML-код, вставляемый после поля
     *
     * @var string
     */
    public $mHTML = '';

    /**
     * Поле обязательно для заполнения (TRUE) или нет (FALSE)
     *
     * @var bool
     */
    private $mRequired = false;

    /**
     * Поле доступно только для чтения
     *
     * @var bool
     */
    public $mReadOnly = false;

    /**
     * Конструктор класса
     *
     * @param string $value ID страницы по умолчанию (значение атрибута <i>value</i>).
     * @param string $name Имя поля (значение атрибута <i>name</i>).
     * @param string $id Идентификатор поля (значение атрибута <i>id</i>).
     * @param string $dop_str Дополнительные св-ва поля (например, стили).
     * @param bool $required Поле обязательно для заполнения (TRUE) или нет (FALSE).
     */
    public function __construct($value='', $name='', $id='', $dop_str='', $required=false)
    {
        if ($id == '') $id = $name;

        $this->mValue = $value;
        $this->mName = $name;
        $this->mId = $id;
        $this->mDopStr = $dop_str;
        $this->mRequired = $required;
    }

    /**
     * Возвращает HTML-код поля
     *
     * @return string HTML-код поля.
     */
    public function getField()
    {
        return '<nobr><input type="text" name="'.$this->mName.'" id="'.$this->mId.'" value="'.sb_htmlspecialchars($this->mValue, ENT_QUOTES).'"'.($this->mDopStr != '' ? ' '.$this->mDopStr : '').' />
               <input type="hidden" id="'.$this->mId.'_title" value="" />'.
               ($this->mRequired ? ' <sup class="red">*</sup>' : '').
               ($this->mReadOnly ? '' : '&nbsp;<img class="button" src="'.SB_CMS_IMG_URL.'/pages.png" width="20" height="20" align="absmiddle" id="'.$this->mId.'_button" onmouseup="sbPress(this, false);" onmousedown="sbPress(this, true);" onclick="sbGetPage(\''.$this->mId.'\')" />').'</nobr>'.$this->mHTML;
    }

    /**
     * Возвращает JavaScript-код поля
     *
     * @return string JavaScript-код поля.
     */
    public function getJavaScript()
    {
        static $echo = true;

        if ($echo)
        {
            $echo = false;
            return '<script>
                var sbPageElId = null;
                function sbGetPage(el_id)
                {
                    var el = sbGetE(el_id);
                    if (el.disabled)
                        return;

                    if (el)
                    {
                        sbPageElId = el_id;
                        var strPage = sb_cms_modal_dialog_file + "?event=pl_pages_get_page&page="+el.value;
                        var strAttr = "resizable=1,width="+(screen.availWidth - 100)+",height="+(screen.availHeight - 100);

                        sbShowModalDialog(strPage, strAttr, sbAfterGetPage);
                    }
                }

                function sbAfterGetPage()
                {
                    if (sbModalDialog.returnValue && sbPageElId)
                    {
                        var el = sbGetE(sbPageElId);
                        var el_title = sbGetE(sbPageElId + "_title");

                        el.value = sbModalDialog.returnValue.url;
                        el_title.value = sbModalDialog.returnValue.title;

                        if (typeof(el.onchange) != "undefined" && el.onchange)
                            el.onchange();
                    }
                }
                </script>';
        }
        else
        {
            return '';
        }
    }
}
?>