<?php
/**
 * Класс, отвечающий за вывод поля планирования
 *
 * @see sbLayout
 * @author Олисаев Азамат <azamat@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2009, OOO "СИБИЭС Групп"
 */
class sbLayoutPlanTable
{
    /**
     * Массив помеченных чекбоксов для дней недели
     *
     * @var array()
     */
    private $mValue = array();

    /**
     * Массив помеченных чекбоксов для месяцев
     *
     * @var array()
     */
    private $mValueMonth = array();

    /**
     * Выводить HTML-код внутри ячейки (false) или полностью строкой без вывода заголовка поля (true)
     *
     * @var bool
     */
    private $mFull = false;

    /**
     * Отображать или нет двоеточие в столбце заголовков полей
     *
     * @var bool
     */
    public $mShowColon = true;

    /**
     * Конструктор класса
     *
     * @param string $values  Строка отмеченных чекбоксов week_day_1,hour_1;week_day_2,hour_2;...||month_1;month_2;...
     *                        где week_day_1, ... день неделя от 1(Пн) до 7(Вс)
     *                        hour_1, ... час от 1(00:00) до 24(23:00)
     *                        month_1, ... номер месяца от 1(Январь) до 12(Декабрь)
     *
     * @param bool $full Выводить HTML-код внутри ячейки (false) или полностью строкой без вывода заголовка поля (true).
     */
    public function __construct($values, $full = false)
    {
        $day_ar = array();
        $month_ar = array();

        if (count($_POST) > 0)
        {
            // если нажата кнопка Применить
            foreach($_POST as $key => $value)
            {
                if(substr($key, 0, 5) == 'month')
                {
                    $month_ar[] = intval(preg_replace('/[^0-9]+/', '', $key));
                }
                elseif(substr($key, 0, 2) == 'dh')
                {
                    $str = substr($key, 3);
                    $d = intval(substr($str, 0, 1));
                    $h = intval(substr($str, 1));
                    $day_ar[$d.'_'.$h] = 1;
                }
            }
        }
        else
        {
            if ($values != '')
            {
                $values = explode('||', $values);

                $days = explode(';', $values[0]);
                $month_ar = explode(';', $values[1]);

                foreach($days as $value)
                {
                    if ($value == '')
                        continue;

                    $d_h = explode(',', $value);
                    if(count($d_h) > 1)
                    {
                        $day_ar[$d_h[0].'_'.$d_h[1]] = 1;
                    }
                }
            }
            else
            {
                for ($i = 1; $i <= 7; $i++)
                {
                    for ($j = 1; $j <= 24; $j++)
                    {
                        $day_ar[$i.'_'.$j] = 1;
                    }
                }

                for ($i = 1; $i <= 12; $i++)
                {
                    $month_ar[] = $i;
                }
            }
        }

        for ($i = 1; $i <= 7; $i++)
        {
            for ($j = 1; $j <= 24; $j++)
            {
                if (!isset($day_ar[$i.'_'.$j]) || $day_ar[$i.'_'.$j] != 1)
                    continue 2;
            }

            $day_ar['d_'.$i] = 1;
        }

        for ($i = 1; $i <= 24; $i++)
        {
            for ($j = 1; $j <= 7; $j++)
            {
                if (!isset($day_ar[$j.'_'.$i]) || $day_ar[$j.'_'.$i] != 1)
                    continue 2;
            }

            $day_ar['h_'.$i] = 1;
        }

        $this->mValue = $day_ar;
        $this->mFull = $full;
        $this->mValueMonth = $month_ar;
    }

    /**
     * Возвращает HTML-код поля
     *
     * @param int      $title_width Ширина столбца заголовка.
     * @param string   $title Наименование заголовка
     * @param bool     $show_title Отображать или нет столбец заголовков полей.
     * @param string   $tr_str Доп. свойства тегов <i>tr</i>.
     * @param string   $th_str Доп. свойства тегов <i>th</i>.
     * @param string   $td_str Доп. свойства тегов <i>td</i>.
     *
     * @return string  HTML-код поля.
     */
    public function getFull($title_width, $title, $show_title, $tr_str, $th_str, $td_str)
    {
        if($this->mFull)
        {
            $colspan = '<td colspan="2"'.($td_str != '' ? ' '.$td_str : '').'>';
        }
        else
        {
            $colspan = ($show_title ? '<th width="'.$title_width.'"'.($th_str != '' ? ' '.$th_str : '').'>
                        <img src="'.SB_CMS_IMG_URL.'/blank.gif" height="1" width="'.$title_width.'" /><br />
                        '.$title.($this->mShowColon && $title != '' ? ':' : '').'</th><td '.($td_str != '' ? ' '.$td_str : '').'>' : '');
        }

        $result = '<tr '.($tr_str != '' ? ' '.$tr_str : '').'>'.$colspan.'<table style=" margin-bottom:20px;"><tr '.($tr_str != '' ? ' '.$tr_str : '').'>';
        for ($i = 1; $i <= 12 ; $i++)
        {
            $mon = 'SB_LAYOUT_PLAN_MONTH_'.$i;
            $result .= '<td style="font-size:12px;" '.($td_str != '' ? ' '.$td_str : '').'>
                          '.constant($mon).'
                        </td>
                        <td style="padding-right:40px;" '.($td_str != '' ? ' '.$td_str : '').'>
                            <input type="checkbox" name="month_'.$i.'" value="1"'.(in_array($i, $this->mValueMonth) ? ' checked="checked"' : '').'>
                        </td>';

            if($i % 4 == 0)
            {
                $result .= '</tr><tr '.($tr_str != '' ? ' '.$tr_str : '').'>';
            }
        }

        $result .= '</tr></table>'.$this->getJavaScript().'
                  <table style="border-collapse:collapse;" cellpadding="0" cellspacing="0"><tr '.($tr_str != '' ? ' '.$tr_str : '').'>
                            <td colspan="2" '.($td_str != '' ? ' '.$td_str : '').'></td>
                            <td '.($td_str != '' ? ' '.$td_str : '').'>&nbsp;00:00</td>
                            <td '.($td_str != '' ? ' '.$td_str : '').'>&nbsp;04:00</td>
                            <td '.($td_str != '' ? ' '.$td_str : '').'>&nbsp;08:00</td>
                            <td '.($td_str != '' ? ' '.$td_str : '').'>&nbsp;12:00</td>
                            <td '.($td_str != '' ? ' '.$td_str : '').'>&nbsp;16:00</td>
                            <td '.($td_str != '' ? ' '.$td_str : '').'>&nbsp;20:00</td>
                    </tr><tr '.($tr_str != '' ? ' '.$tr_str : '').'>';

        if($GLOBALS['sb_browser_agent'] == 'IE')
        {
            $style_mar = '';
        }
        else
        {
            $style_mar = 'style="margin:1px;"';
        }

        for($i = 1; $i <= 7; $i++)
        {
            $con = 'SB_LAYOUT_PLAN_WEEK_DAY_'.$i;

            $result .= '<td'.($td_str != '' ? ' '.$td_str : '').' align="right" style="font-family: Arial,Sans-Serif; font-weight:bold; font-size:11px;">
                           '.constant($con).'
                        </td>
                        <td'.($td_str != '' ? ' '.$td_str : '').' style="padding: 0px 15px 0px 10px;">
                            <input type="checkbox" id="d_'.$i.'" onClick="sbActiveWeek(this.id);" '.(isset($this->mValue['d_'.$i]) && $this->mValue['d_'.$i] == 1 ? ' checked="checked"' : '').' />
                        </td><td '.($td_str != '' ? ' '.$td_str : '').' style="padding-right:15px"><nobr>';

            for($d = 1; $d <= 24; $d++)
            {
                $result .= '<input type="checkbox" '.$style_mar.' name="dh_'.$i.$d.'" value="1" id="d_'.$i.'_h_'.$d.'" onClick="sbActiveTime(this.id);"'.(isset($this->mValue[$i.'_'.$d]) && $this->mValue[$i.'_'.$d] == 1 ? ' checked="checked"' : '').' />';

                if($d % 4 == 0)
                {
                    $result .= '</nobr></td><td '.($td_str != '' ? ' '.$td_str : '').' style= "padding-right:15px;"><nobr>';
                }
            }
            $result .= '</td></tr><tr '.($tr_str != '' ? ' '.$tr_str : '').'>';
        }

        $result .= '<td colspan="2"></td><td '.($td_str != '' ? ' '.$td_str : '').' style= "padding-right:10px; padding-top:10px;"><nobr>';
        for($d = 1; $d <= 24; $d++)
        {
            $result .= '<input type="checkbox" '.$style_mar.' value="1" id="h_'.$d.'" onClick="sbActiveHour(this.id);"'.(isset($this->mValue['h_'.$d]) && $this->mValue['h_'.$d] == 1 ? ' checked="checked"' : '').' />';
            if($d % 4 == 0)
            {
                    $result .= '</nobr></td><td '.($td_str != '' ? ' '.$td_str : '').' style= "padding-right:15px; padding-top:15px;">';
            }
        }
        $result .= '</td></tr>
                        <tr '.($tr_str != '' ? ' '.$tr_str : '').'>
                            <td '.($td_str != '' ? ' '.$td_str : '').' colspan="2"></td>
                            <td '.($td_str != '' ? ' '.$td_str : '').' >&nbsp;00:00</td>
                            <td '.($td_str != '' ? ' '.$td_str : '').' >&nbsp;04:00</td>
                            <td '.($td_str != '' ? ' '.$td_str : '').' >&nbsp;08:00</td>
                            <td '.($td_str != '' ? ' '.$td_str : '').' >&nbsp;12:00</td>
                            <td '.($td_str != '' ? ' '.$td_str : '').' >&nbsp;16:00</td>
                            <td '.($td_str != '' ? ' '.$td_str : '').' >&nbsp;20:00</td>
                       </tr></table></td></tr>';
        return $result;
    }

    /**
     * Возвращает JavaScript-код поля
     *
     * @return string JavaScript-код
     */
    public function getJavaScript()
    {
        static $echo = true;
        if($echo)
        {
            $echo = false;
            return '<script>
                function sbActiveWeek(id_week)
                {
                    var ar = new Array();
                    var checked = sbGetE(id_week).checked;

                    for(var i = 1; i <= 24; i++)
                    {
                        var old_checked = sbGetE(id_week+"_h_"+i).checked;
                        sbGetE(id_week+"_h_"+i).checked = checked;

                        if (old_checked != checked)
                        {
                            ar[ar.length] = id_week+"_h_"+i;
                        }
                    }

                    for (var i = 0; i < ar.length; i++)
                    {
                        sbActiveTime(ar[i]);
                    }
                }

                function sbActiveHour(id_hour)
                {
                    var ar = new Array();
                    var checked = sbGetE(id_hour).checked;

                    for(var i = 1; i <= 7; i++)
                    {
                        var old_checked = sbGetE("d_"+i+"_"+id_hour).checked;
                        sbGetE("d_"+i+"_"+id_hour).checked = checked;

                        if (old_checked != checked)
                        {
                            ar[ar.length] = "d_"+i+"_"+id_hour;
                        }
                    }

                    for (var i = 0; i < ar.length; i++)
                    {
                        sbActiveTime(ar[i]);
                    }
                }

                function sbActiveTime(id_time)
                {
                    var ar = id_time.split("_");
                    var d = ar[1];
                    var h = ar[3];

                    for (var i = 1; i <= 7; i++)
                    {
                        if (!sbGetE("d_" + i + "_h_" + h).checked)
                        {
                            sbGetE("h_" + h).checked = false;
                            break;
                        }
                    }

                    if (i > 7)
                        sbGetE("h_" + h).checked = true;

                    for (var i = 1; i <= 24; i++)
                    {
                        if (!sbGetE("d_" + d + "_h_" + i).checked)
                        {
                            sbGetE("d_" + d).checked = false;
                            break;
                        }
                    }

                    if (i > 24)
                        sbGetE("d_" + d).checked = true;
                }
                </script>';
        }

        return '';
    }

    /**
     * Обрабатывает массив $_POST и возвращает строку (помеченных чекбоксов) для вставки в базу данных
     *
     * @return string Строка для вставки в БД.
     */
    static function getStrForDb()
    {
        if(count($_POST) == 0)
           return '';

        $result = '';
        $month_result = '';

        foreach($_POST as $key => $value)
        {
            if(substr($key, 0, 5) == 'month')
            {
                $month_result .= intval(preg_replace('/[^0-9]+/', '', $key)).';';
            }
            elseif(substr($key, 0, 2) == 'dh')
            {
                $str = substr($key, 3);
                $d = intval(substr($str, 0, 1));
                $h = intval(substr($str, 1));
                $result .= $d.','.$h.';';
            }
        }

        return $result.'||'.$month_result;
    }
}
?>