<?php
/**
 * Класс, отвечающий за вывод полей выбора значений модулей
 *
 * @see sbLayout
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbLayoutPluginData
{
   /** Тип поля
     *
     * Возможные значения:
     * <ul>
     * <li>select - Выбор из выпадающего списка.
     * <li>multiselect - Множественный выбор.
     * <li>checkboxes - Набор флажков.
     * <li>radio - Набор радио-кнопок.
     * </ul>
     *
     * @var string
     */
    private $mType = 'select';

    /**
     * ID выбранных значений модулей
     *
     * @var string
     */
    private $mValue = '';

    /**
     * Имя поля (значение атрибута <i>name</i>)
     *
     * @var string
     */
    private $mName = '';

    /**
     * Идентификатор поля (значение атрибута <i>id</i>)
     *
     * @var string
     */
    private $mId = '';

    /**
     * Дополнительные св-ва поля (например, стили)
     *
     * @var string
     */
    private $mDopStr = '';

    /**
     * Дополнительный HTML-код, вставляемый после поля
     *
     * @var string
     */
    public $mHTML = '';

    /**
     * Выводить (TRUE) или нет (FALSE) значения из вложенных разделов
     *
     * @var boolean
     */
    public $mSubCategs = true;

    /**
     * Подгружать (TRUE) или нет (FALSE) значения модулей с помощью AJAX
     *
     * @var boolean
     */
    public $mAJAX = false;

    /**
     * Уникальные идентификаторы разделов, элементы которых будут выводиться
     *
     * @var string
     */
    private $mCategsIds = '';

    /**
     * Для типов <i>checkboxes</i> и <i>radio</i> - разделитель значений
     *
     * @var string
     */
    public $mSeparator = '&nbsp;';

    /**
     * Кол-во строк в списке (для типа <i>select</i>), либо кол-во значений в строке (для типа <i>radio</i> и <i>checkboxes</i>)
     *
     * @var integer
     */
    public $mRows = 1;

    /**
     * Поле обязательно для заполнения (TRUE) или нет (FALSE)
     *
     * @var bool
     */
    private $mRequired = false;

    /**
     * Поле элемента, использующееся в качестве названия опции
     *
     * @var string
     */
    public $mTitleFld = '';

    /**
     * Поле доступно только для чтения
     *
     * @var bool
     */
    public $mReadOnly = false;

    /**
     * Модуль для которого выводится поле
     */
    public $mPlugin = '';

    /**$
     * Конструктор класса
     *
     * @param string $type Тип поля
     *
     * Возможные значения:
     * <ul>
     * <li>select - Выбор из выпадающего списка.
     * <li>multiselect - Множественный выбор.
     * <li>multiselect_sprav - Множественный выбор (диалоговое окно).
     * <li>checkboxes - Набор флажков.
     * <li>radio - Набор радио-кнопок.
     * </ul>
     *
     * @param string $modules_cat_id Идентификаторы разделов, элементы которых будут выводиться.
     * @param string $value ID выбранных значений справочников (через запятую).
     * @param string $name Имя поля (значение атрибута <i>name</i>).
     * @param string $id Идентификатор поля (значение атрибута <i>id</i>).
     * @param string $dop_str Дополнительные св-ва поля (например, стили).
     * @param bool $required Поле обязательно для заполнения (TRUE) или нет (FALSE).
     */
    public function __construct($type='select', $ident, $modules_cat_id='', $value='', $name='', $id='', $dop_str='', $required=false)
    {
        if ($id == '') $id = $name;

        $this->mType = $type;
        if ($type == 'select' || $type == 'radio')
        {
            $this->mValue = intval($value);
        }
        else
        {
            $this->mValue = $value;
        }

        $this->mCategsIds = str_replace('^', ',', $modules_cat_id);
        $this->mName = $name;
        $this->mId = $id;
        $this->mDopStr = $dop_str;
        $this->mRequired = $required;
        $this->mPlugin = $ident;
        if($this->mRows == 0)
        {
            $this->mRows = 1;
        }
    }

     /** Возвращает HTML-код поля
     *
     * @return string HTML-код поля.
     */
    public function getField()
    {
        $res = false;
        $sql_params = getPluginSqlParams($this->mPlugin);

        if($this->mTitleFld == '')
        {
            $this->mTitleFld = $sql_params['title'];
        }
        $this->mTitleFld = preg_replace('/[^A-Za-z_0-9]/', '', $this->mTitleFld);

        if ($this->mType != 'multiselect_plugin')
        {
            $is_show_sql = '';
            if(isset($sql_params['show']) && $sql_params['show'] != '')
            {
                $is_show_sql = ' AND '.$sql_params['show'].' > 0 ';
            }
            if ($this->mSubCategs)
            {
                $res = sql_query('SELECT cat_left, cat_right FROM sb_categs WHERE cat_id IN (0, '.$this->mCategsIds.')');
                if ($res)
                {
                    $sql = '(';
                    $num = count($res);
                    for ($i = 0; $i < $num; $i++)
                    {
                        list($left, $right) = $res[$i];
                        $sql .= 'c.cat_left >= '.$left.' AND c.cat_right <= '.$right.($i != $num - 1 ? ' OR ' : '');
                    }
                    $sql .= ')';

                    $res = sql_query('SELECT DISTINCT s.'.$sql_params['id'].', s.'.$this->mTitleFld.', c.cat_id, c.cat_title FROM '.$sql_params['table'].' s, sb_catlinks l, sb_categs c WHERE c.cat_ident=? AND '.$sql.$is_show_sql.' AND l.link_cat_id=c.cat_id AND l.link_el_id=s.'.$sql_params['id'].' ORDER BY '.($this->mType == 'radio' || $this->mType == 'checkboxes' ? '' : 'c.cat_left, ').' s.'.$this->mTitleFld, $this->mPlugin);
                }
            }
            else
            {
                $res = sql_query('SELECT DISTINCT s.'.$sql_params['id'].', s.'.$this->mTitleFld.', c.cat_id, c.cat_title FROM '.$sql_params['table'].' s, sb_catlinks l, sb_categs c WHERE c.cat_id IN (0, '.$this->mCategsIds.') AND l.link_cat_id = c.cat_id AND l.link_el_id=s.'.$sql_params['id'].$is_show_sql.' ORDER BY '.($this->mType == 'radio' || $this->mType == 'checkboxes' ? '' : 'c.cat_left, ').' s.'.$this->mTitleFld);
            }
        }
        elseif ($this->mValue != '')
        {
            $res = sql_query('SELECT '.$this->mTitleFld.' FROM '.$sql_params['table'].' WHERE '.$sql_params['id'].' IN (0, '.$this->mValue.') ORDER BY '.$this->mTitleFld.' LIMIT 5');
        }

        if (!$res)
        {
            if ($this->mType != 'multiselect_plugin')
            {
                sb_add_system_message(sprintf(PLUGINS_DESIGN_TEMPS_NO_ENTRIES_IN_LINKED_ERROR, $this->mPlugin), SB_MSG_ERROR);
                return '';
            }
            else
                $num = 0;
        }
        else
        {
            $num = count($res);
        }

        $result = '';
        switch ($this->mType)
        {
            case 'select':
            case 'multiselect':
                $result = '<select name="'.$this->mName.'" id="'.$this->mId.'" size="'.min($num, $this->mRows).'"'.($this->mDopStr != '' ? ' '.$this->mDopStr : '').($this->mType == 'multiselect' ? ' multiple="multiple"' : '').($this->mReadOnly ? ' disabled="disabled"' : '').'>';
                if ($this->mType == 'select')
                    $result .= '<option value="-1"> --- </option>';

                $old_cat_id = -1;
                for ($i = 0; $i < $num; $i++)
                {
                    list($s_id, $s_title, $cat_id, $cat_title) = $res[$i];
                    // Если в качестве названия используется дата
                    if(isset($sql_params['date']) && $sql_params['date'] == $this->mTitleFld)
                    {
                        $s_title = date('d.m.Y h:i', $s_title);
                    }
                    if ($cat_id != $old_cat_id)
                    {
                        if ($result != '')
                            $result .= '</optgroup>';

                        $result .= '<optgroup label="'.$cat_title.'">';
                        $old_cat_id = $cat_id;
                    }
                    $result .= '<option value="'.$s_id.'"'.(($this->mType == 'multiselect' && $this->mValue != '' && in_array($s_id, $this->mValue)) || ($this->mType == 'select' && $s_id == $this->mValue) ? ' selected="selected"' : '').'>'.$s_title.'</option>';
                }
                $result .= ($num != 0 ? '</optgroup>' : '').'</select>'.($this->mRequired ? ' <sup class="red">*</sup>' : '');
                break;

            case 'radio':
                for ($i = 0; $i < $num; $i++)
                {
                    list($s_id, $s_title, $cat_id, $cat_title) = $res[$i];
                    // Если в качестве названия используется дата
                    if(isset($sql_params['date']) && $sql_params['date'] == $this->mTitleFld)
                    {
                        $s_title = date('d.m.Y h:i', $s_title);
                    }
                    if ($i != 0 && $i % $this->mRows == 0)
                    {
                        $result .= '<br>';
                    }
                    $result .= '<input type="radio" value="'.$s_id.'" name="'.$this->mName.'" id="'.$this->mId.$s_id.'"'.($s_id == $this->mValue ? ' checked="checked"' : '').($this->mDopStr != '' ? ' '.$this->mDopStr : '').($this->mReadOnly ? ' disabled="disabled"' : '').'>&nbsp;<label for="'.$this->mId.$s_id.'">'.$s_title.'</label>'.$this->mSeparator;
                }
                $result .= ($this->mRequired ? ' <sup class="red">*</sup>' : '');
                break;

            case 'checkboxes':
                $this->mValue = explode(',', preg_replace('/[^0-9\,]+/i', '', $this->mValue));
                for ($i = 0; $i < $num; $i++)
                {
                    list($s_id, $s_title, $cat_id, $cat_title) = $res[$i];
                    // Если в качестве названия используется дата
                    if(isset($sql_params['date']) && $sql_params['date'] == $this->mTitleFld)
                    {
                        $s_title = date('d.m.Y h:i', $s_title);
                    }
                    if ($i != 0 && $i % $this->mRows == 0)
                    {
                        $result .= '<br>';
                    }
                    $result .= '<input type="checkbox" value="'.$s_id.'" name="'.$this->mName.'[]" id="'.$this->mId.$s_id.'"'.($this->mValue != '' && in_array($s_id, $this->mValue) ? ' checked="checked"' : '').($this->mDopStr != '' ? ' '.$this->mDopStr : '').($this->mReadOnly ? ' disabled="disabled"' : '').'>&nbsp;<label for="'.$this->mId.$s_id.'">'.$s_title.'</label>'.$this->mSeparator;
                }
                $result .= ($this->mRequired ? ' <sup class="red">*</sup>' : '');
                break;
            case 'multiselect_plugin':
                if ($num == 0)
                {
                    $result = '<span id="'.$this->mId.'_text"><b style="font-size:11px;">'.SB_LAYOUT_NO_SPRAV.'</b>';
                }
                else
                {
                    $result = array();
                    $count = min(4, $num);
                    for ($i = 0; $i < $count; $i++)
                    {
                        list($result[]) = $res[$i];
                    }
                    $result = '<span id="'.$this->mId.'_text">'.implode($this->mSeparator, $result);
                    if ($num > 4)
                        $result .= ' ...';
                }

                $result = '<table><tr style="background-color: transparent;"><td>'.$result.'</span>'.($this->mRequired ? '&nbsp;<sup class="red">*</sup>' : '').'
                            </td><td><nobr>&nbsp;<img class="button" src="'.SB_CMS_IMG_URL.'/sprav.png" width="20" height="20" align="absmiddle" id="'.$this->mId.'_button" onmouseup="sbPress(this, false);" onmousedown="sbPress(this, true);" onclick="sbGetModulesData(\''.$this->mId.'\')" />
                            <img class="button" src="'.SB_CMS_IMG_URL.'/sprav_drop.png" width="20" height="20" align="absmiddle" id="'.$this->mId.'_drop_button" onmouseup="sbPress(this, false);" onmousedown="sbPress(this, true);" onclick="sbDropModulesData(\''.$this->mId.'\')" />
                            <input type="hidden" name="'.$this->mName.'" id="'.$this->mId.'" value="'.$this->mValue.'"'.($this->mDopStr != '' ? ' '.$this->mDopStr : '').'>
                            <input type="hidden" id="'.$this->mId.'_plugin" value="'.$this->mPlugin.'">
                            <input type="hidden" id="'.$this->mId.'_fld" value="'.$this->mTitleFld.'">
                            <input type="hidden" id="'.$this->mId.'_cats" value="'.$this->mCategsIds.'">
                            <input type="hidden" id="'.$this->mId.'_subcateg" value="'.($this->mSubCategs ? '1' : '0').'">
                            <input type="hidden" id="'.$this->mId.'_ajax" value="'.($this->mAJAX ? '1' : '0').'">
                            <input type="hidden" id="'.$this->mId.'_readonly" value="'.($this->mReadOnly ? '1' : '0').'">
                            <input type="hidden" id="'.$this->mId.'_separator" value="'.sb_htmlspecialchars($this->mSeparator, ENT_QUOTES).'"></nobr></td></tr></table>';
                break;
        }

        return $result.$this->mHTML;
    }
    /**
     * Возвращает JavaScript-код поля
     *
     * @return string JavaScript-код поля.
     */
    public function getJavaScript()
    {
        static $echo = true;

        if ($echo && $this->mType == 'multiselect_plugin')
        {
            $echo = false;
            return '<script>
                var sbModulesDataElId = null;
                function sbGetModulesData(el_id)
                {
                    var el = sbGetE(el_id);
                    if (el.disabled)
                        return;

                    if (el)
                    {
                        sbModulesDataElId = el_id;
                        var plugin = sbGetE(el_id + "_plugin");
                        var fld_el = sbGetE(el_id + "_fld");
                        var cats_ids = sbGetE(el_id + "_cats");
                        var subcateg_el = sbGetE(el_id + "_subcateg");
                        var ajax_el = sbGetE(el_id + "_ajax");
                        var separator_el = sbGetE(el_id + "_separator");
                        var readonly_el = sbGetE(el_id + "_readonly");

                        var strPage = sb_cms_modal_dialog_file + "?event=pl_plugin_data_get_modules_data&plugin="+plugin.value+"&ids="+el.value+"&cats_ids="+cats_ids.value+"&subcateg="+subcateg_el.value+"&ajax="+ajax_el.value+"&readonly="+readonly_el.value+"&separator="+sbEscapeEx(separator_el.value)+"&fld="+fld_el.value;
                        var strAttr = "resizable=1,width=700,height=500";

                        sbShowModalDialog(strPage, strAttr, sbAfterGetModulesData);
                    }
                }
                function sbAfterGetModulesData()
                {
                    if (sbModalDialog.returnValue && sbModulesDataElId)
                    {
                        var el = sbGetE(sbModulesDataElId);
                        var el_name = sbGetE(sbModulesDataElId + "_text");

                        el.value = sbModalDialog.returnValue.ids;
                        el_name.innerHTML = sbModalDialog.returnValue.text != "" ? sbModalDialog.returnValue.text : "<b style=\'font-size:11px;\'>'.SB_LAYOUT_NO_SPRAV.'</b>";
                    }
                }
                function sbDropModulesData(el_id)
                {
                    var el = sbGetE(el_id);
                    var el_text = sbGetE(el_id + "_text");

                    el.value = "";
                    el_text.innerHTML = "<b style=\'font-size:11px;\'>'.SB_LAYOUT_NO_SPRAV.'</b>";
                }
                </script>';
        }
        else
        {
            return '';
        }
    }
}
?>