<?php
/**
 * Класс, отвечающий за вывод двух связанных списков
 *
 * @see sbLayout
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbLayoutPluginLink
{
    /**
     * ID выбранного значения первого списка
     *
     * @var string
     */
    private $mFirstValue = '';

    /**
     * ID выбранного значения второго списка
     *
     * @var string
     */
    private $mSecondValue = '';

    /**
     * Имя поля (значение атрибута <i>name</i>) первого списка
     *
     * Для второго списка прибавляется суффикс <i>_link</i>
     *
     * @var string
     */
    private $mName = '';

    /**
     * Идентификатор поля (значение атрибута <i>id</i>) первого списка
     *
     * Для второго списка прибавляется суффикс <i>_link</i>
     *
     * @var string
     */
    private $mId = '';

    /**
     * Дополнительные св-ва поля (например, стили) первого списка
     *
     * @var string
     */
    private $mFirstDopStr = '';

    /**
     * Дополнительные св-ва поля (например, стили) второго списка
     *
     * @var string
     */
    private $mSecondDopStr = '';

    /**
     * Дополнительный HTML-код, вставляемый после полей
     *
     * @var string
     */
    public $mHTML = '';

    /**
     * Выводить (TRUE) или нет (FALSE) значения из вложенных справочников
     *
     * @var boolean
     */
    public $mSubCategs = true;

    /**
     * Уникальный идентификатор справочника, значения которого будут выводиться
     *
     * @var array
     */
    private $mCategIds = '';

    /**
     * Наименование связанного списка
     *
     * @var string
     */
    public $mLinkTitle = '';

    /**
     * Кол-во строк в первом списке
     *
     * @var integer
     */
    public $mFirstRows = 1;

    /**
     * Кол-во строк во втором списке
     *
     * @var integer
     */
    public $mSecondRows = 1;

    /**
     * Поле обязательно для заполнения (TRUE) или нет (FALSE)
     *
     * @var bool
     */
    private $mRequired = false;

    /**
     * Поле справочника, использующееся в качестве названия опции второго списка
     *
     * @var string
     */
    public $mTitleFld = '';

    private $mPlugin = '';

    /**
     * Конструктор класса
     *
     * @param integet $sprav_id Уникальный идентификатор справочника, значения которого будут выводиться.
     * @param string $value ID выбранных значений списков (через запятую, первым идет значение первого списка соотв.).
     * @param string $name Имя поля (значение атрибута <i>name</i>).
     * @param string $id Идентификатор поля (значение атрибута <i>id</i>).
     * @param string $first_dop_str Дополнительные св-ва первого списка (например, стили).
     * @param string $second_dop_str Дополнительные св-ва второго списка (например, стили).
     * @param bool $required Поле обязательно для заполнения (TRUE) или нет (FALSE).
     */
    public function __construct($ident, $modules_cat_id='', $value='', $name='', $id='', $first_dop_str='', $second_dop_str='', $required=false)
    {
        if ($id == '') $id = $name;

        if ($value != '')
        {
            $value = explode(',', $value);

            $this->mFirstValue = intval($value[0]);
            $this->mSecondValue = isset($value[1]) ? intval($value[1]) : '';
        }

        $this->mCategIds = intval($modules_cat_id);
        $this->mName = $name;
        $this->mId = $id;
        $this->mFirstDopStr = $first_dop_str;
        $this->mSecondDopStr = $second_dop_str;
        $this->mRequired = $required;
        $this->mPlugin = $ident;
    }

    /**
     * Возвращает HTML-код поля
     *
     * @return string HTML-код поля.
     */
    public function getField()
    {
        $sql_params = getPluginSqlParams($this->mPlugin);
        if($this->mTitleFld == '')
            $this->mTitleFld = $sql_params['title'];
        $this->mTitleFld = preg_replace('/[^A-Za-z_0-9]/', '', $this->mTitleFld);

        $is_show_sql = '';
        if(isset($sql_params['show']) && $sql_params['show'] != '')
        {
            $is_show_sql = ' AND '.$sql_params['show'].' > 0 ';
        }

        $res = sql_query('SELECT cat_left, cat_right, cat_level FROM sb_categs WHERE cat_id=?d', $this->mCategIds);
        if (!$res)
            return '';
        list ($left, $right, $level) = $res[0];
        
        $checkLevel = 'cat_level > '.$level;
        $first_res = sql_query('SELECT cat_id, cat_title, cat_left, cat_right, cat_level FROM sb_categs WHERE cat_left > '.$left.' AND cat_right < '.$right.' AND cat_ident=? AND '.$checkLevel.' ORDER BY cat_left', $this->mPlugin);
        if (!$first_res)
            return '';

        $left = 0;
        $right = 0;
        $found = false;
        if ($this->mFirstValue != '')
        {
            foreach ($first_res as $value)
            {
                if ($value[0] == $this->mFirstValue)
                {
                    $left = $value[2];
                    $right = $value[3];
                    $found = true;
                    break;
                }
            }
        }

        if (!$found)
        {
            $this->mFirstValue = $first_res[0][0];
            $left = $first_res[0][2];
            $right = $first_res[0][3];
        }

        $second_res = false;
        if ($this->mSubCategs)
        {
            $second_res = sql_query('SELECT DISTINCT s.'.$sql_params['id'].', s.'.$this->mTitleFld.' FROM '.$sql_params['table'].' s, sb_catlinks l, sb_categs c WHERE c.cat_ident=? AND c.cat_left >= ?d AND c.cat_right <= ?d AND l.link_cat_id=c.cat_id AND l.link_el_id=s.'.$sql_params['id'].$is_show_sql.' ORDER BY s.'.$sql_params['sort'].', s.'.$this->mTitleFld, $this->mPlugin, $left, $right);
        }
        else
        {
            $second_res = sql_query('SELECT DISTINCT s.'.$sql_params['id'].', s.'.$this->mTitleFld.' FROM '.$sql_params['table'].' s, sb_catlinks l WHERE l.link_cat_id = ?d AND l.link_el_id=s.s_id '.$is_show_sql.' ORDER BY s.'.$sql_params['sort'].', s.'.$this->mTitleFld, $this->mFirstValue);
        }

        if(!$second_res){
            sb_add_system_message(sprintf(PLUGINS_DESIGN_TEMPS_NO_ENTRIES_IN_LINKED_ERROR, $this->mPlugin), SB_MSG_ERROR);
            return '';
        }

        $result = '<input type="hidden" id="'.$this->mId.'_subcateg" value="'.($this->mSubCategs ? '1' : '0').'">
            <input type="hidden" id="'.$this->mId.'_fld" value="'.$this->mTitleFld.'">
            <select name="'.$this->mName.'" id="'.$this->mId.'" onchange="sbGetLinkPluginData(this)" size="'.$this->mFirstRows.'"'.($this->mFirstDopStr != '' ? ' '.$this->mFirstDopStr : '').'>';

        foreach ($first_res as $value)
        {
            list($cat_id, $cat_title, $left, $right, $dopLevel) = $value;
            $r = $dopLevel-$level-1;
            $sep = '';
            if($r > 0)
            {
                for($i = 0; $i < $r; $i++)
                {
                    $sep .= '&nbsp;&nbsp;';
                }
            }
            
            $result .= '<option value="'.$cat_id.'"'.($cat_id == $this->mFirstValue ? ' selected="selected"' : '').'>'.$sep.$cat_title.'</option>';
        }
        $result .= '</select><br><br>'.($this->mLinkTitle != '' ? '<b>'.$this->mLinkTitle.':</b><br><br>' : '');

        $result .= '<select name="'.$this->mName.'_link" id="'.$this->mId.'_link" size="'.$this->mSecondRows.'"'.($this->mSecondDopStr != '' ? ' '.$this->mSecondDopStr : '').'>
            <option value="-1"> --- </option>';

        if ($second_res)
        {
            foreach ($second_res as $value)
            {
                list($s_id, $s_title) = $value;
                if(isset($sql_params['date']) && $sql_params['date'] == $this->mTitleFld)
                    $s_title = date('d.m.Y h:i');
                $result .= '<option value="'.$s_id.'"'.($s_id == $this->mSecondValue ? ' selected="selected"' : '').'>'.$s_title.'</option>';
            }
        }

        $result .= '</select>'.($this->mRequired ? ' <sup class="red">*</sup>' : '');

        return $result.$this->mHTML;
    }

    /**
     * Возвращает JavaScript-код поля
     *
     * @return string JavaScript-код поля.
     */
    public function getJavaScript()
    {
        static $echo = true;

        if ($echo)
        {
            $echo = false;
            return '<script>
                function sbGetLinkPluginData(el)
                {
                    var link_el = sbGetE(el.id + "_link");
                    var subcateg_el = sbGetE(el.id + "_subcateg");
                    var fld_el = sbGetE(el.id + "_fld");

                    if (link_el && subcateg_el)
                    {
                        while (link_el.options.length > 1)
                            link_el.remove(1);
                        var options = sbLoadSync("'.SB_CMS_EMPTY_FILE.'?event=pl_plugin_data_get_options&cat_id="+el.value+"&subcategs="+subcateg_el.value+"&fld="+fld_el.value+"&ident='.$this->mPlugin.'");
                        if (options && options != "")
                        {
                            options = options.split("::");
                            for (var i = 0; i < options.length; i++)
                            {
                                var oOption = document.createElement("OPTION");
                                link_el.options.add(oOption);

                                var opt = options[i].split("^^");
                                oOption.value = opt[0];
                                oOption.innerHTML = opt[1];
                            }
                        }
                    }
                }
                </script>';
        }
        else
        {
            return '';
        }
    }
}
?>