<?php
/**
 * Класс, отвечающий за вывод выпадающего списка
 *
 * @see sbLayout
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbLayoutSelect
{
    /**
     * Имя поля (значение атрибута <i>name</i>)
     *
     * @var string
     */
    private $mName = '';

    /**
     * Идентификатор поля (значение атрибута <i>id</i>)
     *
     * @var string
     */
    private $mId = '';

    /**
     * Дополнительные св-ва поля (например, стили)
     *
     * @var string
     */
    private $mDopStr = '';

    /**
     * Дополнительный HTML-код, вставляемый после поля
     *
     * @var string
     */
    public $mHTML = '';

    /**
     * Дополнительные св-ва поля option (например, стили)
     * Может быть строкой (устанавливается для всех полей option) или массивом,
     * ключи которого совпадают со значением value поля option, а значения
     * массива являются строкой, содержащей свойства данного поля option.
     *
     * @var mixed
     */
    public $mOptionsDopStr = '';

    /**
     * Поле обязательно для заполнения (TRUE) или нет (FALSE)
     *
     * @var bool
     */
    private $mRequired = false;

    /**
     * Ассоциативный массив опций, где ключ - значение атрибута value, значение - текст опции.
     *
     * @var array
     */
    private $mOptions = array();

    /**
     * Массив со значениями помеченных опций.
     *
     * @var array
     */
    public $mSelOptions = array();

    /**
     * Конструктор класса
     *
     * @param array $options Ассоциативный массив опций, где ключ - значение атрибута value, значение - текст опции.
     * @param string $name Имя поля (значение атрибута <i>name</i>).
     * @param string $id Идентификатор поля (значение атрибута <i>id</i>).
     * @param string $dop_str Дополнительные св-ва поля (например, стили).
     * @param bool $required Поле обязательно для заполнения (TRUE) или нет (FALSE).
     */
    public function __construct($options, $name='', $id='', $dop_str='', $required=false)
    {
        if ($id == '') $id = $name;

        $this->mOptions = $options;
        $this->mName = $name;
        $this->mId = $id;
        $this->mDopStr = $dop_str;
        $this->mRequired = $required;
    }

    /**
     * Возвращает HTML-код поля
     *
     * @return string HTML-код поля.
     */
    public function getField()
    {
        if (count($this->mOptions) == 0)
            return '';

        $result =  '<select name="'.$this->mName.'" id="'.$this->mId.'"'.($this->mDopStr != '' ? ' '.$this->mDopStr : '').'>';

        $was_group = false;
        foreach ($this->mOptions as $key => $value)
        {
            if (sb_substr($value, 0, 1) == '-')
            {
                if ($was_group)
                    $result .= '</optgroup>';

                $result .= '<optgroup label="'.sb_substr($value, 1).'">';
                $was_group = true;
                continue;
            }

            // Возможность устанавливать отдельные атрибуты для каждого option
            $attributes = '';
            if(is_string($this->mOptionsDopStr) && $this->mOptionsDopStr != '')
            {
                $attributes = ' '.$this->mOptionsDopStr;
            }
            elseif(is_array($this->mOptionsDopStr) && isset($this->mOptionsDopStr[$key]))
            {
                $attributes = ' '.$this->mOptionsDopStr[$key];
            }

            if (in_array($key, $this->mSelOptions))
                $result .= '<option value="'.$key.'" selected="selected" class="selected"'.$attributes.'>'.$value.'</option>';
            else
                $result .= '<option value="'.$key.'"'.$attributes.'>'.$value.'</option>';
        }

        if ($was_group)
        {
            $result .= '</optgroup>';
        }

        $result .= '</select>'.($this->mRequired ? ' <sup class="red">*</sup>' : '').$this->mHTML;

        return $result;
    }
}
?>