<?php
/**
 * Класс, отвечающий за вывод поля выбора справочника
 *
 * @see sbLayout
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbLayoutSprav
{
    /**
     * ID справочника по умолчанию
     *
     * @var string
     */
    private $mValue = '';

    /**
     * Имя поля (значение атрибута <i>name</i>)
     *
     * @var string
     */
    private $mName = '';

    /**
     * Идентификатор поля (значение атрибута <i>id</i>)
     *
     * @var string
     */
    private $mId = '';

    /**
     * Дополнительные св-ва поля (например, стили)
     *
     * @var string
     */
    private $mDopStr = '';

    /**
     * Дополнительный HTML-код, вставляемый после поля
     *
     * @var string
     */
    public $mHTML = '';

    /**
     * Поле обязательно для заполнения (TRUE) или нет (FALSE)
     *
     * @var bool
     */
    private $mRequired = false;

    /**
     * Возможен ли выбор нескольких справочников (TRUE) или нет (FALSE)
     *
     * @var bool
     */
    public $mMultiSelect = true;

    /**
     * Отображать или нет кнопку "Сбросить значение"
     *
     * @var bool
     */
    public $mDropButton = false;

    /**
     * Конструктор класса
     *
     * @param string $value ID справочника по умолчанию (значение атрибута <i>value</i>).
     * @param string $name Имя поля (значение атрибута <i>name</i>).
     * @param string $id Идентификатор поля (значение атрибута <i>id</i>).
     * @param string $dop_str Дополнительные св-ва поля (например, стили).
     * @param bool $required Поле обязательно для заполнения (TRUE) или нет (FALSE).
     */
    public function __construct($value='', $name='', $id='', $dop_str='', $required=false)
    {
        if ($id == '') $id = $name;

        $this->mValue = $value;
        $this->mName = $name;
        $this->mId = $id;
        $this->mDopStr = $dop_str;
        $this->mRequired = $required;
    }

    /**
     * Возвращает HTML-код поля
     *
     * @return string HTML-код поля.
     */
    public function getField()
    {
        $name = '';
        $value = '';
        if ($this->mMultiSelect && $this->mValue != '')
        {
            $res = sql_query('SELECT cat_id, cat_title FROM sb_categs WHERE cat_id IN ('.$this->mValue.')');
            if ($res)
            {
                $num = count($res);
                for ($i = 0; $i < $num; $i++)
                {
                    list($cat_id, $cat_title) = $res[$i];
                    $name .= $cat_title.($i+1 != $num ? ', ' : '');
                    $value .= $cat_id.($i+1 != $num ? ',' : '');
                }
            }
        }
        elseif ($this->mValue != '')
        {
            $res = sql_query('SELECT cat_title FROM sb_categs WHERE cat_id=?d', $this->mValue);
            if ($res)
            {
                list($name) = $res[0];
                $value = $this->mValue;
            }
        }

        return '<nobr><input type="text" id="'.$this->mId.'_name" readonly="readonly" value="'.sb_htmlspecialchars($name, ENT_QUOTES).'"'.($this->mDopStr != '' ? ' '.$this->mDopStr : '').' />
                <input type="hidden" name="'.$this->mName.'" id="'.$this->mId.'" value="'.$value.'" />'.($this->mRequired ? '<sup class="red">*</sup>' : '').
               '&nbsp;<img class="button" src="'.SB_CMS_IMG_URL.'/categs.png" width="20" height="20" align="absmiddle" id="'.$this->mId.'_button" onmouseup="sbPress(this, false);" onmousedown="sbPress(this, true);" onclick="sbGetSprav(\''.$this->mId.'\', '.($this->mMultiSelect ? '1' : '0').');">
               '.($this->mDropButton ? '<img class="button" src="'.SB_CMS_IMG_URL.'/categs_drop.png" width="20" height="20" id="'.$this->mId.'_drop_button" align="absmiddle" onmouseup="sbPress(this, false);" onmousedown="sbPress(this, true);" onclick="sbDropSprav(\''.$this->mId.'\');" /></nobr>' : '</nobr>').$this->mHTML;
    }

    /**
     * Возвращает JavaScript-код поля
     *
     * @return string JavaScript-код поля.
     */
    public function getJavaScript()
    {
        static $echo = true;

        if ($echo)
        {
            $echo = false;
            return '<script>
                var sbSpravElId = null;
                function sbGetSprav(el_id, multi)
                {
                    var el = sbGetE(el_id);
                    if (el.disabled)
                        return;

                    if (el)
                    {
                        sbSpravElId = el_id;
                        var strPage = sb_cms_modal_dialog_file + "?event=pl_sprav_get_sprav&ids="+el.value+"&multi=" + multi;
                        var strAttr = "resizable=1,width=500,height=700";

                        sbShowModalDialog(strPage, strAttr, sbAfterGetSprav);
                    }
                }
                function sbAfterGetSprav()
                {
                    if (sbModalDialog.returnValue && sbSpravElId)
                    {
                        var el = sbGetE(sbSpravElId);
                        var el_name = sbGetE(sbSpravElId + "_name");

                        el.value = sbModalDialog.returnValue.ids;
                        el_name.value = sbModalDialog.returnValue.text;
                        if (typeof(el.onchange) != "undefined" && el.onchange)
                            el.onchange();
                    }
                }
                function sbDropSprav()
                {
                    var el = sbGetE(el_id);
                    if (el.disabled)
                        return;
                    if (el)
                    {
                        el.value = "";
                    }
                }
                </script>';
        }
        else
        {
            return '';
        }
    }
}
?>