<?php
/**
 * Класс, отвечающий за вывод полей выбора значений справочников
 *
 * @see sbLayout
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbLayoutSpravData
{
    /**
     * Тип поля
     *
     * Возможные значения:
     * <ul>
     * <li>select - Выбор из выпадающего списка.
     * <li>multiselect - Множественный выбор.
     * <li>checkboxes - Набор флажков.
     * <li>radio - Набор радио-кнопок.
     * </ul>
     *
     * @var string
     */
    private $mType = 'select';

    /**
     * ID выбранных значений справочников
     *
     * @var string
     */
    private $mValue = '';

    /**
     * Имя поля (значение атрибута <i>name</i>)
     *
     * @var string
     */
    private $mName = '';

    /**
     * Идентификатор поля (значение атрибута <i>id</i>)
     *
     * @var string
     */
    private $mId = '';

    /**
     * Дополнительные св-ва поля (например, стили)
     *
     * @var string
     */
    private $mDopStr = '';

    /**
     * Дополнительный HTML-код, вставляемый после поля
     *
     * @var string
     */
    public $mHTML = '';

    /**
     * Выводить (TRUE) или нет (FALSE) значения из вложенных справочников
     *
     * @var boolean
     */
    public $mSubCategs = true;

    /**
     * Подгружать (TRUE) или нет (FALSE) значения справочников с помощью AJAX
     *
     * @var boolean
     */
    public $mAJAX = false;

    /**
     * Уникальные идентификаторы справочников, значения которых будут выводиться
     *
     * @var string
     */
    private $mSpravIds = '';

    /**
     * Для типов <i>checkboxes</i> и <i>radio</i> - разделитель значений
     *
     * @var string
     */
    public $mSeparator = '&nbsp;';

    /**
     * Кол-во строк в списке (для типа <i>select</i>), либо кол-во значений в строке (для типа <i>radio</i> и <i>checkboxes</i>)
     *
     * @var integer
     */
    public $mRows = 1;

    /**
     * Поле обязательно для заполнения (TRUE) или нет (FALSE)
     *
     * @var bool
     */
    private $mRequired = false;

    /**
     * Поле справочника, использующееся в качестве названия опции
     *
     * @var string
     */
    public $mTitleFld = 's_title';

    /**
     * Поле доступно только для чтения
     *
     * @var bool
     */
    public $mReadOnly = false;

    /**
     * Элемент создается для модулей
     *
     * @var bool
     */
    public $mForModules = false;

    /**
     * Список идентификаторов элементов справочников, которые должны отображаться в списке.
     * Если массив пустой, то отображаются все доступные элементы
     * @var array
     */
    public $mAllowItems = array();

    /**
     * Возможность редактирования справочников "на лету"
     * @var boolean
     */
    public $mShowEditLink = true;

    /**
     * Конструктор класса
     *
     * @param string $type Тип поля
     *
     * Возможные значения:
     * <ul>
     * <li>select - Выбор из выпадающего списка.
     * <li>multiselect - Множественный выбор.
     * <li>multiselect_sprav - Множественный выбор (диалоговое окно).
     * <li>checkboxes - Набор флажков.
     * <li>radio - Набор радио-кнопок.
     * </ul>
     *
     * @param string $sprav_ids Идентификаторы справочников, значения которых будут выводиться (через запятую).
     * @param string $value ID выбранных значений справочников (через запятую).
     * @param string $name Имя поля (значение атрибута <i>name</i>).
     * @param string $id Идентификатор поля (значение атрибута <i>id</i>).
     * @param string $dop_str Дополнительные св-ва поля (например, стили).
     * @param bool $required Поле обязательно для заполнения (TRUE) или нет (FALSE).
     */
    public function __construct($type='select', $sprav_ids='', $value='', $name='', $id='', $dop_str='', $required=false)
    {
        if ($id == '') $id = $name;

        $this->mType = $type;
        if ($type == 'select' || $type == 'radio')
        {
            $this->mValue = intval($value);
        }
        else
        {
            $this->mValue = $value;
        }

        $this->mSpravIds = $sprav_ids;
        $this->mName = $name;
        $this->mId = $id;
        $this->mDopStr = $dop_str;
        $this->mRequired = $required;
        $this->mAllowItems = array();
    }

    /**
     * Возвращает HTML-код поля
     *
     * @return string HTML-код поля.
     */
    public function getField()
    {
        $this->mTitleFld = preg_replace('/[^A-Za-z_0-9]/', '', $this->mTitleFld);

        $filter = '';
        if(!empty($this->mAllowItems))
        {
            $filter = ' AND s.s_id IN ('.implode(',', $this->mAllowItems).') ';
        }

        $res = false;
        if ($this->mType != 'multiselect_sprav')
        {
            if ($this->mSubCategs)
            {
                $res = sql_query('SELECT cat_left, cat_right FROM sb_categs WHERE cat_id IN (0, '.$this->mSpravIds.')');
                if ($res)
                {
                    $sql = '(';
                    $num = count($res);
                    for ($i = 0; $i < $num; $i++)
                    {
                        list($left, $right) = $res[$i];
                        $sql .= 'c.cat_left >= '.$left.' AND c.cat_right <= '.$right.($i != $num - 1 ? ' OR ' : '');
                    }
                    $sql .= ')';

                    $res = sql_query('SELECT DISTINCT s.s_id, s.'.$this->mTitleFld.', c.cat_id, c.cat_title FROM sb_sprav s, sb_catlinks l, sb_categs c WHERE c.cat_ident=? AND '.$sql.' AND l.link_cat_id=c.cat_id AND l.link_el_id=s.s_id '.$filter.' ORDER BY '.($this->mType == 'radio' || $this->mType == 'checkboxes' ? '' : 'c.cat_left, ').'s.s_sort, s.'.$this->mTitleFld, 'pl_sprav');
                }
            }
            else
            {
                $res = sql_query('SELECT DISTINCT s.s_id, s.'.$this->mTitleFld.', c.cat_id, c.cat_title FROM sb_sprav s, sb_catlinks l, sb_categs c WHERE c.cat_id IN (0, '.$this->mSpravIds.') AND l.link_cat_id = c.cat_id  AND l.link_el_id=s.s_id '.$filter.'ORDER BY '.($this->mType == 'radio' || $this->mType == 'checkboxes' ? '' : 'c.cat_left, ').'s.s_sort, s.'.$this->mTitleFld);
            }
        }
        elseif ($this->mValue != '')
        {
            $res = sql_query('SELECT '.$this->mTitleFld.' FROM sb_sprav WHERE s_id IN (0, '.$this->mValue.') ORDER BY s_sort, '.$this->mTitleFld.' LIMIT 5');
        }

        if (!$res)
        {
            if ($this->mType != 'multiselect_sprav')
                return '';
            else
                $num = 0;
        }
        else
        {
            $num = count($res);
        }

        $result = '';
        switch ($this->mType)
        {
            case 'select':
            case 'multiselect':
                $result = '<select name="'.$this->mName.'" id="'.$this->mId.'" size="'.min($num, $this->mRows).'"'.($this->mDopStr != '' ? ' '.$this->mDopStr : '').($this->mType == 'multiselect' ? ' multiple="multiple"' : '').($this->mReadOnly ? ' disabled="disabled"' : '').'>';
                if ($this->mType == 'select')
                    $result .= '<option value="-1"> --- </option>';

                $old_cat_id = -1;
                for ($i = 0; $i < $num; $i++)
                {
                    list($s_id, $s_title, $cat_id, $cat_title) = $res[$i];
                    if ($cat_id != $old_cat_id)
                    {
                        if ($result != '')
                            $result .= '</optgroup>';

                        $result .= '<optgroup label="'.$cat_title.'">';
                        $old_cat_id = $cat_id;
                    }
                    $result .= '<option value="'.$s_id.'"'.(($this->mType == 'multiselect' && $this->mValue != '' && in_array($s_id, $this->mValue)) || ($this->mType == 'select' && $s_id == $this->mValue) ? ' selected="selected"' : '').'>'.$s_title.'</option>';
                }
                $result .= ($num != 0 ? '</optgroup>' : '').'</select>'.($this->mRequired ? ' <sup class="red">*</sup>' : '');
                if($this->mShowEditLink && !$this->mReadOnly)
                {
                    $result .= '<br/><a href="#" onclick="editSprav(\''.$this->mSpravIds.'\', '.($this->mSubCategs ? '1' : '0').', \''.$this->mId.'\')" class="small" style="background: url(/cms/images/hdelim.gif) 0 100% repeat-x">'.SB_LAYOUT_EDIT_SPRAV_LINK.'</a>';
                }
                break;

            case 'multiselect_sprav':
                if ($num == 0)
                {
                    $result = '<span id="'.$this->mId.'_text"><b style="font-size:11px;">'.SB_LAYOUT_NO_SPRAV.'</b>';
                }
                else
                {
                    $result = array();
                    $count = min(4, $num);
                    for ($i = 0; $i < $count; $i++)
                    {
                        list($result[]) = $res[$i];
                    }
                    $result = '<span id="'.$this->mId.'_text">'.implode($this->mSeparator, $result);
                    if ($num > 4)
                        $result .= ' ...';
                }

                $result = '<table><tr style="background-color: transparent;"><td>'.$result.'</span>'.($this->mRequired ? '&nbsp;<sup class="red">*</sup>' : '').'
                            </td><td><nobr>&nbsp;<img class="button" src="'.SB_CMS_IMG_URL.'/sprav.png" width="20" height="20" align="absmiddle" id="'.$this->mId.'_button" onmouseup="sbPress(this, false);" onmousedown="sbPress(this, true);" onclick="sbGetSpravData(\''.$this->mId.'\')" />
                            <img class="button" src="'.SB_CMS_IMG_URL.'/sprav_drop.png" width="20" height="20" align="absmiddle" id="'.$this->mId.'_drop_button" onmouseup="sbPress(this, false);" onmousedown="sbPress(this, true);" onclick="sbDropSpravData(\''.$this->mId.'\')" />
                            <input type="hidden" name="'.$this->mName.'" id="'.$this->mId.'" value="'.$this->mValue.'"'.($this->mDopStr != '' ? ' '.$this->mDopStr : '').'>
                            <input type="hidden" id="'.$this->mId.'_fld" value="'.$this->mTitleFld.'">
                            <input type="hidden" id="'.$this->mId.'_sprav" value="'.$this->mSpravIds.'">
                            <input type="hidden" id="'.$this->mId.'_subcateg" value="'.($this->mSubCategs ? '1' : '0').'">
                            <input type="hidden" id="'.$this->mId.'_ajax" value="'.($this->mAJAX ? '1' : '0').'">
                            <input type="hidden" id="'.$this->mId.'_readonly" value="'.($this->mReadOnly ? '1' : '0').'">
                            <input type="hidden" id="'.$this->mId.'_separator" value="'.sb_htmlspecialchars($this->mSeparator, ENT_QUOTES).'"></nobr></td></tr></table>';
                if($this->mShowEditLink && !$this->mReadOnly)
                {
                    $result .= '<a href="#" onclick="editSprav(\''.$this->mSpravIds.'\', '.($this->mSubCategs ? '1' : '0').', \''.$this->mId.'\')" class="small" style="background: url(/cms/images/hdelim.gif) 0 100% repeat-x">'.SB_LAYOUT_EDIT_SPRAV_LINK.'</a>';
                }
                break;

            case 'radio':
                for ($i = 0; $i < $num; $i++)
                {
                    list($s_id, $s_title, $cat_id, $cat_title) = $res[$i];
                    if ($i != 0 && $i % $this->mRows == 0)
                    {
                        $result .= '<br>';
                    }
                    $result .= '<input type="radio" value="'.$s_id.'" name="'.$this->mName.'" id="'.$this->mId.$s_id.'"'.($s_id == $this->mValue ? ' checked="checked"' : '').($this->mDopStr != '' ? ' '.$this->mDopStr : '').($this->mReadOnly ? ' disabled="disabled"' : '').'>&nbsp;<label for="'.$this->mId.$s_id.'">'.$s_title.'</label>'.$this->mSeparator;
                }
                $result .= ($this->mRequired ? ' <sup class="red">*</sup>' : '');
                if($this->mShowEditLink && !$this->mReadOnly)
                {
                    $result .= '<br/><a href="#" onclick="editSprav(\''.$this->mSpravIds.'\', '.($this->mSubCategs ? '1' : '0').', \''.$this->mId.'\')" class="small" style="background: url(/cms/images/hdelim.gif) 0 100% repeat-x">'.SB_LAYOUT_EDIT_SPRAV_LINK.'</a>';
                }
                break;

            case 'checkboxes':
                $this->mValue = explode(',', preg_replace('/[^0-9\,]+/i', '', $this->mValue));
                for ($i = 0; $i < $num; $i++)
                {
                    list($s_id, $s_title, $cat_id, $cat_title) = $res[$i];
                    if ($i != 0 && $i % $this->mRows == 0)
                    {
                        $result .= '<br>';
                    }
                    $result .= '<input type="checkbox" value="'.$s_id.'" name="'.$this->mName.'[]" id="'.$this->mId.$s_id.'"'.($this->mValue != '' && in_array($s_id, $this->mValue) ? ' checked="checked"' : '').($this->mDopStr != '' ? ' '.$this->mDopStr : '').($this->mReadOnly ? ' disabled="disabled"' : '').'>&nbsp;<label for="'.$this->mId.$s_id.'">'.$s_title.'</label>'.$this->mSeparator;
                }
                $result .= ($this->mRequired ? ' <sup class="red">*</sup>' : '');
                if($this->mShowEditLink && !$this->mReadOnly)
                {
                    $result .= '<br/><a href="#" onclick="editSprav(\''.$this->mSpravIds.'\', '.($this->mSubCategs ? '1' : '0').', \''.$this->mId.'\')" class="small" style="background: url(/cms/images/hdelim.gif) 0 100% repeat-x">'.SB_LAYOUT_EDIT_SPRAV_LINK.'</a>';
                }
                break;
        }

        return $result.$this->mHTML;
    }

    /**
     * Возвращает JavaScript-код поля
     *
     * @return string JavaScript-код поля.
     */
    public function getJavaScript()
    {
        static $echo = true;
        static $echoEdit = true;
        static $echoHeader = true;

        $js = '';
        if($echoHeader)
        {
            $js = '<script>
                var sbSpravDataElId = null;
                var sbSpravFldType = null;
                var sbSpravIds = null;
                </script>';
            $echoHeader = false;
        }

        if($echoEdit && $this->mShowEditLink && !$this->mReadOnly)
        {
            $js .= '<script>
                function editSprav(sprav_ids, subcategs, el_id)
                {
                    sbSpravDataElId = el_id;


                    sbSpravIds = sprav_ids;
                    var strPage = sb_cms_modal_dialog_file + "?event=pl_sprav_init&show_only="+sprav_ids+"&subcategs="+subcategs;
                    var strAttr = "resizable=1,width=900,height=700";

                    sbShowModalDialog(strPage, strAttr, sbAfterEditSprav);
                }

                function sbAfterEditSprav()
                {
                    if(sbSpravDataElId)
                    {
                        var els = document.getElementsByName(sbSpravDataElId);
                        var size = "";
                        if(els.length == 0)
                        {
                            els = document.getElementsByName(sbSpravDataElId+"[]");
                        }

                        if(els.length > 0)
                        {
                            if(els[0].options)
                            {
                                sbSpravFldType = "select";
                                size = els[0].getAttribute("size");
                            }
                            else if(els[0].getAttribute("type") == "hidden")
                            {
                                sbSpravFldType = "multiselect_sprav";
                            }
                            else
                            {
                                sbSpravFldType = els[0].getAttribute("type");
                            }
                        }
                        sbLoadAsync(sb_cms_empty_file+"?event=pl_sprav_field_reload&fld_type="+sbSpravFldType+"&sprav_ids="+sbSpravIds+"&fld_name="+sbSpravDataElId+"&select_size="+size+"&id='.(isset($_GET['id'])?$_GET['id']:'').'"+"&cat_id='.(isset($_GET['cat_id'])?$_GET['cat_id']:'').'", sbRefreshFld);
                    }
                }

                function sbRefreshFld(res)
                {
                    if(res != "FALSE")
                    {
                        var container = sbGetE(sbSpravDataElId+"_td");
                        container.innerHTML = res;
                    }
                }
                </script>';
                $echoEdit = false;
        }

        if ($echo && $this->mType == 'multiselect_sprav')
        {
            $echo = false;

            $js .= '<script>
                function sbGetSpravData(el_id)
                {
                    var el = sbGetE(el_id);
                    if (el.disabled)
                        return;

                    if (el)
                    {
                        sbSpravDataElId = el_id;
                        var fld_el = sbGetE(el_id + "_fld");
                        var sprav_el = sbGetE(el_id + "_sprav");
                        var subcateg_el = sbGetE(el_id + "_subcateg");
                        var ajax_el = sbGetE(el_id + "_ajax");
                        var separator_el = sbGetE(el_id + "_separator");
                        var readonly_el = sbGetE(el_id + "_readonly");

                        var strPage = sb_cms_modal_dialog_file + "?event=pl_sprav_get_sprav_data&ids="+el.value+"&sprav="+sprav_el.value+"&subcateg="+subcateg_el.value+"&ajax="+ajax_el.value+"&readonly="+readonly_el.value+"&separator="+sbEscapeEx(separator_el.value)+"&fld="+fld_el.value;
                        var strAttr = "resizable=1,width=700,height=500";

                        sbShowModalDialog(strPage, strAttr, sbAfterGetSpravData);
                    }
                }
                function sbAfterGetSpravData()
                {
                    if (sbModalDialog.returnValue && sbSpravDataElId)
                    {
                        var el = sbGetE(sbSpravDataElId);
                        var el_name = sbGetE(sbSpravDataElId + "_text");

                        el.value = sbModalDialog.returnValue.ids;
                        el_name.innerHTML = sbModalDialog.returnValue.text != "" ? sbModalDialog.returnValue.text : "<b style=\'font-size:11px;\'>'.SB_LAYOUT_NO_SPRAV.'</b>";
                    }
                }
                function sbDropSpravData(el_id)
                {
                    var el = sbGetE(el_id);
                    var el_text = sbGetE(el_id + "_text");

                    el.value = "";
                    el_text.innerHTML = "<b style=\'font-size:11px;\'>'.SB_LAYOUT_NO_SPRAV.'</b>";
                }
                </script>';

        }

        return $js;
    }
}
?>