<?php
/**
 * Класс, отвечающий за вывод таблицы с заголовками и возможностью сортировки
 *
 * @see sbLayout
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbLayoutTable
{
    /**
     * Массив заголовков таблицы
     *
     * @var array
     */
    private $mLabel = array();

    /**
     * Двумерный массив значений таблицы
     *
     * @var array
     */
    private $mValue = array();

    /**
     * Массив сортировки
     *
     * @var array
     */
    private $mSort  = array();

    /**
     * Массив, содержит ширину колонок
     *
     * @var array
     */
    public $mWidth = array();

    /**
     * Массив, содержит выравнивание горизонтальное в колонках
     *
     * @var array
     */
    public $mAlign = array();

    /**
     * Массив, содержит выравнивание вертикальное в колонках
     *
     * @var array
     */
    public $mVAlign = array();

    /**
     * Выводить тег TABLE при выводе таблицы или нет
     *
     * @var boolean
     */
    public $mShowTable = false;

    /**
     * Отображать или нет двоеточие в столбце заголовков полей
     *
     * @var bool
     */
    public $mShowColon = true;

    /**
     * Дополнительный HTML-код, вставляемый после поля
     *
     * @var string
     */
    public $mHTML = '';

    /**
     * Подсвечивать строки таблицы при наведении или нет
     *
     * @var boolean
     */
    public $mHighlight = false;

    /**
     * Конструктор класса
     *
     * @param array $label Массив заголовков таблицы.
     * @param array $value Двумерный массив значений таблицы.
     * @param array $sort Массив сортировки.
     */
    public function __construct($label, $value, $sort=array())
    {
        $this->mLabel = $label;
        $this->mValue = $value;
        $this->mSort  = $sort;
    }

    /**
     * Возвращает HTML-код таблицы
     *
     * @param int $title_width Ширина столбца заголовка поля.
     * @param string $title Заголовок поля.
     * @param bool $show_title Отображать или нет столбец заголовков полей.
     * @param string $tr_str Доп. свойства тегов <i>tr</i>.
     * @param string $th_str Доп. свойства тегов <i>th</i>.
     * @param string $td_str Доп. свойства тегов <i>td</i>.
     *
     * @return string HTML-код таблицы.
     */
    public function getFull($title_width, $title, $show_title, $tr_str, $th_str, $td_str)
    {
        if((!isset($this->mValue[0]) || count($this->mLabel) != count($this->mValue[0])) && count($this->mLabel) > 0)
        {
            // кол-во заголовков не соотв. кол-ву значений
            return SB_LAYOUT_COUNT_TITLE_ARRAY_ERROR;
        }

        if ($this->mShowTable)
        {
            if ($title != '' && $show_title)
            {
                $result = '<tr'.($tr_str != '' ? ' '.$tr_str : '').'>
                           <th width="'.$title_width.'"'.($th_str != '' ? ' '.$th_str : '').'><img src="'.SB_CMS_IMG_URL.'/blank.gif" height="1" width="'.$title_width.'" /><br />'.$title.($this->mShowColon && $title != '' ? ':' : '').'</th>
                           <td width="100%"'.($td_str != '' ? ' '.$td_str : '').'>
                           <table width="100%" cellspacing="0" cellpadding="5" class="form" align="center"><tr>';
            }
            else
            {
                $result = '<tr'.($tr_str != '' ? ' '.$tr_str : '').'>
                       <td width="100%" colspan="2"'.($td_str != '' ? ' '.$td_str : '').'>
                       <table width="100%" cellspacing="0" cellpadding="5" class="form" align="center"><tr>';
            }
        }
        else
        {
            $result = '<tr>';
        }

        for($i = 0; $i < count($this->mLabel); $i++)
        {
            // вывод заголовков
            if(isset($this->mSort[$i]))
            {
                // если столбец участвует в сортировке, то выводим стрелки сортировки
                $result .= '
                <th class="header">
                    <nobr>
                        <a href="'.$_SERVER['PHP_SELF'].'?event='.$_GET['event'].'&'.$this->mSort[$i].'=up"><img src="'.SB_CMS_IMG_URL.'/'.(isset($_GET[$this->mSort[$i]]) && $_GET[$this->mSort[$i]] == 'up' ? 's_asc_sel.gif' : 's_asc.gif').'" border="0" width="11" height="9"></a>
                        &nbsp;
                        '.$this->mLabel[$i].'
                        &nbsp;<a href="'.$_SERVER['PHP_SELF'].'?event='.$_GET['event'].'&'.$this->mSort[$i].'=down"><img src="'.SB_CMS_IMG_URL.'/'.(isset($_GET[$this->mSort[$i]]) && $_GET[$this->mSort[$i]] == 'down' ? 's_desc_sel.gif' : 's_desc.gif').'" border="0" width="11" height="9"></a>
                    </nobr>
                </th>';
            }
            else
            {
                $result .= '<th class="header">'.($this->mLabel[$i] != '' ? $this->mLabel[$i] : '&nbsp;').'</th>';
            }
        }

        $result .= '</tr>';

        for($i = 0; $i < count($this->mValue); $i++)
        {
            // вывод значений таблицы
            $result .= '<tr'.($i % 2 != 0 ? ' class="even"' : ' class="odd"').'>';

            for($j = 0; $j < count($this->mValue[$i]); $j++)
            {
                $style = ' style="';
                $style .= (isset($this->mWidth[$j]) && $this->mWidth[$j] != '' ? 'width: '.$this->mWidth[$j].';' : '');
                $style .= (isset($this->mAlign[$j]) && $this->mAlign[$j] != '' ? 'text-align:'.$this->mAlign[$j].';' : '');
                $style .= (isset($this->mVAlign[$j]) && $this->mVAlign[$j] != '' ? 'vertical-align: '.$this->mVAlign[$j].';' : '');

                if ($style == ' style="')
                    $style = '';
                else
                    $style .= '"';

                $result .= '<td'.$style.'>'.$this->mValue[$i][$j].'</td>';
            }

            $result .= '</tr>';
        }

        if ($this->mShowTable)
            $result .= '</table></td></tr>';

        if ($this->mHighlight)
        {
            $result .= '
                <script>
                    sbAddEvent(window, "load", sbLayoutMarkRowsInit);
                </script>
                <style type="text/css">
                    #event_content table.form tr.even:hover,
                    #event_content table.form tr.odd:hover,
                    #event_content table.form tr.hover
                    {
                        background-color: #E1FFE1;
                    }
                </style>';
        }

        return $result.$this->mHTML;
    }
}
?>