<?php
/**
 * Класс, отвечающий за вывод текстовых полей в форме (textarea)
 *
 * @see sbLayout
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbLayoutTextarea
{
    /**
     * Текст по умолчанию (значение атрибута <i>value</i>)
     *
     * @var string
     */
    private $mValue = '';

    /**
     * Имя поля (значение атрибута <i>name</i>)
     *
     * @var string
     */
    private $mName = '';

    /**
     * Идентификатор поля (значение атрибута <i>id</i>)
     *
     * @var string
     */
    private $mId = '';

    /**
     * Дополнительные св-ва поля (например, стили)
     *
     * @var string
     */
    private $mDopStr = '';

    /**
     * Поле обязательно для заполнения (TRUE) или нет (FALSE)
     *
     * @var bool
     */
    private $mRequired = false;

    /**
     * Кол-во столбцов в поле (значение атрибута <i>cols</i>)
     *
     * @var int
     */
    public $mCols = '';

    /**
     * Кол-во строк в поле (значение атрибута <i>rows</i>)
     *
     * @var int
     */
    public $mRows = '4';

    /**
     * Выводить или нет панель инструментов (кнопки поиск и прочее)
     *
     * @var boolean
     */
    public $mShowToolbar = true;

     /**
     * Выводить или нет Счетчик символов
     *
     * @var boolean
     */
    public $mShowCharCount = false;

    /**
     * Выводить или нет кнопку <i>Развернуть/Свернуть</i>
     *
     * @var boolean
     */
    public $mShowEnlargeBtn = true;

    /**
     * Выводить или нет кнопку <i>Редактировать</i>
     *
     * @var boolean
     */
    public $mShowEditorBtn = false;

    /**
     * Массив для построения выпадающего списка тегов
     *
     * Содержит вставляемые теги.
     *
     * @var array
     */
    public $mTags = array();

    /**
     * Массив для построения выпадающего списка тегов
     *
     * Содержит описания тегов.
     *
     * @var array
     */
    public $mValues = array();

    /**
     * Дополнительный HTML-код, выводимый сверху поля
     *
     * @var string
     */
    public $mTopHtml = '';

    /**
     * Дополнительный HTML-код, выводимый под полем
     *
     * @var string
     */
    public $mBottomHtml = '';

    /**
     * Максимальное значение символов
     *
     * @var integer
     */
    public $mMaxValue = '';

    /**
     * Конструктор класса
     *
     * @param string $type Тип поля (значение атрибута <i>type</i>)
     * @param string $value Текст по умолчанию (значение атрибута <i>value</i>).
     * @param string $name Имя поля (значение атрибута <i>name</i>).
     * @param string $id Идентификатор поля (значение атрибута <i>id</i>).
     * @param string $dop_str Дополнительные св-ва поля (например, стили).
     * @param bool $required Поле обязательно для заполнения (TRUE) или нет (FALSE).
     * @param bool $highlight Использовать подсветку синтаксиса или нет.
     */
    public function __construct($value='', $name='', $id='', $dop_str='', $required=false)
    {
        if ($id == '') $id = $name;

        $this->mValue = $value;
        $this->mName = $name;
        $this->mId = $id;
        $this->mDopStr = $dop_str;
        $this->mRequired = $required;
    }

    /**
     * Возвращает HTML-код поля
     *
     * @return string HTML-код поля.
     */
    public function getField()
    {
        $show_table = $this->mShowToolbar || $this->mShowEnlargeBtn || count($this->mTags) > 0;

        $result = '<div id="div_'.$this->mId.'">';

        if ($show_table)
        {
            $result .= '<table border="0" cellpadding="3" cellspacing="0" width="100%"><tr id="toolbar_'.$this->mId.'" style="background-color: transparent;">';

            if ($this->mShowToolbar)
            {
                $result .= '<td width="1" unselectable="on" align="left">
                                <img class="button" onmouseup="sbPress(this, false)" onmousedown="sbPress(this, true);" onclick="sbLayoutFind(\''.$this->mId.'\');" alt="'.SB_LAYOUT_FIND_REPLACE.'" title="'.SB_LAYOUT_FIND_REPLACE.'" align="absmiddle" src="'.SB_CMS_IMG_URL.'/icon_find.gif">
                            </td>
                            <td width="1" unselectable="on" align="left">
                                <img class="button" id="btnHighlight_'.$this->mId.'" onmouseup="sbPress(this, false)" onmousedown="sbPress(this, true);" onclick="sbLayoutHighlight(this, \''.$this->mId.'\', true);" alt="'.SB_LAYOUT_HIGHLIGHT.'" title="'.SB_LAYOUT_HIGHLIGHT.'" align="absmiddle" src="'.SB_CMS_IMG_URL.'/icon_highlight.gif">
                            </td>';
            }

            if ($this->mTopHtml != '')
            {
                $result .= '<td align="left">
                             '.$this->mTopHtml.'
                            </td>';
            }

            if (count($this->mTags) > 0)
            {
                $result .= '<td align="left">
                            <nobr>&nbsp;'.SB_LAYOUT_TAGS.':&nbsp;&nbsp;<select id="sel_'.$this->mId.'">';

                $was_group = false;
                foreach ($this->mTags as $key => $value)
                {
                    $opt_title = $this->mValues[$key];
                    if (sb_strlen($opt_title) > 45)
                        $opt_title = sb_substr($opt_title, 0, 45).'...';

                    if ($value == '-')
                    {
                        if ($was_group)
                            $result .= '</optgroup>';

                        $result .= '<optgroup label="'.$opt_title.'">';
                        $was_group = true;
                        continue;
                    }
                    $result .= '<option value="'.$value.'">'.$opt_title.'</option>';
                }

                if ($was_group)
                    $result .= '</optgroup>';


                $result .= '</select>&nbsp;&nbsp;&nbsp;<input type="button" id="btn_'.$this->mId.'" name="btn_'.$this->mId.'" onclick="sbLayoutChooseTag(\''.$this->mId.'\')" value="'.SB_LAYOUT_INSERT.'"></nobr>
                        </td>';
            }

            if ($this->mShowEnlargeBtn)
            {
                $result .= '<td align="right">
                            &nbsp;&nbsp;<img src="'.SB_CMS_IMG_URL.'/roll_or_off.gif" id="minmax_'.$this->mId.'" onclick="sbLayoutEnlargeArea(this, \''.$this->mId.'\');" width="20" alt="'.SB_LAYOUT_ENLARGE.'" title="'.SB_LAYOUT_ENLARGE.'" style="cursor:hand;cursor:pointer;" />
                            </td>';
            }
            $result .= '</tr></table>';
        }

        $result .= '<table border="0" cellpadding="0" cellspacing="0" width="100%"><tr><td><textarea '.($this->mShowCharCount && is_numeric($this->mMaxValue) && $this->mMaxValue > 0 ? ' oninput = "sbLayoutCharCount(this, \''.$this->mId.'\',\''.$this->mMaxValue.'\')" onfocus = "sbLayoutCharCount(this, \''.$this->mId.'\',\''.$this->mMaxValue.'\')" onkeyup="sbLayoutCharCount(this, \''.$this->mId.'\',\''.$this->mMaxValue.'\')"' : '').' spellcheck="false"'.($this->mName != '' ? ' name="'.$this->mName.'"' : '').($this->mId != '' ? ' id="'.$this->mId.'"' : '').($this->mRows != '' ? ' rows="'.$this->mRows.'" ' : '').($this->mCols != '' ? ' cols="'.$this->mCols.'" ' : '').' '.$this->mDopStr.'>'.sb_htmlspecialchars($this->mValue, ENT_QUOTES).'</textarea></td>'.($this->mRequired ? '<td valign="top" width="1">&nbsp;<sup class="red">*</sup></td>' : '').'</tr></table>';

        if ($this->mShowCharCount && is_numeric($this->mMaxValue) && $this->mMaxValue > 0)
        {
            $result .= '<div style="padding-top:3px; float:left;">'.SB_LAYOUT_VALUE_CHAR_COUNT.' <span id="char_count_'.$this->mId.'" style="font-style:italic;">'.($this->mMaxValue - sb_strlen(sb_htmlspecialchars($this->mValue, ENT_QUOTES))).'</span></div>';
        }
        if ($this->mShowEditorBtn)
            $result .= '<div align="right" style="padding-top: 3px;"><input type="button" id="edit_btn_'.$this->mId.'" name="edit_btn_'.$this->mId.'" value="'.KERNEL_EDITOR.'" onclick="sbLayoutEdit(\''.$this->mId.'\')"></div>';

        $result .= '</div>';

        if($this->mShowCharCount && is_numeric($this->mMaxValue) && $this->mMaxValue > 0)
        {
            $result = $result.'
            <script>
                sbLayoutCharCount(sbGetE("'.$this->mId.'"), \''.$this->mId.'\',\''.$this->mMaxValue.'\');
            </script>
            ';
        }

        $result .= $this->mBottomHtml;

        return $result;
    }

    /**
     * Возвращает JavaScript-код поля
     *
     * @return string JavaScript-код поля.
     */
    public function getJavaScript()
    {
        static $echo = true;
        if ($echo)
        {
            $echo = false;

            return '<script src="'.SB_CMS_JSCRIPT_URL.'/codeditor/sbCodeditor.js.php"></script>';
        }
        else
        {
            return '';
        }
    }
}
?>