<?php
/**
 * Класс, отвечающий за вывод поля выбора файла с компьютера пользователя
 *
 * @see sbLayout
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2009, OOO "СИБИЭС Групп"
 */
class sbLayoutUpload
{
    /**
     * Имя поля (значение атрибута <i>name</i>)
     *
     * @var string
     */
    protected $mName = '';

    /**
     * Идентификатор поля (значение атрибута <i>id</i>)
     *
     * @var string
     */
    protected $mId = '';

    /**
     * Дополнительные св-ва поля (например, стили)
     *
     * @var string
     */
    protected $mDopStr = '';

    /**
     * Событие, вызываемое для загрузки файлов на сервер
     *
     * @var string
     */
    private $mUploadEvent = '';

    /**
     * Имя временного файла.
     *
     * @var string
     */
    private $mTmpFileName = '';

    /**
     * Дополнительный HTML-код, вставляемый после поля
     *
     * @var string
     */
    public $mHTML = '';

    /**
     * Поле обязательно для заполнения (TRUE) или нет (FALSE)
     *
     * @var bool
     */
    protected $mRequired = false;

    /**
     * Строка, содержащая расширения отображаемых файлов через пробел (например, gif jpg png)
     *
     * @var string
     */
    public $mFileTypes = '';

    /**
     * JavaScript-фнукция, вызываемая после выбора файла.
     *
     * @var string
     */
    public $mFileNameChange = '';

    /**
     * JavaScript-фнукция, вызываемая после загрузки файла.
     *
     * @var string
     */
    public $mFileUploadComplete = '';

    /**
     * Сообщение об успешной загрузке файла на сервер.
     *
     * @var string
     */
    public $mSuccessMessage = '';

    /**
     * Значение по умолчанию. Устанавливается, если надо показать путь к текущему файлу.
     * Применяется, как правило, в пользовательских полях
     * @var string
     */
    public $mValue = '';

    /**
     * Конструктор класса
     *
     * @param string $upload_event Событие, вызываемое для загрузки файлов на сервер.
     * @param string $name Имя поля (значение атрибута <i>name</i>).
     * @param string $id Идентификатор поля (значение атрибута <i>id</i>).
     * @param string $dop_str Дополнительные св-ва поля (например, стили).
     * @param bool $required Поле обязательно для заполнения (TRUE) или нет (FALSE).
     */
    public function __construct($upload_event, $name='', $id='', $dop_str='', $required=false)
    {
        if ($id == '') $id = $name;

        $this->mName = $name;
        $this->mId = $id;
        $this->mDopStr = $dop_str;
        $this->mRequired = $required;
        $this->mUploadEvent = $upload_event;
        $this->mTmpFileName = time().'_'.uniqid().'_upload.tmp';
    }

    /**
     * Возвращает HTML-код поля
     *
     * @return string HTML-код поля.
     */
    public function getField()
    {
        if((int)@ini_get('post_max_size') > (int)@ini_get('upload_max_filesize')){
            $maxSize = @ini_get('upload_max_filesize');
        }else{
            $maxSize = @ini_get('post_max_size');
        }
        return '<table cellpadding="0" cellspacing="0" style="border:0;">
                <tr style="background-color:transparent;">
                <td>
                    <input type="text" name="'.$this->mName.'" id="'.$this->mId.'" value="'.$this->mValue.'" readonly="true" '.($this->mDopStr != '' ? ' '.$this->mDopStr : '').' />
                    <input type="hidden" name="tmp_file_name_'.$this->mName.'" id="'.$this->mId.'_tmp" value="'.$this->mTmpFileName.'" />
                    <input type="hidden" name="'.$this->mName.'_tmp" id="'.$this->mId.'_tmp" value="'.$this->mTmpFileName.'" />
                    <input type="hidden" name="'.$this->mName.'_uploaded" id="'.$this->mId.'_uploaded" value="0"/>
                '.($this->mRequired ? ' <sup class="red">*</sup>' : '').'</td>
                <td style="padding-top: 2px;padding-left: 5px;">
                <div style="position: relative">
                <button id="'.$this->mId.'_btn" onclick="customClick(\''.$this->mId.'\'); return false;">'.KERNEL_BROWSE.'</button>&nbsp;&nbsp;<sup>('.SB_LAYOUT_MAX.' '.$maxSize.')</sup>
                <p style="float:left; position: absolute; left: 0; opacity: 0; filter: alpha(opacity=0); margin: 0; top: 0;"><input style="width: 10px; opacity: 0.0; filter: alpha(opacity=0);" id="ajax_upload_'.$this->mId.'" name="sb_file_upload_'.$this->mName.'" type="file" onChange="upload(this, \''.$this->mId.'\')"></p>
                </div>
                </td>
                </tr></table>
            '.$this->mHTML.'

            <div class="progressWrapper" id="result_'.$this->mId.'" style="opacity: 1; display: none;">
                <div id="progressContainer_'.$this->mId.'" class="progressContainer">
                    <div id="progressName_'.$this->mId.'" class="progressName"></div>
                    <div id="progressBarStatus_'.$this->mId.'" class="progressBarStatus"></div>
                    <div id="progressBar_'.$this->mId.'" style=""></div>
                </div>
            </div>';
    }

    /**
     * Возвращает JavaScript-код поля
     *
     * @return string JavaScript-код поля.
     */
    public function getJavaScript()
    {
        static $echo = true;
        $result = '';

        if ($this->mFileTypes != '')
        {
            $this->mFileTypes = explode(' ', $this->mFileTypes);
            foreach ($this->mFileTypes as $key => $value)
            {
                $this->mFileTypes[$key] = '*.'.$value;
            }

            $this->mFileTypes = implode(';', $this->mFileTypes);
        }
        else
        {
            $this->mFileTypes = '*.*';
        }

        if ($echo)
        {
            $echo = false;
            $result .= '<script>
                function customClick(id){
                    if(document.all)
                    {
                    }
                    else
                    {
                        sbGetE("ajax_upload_"+id).click();
                    }
                }

            AIM = {

                frame : function(c) {
                    var n = "f" + Math.floor(Math.random() * 99999);
                    var d = document.createElement("DIV");
                    d.innerHTML = "<iframe uid=\'"+c.uId+"\' style=\'display:none\' src=\'about:blank\' id=\'"+n+"\' name=\'"+n+"\' onload=\'AIM.loaded(\""+n+"\")\'></iframe>";
                    document.body.appendChild(d);

                    var i = document.getElementById(n);
                    if (c && typeof(c.onComplete) == "function") {
                        i.onComplete = c.onComplete;
                    }

                    return n;
                },

                form : function(f, name) {
                    f.setAttribute("target", name);
                },

                submit : function(f, c) {
                    AIM.form(f, AIM.frame(c));
                    if (c && typeof(c.onStart) == "function") {
                        return c.onStart(c.uId);
                    } else {
                        return true;
                    }
                },

                loaded : function(id) {
                    var i = document.getElementById(id);
                    if (i.contentDocument) {
                        var d = i.contentDocument;
                    } else if (i.contentWindow) {
                        var d = i.contentWindow.document;
                    } else {
                        var d = window.frames[id].document;
                    }
                    if (d.location.href == "about:blank") {
                        return;
                    }
                    var uId = i.getAttribute("uid");

                    if (typeof(i.onComplete) == "function") {
                        i.onComplete(d.body.innerHTML, uId);
                    }
                }

            }

            // HTML5 upload files
            window.ondragover = function(e) {
                e.preventDefault();
            }
            window.ondrop = function(e) {
                e.preventDefault();
                upload(e.dataTransfer.files[0], e.srcElement.id);
            }

            function upload(obj, fld_id) {

                if(obj.type != "file"){
                    uniqueId = fld_id;
                }
                else
                {
                var tmp = obj.id.split("_");
                var uniqueId = tmp[2];
                for(var i in tmp){
                    if(i >2){
                        uniqueId += "_"+tmp[i];
                    }
                }
                }

                if (!window.FormData) {
                    var form = sbGetE("main");
                    var tmp = form.action;
                    form.action = "' . SB_CMS_EMPTY_FILE . '?event=' . $this->mUploadEvent . '";
                    AIM.submit(form, {\'onStart\': startUpload, \'onComplete\': completeUpload, \'uId\':uniqueId});
                    form.submit();
                    form.action = tmp;
                    form.setAttribute("target", "thisDialog");
                    return;
                }


                if(obj.type == "file"){
                    var file = obj.files[0];
                }else{
                    var file = obj;
                }

                sbAJAXInit(false);

                var fd = new FormData();
                fd.append("sb_file_upload", file);
                fd.append("tmp_file_name", sbGetE(uniqueId+"_tmp").value);

                var className = sbGetE("progressContainer_"+uniqueId).className.replace(" red","");
                sbGetE("progressContainer_"+uniqueId).className = className;
                sbGetE("progressName_"+uniqueId).innerHTML = file.name;
                sbGetE("progressBarStatus_"+uniqueId).innerHTML = "' . SB_LAYOUT_UPLOAD_UPLOADING . '";
                sbGetE("progressBar_"+uniqueId).className = "progressBarInProgress";
                sbGetE("result_"+uniqueId).style.display = "block";
                sbGetE(uniqueId+"_btn").disabled = true;
                sbGetE(uniqueId).value = file.mozFullPath ? file.mozFullPath : file.name;

                if(typeof(sbGetE(uniqueId).onchange) == "function"){
                    sbGetE(uniqueId).onchange();
                }
                sbXMLHTTP.upload.addEventListener("progress", function(e) {
                    if (e.lengthComputable) {
                        var progress = (e.loaded * 100) / e.total;
                        sbGetE("progressBar_"+uniqueId).style.width = progress+"%";
                    }
                }, false);

                sbXMLHTTP.open("POST", "' . SB_CMS_EMPTY_FILE . '?event=' . $this->mUploadEvent . '");
                sbXMLHTTP.onload = function() {
                    if(sbXMLHTTP.responseText == "TRUE"){
                        var className = sbGetE("progressContainer_"+uniqueId).className.replace(" red","");
                        sbGetE("progressContainer_"+uniqueId).className = className;
                        sbGetE("progressBarStatus_"+uniqueId).innerHTML = "' . SB_LAYOUT_UPLOAD_COMPLETE . '";
                        sbGetE("progressBar_"+uniqueId).className = "progressBarComplete";
                        sbGetE("progressContainer_"+uniqueId).className = "progressContainer blue";
                        sbGetE(uniqueId+"_btn").disabled = false;
                        ' . ($this->mFileUploadComplete != '' ? $this->mFileUploadComplete . '();' : '') . '
                        sbGetE(uniqueId+"_uploaded").value = 1;
                    }else{
                        sbGetE("progressContainer_"+uniqueId).className += " red";
                        sbGetE(uniqueId+"_btn").disabled = false;
                        sbGetE("progressBarStatus_"+uniqueId).innerHTML = sbXMLHTTP.responseText;
                    }
            }
            sbXMLHTTP.onreadystatechange = function(ev){
                if(sbXMLHTTP.readyState ==  4 && sbXMLHTTP.status == 0){
                    alert("'.SB_LAYOUT_DOWNLOAD_ERROR.'");
                }
            }
            sbXMLHTTP.send(fd);
        }

        function startUpload(uId){
            sbGetE(uId+"_btn").disabled = true;
            var className = sbGetE("progressContainer_"+uId).className.replace(" red","");
            sbGetE("progressContainer_"+uId).className = className;
            sbGetE("progressName_"+uId).innerHTML = sbGetE("ajax_upload_"+uId).value;
            sbGetE("progressBarStatus_"+uId).innerHTML = "' . SB_LAYOUT_UPLOAD_UPLOADING . '";
            sbGetE("progressBar_"+uId).className = "progressBarInProgress";
            sbGetE("result_"+uId).style.display = "block";
            var fileName = sbGetE("ajax_upload_"+uId).value.replace(/\\\/g, "/");
            var tmp = fileName.split("/");
            sbGetE(uId).value = tmp[tmp.length-1];
            return true;
        }

        function completeUpload(response, uId){
            if(response == "TRUE"){
                var className = sbGetE("progressContainer_"+uId).className.replace(" red","");
                sbGetE("progressContainer_"+uId).className = className;
                sbGetE("progressBarStatus_"+uId).innerHTML = "' . SB_LAYOUT_UPLOAD_COMPLETE . '";
                sbGetE("progressBar_"+uId).className = "progressBarComplete";
                sbGetE("progressContainer_"+uId).className = "progressContainer blue";
                sbGetE(uId+"_btn").disabled = false;
                ' . ($this->mFileUploadComplete != '' ? $this->mFileUploadComplete . '();' : '') . '
                sbGetE(uId+"_uploaded").value = 1;
                sbGetE(uId).fireEvent("onchange");
            }else{
                sbGetE("progressContainer_"+uId).className += " red";
                sbGetE(uId+"_btn").disabled = false;
                sbGetE("progressBarStatus_"+uId).innerHTML = response;
            }
        }
            </script>';
        }

        return $result;
    }
}
