<?php
require_once (SB_CMS_LIB_PATH . '/oauth/sbSocAuthInterface.php');

/**
 * Класс для авторизации поользователей в Facebook 
 */
class sbFacebook implements sbAuthInterface{
    /**
     * Ключ приложения
     * @var string $client_id 
     */
    private $client_id = null;
    
    /**
     * Секретный ключ приложения
     * @var string $client_secret 
     */
    private $client_secret;
    
    /**
     * Экземпляр этого класса
     * @var sbFacebook $_Instance 
     */
    static $_Instance = null;
    
    private $callback_url;
    
    /**
     * Свойства для возврата в sbSocAuth 
     */
    public $user_id = '';
    public $user_name = '';
    public $user_foto = '';
    public $errors = array(
        'system' => array(),
        'user' => array()
    );
    
    private function __construct(){
        $this->client_id = sbPlugins::getSetting('sb_social_login_facebook_consumer_key');
        $this->client_secret = sbPlugins::getSetting('sb_social_login_facebook_secret_key');
        $this->callback_url = SB_DOMAIN . '/cms/admin/scmedia.php?name=facebook';
    }
    
    private function __clone(){}
    
    static function getInstance(){
        if(self::$_Instance === null){
            self::$_Instance = new sbFacebook;
        }
        return self::$_Instance;
    }
    
    public function sendAuthRequest(){
        if(!$this->client_id || !$this->client_secret){
            $this->addError(get_class($this).': '.PL_SOCIAL_LOGIN_NOSETTINGS);
            $this->addError(PL_SOCIAL_LOGIN_NOAUTH, false);
            return $this;
        }
        $oauth = new FacebookOAuth($this->client_id, $this->client_secret);
        $request_link = $oauth->getCodeUrl(urlencode($this->callback_url));
        header('Location: '.sb_sanitize_header($request_link));
    }
    
    public function getUserData(){
        if(!isset($_GET['code'])){
            $this->addError(get_class($this).': '.SB_NOPARSE_RESPONSE_SERVER);
            $this->addError(PL_SOCIAL_LOGIN_NOAUTH, false);
            unset($_SESSION['scmedia']);
            return $this;
        }
        $oauth = new FacebookOAuth($this->client_id, $this->client_secret);
        $access_key = $oauth->getAccessToken(urlencode($this->callback_url),$_GET['code'],$this->callback_url);
        $res=$oauth->pagesGetInfo($access_key);

        if(isset($res->id)){
            $this->user_id = $res->id;
            $this->user_name = $res->name;
            $this->user_foto = isset($res->avatar) ? $res->avatar : '';
        }else{
            if(isset($res->error)){
                $this->addError(get_class($this).': '.$res->error->message);
            }else{
                $this->addError(get_class($this).': '.SB_NOPARSE_RESPONSE_SERVER);
            }
            $this->addError(PL_SOCIAL_LOGIN_NOAUTH, false);
        }
        return $this;
    }
    
    private function addError($message, $system = true){
        if($system){
            array_push($this->errors['system'], $message);
        }else{
            array_push($this->errors['user'], $message);
        }
    }
}


/**
 * Класс, реализующий протокол OAuth  
 */
class FacebookOAuth
 {

    private $auth_base_url = "https://www.facebook.com";
    private $api_base_url = "https://api.facebook.com";
    private $graph_base_url = "https://graph.facebook.com";
    private $client_id;
    private $client_secret;
    private $redirect_uri;

    public function __construct($client_id, $client_secret) {
        $this->client_id = $client_id;
        $this->client_secret = $client_secret;
    }

    function makeRequest($url, $callbackurl = "") {
        $request = new sbDownload($url);
        return $request->download();
        
    }

    function getCodeUrl($redirect_uri) {
        return $this->auth_base_url . "/dialog/oauth/?client_id=" . $this->client_id . "&redirect_uri=" . $redirect_uri ;
    }

    function getAccessToken($redirect_uri, $code, $callbackurl) {
        $res = $this->makeRequest($this->graph_base_url . "/oauth/access_token?client_id=" . $this->client_id . "&redirect_uri=" . $redirect_uri . "&client_secret=" . $this->client_secret . "&code=" . $code, $callbackurl);
        $res = explode('&', $res);
        $res = explode('=', $res[0]);
        if (isset($res[1])) {
            return $res[1];
        } else {
            return $res;
        }
    }

    function streamPublish($msg, $access_token) {
        if (empty($access_token)) {
            return false;
        }
        $res = $this->makeRequest($this->api_base_url . '/method/stream.publish?message=' . urlencode($msg) . "&access_token=" . $access_token);

        preg_match_all('/<stream_publish_response[^>]*>(.*?)<\/stream_publish_response>/si', $res, $matches, PREG_PATTERN_ORDER);


        if (!empty($matches) && !empty($matches[1][0])) {
            return $matches[1][0];
        } else {
            return false;
        }
    }

    function pagesGetInfo($access_token) {
        if (empty($access_token)) {
            return false;
        }
        $res = $this->makeRequest($this->graph_base_url . "/me/?access_token=" . $access_token);
        $res = sb_json_decode($res);
        if($res->id)
        {
            $pic = $this->makeRequest($this->graph_base_url . '/'.$res->id.'/picture?width=100&height=100&redirect=false&access_token=' . $access_token);
            if($pic)
            {
                $pic = sb_json_decode($pic);
                $res -> avatar = isset($pic -> data -> url) ? $pic -> data -> url : '';
            }
        }

        return $res;
    }

} 