<?php
require_once SB_CMS_LIB_PATH . '/oauth/sbSocAuthInterface.php';

/**
 * Авторизация пользователей через LiveJournal
 *
 * @author Сергей Черячукин <sc@binn.ru>
 */
class sbLiveJournal implements sbAuthInterface{

    static $_Instance = null;

    /**
     * Свойства для возврата в sbSocAuth
     */
    public $user_id = '';
    public $user_name = '';
    public $user_foto = '';
    public $errors = array(
        'system' => array(),
        'user' => array()
    );

    private function __construct(){

    }

    private function __clone(){}

    public function sendAuthRequest(){
        if(!isset($_POST['lj_user']) || !isset($_POST['lj_pass'])
        || empty($_POST['lj_user']) || empty($_POST['lj_pass'])
        ){
            $this->addError(PL_SOCIAL_LOGIN_LJ_NOLOGIN, false);
            return;
        }else{
            $params = array('method' => 'getchallenge');
            $result = $this->userVerify();
            return $result;
        }
    }

    public function getUserData(){
        if(sbPlugins::getSetting('sb_social_login_livejournal_enabled') != 1){
            $this->addError(get_class($this).': '.PL_SOCIAL_LOGIN_NOSETTINGS);
            $this->addError(PL_SOCIAL_LOGIN_NOAUTH, false);
            return $this;
        }
        $user =  $this->sendAuthRequest();
        if(is_array($user)){
            $this->user_id = $user['id'];
            $this->user_name = $user['user_name'];
            $this->user_foto = $user['user_foto'];
        }
        
        return $this;
    }

    /**
     * Отправка запроса серверу соц. сети
     * @param string $url
     * @param string $xml
     * @return array
     */
    private function sendRequestXML($url, $xml){
        if (!function_exists('curl_init')) {
            $this->addError(get_class($this) . ': ' . PL_SOCIAL_LOGIN_NO_CURL);
            $this->addError(PL_SOCIAL_LOGIN_NOAUTH, false);
            return $this;
        }

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt ($ch, CURLOPT_HTTPHEADER, Array("Content-Type: text/xml", "Content-Length: ".  strlen($xml)));
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

        $result = curl_exec($ch);
        curl_close($ch);
        return $result;
    }

    private function getChallengeRequest(){
        $xml = '<?xml version="1.0"?>
                <methodCall>
                <methodName>LJ.XMLRPC.getchallenge</methodName>
                <params>
                <param>
                <value><struct>
                </struct></value>
                </param>
                </params>
                </methodCall>';

        return $this->sendRequestXML('http://livejournal.com/interface/xmlrpc', $xml);
    }

    private function userVerify(){
        $result = $this->getChallengeRequest();

        if(($parse = @simplexml_load_string($result)) instanceof SimpleXMLElement){
            if(!($challenge = $parse->params->param->value->struct->member[2]->value->string)){
                $this->addError(get_class($this).': '.SB_NOPARSE_RESPONSE_SERVER);
                $this->addError(PL_SOCIAL_LOGIN_NOAUTH, false);
                return;
            }
            $auth_response = md5($challenge.md5($_POST['lj_pass']));
            $xml = '<?xml version="1.0"?>
                    <methodCall>
                        <methodName>LJ.XMLRPC.login</methodName>
                        <params>
                            <param>
                                <value><struct>
                                <member>
                                    <name>auth_method</name>
                                    <value><string>challenge</string></value>
                                </member>
                                <member>
                                    <name>username</name>
                                    <value><string>'.$_POST['lj_user'].'</string></value>
                                </member>
                                <member>
                                    <name>auth_challenge</name>
                                    <value><string>'.$challenge.'</string></value>
                                </member>
                                <member>
                                    <name>auth_response</name>
                                    <value><string>'.$auth_response.'</string></value>
                                </member>
                                <member>
                                    <name>getpickws</name>
                                    <value><string>1</string></value>
                                </member>
                                <member>
                                    <name>getpickwurls</name>
                                    <value><string>1</string></value>
                                </member>
                                </struct></value>
                            </param>
                        </params>
                    </methodCall>
            ';

            $user_id = $username = '';

            $result = $this->sendRequestXML('http://livejournal.com/interface/xmlrpc', $xml);
            $parse = simplexml_load_string($result);
            
            $user_id = $parse->xpath('.//member[name="userid"]/value/int');
            $user_id = isset($user_id[0]) ? (int)$user_id[0] : '';
            $username = $parse->xpath('.//member[name="fullname"]/value/string');
            $username = isset($username[0]) ? (string)$username[0] : '';
            $userfoto = $parse->xpath('.//member[name="defaultpicurl"]/value/string');
            $userfoto = isset($userfoto[0]) ? (string)$userfoto[0] : '';

            if($user_id != '' && $username != ''){
                $result = array(
                    'id'=>$user_id,
                    'user_name' =>$username, 
                    'user_foto' =>$userfoto
                );
                return $result;
            }else{
                $error = $parse->xpath('.//member[name="faultString"]/value/string');
                $error = (string)$error[0];
                if($error != 'Invalid password'){
                    $this->addError(get_class($this).': '.$error);
                    $this->addError(PL_SOCIAL_LOGIN_NOAUTH, false);
                }else{
                    $this->addError(PL_SOCIAL_LOGIN_WRONG_LOGIN, false);
                }
            }
        }else{
            $this->addError(get_class($this).': '.SB_NOPARSE_RESPONSE_SERVER);
        }
        return;
    }

    private function addError($message, $system = true){
        if($system){
            array_push($this->errors['system'], $message);
        }else{
            array_push($this->errors['user'], $message);
        }
    }

    static function getInstance(){
        if(self::$_Instance === null){
            self::$_Instance = new sbLiveJournal;
        }
        return self::$_Instance;
    }
}