<?php
require_once SB_CMS_LIB_PATH . '/oauth/sbSocAuthInterface.php';

/**
 * Класс, реализующий авторизацию пользователей в Одноклассниках 
 */
class sbOdnoklassniki implements sbAuthInterface{
    /**
     * ID приложения в  соц. сети, выданный при регистрации приложения
     */
    private $client_id = null;
    
    /**
     * Ключ приложения 
     */
    private $application_key = null;
    
    /**
     * Подпись приложения 
     */
    private $secret_key = null;
    
    /**
     * Адрес API домена соц. сети (с закрывающим слешем)
     */
    private $domainAPI = 'http://api.odnoklassniki.ru/';
    
    /**
     * Адрес на который будет будет оправлен ответ от сервера соц. сети
     * @var string $redirectURL 
     */
    private $redirectURL;
    
    /**
     * Содержит экземпляр данного класса
     * @var sbOdnoklassniki $_Instance 
     */
    static $_Instance = null;
    
    /**
     * Свойства для возврата в sbSocAuth 
     */
    public $user_id = '';
    public $user_name = '';
    public $user_foto = '';
    public $errors = array(
        'system' => array(),
        'user' => array()
    );
    
    private function __construct(){
        $this->client_id = sbPlugins::getSetting('sb_social_login_odk_app_id');
        $this->application_key = sbPlugins::getSetting('sb_social_login_odk_public_key');
        $this->secret_key = sbPlugins::getSetting('sb_social_login_odk_secret_key');
        $this->redirectURL = SB_DOMAIN . '/cms/admin/scmedia.php';
    }
    
    private function __clone(){
        
    }
    
    static function getInstance(){
        if(self::$_Instance === null){
            self::$_Instance = new sbOdnoklassniki();
        }
        return self::$_Instance;
    }
    
    /**
     * Формирование и отправка запроса на авторизацию. 
     */
    public function sendAuthRequest(){
        if(!$this->client_id || !$this->application_key || !$this->secret_key){
            $this->addError(get_class($this).': '.PL_SOCIAL_LOGIN_NOSETTINGS);
            $this->addError(PL_SOCIAL_LOGIN_NOAUTH, false);
            return $this;
        }
        $url = "http://www.odnoklassniki.ru/oauth/authorize?client_id=$this->client_id"
                ."&scope=VALUABLE ACCESS"
                .'&response_type=code'
                .'&redirect_uri='.urlencode($this->redirectURL.'?name=odk');
        header('Location: '.sb_sanitize_header($url));
    }
    
    public function getUserData(){
        if(isset($_GET['code'])){
            if(!function_exists('curl_init')){
                $this->addError(get_class($this).': '.PL_SOCIAL_LOGIN_NO_CURL);
                $this->addError(PL_SOCIAL_LOGIN_NOAUTH, false);
                unset($_SESSION['scmedia']);
                return $this;
            }
            
            $curl = curl_init($this->domainAPI . 'oauth/token.do');
            curl_setopt($curl, CURLOPT_POST, 1);
            curl_setopt($curl, CURLOPT_POSTFIELDS, 'code=' . $_GET['code'] . '&redirect_uri=' . urldecode($this->redirectURL.'?name=odk') . '&grant_type=authorization_code&client_id=' . $this->client_id . '&client_secret=' . $this->secret_key);
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
            $s = curl_exec($curl);
            curl_close($curl);
            $auth = sb_json_decode($s, true);
            if(isset($auth->error)){
                $this->addError(get_class($this).': '.$auth->error_description);
                $this->addError(PL_SOCIAL_LOGIN_NOAUTH, false);
                unset($_SESSION['scmedia']);
                return $this;
            }elseif(empty($auth)){
                $this->addError(get_class($this).': '.SB_NOPARSE_RESPONSE_SERVER);
                $this->addError(PL_SOCIAL_LOGIN_NOAUTH, false);
                unset($_SESSION['scmedia']);
                return $this;
            }
            
            $curl = curl_init($this->domainAPI . 'fb.do?access_token=' . $auth->access_token . '&application_key=' . $this->application_key . '&method=users.getCurrentUser&sig=' . md5('application_key=' . $this->application_key . 'method=users.getCurrentUser' . md5($auth->access_token . $this->secret_key)));
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
            $s = curl_exec($curl);
            curl_close($curl);
            $user = sb_json_decode($s, true);
            
            if(isset($user->error)){
                unset($_SESSION['scmedia']);
                $this->addError(get_class($this).': '.$user->error->error_msg);
                $this->addError(PL_SOCIAL_LOGIN_NOAUTH, false);
                return $this;
            }
             
            if(isset($user->uid)){
                $this->user_id = $user->uid;
                $this->user_name = $user->name;
                 $this->user_foto = $user->pic_2;
            }else{
                $this->addError(get_class($this).': '.$user->error_msg);
                $this->addError(PL_SOCIAL_LOGIN_NOAUTH, false);
            }
            return $this;
        }else{
            unset($_SESSION['scmedia']);
            if(isset($_GET['error']) && $_GET['error'] != 'access_denied'){
                $this->addError(get_class($this).': '.$_GET['error']);
                $this->addError(PL_SOCIAL_LOGIN_NOAUTH, false);
            }elseif(isset($_GET['error'])){
                $this->addError(PL_SOCIAL_LOGIN_WRONG_LOGIN, false);
            }else{
                $this->addError(get_class($this).': '.$_GET['error_description']);
            }
            return $this;
        }
    }
    
    private function addError($message, $system = true){
        if($system){
            array_push($this->errors['system'], $message);
        }else{
            array_push($this->errors['user'], $message);
        }
    }
}
?>