<?php
require_once SB_CMS_LIB_PATH . '/sbDownload.inc.php';

if(!class_exists('sbAuth'))
{
    require_once SB_CMS_LIB_PATH . '/prog/sbAuth.inc.php';
}
/**
 * Класс-фабрика для авторизации пользователей в соц. сетях
 */

class sbSocAuth extends sbAuth
{
    /*
     * Свойство, содержащее экземпляр класса
     */
    static $_Instance = null;
    
    /**
     * Содержит экземпляр класса, реализующего взаимодействие с данной соц. сетью
     * @var mixed $socNetObj 
     */
    private $socNetObj = null;
    private $netName = '';
    
    public function __construct($netName){
        $this->netName = $netName;
        switch (strtolower($netName)){
            case 'odk': 
                require_once (SB_CMS_LIB_PATH . '/oauth/sbOdnoklassniki.php');
                $this->socNetObj = sbOdnoklassniki::getInstance();
                break;
            case 'twitter': 
                require_once (SB_CMS_LIB_PATH . '/oauth/sbTwitter.php');
                $this->socNetObj = sbTwitter::getInstance();
                break;
            case 'livejournal':
                require_once (SB_CMS_LIB_PATH . '/oauth/sbLiveJournal.php');
                $this->socNetObj = sbLiveJournal::getInstance();
                break;
            case 'facebook': 
                require_once (SB_CMS_LIB_PATH . '/oauth/sbFacebook.php');
                $this->socNetObj = sbFacebook::getInstance();
                break;
            case 'vk': 
                require_once (SB_CMS_LIB_PATH . '/oauth/sbVKontakte.php');
                $this->socNetObj = sbVKontakte::getInstance();
                break;
            default: return null;
        }
    }
    
    private function __clone(){
        
    }
    
    static function getInstance($netName){
        if(self::$_Instance === null){
            self::$_Instance = new sbSocAuth($netName);
        }
        return self::$_Instance;
    }
    
    public function sendAuthRequest(){
        return $this->socNetObj->sendAuthRequest();
    }
    
    public function getUserData(){
        $this->socNetObj->getUserData();
        return $this;
    }
    
    public function doLogin()
    {
        if(!$this->user_id)
        {
            $errors = $this->getSystemErrors();
            if(!empty($errors))
            {
                sb_add_system_message(implode('<br/>', $errors), SB_MSG_WARNING);
            }
            
            $errors = $this->getUserErrors();
            if(!empty($errors))
            {
                $messages = implode('<br/>', $errors);
                $_SESSION['error_message'] = trim($messages);
            }
            return $this;
        }
        
        $error = false;
        $message = '';
        
        //Проверяем, зарегистрирован ли такой аккаунт из соц. сети
        $res = sql_param_query('SELECT sbsu_id, sbsu_uid 
            FROM sb_socnet_users 
            LEFT JOIN sb_site_users on sbsu_uid = su_id
            WHERE sbsu_id=?s AND sbsu_sn_type=?s', $this->user_id, $this->netName);
        
        
        if(!$res)
        {
            //Аккаунта соц. сети нет
            if(isset($_SESSION['sbAuth']) && $_SESSION['sbAuth']->getErrorCode() == -1)
            {
                //Пользователь авторизирован под основным аккаунтом
                $data = array(
                    'sbsu_uid' => $_SESSION['sbAuth']->getUserId(),
                    'sbsu_id' => $this->user_id,
                    'sbsu_sn_type' => $this->netName
                );
                
                sql_param_query('INSERT INTO sb_socnet_users SET ?a', $data);
                $this->updateAvatar($_SESSION['sbAuth']->getUserId());
                $_SESSION['sbAuth']->getSocialNetworks(true);
            }
            elseif(!isset($_SESSION['sbAuth']))
            {
                //Создаем новый аккаунт
                if(($soc_login = $this->createSiteUser()) !== false)
                {
                    $auth = new sbAuth('', '', '');
                    if ($auth->getUserId() <= 0)
                    {
                        array_push($this->socNetObj->errors['user'], sprintf(KERNEL_PROG_SITEUSERS_AUTH_ERROR, $soc_login). '(Error code: '.$auth->getErrorCode().')');
                        sb_add_system_message(sprintf(KERNEL_PROG_SITEUSERS_AUTH_ERROR, $soc_login). '(Error code: '.$auth->getErrorCode().')', SB_MSG_WARNING);
                    }
                }
            }
        }
        else
        {
            if(intval($res[0][1]) > 0)
            {
                //Аккаунт соц. сети привязан к основному аккаунту
                if (!isset($_SESSION['sbAuth']) || $_SESSION['sbAuth']->getUserId() == $res[0][1])
                {
                    //Авторизируем по основному аккаунту
                    $hash = md5(uniqid(time()));
                    sql_param_query('UPDATE sb_site_users SET su_auto_login=?s WHERE su_id=?d', $hash, $res[0][1]);
                    $this->updateAvatar($res[0][1]);
                    $_COOKIE['sb_site_auto_login'] = $hash;
                    sb_setcookie('sb_site_auto_login', $hash);
                    $auth = new sbAuth('', '', '');
                }
                else
                {
                    array_push($this->socNetObj->errors['user'], KERNEL_PROG_SITEUSERS_ACCOUNT_ISSET);
                }
            }
        }
        return $this;
    }
    
    public function getSystemErrors(){
        return $this->socNetObj->errors['system'];
    }
    
    public function getUserErrors(){
        return $this->socNetObj->errors['user'];
    }
    
    public function hasError(){
        $user_error = $this->getUserErrors();
        $system_error = $this->getSystemErrors();
        if(!empty($user_error) || !empty($system_error)){
            return true;
        }
        return false;
    }
    
    public function __get($name) {
        if(isset($this->socNetObj->$name)){
            return $this->socNetObj->$name;
        }
        return null;
    }
    
    private function createSiteUser()
    {
        $soc_login = $this->user_id.$this->netName;
        if (!isset($_SESSION['login_params']))
        {
            array_push($this->socNetObj->errors['user'], PL_SOCIAL_LOGIN_MESSAGES_REGISTRATE_ERROR);
            sb_add_system_message(sprintf(KERNEL_PROG_SITEUSERS_ADD_ERROR, $soc_login) . '(' . KERNEL_PROG_SITEUSERS_NO_LOGIN_PARAMS . ')', SB_MSG_WARNING);
            return false;
        }
        else
        {
            $params = stripslashes($_SESSION['login_params']);
            $params = unserialize($params);
            if (!isset($params['ids']))
            {
                sb_add_system_message(sprintf(KERNEL_PROG_SITEUSERS_ADD_ERROR, $soc_login) . '(' . KERNEL_PROG_SITEUSERS_NO_IDS . ')', SB_MSG_WARNING);
                array_push($this->socNetObj->errors['user'], PL_SOCIAL_LOGIN_MESSAGES_REGISTRATE_ERROR);
                return false;
            }
        }


        $group_ids = explode('^', $params['ids']);
        $res       = sql_param_query('SELECT cat_id, cat_fields FROM sb_categs WHERE cat_id IN (?a)', $group_ids);
        if (!$res)
        {
            array_push($this->socNetObj->errors['user'], PL_SOCIAL_LOGIN_MESSAGES_REGISTRATE_ERROR);
            sb_add_system_message(sprintf(KERNEL_PROG_SITEUSERS_ADD_ERROR, $soc_login) . '(' . KERNEL_PROG_SITEUSERS_WRONG_IDS . ')', SB_MSG_WARNING);
            return false;
        }
        else
        {
            $su_status  = 0;
            $su_domains = array();
            $group_ids = array();

            foreach ($res as $val)
            {
                list($cat_id, $cat_fields) = $val;

                $group_ids[] = $cat_id;
                if ($cat_fields != '')
                {
                    $cat_fields = unserialize($cat_fields);
                    if (isset($cat_fields['cat_status']) && $cat_fields['cat_status'] > $su_status)
                        $su_status  = intval($cat_fields['cat_status']);

                    if (isset($cat_fields['cat_domains']))
                    {
                        foreach ($cat_fields['cat_domains'] as $domain)
                        {
                            if (!in_array($domain, $su_domains))
                            {
                                $su_domains[] = $domain;
                            }
                        }
                    }
                }
            }

            $row['su_last_ip'] = sbAuth::getIP();


            $row['su_activation_code'] = '';
            $row['su_pass']            = '';
            $row['su_login']           = $soc_login;
            $row['su_name']            = $this->user_name;
            $row['su_reg_date']        = time();
            $row['su_last_date']       = time();
            $row['su_active_date']     = 0;
            $row['su_domains']         = implode(',', $su_domains);
            $row['su_status']          = 0;
            $row['su_auto_login']      = md5(uniqid(rand(500, 10000)));
            
            if(trim($this->user_foto) != '')
            {
                $row['su_pers_foto']   = $this->downloadAvatar();
            }

            $su_id = sbProgAddElement('sb_site_users', 'su_id', $row, $group_ids);

            if (!$su_id)
            {
                array_push($this->socNetObj->errors['user'], PL_SOCIAL_LOGIN_MESSAGES_REGISTRATE_ERROR);
                sb_add_system_message(sprintf(KERNEL_PROG_SITEUSERS_ADD_ERROR, $soc_login) . '(' . KERNEL_PROG_SITEUSERS_NO_SU_ID . ')', SB_MSG_WARNING);
                return false;
            }
            else
            {
                $_COOKIE['sb_site_auto_login'] = $row['su_auto_login'];
                sb_setcookie('sb_site_auto_login', $row['su_auto_login']);
                $data = array(
                    'sbsu_uid' => $su_id,
                    'sbsu_id' => $this->user_id,
                    'sbsu_sn_type' => $this->netName
                );
                
                sql_param_query('INSERT INTO sb_socnet_users SET ?a', $data);
                sb_add_system_message(sprintf(KERNEL_PROG_SITEUSERS_ADD_OK, $soc_login), SB_MSG_INFORMATION);
            }
        }
        return $soc_login;
    }
    
    private function updateAvatar($userId)
    {
        if($this->user_foto != '')
        {
            $res = sql_query('UPDATE sb_site_users SET su_pers_foto = ? WHERE su_id = ? AND (su_pers_foto IS NULL OR su_pers_foto = "")', $this->downloadAvatar(), $userId);
        }
    }
    
    private function downloadAvatar()
    {
        require_once SB_BASEDIR.'/cms/lib/sbDownload.inc.php';
        $foto = new sbDownload($this->user_foto);
        $mLocal =  $GLOBALS['sbVfs']->mLocal;
        $GLOBALS['sbVfs']->mLocal = true;
        if(!is_dir(SB_BASEDIR.'/upload/pl_site_users'))
        {
            $GLOBALS['sbVfs']->mkdir(SB_BASEDIR.'/upload/pl_site_users');
        }
        $GLOBALS['sbVfs']->mLocal = $mLocal;
        
        $foto = $foto->copyRemoteFile('/upload/pl_site_users');
        
        return $foto;
    }
}
?>