<?php
require_once SB_CMS_LIB_PATH . '/oauth/sbSocAuthInterface.php';

/**
 * Класс для авторизации пользователей в ВКонтакте 
 */
class sbVKontakte implements sbAuthInterface{
    /**
     * ID приложения в  соц. сети, выданный при регистрации приложения
     */
    private $app_id = null;
    
    /**
     * Ключ приложения 
     */
    private $app_key = null;
    
    
    /**
     * Адрес API домена соц. сети (с закрывающим слешем)
     */
    private $domainOAUTH = 'http://oauth.vk.com';
    private $domainOAUTH_SSL = 'https://oauth.vk.com';
    private $domainAPI = 'https://api.vk.com';
    
    /**
     * Адрес на который будет будет оправлен ответ от сервера соц. сети
     * @var string $redirectURL 
     */
    private $redirectURL;
    
    /**
     * Содержит экземпляр данного класса
     * @var sbOdnoklassniki $_Instance 
     */
    static $_Instance = null;
    
    /**
     * Свойства для возврата в sbSocAuth 
     */
    public $user_id = '';
    public $user_name = '';
    public $user_foto = '';
    public $errors = array(
        'system' => array(),
        'user' => array()
    );
    
    private function __construct(){
        $this->app_id = sbPlugins::getSetting('sb_social_login_vk_consumer_key');
        $this->app_key = sbPlugins::getSetting('sb_social_login_vk_secret_key');
        $this->redirectURL = SB_DOMAIN . '/cms/admin/scmedia.php?name=vk';
    }
    
    private function __clone(){
        
    }
    
    static function getInstance(){
        if(self::$_Instance === null){
            self::$_Instance = new sbVKontakte();
        }
        return self::$_Instance;
    }
    
    /**
     * Формирование и отправка запроса на авторизацию. 
     */
    public function sendAuthRequest(){
        if(!$this->app_id || !$this->app_key){
            unset($_SESSION['scmedia']);
            $this->addError(get_class($this).': '.PL_SOCIAL_LOGIN_NOSETTINGS);
            $this->addError(PL_SOCIAL_LOGIN_NOAUTH, false);
            return $this;
        }
        $url = "$this->domainOAUTH/authorize?client_id=$this->app_id"
        		.'&scope=offline'
                .'&response_type=code'
                .'&redirect_uri='.urlencode($this->redirectURL);
        header('Location: '.sb_sanitize_header($url));
    }
    
    public function getUserData(){
        if(isset($_GET['code'])){
            $url = $this->domainOAUTH_SSL . '/access_token?'.'code=' . $_GET['code'] . '&client_id=' . $this->app_id . '&client_secret=' . $this->app_key.'&redirect_uri='.urlencode($this->redirectURL);
            $request = new sbDownload($url);

            $auth = sb_json_decode($request->download());

            if(isset($auth->error) && !isset($auth->access_token)){
                $this->addError(get_class($this).': '.$auth->error_description);
                $this->addError(PL_SOCIAL_LOGIN_NOAUTH, false);
                unset($_SESSION['scmedia']);
                return $this;
            }elseif(empty($auth)){
                $this->addError(get_class($this).': '.SB_NOPARSE_RESPONSE_SERVER);
                $this->addError(PL_SOCIAL_LOGIN_NOAUTH, false);
                unset($_SESSION['scmedia']);
                return $this;
            }
            
            $url = $this->domainAPI . '/method/users.get?access_token=' . $auth->access_token . '&uid=' . $auth->user_id . '&fields=photo_medium';
            $request = new sbDownload($url);
            $user = sb_json_decode($request->download());

            if(isset($user->error)){
                unset($_SESSION['scmedia']);
                $this->addError(get_class($this).': '.$user->error->error_msg);
                $this->addError(PL_SOCIAL_LOGIN_NOAUTH, false);
                return $this;
            }

            if(isset($user->response)){ 
                $this->user_id = $user->response[0]->uid;
                $this->user_name = $user->response[0]->first_name . ' ' . $user->response[0]->last_name;
                $this->user_foto = $user->response[0]->photo_medium;
            }else{
                $this->addError(get_class($this).': '.SB_NOPARSE_RESPONSE_SERVER);
                $this->addError(PL_SOCIAL_LOGIN_NOAUTH, false);
            }
            return $this;
        }else{
            unset($_SESSION['scmedia']);
            if(isset($_GET['error']) && $_GET['error'] != 'access_denied'){
                $this->addError(get_class($this).': '.SB_NOPARSE_RESPONSE_SERVER);
                $this->addError(PL_SOCIAL_LOGIN_NOAUTH, false);
            }elseif(isset($_GET['error'])){
                $this->addError(PL_SOCIAL_LOGIN_WRONG_LOGIN, false);
            }else{
                $this->addError(get_class($this).': '.PL_SOCIAL_LOGIN_UNKNOW_ERROR);
            }
            return $this;
        }
    }
    
    private function addError($message, $system = true){
        if($system){
            array_push($this->errors['system'], $message);
        }else{
            array_push($this->errors['user'], $message);
        }
    }
}
?>