<?php
/**
 * Реализация классов, отвечающих за парсинг BB-кодов.
 *
 * @author Дмитрий Скоробогатов <dima@pc.uz>
 * @version 0.29
 * @package SB_SiteCore
 * @copyright Copyright (с) 2006-2007 Dmitriy Skorobogatov
 */

/**
 * Класс, отвечающий за парсинг BB-кодов.
 *
 * @author Дмитрий Скоробогатов <dima@pc.uz>
 * @version 0.29
 * @package SB_SiteCore
 * @copyright Copyright (с) 2006-2007 Dmitriy Skorobogatov
 */
class sbBBCode 
{
	/**
	 * Имя тега, которому сопоставлен экземпляр класса.
	 * Пустая строка, если экземпляр не сопоставлен никакому тегу.
	 *
	 * @var string
	 */
    public $tag = '';
    
    /**
     * Массив значений атрибутов тега, которому сопоставлен экземпляр класса.
     * Пуст, если экземпляр не сопоставлен никакому тегу.
     *
     * @var array
     */
    public $attrib = array();
    
    /**
     * Текст с BB-кодами
     *
     * @var string
     */
    public $text = '';
    
    /**
     * Результат синтаксического разбора текста с BB-кодами.
     *
     * @var array
     */
    public $syntax = array();
    
    /**
     * Дерево семантического разбора текста с BB-кодами.
     *
     * @var array
     */
    public $tree = array();
    
    /**
     * Список поддерживаемых тегов с указанием специализированных классов.
     *
     * @var array
     */
    public $tags = array(
	    '*'            => 'sbBBCodeTags_Li'     ,
	    'li'           => 'sbBBCodeTags_Li'     ,
        'ol'           => 'sbBBCodeTags_List'   ,
        'ul'           => 'sbBBCodeTags_List'   ,
    	'list'         => 'sbBBCodeTags_List'   ,
    	'b'            => 'sbBBCodeTags_Simple' ,
    	'i'            => 'sbBBCodeTags_Simple' ,
        'u'            => 'sbBBCodeTags_Simple' ,
    	's'            => 'sbBBCodeTags_Simple' ,
    	'big'          => 'sbBBCodeTags_Simple' ,
    	'small'        => 'sbBBCodeTags_Simple' ,
	    'strike'       => 'sbBBCodeTags_Simple' ,
	    'strong'       => 'sbBBCodeTags_Simple' ,
	    'sub'          => 'sbBBCodeTags_Simple' ,
	    'sup'          => 'sbBBCodeTags_Simple' ,
    	'cite'         => 'sbBBCodeTags_Simple' ,
    	'del'          => 'sbBBCodeTags_Simple' ,
	    'em'           => 'sbBBCodeTags_Simple' ,
    	'ins'          => 'sbBBCodeTags_Simple' ,
    	'tt'           => 'sbBBCodeTags_Simple' ,
    	'var'          => 'sbBBCodeTags_Simple' ,
    	'code'         => 'sbBBCodeTags_Simple' ,
    	'pre'          => 'sbBBCodeTags_Simple' ,
    	'address'      => 'sbBBCodeTags_Simple',
    	'hr'           => 'sbBBCodeTags_Hr'     ,
    	'br'           => 'sbBBCodeTags_Br'     ,
    	'a'            => 'sbBBCodeTags_A'      ,
		'anchor'       => 'sbBBCodeTags_A'      ,
    	'url'          => 'sbBBCodeTags_A'		,
    	'email'        => 'sbBBCodeTags_Email'  ,
    	'align'        => 'sbBBCodeTags_Align'  ,
    	'center'       => 'sbBBCodeTags_Align'  ,
    	'justify'      => 'sbBBCodeTags_Align'  ,
	    'left'         => 'sbBBCodeTags_Align'  ,
	    'right'        => 'sbBBCodeTags_Align'  ,
    	'img'          => 'sbBBCodeTags_Img'    ,
    	'table'        => 'sbBBCodeTags_Table'  ,
	    'td'           => 'sbBBCodeTags_Td'     ,
	    'th'           => 'sbBBCodeTags_Td'     ,
	    'tr'           => 'sbBBCodeTags_Tr'     ,
    	'caption'      => 'sbBBCodeTags_Caption',
	    'abbr'         => 'sbBBCodeTags_Abbr'   ,
	    'acronym'      => 'sbBBCodeTags_Acronym',
	    'p'            => 'sbBBCodeTags_P'      ,
    	'color'        => 'sbBBCodeTags_Color'  ,
	    'quote'        => 'sbBBCodeTags_Quote'  
	     );
    
    /**
     * Смайлики и прочие мнемоники. Массив: 'мнемоника' => 'ее_замена'.
     *
     * @var array
     */
    public $mnemonics = array();
    
    /**
     * Модель поведения тега (в плане вложенности), которому сопоставлен экземпляр
	 * данного класса. Модели поведения могут быть следующими:
	 *   'a'       - ссылки, якоря
	 *   'caption' - заголовки таблиц
	 *   'code'    - линейные контейнеры кода
	 *   'div'     - блочные элементы
	 *   'hr'      - горизонталные линии
	 *   'img'     - картинки
	 *   'li'      - элементы списков
	 *   'p'       - блочные элементы типа абзацев и заголовков
	 *   'pre'     - блочные контейнеры кода
	 *   'span'    - линейные элементы
	 *   'table'   - таблицы
	 *   'td'      - ячейки таблиц
	 *   'tr'      - строки таблиц
	 *   'ul'      - списки
	 * 
	 * Конкретное содержание в понятие "модель поведения" вкладывается приватными переменными класса $_ends, $_children.
     *
     * @var string
     */
    public $behaviour = 'div';
    
    /**
     * Вспомогательные переменные
     */
    public $is_close = false;
    public $lbr = 0;
    public $rbr = 0;
    
    /**
     * Позиция очередного обрабатываемого парсером символа.
     *
     * @var int
     */
    private $_cursor;
    
    /**
     * Массив объектов-представителей отдельных тегов.
     *
     * @var array
     */
    private $_tag_objects = array();
    
    /**
     * Массив пар: 'модель_поведения_тегов' => массив_моделей_поведений_тегов.
     * Накладывает ограничения на вложенность тегов. Теги с моделями поведения, не
     * указанными в массиве справа, вложенные в тег с моделью поведения, указанной
     * слева, будут игнорироваться как неправильно вложенные.
     *
     * @var array
     */
    private $_children = array(
	    'a'       => array('code','img','span'),
	    'caption' => array('a','code','img','span'),
	    'code'    => array(),
	    'div'     => array('a','code','div','hr','img','p','pre','span','table','ul'),
	    'hr'      => array(),
	    'img'     => array(),
	    'li'      => array('a','code','div','hr','img','p','pre','span','table','ul'),
	    'p'       => array('a','code','img','span'),
	    'pre'     => array(),
	    'span'    => array('a','code','img','span'),
	    'table'   => array('caption','tr'),
	    'td'      => array('a','code','div','hr','img','p','pre','span','table','ul'),
	    'tr'      => array('td'),
	    'ul'      => array('li'));
    
    /**
     * Массив пар: 'модель_поведения_тегов' => массив_моделей_поведений_тегов.
     * Накладывает ограничения на вложенность тегов.
     * Тег, принадлежещий указанной слева модели поведения тегов должен закрыться,
     * как только начинается тег, принадлежещий какой то из моделей поведения,
     * указанных справа.
     *
     * @var array
     */
    private $_ends = array(
	    'a'       => array(
	        'a','caption','div','hr','li','p','pre','table','td','tr', 'ul'
	    ),
	    'caption' => array('tr'),
	    'code'    => array(),
	    'div'     => array('li','tr','td'),
	    'hr'      => array(
	        'a','caption','code','div','hr','img','li','p','pre','span','table',
	        'td','tr','ul'
	    ),
	    'img'     => array(
	        'a','caption','code','div','hr','img','li','p','pre','span','table',
	        'td','tr','ul'
	    ),
	    'li'      => array('li','tr','td'),
	    'p'       => array('div','hr','li','p','pre','table','td','tr','ul'),
	    'pre'     => array(),
	    'span'    => array('div','hr','li','p','pre','table','td','tr','ul'),
	    'table'   => array('table'),
	    'td'      => array('td','tr'),
	    'tr'      => array('tr'),
	    'ul'      => array());

    /**
	 * Конструктор класса
	 *
	 * @param string $code Текст с BB-кодами.
	 */
    public function __construct($text = '') 
    {
        // Формируем набор смайликов
        $GLOBALS['sbVfs']->mLocal = true;
		if ($GLOBALS['sbVfs']->exists(SB_CMS_IMG_PATH.'/smiles/smiles.pak') && $GLOBALS['sbVfs']->is_file(SB_CMS_IMG_PATH.'/smiles/smiles.pak'))
		{
			$pak = $GLOBALS['sbVfs']->file(SB_CMS_IMG_PATH.'/smiles/smiles.pak');
			if ($pak)
			{
				foreach ($pak as $val) 
				{
    				$val = trim($val);
    				if (! $val || '#' == substr($val, 0, 1)) 
    					continue;

    				$val = explode('=+:', $val);
    				if (count($val) != 3)
    					continue;
    					
    				list($gif, $alt, $symbol) = $val; 
    				$this->mnemonics[$symbol] = '<img src="'.SB_CMS_IMG_URL.'/smiles/'.sb_htmlspecialchars($gif).'" alt="'.$alt.'" border="0" />';
				}
			}
		}
		$GLOBALS['sbVfs']->mLocal = false;      

        $this->parse($text);
    }

    /**
     * Функция возвращает очередную пару "число (тип лексемы) - лексема".
     * Лексема - подстрока строки $this -> text, начинающаяся с позиции $this -> _cursor.
     * 
     * Типы лексем могут быть следующие:
     *   0 - открывющая квадратная скобка ("[")
     *   1 - закрывающая квадратная cкобка ("]")
     *   2 - двойная кавычка ('"')
     *   3 - апостроф ("'")
     *   4 - равенство ("=")
     *   5 - прямой слэш ("/")
     *   6 - последовательность пробельных символов
     *   (" ", "\t", "\n", "\r", "\0" или "\x0B")
     *   7 - последовательность прочих символов, не являющаяся именем тега
     *   8 - имя тега
     */
    private function get_token() 
    {
        $token = '';
        $token_type = false;
        $char_type = false;
        while (true) 
        {
            $token_type = $char_type;
            if (!isset($this->text[ $this->_cursor ])) 
            {
                if (false === $char_type) 
                {
                    return false;
                } 
                else 
                {
                    break;
                }
            }
            
            $char = $this->text[ $this->_cursor ];
            switch ($char) 
            {
                case '[':
                    $char_type = 0;
                    break;
                case ']':
                    $char_type = 1;
                    break;
                case '"':
                    $char_type = 2;
                    break;
                case "'":
                    $char_type = 3;
                    break;
                case '=':
                    $char_type = 4;
                    break;
                case '/':
                    $char_type = 5;
                    break;
                case ' ':
                    $char_type = 6;
                    break;
                case "\t":
                    $char_type = 6;
                    break;
                case "\n":
                    $char_type = 6;
                    break;
                case "\r":
                    $char_type = 6;
                    break;
                case "\0":
                    $char_type = 6;
                    break;
                case "\x0B":
                    $char_type = 6;
                    break;
                default:
                    $char_type = 7;
                    break;
            }
            
            if (false === $token_type) 
            {
                $token = $char;
            } 
            elseif (5 >= $token_type) 
            {
                break;
            } 
            elseif ($char_type == $token_type) 
            {
                $token .= $char;
            } 
            else 
            {
                break;
            }
            
            $this->_cursor++;
        }
        
        if (isset($this->tags[ strtolower($token) ])) 
        {
            $token_type = 8;
        }
        
        return array($token_type, $token);
    }

    /**
     * Функция парсинга текста с BB-кодами.
     *
     * @param mixed $code Текст или массив строк с BB-кодами.
     * 
     * @return mixed Массив со строками с распарсенными BB-кодами.
     */
    private function parse($code = '') 
    {
        if (is_array($code)) 
        {
            $is_tree = false;
            foreach ($code as $val) 
            {
                if (isset($val['val'])) 
                {
                	$this->tree = $code;
                	$this->syntax = $this->get_syntax();
                	$is_tree = true;
                	break;
                }
            }
            if (!$is_tree) 
            {
                $this->syntax = $code;
                $this->get_tree();
            }
            
            $this->text = '';
            foreach ($this->syntax as $val) 
            {
                $this->text .= $val['str'];
            }

            return $this->syntax;
        } 
        elseif ($code) 
        {
        	$this->text = $code;
        }
        
        /*
        Используем метод конечных автоматов
        Список возможных состояний автомата:
        0  - Начало сканирования или находимся вне тега. Ожидаем что угодно.
        1  - Встретили символ "[", который считаем началом тега. Ожидаем имя
             тега, или символ "/".
        2  - Нашли в теге неожидавшийся символ "[". Считаем предыдущую строку
             ошибкой. Ожидаем имя тега, или символ "/".
        3  - Нашли в теге синтаксическую ошибку. Текущий символ не является "[".
             Ожидаем что угодно.
        4  - Сразу после "[" нашли символ "/". Предполагаем, что попали в
             закрывающий тег. Ожидаем имя тега или символ "]".
        5  - Сразу после "[" нашли имя тега. Считаем, что находимся в
             открывающем теге. Ожидаем пробел или "=" или "/" или "]".
        6  - Нашли завершение тега "]". Ожидаем что угодно.
        7  - Сразу после "[/" нашли имя тега. Ожидаем "]".
        8  - В открывающем теге нашли "=". Ожидаем пробел или значение атрибута.
        9  - В открывающем теге нашли "/", означающий закрытие тега. Ожидаем
             "]".
        10 - В открывающем теге нашли пробел после имени тега или имени
             атрибута. Ожидаем "=" или имя другого атрибута или "/" или "]".
        11 - Нашли '"' начинающую значение атрибута, ограниченное кавычками.
             Ожидаем что угодно.
        12 - Нашли "'" начинающий значение атрибута, ограниченное апострофами.
             Ожидаем что угодно.
        13 - Нашли начало незакавыченного значения атрибута. Ожидаем что угодно.
        14 - В открывающем теге после "=" нашли пробел. Ожидаем значение
             атрибута.
        15 - Нашли имя атрибута. Ожидаем пробел или "=" или "/" или "]".
        16 - Находимся внутри значения атрибута, ограниченного кавычками.
             Ожидаем что угодно.
        17 - Завершение значения атрибута. Ожидаем пробел или имя следующего
             атрибута или "/" или "]".
        18 - Находимся внутри значения атрибута, ограниченного апострофами.
             Ожидаем что угодно.
        19 - Находимся внутри незакавыченного значения атрибута. Ожидаем что
             угодно.
        20 - Нашли пробел после значения атрибута. Ожидаем имя следующего
             атрибута или "/" или "]".

        Описание конечного автомата:
        */
        
        $finite_automaton = array(
               // Предыдущие |   Состояния для текущих событий (лексем)   |
               //  состояния |  0 |  1 |  2 |  3 |  4 |  5 |  6 |  7 |  8 |
                   0 => array(  1 ,  0 ,  0 ,  0 ,  0 ,  0 ,  0 ,  0 ,  0 )
                ,  1 => array(  2 ,  3 ,  3 ,  3 ,  3 ,  4 ,  3 ,  3 ,  5 )
                ,  2 => array(  2 ,  3 ,  3 ,  3 ,  3 ,  4 ,  3 ,  3 ,  5 )
                ,  3 => array(  1 ,  0 ,  0 ,  0 ,  0 ,  0 ,  0 ,  0 ,  0 )
                ,  4 => array(  2 ,  6 ,  3 ,  3 ,  3 ,  3 ,  3 ,  3 ,  7 )
                ,  5 => array(  2 ,  6 ,  3 ,  3 ,  8 ,  9 , 10 ,  3 ,  3 )
                ,  6 => array(  1 ,  0 ,  0 ,  0 ,  0 ,  0 ,  0 ,  0 ,  0 )
                ,  7 => array(  2 ,  6 ,  3 ,  3 ,  3 ,  3 ,  3 ,  3 ,  3 )
                ,  8 => array( 13 , 13 , 11 , 12 , 13 , 13 , 14 , 13 , 13 )
                ,  9 => array(  2 ,  6 ,  3 ,  3 ,  3 ,  3 ,  3 ,  3 ,  3 )
                , 10 => array(  2 ,  6 ,  3 ,  3 ,  8 ,  9 ,  3 , 15 , 15 )
                , 11 => array( 16 , 16 , 17 , 16 , 16 , 16 , 16 , 16 , 16 )
                , 12 => array( 18 , 18 , 18 , 17 , 18 , 18 , 18 , 18 , 18 )
                , 13 => array( 19 ,  6 , 19 , 19 , 19 , 19 , 17 , 19 , 19 )
                , 14 => array(  2 ,  3 , 11 , 12 , 13 , 13 ,  3 , 13 , 13 )
                , 15 => array(  2 ,  6 ,  3 ,  3 ,  8 ,  9 , 10 ,  3 ,  3 )
                , 16 => array( 16 , 16 , 17 , 16 , 16 , 16 , 16 , 16 , 16 )
                , 17 => array(  2 ,  6 ,  3 ,  3 ,  3 ,  9 , 20 , 15 , 15 )
                , 18 => array( 18 , 18 , 18 , 17 , 18 , 18 , 18 , 18 , 18 )
                , 19 => array( 19 ,  6 , 19 , 19 , 19 , 19 , 20 , 19 , 19 )
                , 20 => array(  2 ,  6 ,  3 ,  3 ,  3 ,  9 ,  3 , 15 , 15 )
            );
        // Закончили описание конечного автомата
        
        $mode = 0;
        $this->syntax = array();
        $decomposition = array();
        $token_key = -1;
        $value = '';
        $this->_cursor = 0;
        
        // Сканируем массив лексем с помощью построенного автомата:
        while (($token = $this->get_token()) != false) 
        {
            $previous_mode = $mode;
            $mode = $finite_automaton[$previous_mode][$token[0]];
            if (-1 < $token_key) 
            {
                $type = $this->syntax[$token_key]['type'];
            } 
            else 
            {
                $type = false;
            }
            
            switch ($mode) 
            {
	            case 0:
	                if ('text' == $type) 
	                {
	                    $this->syntax[$token_key]['str'] .= $token[1];
	                } 
	                else 
	                {
	                    $this->syntax[++$token_key] = array(
	                            'type' => 'text',
	                            'str' => $token[1]
	                        );
	                }
	                break;
	                
	            case 1:
	                $decomposition = array(
	                    'name'   => '',
	                    'type'   => '',
	                    'str'    => '[',
	                    'layout' => array(array(0, '['))
	                );
	                break;
	                
	            case 2:
	                if ('text' == $type) 
	                {
	                    $this->syntax[$token_key]['str'] .= $decomposition['str'];
	                } 
	                else 
	                {
	                    $this->syntax[++$token_key] = array(
	                        'type' => 'text',
	                        'str' => $decomposition['str']
	                    );
	                }
	                $decomposition = array(
	                    'name'   => '',
	                    'type'   => '',
	                    'str'    => '[',
	                    'layout' => array(array(0, '['))
	                );
	                break;
	                
	            case 3:
	                if ('text' == $type) 
	                {
	                    $this->syntax[$token_key]['str'] .= $decomposition['str'];
	                    $this->syntax[$token_key]['str'] .= $token[1];
	                } else 
	                {
	                    $this->syntax[++$token_key] = array(
	                        'type' => 'text',
	                        'str' => $decomposition['str'].$token[1]
	                    );
	                }
	                $decomposition = array();
	                break;
	                
	            case 4:
	                $decomposition['type'] = 'close';
	                $decomposition['str'] .= '/';
	                $decomposition['layout'][] = array(1, '/');
	                break;
	                
	            case 5:
	                $decomposition['type'] = 'open';
	                $name = sb_strtolower($token[1]);
	                $decomposition['name'] = $name;
	                $decomposition['str'] .= $token[1];
	                $decomposition['layout'][] = array(2, $token[1]);
	                $decomposition['attrib'][$name] = '';
	                break;
	                
	            case 6:
	                if (! isset($decomposition['name'])) 
	                {
	                    $decomposition['name'] = '';
	                }
	                if (13 == $previous_mode || 19 == $previous_mode) 
	                {
	                    $decomposition['layout'][] = array(7, $value);
	                }
	                $decomposition['str'] .= ']';
	                $decomposition['layout'][] = array( 0, ']' );
	                $this->syntax[++$token_key] = $decomposition;
	                $decomposition = array();
	                break;
	                
	            case 7:
	                $decomposition['name'] = sb_strtolower($token[1]);
	                $decomposition['str'] .= $token[1];
	                $decomposition['layout'][] = array(2, $token[1]);
	                break;
	                
	            case 8:
	                $decomposition['str'] .= '=';
	                $decomposition['layout'][] = array(3, '=');
	                break;
	                
	            case 9:
	                $decomposition['type'] = 'open/close';
	                $decomposition['str'] .= '/';
	                $decomposition['layout'][] = array(1, '/');
	                break;
	                
	            case 10:
	                $decomposition['str'] .= $token[1];
	                $decomposition['layout'][] = array(4, $token[1]);
	                break;
	                
	            case 11:
	                $decomposition['str'] .= '"';
	                $decomposition['layout'][] = array(5, '"');
	                $value = '';
	                break;
	                
	            case 12:
	                $decomposition['str'] .= "'";
	                $decomposition['layout'][] = array(5, "'");
	                $value = '';
	                break;
	                
	            case 13:
	                $decomposition['attrib'][$name] = $token[1];
	                $value = $token[1];
	                $decomposition['str'] .= $token[1];
	                break;
	                
	            case 14:
	                $decomposition['str'] .= $token[1];
	                $decomposition['layout'][] = array(4, $token[1]);
	                break;
	                
	            case 15:
	                $name = sb_strtolower($token[1]);
	                $decomposition['str'] .= $token[1];
	                $decomposition['layout'][] = array(6, $token[1]);
	                $decomposition['attrib'][$name] = '';
	                break;
	                
	            case 16:
	                $decomposition['str'] .= $token[1];
	                $decomposition['attrib'][$name] .= $token[1];
	                $value .= $token[1];
	                break;
	                
	            case 17:
	                $decomposition['str'] .= $token[1];
	                $decomposition['layout'][] = array(7, $value);
	                $value = '';
	                $decomposition['layout'][] = array(5, $token[1]);
	                break;
	                
	            case 18:
	                $decomposition['str'] .= $token[1];
	                $decomposition['attrib'][$name] .= $token[1];
	                $value .= $token[1];
	                break;
	                
	            case 19:
	                $decomposition['str'] .= $token[1];
	                $decomposition['attrib'][$name] .= $token[1];
	                $value .= $token[1];
	                break;
	                
	            case 20:
	                $decomposition['str'] .= $token[1];
	                if ( 13 == $previous_mode || 19 == $previous_mode ) 
	                {
	                    $decomposition['layout'][] = array(7, $value);
	                }
	                $value = '';
	                $decomposition['layout'][] = array(4, $token[1]);
	                break;
            }
        }
        
        if (count($decomposition)) 
        {
            if ('text' == $type) 
            {
                $this->syntax[$token_key]['str'] .= $decomposition['str'];
            } 
            else 
            {
                $this->syntax[++$token_key] = array(
                    'type' => 'text',
                    'str' => $decomposition['str']
                );
            }
        }
        
        $this->get_tree();

        return $this->syntax;
    }

    public function specialchars($string) 
    {
        return str_replace(array('[', ']', '"', "'", '@'), array('@l;', '@r;', '@q;', '@a;', '@at;'), $string);
    }

    public function unspecialchars($string) 
    {
    	return str_replace(array('@l;', '@r;', '@q;', '@a;', '@at;'), array('[', ']', '"', "'", '@'), $string);
    }

    /**
     * Функция проверяет, должен ли тег с именем $current закрыться, если
     * начинается тег с именем $next.
     *
     * @param string $current Название тега.
     * @param string $next Название тега.
     * 
     * @return bool TRUE, если тег должен быть закрыт, FALSE иначе.
     */
    private function must_close_tag($current, $next) 
    {
    	$current = (string) $current;
        $next = (string) $next;
        
        if (isset($this->tags[$current])) 
        {
            $this->init_tag_class($current);
            $class_vars = get_class_vars($this->tags[$current]);
            $current_behaviour = $class_vars['behaviour'];
        } 
        else 
        {
            $current_behaviour = $this->behaviour;
        }
        
        if (isset($this->tags[$next])) 
        {
            $this->init_tag_class($next);
            $class_vars = get_class_vars($this->tags[$next]);
            $next_behaviour = $class_vars['behaviour'];
        } 
        else 
        {
            $next_behaviour = $this->behaviour;
        }
        
        $must_close = false;
        if (isset($this->_ends[$current_behaviour])) 
        {
            $must_close = in_array($next_behaviour, $this->_ends[$current_behaviour]);;
        }
        
        return $must_close;
    }

    /**
     * Возвращает true, если тег с именем $parent может иметь непосредственным
     * потомком тег с именем $child. В противном случае - false.
     * Если $parent - пустая строка, то проверяется, разрешено ли $child входить в
     * корень дерева BBCode.
     * 
     * @param string $parent Название тега.
     * @param string $child Название тега.
     * 
     * @return bool TRUE, может иметь потомком, FALSE в ином случае.
     */
    private function may_have_child($parent, $child) 
    {
        $parent = (string) $parent;
        $child = (string) $child;
        
        if (isset($this->tags[$parent])) 
        {
            $this->init_tag_class($parent);
            $class_vars = get_class_vars($this -> tags[$parent]);
            $parent_behaviour = $class_vars['behaviour'];
        } 
        else 
        {
            $parent_behaviour = $this->behaviour;
        }
        
        if (isset($this->tags[$child])) 
        {
            $this->init_tag_class($child);
            $class_vars = get_class_vars($this -> tags[$child]);
            $child_behaviour = $class_vars['behaviour'];
        } 
        else 
        {
            $child_behaviour = $this->behaviour;
        }
        
        $permissibly = true;
        if (isset($this->_children[$parent_behaviour])) 
        {
            $permissibly = in_array($child_behaviour, $this->_children[$parent_behaviour]);
        }
        
        return $permissibly;
    }

    private function normalize_bracket($syntax) 
    {
        $structure = array();
        $structure_key = -1;
        $level = 0;
        $open_tags = array();
        
        foreach ($syntax as $val) 
        {
            unset($val['layout']);
            
            switch ($val['type']) 
            {
                case 'text':
                    $val['str'] = $this->unspecialchars($val['str']);
                    $type = (-1 < $structure_key) ? $structure[$structure_key]['type'] : false;
                    if ('text' == $type) 
                    {
                        $structure[$structure_key]['str'] .= $val['str'];
                    } 
                    else 
                    {
                        $structure[++$structure_key] = $val;
                        $structure[$structure_key]['level'] = $level;
                    }
                    break;
                    
                case 'open/close':
                    $val['attrib'] = array_map(array(&$this, 'unspecialchars'), $val['attrib']);
                    
                    foreach (array_reverse($open_tags, true) as $ult_key => $ultimate) 
                    {
                        if ($this->must_close_tag($ultimate, $val['name'])) 
                        {
                            $structure[++$structure_key] = array(
                                    'type'  => 'close',
                                    'name'  => $ultimate,
                                    'str'   => '',
                                    'level' => --$level
                                );
                            unset($open_tags[$ult_key]);
                        } 
                        else 
                        {
                        	break;
                        }
                    }
                    $structure[++$structure_key] = $val;
                    $structure[$structure_key]['level'] = $level;
                    break;
                    
                case 'open':
                    $this->init_tag_class($val['name']);
                    $val['attrib'] = array_map(array(&$this, 'unspecialchars'), $val['attrib']);
                    
                    foreach (array_reverse($open_tags, true) as $ult_key => $ultimate) 
                    {
                        if ($this->must_close_tag($ultimate, $val['name'])) 
                        {
                            $structure[++$structure_key] = array(
                                    'type'  => 'close',
                                    'name'  => $ultimate,
                                    'str'   => '',
                                    'level' => --$level
                                );
                            unset($open_tags[$ult_key]);
                        } 
                        else 
                        { 
                        	break; 
                        }
                    }
                    
                    $class_vars = get_class_vars($this->tags[$val['name']]);
                    if ($class_vars['is_close']) 
                    {
                        $val['type'] = 'open/close';
                        $structure[++$structure_key] = $val;
                        $structure[$structure_key]['level'] = $level;
                    } 
                    else 
                    {
                        $structure[++$structure_key] = $val;
                        $structure[$structure_key]['level'] = $level++;
                        $open_tags[] = $val['name'];
                    }
                    break;
                    
                case 'close':
                    if (!count($open_tags)) 
                    {
                        $type = (-1 < $structure_key) ? $structure[$structure_key]['type'] : false;
                        if ( 'text' == $type ) 
                        {
                            $structure[$structure_key]['str'] .= $val['str'];
                        } 
                        else 
                        {
                            $structure[++$structure_key] = array(
                                    'type'  => 'text',
                                    'str'   => $val['str'],
                                    'level' => 0
                                );
                        }
                        break;
                    }
                    
                    if (! $val['name']) 
                    {
                        end($open_tags);
                        list($ult_key, $ultimate) = each($open_tags);
                        $val['name'] = $ultimate;
                        $structure[++$structure_key] = $val;
                        $structure[$structure_key]['level'] = --$level;
                        unset($open_tags[$ult_key]);
                        break;
                    }
                    
                    if (!in_array($val['name'],$open_tags)) 
                    {
                        $type = (-1 < $structure_key) ? $structure[$structure_key]['type'] : false;
                        if ('text' == $type) 
                        {
                            $structure[$structure_key]['str'] .= $val['str'];
                        } 
                        else 
                        {
                            $structure[++$structure_key] = array(
                                    'type'  => 'text',
                                    'str'   => $val['str'],
                                    'level' => $level
                                );
                        }
                        break;
                    }
                    
                    foreach (array_reverse($open_tags,true) as $ult_key => $ultimate) 
                    {
                        if ($ultimate != $val['name']) 
                        {
                            $structure[++$structure_key] = array(
                                    'type'  => 'close',
                                    'name'  => $ultimate,
                                    'str'   => '',
                                    'level' => --$level
                                );
                            unset($open_tags[$ult_key]);
                        } 
                        else 
                        {
                        	break;
                        }
                    }
                    
                    $structure[++$structure_key] = $val;
                    $structure[$structure_key]['level'] = --$level;
                    unset($open_tags[$ult_key]);
                    break;
            }
        }
        
        foreach (array_reverse($open_tags,true) as $ult_key => $ultimate) 
        {
            $structure[++$structure_key] = array(
                    'type'  => 'close',
                    'name'  => $ultimate,
                    'str'   => '',
                    'level' => --$level
                );
            unset($open_tags[$ult_key]);
        }
        
        return $structure;
    }

    private function get_tree() 
    {
        // Превращаем $this -> syntax в правильную скобочную структуру
        $structure = $this->normalize_bracket($this->syntax);
        
        /* Отслеживаем, имеют ли элементы неразрешенные подэлементы.
           Соответственно этому исправляем $structure. */
        $normalized = array();
        $normal_key = -1;
        $level = 0;
        $open_tags = array();
        $not_tags = array();

        foreach ($structure as $val) 
        {
            switch ($val['type']) 
            {
                case 'text':
                    $type = (-1 < $normal_key) ? $normalized[$normal_key]['type'] : false;
                    if ('text' == $type) 
                    {
                        $normalized[$normal_key]['str'] .= $val['str'];
                    } 
                    else 
                    {
                        $normalized[++$normal_key] = $val;
                        $normalized[$normal_key]['level'] = $level;
                    }
                    break;
                    
                case 'open/close':
                    $this->init_tag_class($val['name']);
                    $is_open = count($open_tags);
                    end($open_tags);
                    $parent = $is_open ? current($open_tags) : $this->tag;
                    $permissibly = $this->may_have_child($parent, $val['name']);
                    if (! $permissibly) 
                    {
                        $type = (-1 < $normal_key) ? $normalized[$normal_key]['type'] : false;
                        if ( 'text' == $type ) 
                        {
                            $normalized[$normal_key]['str'] .= $val['str'];
                        } 
                        else 
                        {
                            $normalized[++$normal_key] = array(
                                    'type'  => 'text',
                                    'str'   => $val['str'],
                                    'level' => $level
                                );
                        }
                        break;
                    }
                    $normalized[++$normal_key] = $val;
                    $normalized[$normal_key]['level'] = $level;
                    break;
                    
                case 'open':
                    $this->init_tag_class($val['name']);
                    $is_open = count($open_tags);
                    end($open_tags);
                    $parent = $is_open ? current($open_tags) : $this->tag;
                    $permissibly = $this->may_have_child($parent, $val['name']);
                    if (!$permissibly) 
                    {
                        $not_tags[$val['level']] = $val['name'];
                        $type = (-1 < $normal_key) ? $normalized[$normal_key]['type'] : false;
                        if ( 'text' == $type ) 
                        {
                            $normalized[$normal_key]['str'] .= $val['str'];
                        } 
                        else 
                        {
                            $normalized[++$normal_key] = array(
                                    'type'  => 'text',
                                    'str'   => $val['str'],
                                    'level' => $level
                                );
                        }
                        break;
                    }
                    $normalized[++$normal_key] = $val;
                    $normalized[$normal_key]['level'] = $level++;
                    $ult_key = count($open_tags);
                    $open_tags[$ult_key] = $val['name'];
                    break;
                    
                case 'close':
                    $not_normal = isset($not_tags[$val['level']]) && $not_tags[$val['level']] = $val['name'];
                    if ($not_normal) 
                    {
                        unset($not_tags[$val['level']]);
                        $type = (-1 < $normal_key) ? $normalized[$normal_key]['type'] : false;
                        if ( 'text' == $type ) 
                        {
                            $normalized[$normal_key]['str'] .= $val['str'];
                        } 
                        else 
                        {
                            $normalized[++$normal_key] = array(
                                    'type'  => 'text',
                                    'str'   => $val['str'],
                                    'level' => $level
                                );
                        }
                        break;
                    }
                    $normalized[++$normal_key] = $val;
                    $normalized[$normal_key]['level'] = --$level;
                    $ult_key = count($open_tags) - 1;
                    unset($open_tags[$ult_key]);
                    break;
            }
        }
        unset($structure);
        
        // Формируем дерево элементов
        $result = array();
        $result_key = -1;
        $open_tags = array();

        foreach ($normalized as $normal_key => $val) 
        {
            switch ($val['type']) 
            {
                case 'text':
                    if (!$val['level']) 
                    {
                        $result[++$result_key] = array(
                            'type' => 'text',
                            'str' => $val['str']
                        );
                        break;
                    }
                    $open_tags[$val['level']-1]['val'][] = array(
                            'type' => 'text',
                            'str' => $val['str']
                        );
                    break;
                    
                case 'open/close':
                    if (!$val['level']) 
                    {
                        $result[++$result_key] = array(
                            'type'   => 'item',
                            'name'   => $val['name'],
                            'attrib' => $val['attrib'],
                            'val'    => array()
                        );
                        break;
                    }
                    $open_tags[$val['level']-1]['val'][] = array(
                        'type'   => 'item',
                        'name'   => $val['name'],
                        'attrib' => $val['attrib'],
                        'val'    => array()
                    );
                    break;
                    
                case 'open':
                    $open_tags[$val['level']] = array(
                        'type'   => 'item',
                        'name'   => $val['name'],
                        'attrib' => $val['attrib'],
                        'val'    => array()
                    );
                    break;
                    
                case 'close':
                    if (!$val['level']) 
                    {
                        $result[++$result_key] = $open_tags[0];
                        unset($open_tags[0]);
                        break;
                    }
                    $open_tags[$val['level']-1]['val'][] = $open_tags[$val['level']];
                    unset($open_tags[$val['level']]);
                    break;
            }
        }
        $this->tree = $result;
        
        return $result;
    }

    private function get_syntax($tree = false) 
    {
        if (!is_array($tree)) 
        {
            $tree = $this->tree;
        }
        
        $syntax = array();
        foreach ($tree as $elem) 
        {
            if ('text' == $elem['type']) 
            {
            	$syntax[] = array(
            	    'type' => 'text',
            	    'str' => $this->specialchars($elem['str'])
            	);
            } 
            else 
            {
                $sub_elems = $this->get_syntax($elem['val']);
                $str = '';
                $layout = array(array(0, '['));
                foreach ($elem['attrib'] as $name => $val) 
                {
                    $val = $this -> specialchars($val);
                    if ($str) 
                    {
                    	$str .= ' ';
                    	$layout[] = array(4, ' ');
                    	$layout[] = array(6, $name);
                    } 
                    else 
                    {
                        $layout[] = array(2, $name);
                    }
                    
                    $str .= $name;
                    if ($val) 
                    {
                    	$str .= '="'.$val.'"';
                    	$layout[] = array(3, '=');
                    	$layout[] = array(5, '"');
                    	$layout[] = array(7, $val);
                    	$layout[] = array(5, '"');
                    }
                }
                
                if (count($sub_elems)) 
                {
                	$str = '['.$str.']';
                } 
                else 
                {
                    $str = '['.$str.' /]';
                    $layout[] = array(4, ' ');
                    $layout[] = array(1, '/');
                }
                
                $layout[] = array(0, ']');
                $syntax[] = array(
                    'type' => count($sub_elems) ? 'open' : 'open/close',
                    'str' => $str,
                    'name' => $elem['name'],
                    'attrib' => $elem['attrib'],
                    'layout' => $layout
                );
                
                foreach ($sub_elems as $sub_elem) 
                { 
                	$syntax[] = $sub_elem; 
                }
                
                if (count($sub_elems)) 
                {
                	$syntax[] = array(
                	    'type' => 'close',
                	    'str' => '[/'.$elem['name'].']',
                	    'name' => $elem['name'],
                	    'layout' => array(
                	        array(0, '['),
                	        array(1, '/'),
                	        array(2, $elem['name']),
                	        array(0, ']')
                	    )
                	);
                }
            }
        }
        
        return $syntax;
    }

    private function insert_smiles($text) 
    {
        $text = strtr($text, $this->mnemonics);
        
        return $text;
    }

    /**
     * Возвращает распарсенный HTML-код.
     *
     * @param array $elems Дерево семантического разбора текста с BB-кодами.
     * 
     * @return string Распарсенный HTML-код.
     */
    public function get_html($elems = null) 
    {
        if (!is_array($elems)) 
        {
            $elems =& $this->tree;
        }
        
        $result = '';
        $lbr = 0;
        $rbr = 0;
        foreach ($elems as $elem) 
        {
            if ('text' == $elem['type']) 
            {
                $elem['str'] = $this->insert_smiles($elem['str']);
                for ($i = 0; $i < $rbr; ++$i) 
                {
                    $elem['str'] = ltrim($elem['str']);
                    if ('<br />' == substr($elem['str'], 0, 6)) 
                    {
                        $elem['str'] = substr_replace($elem['str'], '', 0, 6);
                    }
                }
                $result .= $elem['str'];
            } 
            else 
            {
            	$this->init_tag_class($elem['name']);
            	$handler = $this->tags[$elem['name']];
            	
                /* Убираем лишние переводы строк */
                $class_vars = get_class_vars($handler);
                $lbr = $class_vars['lbr'];
                $rbr = $class_vars['rbr'];
                for ($i = 0; $i < $lbr; ++$i) 
                {
                    $result = rtrim($result);
                    if ('<br />' == sb_strtolower(sb_substr($result, -6))) 
                    {
                        $result = sb_substr($result, 0, -6);
                    }
                    elseif ('<br>' == sb_strtolower(sb_substr($result, -4))) 
                    {
                    	$result = sb_substr($result, 0, -4);
                    }
                }
                
                /* Обрабатываем содержимое элемента */
                $tag = $this->_tag_objects[$handler];
                $tag->tags = $this->tags;
                $tag->mnemonics = $this->mnemonics;
                $tag->tag = $elem['name'];
                $tag->attrib = $elem['attrib'];
                $tag->tree = $elem['val'];
                $result .= $tag->get_html();
            }
        }
        
        $result = preg_replace(
            '\'\s*<br \/>\s*<br \/>\s*\'si', "\n<br />&nbsp;<br />\n", $result
        );
        $result = preg_replace(
            '\'\s*<br>\s*<br>\s*\'si', "\n<br>&nbsp;<br>\n", $result
        );

        return $result;
    }
    
    /**
     * Функция инициализирует объект обработчика для тега (если он еще не инициализирован).
     * 
     * @param string $tagName Имя тега.
     */
    private function init_tag_class($tagName) 
    {
        if (!class_exists($this->tags[$tagName])) 
        {
           	$this->tags[$tagName] = 'sbBBCode';
        }
        
        $handler = $this->tags[$tagName];
        if (!isset($this->_tag_objects[$handler])) 
        {
            $this->_tag_objects[$handler] = new $handler;
        }
    }
    
	/**
	 *  Функция преобразует URL с целью защиты от javascript-инъекции. 
	 */
    protected function checkUrl($url) 
    {
        if (!$url) 
        { 
        	return ''; 
        }
        
        $protocols = array(
            'ftp://', 'file://', 'http://', 'https://', 'mailto:', 'svn://',
            '#',      '/',       '?',       './',       '../',     'www.'
        );
        
        $is_http = false;
        foreach ($protocols as $val) 
        {
            if ($val == sb_substr($url, 0, strlen($val))) 
            {
                $is_http = true;
                if ('www.' == $val) 
                {
                	$url = 'http://'.$url;
                }
                break;
            }
        }
        
        if (!$is_http) 
        { 
        	$url = './'.$url; 
        }
        
        $url = sb_htmlentities($url, ENT_QUOTES);
        $url = str_replace(array('.', ':', '(', ')'), array('&#'.ord('.').';', '&#'.ord(':').';', '&#'.ord('(').';', '&#'.ord(')').';'), $url);

        return $url;
    }
}

// Класс для простейших тегов
class sbBBCodeTags_Simple extends sbBBCode 
{
    public $behaviour = 'span';
    
    public function get_html($tree = null) 
    {
        switch ($this->tag) 
        {
            case 'b':
                $this->tag = 'strong';
                break;
                
            case 's':
            case 'strike':
                $this->tag = 'del';
        }
        
        return '<'.$this->tag.'>'.parent::get_html().'</'.$this->tag.'>';
    }
}

// Класс для тегов [*] и [li]
class sbBBCodeTags_Li extends sbBBCode 
{
    public $behaviour = 'li';
    public function get_html($tree = null) 
    {
        return '<li>'.parent::get_html($this->tree).'</li>';
    }
}

// Класс для тегов [list], [ol], [ul]
class sbBBCodeTags_List extends sbBBCode 
{
    public $lbr = 1;
    public $rbr = 1;
    public $behaviour = 'ul';
    
    public function get_html($tree = null) 
    {
        $tag_name = 'ul';
        $type = '';
        
        switch ($this->tag) 
        {
            case 'ol':
                $tag_name = 'ol';
                $type = strtolower($this->attrib['ol']);
                break;
                
            case 'list':
                if ($this->attrib['list']) 
                {
                    $tag_name = 'ol';
                }
                $type = strtolower($this->attrib['list']);
                $this->tag = 'del';
		}

		$attr = '';
		if ('1' == $type) 
        {
            $attr = ' type="1"';
        } 
        elseif ($type) 
        {
            $attr = ' type="a"';
        }
        
        return '<'.$tag_name.$attr.'>'.parent::get_html().'</'.$tag_name.'>';
    }
}

// Класс для тега [br]
class sbBBCodeTags_Br extends sbBBCode 
{
    public $is_close = true;
    public $behaviour = 'span';
    
    public function get_html($tree = null) 
    {
        return '<br />'."\n";
    }
}

// Класс для тега [hr]
class sbBBCodeTags_Hr extends sbBBCode 
{
    public $is_close = true;
    public $rbr = 1;
    public $behaviour = 'hr';

    public function get_html($tree = null) 
    {
        return '<hr />';
    }
}

// Класс для тегов [a], [anchor] и [url]
class sbBBCodeTags_A extends sbBBCode 
{
    public $behaviour = 'a';

    public function get_html($tree = null) 
    {
        $text = '';
        foreach ($this->tree as $val) 
        {
            if ('text' == $val['type']) 
            { 
            	$text .= $val['str']; 
            }
        }
        
        $href = '';
        if (isset($this->attrib['url']) && $this->attrib['url']) 
        {
            $href = $this->attrib['url'];
        }
        elseif (isset($this->attrib['a']) && $this->attrib['a']) 
        {
            $href = $this->attrib['a'];
        }
        elseif (isset($this->attrib['href']) && $this->attrib['href']) 
        {
            $href = $this -> attrib['href'];
        }
        elseif (!isset($this->attrib['anchor'])) 
        { 
        	$href = $text; 
        }
        
        $href = $this->checkUrl($href);
        
        $attr = '';
        if ($href) 
        {
            $attr .= ' href="'.$href.'"';
        }
        
        if (isset($this->attrib['title']) && $this->attrib['title']) 
        {
            $attr .= ' title="'.$this->attrib['title'].'"';
        }

        $id = '';
        if (isset($this->attrib['id']) && $this->attrib['id']) 
        {
            $id = $this->attrib['id'];
        }
        elseif (isset($this->attrib['name']) && $this->attrib['name']) 
        {
            $id = $this->attrib['name'];
        }
        elseif (isset($this->attrib['anchor'])) 
        {
            $id = $this->attrib['anchor'];
            if (!$id) 
            { 
            	$id = $text; 
            }
        }
        
        if ($id) 
        {
            if ($id[0] < 'A' || $id[0] > 'z') 
            { 
            	$id = 'bb'.$id; 
            }
            
            $attr .= ' id="'.$id.'"';
        }
        
        if (isset($this->attrib['target']) && $this->attrib['target']) 
        {
            $target = $this->attrib['target'];
        }
        else
        {
        	$target = '_blank';
        }
        
        $attr .= ' target="'.$target.'"';
        
        return '<a '.$attr.'>'.parent::get_html($this->tree).'</a>';
    }
}

// Класс для тега [email]
class sbBBCodeTags_Email extends sbBBCode 
{
    public $behaviour = 'a';

    public function get_html($tree = null) 
    {
        $attr = '';
        $href = $this->attrib['email'];
        if (!$href) 
        {
            foreach ($this->tree as $text) 
            {
                if ('text' == $text['type']) 
                { 
                	$href .= $text['str']; 
                }
            }
        }
        
        if ($href)
        {
        	if ('mailto:' != sb_substr($href, 0, 7)) 
        	{ 
        		$href = 'mailto:'.$href; 
        	}
        	
        	$attr .= ' href="'.$href.'"';
        }
        
        if (isset($this->attrib['title']) && $this->attrib['title']) 
        { 
        	$attr .= ' title="'.$this->attrib['title'].'"'; 
        }
        
        if (isset($this->attrib['name']) && $this->attrib['name']) 
        { 
        	$attr .= ' name="'.$this->attrib['name'].'"'; 
        }
        
        return '<a'.$attr.'>'.parent::get_html($this->tree).'</a>';
    }
}

// Класс для тегов [align], [center], [justify], [left] и [right]
class sbBBCodeTags_Align extends sbBBCode 
{
    public $rbr = 1;
    
    public function get_html($tree = null) 
    {
        $align = '';

        if (isset($this->attrib['justify'])) 
        { 
        	$align = 'justify'; 
        }
        elseif (isset($this->attrib['left'])) 
        { 
        	$align = 'left'; 
        }
        elseif (isset($this->attrib['right'])) 
        { 
        	$align = 'right'; 
        }
        elseif (isset($this->attrib['center'])) 
        { 
        	$align = 'center'; 
        }
        
        if (! $align && isset($this->attrib['align'])) 
        {
            switch (strtolower($this->attrib['align'])) 
            {
                case 'left':
                    $align = 'left';
                    break;
                    
                case 'right':
                    $align = 'right';
                    break;
                    
                case 'center':
                    $align = 'center';
                    break;
                    
                case 'justify':
                    $align = 'justify';
                    break;
            }
        }
        
        return '<div align="'.$align.'">'.parent::get_html($this->tree).'</div>';
    }
}

// Класс для тега [img]
class sbBBCodeTags_Img extends sbBBCode 
{
    public $behaviour = 'img';
    
    public function get_html($tree = null) 
    {
        $attr = '';
    	if (isset($this->attrib['alt']) && $this->attrib['alt']) 
        {
            $attr .= ' alt="'.$this->attrib['alt'].'" title="'.$this->attrib['alt'].'"';
        }
        
        if (isset($this->attrib['width'])) 
        {
            $width = (int) $this->attrib['width'];
            $attr .= $width ? ' width="'.$width.'"' : '';
        }
        
        if (isset($this->attrib['height'])) 
        {
            $height = (int) $this->attrib['height'];
            $attr .= $height ? ' height="'.$height.'"' : '';
        }
        
        if (isset($this->attrib['border'])) 
        {
            $border = (int) $this -> attrib['border'];
            $attr .= ' border="'.$border.'"';
        }
        
        $src = '';
        
        if (isset($this->attrib['img']) && $this->attrib['img'])
        {
        	$src = $this->attrib['img']; 	
        }
        elseif (isset($this->attrib['src']) && $this->attrib['src'])
        {
        	$src = $this->attrib['src']; 	
        }
        else 
        {
	        foreach ($this->tree as $text) 
	        {
	            if ('text' == $text['type']) 
	            { 
	            	$src .= $text['str']; 
	            }
	        }
        }
        
        $src = $this->checkUrl($src);
        
        return '<img src="'.$src.'" '.$attr.' />';
    }
}

// Класс для тега [table]
class sbBBCodeTags_Table extends sbBBCode 
{
    public $rbr = 1;
    public $behaviour = 'table';
    
    public function get_html($tree = null) 
    {
        $attr = '';
        $border = isset($this->attrib['border']) ? (int) $this -> attrib['border'] : null;
        if (null !== $border) 
        { 
        	$attr .= ' border="'.$border.'"'; 
        }
        
        if (isset($this->attrib['width']) && $this->attrib['width']) 
        { 
        	$attr .= ' width="'.$this->attrib['width'].'"'; 
        }
        
        $cellspacing = isset($this->attrib['cellspacing']) ? (int) $this -> attrib['cellspacing'] : null;
        if (null !== $cellspacing) 
        { 
        	$attr .= ' cellspacing="'.$cellspacing.'"'; 
        }
        
        $cellpadding = isset($this -> attrib['cellpadding']) ? (int) $this -> attrib['cellpadding'] : null;
        if (null !== $cellpadding) 
        { 
        	$attr .= ' cellpadding="'.$cellpadding.'"'; 
        }
        
        if (isset($this->attrib['align']) && $this->attrib['align']) 
        { 
        	$attr .= ' align="'.$align.'"'; 
        }
        
        $str = '<table'.$attr.'>';
        foreach ($this->tree as $key => $item) 
        {
            if ('text' == $item['type']) 
            { 
            	unset($this -> tree[$key]); 
            }
        }
        
        $str .= parent::get_html($this->tree).'</table>';
        
        return $str;
    }
}

// Класс для тега [tr]
class sbBBCodeTags_Tr extends sbBBCode 
{
    public $behaviour = 'tr';

    public function get_html($tree = null) 
    {
        $str = '<tr>';
        foreach ($this->tree as $key => $item) 
        {
            if ('text' == $item['type']) 
            { 
            	unset($this -> tree[$key]); 
            }
        }
        
        $str .= parent::get_html($this->tree).'</tr>';
        
        return $str;
    }
}

// Класс для тегов [th], [td]
class sbBBCodeTags_Td extends sbBBCode 
{
    public $behaviour = 'td';

    public function get_html($tree = null) 
    {
        $attr = '';
        if (isset($this->attrib['width']) && $this->attrib['width']) 
        { 
        	$attr .= ' width="'.$this->attrib['width'].'"'; 
        }
        
    	if (isset($this->attrib['height']) && $this->attrib['height']) 
        { 
        	$attr .= ' height="'.$this->attrib['height'].'"'; 
        }
        
    	if (isset($this->attrib['align']) && $this->attrib['align']) 
        { 
        	$attr .= ' align="'.$this->attrib['align'].'"'; 
        }
        
    	if (isset($this->attrib['valign']) && $this->attrib['valign']) 
        { 
        	$attr .= ' valign="'.$this->attrib['valign'].'"'; 
        }
        
        if (isset($this->attrib['colspan'])) 
        {
            $colspan = (int) $this->attrib['colspan'];
            if ($colspan) 
            { 
            	$attr .= ' colspan="'.$colspan.'"'; 
            }
        }
        
        if (isset($this->attrib['rowspan'])) 
        {
            $rowspan = (int) $this->attrib['rowspan'];
            if ($rowspan) 
            { 
            	$attr .= ' rowspan="'.$rowspan.'"'; 
            }
        }
        
        return '<'.$this->tag.$attr.'>'.parent::get_html($this->tree).'</'.$this->tag.'>';
    }
}

// Класс для тега [caption]
class sbBBCodeTags_Caption extends sbBBCode 
{
	public $behaviour = 'caption';

	public function get_html($tree = null) 
	{
        return '<caption>'.parent::get_html($this->tree).'</caption>';
    }
}

// Класс для тега [abbr]
class sbBBCodeTags_Abbr extends sbBBCode 
{
    public $behaviour = 'span';

    public function get_html($tree = null) 
    {
        $str = '<abbr';
        if ($this->attrib['abbr']) 
        {
            $str .= ' title="'.$this->attrib['abbr'].'"';
        }
        
        return $str.'>'.parent::get_html($this->tree).'</abbr>';
    }
}

// Класс для тега [abbr]
class sbBBCodeTags_Acronym extends sbBBCode 
{
    public $behaviour = 'span';

    public function get_html($tree = null) 
    {
        $str = '<acronym';
        if ($this->attrib['acronym']) 
        {
            $str .= ' title="'.$this->attrib['acronym'].'"';
        }
        
        return $str.'>'.parent::get_html($this->tree).'</acronym>';
    }
}

// Класс для тега [p]
class sbBBCodeTags_P extends sbBBCode 
{
    public $lbr = 2;
    public $rbr = 2;
    public $behaviour = 'p';

    public function get_html($tree = null) 
    {
        $str = "\n<p";
        
        if (isset($this->attrib['align']) && $this->attrib['align']) 
        { 
        	$str .= ' align="'.$this->attrib['align'].'"'; 
        }
        
        return $str.'>'.parent::get_html()."</p>\n";
    }
}

// Класс для тега [color]
class sbBBCodeTags_Color extends sbBBCode 
{
    public $behaviour = 'span';
    
    public function get_html($tree = null) 
    {
        $color = $this->attrib['color'];
        
        return '<span style="color:'.$color.';">'.parent::get_html($this->tree).'</span>';
    }
}

// Класс для тега [quote]
class sbBBCodeTags_Quote extends sbBBCode 
{
    public $rbr = 1;
    static $quote_open = '';
    static $quote_close = '';
    static $clear_quotes = false;
    
    public function get_html($tree = null) 
    {
    	if (self::$clear_quotes)
    	{
    		return ' ';
    	}
    	
        $author = trim($this->attrib['quote'], '"');
       	$date = isset($this->attrib['date']) ? trim($this->attrib['date'], '"') : ''; 
        
       	return str_replace(array('{AUTHOR}', '{DATE}'), array($author, $date), self::$quote_open).parent::get_html($this->tree).self::$quote_close;
    }
    
    static public function set_quote_open($str)
    {
    	self::$quote_open = $str;
    }
    
	static public function set_quote_close($str)
    {
    	self::$quote_close = $str;
    }
    
	static public function set_clear_quotes($clear)
    {
    	self::$clear_quotes = $clear;
    }
}
?>