<?php

/**
 * Функция возвращает изображение и код CAPTCHA
 *
 * Вызывает класс sbCaptcha.
 *
 * @see sbCaptcha
 * @return array Массив с ссылкой на изображение и кодом CAPTCHA или FALSE в случае ошибки.
 */
function sbProgGetTuring ()
{
	// запрещаем кэширование у модуля, если используется капча
	$GLOBALS['sbCache']->mCacheOff = true;

    require_once(SB_CMS_LIB_PATH.'/prog/sbCaptcha.inc.php');
    $time = sbCaptcha::getCode();

    return array('/cms/admin/captcha.php?turing_id='.$time, $time);
}

/**
 * Функция проверяет указанный код CAPTCHA
 *
 * @param string $value_field Имя поля, содержащего код CAPTCHA.
 * @param string $hash_field Имя поля, содержащего хэш CAPTCHA.
 *
 * @return bool TRUE, если код верный, FALSE в ином случае.
 */
function sbProgCheckTuring ($value_field, $hash_field)
{
    if (isset($_POST[$value_field]) && isset($_POST[$hash_field]) && trim($_POST[$hash_field]) != '' && isset($_SESSION['sb_captcha']))
	{
		$hash = preg_replace('/[^A-Za-z0-9]+/', '', $_POST[$hash_field]);
	    if(isset($_SESSION['sb_captcha'][$hash]) && $_SESSION['sb_captcha'][$hash] == $_POST[$value_field])
	    {
	    	unset($_SESSION['sb_captcha'][$hash]);
	   		return true;
	    }
	}

	return false;
}

/**
 * Функция стартует сессию
 *
 */
function sbProgStartSession()
{
	if (!@session_id())
    {
    	sb_setcookie('sb_start_session', 1);
    	@session_name('SBPROGID');
        @session_start();
    }
}

/**
 * Добавление/редактирование элемента модуля и прописывание связей с разделами
 *
 * Данная функция не проверяет права доступа пользователя к разделу.
 *
 * @param string $table_name Имя таблицы БД, в которую добавляется элемент.
 * @param string $id_name Имя поля, содержащего уникальный идентифкатор элемента.
 * @param array $row Ассоциативный массив со значениями полей элемента (ключи - имена полей, значения - значения полей).
 * @param array $cat_ids Массив идентификаторов разделов, с которыми необходимо прописать связь элемента.
 * @param int $edit_id Идентификатор редактируемого элемента, если < 1 - элемент добавляется.
 * @param int $now_cat Раздел, в котором располагается редактируемый элемент.
 *
 * @return mixed Идентификатор добавленного элемента, FALSE в случае ошибки.
 */
function sbProgAddElement($table_name, $id_name, $row, $cat_ids, $edit_id = -1, $now_cat = 0)
{
	if ($edit_id == -1 && (!is_array($cat_ids) || !isset($cat_ids[0])))
	{
	    return false;
	}

	if($edit_id > 0)
	{
		$res = sql_query('SELECT cat_ident FROM sb_categs WHERE cat_id=?d', $now_cat);
		if ($res)
		{
			list($cat_ident) = $res[0];
		}
		else
		{
			return false;
		}

		//Редактирую элемент
		$res = sql_query('UPDATE ?# SET ?a WHERE ?#=?d', $table_name, $row, $id_name, $edit_id);

		if(!$res)
		{
			return false;
		}

		sql_query('INSERT INTO sb_catchanges (el_id, cat_ident, change_date, change_user_id, action) VALUES (?d, ?, ?d, 0, ?)', $edit_id, $cat_ident, time(), 'edit');

		if (is_array($cat_ids))
		{
			if (count($cat_ids) == 1 && in_array($now_cat, $cat_ids))
			{
				return $edit_id;
			}

			// Узнаю в каком разделе физически находится элемент
			$res = sql_query('SELECT c.cat_id FROM sb_catlinks l, sb_categs c WHERE c.cat_ident=? AND l.link_el_id = ?d AND l.link_cat_id = c.cat_id AND l.link_src_cat_id = 0', $cat_ident, $edit_id);
			if($res && $res[0][0] != '')
			{
				$fiz_cat_id = $res[0][0];
		    }
			else
			{
			    return false;
			}

			if (count($cat_ids) == 1 && in_array($fiz_cat_id, $cat_ids))
			{
				return $edit_id;
			}

		    // Удаляем связи со старыми разделами
		    sql_query('DELETE FROM sb_catlinks WHERE link_el_id = ?d AND (link_cat_id = ?d OR link_src_cat_id = ?d)', $edit_id, $fiz_cat_id, $fiz_cat_id);

			// Ставим связи с новыми разделами
			$root_id = -1;
	    	foreach($cat_ids as $cat_id)
	    	{
	    		if ($root_id == -1)
	    		{
		    		sql_query('INSERT INTO sb_catlinks (link_cat_id, link_el_id, link_src_cat_id) VALUES (?d, ?d, 0)', $cat_id, $edit_id);
	    			$root_id = $cat_id;
			    }
			    else
			    {
			    	sql_query('INSERT INTO sb_catlinks (link_cat_id, link_el_id, link_src_cat_id) VALUES (?d, ?d, ?d)', $cat_id, $edit_id, $root_id);
			    }
	    	}
		}

		return $edit_id;
	}
	else
	{
		$res = sql_query('SELECT cat_ident FROM sb_categs WHERE cat_id=?d', $cat_ids[0]);
		if ($res)
		{
			list($cat_ident) = $res[0];
		}
		else
		{
			return false;
		}

		if (sql_query('INSERT INTO ?# (?#) VALUES (?a)', $table_name, array_keys($row), array_values($row)))
	    {
	    	$id = sql_insert_id();

	    	$root_id = -1;

	    	foreach($cat_ids as $cat_id)
	    	{
	    		if ($root_id == -1)
	    		{
		    		if (sql_query('INSERT INTO sb_catlinks (link_cat_id, link_el_id, link_src_cat_id) VALUES (?d, ?d, 0)', $cat_id, $id))
		    		{
		    			sql_query('INSERT INTO sb_catchanges (el_id, cat_ident, change_date, change_user_id, action) VALUES (?d, ?, ?d, 0, ?)', $id, $cat_ident, time(), 'add');
		    			$root_id = $cat_id;
		    		}
			    }
			    else
			    {
			    	sql_query('INSERT INTO sb_catlinks (link_cat_id, link_el_id, link_src_cat_id) VALUES (?d, ?d, ?d)', $cat_id, $id, $root_id);
			    }
	    	}

	    	if ($root_id != -1)
	    	{
	        	return $id;
	    	}
	    	else
	    	{
	    	    sql_param_query('DELETE FROM ?# WHERE ?#=?d', $table_name, $id_name, $id);
	    	}
	    }
	}

    return false;
}

/**
 * Удаление элемента модуля и удаление связей с разделами
 *
 * Данная функция не проверяет права доступа пользователя к разделу.
 *
 * @param string $table_name Имя таблицы БД, из которой удаляется элемент.
 * @param string $id_name Имя поля, содержащего уникальный идентифкатор элемента.
 * @param array $id_value Значение уникального идентификатора элемента.
 * @param array $cat_ident Идентификатор разделов модуля.
 *
 * @return bool TRUE в случае успеха, FALSE в случае ошибки.
 */
function sbProgDeleteElement($table_name, $id_name, $id_value, $cat_ident)
{
	$res = sql_query('SELECT l.`link_id` FROM `sb_catlinks` AS l, `sb_categs` AS c, ?# AS p
						WHERE c.`cat_ident`=? AND l.`link_cat_id`=c.`cat_id` AND l.`link_el_id`=?d',
						$table_name, $cat_ident, $id_value);
	if (!$res)
	{
		return false;
	}

	$link_ids = array();
	foreach ($res as $value)
	{
		$link_ids[] = $value[0];
	}

	if (!sql_query('DELETE FROM ?# WHERE ?#=?d', $table_name, $id_name, $id_value))
	{
		return false;
	}

	sql_query('DELETE FROM sb_catlinks WHERE link_id IN (?a)', $link_ids);
	sql_query('DELETE FROM sb_catchanges WHERE cat_ident=? AND el_id=?d', $cat_ident, $id_value);

	return true;
}

/**
 * Парсинг BB-кодов в строке.
 *
 * @param string $str Строка, в которой необходимо произвести парсинг.
 * @param string $quote_open Макет для замены кода [quote].
 * @param string $quote_close Макет для замены кода [/quote].
 * @param bool $clear_quotes Вырезать цитаты.
 *
 * @return string Распарсенная строка.
 */
function sbProgParseBBCodes($str, $quote_open = '<div style="border: 1px solid red; border-style: dashed; margin-left: 30px;">Автор: {AUTHOR}, дата: {DATE}', $quote_close = '</div>', $clear_quotes = false)
{
	require_once(SB_CMS_LIB_PATH.'/prog/sbBBCode.inc.php');

	sbBBCodeTags_Quote::set_quote_open($quote_open);
	sbBBCodeTags_Quote::set_quote_close($quote_close);
	sbBBCodeTags_Quote::set_clear_quotes($clear_quotes);

	$str = str_replace('&quot;', '"', $str);
	$bb = new sbBBCode($str);

	return $bb->get_html();
}

/**
 * Возвращает массив для ссылки "подробнее..."
 *
 * Первый элемент массива - URL страницы, второй - расширение файла.
 *
 * @param string $params_page URL страницы.
 * @return array Первый элемент массива - URL страницы, второй - расширение файла.
 */
function sbGetMorePage($params_page)
{
	if (trim($params_page) == '')
	{
		$params_page = $_SERVER['PHP_SELF'];
	}

	if (sb_stripos($params_page, 'http:') !== 0 && sb_stripos($params_page, 'https:') !== 0 && sb_stripos($params_page, '/') !== 0 && sb_stripos($params_page, '\\') !== 0)
	{
		$params_page = '/'.$params_page;
	}

    $more_ar = @parse_url($params_page);
    if (!$more_ar)
    {
    	return array('', '');
    }

    $path = isset($more_ar['path']) ? $more_ar['path'] : '';
	$page = '';

    if ($path != '')
    {
		$path = explode('/', str_replace('\\', '/', $path));
        if (sb_strpos($path[count($path) - 1], '.') !== false)
        {
            $page = array_pop($path);
        }
    }

    if (sbPlugins::getSetting('sb_static_urls') != 1)
    {
    	if ($page == '')
    	{
    		if (sb_substr($params_page, -1) != '/')
                $params_page .= '/';
    	}

    	return array($params_page, '');
    }

	$index_page = sbPlugins::getSetting('sb_directory_index');
	if (trim($index_page) == '')
	{
		$index_page = 'index.php';
	}

	$more_ext = '';

	if ($page == $index_page)
    {
        $params_page = sb_str_replace($page, '', $params_page);
        if (sb_substr($params_page, -1) != '/')
            $params_page .= '/';

        $ext_pos = sb_strrpos($page, '.');
    	if ($ext_pos !== false)
	    	$more_ext = sb_substr($page, $ext_pos + 1);
	    else
	    	$more_ext = 'php';

        return array($params_page, $more_ext);
    }

    $ext_pos = sb_strrpos($page, '.');
    if ($ext_pos !== false)
    {
		$more_ext = sb_substr($page, $ext_pos + 1);
		$ext_pos = sb_strrpos($params_page, '.');
		$params_page = sb_substr($params_page, 0, $ext_pos);
	}
	else
	{
		$more_ext = 'php';
	}

	if (sb_substr($params_page, -1) != '/')
		$params_page .= '/';

    return array($params_page, $more_ext);
}

/**
 * Возвращает SQL-запроса для поиска по текстовым полям.
 *
 * @param string $field_name Имя поля и префикс таблицы БД.
 * @param string $get_name Имя REQUEST-параметра.
 * @param string $filter_compare Искать по вхождению или по полному совпадению.
 * @param string $filter_logic Логика фильтра.
 * @param string $filter_text_logic Логика поиска слов в фразе.
 * @param object $morph_db Использовать морфологию или нет.
 *
 * @return string SQL-запрос для поиска по текстовым полям.
 */
function sbGetFilterTextSql($field_name, $get_name, $filter_compare, $filter_logic, $filter_text_logic, &$morph_db)
{
	$filter_sql = '';
	if (!isset($_REQUEST[$get_name]) || trim($_REQUEST[$get_name]) == '')
	{
		return $filter_sql;
	}

	$_REQUEST[$get_name] = sb_html_entity_decode($_REQUEST[$get_name]);

	if ($filter_compare == 'IN')
    {
		$words = explode(' ', trim($_REQUEST[$get_name]));
	    $filter_sql .= '(';
	    foreach ($words as $word)
	    {
	      	if ($morph_db)
	        {
	        	$morph_words = $morph_db->get_all_forms($word);
	            if (count($morph_words) > 0)
	            {
	            	$filter_sql .= '(';
	            	foreach ($morph_words as $word)
		            {
		            	$filter_sql .= $GLOBALS['sbSql']->escape($field_name, true, false).' LIKE '.$GLOBALS['sbSql']->escape('%'.$word.'%').' OR ';
		            }

		            $filter_sql = sb_substr($filter_sql, 0, -4).') '.$filter_text_logic.' ';
	            }
	            else
	            {
	            	$filter_sql .= $GLOBALS['sbSql']->escape($field_name, true, false).' LIKE '.$GLOBALS['sbSql']->escape('%'.$word.'%').' '.$filter_text_logic.' ';
	            }
	        }
	        else
	        {
	           	$filter_sql .= $GLOBALS['sbSql']->escape($field_name, true, false).' LIKE '.$GLOBALS['sbSql']->escape('%'.$word.'%').' '.$filter_text_logic.' ';
	        }
	    }

	    $filter_sql = sb_substr($filter_sql, 0, -(2 + strlen($filter_text_logic))).') '.$filter_logic.' ';
    }
   	else
  	{
		$filter_sql .= $GLOBALS['sbSql']->escape($field_name, true, false).' = '.$GLOBALS['sbSql']->escape($_REQUEST[$get_name]).' '.$filter_logic.' ';
	}

	return $filter_sql;
}

/**
 * Возвращает SQL-запрос для точного поиска (на равенство) по полям.
 *
 * @param string $field_name Имя поля и префикс таблицы БД.
 * @param string $get_name Имя REQUEST-параметра.
 * @param string $filter_logic Логика фильтра.
 *
 * @return string SQL-запрос для точного поиска (на равенство) по полям.
 */
function sbGetFilterSql($field_name, $get_name, $filter_logic)
{
	$filter_sql = '';
	if (!isset($_REQUEST[$get_name]) || trim($_REQUEST[$get_name]) == '')
	{
		return $filter_sql;
	}

	$filter_sql = $GLOBALS['sbSql']->escape($field_name, true, false)." = '".intval($_REQUEST[$get_name])."' ".$filter_logic.' ';

	return $filter_sql;
}

/**
 * Возвращает SQL-запрос для поиска по полям типа "Число" или "Дата".
 *
 * @param string $field_name Имя поля и префикс таблицы БД.
 * @param string $get_name Имя REQUEST-параметра.
 * @param string $filter_logic Логика фильтра.
 * @param bool $date Тип поля "Число" (FALSE) или "Дата" (TRUE).
 * @param string $date_temp Формат ввода даты.
 *
 * @return string SQL-запрос для поиска по полям типа "Число" или "Дата".
 */
function sbGetFilterNumberSql($field_name, $get_name, $filter_logic, $date = false, $date_temp = '')
{
	$filter_sql = '';

	if (isset($_REQUEST[$get_name]) && trim($_REQUEST[$get_name]) != '')
	{
		$val = explode(',', $_REQUEST[$get_name]);
		if (count($val) == 1)
		{
			if ($date)
			{
				return $GLOBALS['sbSql']->escape($field_name, true, false).' = "'.sb_datetoint($_REQUEST[$get_name], $date_temp).'" AND ';
			}
			else
			{
				return $GLOBALS['sbSql']->escape($field_name, true, false).' = "'.floatval($_REQUEST[$get_name]).'" AND ';
			}
		}
		else
		{
		    //TODO может не отработать на float полях
		    if($date)
            {
                $sql = '(';
                
                foreach ($val as $value)
                {
                    $sql .= $GLOBALS['sbSql']->escape($field_name, true, false).' = "'.sb_datetoint(trim($value), $date_temp).'" OR ';
                }
                
                return sb_substr($sql, 0, -4).') AND ';
            }
            else
            {
                $data = array();
                foreach ($val as $value)
                {
                    $data[] = intval(trim($value));
                }
                $sql = '('.$GLOBALS['sbSql']->escape($field_name, true, false).' IN ('.implode(',', $data).')) AND ';
                
                return $sql;
                //$sql .= $GLOBALS['sbSql']->escape($field_name, true, false).' = "'.floatval(trim($value)).'" OR ';
            }
		}
	}

	$is_low = (isset($_REQUEST[$get_name.'_lo']) && trim($_REQUEST[$get_name.'_lo']) != '');
    $is_high = (isset($_REQUEST[$get_name.'_hi']) && trim($_REQUEST[$get_name.'_hi']) != '');
    if ($is_low || $is_high)
    {
    	$filter_sql .= '(';
    	if ($is_low)
        {
            if ($date)
            {
                $filter_sql .= $GLOBALS['sbSql']->escape($field_name, true, false).' >= "'.sb_datetoint($_REQUEST[$get_name.'_lo'], $date_temp).'" AND ';
            }
            else
            {
                $filter_sql .= $GLOBALS['sbSql']->escape($field_name, true, false).' >= "'.floatval($_REQUEST[$get_name.'_lo']).'" AND ';
            }
        }

        if ($is_high)
        {
            if ($date)
            {
                $filter_sql .= $GLOBALS['sbSql']->escape($field_name, true, false).' <= "'.sb_datetoint($_REQUEST[$get_name.'_hi'], $date_temp).'" AND ';
            }
            else
            {
                $filter_sql .= $GLOBALS['sbSql']->escape($field_name, true, false).' <= "'.floatval($_REQUEST[$get_name.'_hi']).'" AND ';
            }
        }

        $filter_sql = sb_substr($filter_sql, 0, -5).') '.$filter_logic.' ';
    }

    return $filter_sql;
}

/**
 * Возвращает SQL-запрос для фильтрации каталога по идентификаторам товара.
 *
 * string $field_name Имя поля и префикс таблицы БД.
 * string $cookie_name Имя COOKIE-параметра
 * int $plugin_id Идентификатор модуля конструктора модулей для которого применяется фильтр
 */
function sbGetFilterCookieSql($field_name, $cookie_name, $plugin_id)
{
	$filter_sql = '-1';
	if(isset($_COOKIE[$cookie_name]))
	{
		foreach($_COOKIE[$cookie_name] as $key => $value)
		{
			$pl_id = explode('_', $key);
			if($pl_id[0] == $plugin_id)
			{
				$filter_sql .= ','.intval($value);
			}
		}
	}

	return ' AND '.$GLOBALS['sbSql']->escape($field_name, true, false).' IN ('.$filter_sql.')';
}

/**
 * Функция парсинга макета дизайна вывода разделов.
 *
 * @param array $cat_ids Массив идентификаторов выводимых разделов.
 * @param string $cat_ident Идентификатор принадлежности раздела к модулую.
 * @param array $sel_cat_ids Массив идентификаторов выбранных разделов.
 * @param string $option_temp Макет дизайна вывода опции.
 * @param string $select_temp Макет дизайна вывода списка.
 * @param string $closed_right Права доступа к закрытым разделам.
 *
 * @return string Распарсенный вывод разделов.
 */
function sbProgGetCategsList($cat_ids, $cat_ident, $sel_cat_ids, $option_temp, $select_temp, $closed_right)
{
	if (!is_array($sel_cat_ids))
		$sel_cat_ids = array();

	if(!is_array($cat_ids) || count($cat_ids) <= 0)
		return '';

	$res = sql_query('SELECT c.cat_id FROM sb_categs AS c, sb_categs AS c2
							WHERE c2.cat_left <= c.cat_left
							AND c2.cat_right >= c.cat_right
							AND c.cat_ident = ?
							AND c2.cat_ident = ?
							AND c2.cat_id IN (?a)
							ORDER BY c.cat_left', $cat_ident, $cat_ident, $cat_ids);

	if (!$res)
		return '';

	$cat_ids = array();
	foreach ($res as $value)
	{
		$cat_ids[] = $value[0];
	}

	// проверяем права на добавление
    $res = sql_query('SELECT cat_id FROM sb_categs WHERE cat_closed=1 AND cat_id IN (?a)', $cat_ids);
    if($res)
    {
      	$closed_ids = array();
        foreach($res as $value)
        {
            $closed_ids[] = $value[0];
        }

        $cat_ids = sbAuth::checkRights($closed_ids, $cat_ids, $closed_right);

        if(count($cat_ids) < 1)
        {
            return '';
        }
    }

    $res = sql_query('SELECT cat_id, cat_title, cat_level FROM sb_categs WHERE cat_id IN (?a) ORDER BY cat_left', $cat_ids);
	if (!$res)
		return '';

	$first_level = $res[0][2];
    $result = '';
	foreach($res as $value)
	{
	    list($cat_id, $cat_title, $cat_level) = $value;

	    if(in_array($cat_id, $sel_cat_ids))
	    	$selected_str = sb_stripos($option_temp, 'option') !== false ? ' selected="selected"' : ' checked="checked"';
	    else
	    	$selected_str = '';

	    $result .= str_replace(array('{OPT_TEXT}', '{OPT_VALUE}', '{OPT_SELECTED}'), array(str_repeat('&nbsp;', ($cat_level - $first_level) * 2).$cat_title, $cat_id, $selected_str), $option_temp);
	}

	return str_replace('{OPTIONS}', $result, $select_temp);
}

function sbProgCleanXSS($str, $replace = '<!-- sb-xss-removed -->')
{
	if ($replace == '')
		$replace = '<xss>';

	$str = preg_replace('/([\x00-\x08][\x0b-\x0c][\x0e-\x19])/', '', $str);

	$search = '/&#[xX]0{0,8}(21|22|23|24|25|26|27|28|29|2a|2b|2d|2f|30|31|32|33|34|35|36|37|38|39|3a|3b|3d|3f|40|41|42|43|44|45|46|47|48|49|4a|4b|4c|4d|4e|4f|50|51|52|53|54|55|56|57|58|59|5a|5b|5c|5d|5e|5f|60|61|62|63|64|65|66|67|68|69|6a|6b|6c|6d|6e|6f|70|71|72|73|74|75|76|77|78|79|7a|7b|7c|7d|7e);?/ie';
	$str = preg_replace($search, "chr(hexdec('\\1'))", $str);
	$search = '/&#0{0,8}(33|34|35|36|37|38|39|40|41|42|43|45|47|48|49|50|51|52|53|54|55|56|57|58|59|61|63|64|65|66|67|68|69|70|71|72|73|74|75|76|77|78|79|80|81|82|83|84|85|86|87|88|89|90|91|92|93|94|95|96|97|98|99|100|101|102|103|104|105|106|107|108|109|110|111|112|113|114|115|116|117|118|119|120|121|122|123|124|125|126);?/ie';
	$str = preg_replace($search, "chr('\\1')", $str);

	$ra1 = array('javascript', 'vbscript', 'expression', 'applet', 'meta', 'xml', 'blink', 'link', 'style', 'script', 'embed', 'object', 'iframe', 'frame', 'frameset', 'ilayer', 'layer', 'bgsound', 'title', 'base', 'onabort', 'onactivate', 'onafterprint', 'onafterupdate', 'onbeforeactivate', 'onbeforecopy', 'onbeforecut', 'onbeforedeactivate', 'onbeforeeditfocus', 'onbeforepaste', 'onbeforeprint', 'onbeforeunload', 'onbeforeupdate', 'onblur', 'onbounce', 'oncellchange', 'onchange', 'onclick', 'oncontextmenu', 'oncontrolselect', 'oncopy', 'oncut', 'ondataavailable', 'ondatasetchanged', 'ondatasetcomplete', 'ondblclick', 'ondeactivate', 'ondrag', 'ondragend', 'ondragenter', 'ondragleave', 'ondragover', 'ondragstart', 'ondrop', 'onerror', 'onerrorupdate', 'onfilterchange', 'onfinish', 'onfocus', 'onfocusin', 'onfocusout', 'onhelp', 'onkeydown', 'onkeypress', 'onkeyup', 'onlayoutcomplete', 'onload', 'onlosecapture', 'onmousedown', 'onmouseenter', 'onmouseleave', 'onmousemove', 'onmouseout', 'onmouseover', 'onmouseup', 'onmousewheel', 'onmove', 'onmoveend', 'onmovestart', 'onpaste', 'onpropertychange', 'onreadystatechange', 'onreset', 'onresize', 'onresizeend', 'onresizestart', 'onrowenter', 'onrowexit', 'onrowsdelete', 'onrowsinserted', 'onscroll', 'onselect', 'onselectionchange', 'onselectstart', 'onstart', 'onstop', 'onsubmit', 'onunload');
	$ra_tag = array('applet', 'meta', 'xml', 'blink', 'link', 'style', 'script', 'embed', 'object', 'iframe', 'frame', 'frameset', 'ilayer', 'layer', 'bgsound', 'title', 'base');
	$ra_attribute = array('style', 'onabort', 'onactivate', 'onafterprint', 'onafterupdate', 'onbeforeactivate', 'onbeforecopy', 'onbeforecut', 'onbeforedeactivate', 'onbeforeeditfocus', 'onbeforepaste', 'onbeforeprint', 'onbeforeunload', 'onbeforeupdate', 'onblur', 'onbounce', 'oncellchange', 'onchange', 'onclick', 'oncontextmenu', 'oncontrolselect', 'oncopy', 'oncut', 'ondataavailable', 'ondatasetchanged', 'ondatasetcomplete', 'ondblclick', 'ondeactivate', 'ondrag', 'ondragend', 'ondragenter', 'ondragleave', 'ondragover', 'ondragstart', 'ondrop', 'onerror', 'onerrorupdate', 'onfilterchange', 'onfinish', 'onfocus', 'onfocusin', 'onfocusout', 'onhelp', 'onkeydown', 'onkeypress', 'onkeyup', 'onlayoutcomplete', 'onload', 'onlosecapture', 'onmousedown', 'onmouseenter', 'onmouseleave', 'onmousemove', 'onmouseout', 'onmouseover', 'onmouseup', 'onmousewheel', 'onmove', 'onmoveend', 'onmovestart', 'onpaste', 'onpropertychange', 'onreadystatechange', 'onreset', 'onresize', 'onresizeend', 'onresizestart', 'onrowenter', 'onrowexit', 'onrowsdelete', 'onrowsinserted', 'onscroll', 'onselect', 'onselectionchange', 'onselectstart', 'onstart', 'onstop', 'onsubmit', 'onunload');
	$ra_protocol = array('javascript', 'vbscript', 'expression');

	$str2 = preg_replace('/(&#[xX]?0{0,8}(9|10|13|a|b);)*\s*/i', '', $str);
	$ra = array();

	foreach ($ra1 as $ra1word)
	{
		if (stripos($str2, $ra1word) !== false )
		{
			if (in_array($ra1word, $ra_protocol))
			{
				$ra[] = array($ra1word, 'ra_protocol');
			}
			if (in_array($ra1word, $ra_tag))
			{
				$ra[] = array($ra1word, 'ra_tag');
			}
			if (in_array($ra1word, $ra_attribute))
			{
				$ra[] = array($ra1word, 'ra_attribute');
			}
		}
	}

	if (count($ra) > 0)
	{
		$found = true;
		while ($found == true)
		{
			$str_before = $str;
			for ($i = 0; $i < count($ra); $i++)
			{
				$pattern = '';
				for ($j = 0; $j < strlen($ra[$i][0]); $j++)
				{
					if ($j > 0)
					{
						$pattern .= '((&#[xX]0{0,8}([9ab]);)|(&#0{0,8}(9|10|13);)|\s)*';
					}
					$pattern .= $ra[$i][0][$j];
				}

				switch ($ra[$i][1])
				{
					case 'ra_protocol':
						$pattern .= '((&#[xX]0{0,8}([9ab]);)|(&#0{0,8}(9|10|13);)|\s)*(?=:)';
						break;
					case 'ra_tag':
						$pattern = '(?<=<)' . $pattern . '((&#[xX]0{0,8}([9ab]);)|(&#0{0,8}(9|10|13);)|\s)*(?=[^\da-z])';
						break;
					case 'ra_attribute':
						$pattern .= '[\s\!\#\$\%\&\(\)\*\~\+\-\_\.\,\:\;\?\@\[\/\|\\\\\]\^\`]*(?==)';
						break;
				}
				$pattern = '/' . $pattern . '/i';
				$replacement = substr_replace($ra[$i][0], $replace, 2, 0);

				$str = preg_replace($pattern, $replacement, $str);
				if ($str_before == $str)
				{
					$found = false;
				}
			}
		}
	}

	return $str;
}

/**
 * Добавляет пробелы спереди и в конце переданной строки
 *
 * Используется для вывода облака тегов.
 *
 * @param string $item Строка, к которой добавляются пробелы.
 */
function sbProgSpacer(&$item)
{
	$item = ' '.$item.' ';
}

/**
 * Добавляет клики к баннеру.
 *
 */
function sbProgBannerClick()
{
	$robot = sbIsSearchRobot();
	if($robot)
    {
		return;
	}

	if (isset($_GET['bid']) && $_GET['bid'] != '')
	{
		$res = sql_query('SELECT sb_id, sb_link, sb_statistics FROM sb_banners WHERE sb_id = ?d', $_GET['bid']);
		if($res)
		{
			list($sb_id, $sb_link, $sb_statistics) = $res[0];

			if ($sb_statistics == 1)
			{
				$today = time();
				$year = sb_date('Y', $today);
				$month = sb_date('n', $today);
				$day = sb_date('j', $today);

				sql_query('UPDATE sb_banners_statistics SET sb_count_clicks = sb_count_clicks + 1 WHERE sb_bid=?d AND sb_year = ?d AND sb_month = ?d AND sb_day = ?d', $sb_id, $year, $month, $day);
			}
		}
	}

	if(isset($sb_link) && trim($sb_link) != '')
	{
		header('Location: '.sb_sanitize_header($sb_link));
	}
	else
	{
		header('Location: '.sb_sanitize_header($_SERVER['PHP_SELF']));
	}

	exit(0);
}

/**
 * Возвращает название города по IP-адресу
 *
 * Функция использует сервис ipgeobase (http://www.ipgeobase.ru)
 *
 * @param string $ip IP-адрес.
 * @param bool $reset_cache Использовать кэш или нет.
 * @param string $charset Кодировка, в которую надо перекодировать ответ сервиса
 *
 * @return array Название города, региона, области, страны, широта и долгота или FALSE.
 * 				 Структура массива 'city', 'region', 'district', 'country', 'lat', 'lng'.
 */
function sbProgGetCityByIP($ip = '', $reset_cache = false, $charset = '')
{
	$info = false;
	if(!isset($_COOKIE['sb_geolocation']) || $reset_cache || trim($_COOKIE['sb_geolocation']) == '')
	{
		require_once(SB_CMS_LIB_PATH.'/sbDownload.inc.php');

		$httpRequest = new sbDownload('http://www.ipgeobase.ru:7020/geo?ip='.$ip);
    	$result = $httpRequest->download();

        if ($result != '')
		{
            if ($charset != '' && sb_strtolower($charset) != 'windows-1251')
            {
                $result = iconv('windows-1251', $charset, $result);
                $result = sb_str_replace('Windows-1251', $charset, $result);
            }

			$xml = simplexml_load_string($result);
			if ($xml && isset($xml->ip) && isset($xml->ip->city))
			{
				$info = array();
				$info['country'] = (string) $xml->ip->country;
				$info['city'] = (string) $xml->ip->city;
				$info['region'] = (string) $xml->ip->region;
				$info['district'] = (string) $xml->ip->district;
				$info['lat'] = (string) $xml->ip->lat;
				$info['lng'] = (string) $xml->ip->lng;

				sb_setcookie('sb_geolocation', base64_encode(serialize($info)), time() + 3600 * 24 * 7);
			}
		}

		if ($info === false)
		{
			sb_setcookie('sb_geolocation', 'undefined', time() + 3600 * 24);
		}
	}
	else
	{
		if (trim($_COOKIE['sb_geolocation']) != 'undefined')
		{
		    $cookie = base64_decode(str_replace(' ', '+', ($_COOKIE['sb_geolocation'])));

		    if ($cookie)
		    {
		        $info = unserialize(stripslashes($cookie));
		    }
		}
	}

	return $info;
}

/**
 *
 * Функция определяет является ли тот кто зашел на страницу поисковым роботом.
 * Если переменная $_SERVER['HTTP_USER_AGENT'] не найдена, то функция все равно считает посетителя роботом.
 *
 * @return bool Да, является поисковым роботом (TRUE) или не является роботом (FALSE)
 *
 */
function sbIsSearchRobot()
{
//	если нет user_agent-а то полюбому считаем посетителя роботом.
	if(!isset($_SERVER['HTTP_USER_AGENT']))
	{
		return true;
	}

	$robots = sql_query('SELECT sr_robot FROM sb_search_robots');
	if($robots)
    {
	    foreach ($robots[0] as $robot)
	    {
	        if(isset($_SERVER['HTTP_USER_AGENT']) && $_SERVER['HTTP_USER_AGENT'] != '' && stristr($_SERVER['HTTP_USER_AGENT'], $robot))
	        {
	            return true;
	        }
		}
	}
	return false;
}

/**
 * Функция генерации хэша для строки
 *
 * @param string $str Строка, для которой генерируем хэш.
 */
function sbProgGetHash($str)
{
    $str = $str.sbPlugins::getSetting('sb_site_hash');
    $len = 100 + sb_strlen($str);

    $result = md5($str).sha1($str);

    for($i = 0; $i < $len; $i++)
        $result = md5($result);

    return $result;
}
?>