<?php
/**
 * Реализация класса для полностраничного кеширования
 * @author Artur Fursa
 */
class sbCachePage {

	private static $_Instance = null;

	private $_memcacheInstance = false;

	/**
	 * Запрос пользователя
	 */
	private $_userQuery = null;

	/**
	 * Ключ Memcached
	 */
	private $_key = null;

	/**
	 * Использовать кеш или нет
	 */
	private $_useCachePage = false;

	/**
	 * Хост Memcached
	 */
	private $_memcacheHost = false;

	/**
	 * Порт Memcached
	 */
	private $_memcachePort = false;

	/**
	 * Время валидности кеша в секундах
	 */
	private $_cachePageTime = 0;

	/**
	 * Запрещенные фрагменты урл
	 */
	private $_restrictedPages = null;

	/**
	 * Запрещенные GET-параметры
	 */
	private $_restrictedParams = null;

	/**
	 * Запрещенные куки
	 */
	private $_restrictedCookies = array('sb_page_nocache');

	/**
	 * Куки, значения которых необходимо учесть при формировании кеша
	 */
	private $_usedCookie = array('sb_user_id');

	private function __construct()
	{

		$this -> _useCachePage = sbPlugins::getSetting('sb_use_cache_page');
		//если кеширование включено, то вытаскиваем остальные настройки
		if ($this -> _useCachePage)
		{
		    if(isset($_SESSION['sbAuth']))
            {
                $cVal = base64_encode($_SESSION['sbAuth'] -> getUserId());
                sb_setcookie('sb_user_id', $cVal, '/');
                $_COOKIE['sb_user_id'] = $cVal;
            }
            
			$this -> _userQuery = SB_DOMAIN . $_SERVER['REQUEST_URI'];
			$this -> _cachePageTime = intval(sbPlugins::getSetting('sb_use_cache_page_time'));
			$this -> _memcacheHost = sbPlugins::getSetting('sb_use_mcache_host');
			$this -> _memcachePort = sbPlugins::getSetting('sb_use_mcache_port');

			if (trim(sbPlugins::getSetting('sb_use_cache_page_rpage')) != '')
			{
				$this -> _restrictedPages = $this -> fillSettings(sbPlugins::getSetting('sb_use_cache_page_rpage'));
			}
			if (trim(sbPlugins::getSetting('sb_use_cache_page_rget')) != '')
			{
				$this -> _restrictedParams = $this -> fillSettings(sbPlugins::getSetting('sb_use_cache_page_rget'));
			}
			if (trim(sbPlugins::getSetting('sb_use_cache_page_rcookie')) != '')
			{
				$this -> _restrictedCookies = array_merge($this -> _restrictedCookies, $this -> fillSettings(sbPlugins::getSetting('sb_use_cache_page_rcookie')));
			}
			if (trim(sbPlugins::getSetting('sb_use_cache_page_use_cookie')) != '')
			{
				$this -> _usedCookie = array_merge($this -> _usedCookie, $this -> fillSettings(sbPlugins::getSetting('sb_use_cache_page_use_cookie')));
			}
		}
	}

	private function __clone()
	{
	}

	public static function getInstance()
	{
		if (self::$_Instance === null)
		{
			self::$_Instance = new sbCachePage;
		}

		return self::$_Instance;
	}

	/**
	 * Проверяет наличие кешированных данных для текущего урл с учетом ограничений
	 */
	public function check()
	{
		if ($this -> checkCacheSettings($this -> _userQuery))
		{
			if ($this -> getmemcacheInstance())
			{
				$html = $this -> getmemcacheInstance() -> get($this -> getKey());
				if ($html)
				{
					echo $html;
					exit();
				}
			}
		}
	}

	/**
	 * Выводит содержимое текущей страницы в браузер и выполняет сохранение в кеш
	 * @param String $html HTML-код страницы
	 */
	public function save($html)
	{
		if ($this -> checkCacheSettings($this -> _userQuery))
		{
			if ($this -> getmemcacheInstance())
			{
				$this -> getmemcacheInstance() -> set($this -> getKey(), $html, false, $this -> _cachePageTime); 
			}
		}

		echo $html;
	}

	/**
	 * Возвращает экземпляр соединенния с сервером Memcached
	 */
	private function getmemcacheInstance()
	{
		if (!$this -> _memcacheInstance)
		{
			$this -> _memcacheInstance = new Memcache;
			$memcacheConnect = $this -> _memcacheInstance -> connect($this -> _memcacheHost, $this -> _memcachePort);
			if (empty($memcacheConnect))// не доступен мемкеш-сервер
			{
				return false;
			}
			$this -> _memcacheInstance -> setCompressThreshold(1000000, 0);
		}

		return $this -> _memcacheInstance;
	}

	/**
	 * Формирует уникальный ключ для текущего урл с учетом дополнительных настроек
	 */
	private function getKey()
	{
		if ($this -> _key === null)
		{
			$dopStr = '';

			if ($this -> _usedCookie !== null)
			{
				foreach ($this->_usedCookie as $key => $value)
				{
					if (isset($_COOKIE[$value]))
					{
						$dopStr .= $_COOKIE[$value];
					}
				}
			}

			$this -> _key = rtrim(base64_encode(pack("H*", md5($this -> _userQuery . $dopStr))), "=");
		}

		return $this -> _key;
	}

	/**
	 * Проверяем нужно-ли кешировать переданный урл
	 */
	private function checkCacheSettings()
	{
		if (!$this -> _useCachePage)
		{
			// мемкеш не включен в настройках
			return false;
		}
		elseif (!class_exists('Memcache'))
		{
			// не доступен мемкеш-модуль
			return false;
		}

		//запрещаем кеширование при POST-запросах
		if (count($_POST) > 0)
		{
			return false;
		}

		//проверяем вхождение фрагментов урл в строке запроса
		if ($this -> _restrictedPages !== null)
		{
			foreach ($this->_restrictedPages as $val)
			{
				if (sb_substr_count($this -> _userQuery, $val))
				{
					return false;
				}
			}
		}

		//проверяем наличие запрещенных get-ов
		if ($this -> _restrictedParams !== null && count($_GET) > 0)
		{
			foreach ($this->_restrictedParams as $val)
			{
				if (array_key_exists($val, $_GET))
				{
					return false;
				}
			}
		}

		//проверяем наличие запрещенных кук
		if ($this -> _restrictedCookies !== null && count($_COOKIE) > 0)
		{
			foreach ($this->_restrictedCookies as $val)
			{
				if (array_key_exists($val, $_COOKIE))
				{
					return false;
				}
			}
		}

		return true;
	}

	/**
	 * Метод-хелпер для корректного считывания настроек
	 * @param String $params строка настроек с разделителем
	 * @param String $glue разделитель значений в строке, по умолчанию ","
	 * @return Array массив значений
	 */
	private function fillSettings($params, $glue = ',')
	{
		$data = explode($glue, $params);
		foreach ($data as $key => $val)
		{
			$data[$key] = trim($val);
		}

		return $data;
	}
}
