<?php
/**
 * Реализация класса, отвечающего за вывод дерева разделов и связанного с разделом произвольного HTML-вывода
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Categs
 * @copyright Copyright (c) 2008, OOO "СИБИЭС Групп"
 */

/**
 * @ignore
 */
require_once(SB_CMS_LIB_PATH.'/sbJustCategs.inc.php');

/**
 * Класс, отвечающий за вывод дерева разделов и связанного с разделом произвольного HTML-вывода
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Categs
 * @copyright Copyright (c) 2008, OOO "СИБИЭС Групп"
 */

class sbCategs extends sbJustCategs 
{
    /**
     * Имя функции, которая вызывается для получения HTML-текста каждого из разделов
     * 
     * В качестве параметра функции передается ID выбранного раздела.
     *
     * @var string
     */
    private $mElemsGetFunc = '';
       
    /**
     * Ширина панели разделов по умолчанию
     *
     * @var int
     */
    public $mElemsPanelWidth = '300';
    
    /**
     * HTML-код, выводимый над содержимым раздела и панелью с деревом разделов
     * 
     * @var string
     */
    public $mHeaderStr = '';
    
    /**
     * HTML-код, выводимый под содержимым раздела и панелью с деревом разделов
     * 
     * @var string
     */
    public $mFooterStr = '';
    
    /**
     * HTML-код, выводимый над содержимым раздела
     *
     * @var string
     */
    public $mElemsHeaderStr = '';
    
    /**
     * HTML-код, выводимый под содержимым раздела
     *
     * @var string
     */
    public $mElemsFooterStr = '';

    /**
     * JavaScript-функция, вызываемая после загрузки содержимого раздела
     *
     * @var string
     */
    public $mElemsAfterLoadFunc = '';
    
    /**
     * Конструктор класса
     *
     * @param string $get_func Имя функции, которая вызывается для получения HTML-текста каждого из разделов. 
     *                         В качестве параметра функции передается ID выбранного раздела.
     * @param string $event Событие, вызываемое при клике по разделу. В связанной с этим событием функции происходит создание 
     *                      класса sbCategs.
     * @param string $ident Идентификатор разделов (обычно имя модуля).
     */
    public function __construct($get_func, $event, $ident = '')
    {
        sbJustCategs::__construct($ident == '' ? $event : $ident);

        $this->mElemsGetFunc = $get_func;
        $this->setCategsEvent($event);
        $this->mCategsCount = false;
    }
     
    /**
     * Вывод списка разделов, инициализация списка разделов, вывод всего необходимого JavaScript-кода
     * 
     * Вызывается каждый раз при подгрузке содержимого раздела.
     * JavaScript-код выводится только при первом обращении к содержимому раздела.
     */
    public function init($show = true)
    {
        if (basename($_SERVER['PHP_SELF']) == 'empty.php')
        {
            $this->show();
            return;
        }
        
        $this->mCategsPasteElemsMenu = false;
        $this->mCategsPasteLinksMenu = false;

        sbJustCategs::init(false);
        
        echo '<script src="'.SB_CMS_JSCRIPT_URL.'/sbCategs.js.php"></script>';
        
        echo $this->mHeaderStr;
              
        if ($this->mElemsHeaderStr != '')
        {
            echo '<div id="elems_list_header" style="visibility:hidden;">'.$this->mElemsHeaderStr.'</div>';
        }
            
        echo '<table width="100%" cellpadding="0" cellspacing="0" id="elems_list_table" style="visibility:hidden;margin-top: 1px;">
              <tr>
                  <td valign="top" style="width:100%;">
                      <div id="elems_list">&nbsp;</div>
                  </td>
                  <td class="categs_drag_border" onmousedown="sbCategsDragDel(event, true);">
                    <img src="'.SB_CMS_IMG_URL.'/blank.gif" width="6" height="1" border="0">
                  </td>
                  <td id="categs_list_td">
                    <div id="categs_list"></div>
                  </td>
              </tr></table>';

        if ($this->mElemsFooterStr != '')
        {
            echo '<div id="elems_list_footer" style="visibility:hidden;">'.$this->mElemsFooterStr.'</div>';
        }

        echo '<div id="categs_drag_div"></div>
              <script>';

        if ($this->mElemsAfterLoadFunc != '')
            echo 'var sb_elems_after_load_func = '.$this->mElemsAfterLoadFunc.';';

        echo '  var panel_width = sbGetCookie("sb_panel_width");
                if (!panel_width)
                    panel_width = "'.$this->mElemsPanelWidth.'";
              </script>';
        
        echo $this->mFooterStr;
    }
  
    /**
     * Вывод содержимого раздела
     * 
     * Вызывается каждый раз при подгрузке содержимого раздела.
     *
     */
    function show($view = true)
    {
        if (isset($_GET['id']) && !empty($_GET['id']))
	    {
	        $_SESSION['sb_categs_selected_id'][$this->mCategsIdent] = intval($_GET['id']);
		    $this->mCategsSelectedId = intval($_GET['id']);
	    }

	    $res = sql_query('SELECT cat_level FROM sb_categs WHERE cat_id=?d', $this->mCategsSelectedId);
	    if (!$res)
	    {
	        sb_show_message(SB_ELEMS_NO_CATEG_MSG, true, 'warning');
            return;
	    }

        $show = true;
	    if ($this->mCategsSelectedId != -1)
	    {
            if ($this->mCategsUseRights)
            {
                $show = fCategs_Check_Rights($this->mCategsSelectedId);
            }
	    }
	    else
	    {
	        $show = false;
	    }

	    list($cat_level) = $res[0];
	    
        if (!$show)
        {
             sb_show_message(SB_ELEMS_DENY_MSG, true, 'warning');
             return;
        }

        if (function_exists($this->mElemsGetFunc))
        {
            call_user_func($this->mElemsGetFunc, array('id' => intval($_GET['id'])));
        }
        else 
        {
            sb_show_message(SB_ELEMS_NO_FUNC_MSG, true);
        }
    }
}
