<?php
/**
 * Реализация класса работы с перетаксиваемыми областями на главной странице
 *
 * Перетаскиваемые области добавляются для каждого модуля методом addToMainPage класса sbPlugins.
 *
 * @see $_SESSION['sbPlugins']
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_ActiveDesktop
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */

/**
 * Класс для работы с перетаксиваемыми областями на главной странице
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_ActiveDesktop
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbContainers
{
    /**
     * Контейнеры из настроек пользователя
     *
     * @var int
     */
    private $mContainers = array();

    /**
     * Кол-во столбцов
     *
     * @var int
     */
    private $mColumn = 3;

    /**
     * Конструктор класса
     *
     * @return sbContainers
     */
    public function __construct()
    {
       	$this->mContainers = sbPlugins::getUserSetting('sb_containers');
		if (is_null($this->mContainers))
		{
			$this->mContainers = array();
		}
		
    	$this->mColumn = max(1, sbPlugins::getUserSetting('sb_containers_count'));
    }

    /**
     * Возвращает HTML-код конкретного контейнера
     *
     * @param string $event Событие, вызываемое для получения содержимого контейнера через AJAX.
     * @param string $title Заголовок контейнера.
     * @param string $icon Иконка контейнера.
     * @param bool $roll Сверный или развернутый контейнер.
     * @return string HTML-код контейнера.
     */
    private function getContainer($event, $title, $icon, $roll, $i, $j)
    {
        $result = '<div id="'.$event.'" class="sbDS_div" row_num="'.$i.'" col_num="'.$j.'">
                    <table cellpadding="0" cellspacing="0" width="100%" class="sbDS_block">
                        <tr class="sbDS_title_tr">
                            <td class="sbDS_title" onmousedown="sbDS.mouseDown(\''.$event.'\', event)">
                                <nobr>'.($icon != '' ? '<img src="'.$icon.'" class="sbDS_image" align="absmiddle">' : '').$title.'</nobr>
                            </td>
                            <td align="right" valign="top">
                                <div class="sbDS_icons"><nobr>
                                    <img src="'.SB_CMS_IMG_URL.'/reload.gif" onclick="sbDS.reloadPage(\''.$event.'\')" /><img src="'.SB_CMS_IMG_URL.'/'.($roll? 'maximize.gif' : 'minimize.gif').'" id="sbDS_minmax_'.$event.'" onclick="sbDS.minmax(\''.$event.'\')" /><img src="'.SB_CMS_IMG_URL.'/close.gif" onclick="sbDS.close(\''.$event.'\')" />
                                </nobr></div>
                            </td>
                        </tr>
                        <tr>
                            <td colspan="2" width="100%" valign="top">
                                <div class="sbDS_content" id="sbDS_content_'.$event.'" style="display: '.($roll? 'none' : 'block').'"><br><center><img src="'.SB_CMS_IMG_URL.'/loading.gif"></center><br></div>
                            </td>
                        </tr>
                    </table>
                </div>';
        
        return $result;
    }

    /**
     * Выводит список контейнеров
     *
     * @param bool $echo Выводить HTML-код или вернуть в виде строки.
     * @return mixed Строка с HTML-кодом, если указан параметр $echo = true и void в ином случае.
     */
    function show($echo = true)
    {
    	// массив выводимых контейнеров из настроек
        $output_ar = sbPlugins::getUserSetting('sb_containers_ar');
        if ($this->mColumn == 0 || $output_ar == '' || count($output_ar) == 0)
        {
        	
            sb_show_message(SB_CONTAINERS_NO_ITEMS, 'true');
            return;
        }
        
        // зарегистрированные в системе контейнеры
        $reg_ar = $_SESSION['sbPlugins']->mContainers;
        
        // выводимые контейнеры
        $ar = array();

        $c = 0; // столбец
        $r = 0; // ряд

        if (count($output_ar) != 0)
        {
            // есть контейнеры для вывода
            if(count($this->mContainers) == 0 || (count($this->mContainers) > 0 && count($this->mContainers[0]) != $this->mColumn))
            {
                // изменилось кол-во столбцов, либо первый вход в систему
                // перестраиваем контейнеры заново
                for ($i = 0; $i < count($reg_ar); $i++)
                {
                    if (!in_array($reg_ar[$i]['event'], $output_ar))
                        continue;
    
                    if ($c >= $this->mColumn)
                    {
                        $r++;
                        $c = 0;
                    }
    
                    $ar[$r][$c] = $reg_ar[$i];
                    $c++;
                }
            }
            else
            {
                $empty_idx = array();
                // выводим только контейнеры, зарегистрированные в системе
                foreach ($this->mContainers as $r => $col)
                {
                    foreach ($col as $c => $val)
                    {
                        if (count($val) == 0)
                        {
                            $empty_idx[$r] = $c;
                            $ar[$r][$c] = array();
                            continue;
                        }
                        
                        $el = false;
                        for ($k = 0; $k < count($reg_ar); $k++)
                        {
                            if ($reg_ar[$k]['event'] == $val['event'] && in_array($val['event'], $output_ar))
                            {
                                $el = $reg_ar[$k];
                                $el['roll'] = $val['roll'];

                                $pos = array_search($val['event'], $output_ar);
                                array_splice($output_ar, $pos, 1);
                                break;
                            }
                        }
    
                        if ($el)
                        {
                            $ar[$r][$c] = $el;
                        }
                        else
                        {
                            $empty_idx[$r] = $c;
                            $ar[$r][$c] = array();
                        }
                    }
                }
                $c++;

                if (count($output_ar) > 0)
                {
                    // в настройках включили отображение одного или нескольких контейнеров
                    foreach($output_ar as $val)
                    {
                        for ($k = 0; $k < count($reg_ar); $k++)
                        {
                            if ($reg_ar[$k]['event'] == $val)
                            {
                            	$idx = each($empty_idx);
                                if ($idx)
                                {
                                    $ar[$idx['key']][$idx['value']] = $reg_ar[$k];
                                }
                                else 
                                {
                                    if ($c >= $this->mColumn)
                                    {
                                        $r++;
                                        $c = 0;
                                    }
        
                                    $ar[$r][$c] = $reg_ar[$k];
                                    $c++;
                                }
                                break;
                            }
                        }
                    }
                }
            }
        
            if ($c <= $this->mColumn - 1)
            {
                for ($i = $c; $i < $this->mColumn; $i++)
                {
                    $ar[$r][$i] = array();
                }
            }
        }

        if (isset($ar[0]))
        {
            // упаковываем массив
            $num_i = count($ar);
            $num_j = count($ar[0]);
            $max_i = 0;
            for ($j = 0; $j < $num_j; $j++)
            {
                $n = 0;
                for ($i = 0; $i < $num_i; $i++)
                {
                    if (count($ar[$i][$j]) == 0)
                    {
                        $r_i = $i;
                        $i++;
                        while ($i < $num_i)
                        {
                            if (count($ar[$i][$j]) != 0)
                            {
                                $ar[$r_i][$j] = $ar[$i][$j];
                                $ar[$i][$j] = array();
                                $n++;
                                $i--;
                                break;
                            }
                            $i++;
                        }
                    }
                    else 
                    {
                        $n++;
                    }
                }
                
                if ($n > $max_i)
                    $max_i = $n;
            }
            
            if ($num_i > $max_i)
            {
                for ($i = $max_i; $i < $num_i; $i++)
                {
                    unset($ar[$i]);
                }
            }
        }
        
        // сохраняем полученный расклад в настройках
        $_SESSION['sbPlugins']->setUserSetting('sb_containers', $ar);

        $html_string = '
            <script src="'.SB_CMS_JSCRIPT_URL.'/sbContainers.js.php"></script>
            <script>
                sbDS.num_capacity = '.$this->mColumn.';
            </script>
            <link rel="stylesheet" type="text/css" href="'.SB_CMS_CSS_URL.'/sbContainers.css">';

        if (count($ar) != 0)
        {
            $html_string .= '<script>';
            
            $m = 0;
            $cols = array();
            
            for($i = 0; $i < count($ar); $i++)
            {
                for($j = 0; $j < count($ar[$i]); $j++)
                {
                    if ($i == 0)
                        $cols[$j] = '';
                         
                    if(isset($ar[$i][$j]) && count($ar[$i][$j]) > 0)
                    {
                        $html_string .= 'sbDS.array_container['.$m.'] = "'.$ar[$i][$j]['event'].'";';
                        $m++;
                                         
                        $cols[$j] .= $this->getContainer($ar[$i][$j]['event'], $ar[$i][$j]['title'], $ar[$i][$j]['icon'], $ar[$i][$j]['roll'], $i, $j);
                    }
                }
            }

            $html_string .= '</script><table cellpadding="0" cellspacing="0" height="auto" width="100%" id="sbDS_main"><tr>';
            
            foreach ($cols as $i => $col_str)
            {
                $html_string .= '<td valign="top" id="sbDS_capacity_'.$i.'" align="center" width="'.intval(100 / $this->mColumn).'%" class="sbDS_capacity">'.$col_str.'</td>';
            }
            
            $html_string .= '</tr></table>';
        }

        if ($echo)
        {
            echo $html_string;
            return $html_string;
        }
        
        return $html_string;
    }
}
?>