<?php
/**
 * Реализация класса, отвечающего за подгрузку дерева пользовательских разделов.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Categs
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */

/**
 * Класс, отвечающий за подгрузку дерева пользовательских разделов.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Categs
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbCustomCategs
{
    /**
     * Массив пунктов в дереве
     * 
     * Каждый пункт - это объект sbCustomCategsItem.
     *
     * @var array
     */
    public $mItems = array();
    
    /**
     * JavaScript-код, выводимый перед инициализацией дерева разделов
     *
     * @var string
     */
    public $mCategsJavascriptStr = '';
    
    /**
     * Идентификатор выбранного раздела
     *
     * @var int
     */
    public $mCategsSelectedId = '';
    
    /**
     * Выводить или нет пунктирные линии у дерева
     *
     * @var bool
     */
    public $mCategsTreeLines = true;
    
    /**
     * Относительный путь к картинкам разделов
     *
     * Путь указывается относительно SB_CMS_IMG_URL.
     * 
     * @var string
     */
    public $mImagePath = '/tree/';
    
    /**
     * Уникальный идентификатор дерева.
     *
     * @var string
     */
    private $mCategsIdent = '';
    
    /**
     * Ширина картинок разделов (px)
     * 
     * Все картинки у всех разделов должны быть одинаковой ширины.
     *
     * @var int
     */
    public $mImageWidth = 16;
    
    /**
     * Высота картинок разделов (px)
     * 
     * Все картинки у всех разделов должны быть одинаковой высоты.
     *
     * @var int
     */
    public $mImageHeight = 16;
    
    /**
     * Конструктор класса
     * 
     * @param string $ident Идентификатор дерева.
     */
    public function __construct($ident = '')
    {
        if (isset($_GET['ident']))
        {
            $this->mCategsSelectedId = preg_replace('/[^A-Za-z0-9_]+/', '', $_GET['ident']);
        }
        elseif (isset($_COOKIE['sb_custom_categs_'.$ident]))
        {
            $this->mCategsSelectedId = $_COOKIE['sb_custom_categs_'.$ident];
        }

        $this->mCategsIdent = $ident;
    }
    
    /**
     * Метод добавляет раздел.
     *
     * @param string $itemId Уникальное идентификационное имя раздела. Может содержать буквы латинского алфавита, цифры и знак подчеркивания. 
     * @param string $itemTitle Название раздела.
     * @param string $itemEvent Событие, отвечающее за вывод HTML в основную область при клике по разделу.
     * @param string $itemIcon Название файла, содержащего иконку раздела. Если передана пустая строка, то иконка не выводится.
     * @param string $itemDopStr Доп. строка. Выводится после имени раздела в скобках.
     */
    public function addItem($itemId, $itemTitle, $itemEvent, $itemIcon = '', $itemDopStr = '')
    {
        $itemId = preg_replace('/[^A-Za-z0-9_]+/', '', $itemId);
        if ($itemId == '')
            return;
            
        if ($this->mCategsSelectedId == '')
            $this->mCategsSelectedId = $itemId;

        if (!isset($this->mItems[$itemId]))
        {
            $this->mItems[$itemId] = new sbCustomCategsItem($itemId, $itemTitle, $itemEvent, $itemIcon, $itemDopStr);
        }
    }

    /**
     * Инициализация переменных класса и вывод JavaScript-кода для работы с разделами
     */
    public function init()
    {
    	if (!isset($this->mItems[$this->mCategsSelectedId]))
    	{
    		$this->mCategsSelectedId = key($this->mItems);	
    	}
    	
        $this->mImageWidth = intval($this->mImageWidth).'px';
        $this->mImageHeight = intval($this->mImageHeight).'px';
        
        if (basename($_SERVER['PHP_SELF']) == 'empty.php')
        {
            $this->show();
            return;
        }
        
        echo '
            <link rel="stylesheet" type="text/css" href="'.SB_CMS_CSS_URL.'/sbCategs.css">
            <script src="'.SB_CMS_JSCRIPT_URL.'/sbXMLLoader.js.php"></script>
            <script src="'.SB_CMS_JSCRIPT_URL.'/sbTree.js.php"></script>
            <script src="'.SB_CMS_JSCRIPT_URL.'/sbCustomCategs.js.php"></script>
            <script>
                '.$this->mCategsJavascriptStr.'
                sb_categs_selected_id = "'.$this->mCategsSelectedId.'";
                sb_categs_ident = "'.$this->mCategsIdent.'";
                
                function sbLoadCategs()
                {
                    sbCatTree=new sbTree(sbGetE("categs_list"),"100%","100%",0);
                    sbCatTree.checkBoxes=false;
                    sbCatTree.dragAndDrop=false;
                    sbCatTree.treeLinesOn='.($this->mCategsTreeLines ? 'true' : 'false').';
                    sbCatTree.def_img_x="'.$this->mImageWidth.'";
                    sbCatTree.def_img_y="'.$this->mImageHeight.'";
                    sbCatTree.imPath = sb_cms_img_url + "'.$this->mImagePath.'";
                    sbCatTree.aFunc = sbCategsClick;
                    sbCatTree.loadXML(sb_cms_empty_file + "?event='.$_GET['event'].'", sbCategsAfterLoad);
                }
    
                sbAddEvent(window, "load", sbLoadCategs);
            </script>';
    }
    
    /**
     * Выводит XML-код дерева
     *
     */
    private function show()
    {
	    echo '<?xml version="1.0" encoding="'.SB_CHARSET.'"?>
        <tree id="0">';
        
        foreach ($this->mItems as $value)
        {
            echo $value->getItem();
        }
        
        echo '</tree>';
    }
}

/**
 * Класс, отвечающий за вывод раздела в дереве.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Categs
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbCustomCategsItem
{
    /**
     * Уникальное идентификационное имя раздела
     *
     * @var string
     */
    private $mId = '';
    
    /**
     * Название раздела
     *
     * @var string
     */
    private $mTitle = '';
    
    /**
     * Название файла, содержащего иконку раздела
     * 
     * Если передана пустая строка, то иконка не выводится.
     *
     * @var string
     */
    private $mIcon = '';
    
    /**
     * Событие, отвечающее за вывод HTML в основную область при клике по разделу
     *
     * @var string
     */
    private $mEvent = '';
    
    /**
     * Доп. строка. Выводится после имени раздела в скобках.
     *
     * @var string
     */
    private $mDopStr = '';

    /**
     * Массив подпунктов раздела
     * 
     * Каждый подпункт - это объект sbCustomCategsItem.
     *
     * @var array
     */
    public $mItems = array();

    /**
     * Конструктор объекта.
     *
     * @param string $itemId Уникальное идентификационное имя раздела.
     * @param string $itemTitle Название раздела.
     * @param string $itemEvent Событие, отвечающее за вывод HTML в основную область при клике по разделу.
     * @param string $itemIcon Название файла, содержащего иконку раздела. Если передана пустая строка, то иконка не выводится.
     * @param string $itemDopStr Доп. строка. Выводится после имени раздела в скобках.
     */
    public function __construct($itemId, $itemTitle, $itemEvent, $itemIcon='', $itemDopStr='')
    {
        $this->mId = $itemId;
        $this->mTitle = str_replace('"', '&quot;', sb_htmlspecialchars($itemTitle, ENT_NOQUOTES));
        $this->mIcon = $itemIcon;
        $this->mEvent = $itemEvent;
        $this->mDopStr = str_replace('"', '&quot;', sb_htmlspecialchars($itemDopStr, ENT_NOQUOTES));
    }

    /**
     * Метод добавляет раздел.
     *
     * @param string $itemId Уникальное идентификационное имя раздела.
     * @param string $itemTitle Название раздела.
     * @param string $itemEvent Событие, отвечающее за вывод HTML в основную область при клике по разделу.
     * @param string $itemIcon Название файла, содержащего иконку раздела. Если передана пустая строка, то иконка не выводится.
     * @param string $itemDopStr Доп. строка. Выводится после имени раздела в скобках.
     */
    function addItem($itemId, $itemTitle, $itemEvent, $itemIcon='', $itemDopStr='')
    {
        $itemId = preg_replace('/[^A-Za-z0-9_]+/', '', $itemId);
        if ($itemId == '')
            return;
            
        if (!isset($this->mItems[$itemId]))
        {
            $this->mItems[$itemId] = new sbCustomCategsItem($itemId, $itemTitle, $itemEvent, $itemIcon, $itemDopStr);
        }
    }

    /**
     * Возвращает XML для пункта раздела
     *
     * @param string $sel_item_id Идентификатор выделенного раздела.
     * 
     * @return string XML-код пункта раздела. 
     */
    function getItem()
    {
        $textReturn = '<item text="'.$this->mTitle.($this->mDopStr != '' ? ' &lt;span id=\\\'item_dop'.$this->mId.'\\\'&gt;'.sb_htmlspecialchars($this->mDopStr).'&lt;/span&gt;' : '').'" id="'.$this->mId.'"'.($this->mIcon != '' ? ' im0="'.$this->mIcon.'" im1="'.$this->mIcon.'" im2="'.$this->mIcon.'"' : '').'><userdata name="evt">'.sb_htmlspecialchars($this->mEvent).'</userdata>';
        if (count($this->mItems) > 0)
        {
            foreach ($this->mItems as $value)
            {
                $textReturn .= $value->getItem();
            }
        }
        
        $textReturn .= '</item>';

        return $textReturn;
    }
}
?>