<?php
/**
 * Реализация класса, отвечающего за вывод дерева пользовательских разделов и подгрузку контента раздела.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Categs
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */

/**
 * @ignore
 */
require_once(SB_CMS_LIB_PATH.'/sbCustomCategs.inc.php');

/**
 * Класс выводит дерево пользовательских разделов, созданных классом sbCustomCategs
 *
 * Этот класс создаёт 2 области. Одна содержит перечень разделов, вторая выводит их содержимое.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Categs
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbCustomElements extends sbCustomCategs
{
    /**
     * Невидимый HTML-код, выводимый над списком элементов и панелью с деревом разделов
     *
     * @var string
     */
    public $mHeaderHiddenStr = '';

    /**
     * HTML-код, выводимый над списком элементов и панелью с деревом разделов
     *
     * @var string
     */
    public $mHeaderStr = '';

    /**
     * HTML-код, выводимый под списком элементов и панелью с деревом разделов
     *
     * @var string
     */
    public $mFooterStr = '';

    /**
     * Видимый HTML-код, выводимый над списком элементов и панелью с деревом разделов
     *
     * @var string
     */
    public $mElemsHeaderStr = '';

    /**
     * Видимый HTML-код, выводимый под списком элементов и панелью с деревом разделов
     *
     * @var string
     */
    public $mElemsFooterStr = '';

    /**
     * Ширина панели с деревом разделов по умолчанию
     *
     * @var int
     */
    public $mElemsPanelWidth = 300;

    /**
     * Конструктор класса
     *
     * @param string $categs_ident Уникальный идентификатор дерева.
     *
     * @return object sbCustomElements
     */
    public function __construct($categs_ident)
    {
        sbCustomCategs::__construct($categs_ident);
    }

    /**
     * Выводит две области, одна отображает дерево разделов, в другую производится подгрузка содержимого раздела
     *
     */
    public function init()
    {
        sbCustomCategs::init();

        if (basename($_SERVER['PHP_SELF']) == 'empty.php')
        {
            return;
        }

        echo $this->mHeaderHiddenStr;

        if ($this->mHeaderStr != '')
            echo '<div id="list_header">'.$this->mHeaderStr.'</div>';

        echo '<link rel="stylesheet" type="text/css" href="'.SB_CMS_CSS_URL.'/sbElements.css">
              <script src="'.SB_CMS_JSCRIPT_URL.'/sbCustomElements.js.php"></script>
              <script>
              	  sbGetE("event_content").style.overflow="hidden";
              </script>';

        if ($this->mElemsHeaderStr != '')
        {
            echo '<div id="elems_list_header" style="visibility:hidden;">'.$this->mElemsHeaderStr.'</div>';
        }

        echo '<table width="100%" cellpadding="0" cellspacing="0" id="elems_list_table" style="visibility:hidden;margin-top: 1px;">
              <tr>
                  <td valign="top" style="width:100%;">
                      <div id="elems_list">&nbsp;</div>
                  </td>
                  <td class="categs_drag_border" onmousedown="sbCategsDragDel(event, true);">
                    <img src="'.SB_CMS_IMG_URL.'/blank.gif" width="6" height="1" border="0">
                  </td>
                  <td id="categs_list_td">
                    <div id="categs_list"></div>
                  </td>
              </tr></table>';

        if ($this->mElemsFooterStr != '')
        {
            echo '<div id="elems_list_footer" style="visibility:hidden;">'.$this->mElemsFooterStr.'</div>';
        }

        echo '<div id="categs_drag_div"></div>
              <script>
                var panel_width = sbGetCookie("sb_panel_width");
                if (!panel_width)
                    panel_width = "'.$this->mElemsPanelWidth.'";
              </script>';

        if ($this->mFooterStr != '')
            echo '<div id="list_footer">'.$this->mFooterStr.'</div>';
    }
}
?>