<?php
/**
 * Реализация класса, отвечающего за вывод дерева разделов и списка связанных элементов.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Categs
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */

/**
 * @ignore
 */
require_once(SB_CMS_LIB_PATH.'/sbJustCategs.inc.php');

/**
 * Класс, отвечающий за вывод дерева разделов и списка связанных элементов
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Categs
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbElements extends sbJustCategs
{
    /**
     * Поля таблицы элементов, которые будут передаваться в функцию $mElemsGetFunc
     *
     * Добавляются методом addField().
     *
     * @var array
     */
    private $mElemsFields = array();

    /**
     * Имя функции, которая вызывается для получения HTML-текста каждого из элементов
     *
     * Первые два параметра функци всегда ID элемента и его название, остальные параметры задаются
     * в переменной класса $mElemsFields.
     *
     * @var string
     */
    private $mElemsGetFunc = '';

    /**
     * Поля таблицы элементов, которые будут участвовать в фильтре
     *
     * Добавляются методом addFilter().
     * <code>
     * // $field - название поля таблицы элементов
     * // $name - название пункта фильтра
     * $this->mElemsFilterFields[$field]['name'] = $name;
     * // $type - тип поля фильтра (NUMBER, STRING, BOOL, DATE, SELECT, MULTYSELECT, CHECKBOX)
     * $this->mElemsFilterFields[$field]['type'] = $type;
     * // $values - для типов SELECT, MULTYSELECT двумерный массив значений (значение - описание)
     * $this->mElemsFilterFields[$field]['values'] = $values;
     * </code>
     *
     * @var array
     */
    private $mElemsFilterFields = array();

    /**
     * Пользовательские поля таблицы элементов, которые будут участвовать в фильтре
     *
     * @var array
     */
    private $mElemsFilterUserFields = array();

    /**
     * Описание пользовательских полей таблицы элементов
     *
     * @var array
     */
    private $mElemsUserFields = array();

    /**
     * Поля таблицы элементов, которые будут участвовать в контекстном меню сортировки
     *
     * Добавляются методом addSorting().
     * <code>
     * // $field - поле таблицы элементов
     * // $name - название пункта контекстного меню
     * $this->mElemsSortingFields[$field] = $name;
     * </code>
     *
     * @var array
     */
    private $mElemsSortingFields = array();

    /**
     * Поля таблицы элементов, которые следует "занулять" при копировании
     *
     * Перечисляются через запятую.
     *
     * @var string
     */
    private $mElemsNullFields = '';

    /**
     * Возможно ли создание ссылок на элементы
     *
     * TRUE - возможно, FALSE - нет.
     *
     * @var bool
     */
    public $mElemsUseLinks = false;

    /**
     * Кол-во элементов на странице (для постраничного вывода)
     *
     * Считывается из настроек интерфейса системы, может быть переопределено после вызова конструктора
     * класса.
     *
     * @var int
     */
    public $mElemsPerPage = 15;

    /**
     * Отображать или нет иконки элементов
     *
     * TRUE - отображать, FALSE - нет.
     *
     * @var bool
     */
    public $mShowIcons = true;

    /**
     * URL иконки элемента
     *
     * Если не указан и включен вывод иконок, то класс пытается определить иконку модуля и
     * использует ее.
     *
     * @var string
     */
    public $mElemsIcon = '';

    /**
     * Поле таблицы элементов, используемое для сортировки по умолчанию
     *
     * @var string
     */
    public $mElemsSortField = '';

    /**
     * ID выбранного элемента по умолчанию
     *
     * @var int
     */
    public $mElemsSelectedId = -1;

    /**
     * JavaScript-функция, вызываемая при клике по элементу
     *
     * @var string
     */
    public $mElemsClickFunc = '';

    /**
     * Ширина панели разделов по умолчанию
     *
     * @var int
     */
    public $mElemsPanelWidth = '300';

    /**
     * JavaScript-код, выводимый перед инициализацией списка элементов
     *
     * @var string
     */
    public $mElemsJavascriptStr = '';

    /**
     * HTML-код, выводимый над списком элементов и панелью с деревом разделов
     *
     * @var string
     */
    public $mHeaderStr = '';

    /**
     * HTML-код, выводимый под списком элементов и панелью с деревом разделов
     *
     * @var string
     */
    public $mFooterStr = '';

    /**
     * HTML-код, выводимый над списком элементов
     *
     * @var string
     */
    public $mElemsHeaderStr = '';

    /**
     * HTML-код, выводимый под списком элементов
     *
     * @var string
     */
    public $mElemsFooterStr = '';

    /**
     * JavaScript-функция, вызываемая после загрузки списка элементов
     *
     * @var string
     */
    public $mElemsAfterLoadFunc = '';

    /**
     * JavaScript-функция, вызываемая после редактирования элемента
     *
     * @var string
     */
    public $mElemsAfterEditFunc = '';

    /**
     * JavaScript-функция, вызываемая после добавления элемента
     *
     * @var string
     */
    public $mElemsAfterAddFunc = '';

    /**
     * JavaScript-функция, вызываемая перед удалением элемента
     *
     * В качестве аргумента получает ID раздела, из которого производится удаление.
     * Массив удаляемых элементов хранится в глобальном массиве sbSelectedEls.
     *
     * @var string
     */
    public $mElemsBeforeDeleteFunc = '';

    /**
     * JavaScript-функция, вызываемая после удаления элемента
     *
     * @var string
     */
    public $mElemsAfterDeleteFunc = '';

    /**
     * Массив доп. пунктов меню для списка элементов
     *
     * <code>
     * // текст пункта меню
     * $this->mElemsAdditionalMenu[$i]['item'] = $name;
     * // JavaScript-функция, вызываемая при щелчке по пункту меню
     * $this->mElemsAdditionalMenu[$i]['func'] = $func;
     * // true - скрывать пункт из списка, если меню вызывается не для конкретного элемента
     * // false - показывать пункт всегда
     * $this->mElemsAdditionalMenu[$i]['hide'] = $hide;
     * // уровень разделов, с которого виден пункт меню
     * $this->mElemsAdditionalMenu[$i]['level'] = $level;
     * </code>
     *
     * @var array
     */
    private $mElemsAdditionalMenu = array();

    /**
     * Выводить или нет контекстное меню элементов
     *
     * @var bool
     */
    public $mElemsMenu = true;

    /**
     * Выводить или нет пункт меню "Вырезать элементы"
     *
     * @var bool
     */
    public $mElemsCutMenu = true;

    /**
     * Выводить или нет пункт меню "Копировать элементы"
     *
     * @var bool
     */
    public $mElemsCopyMenu = true;

    /**
     * Выводить или нет пункт меню "Выделить все"
     *
     * @var bool
     */
    public $mElemsSelectAllMenu = true;

    /**
     * Выводить или нет пункт меню "Редактировать элемент"
     *
     * @var bool
     */
    public $mElemsEditMenu = true;

    /**
     * Выводить или нет пункт меню "Добавить элемент"
     *
     * @var bool
     */
    public $mElemsAddMenu = true;

    /**
     * Выводить или нет пункт меню "Удалить элементы"
     *
     * @var bool
     */
    public $mElemsDeleteMenu = true;

    /**
     * Название пункта меню "Вырезать элементы"
     *
     * @var bool
     */
    public $mElemsCutMenuTitle = SB_ELEMS_CUT_MENU;

    /**
     * Название пункта меню "Копировать элементы"
     *
     * @var bool
     */
    public $mElemsCopyMenuTitle = SB_ELEMS_COPY_MENU;

    /**
     * Название пункта меню "Редактировать элемент"
     *
     * @var bool
     */
    public $mElemsEditMenuTitle = SB_ELEMS_EDIT_MENU;

    /**
     * Название пункта меню "Добавить элемент"
     *
     * @var bool
     */
    public $mElemsAddMenuTitle = SB_ELEMS_ADD_MENU;

    /**
     * Название пункта меню "Удалить элементы"
     *
     * @var bool
     */
    public $mElemsDeleteMenuTitle = SB_ELEMS_DELETE_MENU;

    /**
     * Уровень разделов, начиная с которого выводится контекстное меню элементов
     *
     * @var int
     */
    public $mElemsMenuLevel = 1;

    /**
     * Уровень разделов, начиная с которого выводится пункт меню "Вырезать элементы"
     *
     * @var int
     */
    public $mElemsCutMenuLevel = 1;

    /**
     * Уровень разделов, начиная с которого выводится пункт меню "Копировать элементы"
     *
     * @var int
     */
    public $mElemsCopyMenuLevel = 1;

    /**
     * Уровень разделов, начиная с которого выводится пункт меню "Редактировать элемент"
     *
     * @var int
     */
    public $mElemsEditMenuLevel = 1;

    /**
     * Уровень разделов, начиная с которого выводится пункт меню "Добавить элемент"
     *
     * @var int
     */
    public $mElemsAddMenuLevel = 1;

    /**
     * Уровень разделов, начиная с которого выводится пункт меню "Удалить элементы"
     *
     * @var int
     */
    public $mElemsDeleteMenuLevel = 1;

    /**
     * Событие, вызываемое перед редактированием элемента
     *
     * Если возвращает не пустую строку, то возвращенное значение показывается пользователю
     * и окно редактирование не вызывается.
     *
     * Получаемые GET-параметры:
     * <ul>
     * <li>id - ID редактируемого элемента</li>
     * <li>cat_id - ID раздела, в котором располагается редактируемый элемент</li>
     * <li>cat_level - уровень раздела, в котором располагается редактируемый элемент</li>
     * <li>cat_closed - закрытый или открытый раздел (1 - закрытый, 0 - открытый)</li>
     * </ul>
     *
     * @var string
     */
    public $mElemsBeforeEditEvent = '';

    /**
     * Событие, вызываемое при редактировании элемента
     *
     * Данное событие генерирует вывод окна редактирования.
     *
     * Получаемые GET-параметры:
     * <ul>
     * <li>id - ID редактируемого элемента</li>
     * <li>cat_id - ID раздела, в котором располагается редактируемый элемент</li>
     * <li>cat_level - уровень раздела, в котором располагается редактируемый элемент</li>
     * <li>cat_closed - закрытый или открытый раздел (1 - закрытый, 0 - открытый)</li>
     * </ul>
     *
     * @var string
     */
    public $mElemsEditEvent = '';

    /**
     * Ширина окна редактирования элементов
     *
     * @var int
     */
    public $mElemsEditDlgWidth  = 500;

    /**
     * Высота окна редактирования элементов
     *
     * @var int
     */
    public $mElemsEditDlgHeight = 400;

    /**
     * Cобытие, вызываемое после редактирования элемента
     *
     * Если возвращает не пустую строку, то возвращенное значение показывается пользователю.
     *
     * Получаемые GET-параметры:
     * <ul>
     * <li>id - ID редактируемого элемента</li>
     * <li>cat_id - ID раздела, в котором располагается редактируемый элемент</li>
     * <li>cat_level - уровень раздела, в котором располагается редактируемый элемент</li>
     * <li>cat_closed - закрытый или открытый раздел (1 - закрытый, 0 - открытый)</li>
     * </ul>
     *
     * @var string
     */
    public $mElemsAfterEditEvent = '';

    /**
     * Cобытие, вызываемое перед добавлением элемента
     *
     * Если возвращает не пустую строку, то возвращенное значение показывается пользователю и окно добавления не вызывается.
     *
     * Получаемые GET-параметры:
     * <ul>
     * <li>cat_id - ID раздела, в который добавляется элемент</li>
     * <li>cat_level - уровень раздела, в который добавляется элемент</li>
     * <li>cat_closed - закрытый или открытый раздел (1 - закрытый, 0 - открытый)</li>
     * </ul>
     *
     * @var string
     */
    public $mElemsBeforeAddEvent = '';

    /**
     * Событие, вызываемое при добавлении элемента
     *
     * Данное событие генерирует окно добавления элемента.
     *
     * Получаемые GET-параметры:
     * <ul>
     * <li>cat_id - ID раздела, в который добавляется элемент</li>
     * <li>cat_level - уровень раздела, в который добавляется элемент</li>
     * <li>cat_closed - закрытый или открытый раздел (1 - закрытый, 0 - открытый)</li>
     * </ul>
     *
     * @var string
     */
    public $mElemsAddEvent = '';

    /**
     * Ширина окна добавления элементов
     *
     * @var int
     */
    public $mElemsAddDlgWidth  = 500;

    /**
     * Высота окна добавления элементов
     *
     * @var int
     */
    public $mElemsAddDlgHeight = 400;

    /**
     * Событие, вызываемое после добавления элемента
     *
     * Если возвращает не пустую строку, то возвращенное значение показывается пользователю.
     *
     * Получаемые GET-параметры:
     * <ul>
     * <li>id - ID добавленного элемента</li>
     * <li>cat_id - ID раздела, в который добавляется элемент</li>
     * <li>cat_level - уровень раздела, в который добавляется элемент</li>
     * <li>cat_closed - закрытый или открытый раздел (1 - закрытый, 0 - открытый)</li>
     * </ul>
     *
     * @var string
     */
    public $mElemsAfterAddEvent = '';

    /**
     * Дополнительное событие, вызываемое при удалении элемента
     *
     * Используется для удаления связанных с элементами записей из других таблиц, удалять сами элементы в этом событии
     * не надо. Если возвращает не пустую строку, то возвращенное значение показывается пользователю и элемент не удаляется.
     *
     * Получаемые GET-параметры:
     * <ul>
     * <li>id - ID удаляемого элемента</li>
     * <li>cat_id - ID раздела, из которого удаляется элемент</li>
     * <li>cat_level - уровень раздела, из которого удаляется элемент</li>
     * <li>cat_closed - закрытый или открытый раздел (1 - закрытый, 0 - открытый)</li>
     * </ul>
     *
     * @var string
     */
    public $mElemsDeleteEvent = '';

    /**
     * Конструктор класса
     *
     * @param string $table Имя таблицы, содержащей связанные с разделом элементы (таблица элементов).
     * @param string $id_field Название поля таблицы элементов, в котором хранятся ID элементов (обязательно AUTO_INCREMENT).
     * @param string $title_field Название поля таблицы элементов, в котором хранятся названия элементов.
     * @param string $get_func Имя функции, которая вызывается для получения HTML-текста каждого из элементов.
     *                         Первые два параметра функци всегда ID элемента и его название, остальные параметры задаются
     *                         в методе класса addField().
     * @param string $event Событие, вызываемое при клике по разделу. В связанной с этим событием функции происходит создание
     *                      класса sbElements.
     * @param string $ident Идентификатор разделов (обычно имя модуля).
     * @param string $null_fields Поля таблицы элементов, которые следует "занулять" при копировании (через запятую).
     */
    public function __construct($table, $id_field, $title_field, $get_func, $event, $ident='', $null_fields='')
    {
        sbJustCategs::__construct($ident == '' ? $event : $ident);

        $this->mElemsTable = $table;
        $this->mElemsIdField = $id_field;
        $this->mElemsTitleField = $title_field;
        $this->mElemsGetFunc = $get_func;

        $this->setCategsEvent($event);

        $this->mElemsFields[] = 'elems.'.$id_field;
        $this->mElemsFields[] = 'elems.'.$title_field;

        $this->selectElemsPerPage();

        $this->mElemsNullFields = $null_fields;

        // фильтр по пользователю системы
        $uId = $_SESSION['sbAuth']->getUserId();
        $usersArray = array();
        $usersArray[$uId] = SB_AUTH_USER_OWN_ELEMS;
        
        //если доступен модуль пользователей сайта, то вытаскиваем данные
        if($_SESSION['sbPlugins']->isRightAvailable('pl_users', 'read'))
        {
            $queryUsers = '
                SELECT u.u_id AS id, cat.cat_title as ctitle, 
                    IF (LENGTH(u.u_name) > 0, u.u_name, u.u_login) AS title
                FROM sb_users u, sb_catchanges ch, sb_categs cat, sb_catlinks l 
                WHERE 
                    u.u_id = l.link_el_id AND l.link_cat_id = cat.cat_id AND cat.cat_ident = "pl_users" AND l.link_src_cat_id = 0 AND 
                    u.u_id != ?d AND u.u_id = ch.change_user_id AND ch.action = "add" AND ch.cat_ident = ?
                GROUP BY id ORDER BY ctitle, title'; 
            $res = sql_assoc($queryUsers, $uId, $this->mCategsIdent);
            if($res)
            {
               $catTitle = '';
               foreach ($res as $user) {
                   if($user['ctitle'] != $catTitle)
                   {
                       $usersArray['gr_'.$user['id']] = '-'.$user['ctitle'];
                       $catTitle = $user['ctitle'];
                   }
                   
                   $usersArray[$user['id']] = $user['title'];
               }
            }
        }
        
        $this->addFilter(SB_AUTH_USER_LABEL, 'change_user_id', 'select', $usersArray);
    }

    /**
     * Добавление пунктов меню сортировки
     *
     * @param string $name Название пункта меню.
     * @param string $field Имя поля таблицы элементов, по которому будет вестись сортировка.
     */
    public function addSorting($name, $field)
    {
        $this->mElemsSortingFields[$field] = $name;
    }

    /**
     * Добавление полей фильтра
     *
     * @param string $name Выводимое название поля.
     * @param string $field Поле таблицы элементов, для которого будет установлен фильтр.
     * @param string $type Тип поля (NUMBER, STRING, BOOL, DATE, SELECT, MULTYSELECT, CHECKBOX).
     * @param array $values Для типов SELECT, MULTYSELECT двумерный массив значений (значение - описание).
     */
    public function addFilter($name, $field, $type, $values=array())
    {
        $type = strtoupper($type);
        $this->mElemsFilterFields[$field] = array();
        $this->mElemsFilterFields[$field]['name'] = $name;
        $this->mElemsFilterFields[$field]['type'] = $type;
        $this->mElemsFilterFields[$field]['values'] = $values;
        if ($type == 'NUMBER' || $type == 'DATE')
        {
            $this->mElemsFilterFields[$field]['value'] = array();
            $this->mElemsFilterFields[$field]['value']['lo'] = '';
            $this->mElemsFilterFields[$field]['value']['hi'] = '';
        }
        elseif ($type == 'MULTYSELECT')
        {
            $this->mElemsFilterFields[$field]['value'] = array();
        }
        elseif ($type == 'SELECT' || $type == 'CHECKBOX')
        {
            $this->mElemsFilterFields[$field]['value'] = -1;
        }
        else
        {
            $this->mElemsFilterFields[$field]['value'] = '';
        }
    }

    /**
     * Добавление полей, передаваемых в функцию $mElemsGetFunc, отвечающую за вывод каждого элемента
     *
     * @param string $field
     */
    public function addField($field)
    {
        $this->mElemsFields[] = 'elems.'.$field;
    }

    /**
     * Добавляет доп. пункт в контекстное меню списка элементов
     *
     * @param string $name Название пункта меню.
     * @param string $func JavaScript-функция, вызываемая при щелчке по пункту меню.
     * @param bool $hide TRUE Скрывать пункт из списка, если меню вызывается не для конкретного элемента, FALSE - показывать пункт всегда.
     * @param int $level Уровень разделов, начиная с которого виден пункт меню.
     */
    public function addElemsMenuItem($name, $func, $hide=true, $level=1)
    {
        $i = count($this->mElemsAdditionalMenu);
        $this->mElemsAdditionalMenu[$i] = array();
        $this->mElemsAdditionalMenu[$i]['item'] = str_replace('"', '\\"', $name);
        $this->mElemsAdditionalMenu[$i]['func'] = str_replace('"', '\\"', $func);
        $this->mElemsAdditionalMenu[$i]['hide'] = $hide;
        $this->mElemsAdditionalMenu[$i]['level'] = $level;
    }

    /**
     * Вывод списка элементов, инициализация списка элементов и разделов, вывод всего необходимого JavaScript-кода
     *
     * Вызывается каждый раз при подгрузке списка. Осуществляет пейджирование элементов в списке.
     * JavaScript-код выводится только при первом обращении к списку элементов.
     *
     */
    public function init($show = true)
    {
        if (isset($_GET['sb_sel_id']))
            $this->mElemsSelectedId = intval($_GET['sb_sel_id']);

        if ($this->mElemsSelectedId != -1)
        {
            // если нужно подсветить элемент при первой загрузке, определяем раздел, где этот элемент находится и делаем его выделенным
            $res = sql_query('SELECT categs.cat_id FROM sb_catlinks links, sb_categs categs WHERE categs.cat_ident=? AND links.link_el_id=?d AND links.link_cat_id=categs.cat_id AND links.link_src_cat_id=0 LIMIT 1', $this->mCategsIdent, $this->mElemsSelectedId);

            if ($res)
            {
                $this->mCategsSelectedId = $res[0][0];
            }
            else
            {
                // элемент не найден или нет прав доступа к разделу
                $this->mElemsSelectedId = -1;
            }
        }

        $plugin_ident = $_SESSION['sbPlugins']->getEventPlugin($this->mCategsEvent);
        if (!$_SESSION['sbPlugins']->isRightAvailable($plugin_ident, 'read'))
        {
            $this->mElemsEditMenu = false;
        }

        $elems_edit_right = $_SESSION['sbPlugins']->isRightAvailable($plugin_ident, 'elems_edit');
        if (!$elems_edit_right)
        {
            $this->mElemsCutMenu = false;
            $this->mElemsCopyMenu = false;
            $this->mElemsAddMenu = false;
        }

        if (!$_SESSION['sbPlugins']->isRightAvailable($plugin_ident, 'elems_delete'))
        {
            $this->mElemsDeleteMenu = false;
        }

        if ($this->mCategsPasteElemsMenu != 'none')
        {
            if ($this->mCategsPasteElemsMenu == 'all' && !$this->mElemsCopyMenu && !$this->mElemsCutMenu)
            {
                $this->mCategsPasteElemsMenu = 'none';
            }
            elseif ($this->mCategsPasteElemsMenu == 'copy' && !$this->mElemsCopyMenu)
            {
                $this->mCategsPasteElemsMenu = 'none';
            }
            elseif ($this->mCategsPasteElemsMenu == 'cut' && !$this->mElemsCutMenu)
            {
                $this->mCategsPasteElemsMenu = 'none';
            }
        }

        $this->mCategsPasteLinksMenu = ($this->mElemsCopyMenu && $this->mElemsUseLinks && $this->mCategsPasteLinksMenu);
        $this->mElemsSelectAllMenu = ($this->mElemsCopyMenu || $this->mElemsCutMenu || $this->mElemsDeleteMenu) && $this->mElemsSelectAllMenu;

        $this->mCategsPasteElemsMenuLevel = min($this->mCategsPasteElemsMenuLevel, $this->mElemsMenuLevel);

        sbJustCategs::init(false);

        // Добавляем в вывод пользовательские поля
        $pd_fields = array();
        $res = sql_query('SELECT pd_fields FROM sb_plugins_data WHERE pd_plugin_ident=?', $this->mCategsIdent);
        if ($res && $res[0][0] != '')
        {
            $fields = unserialize($res[0][0]);
            foreach ($fields as $value)
            {
                if ($value['info'] == 1 && isset($value['sql']) && $value['sql'] == 1)
                    $this->addField('user_f_'.$value['id']);

                if (isset($value['active']) && $value['active'] == 1 && $elems_edit_right)
                {
                    $this->addElemsMenuItem(SB_ELEMS_ACTIVE_BY.' '.$value['title'], 'sbElemsSetActive("user_f_'.$value['id'].'")', true, $this->mElemsEditMenuLevel);
                }

                $pd_fields[$value['id']] = $value;
            }
        }

        $this->mElemsUserFields = $pd_fields;

        if (isset($_GET['sort_field']) && '' != $_GET['sort_field'])
        {
            $this->mElemsSortField = $_GET['sort_field'];
            if (isset($_SESSION['sb_elems_sort_field']))
            {
                if (isset($_SESSION['sb_elems_sort_field'][$this->mCategsIdent]))
                {
                    if ($_SESSION['sb_elems_sort_field'][$this->mCategsIdent] == $this->mElemsSortField.'A')
                    {
                        $this->mElemsSortField .= 'D';
                    }
                    else if ($_SESSION['sb_elems_sort_field'][$this->mCategsIdent] == $this->mElemsSortField.'D')
                    {
                        $this->mElemsSortField .= 'A';
                    }
                    else
                    {
                        $this->mElemsSortField .= 'D';
                    }
                }
                else
                {
                    $this->mElemsSortField .= 'D';
                }

                $_SESSION['sb_elems_sort_field'][$this->mCategsIdent] = $this->mElemsSortField;
            }
            else
            {
                $this->mElemsSortField .= 'D';
                $_SESSION['sb_elems_sort_field'] = array();
                $_SESSION['sb_elems_sort_field'][$this->mCategsIdent] = $this->mElemsSortField;
            }

            sb_setcookie('sb_sort_'.$this->mCategsIdent, $this->mElemsSortField);
        }
        elseif (isset($_SESSION['sb_elems_sort_field']) && isset($_SESSION['sb_elems_sort_field'][$this->mCategsIdent]))
        {
            $this->mElemsSortField = $_SESSION['sb_elems_sort_field'][$this->mCategsIdent];
        }
        elseif (isset($_COOKIE['sb_sort_'.$this->mCategsIdent]))
        {
            $this->mElemsSortField = $_COOKIE['sb_sort_'.$this->mCategsIdent];
        }

        $filter_set = false;
        if (isset($_GET['nofilter']) && $_GET['nofilter'] == 1)
        {
            if (isset($_SESSION['sb_elems_filter']) && isset($_SESSION['sb_elems_filter'][$this->mCategsIdent]))
            {
                $_SESSION['sb_elems_filter'][$this->mCategsIdent] = array();
            }
        }
        else
        {
            if (!isset($_SESSION['sb_elems_filter']))
                $_SESSION['sb_elems_filter'] = array();

            if (!isset($_SESSION['sb_elems_filter'][$this->mCategsIdent]))
                $_SESSION['sb_elems_filter'][$this->mCategsIdent] = array();

            $found = false;
            foreach ($_GET as $key => $value)
            {
                if (substr($key, 0, 9) == 'f_user_f_')
                {
                    if (!$found)
                    {
                        $_SESSION['sb_elems_filter'][$this->mCategsIdent] = array();
                        $found = true;
                    }
                    $key = substr($key, 2);
                    $id = preg_replace('/[^0-9]+/', '', $key);
                    if (isset($pd_fields[$id]) && $pd_fields[$id]['filter'] == 1)
                    {
                        switch ($pd_fields[$id]['type'])
                        {
                            case 'date':
                            case 'number':
                                $lohi = substr($key, -2);
                                $key = substr($key, 0, -3);
                                if (!isset($this->mElemsFilterUserFields[$key]))
                                {
                                    $this->mElemsFilterUserFields[$key] = array();
                                    $this->mElemsFilterUserFields[$key]['hi'] = '';
                                    $this->mElemsFilterUserFields[$key]['lo'] = '';
                                }
                                $this->mElemsFilterUserFields[$key][$lohi] = $value;
                                $_SESSION['sb_elems_filter'][$this->mCategsIdent][$key] = $this->mElemsFilterUserFields[$key];
                                break;

                            default:
                                $this->mElemsFilterUserFields[$key] = $value;
                                $_SESSION['sb_elems_filter'][$this->mCategsIdent][$key] = $value;
                                break;
                        }
                    }
                }
                elseif (substr($key, 0, 2) == 'f_')
                {
                    if (!$found)
                    {
                        $_SESSION['sb_elems_filter'][$this->mCategsIdent] = array();
                        $found = true;
                    }

                    $key = substr($key, 2);
                    if (!isset($this->mElemsFilterFields[$key]))
                    {
                        $lohi = substr($key, -2);
                        $key = substr($key, 0, -3);
                    }
                    if (!isset($this->mElemsFilterFields[$key]))
                        continue;

                    $type = $this->mElemsFilterFields[$key]['type'];
                    switch ($type)
                    {
                        case 'NUMBER':
                        case 'DATE':
                            if (!isset($_SESSION['sb_elems_filter'][$this->mCategsIdent][$key]))
                            {
                                $_SESSION['sb_elems_filter'][$this->mCategsIdent][$key] = array();
                                $_SESSION['sb_elems_filter'][$this->mCategsIdent][$key]['hi'] = '';
                                $_SESSION['sb_elems_filter'][$this->mCategsIdent][$key]['lo'] = '';
                            }
                            $_SESSION['sb_elems_filter'][$this->mCategsIdent][$key][$lohi] = $value;
                            break;

                        default:
                            $_SESSION['sb_elems_filter'][$this->mCategsIdent][$key] = $value;
                            break;
                    }
                }
            }
        }

        if (isset($_SESSION['sb_elems_filter']) && isset($_SESSION['sb_elems_filter'][$this->mCategsIdent]) && count($_SESSION['sb_elems_filter'][$this->mCategsIdent]) > 0)
        {
            $filter_set = true;
            foreach ($_SESSION['sb_elems_filter'][$this->mCategsIdent] as $key => $value)
            {
                if (substr($key, 0, 7) == 'user_f_')
                    $this->mElemsFilterUserFields[$key] = $_SESSION['sb_elems_filter'][$this->mCategsIdent][$key];
                else
                    $this->mElemsFilterFields[$key]['value'] = $_SESSION['sb_elems_filter'][$this->mCategsIdent][$key];
            }
        }

        if (basename($_SERVER['PHP_SELF']) == 'empty.php')
        {
            $this->show();
            return;
        }

        // Добавляем данные из пользовательских полей
        if ($pd_fields)
        {
            foreach ($pd_fields as $value)
            {
                if ($value['sort'] == 1)
                    $this->addSorting(SB_ELEMS_SORT_BY.' '.$value['title'], 'user_f_'.$value['id']);
            }
        }

        echo '<link rel="stylesheet" type="text/css" href="'.SB_CMS_CSS_URL.'/sbElements.css">
        <script src="'.SB_CMS_JSCRIPT_URL.'/sbElements.js.php"></script>';

        echo $this->mHeaderStr;

        echo '<script>
            sbGetE("event_content").style.overflow="hidden";

           '.$this->mElemsJavascriptStr.'

            var sb_elems_table = "'.$this->mElemsTable.'";
            var sb_elems_id_field = "'.$this->mElemsIdField.'";
            var sb_elems_title_field = "'.$this->mElemsTitleField.'";
            var sb_elems_selected_id = '.$this->mElemsSelectedId.';
            var sb_elems_menu_level = '.$this->mElemsMenuLevel.';
            var sb_elems_cut_menu_level = '.$this->mElemsCutMenuLevel.';
            var sb_elems_copy_menu_level = '.$this->mElemsCopyMenuLevel.';
            var sb_elems_paste_menu_level = '.$this->mCategsPasteElemsMenuLevel.';
            var sb_elems_edit_menu_level = '.$this->mElemsEditMenuLevel.';
            var sb_elems_add_menu_level = '.$this->mElemsAddMenuLevel.';
            var sb_elems_delete_menu_level = '.$this->mElemsDeleteMenuLevel.';

            var sb_elems_selectall_menu = '.($this->mElemsSelectAllMenu ? 'true':'false').';
            var sb_elems_copy_menu = '.($this->mElemsCopyMenu ? 'true':'false').';
            var sb_elems_cut_menu = '.($this->mElemsCutMenu ? 'true':'false').';
            var sb_elems_sorting = '.(count($this->mElemsSortingFields) > 0 ? 'true':'false').';

            var sb_elems_before_edit_event = "'.$this->mElemsBeforeEditEvent.'";
            var sb_elems_edit_event = "'.$this->mElemsEditEvent.'";
            var sb_elems_after_edit_event = "'.$this->mElemsAfterEditEvent.'";

            var sb_elems_before_add_event = "'.$this->mElemsBeforeAddEvent.'";
            var sb_elems_add_event = "'.$this->mElemsAddEvent.'";
            var sb_elems_after_add_event = "'.$this->mElemsAfterAddEvent.'";

            var sb_elems_delete_event = "'.$this->mElemsDeleteEvent.'";

            var sb_elems_add_dlg_width = '.$this->mElemsAddDlgWidth.';
            var sb_elems_add_dlg_height = '.$this->mElemsAddDlgHeight.';
            var sb_elems_edit_dlg_width = '.$this->mElemsEditDlgWidth.';
            var sb_elems_edit_dlg_height = '.$this->mElemsEditDlgHeight.';

            var sb_elems_add_menu = [];

            var sb_elems_before_delete_func = '.($this->mElemsBeforeDeleteFunc != '' ? $this->mElemsBeforeDeleteFunc : 'null').';
            var sb_elems_after_delete_func = '.($this->mElemsAfterDeleteFunc != '' ? $this->mElemsAfterDeleteFunc : 'null').';

            var sb_elems_icon = "'.$this->mElemsIcon.'";
            var sb_elems_null_fields = "'.$this->mElemsNullFields.'";
        ';

        if (count($this->mElemsAdditionalMenu) > 0)
        {

            for ($i = 0; $i < count($this->mElemsAdditionalMenu); $i++)
            {
                echo '
                var i = sb_elems_add_menu.length;
                sb_elems_add_menu[i] = [];
                sb_elems_add_menu[i]["level"] = '.$this->mElemsAdditionalMenu[$i]['level'].';
                sb_elems_add_menu[i]["hide"] = '.($this->mElemsAdditionalMenu[$i]['hide'] ? '1':'0').';';
            }
        }

        if ($this->mElemsMenu)
        {
            echo 'sbElemsMenu = new sbCMenu("sbElemsMenu", "15#FF9933", "15#fff5e0", "highItemText", "lowItemText", 90);';
            if ($this->mCategsPasteElemsMenu) echo 'sbElemsMenu.addItem("paste", "'.$this->mCategsPasteElemsMenuTitle.'", sbElemsPaste);';
            if ($this->mCategsPasteLinksMenu) echo 'sbElemsMenu.addItem("paste_links", "'.$this->mCategsPasteLinksMenuTitle.'", sbElemsPasteLinks);';
            echo 'sbElemsMenu.addItem("paste_delimeter");';
            if ($this->mElemsCopyMenu) echo 'sbElemsMenu.addItem("copy", "'.$this->mElemsCopyMenuTitle.'", sbElemsCopy);';
            if ($this->mElemsCutMenu) echo 'sbElemsMenu.addItem("cut", "'.$this->mElemsCutMenuTitle.'", sbElemsCut);';
            if ($this->mElemsSelectAllMenu) echo 'sbElemsMenu.addItem("select_all", "'.SB_ELEMS_SELECTALL_MENU.'", sbElemsSelectAll);';
            echo 'sbElemsMenu.addItem("cutcopy_delimeter");';

            if ($this->mElemsEditMenu && $this->mElemsEditEvent != '')
            {
                echo 'sbElemsMenu.addItem("edit", "'.$this->mElemsEditMenuTitle.'", sbElemsEdit);';
            }
            else
            {
                echo 'function sbElemsEdit(e) { return; }';
            }

            if ($this->mElemsAddMenu && $this->mElemsAddEvent != '')
            {
                echo 'sbElemsMenu.addItem("add", "'.$this->mElemsAddMenuTitle.'", sbElemsAdd);';
            }

            if ($this->mElemsDeleteMenu)
            {
                echo 'sbElemsMenu.addItem("delete", "'.$this->mElemsDeleteMenuTitle.'", sbElemsDelete);';
            }
            else
            {
                echo 'function sbElemsDelete() { return; }';
            }

            if ($this->mCategsRefreshMenu)
            {
                echo 'sbElemsMenu.addItem("refresh", "'.SB_ELEMS_REFRESH_MENU.'", sbElemsRefresh);';
            }

            echo 'sbElemsMenu.addItem("addedit_delimeter");';

            if (count($this->mElemsAdditionalMenu) > 0)
            {
                for ($i = 0; $i < count($this->mElemsAdditionalMenu); $i++)
                {
                    echo 'sbElemsMenu.addItem("additional_'.$i.'", "'.$this->mElemsAdditionalMenu[$i]['item'].'", "'.$this->mElemsAdditionalMenu[$i]['func'].'");';
                }

                echo 'sbElemsMenu.addItem("additional_delimeter");';
            }

            if (count($this->mElemsSortingFields) > 0)
            {
                $i = 0;
                foreach ($this->mElemsSortingFields as $key => $value)
                {
                    echo 'sbElemsMenu.addItem("sorting_'.$i.'", "'.str_replace('"', '\"', $value).'", "sbElemsSortBy(\''.$key.'\');");';
                    $i++;
                }
            }
        }
        echo '</script>';

        $num1 = count($this->mElemsFilterFields);
        $num2 = count($this->mElemsFilterUserFields);

        if ($this->mElemsHeaderStr != '' || $num1 > 0 || $num2 > 0)
        {
            echo '<div id="elems_list_header" style="visibility:hidden;">'.$this->mElemsHeaderStr;
            if ($num1 > 0 || $num2 > 0)
            {
                $cols = 2;

                echo '<table cellpadding="0" cellspacing="0" width="100%">
                <tr>
                    <td width="98%" class="sb_filter_header">
                        <table cellpadding="0" cellspacing="0">
                        <tr>
                            <td class="sb_filter_text"><nobr><span onclick="sbElemsShowFilter();" ondblclick="sbElemsShowFilter();" style="cursor: hand;cursor: pointer;">'.SB_ELEMS_FILTER.'&nbsp;&nbsp;<img src="'.SB_CMS_IMG_URL.'/'.($filter_set != '' ? 'fil_arrow_up' : 'fil_arrow_down').'.gif" id="filter_arrow" align="absmiddle"></span></nobr></td>
                            <td class="sb_filter_on" id="sb_filter_on" style="'.($filter_set ? '' : 'visibility: hidden;').'"><nobr><img src="'.SB_CMS_IMG_URL.'/filter_on.png" align="absmiddle" width="12" height="12">&nbsp;&nbsp;'.SB_ELEMS_FILTER_ON.'&nbsp;&nbsp;<img src="'.SB_CMS_IMG_URL.'/filter_delim.gif" align="absmiddle" width="1" height="14">&nbsp;&nbsp;<span style="cursor: hand; cursor: pointer;" onclick="sbElemsRemoveFilter(event);"><img src="'.SB_CMS_IMG_URL.'/filter_off.png" align="absmiddle" width="12" height="12">&nbsp;&nbsp;'.KERNEL_CANCEL.'</span></nobr></td>
                        </tr>
                        </table>
                    </td>
                </tr>
                <tr><td valign="top">
                    <form id="filter_form" style="padding:0px;margin:0px;">
                    <div id="sb_filter" style="display:none;">
                    <table cellpadding="0" cellspacing="0" class="sb_filter_table"><tr>';

                require_once(SB_CMS_LIB_PATH.'/sbLayout.inc.php');

                $i = 0;
                foreach ($this->mElemsFilterFields as $key => $value)
                {
                    if ($i % $cols == 0 && $i != 0)
                    {
                        echo '</tr><tr>';
                    }

                    echo '<th class="sb_filter_th">'.$value['name'].':</th><td '.(($i + 1) % $cols == 0 ? 'class="sb_filter_td_noborder"' : 'class="sb_filter_td"').'><nobr>';
                    switch ($value['type'])
                    {
                        case 'NUMBER':
                            $fld_lo = new sbLayoutInput('text', $value['value']['lo'], 'f_'.$key.'_lo', 'spin_f_'.$key.'_lo', 'style="width:50px;"');
                            $fld_hi = new sbLayoutInput('text', $value['value']['hi'], 'f_'.$key.'_hi', 'spin_f_'.$key.'_hi', 'style="width:50px;"');

                            echo $fld_lo->getJavaScript();
                            echo '<table cellpadding="0" cellspacing="0"><tr><td>'.KERNEL_FROM.'&nbsp;&nbsp;</td><td>'.$fld_lo->getField().'</td><td>&nbsp;&nbsp;'.KERNEL_TO.'&nbsp;&nbsp;</td><td>'.$fld_hi->getField().'</td></tr></table>';
                            break;

                        case 'STRING':
                            $fld = new sbLayoutInput('text', $value['value'], 'f_'.$key, '', 'style="width:100%;"');
                            echo $fld->getField();
                            break;

                        case 'DATE':
                            $fld_lo = new sbLayoutDate($value['value']['lo'], 'f_'.$key.'_lo', 'date_'.$key.'_lo');
                            $fld_lo->mDropButton = true;

                            $fld_hi = new sbLayoutDate($value['value']['hi'], 'f_'.$key.'_hi', 'date_'.$key.'_hi');
                            $fld_hi->mDropButton = true;

                            echo $fld_lo->getJavaScript();
                            echo '<table cellpadding="0" cellspacing="0"><tr><td>'.KERNEL_FROM.'&nbsp;&nbsp;</td><td>'.$fld_lo->getField().'</td><td>&nbsp;&nbsp;'.KERNEL_TO.'&nbsp;&nbsp;</td><td>'.$fld_hi->getField().'</td></tr></table>';
                            break;

                        case 'SELECT':
                            $options = array('-1' => ' --- ');
                            $options = $options + $value['values'];

                            $fld = new sbLayoutSelect($options, 'f_'.$key);
                            $fld->mSelOptions[] = $value['value'];
                            echo $fld->getField();
                            break;

                        case 'MULTYSELECT':
                            $fld = new sbLayoutSelect($value['values'], 'f_'.$key, '', 'multiple="multiple" size="'.min(3, count($value['values'])).'"');
                            foreach ($value['values'] as $sub_key => $sub_value)
                            {
                                if (in_array($sub_key, $value['value']))
                                    $fld->mSelOptions[] = $sub_key;
                            }
                            echo $fld->getField();
                            break;

                        case 'CHECKBOX':
                            $options = array('-1' => ' --- ', '1' => KERNEL_YES, '0' => KERNEL_NO);
                            $fld = new sbLayoutSelect($options, 'f_'.$key);
                            $fld->mSelOptions[] = $value['value'];
                            echo $fld->getField();
                            break;
                    }
                    echo '</nobr></td>';
                    $i++;
                }

                sbLayout::getPluginFieldsFilter($pd_fields, $i, $cols, $this->mElemsFilterUserFields);

                if ($i % $cols != 0)
                {
                    for ($j = 0; $j < $cols - $i % $cols; $j++)
                        echo '<th class="sb_filter_th">&nbsp;</th><td '.(($j + 1) == $cols - $i % $cols ? 'class="sb_filter_td_noborder"' : 'class="sb_filter_td"').' width="1">&nbsp;</td>';
                }
                echo '</tr></table>
                      <div class="sb_filter_footer">
                          <input type="button" value="'.KERNEL_APPLY.'" onclick="sbElemsSetFilter();">
                      </div>
                      </div></form></td></tr></table>';
            }
            echo '</div>';
        }

        echo '<table width="100%" cellpadding="0" cellspacing="0" id="elems_list_table" style="visibility:hidden;margin-top: 1px;">
              <tr>
                  <td valign="top" style="width:100%;">
                      <div id="elems_list" oncontextmenu="sbElemsShowMenu(event);">&nbsp;</div>
                  </td>
                  <td class="categs_drag_border" onmousedown="sbCategsDragDel(event, true);">
                    <img src="'.SB_CMS_IMG_URL.'/blank.gif" width="6" height="1" border="0">
                  </td>
                  <td id="categs_list_td">
                    <div id="categs_list"></div>
                  </td>
              </tr></table>';

        if ($this->mElemsFooterStr != '')
        {
            echo '<div id="elems_list_footer" style="visibility:hidden;">'.$this->mElemsFooterStr.'</div>';
        }

        echo '<div id="categs_drag_div"></div>
              <script>';

        if ($this->mElemsAfterLoadFunc != '')
            echo 'var sb_elems_after_load_func = '.$this->mElemsAfterLoadFunc.';';

        if ($this->mElemsAfterEditFunc != '')
            echo 'var sb_elems_after_edit_func = '.$this->mElemsAfterEditFunc.';';

        if ($this->mElemsAfterAddFunc != '')
            echo 'var sb_elems_after_add_func = '.$this->mElemsAfterAddFunc.';';

        if ($this->mElemsClickFunc != '')
            echo 'var sb_elems_on_click_func = '.$this->mElemsClickFunc.';';

        echo '  var panel_width = sbGetCookie("sb_panel_width");
                if (!panel_width)
                    panel_width = "'.$this->mElemsPanelWidth.'";
              </script>';

        echo $this->mFooterStr;
    }

    /**
     * Определяем количество элементов, выводимое на странице.
     */
    private function selectElemsPerPage()
    {
        $elemsPerPage = sbPlugins::getUserSetting('sb_elems_per_page');

        $needInsert = false;
        $needUpdate = false;

        if(is_array($elemsPerPage))
        {
            if(isset($elemsPerPage[$this->mElemsTable]))
            {
                $this->mElemsPerPage = $elemsPerPage[$this->mElemsTable];
            }
        }
        elseif($elemsPerPage > 0)
        {
            $this->mElemsPerPage = $elemsPerPage;
        }
        else
        {
            $needInsert = true;
        }

        if(isset($_GET['per_page']) && (int)$_GET['per_page'] > 0)
        {
            $this->mElemsPerPage = (int)$_GET['per_page'];

            if(!is_array($elemsPerPage))
            {
                $elemsPerPage = array();
            }

            if((isset($elemsPerPage[$this->mElemsTable]) && $elemsPerPage[$this->mElemsTable] != $this->mElemsPerPage) || !isset($elemsPerPage[$this->mElemsTable])
            )
            {
                $elemsPerPage[$this->mElemsTable] = $this->mElemsPerPage;
                $needUpdate = true;
            }

            if($needInsert)
            {
                sql_query('INSERT INTO sb_users_settings (us_setting, us_value, us_user_id) VALUES (?s, ?s, ?d)', 'sb_elems_per_page', serialize($elemsPerPage), $_SESSION['sbAuth']->getUserId());
            }
            elseif($needUpdate)
            {
                sql_query('UPDATE sb_users_settings SET us_value=?s WHERE us_setting="sb_elems_per_page" AND us_user_id=?d', serialize($elemsPerPage), $_SESSION['sbAuth']->getUserId());
            }
        }
    }

    /**
     * Вывод списка элементов
     *
     * Вызывается каждый раз при подгрузке списка. Осуществляет пейджирование элементов в списке.
     *
     */
    private function show($view = true)
    {
        if (isset($_GET['id']) && !empty($_GET['id']))
        {
            $_SESSION['sb_categs_selected_id'][$this->mCategsIdent] = intval($_GET['id']);
            $this->mCategsSelectedId = intval($_GET['id']);
        }

        $res = sql_query('SELECT cat_level, cat_ident FROM sb_categs WHERE cat_id=?d', $this->mCategsSelectedId);
        if (!$res)
        {
            sb_show_message(SB_ELEMS_NO_CATEG_MSG, true, 'warning');
            echo '<span id="elems_show_menu" style="display:none;">0</span>';
            return;
        }

        $show = true;
        if ($this->mCategsSelectedId != -1)
        {
            if ($this->mCategsUseRights)
            {
                $show = fCategs_Check_Rights($this->mCategsSelectedId);
            }
        }
        else
        {
            $show = false;
        }

        list($cat_level, $cat_ident) = $res[0];

        if (!$show)
        {
             sb_show_message(SB_ELEMS_DENY_MSG, true, 'warning');
             echo '<span id="elems_show_menu" style="display:none;">0</span>';
             return;
        }
        else
        {
            echo '<span id="elems_show_menu" style="display:none;">1</span>';
        }

        // определяем поле, по которому будет идти сортировка
        $sort_fld = ' elems.'.$this->mElemsIdField.' DESC';

        if ($this->mElemsSortField != '')
        {
            $sort_fld = substr($this->mElemsSortField, 0, -1);
            $res = sql_query('SELECT ?# FROM ?# LIMIT 1', $sort_fld, $this->mElemsTable);
            if ($res)
            {
                $sort_fld = 'elems.'.$sort_fld;
                $sort_flag = substr($this->mElemsSortField, -1);

                if ($sort_flag == 'D')
                    $sort_fld = ' '.$sort_fld.' DESC';
                else
                    $sort_fld = ' '.$sort_fld.' ASC';
            }
        }

        // формируем фильтр по запросу
        $filter_flds = ''; 
        foreach ($this->mElemsFilterFields as $key => $value)
        {
            if ($value['value'] != '')
            {
                $pref = 'elems.';
                //FIXME хардкод по имени поля, возможно в будущем понадобится фильтрация по другим таблицам
                if($key == 'change_user_id')
                {
                    $pref = 'ch.';
                }
                switch ($value['type'])
                {
                    case 'NUMBER':
                        if ($value['value']['lo'] != '')
                            $filter_flds .= ' AND '.$pref.$key.'>='.floatval($value['value']['lo']);
                        if ($value['value']['hi'] != '')
                            $filter_flds .= ' AND '.$pref.$key.'<='.floatval($value['value']['hi']);
                        break;

                    case 'STRING':
                        if (sb_substr($value['value'], 0, 1) == '^')
                            $filter_flds .= ' AND '.$pref.$key.' LIKE '.$GLOBALS['sbSql']->escape(sb_substr($value['value'], 1).'%');
                        elseif (sb_substr($value['value'], -1) == '$')
                            $filter_flds .= ' AND '.$pref.$key.' LIKE '.$GLOBALS['sbSql']->escape('%'.sb_substr($value['value'], 0, -1));
                        else
                            $filter_flds .= ' AND '.$pref.$key.' LIKE '.$GLOBALS['sbSql']->escape('%'.$value['value'].'%');
                        break;

                    case 'SELECT':
                    case 'CHECKBOX':
                        if ($value['value'] != -1)
                            $filter_flds .= ' AND '.$pref.$key."='".intval($value['value'])."'";
                        break;

                    case 'MULTYSELECT':
                        $in_str = implode(',', $value['value']);
                        if ($in_str != '')
                        {
                            $filter_flds .= ' AND '.$pref.$key.' IN ('.$in_str.')';
                        }
                        break;

                    case 'DATE':
                        if ($value['value']['lo'] != '')
                        {
                            $filter_flds .= ' AND '.$pref.$key.'>='.sb_datetoint($value['value']['lo']);
                        }
                        if ($value['value']['hi'] != '')
                        {
                            $filter_flds .= ' AND '.$pref.$key.'<='.sb_datetoint($value['value']['hi']);
                        }
                        break;
                }
            }
        }

        foreach ($this->mElemsFilterUserFields as $key => $value)
        {
            if (substr($key, -5, 5) == '_link')
                continue;

            $id = preg_replace('/[^0-9]+/', '', $key);
            if (isset($this->mElemsUserFields[$id]) && $this->mElemsUserFields[$id]['filter'] == 1)
            {
                switch ($this->mElemsUserFields[$id]['type'])
                {
                    case 'number':
                        if ($value['lo'] != '')
                            $filter_flds .= ' AND elems.'.$key.'>='.floatval($value['lo']);
                        if ($value['hi'] != '')
                            $filter_flds .= ' AND elems.'.$key.'<='.floatval($value['hi']);
                        break;

                    case 'date':
                        if ($value['lo'] != '')
                            $filter_flds .= ' AND elems.'.$key.'>='.sb_datetoint($value['lo']);
                        if ($value['hi'] != '')
                            $filter_flds .= ' AND elems.'.$key.'<='.sb_datetoint($value['hi']);
                        break;

                    case 'file':
                    case 'longtext':
                    case 'image':
                    case 'string':
                    case 'link':
                    case 'text':
                    case 'php':
                        $filter_flds .= ' AND elems.'.$key.' LIKE '.$GLOBALS['sbSql']->escape('%'.$value.'%');
                        break;

                    case 'categs':
                        if ($value != -1)
                            $filter_flds .= ' AND elems.'.$key.' REGEXP "[[:<:]]'.intval($value).'[[:>:]]"';
                        break;
                    case 'checkbox':
                    case 'color':
                    case 'select_sprav':
                    case 'radio_sprav':
                    //case 'categs':
                    case 'select_plugin':
                    case 'radio_plugin':
                    case 'elems_plugin':
                        if ($value != -1)
                            $filter_flds .= ' AND elems.'.$key.'='.intval($value);
                        break;

                    case 'multiselect_sprav':
                    case 'checkbox_sprav':
                    case 'checkbox_plugin':
                    case 'multiselect_plugin':
                        if ($value != '' && count($value) != 0)
                        {
                            $filter_flds .= ' AND (';
                            foreach ($value as $sub_value)
                            {
                                $filter_flds .= 'CONCAT(",", elems.'.$key.', ",") LIKE "%,'.$sub_value.',%" AND ';
                            }
                            $filter_flds = substr($filter_flds, 0, -5).')';
                        }
                        break;

                    case 'link_sprav':
                    case 'link_plugin':
                        if (isset($this->mElemsFilterUserFields[$key.'_link']) && $this->mElemsFilterUserFields[$key.'_link'] != -1 && $this->mElemsFilterUserFields[$key.'_link'] != '')
                        {
                            $filter_flds .= ' AND elems.'.$key.'="'.intval($value).','.intval($this->mElemsFilterUserFields[$key.'_link']).'"';
                        }
                        break;
                }
            }
        }

        // формируем список полей для выборки
        $select_str = implode(', ', $this->mElemsFields);
        foreach ($this->mElemsFields as $key => $value)
        {
            $this->mElemsFields[$key] = str_replace('elems.', '', $value);
        }

        // пейджирование
        require_once(SB_CMS_LIB_PATH.'/sbDBPager.inc.php');
        // текущая страница для вывода
        $from = false;

        if (isset($_GET['sel_el_id']) && $_GET['sel_el_id'] != -1 && !empty($_GET['sel_el_id']))
        {
            // вытаскиваем страницу, на которой находится нужный нам элемент
            if(isset($this->mElemsFilterFields['change_user_id'], $this->mElemsFilterFields['change_user_id']['value']) && $this->mElemsFilterFields['change_user_id']['value'] != -1)
            {
                $query = 'SELECT elems.'.$this->mElemsIdField.', links.link_src_cat_id 
                    FROM '.$this->mElemsTable.' elems, sb_catlinks links, sb_catchanges ch 
                    WHERE links.link_el_id=elems.'.$this->mElemsIdField.' AND links.link_cat_id=?d 
                    AND links.link_el_id = ch.el_id AND ch.action = "add" AND cat_ident = "'.$this->mCategsIdent.'"
                    '.$filter_flds.' ORDER BY '.$sort_fld;
            }
            else
            {
                $query = 'SELECT elems.'.$this->mElemsIdField.', links.link_src_cat_id 
                    FROM '.$this->mElemsTable.' elems, sb_catlinks links 
                    WHERE links.link_el_id=elems.'.$this->mElemsIdField.' AND links.link_cat_id=?d '.$filter_flds.' ORDER BY '.$sort_fld;
            }
            
            $res = sql_query($query, $this->mCategsSelectedId);

            $pos = false;
            if ($res)
            {
                $pos = array_search(array($_GET['sel_el_id'], 0), $res);
            }

            if ($pos !== false)
            {
                $from = ceil(($pos + 1) / $this->mElemsPerPage);
                $_GET['page_elems'] = $from;
            }
        }

        if($from === false)
        {
            if (isset($_GET['page_elems']) && $_GET['page_elems'] != '')
                $from = intval($_GET['page_elems']);
            else
                $from = 1;
        }

        $total = true;
        $pager = new sbDBPager('elems', 7, $this->mElemsPerPage, 'content');

        require_once SB_CMS_KERNEL_PATH . '/plugins.settings.inc.php';
         
        if(isset($this->mElemsFilterFields['change_user_id'], $this->mElemsFilterFields['change_user_id']['value']) && $this->mElemsFilterFields['change_user_id']['value'] != -1)
        {
            $query = 'SELECT '.$select_str.', links.link_src_cat_id, links.link_id
                FROM sb_catlinks links, '.$this->mElemsTable.' elems, sb_catchanges ch 
                WHERE links.link_el_id=elems.'.$this->mElemsIdField.' AND links.link_cat_id=?d '.$filter_flds.' 
                AND links.link_el_id = ch.el_id AND ch.action = "add" AND cat_ident = "'.$this->mCategsIdent.'"'
                .(isset($GLOBALS['sb_dop_where'][$this->mElemsTable])? $GLOBALS['sb_dop_where'][$this->mElemsTable] : '').'
                ORDER BY '.$sort_fld;
        }
        else
        {
            $query = 'SELECT '.$select_str.', links.link_src_cat_id, links.link_id
                FROM sb_catlinks links, '.$this->mElemsTable.' elems 
                WHERE links.link_el_id=elems.'.$this->mElemsIdField.' AND links.link_cat_id=?d '.$filter_flds.
                (isset($GLOBALS['sb_dop_where'][$this->mElemsTable])? $GLOBALS['sb_dop_where'][$this->mElemsTable] : '').'
                ORDER BY '.$sort_fld;
        }
        
        $res = $pager->init($total, $query, $this->mCategsSelectedId);

        $from = $pager->mPage;

        if ($res && function_exists($this->mElemsGetFunc))
        {
            // иконки элементов
            if ($this->mElemsIcon == '' && $this->mShowIcons)
            {
                $icon = $_SESSION['sbPlugins']->getPluginIcon($this->mCategsEvent);
                if ($icon != '')
                {
                    $icon = str_replace('_24', '_32', $icon);
                    if (@file_exists(str_replace(SB_CMS_IMG_URL, SB_CMS_IMG_PATH, $icon)))
                    {
                        $this->mElemsIcon = $icon;
                    }
                }
            }

            //вывод ссылок для выбора количества элементов на странице
            $elemPerPage = '';

            echo '<div class="clearfix">';

            if($total > $pager->mElemsPerPageArray[0])
            {
                $pager->mElementPerPageTempActive = '&nbsp;<a href="javascript:sbElemsShow(sb_cms_empty_file + \'?event=\' + sb_elems_event + \'&id=\'+sbCatTree.getSelectedItemId()+\'&per_page={NUMBER}\');">{NUMBER}</a>&nbsp;';
                $pager->CurrentElementsPerPage = $this->mElemsPerPage;

                $elemPerPage = $pager->showElemsPerPageList();

                echo '<div class="smalltext" style="float:left;padding:5px;width:300px;">'.KERNEL_COUNT_ELEMS_PER_PAGE.': ';
                echo $elemPerPage;
                echo '</div>';
            }

            // вывод пейджирования
            if ($total > $this->mElemsPerPage)
                $pager_display = 'block';
            else
                $pager_display = 'none';

            echo '<div id="elems_nav_table_top" style="padding:5px 5px 5px 300px;text-align:center;display:'.$pager_display.';"><nobr>';

            $pager->mBeginTemp = '<img id="begin_btn" src="'.SB_CMS_IMG_URL.'/begin.gif" width="14" height="15" alt="'.SB_ELEMS_BEGIN.'" onclick="sbElemsShow(sb_cms_empty_file + \'?event=\' + sb_elems_event + \'&id=\'+sbCatTree.getSelectedItemId()+\'&page_elems=1\');" style="cursor: hand;cursor:pointer;" align="absmiddle">&nbsp;';
            $pager->mBeginTempDisabled = '<img id="begin_btn" src="'.SB_CMS_IMG_URL.'/begin_gray.gif" width="14" height="15" alt="'.SB_ELEMS_BEGIN.'" align="absmiddle">&nbsp;';
            $pager->mPrevTemp = '<img src="'.SB_CMS_IMG_URL.'/previous.gif" width="14" height="15" alt="'.SB_ELEMS_PREV.'" onclick="sbElemsShow(sb_cms_empty_file + \'?event=\' + sb_elems_event + \'&id=\'+sbCatTree.getSelectedItemId()+\'&page_elems='.($from - 1).'\');" style="cursor: hand;cursor:pointer;" align="absmiddle">&nbsp;';
            $pager->mPrevTempDisabled = '<img src="'.SB_CMS_IMG_URL.'/previous_gray.gif" width="14" height="15" alt="'.SB_ELEMS_PREV.'" align="absmiddle">&nbsp;';
            $pager->mNextTemp = '<img src="'.SB_CMS_IMG_URL.'/next.gif" width="14" height="15" alt="'.SB_ELEMS_NEXT.'" onclick="sbElemsShow(sb_cms_empty_file + \'?event=\' + sb_elems_event + \'&id=\'+sbCatTree.getSelectedItemId()+\'&page_elems='.($from + 1).'\');" style="cursor: hand;cursor:pointer;" align="absmiddle">&nbsp;';
            $pager->mNextTempDisabled = '<img src="'.SB_CMS_IMG_URL.'/next_gray.gif" width="14" height="15" alt="'.SB_ELEMS_NEXT.'" align="absmiddle">&nbsp;';
            $pager->mEndTemp = '<img id="begin_btn" src="'.SB_CMS_IMG_URL.'/end.gif" width="14" height="15" alt="'.SB_ELEMS_END.'" onclick="sbElemsShow(sb_cms_empty_file + \'?event=\' + sb_elems_event + \'&id=\'+sbCatTree.getSelectedItemId()+\'&page_elems={ALL_PAGES}\');" style="cursor: hand;cursor:pointer;" align="absmiddle">&nbsp;';
            $pager->mEndTempDisabled = '<img id="begin_btn" src="'.SB_CMS_IMG_URL.'/end_gray.gif" width="14" height="15" alt="'.SB_ELEMS_END.'" align="absmiddle">&nbsp;';
            $pager->mNumberTemp = '&nbsp;<a href="javascript:sbElemsShow(sb_cms_empty_file + \'?event=\' + sb_elems_event + \'&id=\'+sbCatTree.getSelectedItemId()+\'&page_elems={NUMBER}\');">{NUMBER}</a>&nbsp;';

            $nav_bar_str = $pager->show();

            echo $nav_bar_str.'</nobr></div>
                </div>
                <div align="center" id="elems_list_div" from="'.$from.'">';

            $num_fields = count($this->mElemsFields);
            $num = count($res);

            $view_info = $_SESSION['sbPlugins']->isRightAvailable('pl_users', 'read');
            $users = array();

            for ($i = 0; $i < $num; $i++)
            {
                $args = array();
                foreach ($this->mElemsFields as $key => $value)
                {
                    $args[$value] = $res[$i][$key];
                }

                $elem_str = call_user_func($this->mElemsGetFunc, $args);
                if ($elem_str == '')
                    continue;

                $res_changes = sql_query('SELECT changes.change_user_id, changes.change_date
                    FROM sb_catchanges changes
                    JOIN (
                        SELECT el_id, ? AS cat_ident, MAX( change_date ) AS change_date
                        FROM sb_catchanges
                        WHERE cat_ident = ? AND el_id = ?d
                        GROUP BY (el_id)
                    ) changes2 ON changes.el_id = changes2.el_id AND changes.cat_ident = changes2.cat_ident
                    AND changes.change_date = changes2.change_date', $cat_ident, $cat_ident, $res[$i][0]);

                if ($res_changes)
                {
                    $change_user_id = $res_changes[0][0];
                    $change_date = $res_changes[0][1];
                    $change_date = sb_date('d.m.Y '.KERNEL_IN.' H:i:s', $change_date);

                    if (isset($users[$change_user_id]))
                    {
                        $change_login = $users[$change_user_id];
                    }
                    else
                    {
                        $u_res = sql_query('SELECT u_login FROM sb_users WHERE u_id=?d', $change_user_id);
                        if ($u_res)
                        {
                            list($change_login) = $u_res[0];
                        }
                        else
                        {
                            $change_login = '-';
                        }

                        $users[$change_user_id] = $change_login;
                    }
                }
                else
                {
                    $change_date = '';
                    $change_login = '-';
                }

                $link_src_cat_id = $res[$i][$num_fields];
                $link_id = $res[$i][$num_fields + 1];

                $img = '';
                if ($this->mShowIcons && $this->mElemsIcon != '')
                {
                    $img = '<td width="1" valign="top"><img src="'.$this->mElemsIcon.'" align="absMiddle" width="32" height="32" border="0" title="ID - '.$res[$i][0].'"></td>';
                }

                echo '<div id="el_'.$link_id.'" el_id="'.$res[$i][0].'" true_id="'.$link_id.'" is_link="'.$link_src_cat_id.'" class="list_element" onclick="sbElemsClick(event);" ondblclick="sbElemsEdit(event);">
                <table cellpadding="5" cellspacing="0" width="98%">
                <tr>
                    '.$img.'
                    <td id="el_td_'.$link_id.'" class="list_element_td">'.$elem_str.'</td>
                </tr>
                <tr>
                    <td id="el_change_td_'.$link_id.'" class="list_element_date"'.($img != '' ? ' colspan="2"':'').'>'.($link_src_cat_id != 0 ? '<a href="javascript:void(0);" onclick="sbElemsHighlight(event, \''.$res[$i][0].'\');" ondblclick="sbCancelEvent(event);" class="small"><img src="'.SB_CMS_IMG_URL.'/link.png" title="'.SB_ELEMS_IS_LINK.'" align="absmiddle" border="0" width="12" height="12" /> '.SB_ELEMS_LINK_TEXT.'</a>&nbsp;|&nbsp;' : '').SB_ELEMS_LAST_MODIFIED.': '.$change_date.
                    ' ('.(($_SESSION['sbAuth']->isAdmin() || $view_info) && $change_login != '-' ? '<a href="javascript:void(0);" onclick="sbShowModalDialog(\''.SB_CMS_DIALOG_FILE.'?event=pl_kernel_user_info&id='.$change_user_id.'\', \'resizable=1,width=800,height=600\');" class="small">'.$change_login.'</a>' : $change_login).').
                    <a href="javascript:void(0);" onclick="sbShowModalDialog(\''.SB_CMS_DIALOG_FILE.'?event=pl_categs_changes&el_id='.$res[$i][0].'&cat_ident='.$cat_ident.'\', \'resizable=1,width=500,height=700\');" class="small">'.SB_ELEMS_HISTORY.'</a>.</td>
                </tr>
                </table></div>';
            }

            echo '</div><div class="clearfix" style="padding-top: 10px">';

            if($elemPerPage != '')
            {
                echo '<div class="smalltext" style="float:left;padding:5px;width:300px;">'.KERNEL_COUNT_ELEMS_PER_PAGE.': ';
                echo $elemPerPage;
                echo '</div>';
            }

            echo '<div id="elems_nav_table_bottom" style="padding:5px 5px 5px 300px; text-align:center; display:'.$pager_display.';">
                  <nobr>'.$nav_bar_str.'</nobr></div></div>';
        }
        elseif (!$res)
        {
            sb_show_message(SB_ELEMS_NO_ELEMS_MSG.($this->mElemsAddMenu && ($cat_level + 1) >= $this->mElemsMenuLevel && ($cat_level + 1) >= $this->mElemsAddMenuLevel ? '<br><br><a href="#" onclick="sbElemsAdd();">'.$this->mElemsAddMenuTitle.'</a>' : ''), true);
        }
        else
        {
            sb_show_message(SB_ELEMS_NO_FUNC_MSG, true);
        }
    }
}
