<?php
/**
 * Реализация класса, отвечающего за построение и подгрузку списка файлов в файловой панели.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_FileSystem
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */

/**
 * @ignore
 */
require_once(SB_CMS_LIB_PATH.'/sbFolders.inc.php');

/**
 * Класс, отвечающий за построение и подгрузку списка файлов в файловой панели.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_FileSystem
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbFilesList extends sbFolders
{
    /**
     * JavaScript-функция, вызываемая при щелчке по файлу
     *
     * @var string
     */
    public $mFilesClickFunc = '';

    /**
     * JavaScript-функция, вызываемая после загрузки списка файлов
     *
     * @var string
     */
    public $mFilesAfterLoadFunc = '';

    /**
     * Ширина панели с деревом папок по умолчанию
     *
     * @var int
     */
    public $mFilesPanelWidth = '300'; //ширина панели папок по умолчанию

    /**
     * Выводить или нет контекстное меню файлов
     *
     * @var bool
     */
    public $mFilesMenu = true;

    /**
     * Выводить или нет пункт меню "Вырезать файлы"
     *
     * @var bool
     */
    public $mFilesCutMenu = true;

    /**
     * Выводить или нет пункт меню "Копировать файлы"
     *
     * @var bool
     */
    public $mFilesCopyMenu = true;

    /**
     * Выводить или нет пункт меню "Переименовать файл"
     *
     * @var bool
     */
    public $mFilesRenameMenu = true;

    /**
     * Выводить или нет пункт меню "Удалить файлы"
     *
     * @var bool
     */
    public $mFilesDeleteMenu = true;

    /**
     * Выводить или нет пункт меню "Выделить все"
     *
     * @var bool
     */
    public $mFilesSelectAllMenu = true;

     /**
     * Выводить или нет пункт меню "Сортировать по ..."
     *
     * @var bool
     */
    public $mFilesSortMenu = true;

    /**
     * Флаг для сортировки по умолчанию
     *
     * @var int
     */
    public $mFilesSort = SB_VFS_SCANDIR_ORDER_TIME_DESC;

    /**
     * HTML-код, выводимый над списком файлов и панелью с деревом папок
     *
     * @var string
     */
    public $mFilesHeaderStr = '';

    /**
     * HTML-код, выводимый под списком файлов и панелью с деревом папок
     *
     * @var string
     */
    public $mFilesFooterStr = '';

    /**
     * Вспомогательный JavaScript-код (обычно содержит реализацию функций для доп. пунктов меню)
     *
     * @var string
     */
    public $mFilesJavaScriptStr = '';

    /**
     * Массив расширений файлов
     *
     * В списке файлов будут отображаться только файлы с указанными расширениями.
     *
     * @var array
     */
    public $mFilesExtensions = array();

    /**
     * Путь к выбранному файлу
     *
     * @var string
     */
    public $mFilesSelectedFile = '';

    /**
     * Выводить имя файла в виде ссылки (TRUE) или нет (FALSE)
     *
     * @var boolean
     */
    public $mFilesLinkTitle = true;

    /**
     * Массив доп. пунктов меню для списка файлов
     *
     * <code>
     * // текст пункта меню
     * $this->mFilesAdditionalMenu[$i]['item'] = ...;
     * // JavaScript-функция, вызываемая при щелчке по пункту меню
     * $this->mFilesAdditionalMenu[$i]['func'] = ...;
     * // true - скрывать пункт из списка, если меню вызывается не для конкретного файла
     * // false - показывать пункт всегда
     * $this->mFilesAdditionalMenu[$i]['hide'] = ...;
     * </code>
     *
     * @var array
     */
    private $mFilesAdditionalMenu = array();

    /**
     * Кол-во строк в таблице списка файлов
     *
     * @var int
     */
    private $mFilesRows = 10;

    /**
     * Кол-во столбцов в таблице списка файлов
     *
     * @var int
     */
    private $mFilesCols = 3;

    /**
     * Конструктор класса
     */
    public function __construct()
    {
        sbFolders::__construct();

        $this->mFilesCols = max(1, sbPlugins::getUserSetting('sb_files_num_cols'));
        $this->mFilesRows = max(1, sbPlugins::getUserSetting('sb_files_num_rows'));
    }

    /**
     * Добавляет доп. пункт в контекстное меню списка файлов
     *
     * @param string $name Название пункта меню.
     * @param string $func JavaScript-функция, вызываемая при щелчке по пункту меню.
     * @param bool $hide TRUE - скрывать пункт из списка, если меню вызывается не для конкретного файла, FALSE - показывать пункт всегда.
     */
    public function addFileMenuItem($name, $func, $hide=true)
    {
        $i = count($this->mFilesAdditionalMenu);
        $this->mFilesAdditionalMenu[$i] = array();
        $this->mFilesAdditionalMenu[$i]['item'] = $name;
        $this->mFilesAdditionalMenu[$i]['func'] = $func;
        $this->mFilesAdditionalMenu[$i]['hide'] = $hide;
    }

    /**
     * Инициализирует файловую панель, выводит JavaScript-код
     *
     * Метод вызывается только при первом обращении к файловой панели.
     *
     */
    public function init()
    {
    	if (isset($_GET['sort']) && '' != $_GET['sort'])
        {
		    $this->mFilesSort = intval($_GET['sort']);
		    if (isset($_SESSION['sb_files_sort']))
		    {
		    	switch ($this->mFilesSort)
		    	{
		    		case SB_VFS_SCANDIR_ORDER_NAME_ASC:
		    			if ($_SESSION['sb_files_sort'] == SB_VFS_SCANDIR_ORDER_NAME_ASC)
		    			{
		    				$this->mFilesSort = SB_VFS_SCANDIR_ORDER_NAME_DESC;
		    			}
		    			break;

		    		case SB_VFS_SCANDIR_ORDER_SIZE_ASC:
		    			if ($_SESSION['sb_files_sort'] == SB_VFS_SCANDIR_ORDER_SIZE_ASC)
		    			{
		    				$this->mFilesSort = SB_VFS_SCANDIR_ORDER_SIZE_DESC;
		    			}
		    			break;

		    		case SB_VFS_SCANDIR_ORDER_TIME_ASC:
		    			if ($_SESSION['sb_files_sort'] == SB_VFS_SCANDIR_ORDER_TIME_ASC)
		    			{
		    				$this->mFilesSort = SB_VFS_SCANDIR_ORDER_TIME_DESC;
		    			}
		    			break;
		    	}
		    }

            $_SESSION['sb_files_sort'] = $this->mFilesSort;
            sb_setcookie('sb_files_sort', $this->mFilesSort);
        }
        elseif (isset($_SESSION['sb_files_sort']))
        {
            $this->mFilesSort = intval($_SESSION['sb_files_sort']);
        }
        elseif (isset($_COOKIE['sb_files_sort']))
        {
        	$this->mFilesSort = intval($_COOKIE['sb_files_sort']);
        }

    	if (basename($_SERVER['PHP_SELF']) == 'empty.php')
        {
            $this->show();
            return;
        }

        if (!$_SESSION['sbPlugins']->isRightAvailable('pl_filelist', 'files_cut_copy'))
    	{
    	    $this->mFilesCutMenu = false;
    	    $this->mFilesCopyMenu = false;
    	}

        if (!$_SESSION['sbPlugins']->isRightAvailable('pl_filelist', 'files_edit'))
    	{
    	    $this->mFilesRenameMenu = false;
    	}

    	if (!$_SESSION['sbPlugins']->isRightAvailable('pl_filelist', 'files_delete'))
    	{
    	    $this->mFilesDeleteMenu = false;
    	}

        $this->mFoldersPasteFilesMenu = (($this->mFilesCopyMenu || $this->mFilesCutMenu) && $this->mFoldersPasteFilesMenu);
        $this->mFilesSelectAllMenu = ($this->mFilesCopyMenu || $this->mFilesCutMenu || $this->mFilesDeleteMenu) && $this->mFilesSelectAllMenu;

        if ($this->mFilesSelectedFile != '')
        {
            $path = dirname($this->mFilesSelectedFile);
            $this->mFoldersSelectedPath = $path;
        }

        sbFolders::init();

        echo '
        <link rel="stylesheet" type="text/css" href="'.SB_CMS_CSS_URL.'/sbFiles.css">
        <script src="'.SB_CMS_JSCRIPT_URL.'/sbFilesList.js.php"></script>
        <script>
        	sbGetE("event_content").style.overflow="hidden";

            var sb_files_selectall_menu = '.($this->mFilesSelectAllMenu ? 'true' : 'false').';
            var sb_files_sort_menu = '.($this->mFilesSortMenu ? 'true' : 'false').';
            var sb_files_copy_menu = '.($this->mFilesCopyMenu ? 'true' : 'false').';
            var sb_files_cut_menu = '.($this->mFilesCutMenu ? 'true' : 'false').';
            var sb_files_paste_menu = '.($this->mFoldersPasteFilesMenu ? 'true' : 'false').';

            var sb_files_extensions = "'.implode(' ', $this->mFilesExtensions).'";
            var sb_files_selected_file = "'.$this->mFilesSelectedFile.'";
            var sb_files_link_title = "'.($this->mFilesLinkTitle ? '1' : '0').'";

        '.$this->mFilesJavaScriptStr.'

            var sb_files_add_menu = [];';

        if (count($this->mFilesAdditionalMenu) > 0)
        {
            for ($i = 0; $i < count($this->mFilesAdditionalMenu); $i++)
            {
                echo '
                var i = sb_files_add_menu.length;
                sb_files_add_menu[i] = [];
                sb_files_add_menu[i]["item"] = "'.$this->mFilesAdditionalMenu[$i]['item'].'";
                sb_files_add_menu[i]["func"] = "'.$this->mFilesAdditionalMenu[$i]['func'].'";
                sb_files_add_menu[i]["hide"] = '.($this->mFilesAdditionalMenu[$i]['hide'] ? '1':'0').';
                ';
            }
        };

        if ($this->mFilesMenu)
        {
            echo 'sbFilesMenu = new sbCMenu("sbFilesMenu", "15#FF9933", "15#fff5e0", "highItemText", "lowItemText", 90);';
            if ($this->mFoldersPasteFilesMenu)
            {
                echo 'sbFilesMenu.addItem("paste", "'.SB_FOLDERS_PASTE_FILES_MENU.'", sbFoldersPasteFiles);';
            }

            echo 'sbFilesMenu.addItem("paste_delimeter");';

            if ($this->mFilesCopyMenu)
            {
                echo 'sbFilesMenu.addItem("copy", "'.SB_FILELIST_COPY_MENU.'", sbCopyFiles);';
            }

            if ($this->mFilesCutMenu)
            {
                echo 'sbFilesMenu.addItem("cut", "'.SB_FILELIST_CUT_MENU.'", sbCutFiles);';
            }

            if ($this->mFilesSelectAllMenu)
            {
                echo 'sbFilesMenu.addItem("select_all", "'.SB_FILELIST_SELECTALL_MENU.'", sbSelectAll);';
            }

            echo 'sbFilesMenu.addItem("cutcopy_delimeter");';

            if ($this->mFilesRenameMenu)
            {
                echo 'sbFilesMenu.addItem("rename", "'.SB_FILELIST_RENAME_MENU.'", sbRenameFile);';
            }

            if ($this->mFilesDeleteMenu)
            {
                echo 'sbFilesMenu.addItem("delete", "'.SB_FILELIST_DELETE_MENU.'", sbDeleteFiles);';
            }

            if ($this->mFoldersUploadMenu)
            {
                echo 'sbFilesMenu.addItem("upload", "'.SB_FOLDERS_UPLOAD_MENU.'", sbFoldersUploadFiles);';
            }

            if ($this->mFoldersRefreshMenu)
            {
                echo 'sbFilesMenu.addItem("refresh", "'.SB_FILELIST_REFRESH_MENU.'", sbRefreshFiles);';
            }

            echo 'sbFilesMenu.addItem("main_delimeter");';

            if (count($this->mFilesAdditionalMenu) > 0)
            {
                for ($i = 0; $i < count($this->mFilesAdditionalMenu); $i++)
                {
                	echo 'sbFilesMenu.addItem("additional_'.$i.'", "'.str_replace('"', '\"', $this->mFilesAdditionalMenu[$i]['item']).'", "'.$this->mFilesAdditionalMenu[$i]['func'].'");';
                }

                echo 'sbFilesMenu.addItem("additional_delimeter");';
            }

            if ($this->mFilesSortMenu)
            {
            	echo 'sbFilesMenu.addItem("sort_time", "'.SB_FILELIST_SORT_BY_TIME_MENU.'", "sbFilesSortBy(\''.SB_VFS_SCANDIR_ORDER_TIME_ASC.'\');");';
            	echo 'sbFilesMenu.addItem("sort_size", "'.SB_FILELIST_SORT_BY_SIZE_MENU.'", "sbFilesSortBy(\''.SB_VFS_SCANDIR_ORDER_SIZE_ASC.'\');");';
            	echo 'sbFilesMenu.addItem("sort_name", "'.SB_FILELIST_SORT_BY_NAME_MENU.'", "sbFilesSortBy(\''.SB_VFS_SCANDIR_ORDER_NAME_ASC.'\');");';
            }
        }

        echo '</script>';

        if ($this->mFilesHeaderStr != '')
        {
            echo '<div id="files_list_header" style="visibility:hidden;">'.$this->mFilesHeaderStr.'</div>';
        }

        echo '<div id="sb_files_show_image_div"></div>
              <img id="sb_files_show_image_img" src="" />
              <table width="100%" cellpadding="0" cellspacing="0" id="files_list_table" style="visibility:hidden;margin-top: 1px;">
              <tr>
                  <td valign="top" style="width:100%;">
                      <div id="files_list" oncontextmenu="sbFilesShowMenu(event);">&nbsp;</div>
                  </td>
                  <td class="sb_folders_drag_border" onmousedown="sbFoldersDragDel(event, true);">
                    <img src="'.SB_CMS_IMG_URL.'/blank.gif" width="6" height="1" border="0">
                  </td>
                  <td id="sb_folders_list_td">
                    <div id="sb_folders_list"></div>
                  </td>
              </tr></table>';

        if ($this->mFilesFooterStr != '')
        {
            echo '<div id="files_list_footer" style="visibility:hidden;">'.$this->mFilesFooterStr.'</div>';
        }

        echo '<div id="sb_folders_drag_div"></div>
            <script>';

        if ($this->mFilesAfterLoadFunc != '')
            echo 'var sbFilesAfterLoad = '.$this->mFilesAfterLoadFunc.';';
        else
            echo 'var sbFilesAfterLoad = null;';

        if ($this->mFilesClickFunc != '')
            echo 'var sbFilesOnClick = '.$this->mFilesClickFunc.';';
        else
            echo 'var sbFilesOnClick = null;';

        echo '  var panel_width = sbGetCookie("sb_panel_width");
                if (!panel_width)
                    panel_width = "'.$this->mFilesPanelWidth.'";
            </script>';
    }

    /**
     * Возвращает HTML-код изображения для переданного расширения файла
     *
     * @param string $ext Расширение файла.
     * @return string HTML-код изображения.
     */
    static function getFileIcon($ext, $path='')
    {
    	$small_icons = sbPlugins::getUserSetting('sb_files_small_icons');
    	if ($small_icons)
    		$size = 30;
    	else
    		$size = 50;

    	$images = array('gif', 'png', 'jpg', 'jpeg', 'bmp');
        $ext = strtolower($ext);
        $file_time = '';

        if(browser_is_safari())
        {
            $file_time = $GLOBALS['sbVfs']->filectime($path);
            $file_time = (is_int($file_time) && $file_time > 0)? '?a='.$file_time : '';
        }

        if ($GLOBALS['sbVfs']->exists('/cms/images/files/'.$ext.'.png'))
            return '<img src="'.SB_CMS_IMG_URL.'/files/'.$ext.'.png" align="absMiddle" width="'.$size.'" height="'.$size.'" border="0"'.(in_array($ext, $images) && $path != '' ? ' onmouseover="sbFilesShowImage(event, \''.$path.$file_time.'\');"' : '').' />';
        else
            return '<img src="'.SB_CMS_IMG_URL.'/files/bin.png" align="absMiddle" width="'.$size.'" height="'.$size.'" border="0" />';
    }

    /**
     * Выводит список файлов
     *
     * Вызывается каждый раз при подгрузке списка. Осуществляет пейджирование файлов в списке.
     *
     */
    public function show()
    {
        if (isset($_GET['path']) && !empty($_GET['path']))
	    {
	        $_GET['path'] = preg_replace('/[^\s0-9a-zA-Z\/\\\\_\-\.,\[\]]+/i', '', urldecode($_GET['path']));
	        $_SESSION['sb_folders_selected_path'] = $_GET['path'];
		    $this->mFoldersSelectedPath = $_GET['path'];
	    }

	    if (isset($_GET['files_exts']) && !empty($_GET['files_exts']))
	    {
	        $this->mFilesExtensions = explode(' ', $_GET['files_exts']);
	    }

    	if (isset($_GET['files_link_title']))
	    {
	        $this->mFilesLinkTitle = intval($_GET['files_link_title']);
	    }

	    if (isset($_GET['files_selected_file']) && !empty($_GET['files_selected_file']))
	    {
	       $this->mFilesSelectedFile = basename(urldecode($_GET['files_selected_file']));
	    }

	    // проверяем, существует ли такая папка
        if (!$GLOBALS['sbVfs']->exists($this->mFoldersSelectedPath))
        {
            // если не существует, выводим соотв. предупреждение
            sb_show_message(sprintf(SB_FILELIST_READ_ERROR_MSG, $this->mFoldersSelectedPath), true, 'warning');
            echo '<span id="files_show_menu" style="display:none;">0</span>';
            return;
        }

        $path = $this->mFoldersSelectedPath;
        $show = fFolders_Check_Rights($path);

        if (!$show)
        {
             // если не существует, выводим соотв. предупреждение
            sb_show_message(SB_FILELIST_DENY_ERROR_MSG, true, 'warning');
            echo '<span id="files_show_menu" style="display:none;">0</span>';
            return;
        }

        echo '<span id="files_show_menu" style="display:none;">1</span>';

        // пейджирование
        $files_per_page = $this->mFilesRows * $this->mFilesCols;
        if ($files_per_page <= 0)
        {
            // если кол-во выводимых файлов меньше 0, ничего не выводим
            return;
        }

        // текущая страница для вывода
        if (isset($_GET['files_from']) && $_GET['files_from'] != '')
            $from = intval($_GET['files_from']);
        else
            $from = 0;

        // считываем содержимое папки
        $files = $GLOBALS['sbVfs']->scandir($path, $this->mFilesSort);
        if ($files === false)
        {
            // считать содержимое не удалось
            sb_show_message(sprintf(SB_FILELIST_READ_ERROR_MSG, $path), true, 'warning');
            return;
        }

        $show_files = array();
        $j = 0;
        $sel_i = -1;
        for ($i = 0; $i < count($files); $i++)
        {
            $file_name = $files[$i];
            $file_ext = pathinfo($file_name);
            $file_ext = isset($file_ext['extension']) ? strtolower($file_ext['extension']) : '';

            // отбрасываем ненужные файлы
            if ($file_name == '.' || $file_name == '..' || $GLOBALS['sbVfs']->is_dir($path.'/'.$file_name)
                || (count($this->mFilesExtensions) != 0 && !in_array($file_ext, $this->mFilesExtensions)))
            {
                continue;
            }

            // Применяем фильтры
            if(isset($_SESSION['sb_files_filter_name']) && $_SESSION['sb_files_filter_name'] != '')
            {
            	// Имя файла
            	$matches = array();
            	$filter_name_pattern = sb_str_replace('*','.*', $_SESSION['sb_files_filter_name']);
            	preg_match('/(^'.$filter_name_pattern.')/is'.SB_PREG_MOD, $file_name, $matches);
            	if(count($matches) < 1)
            		continue;
            }

            if((isset($_SESSION['sb_files_filter_datefrom']) && $_SESSION['sb_files_filter_datefrom'] != '') || (isset($_SESSION['sb_files_filter_dateto']) && $_SESSION['sb_files_filter_dateto'] != ''))
            {
            	// Дата последнего изменения
            	$file_time = $GLOBALS['sbVfs']->filectime($path.'/'.$file_name);
            	// От
            	if(isset($_SESSION['sb_files_filter_datefrom']) && $_SESSION['sb_files_filter_datefrom'] != '' && $file_time < $_SESSION['sb_files_filter_datefrom'])
            		continue;
            	// До
            	if(isset($_SESSION['sb_files_filter_dateto']) && $_SESSION['sb_files_filter_dateto'] != '' && $file_time > $_SESSION['sb_files_filter_dateto'])
					continue;
            }

            $show_files[$j] = array();
            $show_files[$j]['file_name'] = $file_name;
            $show_files[$j++]['file_ext'] = $file_ext;

            if ($file_name == $this->mFilesSelectedFile)
                $sel_i = $j;
        }

        $num = count($show_files);
        if ($num == 0)
        {
            // файлов нет
            sb_show_message(SB_FILELIST_NO_FILES.($this->mFoldersUploadMenu ? '<br><br><a href="#" onclick="sbFoldersUploadFiles();">'.SB_FOLDERS_UPLOAD_MENU.'</a>' : ''), true);
            return;
        }

        if ($sel_i != -1)
        {
            $from = ceil($sel_i/$files_per_page) - 1;
        }

        if ($from * $files_per_page >= $num)
        {
            $from = ceil($num/$files_per_page) - 1;
        }

        $to = min($num, ($from + 1) * $files_per_page);

        $result = '<table cellpadding="0" cellspacing="0" border="0" width="98%"><tr>';
        $col_width = floor(100 / $this->mFilesCols);

        for ($i = $from * $files_per_page; $i < $to; $i++)
        {
            if ($i % $this->mFilesCols == 0 && $i != 0)
            {
                $result .= '</tr><tr>';
            }

            $file_name = $show_files[$i]['file_name'];
            $file_ext = $show_files[$i]['file_ext'];

            if ($file_name == '.' || $file_name == '..' || $GLOBALS['sbVfs']->is_dir($path.'/'.$file_name))
                continue;

            $file_id = str_replace('//', '/', $path.'/'.$file_name);
            $file_icon = $this->getFileIcon($file_ext, $file_id);
            $file_size = $GLOBALS['sbVfs']->filesize($path.'/'.$file_name);
            $file_time = $GLOBALS['sbVfs']->filectime($path.'/'.$file_name);
            if (!$file_size)
            {
                $file_size = 0;
            }

            if (!is_string($file_time) && $file_time <= 0)
            {
                $file_time = '';
            }
            else
            {
                if (!is_string($file_time))
                    $file_time = sb_date('d.m.Y '.KERNEL_IN.' H:i', $file_time);
            }

            $kb = floor($file_size / 1024);
            if ($kb > 0)
            {
                $file_size = $kb.' '.SB_FILELIST_FILE_SIZE_KB;
            }
            else
            {
                $file_size .= ' '.SB_FILELIST_FILE_SIZE_BYTE;
            }

            $file_raw_name = str_replace('.'.$file_ext, '', $file_name);
            if (sbPlugins::getUserSetting('sb_files_truncate_names') != 1 && sb_strlen($file_raw_name) > 20)
            {
            	$file_raw_name = substr($file_raw_name, 0, 20).'(...).'.$file_ext;
            }
            else
            {
            	$file_raw_name = $file_name;
            }

            $result .= '<td width="'.$col_width.'%"><div id="el_'.$file_id.'" el_id="'.$file_id.'" class="list_element" onclick="sbClickFile(event);" style="-moz-user-select: none;">
            <table cellpadding="5" cellspacing="0" width="100%">
            <tr>
                <td id="el_icon_'.$file_id.'" width="1" valign="top" rowspan="2">'.$file_icon.'</td>
                <td class="list_element_td">
                <div id="el_title_'.$file_id.'" style="padding-bottom: 3px;">'.($this->mFilesLinkTitle ? '<a href="'.str_ireplace('https://', 'http://', SB_DOMAIN).$file_id.'" target="_blank" title="'.$file_name.'"><b>'.$file_raw_name.'</b></a>' : '<b title="'.$file_name.'">'.$file_raw_name.'</b>').'</div>
                <span class="smalltext">
                '.SB_FILELIST_FILE_SIZE.': <i>'.$file_size.'</i><br />
                </span>
                </td>
            </tr>
            <tr><td class="list_element_date">'.($file_time != '' ? SB_FILELIST_FILE_TIME.': <i>'.$file_time.'</i>' : '').'</td></tr>
            </table></div></td>';
        }

        $count = $i;

        for ($i = 0; $i < $count % $this->mFilesCols; $i++)
        {
            $result .= '<td width="'.$col_width.'%">&nbsp;</td>';
        }
        $result .= '</tr></table>';

        if ($num > $files_per_page)
            $pager_display = 'block';
        else
            $pager_display = 'none';

        echo '<div style="width:98%;padding:5px;display:'.$pager_display.';" align="center"><nobr>';
        $nav_bar_str = '';
        if ($from > 0)
        {
            $nav_bar_str .= '<img src="'.SB_CMS_IMG_URL.'/begin.gif" width="14" height="15" alt="'.KERNEL_PAGER_BEGIN.'" onclick="sbFilesShow(sb_cms_empty_file + \'?event=pl_filelist_read&path=\'+sbFoldersTree.getSelectedItemId()+\'&files_from=0\'+\'&files_exts=\'+sb_files_extensions);" style="cursor: hand;cursor:pointer;" align="absmiddle">&nbsp;&nbsp;';
            $nav_bar_str .= '<img src="'.SB_CMS_IMG_URL.'/previous.gif" width="14" height="15" alt="'.KERNEL_PAGER_PREV.'" onclick="sbFilesShow(sb_cms_empty_file + \'?event=pl_filelist_read&path=\'+sbFoldersTree.getSelectedItemId()+\'&files_from='.($from - 1).'\'+\'&files_exts=\'+sb_files_extensions);" style="cursor: hand;cursor:pointer;" align="absmiddle">&nbsp;&nbsp;';
        }
        else
        {
            $nav_bar_str .= '<img src="'.SB_CMS_IMG_URL.'/begin_gray.gif" width="14" height="15" alt="'.KERNEL_PAGER_BEGIN.'" align="absmiddle">&nbsp;&nbsp;';
            $nav_bar_str .= '<img src="'.SB_CMS_IMG_URL.'/previous_gray.gif" width="14" height="15" alt="'.KERNEL_PAGER_PREV.'" align="absmiddle">&nbsp;&nbsp;';
        }

        if ($pager_display != 'none')
        {
            $nav_bar_str .= '<b>'.($from + 1).'</b> '.KERNEL_WITHIN.' <b>'.(ceil($num/$files_per_page)).'</b>';
        }
        else
        {
            $nav_bar_str .= '<b>1</b> '.KERNEL_WITHIN.' <b>2</b>';
        }

        if (($from + 1) * $files_per_page < $num || $pager_display == 'none')
        {
            $nav_bar_str .= '&nbsp;&nbsp;<img src="'.SB_CMS_IMG_URL.'/next.gif" width="14" height="15" alt="'.KERNEL_PAGER_NEXT.'" onclick="sbFilesShow(sb_cms_empty_file + \'?event=pl_filelist_read&path=\'+sbFoldersTree.getSelectedItemId()+\'&files_from='.($from + 1).'\'+\'&files_exts=\'+sb_files_extensions);" style="cursor: hand;cursor:pointer;" align="absmiddle">&nbsp;&nbsp;';
            $nav_bar_str .= '<img src="'.SB_CMS_IMG_URL.'/end.gif" width="14" height="15" alt="'.KERNEL_PAGER_END.'" onclick="sbFilesShow(sb_cms_empty_file + \'?event=pl_filelist_read&path=\'+sbFoldersTree.getSelectedItemId()+\'&files_from='.(ceil($num/$files_per_page)).'\'+\'&files_exts=\'+sb_files_extensions);" style="cursor: hand;cursor:pointer;" align="absmiddle">';
        }
        else
        {
            $nav_bar_str .= '&nbsp;&nbsp;<img src="'.SB_CMS_IMG_URL.'/next_gray.gif" width="14" height="15" alt="'.KERNEL_PAGER_NEXT.'" align="absmiddle">&nbsp;&nbsp;';
            $nav_bar_str .= '<img src="'.SB_CMS_IMG_URL.'/end_gray.gif" width="14" height="15" alt="'.KERNEL_PAGER_END.'" align="absmiddle">';
        }

        echo $nav_bar_str.'</nobr></div>
        <div align="center" id="files_list_div" from="'.$from.'">'.$result.'</div>
        <div style="width:98%;padding:5px;display:'.$pager_display.';" align="center"><nobr>'.$nav_bar_str.'</nobr></div>';
    }
}
?>