<?php
/**
 * Реализация класса, отвечающего за подгрузку дерева разделов.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Categs
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */

/**
 * Класс, отвечающий за подгрузку дерева разделов.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Categs
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbJustCategs
{	
    /**
     * Уникальный идентификатор разделов модуля
     *
     * @var string
     */
    public $mCategsIdent = '';

    /**
     * Событие, вызываемое для подгрузки элементов раздела
     *
     * @var string
     */
    public $mCategsEvent = '';

    /**
     * Название корневого раздела по умолчанию
     *
     * Если пользователь запрашивает дерево разделов и корневого раздела нет,
     * то он будет создан автоматически.
     *
     * @var string
     */
    public $mCategsRootName = '';

    /**
     * JavaScript-код, выводимый перед инициализацией дерева разделов
     *
     * @var string
     */
    public $mCategsJavascriptStr = '';
    
    /**
     * ID выбранного раздела
     *
     * @var int
     */
    public $mCategsSelectedId = -1;

    /**
     * ID выбранных разделов (если используется множественный выбор)
     *
     * @var array
     */
    public $mCategsSelectedIds = array();
    
    /**
     * Максимальное кол-во уровней вложенности в дереве разделов
     *
     * @var int
     */
    public $mCategsMaxDeep = 999;

    /**
     * Массив идентификаторов разделов, которые не будут выводиться
     *
     * @var array
     */
    public $mCategsNeverShowCats = array(); 
    
    /**
     * Массив идентификаторов разделов, которые будут выводиться.
     * Имеет более высокий приоритет по сравнению с mCategsNeverShowCats
     * @var array
     */
    public $mCategsShowOnlyCats = array();

    /**
     * Проверять права доступа текущего пользователя системы к разделу или нет
     *
     * Разделы, к которым пользователь не имеет прав доступа, выводиться не будут.
     *
     * @var bool
     */
    public $mCategsUseRights = true;

    /**
     * Используется рубрикатор модулем или нет
     *
     * @var bool
     */
    public $mCategsRubrikator = true;

    /**
     * Уровень, с которого разрешен рубрикатор
     *
     * @var bool
     */
    public $mCategsRubrikatorLevel = 0;
    
    /**
     * Используются ли закрытые разделы
     *
     * Для закрытых разделов есть возможность при редактировании выбрать группу пользователей, имеющих доступ к разделу.
     *
     * @var bool
     */
    public $mCategsClosed = true;
    
    /**
     * Выводить поле "Псевдостатический адрес" при редактировании и добавлении раздела
     *
     * @var boolean
     */
    public $mCategsUrl = false;

    /**
     * Выводить закладку "Модераторы раздела" при редактировании и добавлении раздела
     *
     * @var boolean
     */
    public $mCategsModerators = false;
    
    /**
     * Используются ли пользовательские поля у разделов
     *
     * @var bool
     */
    public $mCategsFields = false;
    
    /**
     * Показывать ли кол-во элементов в разделе рядом с названием раздела
     *
     * @var bool
     */
    public $mCategsCount = true;
    
    /**
     * Использовать AJAX для подгрузки разделов или нет
     *
     * @var bool
     */
    public $mCategsAutoloading = true;

    /**
     * Использовать или нет множественный выбор у разделов (используется в некоторых диалоговых окнах)
     *
     * @var bool
     */
    public $mCategsMultiSelect = false;

    /**
     * Использовать или нет Drag And Drop для дерева разделов
     *
     * @var bool
     */
    public $mCategsDad = true;

    /**
     * Режим Drag And Drop (0 - только вложения, 1 - только вниз и вверх, 2 - и то, и другое)
     *
     * @var int
     */
    public $mCategsDadMode = 2;

    /**
     * Уровень, с которого разрешено перетаскивание (Drag And Drop)
     *
     * @var int
     */
    public $mCategsDadStartLevel = 0;

    /**
     * Выводить или нет контекстное меню разделов
     *
     * @var bool
     */
    public $mCategsMenu = true;

    /**
     * Выводить или нет пункт меню "Права на раздел"
     *
     * @var bool
     */
    public $mCategsRightsMenu = true;

    /**
     * Выводить или нет пункт меню "Обновить раздел"
     *
     * @var bool
     */
    public $mCategsRefreshMenu = true;

    /**
     * Выводить или нет пункт меню "Добавить подраздел"
     *
     * @var bool
     */
    public $mCategsAddMenu = true;

    /**
     * Выводить или нет пункт меню "Редактировать раздел"
     *
     * @var bool
     */
    public $mCategsEditMenu = true;

    /**
     * Выводить или нет пункт меню "Удалить раздел"
     *
     * @var bool
     */
    public $mCategsDeleteMenu = true;
    
     /**
     * Выводить или нет пункт меню "Удалить раздел c элементами"
     *
     * @var bool
     */
    public $mCategsDeleteWithElementsMenu = false;

    /**
     * Выводить или нет пункт меню "Копировать разделы"
     *
     * @var bool
     */
    public $mCategsCopyMenu = true;

    /**
     * Выводить или нет пункт меню "Вырезать разделы"
     *
     * @var bool
     */
    public $mCategsCutMenu = true;

    /**
     * Выводить или нет пункт меню "Вставить элементы"
     *
     * Возможные значения:
     * <ul>
     * <li>none - не выводить</li>
     * <li>copy - выводить только для скопированных элементов</li>
     * <li>cut - выводить только для вырезанных элементов</li>
     * <li>all - выводить для всех элементов<li>
     * </ul>
     * 
     * @var string
     */
    public $mCategsPasteElemsMenu = 'all';

    /**
     * Выводить или нет пункт меню "Вставить разделы и элементы"
     *
     * @var string
     */
    public $mCategsPasteWithElementsMenu = false;
    
    /**
     * Название пункта меню "Вставить разделы и элементы"
     *
     * @var string
     */
    public $mCategsPasteWithElementsMenuTitle = SB_CATEGS_ASTE_CATEGS_WITH_ELEMENTS_MENU;
    
    /**
     * JavaScript-функция, вызываемая после добавления раздела с элементами
     *
     * @var string
     */
    public $mCategsAfterPasteWithElementsFunc = '';
    
     /**
     * Событие, вызываемое перед вставкой раздела с элементами
     *
     * Если возвращает не пустую строку, то возвращенное значение показывается пользователю
     * и вставка раздела не осуществляется.
     * 
     * Получаемые GET-параметры:
     * <ul>
     * <li>cat_id - ID раздела, в который производится вставка скопированного или вырезанного раздела</li>
     * <li>paste_cat_id - ID скопированного или вырезанного раздела</li>
     * <li>before_cat_id - ID раздела, находящегося в иерархии дерева непосредственно над разделом, в который производится вставка</li>
     * </ul>
     * 
     * @var string
     */
    public $mCategsBeforePasteWithElementsEvent = '';
    
     /** 
     * Cобытие, вызываемое после вставки раздела
     *
     * Если возвращает не пустую строку, то возвращенное значение показывается пользователю.
     *
     * Получаемые GET-параметры:
     * <ul>
     * <li>cat_id - ID раздела, в который производится вставка скопированного или вырезанного раздела</li>
     * <li>paste_cat_id - ID скопированного или вырезанного раздела</li>
     * <li>before_cat_id - ID раздела, находящегося в иерархии дерева непосредственно над разделом, в который производится вставка</li>
     * <li>new_cat_id - если раздел был скопирован, содержит ID копии раздела</li>
     * </ul>
     * 
     * @var string
     */
    public $mCategsAfterPasteWithElementsEvent = '';
    
    /**
     * Выводить или нет пункт меню "Вставить ссылки на элементы"
     *
     * @var bool
     */
    public $mCategsPasteLinksMenu = true;

    /**
     * Название пункта меню "Добавить подраздел"
     *
     * @var string
     */
    public $mCategsAddMenuTitle = SB_CATEGS_ADD_MENU;

    /**
     * Название пункта меню "Редактировать раздел"
     *
     * @var string
     */
    public $mCategsEditMenuTitle = SB_CATEGS_EDIT_MENU;

    /**
     * Название пункта меню "Удалить раздел"
     *
     * @var string
     */
    public $mCategsDeleteMenuTitle = SB_CATEGS_DELETE_MENU;

	/**
     * Название пункта меню "Удалить разделы и элементы"
     *
     * @var string
     */
    public $mCategsDeleteWithElementsMenuTitle = SB_CATEGS_DELETE_WITH_ELEMENT_MENU;

    /**
     * Название пункта меню "Копировать разделы"
     *
     * @var string
     */
    public $mCategsCopyMenuTitle = SB_CATEGS_COPY_MENU;

    /**
     * Название пункта меню "Вставить разделы"
     *
     * @var string
     */
    public $mCategsPasteMenuTitle = SB_CATEGS_PASTE_CATEGS_MENU;
    
    /**
     * Название пункта меню "Вырезать разделы"
     *
     * @var string
     */
    public $mCategsCutMenuTitle = SB_CATEGS_CUT_MENU;

    /**
     * Название пункта меню "Вставить элементы"
     *
     * @var string
     */
    public $mCategsPasteElemsMenuTitle = SB_CATEGS_PASTE_MENU;

    /**
     * Название пункта меню "Вставить ссылки на элементы"
     *
     * @var string
     */
    public $mCategsPasteLinksMenuTitle = SB_CATEGS_PASTE_LINKS_MENU;
    
    /**
     * Уровень дерева, с которого доступно контекстное меню
     *
     * @var int
     */
    public $mCategsMenuLevel = 1;

    /**
     * Уровень дерева, с которого доступен пункт контекстного меню "Добавить подраздел"
     *
     * @var int
     */
    public $mCategsAddMenuLevel = 1;

    /**
     * Уровень дерева, с которого доступен пункт контекстного меню "Редактировать раздел"
     *
     * @var int
     */
    public $mCategsEditMenuLevel = 1;

    /**
     * Уровень дерева, с которого доступен пункт контекстного меню "Удалить разделы"
     *
     * @var int
     */
    public $mCategsDeleteMenuLevel = 2;

    /**
     * Уровень дерева, с которого доступен пункт контекстного меню "Копировать разделы"
     *
     * @var int
     */
    public $mCategsCopyMenuLevel = 2;

    /**
     * Уровень дерева, с которого доступен пункт контекстного меню "Вырезать разделы"
     *
     * @var int
     */
    public $mCategsCutMenuLevel = 2;

    /**
     * Уровень дерева, с которого доступен пункты контекстного меню "Вставить элементы" и "Вставить ссылки на элементы"
     *
     * @var int
     */
    public $mCategsPasteElemsMenuLevel = 1;

    /**
     * Уровень дерева, с которого доступен пункт контекстного меню "Вставить разделы"
     *
     * @var int
     */
    public $mCategsPasteCategsMenuLevel = 1;

    /**
     * Массив доп. пунктов меню для дерева разделов
     *
     * <code>
     * // текст пункта меню
     * $this->mFoldersAdditionalMenu[$i]['item'] = ...;
     * // JavaScript-функция, вызываемая при щелчке по пункту меню
     * $this->mFoldersAdditionalMenu[$i]['func'] = ...;
     * // начиная с какого уровня дерева показывать пункт меню
     * $this->mFoldersAdditionalMenu[$i]['level'] = ...;
     * </code>
     *
     * @var array
     */
    private $mCategsAdditionalMenu = array();
    
    /**
     * JavaScript-функция, вызываемая после редактирования раздела
     *
     * @var string
     */
    public $mCategsAfterEditFunc = '';
    
    /**
     * JavaScript-функция, вызываемая после добавления раздела
     *
     * @var string
     */
    public $mCategsAfterAddFunc = '';

    /**
     * Событие, вызываемое перед пометкой раздела
     * 
     * Если возвращает не пустую строку, то возвращенное значение показывается пользователю 
     * и пометка раздела не осуществляется.
     * 
     * Получаемые GET-параметры: 
     * <ul>
     * <li>cat_id - ID помечаемого раздела</li>
     * </ul>
     *
     * @var string
     */
    public $mCategsBeforeCheckEvent = '';
    
    /**
     * Массив, содержащий ID разделов, которые помечать нельзя
     * 
     * @var array
     */
    public $mCategsNotCheckable = array();
    
    /**
     * Массив, содержащий ID разделов, которые нельзя удалять
     * 
     * @var array
     */
    public $mCategsNotDeletable = array();
    
    /**
     * Массив, содержащий ID разделов, которые нельзя редакитровать
     * 
     * @var array
     */
    public $mCategsNotEditable = array();
    
    /**
     * Массив, содержащий ID разделов, в которые нельзя добавлять подразделы
     * 
     * @var array
     */
    public $mCategsNotAddable = array();
    
    /**
     * Массив, содержащий ID разделов, которые нельзя копировать
     * 
     * @var array
     */
    public $mCategsNotCopyable = array();
    
    /**
     * Cобытие, вызываемое перед редактированием раздела
     *
     * Если возвращает не пустую строку, то возвращенное значение показывается пользователю
     * и окно редактирования не вызывается.
     * 
     * Получаемые GET-параметры: 
     * <ul>
     * <li>cat_id - ID редактируемого раздела</li>
     * </ul>
     *
     * @var string
     */
    public $mCategsBeforeEditEvent = '';

    /**
     * Cобытие, генерирующее вывод собственного окна редактирования раздела
     *
     * Если не указано, используется стандартное окно редактирования раздела.
     *
     * Получаемые GET-параметры: 
     * <ul>
     * <li>cat_id - ID редактируемого раздела</li>
     * </ul>
     * 
     * @var string
     */
    public $mCategsEditEvent = '';

    /**
     * Ширина собственного окна редактирования раздела
     *
     * @var int
     */
    public $mCategsEditDlgWidth = 550;

    /**
     * Высота собственного окна редактирования раздела
     *
     * @var int
     */
    public $mCategsEditDlgHeight = 400;

    /**
     * Событие, вызываемое после редактирования раздела
     *
     * Если возвращает не пустую строку, то возвращенное значение показывается пользователю.
     *
     * Получаемые GET-параметры: 
     * <ul>
     * <li>cat_id - ID редактируемого раздела</li>
     * </ul>
     * 
     * @var string
     */
    public $mCategsAfterEditEvent = '';

    /**
     * Событие, вызываемое перед добавлением раздела
     *
     * Если возвращает не пустую строку, то возвращенное значение показывается пользователю
     * и окно добавления не показывается.
     * 
     * Получаемые GET-параметры: 
     * <ul>
     * <li>p_cat_id - ID родительского раздела, в который осуществялется добавление</li>
     * </ul>
     *
     * @var string
     */
    public $mCategsBeforeAddEvent = '';

    /**
     * Событие, генерирующее вывод собственного окна добавления раздела
     *
     * Если не указано, используется стандартное окно добавления раздела.
     *
     * Получаемые GET-параметры: 
     * <ul>
     * <li>p_cat_id - ID родительского раздела, в который осуществялется добавление</li>
     * </ul>
     * 
     * @var string
     */
    public $mCategsAddEvent = '';

    /**
     * Ширина собственного окна добавления раздела
     *
     * @var int
     */
    public $mCategsAddDlgWidth = 550;

    /**
     * Высота собственного окна добавления раздела
     *
     * @var int
     */
    public $mCategsAddDlgHeight = 400;

    /**
     * Событие, вызываемое после добавления раздела
     *
     * Если возвращает не пустую строку, то возвращенное значение показывается пользователю.
     *
     * Получаемые GET-параметры: 
     * <ul>
     * <li>p_cat_id - ID родительского раздела, в который осуществялется добавление</li>
     * <li>cat_id - ID добавленного раздела</li>
     * </ul>
     * 
     * @var string
     */
    public $mCategsAfterAddEvent = '';

    /**
     * Дополнительное событие, вызываемое при удалении раздела
     *
     * Если возвращает не пустую строку, то возвращенное значение показывается пользователю
     * и раздел не удаляется.
     * 
     * Получаемые GET-параметры: 
     * <ul>
     * <li>cat_id - ID удаляемого раздела</li>
     * </ul>
     *
     * @var string
     */
    public $mCategsDeleteEvent = '';
    
    /**
     * Дополнительное событие, вызываемое после удаления раздела c элементами
     *
     * Если возвращает не пустую строку, то возвращенное значение показывается пользователю
     * и раздел не удаляется.
     * 
     * Получаемые GET-параметры: 
     * <ul>
     * <li>cat_id - ID удаляемого раздела</li>
     * </ul>
     *
     * @var string
     */
    public $mCategsAfterDeleteWithElementsEvent = '';
    
    /**
     * Дополнительное событие, вызываемое передудалением раздела c элементами
     *
     * Если возвращает не пустую строку, то возвращенное значение показывается пользователю
     * и раздел не удаляется.
     * 
     * Получаемые GET-параметры: 
     * <ul>
     * <li>cat_id - ID удаляемого раздела</li>
     * </ul>
     *
     * @var string
     */
    public $mCategsBeforeDeleteWithElementsEvent = '';

    /**
     * Событие, вызываемое перед вставкой раздела
     *
     * Если возвращает не пустую строку, то возвращенное значение показывается пользователю
     * и вставка раздела не осуществляется.
     * 
     * Получаемые GET-параметры:
     * <ul>
     * <li>cat_id - ID раздела, в который производится вставка скопированного или вырезанного раздела</li>
     * <li>paste_cat_id - ID скопированного или вырезанного раздела</li>
     * <li>before_cat_id - ID раздела, находящегося в иерархии дерева непосредственно над разделом, в который производится вставка</li>
     * <li>action - CUT - раздел был вырезан, COPY - раздел был скопирован</li>
     * </ul>
     * 
     * @var string
     */
    public $mCategsBeforePasteEvent = '';

    /**
     * Cобытие, вызываемое после вставки раздела
     *
     * Если возвращает не пустую строку, то возвращенное значение показывается пользователю.
     *
     * Получаемые GET-параметры:
     * <ul>
     * <li>cat_id - ID раздела, в который производится вставка скопированного или вырезанного раздела</li>
     * <li>paste_cat_id - ID скопированного или вырезанного раздела</li>
     * <li>before_cat_id - ID раздела, находящегося в иерархии дерева непосредственно над разделом, в который производится вставка</li>
     * <li>action - CUT - раздел был вырезан, COPY - раздел был скопирован</li>
     * <li>new_cat_id - если раздел был скопирован, содержит ID копии раздела</li>
     * </ul>
     * 
     * @var string
     */
    public $mCategsAfterPasteEvent = '';

    /**
     * Cобытие, вызываемое перед вставкой элемента или ссылки на элемент
     *
     * Если возвращает не пустую строку, то возвращенное значение показывается пользователю
     * и вставка элемента или ссылки на элемент не осуществляется.
     * 
     * Получаемые GET-параметры: 
     * <ul>
     * <li>new_cat_id - ID раздела, в который вставляются элементы</li>
     * <li>old_cat_id - ID раздела, из которого вставляются элементы</li>
     * <li>action - CUT - элемент был вырезан, copy - элемент был скопирован</li>
     * <li>e_.. - ID вставляемых элементов (.. - уникальный набор цифр)</li>
     * </ul>
     *
     * @var string
     */
    public $mElemsBeforePasteEvent = '';

    /**
     * Cобытие, вызываемое после вставки элемента
     *
     * Если возвращает не пустую строку, то возвращенное значение показывается пользователю.
     *
     * Получаемые GET-параметры: 
     * <ul>
     * <li>new_cat_id - ID раздела, в который вставляются элементы</li>
     * <li>old_cat_id - ID раздела, из которого вставляются элементы</li>
     * <li>action - CUT - элемент был вырезан, copy - элемент был скопирован</li>
     * <li>e_.. - ID вставляемых элементов (.. - уникальный набор цифр)</li>
     * <li>ne_.. - если производится копирование элементов, то содержит ID копий элементов</li>
     * </ul>
     * 
     * @var string
     */
    public $mElemsAfterPasteEvent = '';

    /**
     * Имя функции, вызываемой перед формированием названия раздела.
     *
     * @var function
     */
    public $mCategsTitleHandler = '';
    
    /**
     * Имя таблицы, содержащей связанные с разделом элементы (таблица элементов)
     *
     * @var string
     */
    public $mElemsTable = '';

    /**
     * Автоинкрементное поле таблицы элементов, содержащее уникальный идентификатор элемента
     *
     * @var string
     */
    public $mElemsIdField = '';
    
    /**
     * Поле таблицы элементов, содержащее название элемента
     *
     * @var string
     */
    public $mElemsTitleField = '';

    /**
     * Конструктор класса
     *
     * В конструкторе класса проверяется, использует ли модуль рубрикатор или нет.
     *
     * @param string $ident Уникальный идентификатор разделов модуля.
     */
    public function __construct($ident)
    {
    	$this->mCategsIdent = $ident;

        // использует ли модуль рубрикатор
        if ($this->mCategsRubrikator)
            $this->mCategsRubrikator = isset($_SESSION['sbPlugins']->mRubrikator[$ident]);

        // определяем ID выбранного раздела
        if (isset($_SESSION['sb_categs_selected_id']) && isset($_SESSION['sb_categs_selected_id'][$ident]))
            $this->mCategsSelectedId = $_SESSION['sb_categs_selected_id'][$ident];
        else
            $_SESSION['sb_categs_selected_id'][$ident] = -1;

        if (!isset($_SESSION['sb_categs_closed_descr']))
        {
        	$_SESSION['sb_categs_closed_descr'] = array();
        }
        
        if ($_GET['event'] != 'pl_categs_read')
        {
	        $_SESSION['sb_categs_closed_descr'][$ident] = array();
        }
    }

    /**
     * Добавляет права доступа к закрытому разделу
     *
     * @param string $ident Идентификатор права (например, read, write, delete и пр.). Реальный идентификатор - идентификатор раздела + знак
     *                      подчеркивания + идентификатор права (например, pl_news_read или pl_users_write).
     * @param string $name Описание права.
     */
    public function addCategsClosedDescr($ident, $name)
    {
        $_SESSION['sb_categs_closed_descr'][$this->mCategsIdent][$ident] = $name;
    }
    
    /**
     * Добавляет доп. пункт в контекстное меню разделов
     *
     * @param string $title Название пункта меню.
     * @param string $func JavaScript-функция, вызываемая при щелчке по пункту меню.
     * @param int $level Уровень, начиная с которого будет показываться пункт меню.
     */
    public function addCategsMenuItem($title, $func, $level=1)
    {
        $i = count($this->mCategsAdditionalMenu);

        $this->mCategsAdditionalMenu[$i] = array();
        $this->mCategsAdditionalMenu[$i]['item'] = str_replace('"', '\\"', $title);
        $this->mCategsAdditionalMenu[$i]['func'] = str_replace('"', '\\"', $func);
        $this->mCategsAdditionalMenu[$i]['level'] = $level;
    }

    /**
     * Инициализация переменных класса и вывод JavaScript-кода для работы с разделами
     */
    public function init($show = true)
    {
        if (basename($_SERVER['PHP_SELF']) == 'empty.php')
        {
            if ($show) $this->show();
            return;
        }
        
        if(!is_array($this->mCategsShowOnlyCats))
        {
            $this->mCategsShowOnlyCats = array();
        }

        // проверяем, есть ли корневой раздел, если нет, то создаем его
	    $res = sql_query('SELECT cat_id FROM sb_categs WHERE cat_ident=? ORDER BY cat_left LIMIT 1', $this->mCategsIdent);

        if (!$res)
        {
            if ($this->mCategsRootName == '')
            {
                $this->mCategsRootName = SB_CATEGS_ROOT_NAME;
            }

            if (!$_SESSION['sbAuth']->isAdmin())
            {
                $cat_rights = 'u'.$_SESSION['sbAuth']->getUserId();
            }
            else
            {
                $cat_rights = '';
            }

            sql_query('INSERT INTO sb_categs (cat_ident, cat_title, cat_left, cat_right, cat_level, cat_rubrik, cat_closed, cat_rights) VALUES (?, ?, 1, 2, 0, 1, 0, ?)', $this->mCategsIdent, $this->mCategsRootName, $cat_rights);
            $root_id = sql_insert_id();

            if (!$root_id)
                return;

            if ($this->mCategsSelectedId == -1)
            {
                $this->mCategsSelectedId = $root_id;
            }
        }
        else 
        {
            if (count($this->mCategsNeverShowCats) != 0 || count($this->mCategsShowOnlyCats) != 0)
            {
                if(count($this->mCategsShowOnlyCats) != 0)
                {
                    //Исключим из списка запрещенных идентификаторов те, которые явно разрешены
                    $this->mCategsNeverShowCats = array_diff($this->mCategsNeverShowCats, $this->mCategsShowOnlyCats);
                    $where = ' AND cat_id IN (?a) ';
                    $arr = $this->mCategsShowOnlyCats;
                }
                else
                {
                    $where = ' AND cat_id NOT IN (?a) ';
                    $arr = $this->mCategsNeverShowCats;
                }
                
                $res_sub = sql_query('SELECT cat_id FROM sb_categs WHERE cat_ident=? '.$where.' ORDER BY cat_left LIMIT 1', $this->mCategsIdent, $arr);
                if ($res_sub)
                {
                    list($root_id) = $res_sub[0];
                }
                else 
                {
                	$root_id = -1;
                	$this->mCategsSelectedId = -1;
                }
            }
            else
            {
                list($root_id) = $res[0];
            }
        }

        if ($this->mCategsSelectedId == -1)
        {
            $this->mCategsSelectedId = $root_id;
        }

        // проверяем, существует ли выбранный раздел
        if ($this->mCategsSelectedId != $root_id)
        {
            if (!sql_query('SELECT cat_id FROM sb_categs WHERE cat_id=?d', $this->mCategsSelectedId))
            {
                $this->mCategsSelectedId = $root_id;
            }
        }

        if (!fCategs_Check_Rights($this->mCategsSelectedId))
        {
        	$where_str = ''; 
        	foreach ($_SESSION['sbAuth']->getUserGroups() as $gid)
        	{
        		$where_str .= "CONCAT('^', cat_rights, '^') LIKE '%^g$gid^%' OR ";
        	}
        	
        	$where_str .= "CONCAT('^', cat_rights, '^') LIKE '%^u".$_SESSION['sbAuth']->getUserId()."^%'";
        	$where_str = '('.$where_str.')';

        	$res = sql_query('SELECT cat_id FROM sb_categs WHERE cat_ident=? AND '.$where_str.' ORDER BY cat_left LIMIT 1', $this->mCategsIdent);
            if ($res)
            {
            	$this->mCategsSelectedId = $res[0][0];	
            }
        }
        
        if (!is_array($this->mCategsNeverShowCats))
            $this->mCategsNeverShowCats = array();

        $plugin = $_SESSION['sbPlugins']->getPluginTitle($this->mCategsEvent);
        $plugin_ident = $_SESSION['sbPlugins']->getEventPlugin($this->mCategsEvent);

        // проверяем права пользователя
        if (!$_SESSION['sbPlugins']->isRightAvailable($plugin_ident, 'categs_edit'))
        {
            $this->mCategsRubrikator = false;
            $this->mCategsDad = false;
            $this->mCategsAddMenu = false;
            $this->mCategsEditMenu = false;
            $this->mCategsCopyMenu = false;
            $this->mCategsCutMenu = false;
        }
        
        if (!$_SESSION['sbPlugins']->isRightAvailable($plugin_ident, 'categs_delete'))
        {
            $this->mCategsDeleteMenu = false;
            $this->mCategsDeleteWithElementsMenu = false;
        }
        
        if (!$_SESSION['sbPlugins']->isRightAvailable($plugin_ident, 'categs_rights'))
        {
            $this->mCategsRightsMenu = false;
        }

        echo '<link rel="stylesheet" type="text/css" href="'.SB_CMS_CSS_URL.'/sbCategs.css">
        <script src="'.SB_CMS_JSCRIPT_URL.'/sbDAD.js.php"></script>
        <script src="'.SB_CMS_JSCRIPT_URL.'/sbCMenu.js.php"></script>
        <script src="'.SB_CMS_JSCRIPT_URL.'/sbXMLLoader.js.php"></script>
        <script src="'.SB_CMS_JSCRIPT_URL.'/sbTree.js.php"></script>
        <script src="'.SB_CMS_JSCRIPT_URL.'/sbJustCategs.js.php"></script>
        <script>
            '.$this->mCategsJavascriptStr.'
            var sb_elems_plugin = "'.str_replace('"', '\\"', $plugin).'";
            var sb_elems_plugin_ident = "'.$plugin_ident.'";
            var sb_elems_event = "'.$this->mCategsEvent.'";
            var sb_elems_table = "'.$this->mElemsTable.'";
            var sb_elems_id_field = "'.$this->mElemsIdField.'";
            var sb_elems_title_field = "'.$this->mElemsTitleField.'";

            var sb_categs_selected_id = "'.$this->mCategsSelectedId.'";
            var sb_categs_selected_ids = [];
            var sb_categs_not_editable = [];
            var sb_categs_not_addable = [];
            var sb_categs_not_deletable = [];
            var sb_categs_not_copyable = [];';

        for ($i = 0; $i < count($this->mCategsSelectedIds); $i++)
        {
            echo '
            var i = sb_categs_selected_ids.length;
            sb_categs_selected_ids[i] = '.intval($this->mCategsSelectedIds[$i]).';
            ';
        }
        
        foreach ($this->mCategsNotDeletable as $cat_id)
        {
            echo 'sb_categs_not_deletable[sb_categs_not_deletable.length] = "'.$cat_id.'";';
        }
         
        foreach ($this->mCategsNotAddable as $cat_id)
        {
            echo 'sb_categs_not_addable[sb_categs_not_addable.length] = "'.$cat_id.'";';
        }
        
        foreach ($this->mCategsNotEditable as $cat_id)
        {
            echo 'sb_categs_not_editable[sb_categs_not_editable.length] = "'.$cat_id.'";';
        }
        
        foreach ($this->mCategsNotCopyable as $cat_id)
        {
            echo 'sb_categs_not_copyable[sb_categs_not_copyable.length] = "'.$cat_id.'";';
        }
           
        echo '
            var sb_elems_before_paste_event = "'.$this->mElemsBeforePasteEvent.'";
            var sb_elems_after_paste_event = "'.$this->mElemsAfterPasteEvent.'";

            var sb_categs_max_deep = '.$this->mCategsMaxDeep.';
            var sb_categs_ident = "'.$this->mCategsIdent.'";
            var sb_categs_closed = '.($this->mCategsClosed ? '1':'0').';
            var sb_categs_count = '.($this->mCategsCount ? '1':'0').';
            var sb_categs_url = '.($this->mCategsUrl ? '1':'0').';
            var sb_categs_moderators = '.($this->mCategsModerators ? '1':'0').';
            var sb_categs_fields = '.($this->mCategsFields ? '1':'0').';
            var sb_categs_autoloading = '.($this->mCategsAutoloading ? '1':'0').';

            var sb_categs_add_dlg_width = '.$this->mCategsAddDlgWidth.';
            var sb_categs_add_dlg_height = '.$this->mCategsAddDlgHeight.';
            var sb_categs_edit_dlg_width = '.$this->mCategsEditDlgWidth.';
            var sb_categs_edit_dlg_height = '.$this->mCategsEditDlgHeight.';

            var sb_categs_before_check_event = "'.$this->mCategsBeforeCheckEvent.'";
            
            var sb_categs_before_add_event = "'.$this->mCategsBeforeAddEvent.'";
            var sb_categs_add_event = "'.$this->mCategsAddEvent.'";
            var sb_categs_after_add_event = "'.$this->mCategsAfterAddEvent.'";

            var sb_categs_before_edit_event = "'.$this->mCategsBeforeEditEvent.'";
            var sb_categs_edit_event = "'.$this->mCategsEditEvent.'";
            var sb_categs_after_edit_event = "'.$this->mCategsAfterEditEvent.'";

            var sb_categs_delete_event = "'.$this->mCategsDeleteEvent.'";
            var sb_categs_delete_with_elements_event = "'.$this->mCategsAfterDeleteWithElementsEvent.'";
            var sb_categs_delete_with_elements_event_before = "'.$this->mCategsBeforeDeleteWithElementsEvent.'";

            var sb_categs_before_paste_event = "'.$this->mCategsBeforePasteEvent.'";
            var sb_categs_before_paste_with_elements_event = "'.$this->mCategsBeforePasteWithElementsEvent.'";
            var sb_categs_after_paste_event = "'.$this->mCategsAfterPasteEvent.'";
            var sb_categs_after_paste_with_elements_event = "'.$this->mCategsAfterPasteWithElementsEvent.'";

            var sb_categs_menu_level = '.$this->mCategsMenuLevel.';
            var sb_categs_add_menu_level = '.$this->mCategsAddMenuLevel.';
            var sb_categs_edit_menu_level = '.$this->mCategsEditMenuLevel.';
            var sb_categs_delete_menu_level = '.$this->mCategsDeleteMenuLevel.';
            var sb_categs_copy_menu_level = '.$this->mCategsCopyMenuLevel.';
            var sb_categs_cut_menu_level = '.$this->mCategsCutMenuLevel.';
            var sb_categs_paste_menu_level = '.$this->mCategsPasteElemsMenuLevel.';
            var sb_categs_paste_menu = "'.strtolower($this->mCategsPasteElemsMenu).'";
            var sb_categs_paste_menu_with_elements = "'.strtolower($this->mCategsPasteWithElementsMenu).'";
            var sb_categs_pastelinks_menu = "'.($this->mCategsPasteLinksMenu ? 'true' : 'false').'";
            var sb_categs_paste_categs_menu_level = '.$this->mCategsPasteCategsMenuLevel.';

            var sb_categs_add_menu = [];';

        if (count($this->mCategsAdditionalMenu) > 0)
        {

            for ($i = 0; $i < count($this->mCategsAdditionalMenu); $i++)
                echo '
                var i = sb_categs_add_menu.length;
                sb_categs_add_menu[i] = [];
                sb_categs_add_menu[i]["level"] = '.$this->mCategsAdditionalMenu[$i]['level'].';
                sb_categs_add_menu[i]["item"] = "'.$this->mCategsAdditionalMenu[$i]['item'].'";
                sb_categs_add_menu[i]["func"] = "'.$this->mCategsAdditionalMenu[$i]['func'].'";
                ';
        }

        echo 'function sbLoadCategs()
            {
                sbCatTree=new sbTree(sbGetE("categs_list"), "100%", "100%", 0);
                sbCatTree.imPath = sb_cms_img_url + "/tree/";
                '.(!$this->mCategsRubrikator ? 'sbCatTree.checkBoxes=false;' : 'sbCatTree.checkFuncHandler=sbCategsCheck;').
                (!$this->mCategsDad || !$this->mCategsMenu? 'sbCatTree.dragAndDrop=false;' : 'sbCatTree.dadmode='.$this->mCategsDadMode.';sbCatTree.dragFunc=sbCategsPasteCategs;').
                ($this->mCategsMultiSelect ? 'sbCatTree.multiSelect=true;':'').'
                sbCatTree.aFunc=sbCategClick;
                sbCatTree.cMenu = sbCatMenu;
                sbCatTree.dadlevel = '.$this->mCategsDadStartLevel.';
                sbCatTree.checkBoxesLevel = '.$this->mCategsRubrikatorLevel.';
                '.($this->mCategsAutoloading ? 'sbCatTree.setXMLAutoLoading(sb_cms_empty_file + "?event=pl_categs_read&cat_ident='.$this->mCategsIdent.'&title_handler='.$this->mCategsTitleHandler.'&cat_rubrik='.$this->mCategsRubrikator.'&cat_count='.$this->mCategsCount.'&cat_closed='.$this->mCategsClosed.'&cat_rights='.$this->mCategsUseRights.'&cat_never_show='.implode(',', $this->mCategsNeverShowCats).'&cat_only_show='.implode(',', $this->mCategsShowOnlyCats).'");' : '').
                'sbCatTree.loadXML(sb_cms_empty_file + "?event=pl_categs_read&cat_ident='.$this->mCategsIdent.'&title_handler='.$this->mCategsTitleHandler.'&cat_rubrik='.$this->mCategsRubrikator.'&cat_count='.$this->mCategsCount.'&cat_closed='.$this->mCategsClosed.'&cat_rights='.$this->mCategsUseRights.'&cat_sel_id='.$this->mCategsSelectedId.'&cat_autoloading='.$this->mCategsAutoloading.'&cat_never_show='.implode(',', $this->mCategsNeverShowCats).'&cat_only_show='.implode(',', $this->mCategsShowOnlyCats).'&cat_sel_ids='.implode(',', $this->mCategsSelectedIds).'", sbCategsAfterLoad);
            }

            sbAddEvent(window, "load", sbLoadCategs);';

        if ($this->mCategsRubrikator)
        {
            echo '
            var sb_categs_not_checkable = [];
            ';
            foreach ($this->mCategsNotCheckable as $cat_id)
            {
                echo 'sb_categs_not_checkable[sb_categs_not_checkable.length] = "'.$cat_id.'";';
            }
            
            echo 'function sbCategsCheck(catId, state)
            {
                for (var i = 0; i < sb_categs_not_checkable.length; i++)
                {
                    if (sb_categs_not_checkable[i] == catId)
                    {
                        return 0;
                    }
                }
                
                if (sb_categs_before_check_event != "")
                {
                    var res = sbLoadSync(sb_cms_empty_file + "?event="+sb_categs_before_check_event+"&cat_id=" + catId);
                    if (res != "")
                    {
                        sbShowMsgDiv(res, "warning.png");
                        return 0;
                    }
                }
                
                var res = sbLoadSync(sb_cms_empty_file + "?event=pl_categs_check&cat_id="+catId+"&state="+state+"&plugin="+sb_elems_plugin+"&plugin_ident="+sb_elems_plugin_ident);
                if (res != "")
                {
                    sbShowMsgDiv(res, "warning.png");
                    return 0;
                }

                return 1;
            }';
        }
        else
        {
            echo 'function sbCategsCheck(catId, state)
            {
                return 0;
            }';
        }

        if ($this->mCategsMenu)
        {
            echo 'var sbCatMenu = new sbCMenu("sbCatMenu", "15#FF9933", "15#fff5e0", "highItemText", "lowItemText", 90);
            sbCatMenu.showFunc = sbShowCategsMenu;';

            if ($this->mCategsPasteElemsMenu != 'none') echo 'sbCatMenu.addItem("paste", "'.$this->mCategsPasteElemsMenuTitle.'", sbElemsPaste);';
            if ($this->mCategsPasteLinksMenu) echo 'sbCatMenu.addItem("paste_links", "'.$this->mCategsPasteLinksMenuTitle.'", sbElemsPasteLinks);';

            echo 'sbCatMenu.addItem("paste_delimeter");';

            if ($this->mCategsCopyMenu)
            {
                echo 'sbCatMenu.addItem("copy", "'.$this->mCategsCopyMenuTitle.'", sbCategsCopy);';
            }

            if ($this->mCategsCutMenu)
            {
                echo 'sbCatMenu.addItem("cut", "'.$this->mCategsCutMenuTitle.'", sbCategsCut);';
            }
            
            if ($this->mCategsCopyMenu || $this->mCategsCutMenu)
            {
                echo 'sbCatMenu.addItem("paste_categs", "'.$this->mCategsPasteMenuTitle.'", sbCategsPasteCategs);';
                
            	if ($this->mCategsPasteWithElementsMenu) 
            	{
            		echo 'sbCatMenu.addItem("paste_categs_with_element", "'.$this->mCategsPasteWithElementsMenuTitle.'", sbElemsPasteWithElements);';
            	}
                
            	echo 'sbCatMenu.addItem("paste_categs_delimeter");';
            }

            if ($this->mCategsAddMenu)
            {
                echo 'sbCatMenu.addItem("add", "'.$this->mCategsAddMenuTitle.'", sbCategsAdd);';
                if ($this->mCategsAfterAddFunc != '')
                    echo 'var sb_categs_after_add_func = '.$this->mCategsAfterAddFunc.';';
            }

            if ($this->mCategsEditMenu)
            {
                echo 'sbCatMenu.addItem("edit", "'.$this->mCategsEditMenuTitle.'", sbCategsEdit);';
                if ($this->mCategsAfterEditFunc != '')
                    echo 'var sb_categs_after_edit_func = '.$this->mCategsAfterEditFunc.';';
            }

            if ($this->mCategsDeleteMenu)
            {
                echo 'sbCatMenu.addItem("delete", "'.$this->mCategsDeleteMenuTitle.'", sbCategsDelete);';
            }
            if($this->mCategsDeleteWithElementsMenu)
            {
            	echo 'sbCatMenu.addItem("delete_with_elements", "'.$this->mCategsDeleteWithElementsMenuTitle.'", sbCategsDeleteWithElements);';
            }

            if ($this->mCategsRefreshMenu)
            {
                echo 'sbCatMenu.addItem("refresh", "'.SB_CATEGS_REFRESH_MENU.'", sbCategsRefresh);';
            }

            echo 'sbCatMenu.addItem("delimeter");';

            if ($this->mCategsDad && $this->mCategsDadMode != 0)
            {
            	echo 'sbCatMenu.addItem("up", "'.SB_CATEGS_UP_MENU.'", sbCategsUp);
            	sbCatMenu.addItem("down", "'.SB_CATEGS_DOWN_MENU.'", sbCategsDown);
            	sbCatMenu.addItem("updown_delimeter");
            	';
            }
            
            if (count($this->mCategsAdditionalMenu) > 0)
            {
                for ($i = 0; $i < count($this->mCategsAdditionalMenu); $i++)
                    echo 'sbCatMenu.addItem("additional_'.$i.'", "'.$this->mCategsAdditionalMenu[$i]['item'].'", "'.$this->mCategsAdditionalMenu[$i]['func'].'");';

                echo 'sbCatMenu.addItem("additional_delimeter");';
            }

            if ($this->mCategsRightsMenu)
            {
                echo 'sbCatMenu.addItem("rights", "'.SB_CATEGS_RIGHTS_MENU.'", sbCategsRights);';
            }
        }

        echo '</script>';
    }

    /**
     * Устанавливает событие, вызываемое для подгрузки элементов раздела
     *
     * @param string $event Идентификатор события, вызываемого для подгрузки элементов раздела.
     */
    public function setCategsEvent($event)
    {
        $this->mCategsEvent = $event;	
    }
    
    /**
     * Выводит дерево разделов
     *
     */
    public function showTree($footer_str = '', $header_str = '', $on_click_func = '', $echo = true)
    {
        $result = '<script>
            function sbCategClick(el)
            {
                '.($on_click_func != '' ? $on_click_func.'(); ':'').'
                return;
            }
            
            function sbResizeCategsDiv()
            {
                var cl = sbGetE("categs_list_td");
                var clp = sbGetE("sb_tabs_con");
                var clf = sbGetE("categs_list_footer_td");
                var clh = sbGetE("categs_list_header_td");
                var sub = 5;
                
                if (!clp)
                {
                    clp = document.body;
                    sub = 0;
                }
                   
                cl.style.height = Math.max(0, parseInt(clp.clientHeight) - (clf ? parseInt(clf.offsetHeight) : 0) - (clh ? parseInt(clh.offsetHeight) : 0) - 25 - sub);
                if (sbCatTree) 
                {
                    if (_isIE)
                        sbCatTree.allTree.style.height = cl.style.height;
                    else
                        sbCatTree.allTree.style.height = cl.offsetHeight - 10;
                }
                cl.style.visibility = "visible";
                if (clf)
                    clf.style.visibility = "visible";
                if (clh)
                    clh.style.visibility = "visible";
            }
            sbResizeFuncs[sbResizeFuncs.length]=sbResizeCategsDiv;
        </script>
        <table cellpadding="0" cellspacing="0" width="100%">';
            
        if ($header_str != '')
            $result .= '<tr><td><div id="categs_list_header_td" style="visibility:hidden;">'.$header_str.'</div></td></tr>';
            
        $result .= '<tr><td valign="top" id="categs_list_td" style="visibility: hidden;height:100%;"><div id="categs_list" style="border-top: 0px; background-color: #EFE5D0;"></div></td></tr>';
        
        if ($footer_str != '')
            $result .= '<tr><td><div id="categs_list_footer_td" style="visibility:hidden;">'.$footer_str.'</div></td></tr>';
        
        $result .= '</table>';
        
        if ($echo)
            echo $result;
            
        return $result;
    }
    
    /**
     * Выводит XML для дерева разделов
     * 
     * @param boolean $view Выводить в браузер (true) или возвращать как строку (false)
     */
    private function show($view = true)
    {
        // инициализация переменных класса
        if (isset($_GET['id']))
            $root_id = intval($_GET['id']);
        else
            $root_id = -1;

        $this->mCategsRubrikator = (bool)$_GET['cat_rubrik'];
        $this->mCategsClosed = (bool)$_GET['cat_closed'];
        $this->mCategsUseRights = (bool)$_GET['cat_rights'];
        $this->mCategsCount = (bool)$_GET['cat_count'];
        $this->mCategsTitleHandler = isset($_GET['title_handler']) ? trim($_GET['title_handler']) : '';
         
        if ($_GET['cat_never_show'] != '')
            $this->mCategsNeverShowCats = explode(',', $_GET['cat_never_show']);
        else    
            $this->mCategsNeverShowCats = array();
        
        if ($_GET['cat_only_show'] != '')
            $this->mCategsShowOnlyCats = explode(',', $_GET['cat_only_show']);
        else    
            $this->mCategsShowOnlyCats = array();
        
        //Исключаем из списка запрещенных идентификаторов те, которые разрешены
        if(!empty($this->mCategsShowOnlyCats) && !empty($this->mCategsNeverShowCats))
        {
            $this->mCategsNeverShowCats = array_diff($this->mCategsNeverShowCats, $this->mCategsShowOnlyCats);
        }
            
        if (isset($_GET['cat_sel_ids']) && $_GET['cat_sel_ids'] != '')
            $this->mCategsSelectedIds = explode(',', $_GET['cat_sel_ids']);
        else 
            $this->mCategsSelectedIds = array();
        
        if (isset($_GET['cat_autoloading']))
            $this->mCategsAutoloading = (bool)$_GET['cat_autoloading'];

        if (isset($_GET['cat_sel_id']) && !empty($_GET['cat_sel_id']))
	    {
	        $_GET['cat_sel_id'] = intval($_GET['cat_sel_id']);
	        $_SESSION['sb_categs_selected_id'][$this->mCategsIdent] = $_GET['cat_sel_id'];
		    $this->mCategsSelectedId = $_GET['cat_sel_id'];
	    }

	    $allowed_cats = array();
	    $sel_cat_id = array();
	    $sel_level = 999;
        $tree_id = 0;

	    if ($this->mCategsAutoloading)
	    {
	        // используется подгрузка
	        $sign_less = '<';
	        $sign_more = '>';
	        $level_incr = 4;
            if ($root_id == -1)
    	    {
    	        $res = sql_query('SELECT cat_id FROM sb_categs WHERE cat_level=0 AND cat_ident=?', $this->mCategsIdent);
    	        if (!$res) return;

    	        list($root_id) = $res[0];
    	        $sign_less = '<=';
    	        $sign_more = '>=';
    	        $level_incr = 3;
    	    }
    	    else
    	    {
    	        $tree_id = $root_id;
    	        $this->mCategsSelectedId = -1;
    	    }

            $result = sql_query('SELECT cat_left, cat_right, cat_level FROM sb_categs WHERE cat_id=?d', $root_id);
            if (!$result)
                return;

            list ($root_left, $root_right, $root_level) = $result[0];

            if ($this->mCategsUseRights && !$_SESSION['sbAuth']->isAdmin())
            {
                $result = sql_query('SELECT cat_id, cat_left, cat_right, cat_level, cat_title, cat_rubrik, cat_closed
                                           FROM sb_categs WHERE cat_ident=? AND cat_right '.$sign_less.' ?d AND cat_left '.$sign_more.' ?d
                                           ORDER BY cat_left ASC',
                                           $this->mCategsIdent, $root_right, $root_left);

                $res = sql_query('SELECT cat_id, cat_level, cat_rights FROM sb_categs
                                  WHERE cat_ident=? AND cat_right '.$sign_less.' ?d AND cat_left '.$sign_more.' ?d
                                  ORDER BY cat_left ASC',
                                  $this->mCategsIdent, $root_right, $root_left);

                $allowed_cats = array();
                for ($i = 0; $i < count($res); $i++)
                {
                    list($right_cat_id, $rights_cat_level, $rights_cat_rights) = $res[$i];
                    
                    if (fCategs_Check_Rights($rights_cat_rights, true))
                    {
                        $allowed_cats[] = $right_cat_id;
                    }
                }
            }
            else
            {
                $result = sql_query('SELECT cat_id, cat_left, cat_right, cat_level, cat_title, cat_rubrik, cat_closed
                                           FROM sb_categs WHERE cat_ident=? AND cat_right '.$sign_less.' ?d AND cat_left '.$sign_more.' ?d
                                           ORDER BY cat_left ASC', $this->mCategsIdent, $root_right, $root_left);
            }

    	    if (count($this->mCategsSelectedIds) != 0)
    	    {
    	        for ($i = 0; $i < count($this->mCategsSelectedIds); $i++)
    	        {
    	            if (!in_array($this->mCategsSelectedIds[$i], $sel_cat_id))
    	            {
            	        $res = sql_query('SELECT cat_left, cat_right, cat_level FROM sb_categs WHERE cat_id=?d', $this->mCategsSelectedIds[$i]);
                        if ($res)
                        {
                            list($cat_left, $cat_right, $cat_level) = $res[0];
                            if ($cat_left > $root_left && $cat_right < $root_right && $cat_level > $root_level + $level_incr - 2)
                            {
                            	if ($cat_level > $sel_level)
                                	$sel_level = $cat_level;
                                	
                                $res = sql_query('SELECT cat_id, cat_left, cat_right FROM sb_categs
                                                        WHERE cat_ident=? AND cat_right >= ?d
                                                        AND cat_left <= ?d AND cat_level = 1',
                                                        $this->mCategsIdent, $cat_right, $cat_left);
        
                                if ($res)
                                {
                                    list($cat_id, $cat_left, $cat_right) = $res[0];
                                    if (!in_array($cat_id, $sel_cat_id))
                                    {
                                        $res = sql_query('SELECT cat_id FROM sb_categs
                                                          WHERE cat_ident=? AND cat_left > ?d
                                                          AND cat_right <= ?d',
                                                          $this->mCategsIdent, $cat_left, $cat_right);
            
                                        for ($j = 0; $j < count($res); $j++)
                                        {
                                            list($sel_cat_id[]) = $res[$j];
                                        }
                                    }
                                }
                            }
                        }
    	            }
    	        }
    	    }
    	    elseif ($this->mCategsSelectedId != -1)
    	    {
    	        $res = sql_query('SELECT cat_left, cat_right, cat_level FROM sb_categs WHERE cat_id=?d', $this->mCategsSelectedId);
                if ($res)
                {
                    list($cat_left, $cat_right, $cat_level) = $res[0];
                    if ($cat_left > $root_left && $cat_right < $root_right && $cat_level > $root_level + $level_incr - 2)
                    {
                        $sel_level = $cat_level;
                        $res = sql_query('SELECT cat_left, cat_right FROM sb_categs
                                                WHERE cat_ident=? AND cat_right >= ?d
                                                AND cat_left <= ?d AND cat_level = 1',
                                                $this->mCategsIdent, $cat_right, $cat_left);

                        if ($res)
                        {
                            list($cat_left, $cat_right) = $res[0];
                            $res = sql_query('SELECT cat_id FROM sb_categs
                                              WHERE cat_ident=? AND cat_left > ?d
                                              AND cat_right <= ?d',
                                              $this->mCategsIdent, $cat_left, $cat_right);

                            for ($i = 0; $i < count($res); $i++)
                            {
                                list($sel_cat_id[]) = $res[$i];
                            }
                        }
                    }
                }
    	    }
	    }
	    else
	    {
	        // дерево загружается все сразу
	        if ($this->mCategsUseRights && !$_SESSION['sbAuth']->isAdmin())
            {
                $result = sql_query('SELECT DISTINCT cat_id, cat_left, cat_right, cat_level, cat_title, cat_rubrik, cat_closed, cat_rights
                                           FROM sb_categs WHERE cat_ident=? ORDER BY cat_left ASC',
                                           $this->mCategsIdent);
                                     
                for ($i = 0; $i < count($result); $i++)
                {
                    $rights_cat_level = $result[$i][3];
                    $rights_cat_rights = $result[$i][7];
                    
                    if ($rights_cat_level == 0)
                    {
                        continue;
                    }
                    
                    if (!fCategs_Check_Rights($rights_cat_rights, true))
                    {
                        $result = array_splice($result, $i, 1);
                        $i--;
                    }
                }
            }
            else
            {
                $result = sql_query('SELECT cat_id, cat_left, cat_right, cat_level, cat_title, cat_rubrik, cat_closed
                                     FROM sb_categs WHERE cat_ident=? ORDER BY cat_left ASC',
                                     $this->mCategsIdent);
            }
	    }

        if (!$result)
            return;

        $str = '';
        $num = count($result);
        $old_level = 1;
        $stack = array();
        $closed = false;
        $show = false;
        $open_tags = array();
        $rights = $this->mCategsUseRights && !$_SESSION['sbAuth']->isAdmin() && $this->mCategsAutoloading;

        for ($i = 0; $i < $num; $i++)
        {
            list($cat_id, $cat_left, $cat_right, $cat_level, $cat_title, $cat_rubrik, $cat_closed) = $result[$i];
            if ((!empty($this->mCategsShowOnlyCats) && !in_array($cat_id, $this->mCategsShowOnlyCats)) || in_array($cat_id, $this->mCategsNeverShowCats))
            {
                continue;
            }

            if ($rights && !in_array($cat_id, $allowed_cats)){
                continue;
            }
                
            $count_res = sql_query('SELECT COUNT(*) FROM sb_categs categs, sb_catlinks links WHERE categs.cat_ident=? AND categs.cat_left >= ?d AND categs.cat_right <= ?d AND links.link_cat_id=categs.cat_id', $this->mCategsIdent, $cat_left, $cat_right);
            if ($count_res)
            {
                $cat_count = $count_res[0][0];
            }
            else
            {
                $cat_count = 0;
            }

        	if ($this->mCategsTitleHandler != '' && function_exists($this->mCategsTitleHandler))
            {
              	$cat_title = call_user_func_array($this->mCategsTitleHandler, array($cat_id, $cat_title, $cat_level, $cat_rubrik, $cat_closed));	
            }
                
            $cat_title = sb_htmlspecialchars($cat_title, ENT_QUOTES).($this->mCategsCount ? ' ['.$cat_count.']' : '');
            $cat_rubrik = ($cat_rubrik && $this->mCategsRubrikator ? 1 : 0);
            $cat_closed = ($cat_closed && $this->mCategsClosed ? 1 : 0);

            if (count($stack) > 0)
            {
                while (array_key_exists(count($stack)-1, $stack) && $stack[count($stack)-1] < $cat_right)
                {
                    array_pop($stack);
                }
            }

            $stack[] = $cat_right;
            $new_level = count($stack);
            if ($new_level >= 2)
            {
                $show = in_array($cat_id, $sel_cat_id);
            }
            
            if ($new_level > $old_level)
            {
                $str .= 'child="1">'."\n";
                $open_tags[] = $cat_id;
                if (($new_level > 2 && $this->mCategsAutoloading && !$show) || ($show && $cat_level > $sel_level))
                {
                    array_pop($stack);
                    $cur_level = $cat_level;
                    for ($i = $i + 1; $i < $num; $i++)
                    {
                        list($cat_id, $cat_left, $cat_right, $cat_level, $cat_title, $cat_rubrik, $cat_closed) = $result[$i];
                        if ($cat_level < $cur_level)
                        {
                            break;
                        }
                    }
                    $str .= '</item>'."\n";
                    $closed = true;
                    array_pop($open_tags);
                    if ($cat_level >= $cur_level)
                    {
                        break;
                    }
                    array_pop($stack);
                    $i--;
                    continue;
                }
                
                $str .= '<item text="'.$cat_title.'" id="'.$cat_id.'" level="'.($cat_level + 1).'"'.($cat_closed == 1 ? ' secure="1"':'').($cat_rubrik==1 ? ' checked="1"':'').' ';
                $closed = false;
                $old_level = $new_level;
            }
            else if ($new_level < $old_level)
            {
                $str .= (!$closed ? '/>'."\n" : '');
                for ($j = 0; $j < $old_level-$new_level; $j++)
                {
                    $str .= '</item>'."\n";
                    array_pop($open_tags);
                }
                $old_level = $new_level;
                
                $str .= '<item text="'.$cat_title.'" id="'.$cat_id.'" level="'.($cat_level + 1).'"'.($cat_closed == 1 ? ' secure="1"':'').($cat_rubrik==1 ? ' checked="1"':'').' ';
                $closed = false;
            }
            else
            {
                $str .= (!$closed && $str != '' ? '/>'."\n" : '').'<item text="'.$cat_title.'" id="'.$cat_id.'" level="'.($cat_level + 1).'"'.($cat_level==0 ? ' open="1"':'').($cat_closed == 1 ? ' secure="1"':'').($cat_rubrik==1 ? ' checked="1"':'').' ';
                $closed = false;
            }
        }

        if (!$closed && $str != '') $str .= '/>'."\n";

        for ($i = 0; $i < count($open_tags); $i++)
            $str .= '</item>'."\n";

        $str = '<?xml version="1.0" encoding="'.SB_CHARSET.'" ?>'."\n".'<tree id="'.$tree_id.'">'."\n".$str.'</tree>';

        if($view){
            echo $str;
        }else{
            return $str;
        }
    }
    
    /**
     * Обертка над функцией show
     * @param type $view
     * @return type 
     */
    public function showXML($view = true){
        return $this->show($view);
    }
    
    /**
     * Выборка списка категорий без привязки к событию
     * @param string $categ
     * @return string 
     */
    static function getCategoriesXML($categ){
        $_GET['event'] = 'pl_categs_read';
        $_GET['cat_rubrik'] = '';
        $_GET['cat_closed'] = '';
        $_GET['cat_rights'] = '';
        $_GET['cat_count'] = '';
        $_GET['cat_never_show'] = '';
        $obj = new sbJustCategs($categ);
        return $obj->showXML(false);
    }
}
?>