<?php
/**
 * Реализация класса работы с базой морфологии
 *
 * С помощью данного класса реализуется поиск всех словоформ заданного слова.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Plugins
 * @copyright Copyright (c) 2009, OOO "СИБИЭС Групп"
 */

define ('SB_SEARCH_ALF', 20);
define ('SB_SEARCH_TALF', pow(SB_SEARCH_ALF, 3) * 4);

/**
 * Класс работы с базой морфологии
 *
 * С помощью данного класса реализуется поиск всех словоформ заданного слова.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Plugins
 * @copyright Copyright (c) 2009, OOO "СИБИЭС Групп"
 */
class sbSearch
{
	/**
	 * Дескриптор базы морфологии.
	 *
	 * @var handle
	 */
	private $mHandle = null;
	
	private $mProfiles = array();
	
	private $mProfilesLength = array();
	
	/**
	 * Конструктор класса
	 *
	 */
	function __construct()
    {
        $this->mHandle = @fopen(SB_CMS_LIB_PATH.'/sbSearch.db', 'rb');
    }
    
    private function get_word($word)
    {
    	if (SB_CHARSET != 'WINDOWS-1251')
    		return iconv('WINDOWS-1251', SB_CHARSET.'//IGNORE', $word);
    		
    	return $word;
    }
    
    private function set_word($word)
    {
    	if (SB_CHARSET != 'WINDOWS-1251')
			return iconv(SB_CHARSET, 'WINDOWS-1251//IGNORE', sb_strtolower($word));

		return $word;
    }
    
	private function get_word_HD($word)
	{
		$res = array();
		if (!$this->mHandle)
			return $res;
		
	    $next_pos = 0;
	    if (strlen($word) > 2)
	    {
	        $pos = (ord($word[0]) % SB_SEARCH_ALF + (ord($word[1]) % SB_SEARCH_ALF) * SB_SEARCH_ALF + (ord($word[2]) % SB_SEARCH_ALF) * SB_SEARCH_ALF * SB_SEARCH_ALF) * 4;
	    }
	    elseif (strlen($word) == 2)
	    {
	        $pos = (ord($word[0]) % SB_SEARCH_ALF + (ord($word[1]) % SB_SEARCH_ALF) * SB_SEARCH_ALF) * 4;
	    }
	    elseif (strlen($word) == 1)
	    {
	        $pos = (ord($word[0]) % SB_SEARCH_ALF) * 4;
	    }
	    else
	    {
	        return $res;
	    }
	
	    @fseek($this->mHandle, $pos, 0);
	    $dum = @unpack('Npos', @fread($this->mHandle, 4));
	    $next_pos = $dum['pos'];
	    if ($next_pos == 0)
	    {
	        return $res;
	    }
	
	    $base = substr($word, 0, 3);
	
	    @fseek($this->mHandle, $next_pos + 1, 0);
	    $dum = @unpack('Ndum', @fread($this->mHandle, 4));
	    $pos = $dum['dum'];
	
	    while ($pos)
	    {
	        @fseek($this->mHandle, $pos, 0);
	        $dum = unpack("Clen/Npos", @fread($this->mHandle, 5));
	        $len = $dum['len'];
	        $res[] = @fread($this->mHandle, $len);
	        $pos = $dum['pos'];
	    }
	    $next_pos += 5;
	
	    $base = '';
	    for($i = 0; $i < strlen($word); $i++)
	    {
	        $letter = ord($word[$i]);
	        $base .= $word[$i];
	
	        while (1)
	        {
	            @fseek($this->mHandle, $next_pos, 0);
	        	$dum = unpack("Clet/Nless/Nequal/Nbigger/Nval_pos", fread($this->mHandle, 17));
	          	if ($letter == $dum['let'])
	          	{
	          	    $next_pos = $dum['equal'];
	          	    break;
	          	}
	          	if ($letter < $dum['let'])
	          	{
	          	    $next_pos = $dum['less'];
	          	}
	          	else
	          	{
	          	    $next_pos = $dum['bigger'];
	          	}
	          	if ($next_pos == 0)
	          	{
	          	    break;
	          	}
	        }
	
	        $val_pos = $dum['val_pos'];
	        $let = $dum['let'];
	        $equal = $dum['equal'];
	    	if ( $let == $letter && $val_pos != 0 )
	    	{
	    	    while ($val_pos != 0)
	    	    {
	    	     	@fseek($this->mHandle, $val_pos, 0);
	    	     	$dum = unpack("Clen/Nval_pos", @fread($this->mHandle, 5));
	    	     	$len = $dum['len'];
	    	     	$val_pos = $dum['val_pos'];
	    	     	$res[] = $base.@fread($this->mHandle, $len);
	    	    }
	    	}
	
	    	if ($next_pos == 0)
	    	{
	    	    break;
	    	}
	
	    }
	
	    return $res;
	}
	
	/**
	 * Возвращает базовые формы слова.
	 *
	 * @param string $query Слово.
	 * 
	 * @return array Массив базовых форм слова.
	 */
	public function get_base_form($query)
	{
	    $base = array();
	    if (!$this->mHandle)
			return $base;
			
	    $query = $this->set_word($query);
	    $words = $this->get_word_HD($query);
	
	    foreach ($words as $word)
	    {
	        $flag = substr($word, strpos($word, '/') + 1);
	        $word = substr($word, 0, strpos($word, '/'));
	        if ($word == $query)
	        {
	            $base[] = $this->get_word($word);
	        }
	        else
	        {
	            $dum = unpack("nflag", $flag);
	            $flag = $dum['flag'];
	            if ( !isset($this->mProfiles[$flag] ) )
	            {
	                @fseek($this->mHandle, SB_SEARCH_TALF + 4 + $flag * 6,0);
	                $dum = unpack("Npos/nlen", @fread($this->mHandle, 6));
	                @fseek($this->mHandle, $dum['pos'], 0);
	                $dum = @fread($this->mHandle, $dum['len']);
	                $this->mProfiles[$flag] = $dum;
	                $this->mProfilesLength[$flag] = strpos($dum, '|', 1) - 1;
	            }
	            
	            $pref_len = strlen($word) - $this->mProfilesLength[$flag];
	            
	            if ($pref_len > strlen($query))
	            {
	                continue;
	            }
	            
	            if ( $pref_len > 0 && strpos($query, substr($word, 0, $pref_len)) !== 0 )
	            {
	                continue;
	            }
	            
	            $aff = substr($query, $pref_len);
	            
	            if ( strpos($this->mProfiles[$flag], "|$aff|") !== FALSE)
	            {
	                $base[] = $this->get_word($word);
	            }
	        }
	    }
	
	    return $base;
	}
	
	private function gen_all_words($word, $flag)
	{
	    $aff = preg_split ('/\|/', substr($this->mProfiles[$flag], 1));
	    unset($aff[count($aff)-1]);
	    foreach ($aff as $k => $v)
	    {
	        $aff[$k] = $word.$v;
	    }
	    return $aff;
	}
	
	/**
	 * Возвращает массив словоформ для переданного слова.
	 *
	 * @param string $query Слово, для которого ищем словоформы.
	 * 
	 * @return array Массив словоформ.
	 */
	public function get_all_forms($query)
	{
	    $forms = array();
	    if (!$this->mHandle)
			return $forms;
			
		$query = $this->set_word($query);
	    $words = $this->get_word_HD($query);
	    
	    foreach ($words as $word)
	    {
	        $flag = substr($word, strpos($word, '/') + 1);
	        $word = substr($word, 0, strpos($word, '/'));
	        $dum = unpack("nflag", $flag);
	        $flag = $dum['flag'];

	        if ( !isset($profiles[$flag]) )
	        {
	            @fseek($this->mHandle, SB_SEARCH_TALF + 4 + $flag * 6,0);
	            $dum = unpack("Npos/nlen", @fread($this->mHandle, 6));
	            $pos = $dum['pos'];
	            $profile_length = $dum['len'];
	            @fseek($this->mHandle, $pos, 0);
	            $dum = @fread($this->mHandle, $profile_length);
	            $this->mProfiles[$flag] = $dum;
	            $this->mProfilesLength[$flag] = strpos($dum, '|', 1) - 1;
	        }
	        
	        if ($word == $query)
	        {
	            $new_forms = $this->gen_all_words(substr($word, 0, strlen($word) - $this->mProfilesLength[$flag]), $flag);
	            foreach ($new_forms as $v)
	            {
	                $forms[] = $this->get_word($v);
	            }
	        }
	        else
	        {
	            $pref_len = strlen($word) - $this->mProfilesLength[$flag];
	            if ($pref_len > strlen($query))
	            {
	                continue;
	            }
	            
	            if ( $pref_len > 0 && strpos($query, substr($word, 0, $pref_len)) !== 0 )
	            {
	                continue;
	            }
	            
	            $aff = substr($query, $pref_len);
	            if ( strpos($this->mProfiles[$flag], "|$aff|") !== FALSE)
	            {
	                $new_forms = $this->gen_all_words(substr($word, 0, strlen($word) - $this->mProfilesLength[$flag]), $flag);
	                foreach ($new_forms as $v)
	                {
	                    $forms[] = $this->get_word($v);
	                }
	            }
	        }
	    }
	    return $forms;
	}
}
?>