<?php
define ('SB_SOAP_INSERT_AND_UPDATE', '1'); // добавлять и изменять элементы
define ('SB_SOAP_ONLY_INSERT', '2'); // только добавлять элементы
define ('SB_SOAP_ONLY_UPDATE', '3'); // только изменять элементы
define ('SB_SOAP_INSERT', '4'); // добавлять элемент, даже если есть элемент с таким айдишником

/**
 * Класс работы для с SOAP
 *
 * С помощью данного класса реализуется работа через SOAP
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Plugins
 * @copyright Copyright (c) 2009, OOO "СИБИЭС Групп"
 */
class sbSoap
{
	/**
     * Идентификатор модуля
     *
     * @var string
     */
	private $mPlugin = '';

	/**
     * Имя плагина (title)
     *
     * @var string
     */
	private $mPluginName = '';

	/**
     * Таблица для данных плагина в БД
     *
     * @var string
     */
	private $mTable = '';

	/**
     * Поле в таблице плагина, хранящее чпу элементов
     *
     * @var string
     */
	private $mUrlField = '';

	/**
     * Поле в таблице плагина, хранящее id элементов
     *
     * @var string
     */
	private $mIdField = '';

	/**
     * Поле в таблице плагина, хранящее названия элементов
     *
     * @var string
     */
	private $mNameField = '';

	/**
     * Поле в таблице плагина, хранящее внешние id элементов
     *
     * @var string
     */
	private $mExtIdField = '';

	/**
     * Поле в таблице плагина, хранящее внешние статус публикации
     *
     * @var string
     */
	private $mWorkflowField = '';

	/**
     * Является ли подключившийся пользователь администратором
     *
     * @var bool
     */
	private $mIsAdmin = false;

	/**
     * Идентификатор подключившегося пользователя
     *
     * @var int
     */
	private $mUserId = '';

	/**
     * Логин подключившегося пользователя
     *
     * @var string
     */
	private $mUserLogin = '';

	/**
     * Массив разделов в которых находится пользователь
     *
     * @var array
     */
	private $mUserCats = array();

	/**
     * Ассоциативный массив, key - коды сообщений, text - тексты ошибок
     *
     * @var array
     */
	private $mMess = '';

	/**
     * Ассоциативный массив, права для полей разделов и элементов плагина
     *
     * @var array
     */
	private $mFieldsRight = array();

	/**
     * Массив ошибок при доступе к полям разделов и разделов
     *
     * @var array
     */
	private $mFieldsRightErrors = array();

	/**
     * Ассоциативный массив, права доступа к модулю
     *
     * @var array
     */
	private $mPluginRight = array();

	/**
     * Массив ошибок при нехватки прав на удаление разделов или элементов модуля
     *
     * @var array
     */
	private $mPluginRightDelErrors = array('elems' => false, 'categs' => false);

	/**
     * Массив, цепочки публикации доступные пользователю
     *
     * @var array
     */
	private $mWorkFlowRight = array();

	/**
     * Массив, существующие цепочки публикации
     *
     * @var array
     */
	private $mWorkFlow = array();

	/**
	 * Массив, пользовательские поля для разделов
	 */
	private $mFieldsCats = array();

	/**
	 * Массив, пользовательские поля для элементов
	 */
	private $mFieldsElems = array();

	/**
	 * Родительский раздел для ветки, если нужно добавить в раздел, отличный от указанного в xml
	 */
	public $mCatId = 0;

	/**
	 * Если установлен параметр, то добавятся записи и разделы относящиеся только к этому плагину
	 */
	public $mPluginOnly = '';

	/**
	 * Если true то скачивать изображения по http адресу переданному в поле
	 */
	public $mDownloadPics = false;

	/**
	 * Директория, в которую сохраняются закачанные через http:// изображения
	 */
	public $mDownloadPicsFolder = '';

	/**
	 * Массив настроек пользовательских полей
	 */
	private $mUserFieldsSettings = array();

	/**
	 * Кол-во отредактированных разделов
	 */
	public $mNumOfEditedCats = 0;

	/**
	 * Кол-во добавленных разделов
	 */
	public $mNumOfAddedCats = 0;

	/**
	 * Кол-во отредактированных элементов
	 */
	public $mNumOfEditedEls = 0;

	/**
	 * Кол-во добавленных элементов
	 */
	public $mNumOfAddedEls = 0;

	/**
	 * Названия добавленных элементов
	 */
	public $mNameOfSuccessElems = array();

	/**
	 * Название недобавленных элементов
	 */
	public $mNameOfErrorElems = array();

	/**
	 * Названия добавленных разделов
	 */
	public $mNameOfSuccessCats = array();

	/**
	 * Название недобавленных разделов
	 */
	public $mNameOfErrorCats = array();

	/*
	 * Идентификаторы добавленных элементов
	 */
	public $mIdOfSuccessElems = array();

	/*
	 * Идентификаторы отредактированных элементов
	 */
	public $mIdOfSuccessEditElems = array();

	/*
	 * Идентификаторы добавленных разделов
	 */
	public $mIdOfSuccessCats = array();

	/*
	 * Идентификаторы отредактированных разделов
	 */
	public $mIdOfSuccessEditCats = array();

	/**
	 * Логика обновления информации
	 *
	 * @var int
	 */
	public $mUpdateMethod = SB_SOAP_INSERT_AND_UPDATE;

	/**
	 * Конструктор класса.
	 *
	 * @param int $update_method Логика обновления информации.
	 */
	public function __construct($update_method = SB_SOAP_INSERT_AND_UPDATE)
	{
	    $this->mUpdateMethod = $update_method;
	}

	/**
     * Заносит в свойства класса информацию о модуле.
     */
	private function getInfo()
    {
    	// Обновляю массивы прав
    	$this->mFieldsRight = array();
    	$this->mPluginRight = array();
    	$this->mFieldsCats = array();
    	$this->mFieldsElems = array();
    	// Если конструктор модулей
    	if(sb_strpos($this->mPlugin,'pl_plugin_') !== false)
    	{
	    	// Проверка модуля в ключе
	    	if(!sbAuth::checkPlugin('pl_plugin_maker'))
	    	{
	    		$this->mMess .= '<sbmessage code="2010">'.SOAP_RIGTS_PLUGIN_KEY_ERROR.'</sbmessage>';
	        	sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.SOAP_RIGTS_PLUGIN_KEY_ERROR, SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
	    		$this->getReturnXml();
	    		return false;
	    	}
	    	// Установка переменных
    		$pluginId = sb_str_replace('pl_plugin_','',$this->mPlugin);
    		$this -> mTable = 'sb_plugins_'.$pluginId;
    		$this -> mUrlField = 'p_url';
    		$this -> mIdField = 'p_id';
    		$this -> mExtIdField = 'p_ext_id';
    		$this -> mNameField = 'p_title';
    		$this -> mWorkflowField = 'p_active';
    		$res = sql_query ('SELECT pm_title FROM  sb_plugins_maker WHERE pm_id = ?d', $pluginId);
    		if($res && $res[0][0] != '')
    		{
    			$this->mPluginName = $res[0][0];
    		}
    	}
    	elseif($this->mPlugin == 'pl_sprav')
    	{
    		// Если справочник
    		// Установка переменных
    		$this -> mTable = 'sb_sprav';
    		$this -> mUrlField = '';
    		$this -> mIdField = 's_id';
    		$this -> mExtIdField = 's_ext_id';
    		$this -> mNameField = 's_title';
    		$this -> mWorkflowField = '';
    		$this -> mPluginName = SOAP_SPRAV_TITLE;

    	}
    	elseif($this->mPlugin == 'pl_news')
    	{
   			// Новостная лента
	    	// Установка переменных
    		$this -> mTable = 'sb_news';
    		$this -> mUrlField = 'n_url';
    		$this -> mIdField = 'n_id';
    		$this -> mExtIdField = 'n_ext_id';
    		$this -> mNameField = 'n_title';
    		$this -> mWorkflowField = 'n_active';
    		$this -> mPluginName = SOAP_NEWS_TITLE;
    	}
    	elseif($this->mPlugin == 'pl_imagelib')
    	{
   			// Библиотека изображений
	    	// Установка переменных
    		$this -> mTable = 'sb_imagelib';
    		$this -> mUrlField = 'im_url';
    		$this -> mIdField = 'im_id';
    		$this -> mExtIdField = 'im_ext_id';
    		$this -> mNameField = 'im_title';
    		$this -> mWorkflowField = 'im_gal';
    		$this -> mPluginName = SOAP_IMAGELIB_TITLE;
    	}
    	elseif($this->mPlugin == 'pl_faq')
    	{
   			// Вопрос-Ответ
	    	// Установка переменных
    		$this -> mTable = 'sb_faq';
    		$this -> mUrlField = 'f_url';
    		$this -> mIdField = 'f_id';
    		$this -> mExtIdField = 'f_ext_id';
    		$this -> mNameField = 'f_question';
    		$this -> mWorkflowField = 'f_show';
    		$this -> mPluginName = SOAP_FAQ_TITLE;
    	}
    	elseif($this->mPlugin == 'pl_polls')
    	{
    		// Опросы
	    	// Установка переменных
    		$this -> mTable = 'sb_polls';
    		$this -> mUrlField = 'sp_url';
    		$this -> mIdField = 'sp_id';
    		$this -> mExtIdField = 'sp_ext_id';
    		$this -> mNameField = 'sp_question';
    		$this -> mWorkflowField = 'sp_active';
    		$this -> mPluginName = SOAP_POLLS_TITlE;
    	}
    	elseif($this->mPlugin == 'pl_texts')
    	{
   			// Текстовые блоки
	    	// Установка переменных
    		$this -> mTable = 'sb_texts';
    		$this -> mUrlField = '';
    		$this -> mIdField = 't_id';
    		$this -> mExtIdField = 't_ext_id';
    		$this -> mNameField = 't_name';
    		$this -> mWorkflowField = 't_status';
    		$this -> mPluginName = SOAP_TEXTS_TITLE;
    	}
    	elseif($this->mPlugin == 'pl_banners')
    	{
   			// Рекламные компании
	    	// Установка переменных
    		$this -> mTable = 'sb_banners';
    		$this -> mUrlField = '';
    		$this -> mIdField = 'sb_id';
    		$this -> mExtIdField = 'sb_ext_id';
    		$this -> mNameField = 'sb_name';
    		$this -> mWorkflowField = 'sb_active';
    		$this -> mPluginName = SOAP_BANNERS_TITLE;
    	}
    	elseif($this->mPlugin == 'pl_maillist')
    	{
   			// Листы рассылки
	    	// Установка переменных
    		$this -> mTable = 'sb_maillist';
    		$this -> mUrlField = '';
    		$this -> mIdField = 'm_id';
    		$this -> mExtIdField = 'm_ext_id';
    		$this -> mNameField = 'm_name';
    		$this -> mWorkflowField = 'm_active';
    		$this -> mPluginName = SOAP_MAILLIST_TITLE;
    	}
    	elseif($this->mPlugin == 'pl_services_rutube')
    	{
   			// Библиотека видеороликов
	    	// Установка переменных
    		$this -> mTable = 'sb_services_rutube';
    		$this -> mUrlField = 'ssr_url';
    		$this -> mIdField = 'ssr_id';
    		$this -> mExtIdField = 'ssr_ext_id';
    		$this -> mNameField = 'ssr_name';
    		$this -> mWorkflowField = 'ssr_show';
    		$this -> mPluginName = SOAP_SERVICES_RUTUBE_TITLE;
    	}
    	elseif($this->mPlugin == 'pl_site_users')
    	{
   			// Пользователи сайта
	    	// Установка переменных
    		$this -> mTable = 'sb_site_users';
    		$this -> mUrlField = '';
    		$this -> mIdField = 'su_id';
    		$this -> mExtIdField = 'su_ext_id';
    		$this -> mNameField = 'su_name';
    		$this -> mWorkflowField = 'su_status';
    		$this -> mPluginName = SOAP_SITE_USERS_TITLE;
    	}
    	elseif($this->mPlugin == 'pl_users')
    	{
   			// Пользователи системы
	    	// Установка переменных
    		$this -> mTable = 'sb_users';
    		$this -> mUrlField = '';
    		$this -> mIdField = 'u_id';
    		$this -> mExtIdField = 'u_ext_id';
    		$this -> mNameField = 'u_login';
    		$this -> mWorkflowField = 'u_block';
    		$this -> mPluginName = SOAP_USERS_TITLE;
    	}
    	else
    	{
    		return false;
    	}

    	return true;
    }

    /**
     * Добавляет/редактирует разделы/элементы
	 *
	 * @param string $soap_token - soap_token пользователя.
	 * @param string $xml - xml.
	 *
	 * @return string xml - результат выполнения
	 */
	public function plPluginsAdd($soap_token = '', $xml = '')
    {
    	//Проверка soap_token
    	if(!$this->checkSoapToken($soap_token))
    		return $this->getReturnXml();

    	//Проверка xml
    	$struct = $this->checkXml($xml);
    	if(!$struct)
    		return $this->getReturnXml();

     	// Достаю из xml справочник
     	$sprav = $struct->xpath('//sb_plugin[@p_id="pl_sprav"]');
     	foreach($sprav as $value)
     	{
     		$this->mPlugin = (string)$value['p_id'];

     		//Установка свойств класса
     		$this->getInfo();

     		//Права на управление разделами и элементами
	    	if(!$this->checkPluginRight('categs_edit|elems_edit'))
	    	{
	    		// У пользователя нет требуемого уровня
	        	$this->mMess .= '<sbmessage code="2010">'.sprintf(SOAP_RIGTS_PLUGIN_ERROR, $this->mPluginName).'</sbmessage>';
	        	sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_RIGTS_PLUGIN_ERROR, $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
	    		continue;
	    	}

	    	//Добавление/редактирование разделов/элементов
	    	$this->addStruct($value);

	    	return $this->getReturnXml();
     	}

     	foreach($struct->sb_plugin as $value)
     	{
     		$this->mPlugin = (string)$value['p_id'];

     		// Если импорт для конкретного модуля - остальные пропускаем
     		if($this -> mPluginOnly != '' && $this -> mPluginOnly != $this->mPlugin)
     		{
     			// Если импорт из системы и sb_plugin p_id указан неверно
        		$this->mMess .= '<sbmessage code="2024">'.sprintf(SOAP_FIELD_CAT_WRONG_XML_PLUGIN, (isset($_SESSION) ? $_SESSION['sbPlugins']->getPluginTitle($this->mPlugin) : $this->mPlugin), (isset($_SESSION) ? $_SESSION['sbPlugins']->getPluginTitle($this -> mPluginOnly) : $this -> mPluginOnly)).'</sbmessage>';
        		sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_FIELD_CAT_WRONG_XML_PLUGIN, (isset($_SESSION) ? $_SESSION['sbPlugins']->getPluginTitle($this->mPlugin) : $this->mPlugin), (isset($_SESSION) ? $_SESSION['sbPlugins']->getPluginTitle($this -> mPluginOnly) : $this -> mPluginOnly)), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
    			continue;
     		}

     		if(!$this->getInfo())
    			return false;

     		//Права на управление разделами и элементами
    		if(!$this->checkPluginRight('categs_edit|elems_edit'))
    		{
    			// У пользователя нет требуемого уровня
        		$this->mMess .= '<sbmessage code="2010">'.sprintf(SOAP_RIGTS_PLUGIN_ERROR, $this->mPluginName).'</sbmessage>';
        		sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_RIGTS_PLUGIN_ERROR, $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
    			continue;
    		}

    		//Добавление/редактирование разделов/элементов
    		$this->addStruct($value, $this->mCatId);
     	}

    	//Возвращаю xml
    	return $this->getReturnXml();
    }

     /**
	 * Добавляет/редактирует разделы (рекурсия)
	 *
	 * @param string $object - xml объект раздела.
	 * @param string $par_id - идентификатор родительского раздела.
	 *
	 * @return bool - результат добавления
	 */
    private function addStruct($object, $p_id = 0)
    {
    	// Проверяю поля на доступность
    	foreach($object as $k => $v)
    	{
    		if($k == 'sb_cat')
    		{
    			// Добавление/редактирование раздела
	    		$row = array();
	    		$user_row = array();

	    		// Если есть сериализованный массив пользовательских полей
	    		if (($user_cat_fields = $v->xpath('sb_field[@name="cat_fields"]')) != false)
	    		{
	       			$user_row = unserialize(trim((string)$user_cat_fields[0]));
	    		}

	    		foreach($v->sb_field as $v2)
	    		{
	    			$key = (string)$v2->attributes()->name;
	    			if($key == 'cat_id' || $key == 'cat_ident' || $key == 'cat_left' || $key == 'cat_right' || $key == 'cat_level' || $key == 'cat_fields')
	    			{
	    				continue;
	    			}

	    			if($key == 'moderates_list' || $key == 'categs_moderate_email' || $key == 'answer_temp_id' || $key == 'show_prop1' || $key == 'show_prop2' || $key == 'show_prop3')
	    			{
	    				$user_row[$key] = $this->check_string((string)$v2);
	    				continue;
	    			}

	    			if($key == 'cat_url')
	    			{
	    				$row['cat_url'] = sb_check_chpu('', $this->check_string((string)$v2), $row['cat_title'], 'sb_categs', 'cat_url', 'cat_id');
	    				continue;
	    			}

	    			if(sb_strpos($key,'user_f_') !== false)
	    			{
	    				// Проверяю поля на права
    					if(!$this->checkFieldsRight((string)$v2['name'], true))
    					{
    						continue;
    					}

    					$val = $this->check_string((string)$v2);
		    			if(isset($v2->attributes()->ext_id) && (string)$v2->attributes()->ext_id == 'true')
		    			{
		    				// Если есть атрибут ext_id - значит поле содержит внешний идентификатор справочника
		    				$val = $this->getSpravIdByExt($val); // Достаем внутренний идентификатор по внешнему
		    				if(!$val)
		    				{
		    					$f_name = $this->getFieldName($key, true);
		    					$this->mMess .= '<sbmessage code="1020">'.sprintf(SOAP_SPRAV_CATS_ID_ERROR,$f_name, $key, $row['cat_title'], $this->mPluginName).'</sbmessage>';
		            			sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_SPRAV_CATS_ID_ERROR,$f_name, $key, $row['cat_title'], $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
		    					continue;
		    				}
		    			}
    					$user_row[$key] = $val; //Пользовательские
	    			}
	    			else
	    				$row[$key] = $this->check_string((string)$v2);  // Обычные
	    		}

	    		// Внешний идентификатор
	    		if(!isset($row['cat_ext_id']) && isset($v['c_ext_id']))
    				$row['cat_ext_id'] = (string)$v['c_ext_id'];

    			if(!isset($row['cat_title']) || $row['cat_title'] == '')
    			{
    				$this->mMess .= '<sbmessage code="1023">'.sprintf(SOAP_EDIT_ERROR_CATS_NO_TITLE, $this->mPluginName).'</sbmessage>';
					sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_EDIT_ERROR_CATS_NO_TITLE, $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
					continue;
    			}

	    		// Проверяю существование пользовательских полей
	    		$user_row = $this->checkFieldsExist($user_row, true, (isset($row['cat_title']) ? $row['cat_title'] : ''));

    		   // Узнаю какой раздел нужно редактировать
    		   $cat_id = 0;
    			if(isset($v['c_ext_id']) && $v['c_ext_id'] != '')
    			{
    				$res = sql_query('SELECT cat_id FROM  sb_categs WHERE cat_ident = ? AND cat_ext_id = ?d', $this->mPlugin, $v['c_ext_id']);
    				if(isset($res[0][0]) && $res[0][0] != '')
    				{
    					// Редактируем по ext_id
    					$cat_id = $res[0][0];
    				}
    			}
    			elseif(isset($v['c_id']) && $v['c_id'] != '')
    			{
    				$res = sql_query('SELECT COUNT(*) FROM  sb_categs WHERE cat_ident = ? AND cat_id = ?d', $this->mPlugin, $v['c_id']);
    				if(isset($res[0][0]) && $res[0][0] > 0)
    				{
    					// Редактируем по cat_id
    					$cat_id = $v['c_id'];
    				}
    			}

    			if(isset($v['delete']) && $v['delete'] == 'delete')
    			{
    				// Если нужно удалить ветку
    				if($cat_id > 0) // Если неуказан id или ext_id, либо указан неверно - пропускаю
    					$this->deleteStruct($v);
    				continue;
    			}

    			if ($cat_id > 0 && ($this->mUpdateMethod == SB_SOAP_INSERT_AND_UPDATE || $this->mUpdateMethod == SB_SOAP_ONLY_UPDATE))
    			{
    				// Если нужно отредактировать раздел
    				if(!$this->checkCategsRight($cat_id))
    				{
    					$cat_name = $this->getCatName($cat_id);
    					$this->mMess .= '<sbmessage code="1004">'.sprintf(SOAP_RIGTS_CATS_EDIT_ERROR,$cat_name, $this->mPluginName).'</sbmessage>';
            			sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_RIGTS_CATS_EDIT_ERROR, $cat_name, $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
    					if(isset($v->sb_cat))
		    			{
		    				$this->addStruct($v->sb_cat, $cat_id);
		    			}
		    			continue;
    				}

    				// Достаю старые значения пользовательских полей поля
    				$res = sql_query('SELECT cat_fields FROM sb_categs WHERE cat_ident = ? AND cat_id = ?d', $this->mPlugin, $cat_id);
    				$user_values = array();
    				if(isset($res) && $res[0][0] != '')
    					$user_values = unserialize($res[0][0]);

    				// Достаю ключи полей
    				$user_fields = $this->getFields(true);
    				if(is_array($user_fields))
    				{
    					foreach($user_fields as $v2)
    					{
    						// Переопределяю заданными в xml
    						if(isset($user_row['user_f_'.$v2['id']]))
    							$user_values['user_f_'.$v2['id']] = $user_row['user_f_'.$v2['id']];
    					}
    				}

    				// Другие поля в cat_fields
    				if(isset($user_row['moderates_list']))
    					$user_values['moderates_list'] = $user_row['moderates_list'];
    				if(isset($user_row['categs_moderate_email']))
    					$user_values['categs_moderate_email'] = $user_row['categs_moderate_email'];
    				if(isset($user_row['answer_temp_id']))
    					$user_values['answer_temp_id'] = $user_row['answer_temp_id'];
    				if(isset($user_row['answer_temp_id']))
    					$user_values['answer_temp_id'] = $user_row['answer_temp_id'];
    				if(isset($user_row['answer_temp_id']))
    					$user_values['answer_temp_id'] = $user_row['answer_temp_id'];
    				if(isset($user_row['show_prop1']))
    					$user_values['show_prop1'] = $user_row['show_prop1'];
    				if(isset($user_row['show_prop2']))
    					$user_values['show_prop2'] = $user_row['show_prop2'];
    				if(isset($user_row['show_prop3']))
    					$user_values['show_prop3'] = $user_row['show_prop3'];

    				if(isset($user_values) && count($user_values) > 0)
    					$row['cat_fields'] = serialize($user_values);
    				$res = sql_query('UPDATE sb_categs SET ?a WHERE cat_ident = ? AND cat_id = ?d',$row, $this->mPlugin, $cat_id);
    				if(!$res)
    				{
						$this->mMess .= '<sbmessage code="1011">'.sprintf(SOAP_EDIT_ERROR_CATS, (isset($row['cat_title']) ? $row['cat_title'] : ''), $this->mPluginName).'</sbmessage>';
						sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_EDIT_ERROR_CATS, $row['cat_title'],$this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
						$this->mNameOfErrorCats[] = $row['cat_title'];
    					$this->mNumOfEditedCats++;
					}
    				else
					{
    					$this->mMess .= '<sbmessage code="0001">'.sprintf(SOAP_EDIT_OK_CATS, (isset($row['cat_title']) ? $row['cat_title'] : ''), $this->mPluginName).'</sbmessage>';
            			sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_EDIT_OK_CATS, $row['cat_title'], $this->mPluginName), SB_MSG_INFORMATION, $this->mUserId, $this->mUserLogin);
            			$this->mNameOfSuccessCats[] = $row['cat_title'];
            			$this->mIdOfSuccessEditCats[] = $cat_id;
    				}
    				$id = intval($cat_id);
    			}
    			elseif (($cat_id <= 0 && ($this->mUpdateMethod == SB_SOAP_INSERT_AND_UPDATE || $this->mUpdateMethod == SB_SOAP_ONLY_INSERT)) || $this->mUpdateMethod == SB_SOAP_INSERT)
    			{
    				// Если нужно добавить раздел
    				require_once(SB_CMS_LIB_PATH.'/sbTree.inc.php');
    				$tree = new sbTree($this->mPlugin);
    				$par_id = 0;

    				if(!isset($user_row['categs_moderate_email']))
						$user_row['categs_moderate_email'] = '';
					if(!isset($user_row['moderates_list']))
    					$user_row['moderates_list'] = '';
    				if(!isset($row['cat_closed']))
    					$row['cat_closed'] = '0';
    				if(!isset($row['cat_rubrik']))
						$row['cat_rubrik'] = '1';
					if(!isset($row['cat_rights']))
						$row['cat_rights'] = '';
					$row['cat_fields'] = serialize($user_row);

					if(!isset($row['cat_url']))
						$row['cat_url'] = sb_check_chpu('', '', $row['cat_title'], 'sb_categs', 'cat_url', 'cat_id');

					if($p_id != 0)
					{
    					$par_id = $p_id;
					}
    				elseif(!empty($v['c_p_id']))
    				{
    					$par_id = (int)$v['c_p_id'];
    				}

					if($par_id != 0)
					{
						// Проверяю существует ли полученный из xml коренвой раздел
						$res = sql_query('SELECT cat_rights FROM  sb_categs WHERE cat_ident = ? AND cat_id = ?d',$this->mPlugin,$par_id);
						if(!$res || count($res) < 1)
						{
							$par_id = 0;
						}
						else
						{
							$row['cat_rights'] = $res[0][0];
						}
					}

					if($par_id == 0)
					{
						// Если par_id не введен или равен нулю - это раздел корневой.
						// Если корневой раздел уже существует - то устанавливаем
						// значение его id в переменную $par_id
						$res = sql_query('SELECT cat_id, cat_rights  FROM  sb_categs WHERE cat_ident = ? AND cat_level = 0',$this->mPlugin);
						if($res && $res[0][0] != '')
						{
							$par_id = $res[0][0];
							$row['cat_rights'] = $res[0][1];
						}
					}

					// Проверяю права на добавление
    				if(!$this->checkCategsRight($par_id))
    				{
    					// Если нет прав, генерирует ворнинги для ссылки и продолжаем цикл
    					$cat_name = $this->getCatName($par_id);
    					$this->mMess .= '<sbmessage code="1001">'.sprintf(SOAP_RIGTS_CATS_ERROR,$row['cat_title'] ,$cat_name, $this->mPluginName).'</sbmessage>';
            			sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_RIGTS_CATS_ERROR, $row['cat_title'], $cat_name, $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
            			$this->setErrorsForChildren($v, 'NO_RIGHTS_FOR_PARENT');
						continue;
    				}
    				else
    				{
    					// Добавляю раздел
    					$id = $tree->insertNode($par_id, $row);
    				}

					if(!$id)
					{
						// Если раздел не добавлен, генерирует ворнинги для ветки и продолжаем цикл
						$this->mMess .= '<sbmessage code="1011">'.sprintf(SOAP_EDIT_ERROR_CATS, $row['cat_title'], $this->mPluginName).'</sbmessage>';
            			sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_EDIT_ERROR_CATS, $row['cat_title'], $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
            			$this->setErrorsForChildren($v, 'NO_RIGHTS_FOR_PARENT');
            			$this->mNameOfErrorCats[] = $row['cat_title'];
						continue;
					}

					// Сообщение
					$cat_name = $this->getCatName($par_id);
					$this->mMess .= '<sbmessage code="0002">'.sprintf(SOAP_ADD_OK_CATS,$row['cat_title'], $this->mPluginName).'</sbmessage>';

					sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_ADD_OK_CATS, $row['cat_title'], $this->mPluginName), SB_MSG_INFORMATION, $this->mUserId, $this->mUserLogin);

					$this->mNumOfAddedCats++;
            		$this->mNameOfSuccessCats[] = $row['cat_title'];
            			$this->mIdOfSuccessCats[] = $id;
    			}
    			else
    			{
    			    continue;
    			}

    			// Добавление элементов
    			if(isset($v->sb_elem))
    			{
    				foreach($v->sb_elem as $v2)
    				{
	    				if(isset($v2['delete']) && (string)$v2['delete'] == 'delete')
	    				{
	    					// Если нужно удалить
	    					$this->deleteElement($v2, $id);
	    				}
	    				else
	    				{
	    					// Если нужно добавть/отредактировать
	    					$this->addElement($v2, $id);
	    				}
    				}
    			}

    			if(isset($v->sb_cat))
    			{
    				$this->addStruct($v->sb_cat, $id);
    			}
    		}
    	}
    	return true;
    }

     /**
	 * Добавляет/редактирует элементы
	 *
	 * @param string $element - xml объект элемента.
	 * @param string $cat_id - идентификатор родительского раздела.
	 *
	 * @return bool - результат выполнения
	 */
    private function addElement($element, $cat_id)
    {
    	// Добавление/редактирование элемента
    	require_once SB_CMS_LIB_PATH.'/prog/sbFunctions.inc.php';
    	$row = array();

    	$edit_id = -1;
     	// Узнаю какой элемент нужно редактировать
        if(isset($element['e_ext_id']) && trim((string) $element['e_ext_id']) != '')
    	{
    		$el_ext_id = (string) $element['e_ext_id'];
    		$res = sql_query('SELECT '.$this->mIdField.' FROM  '.$this->mTable.' WHERE '.$this->mExtIdField.' = ?', $el_ext_id);
    		if(isset($res[0][0]) && $res[0][0] != '')
    		{
    			// Редактируем по ext_id
    			$edit_id = $res[0][0];
    		}
    	}
    	elseif(isset($element['e_id']) && $element['e_id'] != '')
    	{
    		$el_id = (string) $element['e_id'];
    		$res = sql_query('SELECT COUNT(*) FROM  '.$this->mTable.' WHERE '.$this->mIdField.' = ?d', $el_id);
    		if(isset($res[0][0]) && $res[0][0] > 0)
    		{
    			// Редактируем по e_id
    			$edit_id = $element['e_id'];
    		}
    	}

    	if($edit_id > 0 && ($this->mUpdateMethod == SB_SOAP_INSERT_AND_UPDATE || $this->mUpdateMethod == SB_SOAP_ONLY_UPDATE))
    	{
    	    // Если редактирование
    		// Узнаю в каком разделе сейчас находится элемент
    		$res = sql_query('SELECT c.cat_id FROM sb_categs AS c, sb_catlinks AS l WHERE
    			l.link_el_id = ?d AND c.cat_id = l.link_cat_id AND cat_ident = ?', $edit_id, $this->mPlugin);

    		if(!isset($res[0][0]) || $res[0][0] < 1)
    		{
    			$el_name = (string)$element->sb_field;
    			$this->mMess .= '<sbmessage code="1005">'.sprintf(SOAP_EL_PARENT_CAT_ERROR,$el_name, $this->mPluginName).'</sbmessage>';
            	sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_EL_PARENT_CAT_ERROR,$el_name, $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
    			return false;
    		}
    		$current_cat = $res[0][0];

    		// Проверяю права на раздел, содержащий элемент
    		if(!$this->checkCategsRight($current_cat))
    		{
    			$el_name = (string)$element->sb_field;
    			$cat_name = $this->getCatName($current_cat);
    			$this->mMess .= '<sbmessage code="1006">'.sprintf(SOAP_RIGTS_EL_EDIT_ERROR,$el_name, $this->mPluginName, $cat_name).'</sbmessage>';
            	sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_RIGTS_EL_EDIT_ERROR, $el_name, $this->mPluginName, $cat_name), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
    			return false;
    		}

    		// Если нужно переместить в другой раздел
    		if($current_cat != $cat_id)
    		{
    		    // Хотя элемент редактируется, содержащие его разделы в xml и в БД не совпадают
    			// Проверяю права на раздел, в который переносим
	    		if(!$this->checkCategsRight($cat_id))
    			{
    				$el_name = (string)$element->sb_field;
    				$cat_name = $this->getCatName($cat_id);
    				$this->mMess .= '<sbmessage code="1007">'.sprintf(SOAP_RIGTS_EL_REMOVE_ERROR,$el_name, $cat_name, $this->mPluginName).'</sbmessage>';
            		sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_RIGTS_EL_REMOVE_ERROR, $el_name, $cat_name, $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
    				return false;
				}
			}
		}
        else
        {
            if ($edit_id > 0 && $this->mUpdateMethod == SB_SOAP_INSERT)
                $edit_id = -1;
            elseif ($edit_id > 0 && $this->mUpdateMethod == SB_SOAP_ONLY_INSERT)
                return true;
        }

    	// Собираю поля
    	$i = 0;
    	$pics = array();

    	if ($element->sb_field)
    	{
        	foreach($element->sb_field as $v2)
        	{
        		$key = (string)$v2->attributes()->name;

        		if ($edit_id <= 0 && $key == $this->mIdField)
        		    continue;

        		// Проверяю права на пользовательские поля
        		if((sb_strpos($key, 'user_f_') !== false && $this->checkFieldsRight($key)) || sb_strpos($key, 'user_f_') === false)
        		{
        			$val = $this->check_string((string)$v2);
        			if(isset($v2->attributes()->ext_id) && (string)$v2->attributes()->ext_id == 'true')
        			{
        				// Если есть атрибут ext_id - значит поле содержит внешний идентификатор справочника
        				$val = $this->getSpravIdByExt($val); // Достаем внутренний идентификатор по внешнему
        				if(!$val)
        				{
        					$el_name = (string)$element->sb_field;
        					$f_name = $this->getFieldName($key);
        					$this->mMess .= '<sbmessage code="1019">'.sprintf(SOAP_SPRAV_FIELD_ID_ERROR,$f_name, $key, $el_name, $this->mPluginName).'</sbmessage>';

        					sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_SPRAV_FIELD_ID_ERROR,$f_name, $key, $el_name, $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);

        					continue;
        				}
        			}

        			if($this->mDownloadPics)
        			{
        				// Ели установлен флаг, разрешающий загрузку фото с урл
        				if(preg_match('/(^(http:\/\/)(.*)((\.gif)|(\.jpg)|(\.png)|(\.jpeg))$)/',$val))
        				{
        					$pics[$i] = $this->imageDownload($val, $key, true);
        					$val = $pics[$i]['url'];
        				}
        			}

        			$row[$key] = $val;
        		}
        		$i++;
    	    }
    	}

    	foreach($pics as $value)
    	{
    		//	Накладываю водяной знак, если нужно
    		if($value['settings']['watermark'] == 1)
    		{
    			if (!sb_watermark_image($value['url'], $value['url'],
    				$value['settings']['watermark_position'], $value['settings']['watermark_opacity'],
    				$value['settings']['watermark_margin'], $value['settings']['watermark_file'], $value['settings']['copyright'],
    				$value['settings']['copyright_color'], $value['settings']['copyright_font'],
    				$value['settings']['copyright_size']))
    	        	{
            			$this->mMess .= '<sbmessage code="1025">'.sprintf(SOAP_IMAGE_UPLOAD_WATERMARK_ERROR, $value['field']).'</sbmessage>';
    		    		sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_IMAGE_UPLOAD_WATERMARK_ERROR, $value['field']), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
    	        	}
    		}

    		//	Изменяю размер изображения
    		if($value['settings']['resize'] == 1 && $value['settings']['img_width'] !== '' && $value['settings']['img_height'] !== '')
    		{
    			if (!sb_resize_image($value['url'], $value['url'], $value['settings']['img_width'],
    				$value['settings']['img_height'], $value['settings']['img_quality']))
            		{
            			$this->mMess .= '<sbmessage code="1026">'.sprintf(SOAP_IMAGE_UPLOAD_RESIZE_ERROR, $value['field']).'</sbmessage>';
    	    			sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_IMAGE_UPLOAD_RESIZE_ERROR, $value['field']), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
    	        	}
    		}
    	}

    	//	Проверяю пользовательские поля на существование
    	if($this->mPlugin != 'pl_sprav' && $this->mPlugin != 'pl_texts')
        	$row = $this->checkFieldsExist($row);

    	if ($edit_id <= 0 && $this->mUrlField != '' && !isset($row[$this->mUrlField]))
    	{
    	    $row[$this->mUrlField] = sb_check_chpu('', '', $row[$this->mNameField], $this->mTable, $this->mUrlField, $this->mIdField);
    	}

    	//	Устанавливаю внешний идентификатор
    	if(isset($element['e_ext_id']) && trim((string) $element['e_ext_id']) != '')
    	{
    		$row[$this->mExtIdField] = (string)	$element['e_ext_id'];
    	}

    	//	Проверяю статус публикации
    	if(isset($row[$this->mWorkflowField]) && $this->mPlugin != 'pl_sprav')
    	{
    		$row[$this->mWorkflowField] = (int)$this->checkWorkflowRights($row[$this->mWorkflowField]);
    	}
    	elseif($this->mPlugin != 'pl_sprav')
    	{
    		$row[$this->mWorkflowField] = (int)$this->checkWorkflowRights();
    	}

    	$id = sbProgAddElement($this->mTable, $this->mIdField, $row, array($cat_id), $edit_id, $cat_id);
    	$el_name = (string)$element->sb_field;
    	if(!$id)
    	{
    		$this->mMess .= '<sbmessage code="1008">'.sprintf(SOAP_EDIT_ERROR,$el_name, $this->mPluginName).'</sbmessage>';
    		$this -> mNameOfErrorElems[] = $el_name;
		sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_EDIT_ERROR, $el_name, $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
		return false;
    	}
    	// Сообщение об успешном добавлении
    	$el_name = (string)$element->sb_field;
    	$this->mMess .= '<sbmessage code="0003">'.sprintf(SOAP_ADD_OK_EL,$el_name, $this->mPluginName).'</sbmessage>';

    	$this->mNameOfSuccessElems[] =  $el_name;
    	sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_ADD_OK_EL, $el_name, $this->mPluginName), SB_MSG_INFORMATION, $this->mUserId, $this->mUserLogin);

    	if($edit_id > 0)
		{
			$this->mNumOfEditedEls++;
			$this->mIdOfSuccessEditElems[] = $id;
		}
		else
		{
			$this->mNumOfAddedEls++;
			$this->mIdOfSuccessElems[] =  $id;
		}
	//	Создаю новые ссылки
    	if(isset($element->sb_link))
    	{
    		foreach($element->sb_link as $v1)
    		{
    			$sb_link = (int) $v1;
    			// Проверяю права на раздел
    			if(!$this->checkCategsRight($sb_link))
    			{
    				$el_name = (string)$element->sb_field;
    				$cat_name = $this->getCatName($sb_link);
    				$this->mMess .= '<sbmessage code="1009">'.sprintf(SOAP_RIGTS_EL_CREATE_LINK_ERROR,$el_name ,$cat_name, $this->mPluginName, $cat_name).'</sbmessage>';
    				sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_RIGTS_EL_CREATE_LINK_ERROR,$el_name ,$cat_name, $this->mPluginName, $cat_name), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
            		continue;
    			}
    			// Проверка на повторы
    			$res = sql_query('SELECT COUNT(*) FROM sb_catlinks WHERE link_cat_id = ?d AND link_el_id = ?d', $sb_link, $id);
    			if(isset($res[0][0]) && $res[0][0] < 1)
    			{
    				// Добавляю ссылку
    				$row = array(
    					'link_cat_id'		=> $sb_link,
    					'link_el_id'		=>	$id,
    					'link_src_cat_id'	=>	$cat_id
    				);
    				$res = sql_query('INSERT INTO sb_catlinks SET ?a', $row);
    				if(!$res)
    				{
    					$el_name = (string)$element->sb_field;
    					$cat_name = $this->getCatName($sb_link);
    					$this->mMess .= '<sbmessage code="1012">'.sprintf(SOAP_EL_CREATE_LINK_ERROR,$el_name ,$cat_name, $this->mPluginName).'</sbmessage>';
            			sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_EL_CREATE_LINK_ERROR, $el_name, $cat_name, $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
    					continue;
    				}
    				// Сообщение об успешном добавлении
    				$el_name = (string)$element->sb_field;
    				$cat_name = $this->getCatName($sb_link);
    				$this->mMess .= '<sbmessage code="0004">'.sprintf(SOAP_EL_CREATE_LINK_OK,$el_name ,$cat_name, $this->mPluginName).'</sbmessage>';
            		sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_EL_CREATE_LINK_OK, $el_name, $cat_name, $this->mPluginName), SB_MSG_INFORMATION, $this->mUserId, $this->mUserLogin);
    			}
    		}
    	}
    	return true;
    }

    /**
	 * Добавляет в xml результата ошибки для всех элементов ветки, если неудалось добавить раздел
	 *
	 * @param string $cat - xml объект раздела, который неудалось добавить.
	 * @param string $reason - код ошибки добавления родительского раздела.
	 *
	 * КОДЫ
	 *   NO_RIGHTS_FOR_PARENT - Неудалось добавить, т.к. нет прав на добавление родительского раздел
     *
     */
    private function setErrorsForChildren($cat, $reason)
    {
    	foreach($cat as $k => $v)
    	{
    		if($k == 'sb_cat')
    		{
	    		foreach($v->sb_field as $v2)
		    	{
		    		$key = (string)$v2->attributes()->name;
		    		$row[$key] = (string)$v2;
		    	}

    			if($reason == 'NO_RIGHTS_FOR_PARENT')
    			{
    				$this->mMess .= '<sbmessage code="1002">'.sprintf(SOAP_NO_RIGHTS_FOR_PARENT_CAT, $row['cat_title'], $this->mPluginName).'</sbmessage>';
            		sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_NO_RIGHTS_FOR_PARENT_CAT, $row['cat_title'], $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
    			}
    		}
    		elseif($k == 'sb_elem')
    		{
	    		foreach($v->sb_field as $v2)
		    	{
		    		$key = (string)$v2->attributes()->name;
		    		$row[$key] = (string)$v2;
		    	}

    			if($reason == 'NO_RIGHTS_FOR_PARENT')
    			{
    				$this->mMess .= '<sbmessage code="1003">'.sprintf(SOAP_NO_RIGHTS_FOR_PARENT_EL, $row[$this->mNameField], $this->mPluginName).'</sbmessage>';
            		sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_NO_RIGHTS_FOR_PARENT_EL, $row[$this->mNameField], $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
    			}
    		}

    		if(isset($v->sb_elem))
    		{
    			$this->setErrorsForChildren($v->sb_elem, $reason);
    		}
    		if(isset($v->sb_cat))
    		{
    			$this->setErrorsForChildren($v->sb_cat, $reason);
    		}

    	}
    }

    /**
	 * Проверяет есть ли сообщения, если есть - возвращает xml с сообщениями
     *
     * @return string xml с сообщениями
     */
    private function getReturnXml()
    {
    	/*
    	 * КОДЫ СООБЩЕНИЙ
    	 * 0001		- Раздел <i>%s</i> успешно изменен
		 * 0002		- Раздел <i>%s</i> успешно добавлен
		 * 0003		- Элемент <i>%s</i> успешно добавлен/изменен
		 * 0004		- Ссылка на элемент <i>%s</i> в разделе <i>%s</i> успешно добавлена.
		 * 0005		- Элемент <i>%s</i> успешно удален.
		 * 0006		- Раздел <![CDATA[<i>%s</i>]]> успешно удален.
		 * ----
    	 * 1001		- Предупреждение! У Вас нет прав на редактирование раздела <i>%s</i>
    	 * 1002		- Предупреждение! Невозможно добавить раздел <i>%s</i>, т.к. не добавлен родительский раздел
    	 * 1003		- Предупреждение! Невозможно добавить элемент <i>%s</i>, т.к. не добавлен родительский раздел
    	 * 1004		- Предупреждение! Невозможно изменить раздел <i>%s</i>, т.к. у Вас нет прав на редактирование этого раздела.
    	 * 1005		- Предупреждение! Невозможно добавить/изменить элемент <i>%s</i>, т.к. не удалось определить родительский раздел.
    	 * 1006		- Предупреждение! Невозможно изменить элемент <i>%s</i>, т.к. у Вас нет прав на редактирование родительского раздела <i>%s</i>.
    	 * 1007		- Предупреждение! Невозможно переместить элемент <i>%s</i>, т.к. у Вас нет прав на редактирование раздела <i>%s</i>
    	 * 1008		- Предупреждение! Невозможно добавить/изменить элемент <i>%s</i>. (Возможно неверный xml)
    	 * 1009		- Предупреждение! Невозможно создать ссылку на элемент <i>%s</i> в разделе <i>%s</i>, т.к. у Вас нет прав на редактирование раздела <i>%s</i>.
    	 * 1010		- Предупреждение! Невозможно изменить поле <i>%s</i> раздела <i>%s</i>, т.к. у Вас нет прав на редактирование этого поля.
    	 * 1011		- Предупреждение! Невозможно добавить/изменить раздел <i>%s</i>
    	 * 1012		- Предупреждение! Невозможно создать ссылку на элемент <i>%s</i> в разделе <i>%s</i>.
    	 * 1013		- Предупреждение! Невозможно удалить раздел <i>%s</i>. Возможно раздел, или его подразделы содержат элементы
    	 * 1014		- Предупреждение! Невозможно удалить элемент <i>%s</i>, т.к. не задан идентификатор
    	 * 1015		- Предупреждение! Невозможно удалить элемент.
    	 * 1016		- Предупреждение! Невозможно удалить раздел <i>%s</i>, т.к. не задан идентификатор.
    	 * 1017		- Предупреждение! У Вас нет прав на удаления элементов модуля <![CDATA[<i>%s</i>]]>.
    	 * 1018		- Предупреждение! У Вас нет прав на удаления разделов модуля <![CDATA[<i>%s</i>]]>
    	 * 1019		- Предупреждение! Невозможно изменить поле <i>%s (%s)</i> элемента <i>%s</i> в модуле <i>%s</i>, т.к. неверно задан внешний идентификатор справочника.
    	 * 1020		- Предупреждение! Невозможно изменить поле <i>%s (%s)</i> раздела <i>%s</i> в модуле <i>%s</i>, т.к. неверно задан внешний идентификатор справочника.
    	 * 1021		- Предупреждение! Неизвестное поле <i>%s</i> в элементе модуля <i>%s</i>.
    	 * 1022		- Предупреждение! Неизвестное поле <i>%s</i> в разделе модуля <i>%s</i>
    	 * 1023		- Предупреждение! Невозможно добавить/изменить раздел в модуле <i>%s</i>, т.к. не задано название раздела
    	 * 1024		- Предупреждение! Невозможно добавить разделы модуля <i>%s</i> в модуль <i>%s</i>
    	 * 1025		- Предупреждение! Невозможно Наложить водяной знак на изображение в поле <i>%s</i
    	 * 1026		- Предупреждение! Невозможно изменить размер изображения в поле <i>%s</i>
    	 * ----
    	 * 2001		- Ошибка! Вы не передали soap token.
    	 * 2002		- Ошибка! Вы передали неверный soap token.
    	 * 2003		- Ошибка! Вы передали устаревший soap token.
    	 * 2004		- Ошибка! Вы не передали XML код.
    	 * 2005		- Ошибка! Ваша учетная запись заблокирована администратором системы.
    	 * 2006		- Ошибка! У Вас нет прав на администрирование домена.
    	 * 2007 	- Ошибка! Вход в систему с Вашего IP-адреса запрещен.
    	 * 2008 	- Ошибка! Группа пользователей, в которую входит Ваша учетная запись, заблокирована администратором системы.
    	 * 2009 	- Ошибка! Вы передали неверный xml код.
    	 * 2010		- Ошибка! У Вас нет прав на редактирование разделов модуля <i>%s</i>
		 *
    	 */
    	$ret = '';
    	if(!empty($this->mMess))
    	{
    		$this->mMess = preg_replace('/<i>(.*?)<\/i>/im'.SB_PREG_MOD, '<![CDATA[<i>$1</i>]]>',$this->mMess);
    		$ret .= '<?xml version="1.0" encoding="UTF-8"?><sbanswer>';
    		$ret .= $this->mMess;
    		$ret .= '</sbanswer>';
    		return $ret;
    	}
    	return '';
    }

 	/**
	 * Возвращает soap_token ключ
	 *
	 * @param string $login - логин пользователя для которого генерируеться soap_token.
	 * @param string $password - md5 хэш пароля пользователя.
	 * @param int $id - Идентификатор пользователя.
	 * @param string $word - Секретное слово.
	 * @param int $num - Генерируеться при изменении soap_token в форме редактирования пользователя.
	 *
	 * @return string soap_token пользователя
	 */
	public function getSoapTokenHash($login, $password, $id, $word, $num)
	{
		return md5(sb_substr(md5($login.$password.$id),0,$num).$word);
	}

	/**
	 * Проверяет пришедший от пользователя soap_token
	 *
	 * @param string $soap_token - soap_token.
	 *
	 * @return bool TRUE - если soap token валидный, FALSE - если нет.
	 */
	private function checkSoapToken($soap_token = '')
	{
		// Если идет импорт из системы
		if(isset($_SESSION['sbAuth']))
		{
			$this->mUserId = $_SESSION['sbAuth'] -> getUserId();
			$this->mUserLogin = $_SESSION['sbAuth'] -> getUserLogin();
			$this->mIsAdmin = $_SESSION['sbAuth'] -> isAdmin();

			return true;
		}
		// Если идет импорт через SOAP
    	if($soap_token  == '')
    	{
    		// Пришел ли soap_key в ф-цию
    		$this->mMess .= '<sbmessage code="2001">'.SOAP_AT_EMPTY_ERROR.'</sbmessage>';
            sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.SOAP_AT_EMPTY_ERROR, SB_MSG_WARNING);
    		return false;
    	}

    	// Есть ли пользоватль с таким soap_key
		$res = sql_query('SELECT u_id, u_login, u_pass, u_soap_token, u_block, u_domains FROM sb_users WHERE u_soap_token LIKE "%|'.$soap_token.'"');
		if(!$res)
		{
			$this->mMess .= '<sbmessage code="2002">'.SOAP_AT_WRONG_ERROR.'</sbmessage>';
            sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.SOAP_AT_WRONG_ERROR, SB_MSG_WARNING);
			return false;
		}
		list($this->mUserId, $this->mUserLogin, $u_pass, $u_soap_token, $u_block, $u_domains) = $res[0];

		// Не менялся ли пароль или логин у пользователя
		$user_soap_num = '';
		$user_soap_word = explode('|',$u_soap_token);

		if(count($user_soap_word) == 3)
		{
			$user_soap_num = $user_soap_word[1];
			$user_soap_word = $user_soap_word[0];
		}
		$new_soap = $this->getSoapTokenHash($this->mUserLogin, $u_pass, $this->mUserId, $user_soap_word, $user_soap_num);
		if($new_soap != $soap_token)
		{
			$this->mMess .= '<sbmessage code="2003">'.SOAP_AT_OLD_ERROR.'</sbmessage>';
            sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.SOAP_AT_OLD_ERROR, SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
			return false;
		}

		// Если пользователь заблокирован
		if($u_block == 1)
		{
			$this->mMess .= '<sbmessage code="2005">'.SOAP_AT_USER_BLOCKED.'</sbmessage>';
            sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.SOAP_AT_USER_BLOCKED, SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
			return false;
		}

		// Находится ли пользователь в группе администраторов
		$res = sql_query('SELECT c.cat_id FROM sb_categs c, sb_catlinks l
                         WHERE c.cat_closed=1 AND c.cat_ident=? AND l.link_cat_id=c.cat_id AND l.link_el_id=?d', 'pl_users', $res[0][0]);
		if(!$res)
		{	//Если нет, то может ли он админить выбранный домен
			$u_domains = explode(',', $u_domains);
		 	if (!in_array(SB_COOKIE_DOMAIN, $u_domains))
            {
            	$this->mMess .= '<sbmessage code="2006">'.sprintf(SOAP_AT_WRONG_DOMAIN, SB_COOKIE_DOMAIN).'</sbmessage>';
            	sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_AT_WRONG_DOMAIN, SB_COOKIE_DOMAIN), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
				return false;
            }
		}
		else
			$this -> mIsAdmin = true;

		// Разрешен ли доступ с данного IP
		$availIPs = sbPlugins::getSetting('sb_admin_soap_ip');
		if ($availIPs != '')
        {
        	$u_ip = sbAuth::getIP();
            $error = true;
            $availIPs = explode(' ', $availIPs);
            foreach ($availIPs as $value)
            {
                if (trim($value) != '')
                {
                    if (sbAuth::compareIP($value, $u_ip))
                    {
                        $error = false;
                        break;
                    }
                }
            }
            if ($error)
            {
                // Пытаемся зайти не с разрешенного IP-адреса
        	    $this->mMess .= '<sbmessage code="2007">'.SOAP_AT_IP_ERROR.'</sbmessage>';
            	sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_AT_IP_ERROR_MSG, $this->mUserLogin, $u_ip), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
                return false;
            }
        }

	 	// Группа пользователя (cat_rubrik)
        $res = sql_query('SELECT DISTINCT links.link_cat_id FROM sb_categs categs, sb_catlinks links WHERE categs.cat_ident=? AND categs.cat_rubrik=1 AND links.link_cat_id=categs.cat_id AND links.link_el_id=?d', 'pl_users', $this->mUserId);
        if(!$res)
        {
			$this->mMess .= '<sbmessage code="2008">'.SOAP_AT_CATEG_BLOCKED.'</sbmessage>';
            sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.SOAP_AT_CATEG_BLOCKED, SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
			return false;
        }
		return true;
	}

	/**
     * Проверяет пришедший от пользователя xml
     *
     * @param string $xml - xml пришедьший от пользователя
     *
     * @return mixed - xml-объект, если если пришел валидный xml, FALSE в ином случае .
     */
	private function checkXml($xml = '')
	{
		$xml = trim($xml);
		if($xml == '')
    	{
    		$this->mMess .= '<sbmessage code="2004">'.SOAP_XML_EMPTY_ERROR.'</sbmessage>';
            sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.SOAP_XML_EMPTY_ERROR, SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
    		return false;
    	}

    	if(!$struct = simplexml_load_string($xml))
    	{
    		$this->mMess .= '<sbmessage code="2009">'.SOAP_XML_WRONG_ERROR.'</sbmessage>';
            sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.SOAP_XML_WRONG_ERROR, SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
            return false;
    	}
    	return $struct;
	}

 	/**
     * Проверяет, есть ли у пользователя указанные права доступа к модулю
     *
     * @param string $rights Права доступа. Разделяются через |.
     *
     * @return bool TRUE, если у пользователя есть права доступа, и FALSE в ином случае.
     */
    private function checkPluginRight($rights)
    {
    	if($this->mIsAdmin)
    		return true;

    	// Если массива прав еще нету - создаю его
    	if(count($this->mPluginRight) < 1)
    	{
    		// Достаю группу пользователя
	    	if(!$this -> getUserCats())
		    {
		    	$this->mMess .= '<sbmessage code="1001">'.SOAP_AT_CATEG_BLOCKED.'</sbmessage>';
	            sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.SOAP_AT_CATEG_BLOCKED, SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
				return false;
		    }

	        // Проверяем права пользователя
	        $res = sql_query('SELECT ur_plugin_rights FROM sb_users_rights WHERE ur_plugin=? AND ur_cat_id IN (?a) AND ur_plugin_rights <> ?', $this->mPlugin, $this->getUserCats(), '');

	        if (!$res)
	        {
	        	// Информации о правах доступа нет
	      	   	return false;
	        }
	        // Создаю массив прав
	        foreach($res as $value)
	        {
	        	$this->mPluginRight[] = explode('|',$value[0]);
	        }
    	}
        //Проверяю права
        $rights = explode('|', $rights);
        $i = 0;
        foreach($this->mPluginRight as $value)
		{
			foreach($rights as $value2)
			{
				if(in_array($value2, $value))
				$i++;
			}
		}

        if(count($rights) == $i)
        	return true;
        return false;
    }

    /**
     * Проверяет, есть ли у пользователя указанные права доступа к разделу
     *
     * @param string $cat_id - Идентификатор раздела.
     *
     * @return bool TRUE, если у пользователя есть права доступа, и FALSE в ином случае.
     */
    private function checkCategsRight($cat_id)
    {
    	if($this->mIsAdmin)
    	{
    		return true;
    	}
    	$res = sql_query('SELECT cat_rights, cat_title FROM sb_categs WHERE cat_id=?d', $cat_id);
    	if (!$res || $res[0][0] == '')
        {
            return false;
        }
	    $cat_rights = explode('^', $res[0][0]);
	    $cat_groups = array();

	    foreach ($cat_rights as $right)
	    {
	    	if ($right[0] == 'g')
	    	{
	    		$cat_groups[] = intval(substr($right, 1));
	    	}
	    	elseif ($right[0] == 'u')
	    	{
	    		if (intval(substr($right, 1)) == $this->mUserId)
	    		{
	    			return true;
	    		}
	    	}
	    	else
	    	{
	    		$cat_groups[] = intval($right);
	    	}
	    }

	    $groups = $this->getUserCats();
	    if (count(array_intersect($groups, $cat_groups)) > 0)
	    {
	        return true;
	    }

	     return false;
    }

 	/**
     * Проверяет, есть ли у пользователя указанные права доступа к полю
     *
     * @param string $field поле, для которого проверяются права
     * @param bool $cats_for - если TRUE - поле относится к разделу, false - к элементу
     *
     * @return bool TRUE, если у пользователя есть права доступа, и FALSE в ином случае.
     */
    private function checkFieldsRight($field, $cats_for = false)
    {
    	if($this->mIsAdmin)
    		return true;
    	if(sb_strpos($field,'user_f_') !== FALSE)
    	{
    		$field = sb_str_replace('user_f_','',$field);
    	}
    	if($cats_for)
    		$fld = 'pd_categs';
    	else
    		$fld = 'pd_fields';
    	if(!isset($this->mFieldsRight[$fld]))
    	{
    		// Если массива прав разделов еще не существует - создаем его
	    	$rights = $this->getFields($cats_for);
	    	foreach($rights as $v)
	    	{
	    		if(isset($v['rights_set']) && $v['rights_set'] == 1)
	    		{
	    			$key = $v['id'];
	    			if (!isset($v['rights_edit_list']) || $v['rights_edit_list'] == '')
	    			{
	    				// Если нужно проверять права, но права неустановленны ни для кого
	    				$this->mFieldsRight[$fld][$key]['cats'] = array();
	    				$this->mFieldsRight[$fld][$key]['users'] = array();
	    				continue;
	    			}
	    			else
	    			{
		    			$r = explode('^',$v['rights_edit_list']);
					    foreach ($r as $v2)
					    {
					    	if ($v2[0] == 'g')
					    	{
					    		$this->mFieldsRight[$fld][$key]['cats'][] = intval(substr($v2, 1));
					    	}
					    	elseif ($v2[0] == 'u')
					    	{
					    		$this->mFieldsRight[$fld][$key]['users'][] = intval(substr($v2, 1));
					    	}
					    	else
					    	{
					    		$this->mFieldsRight[$fld][$key]['cats'][] = intval($v2);
					    	}
					    }
	    			}
	    		}
	    	}
    	}

    	/*
    	 * Проверка прав
    	 */
    	// Если нет записи в массиве, значит галочка "Проверять права доступа" не установлена
    	if(!isset($this->mFieldsRight[$fld][$field]['cats']) && !isset($this->mFieldsRight[$fld][$field]['users']))
    		return true;
    	// Если поле разрешенно к редактирванию разделу юзера
    	if(isset($this->mFieldsRight[$fld][$field]['cats']) && is_array($this->mFieldsRight[$fld][$field]['cats']) && count(array_intersect($this->getUserCats(), $this->mFieldsRight[$fld][$field]['cats'])) > 0)
    		return true;
    	// Если	поле разрешенно к редактированию этому пользователю
    	if(in_array($this->mUserId,$this->mFieldsRight[$fld][$field]['users']))
    		return true;
    	// Прав нет
    	if(!isset($this->mFieldsRightErrors[$field]))
    	{
    		// Если еще не выводили ошибку - выводим ее
    		$this->mFieldsRightErrors[$field] = $field;
    		$f_name = $this->getFieldName($field, $cats_for);
    		$this->mMess .= '<sbmessage code="1010">'.sprintf(SOAP_RIGTS_FIELD_ERROR, $f_name, 'user_f_'.$field, $this->mPluginName).'</sbmessage>';
            sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_RIGTS_FIELD_ERROR, $f_name, 'user_f_'.$field, $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
    	}
    	return false;
    }

    /**
     * Достает разделы, в которых находится пользователь, и записывает их в массив $this->mUserCats
     */
    private function getUserCats()
    {
    	if(count($this->mUserCats) > 0)
    	{
    		return $this->mUserCats;
    	}
    	else
    	{
	    	$res = sql_query('SELECT DISTINCT links.link_cat_id FROM sb_categs categs, sb_catlinks links WHERE categs.cat_ident=? AND categs.cat_rubrik=1 AND links.link_cat_id=categs.cat_id AND links.link_el_id=?d', 'pl_users', $this->mUserId);
	        if ($res)
	        {
	            foreach ($res as $value)
	            {
	                $this->mUserCats[] = $value[0];
	            }
	            return $this->mUserCats;
	        }
	        else
	        {
	        	return false;
	        }
    	}
    }

    /**
     * Возвращает название раздела по идентификатору
     *
     * @param int $cat_id - идентификатор раздела
     *
     * @return string - Название раздела, или false - если неудалось найти раздел.
     */
    private function getCatName($cat_id)
    {
    	$res = sql_query('SELECT cat_title FROM sb_categs WHERE cat_id = ?d', $cat_id);
    	if($res && $res[0][0] != '')
    	{
    		return $res[0][0];
    	}
    	else
    	{
    		return false;
    	}
    }

	/**
     * Возвращает название элемента по идентификатору
     *
     * @param int $el_id - идентификатор раздела
     *
     * @return string - Название элемента, или false - если неудалось найти раздел.
     */
    private function getElName($el_id)
    {
    	$res = sql_query('SELECT '.$this->mNameField.' FROM '.$this->mTable.' WHERE '.$this->mIdField.' = ?d', $el_id);
    	if($res && $res[0][0] != '')
    	{
    		return $res[0][0];
    	}
    	else
    	{
    		return false;
    	}
    }

	/**
     * Возвращает название поля
     *
     * @param string $field_id идентификатор поля
     * @param bool $is_cat TRUE если нужно вернуть названия поля раздела, FALSE - если элемента
     *
     * @return string - Названия поля, или FALSE - если поле найти неудалось
     */
    private function getFieldName($field_id, $is_cat = false)
    {
    	if(sb_strpos($field_id, 'user_f_') !== FALSE)
    		$field_id = sb_str_replace('user_f_','', $field_id);
    	$field = $this->getFields($is_cat);
    	foreach ($field as $v)
    	{
    		if($v['id'] == $field_id)
    			return $v['title'];
    	}
    	return false;
    }

    /**
     * Вырезает из поля php код
     *
     * @param string $str Значение поля
     *
     * @return string - Значение поля без php кода
     */
    private function check_string($str)
    {
    	return preg_replace('/(((<\?php)|(<\?\s))(.*)(\?>))/im'.SB_PREG_MOD, '', $str);
    }

    /**
     * Удаляет из xml-объекта ветки структуры все разделы и элементы(если есть тэг delete="delete")
     *
     * @param object $xml объект
     *
     * @return bool - результат выполнения
     */
    private function deleteStruct($xml)
    {
    	// Узнаю имя, элемента
		$cat_name = '';
		if(isset($xml->sb_field))
		{
			foreach ($xml->sb_field as $v2)
			{
				if(isset($v2['name']) && $v2['name'] == 'cat_title')
				$cat_name = (string)$v2;
			}
		}

		//Узнаю идентификатор раздела
    	$cat_id = 0;
    	if(isset($xml['c_ext_id']) && (int) $xml['c_ext_id'] > 0)
		{
			$res = sql_query('SELECT cat_id FROM sb_categs WHERE cat_ext_id = ?d AND cat_ident = ?', (int)$xml['c_ext_id'], $this->mPlugin);
			if($res && $res[0][0] != '')
				$cat_id = $res[0][0];
		}
		elseif(isset($xml['c_id']) && (int) $xml['c_id'] > 0)
		{
			$cat_id = (int) $xml['c_id'];
		}
		if($cat_id < 1)
		{
			// Если есть атрибут delete = 'delete', но не указан id - вывожу ошибку
			$this->mMess .= '<sbmessage code="1016">'.sprintf(SOAP_DELETE_ERROR_CATS_WRONG_ID,$cat_name, $this->mPluginName).'</sbmessage>';
    		sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_DELETE_ERROR_CATS_WRONG_ID, $cat_name, $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
    		return false;
		}
        // Сначала удаляю все элементы ветки
    	foreach($xml->xpath('//sb_elem[@delete="delete"]') as $v2)
    	{
			$this->deleteElement($v2);
    	}
		// Получаю список дочерних разделов, и пытаюсь их удалить
		if(!$this->deleteCats($cat_id))
		{
			return false;
		}
		return true;

    }

    private function deleteElement($el, $cat_id = 0)
    {
    	// Проверяю права на возможность удаления элементов модуля
    	if(!$this->checkPluginRight('elems_delete'))
    	{
    		if (!$this->mPluginRightDelErrors['elems'])
    		{
    			// Если ошибка еще не выводилась - вывожу
    			$this->mPluginRightDelErrors['elems'] = true;
    			$this->mMess .= '<sbmessage code="1017">'.sprintf(SOAP_RIGTS_DEL_ELEMS_PLUGIN_ERROR,$this->mPluginName).'</sbmessage>';
    			sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_RIGTS_DEL_ELEMS_PLUGIN_ERROR, $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
    		}
    		return false;
    	}

		//Узнаю id удаляемого элемента
		$el_id = 0;
		if(isset($el['e_ext_id']) && trim((string) $el['e_ext_id']) != '')
		{
			$el_ext_id = (string) $el['e_ext_id'];
    		$res = sql_query('SELECT '.$this->mIdField.' FROM  '.$this->mTable.' WHERE '.$this->mExtIdField.' = ?', $el_ext_id);
    		if(isset($res[0][0]) && $res[0][0] != '')
    		{
    			// Редактируем по ext_id
    			$el_id = $res[0][0];
    		}
		}
		elseif(isset($el['e_id']) && (int) $el['e_id'] > 0)
		{
			$el_id = (int) $el['e_id'];
		}

		if($el_id < 1)
		{
			//Узнаю имя элемента из xml, для вывода ошибки id
			$el_name = '';
			if(isset($el->sb_field))
			{
				foreach ($el->sb_field as $v2)
				{
					if(isset($v2['name']) && $v2['name'] == $this->mNameField)
					$el_name = $v2;
				}
			}
			// Если есть атрибут delete = 'delete', но не указан id - вывожу ошибку
			$this->mMess .= '<sbmessage code="1014">'.sprintf(SOAP_DELETE_ERROR_ELEM_WRONG_ID,$el_name, $this->mPluginName).'</sbmessage>';
    		sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_DELETE_ERROR_ELEM_WRONG_ID, $el_name, $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
    		return false;
		}

		$el_name = $this->getElName($el_id);
	    $res = sql_query('DELETE FROM '.$this->mTable.' WHERE '.$this->mIdField.' = ?d', $el_id);
		if(!$res)
		{
			$this->mMess .= '<sbmessage code="1015">'.sprintf(SOAP_DELETE_ERROR_ELEM,$el_name, $this->mPluginName).'</sbmessage>';
	    	sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_DELETE_ERROR_ELEM, $el_name, $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
	    	return false;
		}

		if($cat_id == 0)
		{
			// Если не перердан идентификатор раздела - узнаю его
			$res = sql_query("SELECT c.cat_id FROM sb_categs as c, sb_catlinks as l WHERE
							  l.link_el_id = ?d AND c.cat_ident = ? AND l.link_cat_id = c.cat_id",$el_id, $this->mPlugin);
			$cat_id = $res[0][0];
		}

		$res = sql_query('DELETE FROM sb_catlinks WHERE link_el_id = ?d and link_cat_id = ?d', $el_id, $cat_id);
		if(!$res)
		{
			$this->mMess .= '<sbmessage code="1015">'.sprintf(SOAP_DELETE_ERROR_ELEM,$el_name, $this->mPluginName).'</sbmessage>';
	    	sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_DELETE_ERROR_ELEM, $el_name, $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
	    	return false;
		}

		$this->mMess .= '<sbmessage code="0005">'.sprintf(SOAP_DELETE_OK_EL,$el_name, $this->mPluginName).'</sbmessage>';
    	sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_DELETE_OK_EL, $el_name, $this->mPluginName), SB_MSG_INFORMATION, $this->mUserId, $this->mUserLogin);
    	return true;
    }

    private function deleteCats($cat_id)
    {
    	// Проверяю права на возможность удаления разделов модуля
    	if(!$this->checkPluginRight('categs_delete'))
    	{
    		if (!$this->mPluginRightDelErrors['categs'])
    		{
    			// Если ошибка еще не выводилась - вывожу
    			$this->mPluginRightDelErrors['categs'] = true;
    			$this->mMess .= '<sbmessage code="1018">'.sprintf(SOAP_RIGTS_DEL_CAT_PLUGIN_ERROR,$this->mPluginName).'</sbmessage>';
    			sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_RIGTS_DEL_CAT_PLUGIN_ERROR, $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
    		}
    		return false;
    	}
    	//Проверяю существование раздела
    	$res = sql_query('SELECT cat_level, cat_left, cat_right FROM sb_categs WHERE cat_id = ?d', $cat_id);
    	if(!$res)
    	{
    		// Если такого раздела нет - считаем что он удален и незамечаем его
    		return true;
    	}
    	$cat_name = $this->getCatName($cat_id);
    	$cat_level = $res[0][0];
    	$cat_left  = $res[0][1];
    	$cat_right = $res[0][2];
    	// Проверяю есть ли в разделе элементы
    	$res = sql_query('SELECT COUNT(*) FROM sb_catlinks WHERE link_cat_id = ?d OR link_src_cat_id = ?d', $cat_id, $cat_id);
    	if(!$res || $res[0][0] > 0)
    	{
    		$this->mMess .= '<sbmessage code="1013">'.sprintf(SOAP_DELETE_ERROR_CATS,$cat_name, $this->mPluginName).'</sbmessage>';
	    	sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_DELETE_ERROR_CATS, $cat_name, $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
	    	return false;
    	}
    	// Проверяю дочерние разделы
    	$res = sql_query('SELECT cat_id FROM sb_categs WHERE cat_left > ?d AND cat_right < ?d AND cat_level = ?d AND cat_ident = ?', $cat_left, $cat_right, $cat_level+1, $this->mPlugin);
    	if($res)
    	{
    		foreach($res as $value)
    		{
    			if(!$this->deleteCats($value[0]))
    			{
    				return false;
    			}
    		}
    	}
    	// Удаляю
    	$res = sql_query('DELETE FROM sb_categs WHERE cat_id = ?d', $cat_id);
    	$res1 = sql_query('DELETE FROM sb_catlinks WHERE link_cat_id = ?d OR link_src_cat_id = ?d', $cat_id, $cat_id);
    	if(!$res || !$res1)
    	{
    		$this->mMess .= '<sbmessage code="1013">'.sprintf(SOAP_DELETE_ERROR_CATS,$cat_name, $this->mPluginName).'</sbmessage>';
	    	sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_DELETE_ERROR_CATS, $cat_name, $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
	    	return false;
    	}
    	// Успешно
    	$this->mMess .= '<sbmessage code="0006">'.sprintf(SOAP_DELETE_OK_CAT,$cat_name, $this->mPluginName).'</sbmessage>';
	    sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_DELETE_OK_CAT, $cat_name, $this->mPluginName), SB_MSG_INFORMATION, $this->mUserId, $this->mUserLogin);
	    return true;
    }

     /**
     * Проверяет, есть ли у пользователя права на указанные публикации
     *
     * @param string $rights Права доступа. Разделяются через |.
     *
     * @return bool TRUE, если у пользователя есть права доступа, и FALSE в ином случае.
     */
    private function checkWorkflowRights($wf = -1)
    {
    	if(!sbAuth::checkPlugin('pl_workflow'))
    	{
    		// Если в системе нет модуля "Цепочки публикаций"
    		if($wf >= 0 && $wf <= 1)
    			return $wf;
    		else
    			return 0;
    	}
    	if(count($this->mWorkFlow) < 1)
    	{
    		// Если массива существующих цепочек публикаций еще нет - создаю
    		$res = sql_query('SELECT w_code FROM sb_workflow');
    		if(!$res)
    		{
    			$this->mWorkFlow[] = '';
    		}
    		else
    		{
		    	foreach($res as $v)
		    	{
		    		$this->mWorkFlow[] = $v[0];
		    	}
    		}
    	}
    	//Если админ
    	if($this->mIsAdmin)
    	{
    		// Проверяю существование статуса
    		if(in_array($wf,$this->mWorkFlow))
    		{
    			return $wf;
    		}
    		else
    		{
    			return max($this->mWorkFlow);
    		}
    	}
    	if(count($this->mWorkFlowRight) < 1)
    	{
    		// Если массива доступных пользователю цепочек еще нет - сздаю его
    		$res = sql_query('SELECT ur_workflow_rights FROM sb_users_rights WHERE ur_plugin=? AND ur_cat_id IN (?a) AND ur_plugin_rights <> ?', $this->mPlugin, $this->getUserCats(), '');
    		if(!$res)
    		{
    			$this->mWorkFlowRight[] = '';
    		}
    		else
    		{
    			$this->mWorkFlowRight = explode('|',$res[0][0]);
    		}

    	}
    	// Проверяю доступность статуса публикации
    	if(in_array($wf, $this->mWorkFlowRight))
    	{
    		return $wf;
    	}
    	else
    	{
    		return max($this->mWorkFlowRight);
    	}
    }

     /**
     * Возвращает внутренние идентификаторы справочников по внешним
     *
     * @param string $id Внешние идентификаторы. Разделяются через запятую.
     *
     * @return string  внутренние идентификаторы спраочников, перечисленные через запятую.
     */
    private function getSpravIdByExt($id)
    {
    	// Если поле содержит внешний идентификатор справочника, достаем внутренний
    	$id = str_replace(' ', '', $id);
    	$spr_ids = explode(',', $id);
    	if(count($spr_ids) > 0)
    	{
    		$res = sql_query('SELECT s_id FROM sb_sprav WHERE s_ext_id IN (?a)', $spr_ids);
    		if($res)
    		{
    			$val = '';
    			foreach($res as $v3)
    			{
    				$val .= $v3[0].',';
    			}
    			$val = sb_substr($val,0,-1);
    			return $val;
    		}
    	}

    	return false;
    }

    /**
     * Проверяет существуют ли поля, введенные пользователем.
     *
     * @param array $fields массив полей
     * @param bool $catsfor - true - если проверка для полей раздела, false - если для полей элемента
     * @param bool $name - названи раздела, для вывода ошибки
     *
     * @return массив существующих пользовательских полей
     */
    private function checkFieldsExist($xml_fields, $catsfor = false, $name = '')
    {
    	$result = array();
    	$cms_fields = $this->getFields($catsfor);
    	foreach($xml_fields as $key => $value)
    	{
    		if(sb_strpos($key,'user_f_') === false)
			{
    			//	Если поле не пользовательское - оставляем как есть
				$result[$key] = $value;
				continue;
			}

    		$exist = false; //	Найдено ли введенное пользователем поле
    		foreach($cms_fields as $v)
    		{
    			if('user_f_'.$v['id'] == $key)
    			{
    				//	Если поле найденно
    				$exist = true;
    				$result[$key] = $value;
    				break;
			}
		}

		if(!$exist)
		{
    			//	Если поле найденно небыло
    			if(!$catsfor)
    			{
    				$this->mMess .= '<sbmessage code="1021">'.sprintf(SOAP_FIELD_ELEM_WRONG_ERROR, $key, (isset($xml_fields[$this->mNameField]) ? $xml_fields[$this->mNameField] : ''), $this->mPluginName).'</sbmessage>';
    				sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_FIELD_ELEM_WRONG_ERROR, $key, (isset($xml_fields[$this->mNameField]) ? $xml_fields[$this->mNameField] : ''), $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
    			}
    			else
    			{
    				$this->mMess .= '<sbmessage code="1022">'.sprintf(SOAP_FIELD_CAT_WRONG_ERROR, $key, $name, $this->mPluginName).'</sbmessage>';
    				sb_add_system_message(SOAP_SYSTEM_MESSAGE_PREFIX.sprintf(SOAP_FIELD_CAT_WRONG_ERROR, $key, $name, $this->mPluginName), SB_MSG_WARNING, $this->mUserId, $this->mUserLogin);
    			}
    		}
    	}
    	return $result;
    }

    /**
     * Возвращает массив пользовательских полей для раздела или элемента
     *
     * @param bool $catsfor - true - если нужен массив пользовательских полей раздела, false - если элемента
     *
     * @return array - массив пользователских полей для раздела или элемента
     */
    private function getFields($catsfor = false)
    {
    	if($catsfor && count($this->mFieldsCats) > 0)
    	{
    		// Если массив полей разделов уже есть
    		return $this->mFieldsCats;
    	}
    	elseif(!$catsfor && count($this->mFieldsElems) > 0)
    	{
    		// Если массив полей элементов уже есть
    		return $this->mFieldsElems;
    	}
    	// Если массивов еще нет
    	if($catsfor)
    		$fld = 'pd_categs';
    	else
    		$fld = 'pd_fields';
    	$res = sql_query('SELECT '.$fld.' FROM sb_plugins_data WHERE pd_plugin_ident = ?', $this->mPlugin);
    	if(!$res)
    	{
    		return array();
    	}

    	$cms_fields = unserialize($res[0][0]);
    	if($catsfor)
    		$this -> mFieldsCats = $cms_fields;
    	else
    		$this -> mFieldsElems = $cms_fields;
    	return $cms_fields;
    }

	/**
	 * Загружает фото на сервер через урл
	 *
	 * @param string url - урл изображения
	 * @param string $field - название поля
	 * @param bool $elem - если true - загружается фото для элемента, если false - для раздела
	 *
	 * @return string путь к новому изображению на сервере
	 */
	private function imageDownload($url, $field, $elem = true)
	{
		if($this->mDownloadPicsFolder == '')
			$this->mDownloadPicsFolder = $_SERVER['DOCUMENT_ROOT'].'/upload';

		if(isset($this->mUserFieldsSettings[$field]))
		{
			// Если есть настройки поля - беру их
			$settings = $this->mUserFieldsSettings[$field];
		}
		else
		{
			//	Если нету, достаю из базы
			if($elem)
				$from = 'pd_fields'; // Поле элемента
			else
				$from = 'pd_categs'; // Поле раздела

			$res = sql_query('SELECT '.$from.' FROM sb_plugins_data WHERE pd_plugin_ident = ?',$this->mPlugin);
			if(!$res)
				return '';

			$fields_settings = unserialize($res[0][0]);
			foreach($fields_settings as $v)
			{
				if('user_f_'.$v['id'] == $field)
				{
					$settings = $v['settings'];
					// Сохраняю в переменную
					$this->mUserFieldsSettings['user_f_'.$v['id']] = $settings;
					break;
				}
			}
		}

		// Привожу в порядок путь
		$file_name = basename($url);
		if(!preg_match('/\/$/', $this->mDownloadPicsFolder))
		{
			$this->mDownloadPicsFolder .= '/'; // Если путь пришел без последнего слеша - добавляем
		}
		$full_path = $this->mDownloadPicsFolder.md5($field.time()).$file_name;
		$full_path = sb_str_replace('//','/',$full_path);

		// Скачиваю
		require_once(SB_CMS_LIB_PATH.'/sbDownload.inc.php');
		$httpRequest = new sbDownload($url);
    	$pic = $httpRequest->download();

    	$GLOBALS['sbVfs']->mLocal = true;
    	$GLOBALS['sbVfs']->file_put_contents($full_path, $pic);
		$GLOBALS['sbVfs']->mLocal = false;
		return array(
			'url' => sb_str_replace(SB_BASEDIR,'',$full_path),
			'patch' => $full_path,
			'settings' => $settings,
			'field' => $field
		);
	}

// Конец класса
}

?>