<?php
/**
 * Подключение библиотеки работы с СУБД MySQL
 *
 * Если доступно расширение PHP MySQLi и версия MySQL больше или равна 4.1.3, то вся работа идет через данное расширение, иначе - MySQL.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_DB
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */

/**
 * Тип системного сообщения - ИНФОРМАЦИЯ
 */
define('SB_MSG_INFORMATION', 1);
/**
 * Тип системного сообщения - ПРЕДУПРЕЖДЕНИЕ
 */
define('SB_MSG_WARNING', 2);
/**
 * Тип системного сообщения - ОШИБКА
 */
define('SB_MSG_ERROR', 3);
/**
 * Тип системного сообщения - PHP-ОШИБКА
 */
define('SB_MSG_PHP_ERROR', 4);
/**
 * Тип системного сообщения - MYSQL-ОШИБКА
 */
define('SB_MSG_SQL_ERROR', 5);

define('SB_TEN_PERCENT', (rand(1, 10) == 10));

require_once SB_CMS_LIB_PATH.'/sbTotalDBCache.inc.php';
require_once SB_CMS_LIB_PATH.'/sbCacheSql.inc.php';

/**
 * Добавляет системное сообщение в системный журнал
 *
 * @see SB_MSG_INFORMATION
 * @see SB_MSG_WARNING
 * @see SB_MSG_ERROR
 * @see SB_MSG_PHP_ERROR
 * @see SB_MSG_SQL_ERROR
 *
 * @param string $message Текст сообщения
 * @param int $type Тип сообщения (SB_MSG_INFORMATION, SB_MSG_WARNING и т.п.)
 */
function sb_add_system_message($message, $type = SB_MSG_INFORMATION, $user_id = 0, $user_login = '')
{
    static $needClear = SB_TEN_PERCENT; // запускаем чистку журнала с вероятностью 10%

    if (empty($message) || !isset($GLOBALS['sbSql']) || sb_stripos($message, 'Cannot set time limit') !== false)
        return;

    if (!is_resource($GLOBALS['sbSql']->mLinkId) && !($GLOBALS['sbSql']->mLinkId instanceof mysqli))
        return;

    sql_query('UNLOCK TABLES');

	if (class_exists('sbPlugins'))
	{
		$system_log_date = intval(sbPlugins::getSetting('sb_system_log_date'));
		if ($system_log_date <= 0)
		{
			return;
		}

        if ($needClear)
        {
        	$time = time() - $system_log_date * 24 * 60 * 60;

            //Удаляем старые записи только один раз за время выполнения скрипта
            sql_query('DELETE FROM sb_system_log WHERE sl_date < '.$time);
            $needClear = false;
        }
    }

    /**
     * Логин пользователя, инициировавшего запись в системный журнал
     */
    if (isset($_SESSION['sbAuth']) && $_SESSION['sbAuth'] instanceof sbAuth && method_exists($_SESSION['sbAuth'], 'isAdmin'))
    {
        $user_id = $_SESSION['sbAuth']->getUserId();
        $user_login = $_SESSION['sbAuth']->getUserLogin() == '' ? 'SYSTEM' : $_SESSION['sbAuth']->getUserLogin();
    }
    elseif($user_id < 1 || $user_login == '')
    {
        $user_id = -1;
        $user_login = 'SYSTEM';
    }

    /**
     * Записываем сообщение в системный журнал
     */
    $GLOBALS['sbSql']->paramQuery('INSERT INTO sb_system_log (sl_date, sl_user_id, sl_user_login, sl_user_ip, sl_message, sl_type, sl_domain) VALUES (?d, ?d, ?, ?, ?, ?, ?)', time(), $user_id, $user_login, (isset($_SESSION['sbAuth']) ? $_SESSION['sbAuth']->getUserIP() : (class_exists('sbAuth') ? sbAuth::getIP() : '-')), $message, $type, SB_COOKIE_DOMAIN);
}

// Подключаем библиотеку работы с СУБД mySQL
$GLOBALS['sbSql'] = sbSql::factory();

/**
 * Вызывает метод $GLOBALS['sbSql']->paramQuery().
 *
 * @see sbMySql::paramQuery()
 * @see sbMySqli::paramQuery()
 * @see sbMSSql::paramQuery()
 *
 * @param string $query SQL-запрос
 * @param mixed $arg1,... Значения макросов в запросе
 * @param string $message Текст системного сообщения
 * @param int $type Тип сообщения (SB_MSG_INFORMATION, SB_MSG_WARNING и т.д.)
 *
 * @return mixed Двумерный массив для SELECT-запросов или идентификатор запроса. FALSE в случае
 *         возникновения ошибки.
 */
function sql_query()
{
	if (!is_object($GLOBALS['sbSql']))
        return false;

    $args = func_get_args();
    if (!is_array($args) || trim($args[0]) == '')
    {
        return false;
    }

    $message = '';
    $type = SB_MSG_INFORMATION;
    $num = count($args) - 1;

    if ($num > 0)
    {
	    /**
	     * Определяем кол-во макросов в запросе
	     */
	    $num_placeholders = $GLOBALS['sbSql']->placeholdersCount($args[0]);

	    /**
	     * Определяем, указано ли системное сообщение и его тип
	     */
	    $diff = $num - $num_placeholders;
	    switch ($diff)
	    {
	        case 1:
	            $message = array_pop($args);
	            break;

	        case 2:
	            $type = array_pop($args);
	            $message = array_pop($args);
	            break;
	    }
    }

    $result = $GLOBALS['sbSql']->paramQuery($args);

    /**
     * Если запрос выполнен и указано системное сообщение, записываем системное сообщение в журнал
     */
    if ($result && $message != '')
    {
        sb_add_system_message($message, $type);
    }

    return $result;
}

/**
 * Синоним sql_query().
 */
function sql_param_query()
{
	$args = func_get_args ();
    return call_user_func_array('sql_query', $args);
}

/**
 * Вызывает метод $GLOBALS['sbSql']->paramAssoc().
 *
 * @see sbMySql::paramAssoc()
 * @see sbMySqli::paramAssoc()
 * @see sbMSSql::paramAssoc()
 *
 * @param string $query SQL-запрос
 * @param mixed $arg1,... Значения макросов в запросе
 * @param string $message Текст системного сообщения
 * @param int $type Тип сообщения (SB_MSG_INFORMATION, SB_MSG_WARNING и т.д.)
 *
 * @return mixed Ассоциативный массив для SELECT-запросов или идентификатор запроса. FALSE в случае
 *         возникновения ошибки.
 */
function sql_assoc()
{
    if (!is_object($GLOBALS['sbSql']))
        return false;

    $args = func_get_args();
    if (!is_array($args) || trim($args[0]) == '')
    {
        return false;
    }

    $message = '';
    $type = SB_MSG_INFORMATION;
    $num = count($args) - 1;

    if ($num > 0)
    {
	    /**
	     * Определяем кол-во макросов в запросе
	     */
	    $num_placeholders = $GLOBALS['sbSql']->placeholdersCount($args[0]);

	    /**
	     * Определяем, указано ли системное сообщение и его тип
	     */
	    $diff = $num - $num_placeholders;
	    switch ($diff)
	    {
	        case 1:
	            $message = array_pop($args);
	            break;

	        case 2:
	            $type = array_pop($args);
	            $message = array_pop($args);
	            break;
	    }
    }

    $result = $GLOBALS['sbSql']->paramAssoc($args);

    /**
     * Если запрос выполнен и указано системное сообщение, записываем системное сообщение в журнал
     */
    if ($result && $message != '')
    {
        sb_add_system_message($message, $type);
    }

    return $result;
}

/**
 * Синоним sql_assoc().
 */
function sql_param_assoc()
{
	$args = func_get_args ();
    return call_user_func_array('sql_assoc', $args);
}

/**
 * Вызывает метод $GLOBALS['sbSql']->getInsertId()
 *
 * @see sbMySql::getInsertId()
 * @see sbMySqli::getInsertId()
 *
 * @return int Значение AUTO_INCREMENT поля или 0, если предыдущий запрос INSERT был неудачным.
 */
function sql_insert_id()
{
    return $GLOBALS['sbSql']->getInsertId();
}

/**
 * Вызывает метод $GLOBALS['sbSql']->getAffectedRows()
 *
 * @see sbMySql::getAffectedRows()
 * @see sbMySqli::getAffectedRows()
 *
 * @return int Кол-во записей или 0, если предыдущий запрос был неудачным.
 */
function sql_num_rows()
{
    return $GLOBALS['sbSql']->getAffectedRows();
}

/**
 * Интерфейс для классов, работающих с БД
 *
 * @author Сергей Черячукин <sergey.cheryachhukin@binn.com.ua>
 */
interface sbISql
{
    /**
     * Осуществляет подключение к БД
     *
     * @param string $database Имя базы данных, к которой производится подключение
     * @param strign $host Хост базы данных
     * @param strign $user Пользователь, под которым производится подключение
     * @param strign $password Пароль пользователя
     *
     * @return resource Идентификатор соединения или FALSE в случае возникновения ошибки
     */
    function connect($database, $host, $user, $password = '');

    /**
     * Выбор рабочей БД
     *
     * @param string $database
     * @return bool TRUE - если ошибок не было и FALSE в случае возникновения ошибки
     */
    function changeDB($database);

    /**
     * Закрывает соединение с БД
     *
     * Закрывает соединение с базой данных. Если в качестве параметра передан объект mysqli соединения с БД,
     * то закрывается это соединение, иначе соединение, на которое указывает переменная класса mLinkId.
     *
     * @param mysqli $link_id
     */
    function disconnect($link_id = false);

    /**
     * Освобождает память, выделенную под результат запроса
     *
     * В качестве объекта запроса используется либо переданный в функцию объект mysqli_result,
     * либо, если переданный параметр не является объектом mysqli_result, значение переменной класса mQueryId.
     *
     *
     * @param resource $query_id
     *
     */
    function free($query_id = false);

    /**
     * Возвращает значение, сгенерированное для AUTO_INCREMENT поля предыдущим запросом INSERT
     *
     * @return int Значение AUTO_INCREMENT поля или 0, если предыдущий запрос INSERT был неудачным.
     */
    function getInsertId();

    /**
     * Возвращает кол-во записей, затронутых предыдущим SQL-запросом.
     *
     * Для SELECT-запроса это кол-во выбранных из БД записей, для запросов INSERT, UPDATE и DELETE - кол-во записей,
     * затронутых данными запросами.
     *
     * @return int Кол-во записей или 0, если предыдущий запрос был неудачным.
     */
    function getAffectedRows();

    /**
     * Дублирует запись в таблице
     *
     * @param int $id Уникальный идентификатор дублируемой записи (обычно значение AUTO_INCREMENT поля).
     * @param string $id_field Название поля таблицы, в котором хранятся значения $id.
     * @param string $table Название таблицы, запись которой мы хотим продублировать.
     * @param string $null_fields Названия полей, которым следует присвоить NULL-значения.
     * @param array $fields_values Поля, которым следует присвоить новые значения. key-название поля, value-новое значение.
     *
     * @return int Значение AUTO_INCREMENT поля новой записи или 0 в случае возникновения ошибки
     */
    function duplicateRow($id, $id_field, $table, $null_fields='', $fields_values = array());

    /**
     * Выполняет параметризованный запрос
     *
     * @param string $query Текст SQL-запрос
     * @param mixed $arg1,... Значения макросов в запросе
     *
     * @return mixed Двумерный массив для SELECT-запросов или результат работы функции mysqli_query(). FALSE в случае
     *         возникновения ошибки.
     */
    function paramQuery();

    /**
     * Аналогичен методу paramQuery за исключением того, что в переменную $total записывается общее кол-во записей в таблице
     *
     * Удобно применять данный метод при использовании пейджирования результатов запроса.
     *
     * @param int $total Получает общее кол-во найденных записей (независимо от использования LIMIT)
     * @param string $query Текст SQL-запрос
     * @param mixed $arg1,... Значения макросов в запросе
     *
     * @return mixed Двумерный массив для SELECT-запросов или результат работы функции mysqli_query(). FALSE в случае
     *         возникновения ошибки.
     */
    function paramQueryPage(&$total);

    /**
     * Аналогичен paramQuery
     *
     * @see query
     *
     * @param string $query Текст SQL-запрос
     * @param mixed $arg1,... Значения макросов в запросе
     *
     * @return mixed Ассоциативный массив для SELECT-запросов или результат работы функции mysqli_query(). FALSE в случае
     *         возникновения ошибки.
     */
    public function paramAssoc();

    /**
     * Аналогичен paramQueryPage, но возвращает набор ассоциативных массивов
     *
     * @see paramQueryPage
     *
     * @param int $total Получает общее кол-во найденных записей (независимо от использования LIMIT)
     * @param string $query Текст SQL-запрос
     * @param mixed $arg1,... Значения макросов в запросе
     *
     * @return mixed Ассоциативный массив для SELECT-запросов или результат работы функции mysqli_query(). FALSE в случае
     *         возникновения ошибки.
     */
    public function paramAssocPage(&$total);

    /**
     * Экранирует переданную строку в соотв. с правилами СУБД
     *
     * @param string $str Строка для экранирования
     * @param $is_ident Название поля или таблицы (TRUE) или значение (FALSE)
     * @param $add_quotes Обрамляет результат в ` или '
     *
     * @return string Экранированная строка
     */
    function escape($str, $is_ident=false, $add_quotes=true);

    /**
     * Возвращает кол-во макросов в запросе
     *
     * @param string $query Текст SQL-запроса
     *
     * @return int Кол-во найденных макросов
     */
    function placeholdersCount($query);

    /**
     * Возвращает массив с названиями полей для указанной таблицы.
     *
     * @param string $table Имя таблицы
     *
     * @return array Массив с названиями полей
     */
    function getFields($table);

    /**
     * Возвращает версию mySQL
     *
     * @return string Описание версии mySQL
     */
    function getVersion();

    /**
     * Возвращает количество SQL-запросов
     *
     * @return int Количество SQL-запросов.
     */
    function getSqlCount();
}

/**
 * Класс-фабрика, подключающий соответствующий класс для работы с БД
 *
 * @author Сергей Черячукин <sergey.cheryachhukin@binn.com.ua>
 */

class sbSql
{
    static function factory()
    {
        $database = $GLOBALS['sb_db_database'];
        $host = $GLOBALS['sb_db_host'];
        $user = (defined('SB_SITE_MODE') && isset($GLOBALS['sb_db_site_user']) && trim($GLOBALS['sb_db_site_user']) != '' ? $GLOBALS['sb_db_site_user'] : $GLOBALS['sb_db_user']);
        $password = (defined('SB_SITE_MODE') && isset($GLOBALS['sb_db_site_password']) && isset($GLOBALS['sb_db_site_user']) && trim($GLOBALS['sb_db_site_user']) != '' ? $GLOBALS['sb_db_site_password'] : $GLOBALS['sb_db_password']);

        // Подключаем библиотеку работы с СУБД mySQL
        $dbAdapter = '';
        if (defined('SB_DB_TYPE') && SB_DB_TYPE == 'mssql')
        {
            $dbAdapter = 'sbMssql';
        }
        elseif (defined('SB_DB_TYPE') && SB_DB_TYPE == 'mysql')
        {
            $dbAdapter = 'sbMysql';
        }
        elseif (defined('SB_DB_TYPE') && SB_DB_TYPE == 'mysqli')
        {
            $dbAdapter = 'sbMysqli';
        }
        else if (extension_loaded('mysqli'))
        {
            @ini_set('zend.ze1_compatibility_mode', 0);
            if (@ini_get('zend.ze1_compatibility_mode') == 0 || strtoupper(@ini_get('zend.ze1_compatibility_mode')) == 'OFF')
                $dbAdapter = 'sbMysqli';
            else
                $dbAdapter = 'sbMysql';
        }
        else
        {
            $dbAdapter = 'sbMysql';
        }

        require_once SB_CMS_LIB_PATH . "/$dbAdapter.inc.php";

        switch($dbAdapter){
            case 'sbMysqli':
                $dbClass = new sbMySqli($database, $host, $user, $password);
                if (!($dbClass->mLinkId instanceof mysqli) && !isset($GLOBALS['sb_sql_not_exit']))
                {
                    sb_exit(KERNEL_ERROR_CONNECT_DB);
                }
                break;

            case 'sbMysql':
                if (extension_loaded('mysql'))
                {
                    $dbClass = new sbMySql($database, $host, $user, $password);
                    if (!is_resource($dbClass->mLinkId) && !isset($GLOBALS['sb_sql_not_exit']))
                    {
                        sb_exit(KERNEL_ERROR_CONNECT_DB);
                    }

                    if (!$dbClass->checkVersion() && !isset($GLOBALS['sb_sql_not_exit']))
                    {
                        sb_exit(KERNEL_ERROR_VERSION_DB);
                    }
                }
                else if (!isset($GLOBALS['sb_sql_not_exit']))
                {
                    sb_exit(KERNEL_ERROR_CONNECT_DB);
                }
                break;

            case 'sbMssql':
                if (extension_loaded('sqlsrv'))
                {
                    $dbClass = new sbMSSql($database, $host, $user, $password);
                    if (!is_resource($dbClass->mLinkId) && !isset($GLOBALS['sb_sql_not_exit']))
                    {
                        sb_exit(KERNEL_ERROR_CONNECT_DB);
                    }
                }
                else if (!isset($GLOBALS['sb_sql_not_exit']))
                {
                    sb_exit(KERNEL_ERROR_CONNECT_DB);
                }
                break;

            default :
                sb_exit(KERNEL_ERROR_CONNECT_DB);
        }
        unset($GLOBALS['sb_db_user']);
        unset($GLOBALS['sb_db_password']);

        if (isset($GLOBALS['sb_db_site_user']))
        {
            unset($GLOBALS['sb_db_site_user']);
        }

        if (isset($GLOBALS['sb_db_site_password']))
        {
            unset($GLOBALS['sb_db_site_password']);
        }

        unset($GLOBALS['sb_db_database']);
        unset($GLOBALS['sb_db_host']);

        return $dbClass;
    }
}
?>