<?php
/**
 * Реализация класса шаблонизатора
 *
 * Класс шаблонизатора используется для генерации статичных страниц сайта.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Templates
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */

/**
 * Класс шаблонизатора
 *
 * Класс шаблонизатора используется для генерации статичных страниц сайта.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Templates
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbTemplates
{
    /**
     * Связанные со страницей элементы
     *
     * @var array
     */
    private $mPageElems = array();

    /**
     * Идентификатор генерируемой страницы
     *
     * @var int
     */
    private $mPageId = -1;

    /**
     * Полный путь и имя файла страницы
     *
     * @var string
     */
    private $mPageFile = '';

    /**
     * Полный путь к файлу страницы
     *
     * @var string
     */
    private $mPagePath = '';

    /**
     * Свойства генерируемой страницы
     *
     * @var array
     */
    private $mPageProps = array();

    /**
     * Включено или выключено профилирование модулей
     * @var boolean
     */
    private $mWrapping = false;

    /**
     * Конструктор класса
     *
     * Инициализирует члены класса, проверяет доступность файла страницы для записи и производит другие проверки.
     *
     * @param int $page_id Идентификатор генерируемой страницы.
     * @param bool $error Получает значение TRUE, если во время конструирования объекта произошли ошибки.
     */
    public function __construct($page_id, &$error)
    {
        $error = true;
        $this->mPageId = $page_id;

        // получаем статус страницы
        $res = sql_query('SELECT p_name, p_filepath, p_filename, p_state, p_temp_id, p_xslt, p_title, p_keywords, p_description, p_meta, p_ctype, p_nocache FROM sb_pages WHERE p_id=?d', $this->mPageId);
        if ($res)
        {
            list($p_name, $p_filepath, $p_filename, $p_state, $p_temp_id, $p_xslt, $p_title, $p_keywords, $p_description, $p_meta, $p_ctype, $p_nocache) = $res[0];

            if ($p_filepath != '')
                $p_filepath = '/'.$p_filepath;

            $this->mPageProps = array(
                'p_name' => $p_name,
                'p_state' => $p_state,
                'p_xslt' => $p_xslt,
                'p_ctype' => $p_ctype,
                'p_nocache' => $p_nocache
                );
        }
        else
        {
            return;
        }

        $this->mPageFile = $p_filepath.'/'.$p_filename;
        $this->mPagePath = $p_filepath;

        // вытаскиваем все теги макета дизайна и свойства раздела страницы
        $res = sql_query('SELECT c.cat_id, c.cat_title, c.cat_left, c.cat_right, c.cat_closed, c.cat_fields,
                                       t.t_name, t.t_html, t.t_xslt, t.t_ctype
                                FROM sb_categs c, sb_catlinks l, sb_templates t
                                WHERE l.link_el_id=?d AND c.cat_id=l.link_cat_id AND c.cat_ident=? AND t.t_id=?d', $this->mPageId, 'pl_pages', $p_temp_id);

        if (!$res)
        {
            sb_add_system_message(sprintf(SB_TEMPLATES_ERROR1, $this->mPageProps['p_name']), SB_MSG_WARNING);
            return;
        }

        list ($cat_id, $cat_title, $cat_left, $cat_right, $cat_closed, $cat_fields, $t_name, $t_html, $t_xslt, $t_ctype) = $res[0];

        if (count($res) > 1 && $cat_closed == 1)
        {
            for ($i = 0; $i < count($res); $i++)
            {
                if ($res[$i][4] == 0)
                {
                    $cat_closed = 0;
                    break;
                }
            }
        }

        // вытаскиваем домен
        $res = sql_query('SELECT cat_title FROM sb_categs WHERE cat_left <= ?d AND cat_right >= ?d AND cat_level=1 AND cat_ident=?', $cat_left, $cat_right, 'pl_pages');
	    list($domain) = $res[0];

	    if ($domain == '')
	    {
	        sb_add_system_message(sprintf(SB_TEMPLATES_ERROR5, $this->mPageProps['p_name']), SB_MSG_WARNING);
            return;
	    }

	    $this->mPageProps['p_domain'] = $domain;
	    $this->mPageProps['p_name'] = $cat_title.' -> '.$this->mPageProps['p_name'];
	    $this->mPageProps['cat_id'] = $cat_id;
	    $this->mPageProps['cat_closed'] = $cat_closed;
        $this->mWrapping = (sbPlugins::getSetting('sb_profiler_plugins', $domain) == 1)? true : false;

        if ($t_html != '')
        {
        	// Устанавливаем версии для стилей и скриптов.
			$version_vals = $version_tags = array();

			$version_vals[] = intval(sbPlugins::getSetting('sb_new_version_css', $this->mPageProps['p_domain']));
			$version_tags[] = '{SB_CSS_VERSION}';

			$version_vals[] = intval(sbPlugins::getSetting('sb_new_version_js', $this->mPageProps['p_domain']));
			$version_tags[] = '{SB_JS_VERSION}';

            // заменяем title, keywords и description
            if (trim($p_title) != '')
            {
                $t_html = preg_replace('/<title>(.*?)<\/title>/si'.SB_PREG_MOD, '<title>'.$p_title.'</title>', $t_html);
            }

            if (trim($p_keywords) != '')
            {
		        if (preg_match('/(<meta[^>]+name\s*=\s*[\'|"]{0,1}\s*keywords\s*[\'|"]{0,1}[^>]*>)/is'.SB_PREG_MOD, $t_html, $matches))
		        {
		        	$version_tags[] = $matches[0];
		        	$version_vals[] = preg_replace('/(\s*content\s*=\s*[\'|"]?)([^\'"]*)([\'|"]?[^>]*>)/is'.SB_PREG_MOD, '$1'.$p_keywords.'$3', $matches[0]);
		        }
            }

        	if (trim($p_description) != '')
            {
		        if (preg_match('/(<meta[^>]+name\s*=\s*[\'|"]{0,1}\s*description\s*[\'|"]{0,1}[^>]*>)/is'.SB_PREG_MOD, $t_html, $matches))
		        {
		        	$version_tags[] = $matches[0];
		        	$version_vals[] = preg_replace('/(\s*content\s*=\s*[\'|"]?)([^\'"]*)([\'|"]?[^>]*>)/is'.SB_PREG_MOD, '${1}'.$p_description.'${3}', $matches[0]);
		        }
            }

            $t_html = sb_str_replace($version_tags, $version_vals, $t_html);
            if (trim($p_meta) != '')
            {
                $pos = sb_stripos($t_html, '</head>');
                if ($pos !== false)
                {
                    $t_html = sb_str_replace('</head>', $p_meta.'</head>', $t_html);
                }
                else
                {
                    $t_html .= $p_meta;
                }
            }
        }

	    if ($cat_fields != '')
	    {
	        $cat_fields = unserialize($cat_fields);
	        if (isset($cat_fields['page']) && $cat_fields['page'] != '')
	        {
	            $this->mPageProps['cat_closed_page'] = $cat_fields['page'];
	        }
	        else
	        {
	            $this->mPageProps['cat_closed_page'] = '/index.php';
	        }
	    }
	    else
	    {
	        $this->mPageProps['cat_closed_page'] = '/index.php';
	    }

	    $this->mPageProps['t_name'] = $t_name;
	    $this->mPageProps['t_html'] = $t_html;
	    if (trim($this->mPageProps['p_xslt']) == '')
	    {
	        $this->mPageProps['p_xslt'] = $t_xslt;
	    }

    	if (trim($this->mPageProps['p_ctype']) == '')
	    {
	        if (trim($t_ctype) != '')
	        	$this->mPageProps['p_ctype'] = $t_ctype;
	        else
	        	$this->mPageProps['p_ctype'] = 'text/html';
	    }

        // вытаскиваем связанные элементы
        $tags = array();
        preg_match_all('/\{[_A-Z0-9'.$GLOBALS['sb_reg_upper_interval'].']+\}/'.SB_PREG_MOD, $t_html, $tags);

        if ($tags[0])
        {
            sort($tags[0]);
            $elems = $_SESSION['sbPlugins']->getElems();

            $res = sql_query('SELECT e_id, e_tag, e_ident, e_temp_id, e_el_id, e_params, e_search, e_edit, e_yandex, e_link
                                    FROM sb_elems
      		                        WHERE e_p_id=?d AND e_link=? AND e_tag IN (?a)', $this->mPageId, 'page', $tags[0]);
            if ($res)
            {
                foreach ($res as $value)
                {
                    list($e_id, $e_tag, $e_ident, $e_temp_id, $e_el_id, $e_params, $e_search, $e_edit, $e_yandex, $e_link) = $value;

                    $e_params = trim($e_params);

                    if ($e_params != '')
                        $e_params = unserialize($e_params);

                    if (!$e_params)
                        $e_params = array();

                    if (isset($elems[$e_ident]) && function_exists($elems[$e_ident]['get_func']))
                    {
                        $this->mPageElems[$e_tag] = array();


                        $this->mPageElems[$e_tag]['html'] = $this->Profiler_Wrapper($elems[$e_ident]['get_func']($e_el_id, $e_temp_id, $e_params, array_search($e_tag, $tags[0]), $e_ident), $e_ident);
                        $this->mPageElems[$e_tag]['e_search'] = $e_search;
                        $this->mPageElems[$e_tag]['e_edit'] = $e_edit;
                        $this->mPageElems[$e_tag]['e_yandex'] = $e_yandex;
                        $this->mPageElems[$e_tag]['e_ident'] = $e_ident;
                        $this->mPageElems[$e_tag]['e_el_id'] = $e_el_id;
                        $this->mPageElems[$e_tag]['e_id'] = $e_id;
                        $this->mPageElems[$e_tag]['e_link'] = $e_link;
                        $this->mPageElems[$e_tag]['e_temp_id'] = $e_temp_id;
                        $this->mPageElems[$e_tag]['template_event'] = $elems[$e_ident]['template_event'];
                        $this->mPageElems[$e_tag]['template_ident'] = $elems[$e_ident]['template_ident'];
                        $this->mPageElems[$e_tag]['edit_event'] = $elems[$e_ident]['edit_event'];
                        $this->mPageElems[$e_tag]['get_event'] = $elems[$e_ident]['get_event'];
                        $this->mPageElems[$e_tag]['settings_width'] = $elems[$e_ident]['dlg_width'];
                        $this->mPageElems[$e_tag]['settings_height'] = $elems[$e_ident]['dlg_height'];
                    }
                }
            }

            $res = sql_query('SELECT e_id, e_tag, e_ident, e_temp_id, e_el_id, e_params, e_search, e_edit, e_yandex, e_link
                                    FROM sb_elems
      		                        WHERE e_p_id=?d AND e_link=? AND e_tag IN (?a)', $p_temp_id, 'temp', $tags[0]);

            if ($res)
            {
                foreach ($res as $value)
                {
                    list($e_id, $e_tag, $e_ident, $e_temp_id, $e_el_id, $e_params, $e_search, $e_edit, $e_yandex, $e_link) = $value;

                    $e_params = trim($e_params);

                    if ($e_params != '')
                        $e_params = unserialize($e_params);

                    if (!$e_params)
                        $e_params = array();

                    if (!isset($this->mPageElems[$e_tag]) && (isset($elems[$e_ident]) && function_exists($elems[$e_ident]['get_func']) || $e_ident == ''))
                    {
                        $this->mPageElems[$e_tag] = array();

                        if ($e_ident != '')
                        	//$this->mPageElems[$e_tag]['html'] = $elems[$e_ident]['get_func']($e_el_id, $e_temp_id, $e_params, array_search($e_tag, $tags[0]), $e_ident);
                            $this->mPageElems[$e_tag]['html'] = $this->Profiler_Wrapper($elems[$e_ident]['get_func']($e_el_id, $e_temp_id, $e_params, array_search($e_tag, $tags[0]), $e_ident), $e_ident);
                        else
                        	$this->mPageElems[$e_tag]['html'] = '';

                        $this->mPageElems[$e_tag]['e_search'] = $e_search;
                        $this->mPageElems[$e_tag]['e_edit'] = $e_edit;
                        $this->mPageElems[$e_tag]['e_yandex'] = $e_yandex;
                        $this->mPageElems[$e_tag]['e_ident'] = $e_ident;
                        $this->mPageElems[$e_tag]['e_el_id'] = $e_el_id;
                        $this->mPageElems[$e_tag]['e_id'] = $e_id;
                        $this->mPageElems[$e_tag]['e_link'] = $e_link;
                        $this->mPageElems[$e_tag]['e_temp_id'] = $e_temp_id;

                        if ($e_ident != '')
                        {
	                        $this->mPageElems[$e_tag]['template_event'] = $elems[$e_ident]['template_event'];
	                        $this->mPageElems[$e_tag]['template_ident'] = $elems[$e_ident]['template_ident'];
	                        $this->mPageElems[$e_tag]['edit_event'] = $elems[$e_ident]['edit_event'];
	                        $this->mPageElems[$e_tag]['get_event'] = $elems[$e_ident]['get_event'];
	                        $this->mPageElems[$e_tag]['settings_width'] = $elems[$e_ident]['dlg_width'];
	                        $this->mPageElems[$e_tag]['settings_height'] = $elems[$e_ident]['dlg_height'];
                        }
                        else
                        {
                        	$this->mPageElems[$e_tag]['template_event'] = '';
	                        $this->mPageElems[$e_tag]['template_ident'] = '';
	                        $this->mPageElems[$e_tag]['edit_event'] = '';
	                        $this->mPageElems[$e_tag]['get_event'] = '';
	                        $this->mPageElems[$e_tag]['settings_width'] = '';
	                        $this->mPageElems[$e_tag]['settings_height'] = '';
                        }
                    }
                }
            }

            foreach ($tags[0] as $tag)
            {
            	if ( !isset( $this->mPageElems[$tag] ) )
            	{
            		$this->mPageElems[$tag] = false;
            	}
            }
        }

        $error = false;
    }

    /**
     * Сохраняет генерируемую страницу в файл.
     *
     * @return bool TRUE, если все прошло успешно, FALSE в случае возникновения ошибки.
     */
    public function saveToFile()
    {
		$demo_statuses = explode(',', sb_get_workflow_demo_statuses(false));

        if (!in_array($this->mPageProps['p_state'], $demo_statuses))
        {
        	if ($this->mPageProps['p_state'] == 0)
        	{
        		if ($GLOBALS['sbVfs']->exists($this->mPageFile) && $GLOBALS['sbVfs']->is_file($this->mPageFile))
        			$GLOBALS['sbVfs']->delete($this->mPageFile);

	        	if ($GLOBALS['sbVfs']->exists($this->mPageFile.'.demo') && $GLOBALS['sbVfs']->is_file($this->mPageFile.'.demo'))
	        	{
	        		$GLOBALS['sbVfs']->delete($this->mPageFile.'.demo');
	        	}
        	}

            return true;
        }
        elseif ($this->mPageProps['p_state'] != 1 && in_array($this->mPageProps['p_state'], $demo_statuses))
        {
        	$this->mPageFile .= '.demo';
        }
        else
        {
        	if ($GLOBALS['sbVfs']->exists($this->mPageFile.'.demo') && $GLOBALS['sbVfs']->is_file($this->mPageFile.'.demo'))
        	{
        		$GLOBALS['sbVfs']->delete($this->mPageFile.'.demo');
        	}
        }

        $old_domain = $GLOBALS['sbVfs']->mDomain;
        $GLOBALS['sbVfs']->mDomain = $this->mPageProps['p_domain'];

        // проверяем, доступен ли файл страницы для записи
        if ($GLOBALS['sbVfs']->exists($this->mPageFile))
        {
            if (!$GLOBALS['sbVfs']->is_writable($this->mPageFile))
            {
                $GLOBALS['sbVfs']->mDomain = $old_domain;
                sb_add_system_message(sprintf(SB_TEMPLATES_ERROR2, $this->mPageProps['p_name'], $this->mPageFile), SB_MSG_WARNING);
                return false;
            }
        }
        elseif ($GLOBALS['sbVfs']->exists($this->mPagePath))
        {
            if (!$GLOBALS['sbVfs']->is_writable($this->mPagePath))
            {
                $GLOBALS['sbVfs']->mDomain = $old_domain;
                sb_add_system_message(sprintf(SB_TEMPLATES_ERROR3, $this->mPageProps['p_name'], $this->mPagePath), SB_MSG_WARNING);
                return false;
            }
        }
        elseif (!$GLOBALS['sbVfs']->mkdir($this->mPagePath))
        {
            $GLOBALS['sbVfs']->mDomain = $old_domain;
            sb_add_system_message(sprintf(SB_TEMPLATES_ERROR4, $this->mPageProps['p_name'], $this->mPagePath), SB_MSG_WARNING);
            return false;
        }

        $GLOBALS['sbVfs']->mDomain = $old_domain;

        // включаем prog_header.inc.php
        $html = "<?php
/**
 * ВНИМАНИЕ! Данная страница сгенерирована системой управления сайтом S.Builder.
 * Не изменяйте код страницы вручную. ВСЕ изменения должны вноситься
 * через административный интерфейс системы управления сайтом S.Builder.
 * Любые внесенные Вами вручную изменения будут утеряны при следующей генерации
 * страницы.
 *
 * WARNING! This page was generated with content management system S.Builder.
 * Do not change the page code manually. All changes should be made
 * through the administrative interface of CMS S.Builder.
 * Any manual changes you made will be lost with the next generation of the page.
 **/

// Идентификатор страницы
if (!defined('SB_PAGE_ID'))
{
	define ('SB_PAGE_ID', ".intval($this->mPageId).");
}

// Content-Type страницы
if (!defined('SB_PAGE_CONTENT_TYPE'))
{
	define('SB_PAGE_CONTENT_TYPE', '".$this->mPageProps['p_ctype']."');
}
";

        if ($this->mPageProps['p_xslt'] != '')
        {
            $html .= "
// Указана таблица стилей XSLT
define ('SB_XSLT', '".$this->mPageProps['p_xslt']."');

";
        }

        if ($this->mPageProps['p_nocache'] == 1)
        {
            $html .= "
// Включен запрет кэширования страницы сервером и браузером
define ('SB_PAGE_NOCACHE', true);

";
        }

        $html .= "require_once '".SB_CMS_KERNEL_PATH."/prog/header.inc.php';
require_once '".SB_CMS_KERNEL_PATH."/prog/demo.inc.php';\n";

        if ($this->mPageProps['cat_closed'])
        {
            // закрытая страница, включаем код проверки
            $html .= 'if (!isset($_SESSION[\'sbAuth\']) || !$_SESSION[\'sbAuth\']->checkPage('.$this->mPageProps['cat_id'].'))'."
            	{\n".
            	'$query_string = (trim($_SERVER[\'QUERY_STRING\']) != \'\' ? \'&\'.$_SERVER[\'QUERY_STRING\'] : \'\');'."\n".
            	'$page_url = \''.$this->mPageProps['cat_closed_page'].(sb_strpos($this->mPageProps['cat_closed_page'], '?') !== false ? '&' : '?').'su_lm=\'.sbAuth::getErrorCode().$query_string;'."\n".
                'header(\'Location: \'.sb_sanitize_header($page_url));'."\n
        		exit(0);
        		}";
        }



        $html .= "?>";

        $html .= $this->mPageProps['t_html'];

        foreach ($this->mPageElems as $tag => $value)
        {
        	if (!$value || $value['html'] == '')
        		continue;

            $elem_html = $value['html'];
            if ($value['e_yandex'] == 1)
            {
                $elem_html = '<!--noindex-->'.$elem_html.'<!--/noindex-->';
            }

            switch ($value['e_search'])
            {
                case 'all':
                    $elem_html = '<!--sb_index_start-->'.$elem_html.'<!--sb_index_end-->';
                    break;

                case 'links':
                    $elem_html = '<!--sb_index_start--><!--sb_index_links_start-->'.$elem_html.'<!--sb_index_links_end--><!--sb_index_end-->';
                    break;

                case 'text':
                    $elem_html = '<!--sb_index_start--><!--sb_index_not_links_start-->'.$elem_html.'<!--sb_index_not_links_end--><!--sb_index_end-->';
                    break;
            }

            $html = sb_str_replace($tag, $elem_html, $html);
        }

        $html = preg_replace('/\{[_A-Z0-9'.$GLOBALS['sb_reg_upper_interval'].']+\}/'.SB_PREG_MOD, '', $html);


        $html .= "<?php if (!isset(\$sb_no_footer) || !\$sb_no_footer) require_once SB_PROG_KERNEL_PATH.'/footer.inc.php'; ?>";

        $google_analytics_id = trim(sbPlugins::getSetting('sb_google_analytics_id', $this->mPageProps['p_domain']));
        if ($google_analytics_id != '')
        {
        	// Достаю макет счетчика из бд
        	$res = sql_query('SELECT cs_code FROM sb_counts WHERE cs_domain = ? AND cs_service = "google"', $this->mPageProps['p_domain']);
        	if($res && $res[0][0] != '')
        	{
        		$html = preg_replace('/(<\/head>)/i', str_replace('{GOOGLE_ID}', trim($google_analytics_id), $res[0][0]).'$1', $html, 1);
        	}
        	else
        	{
        		require_once(SB_CMS_LANG_PATH.'/pl_settings.lng.php');
        		$html = preg_replace('/(<\/head>)/i', str_replace('{GOOGLE_ID}', trim($google_analytics_id), PL_SETTINGS_GOOGLE_ANALYTICS_CODE).'$1', $html, 1);
        	}

        }

		$yandex_metrika_id = trim(sbPlugins::getSetting('sb_yandex_maps_metrika', $this->mPageProps['p_domain']));
        if($yandex_metrika_id != '')
        {
        	// Достаю макет счетчика из бд
        	$res = sql_query('SELECT cs_code FROM sb_counts WHERE cs_domain = ? AND cs_service = "yandex"', $this->mPageProps['p_domain']);
        	if($res && $res[0][0] != '')
        	{
				$html = preg_replace('/(<\/body>)/i', str_replace('{YANDEX_METRIKA_ID}', trim($yandex_metrika_id), $res[0][0]).'$1', $html, 1);
        	}
			else
        	{
				require_once(SB_CMS_LANG_PATH.'/pl_settings.lng.php');
				$html = preg_replace('/(<\/body>)/i', str_replace('{YANDEX_METRIKA_ID}', trim($yandex_metrika_id), PL_SETTINGS_YANDEX_METRIKA_CODE).'$1', $html, 1);
			}
		}

		$old_domain = $GLOBALS['sbVfs']->mDomain;
		$GLOBALS['sbVfs']->mDomain = $this->mPageProps['p_domain'];

		$crc = sb_crc($html);
        sql_param_query('UPDATE sb_pages SET p_crc=?d WHERE p_id=?d', $crc, $this->mPageId);

        if ($GLOBALS['sbVfs']->file_put_contents($this->mPageFile, $html) === false)
        {
            $GLOBALS['sbVfs']->mDomain = $old_domain;
            sb_add_system_message(sprintf(SB_TEMPLATES_ERROR2, $this->mPageProps['p_name'], $this->mPageFile), SB_MSG_WARNING);
            return false;
        }
        else
        {
            sb_add_system_message(sprintf(SB_TEMPLATES_OK, $this->mPageProps['p_name']));
        }

        $GLOBALS['sbVfs']->mDomain = $old_domain;

        return $crc;
    }

    /**
     * Выводит страницу для редактирования в визуальном редакторе.
     *
     * @return bool TRUE, если все прошло успешно, FALSE в случае возникновения ошибки.
     */
    public function printToFile()
    {
    	// включаем prog/header.inc.php
    	$html = "<?php \$_SERVER['PHP_SELF'] = '".$this->mPageFile."'; require_once '".SB_CMS_KERNEL_PATH."/prog/header.inc.php';\n";

        if ($this->mPageProps['p_xslt'] != '')
        {
            $html .= "define ('SB_XSLT', '".$this->mPageProps['p_xslt']."');\n";
        }

        $html .= " define('SB_VISUAL_EDITOR', 1); ?>";

        $html .= $this->mPageProps['t_html'];

        $system_tags = array();
        $matches = array();

        if (preg_match('/<title>(.*?)(\{[_A-Z0-9'.$GLOBALS['sb_reg_upper_interval'].']+\})(.*?)<\/title>/si'.SB_PREG_MOD, $this->mPageProps['t_html'], $matches))
            $system_tags[] = $matches[2];

        if (preg_match('/(<meta[^>]+name\s*=\s*[\'|"]{0,1}\s*keywords\s*[\'|"]{0,1}[^>]*>)/is'.SB_PREG_MOD, $this->mPageProps['t_html'], $matches))
        {
        	if (preg_match('/(\{[_A-Z0-9'.$GLOBALS['sb_reg_upper_interval'].']+\})/is'.SB_PREG_MOD, $matches[0], $matches))
        		$system_tags[] = $matches[0];
        }

    	if (preg_match('/(<meta[^>]+name\s*=\s*[\'|"]{0,1}\s*description\s*[\'|"]{0,1}[^>]*>)/is'.SB_PREG_MOD, $this->mPageProps['t_html'], $matches))
        {
        	if (preg_match('/(\{[_A-Z0-9'.$GLOBALS['sb_reg_upper_interval'].']+\})/is'.SB_PREG_MOD, $matches[0], $matches))
        		$system_tags[] = $matches[0];
        }

        $i = 0;
        foreach ($this->mPageElems as $tag => $value)
        {
        	if ($this->mPageProps['p_xslt'] != '')
        	{
        		$html .= $value ? $value['html'] : '';
        		continue;
        	}

            if (in_array($tag, $system_tags))
            {
                $html = sb_str_replace($tag, $value ? $value['html'] : '', $html);
                continue;
            }

            $tag_name = str_replace('_', ' ', trim($tag, '{}'));

        	if (!$value)
        	{
        		$value = array();

        		$value['html'] = '<center style="color:red;font-family:Verdana,Arial,Tahoma;font-size:12px;">&nbsp;'.sprintf(SB_TEMPLATES_NO_ELEM, $tag_name).'&nbsp;</center>';
        		$value['e_ident'] = '';
        		$value['e_el_id'] = -1;
        		$value['e_id'] = -1;
        		$value['e_link'] = '';
        		$value['e_temp_id'] = -1;
        		$value['template_event'] = '';
        		$value['template_ident'] = '';
        		$value['edit_event'] = '';
        		$value['get_event'] = '';
        		$value['settings_width'] = '';
        		$value['settings_height'] = '';
        		$value['e_edit'] = 1;
        	}
			else
			{
				if ($value['e_ident'] == '')
				{
					$value['html'] = '<center style="color:red;font-family:Verdana,Arial,Tahoma;font-size:12px;">&nbsp;'.sprintf(SB_TEMPLATES_NO_ELEM, $tag_name).'&nbsp;</center>';
				}

	            if ($value['e_edit'] != 1)
	            {
	                $html = sb_str_replace($tag, $value['html'], $html);
	                continue;
	            }
			}

        	$avail = false;
        	if ($value['template_event'] != '' && $value['template_ident'] != '' && $value['e_temp_id'] != -1)
        	{
        		$plugin_ident = $_SESSION['sbPlugins']->getEventPlugin($value['template_event']);

        		if ($plugin_ident && $_SESSION['sbPlugins']->isRightAvailable($plugin_ident, 'design_edit'))
        		{
	        		$res = sql_param_query('SELECT links.link_id, categs.cat_id FROM sb_catlinks links, sb_categs categs WHERE categs.cat_ident=? AND links.link_cat_id=categs.cat_id AND links.link_el_id=?d', $value['template_ident'], $value['e_temp_id']);
	        		if ($res)
	        		{
	        			list($link_id, $link_cat_id) = $res[0];

	        			if (fCategs_Check_Rights($link_cat_id))
					    {
					        $value['template_event'] = '?event='.$value['template_event'].'&plugin_ident='.$plugin_ident.'&cat_id='.$link_cat_id.'&link_id='.$link_id.'&link_src_cat_id=0&id='.$value['e_temp_id'];
	        				$avail = true;
					    }
	        		}
        		}
        	}

        	if (!$avail)
        	{
        		$value['template_event'] = '';
        	}

        	$avail = false;
        	if ($value['e_ident'] == 'pl_texts_html' || $value['e_ident'] == 'pl_texts_plain')
        	{
        	    $res = sql_param_query('SELECT categs.cat_id FROM sb_categs categs, sb_catlinks links WHERE categs.cat_ident=? AND links.link_cat_id=categs.cat_id AND links.link_el_id=?d', 'pl_texts', $value['e_el_id']);
        	    if ($res)
        	    {
        	    	foreach ($res as $cat_id)
        	    	{
	        	    	if (fCategs_Check_Rights($cat_id[0]))
		        	    {
		        	    	$avail = true;
		        	    	break;
		        	    }
        	    	}
        	    }
        	}
        	else
        	{
        		$avail = true;
        	}

        	if ($value['e_ident'] == '')
        		$brd_color = '#FF0000';
        	else if ($value['e_link'] == 'temp')
        		$brd_color = '#707070';
        	else
        		$brd_color = '#FF9933';

            $elem_html = '<div style="cursor:default;outline: 1px dotted '.$brd_color.'" title="'.$tag_name.'" ondblclick="sbEditorTopWnd.sbEditorEditElem(event);" id="sb_el_'.$i.'" sb_tag="'.trim($tag, '{}').'" sb_type="'.$value['e_ident'].'" sb_el_id="'.$value['e_el_id'].'" sb_avail="'.($avail ? '1' : '0').'" sb_e_id="'.$value['e_id'].'" sb_e_link="'.$value['e_link'].'" sb_template="'.$value['template_event'].'" sb_edit="'.$value['edit_event'].'" sb_get_event="'.trim($value['get_event']).'" sb_settings_width="'.$value['settings_width'].'" sb_settings_height="'.$value['settings_height'].'" oncontextmenu="sbEditorTopWnd.sbEditorShowMenu(event);">'.(trim($value['html']) != '' ? $value['html'] : '&nbsp;').'</div>';
            $html = sb_str_replace($tag, $elem_html, $html);

            $i++;
        }

        $html = preg_replace('/\{[_A-Z0-9'.$GLOBALS['sb_reg_upper_interval'].']+\}/'.SB_PREG_MOD, '', $html);

        if ($this->mPageProps['p_xslt'] != '')
        {
        	$html .= "<?php require_once SB_PROG_KERNEL_PATH.'/footer.inc.php'; ?>";
        }
        else
        {
        	$scripts = "
		        <link rel='stylesheet' type='text/css' href='".SB_CMS_CSS_URL."/editor/sbArea.css' />
		        <script type='text/javascript' src='".SB_CMS_JSCRIPT_URL."/sbFunctions.js.php'></script>
		        <script src='".SB_CMS_JSCRIPT_URL."/sbDAD.js.php'></script>
		        <script src='".SB_CMS_JSCRIPT_URL."/editor/sbVisualEditorArea.js.php'></script>
		        <script type='text/javascript'>
		        	var sbEditorTopWnd = window.parent;
		        	var sbNumContainers = $i;
		        	var sbPageId = ".$this->mPageId.";
		        	var sbDragger = new sbDragAndDrop();
					var sbDraggingStr = '".SB_TEMPLATES_DRAGGING."';

		        	sbAddEvent(window, 'load', sbEditorTopWnd.sbEditorAreaLoad);
		        	sbAddEvent(window, 'load', sbEditorAreaLoad);
				</script>";

        	$pos = sb_stripos($html, '<head>');
            if ($pos !== false)
            {
            	$html = sb_substr($html, 0, $pos + 6).'<base href="'.SB_DOMAIN.$this->mPagePath.'/" />'.$scripts.sb_substr($html, $pos + 6);
            }
            else
            {
		        $html .= $scripts;
			}

			$html .= "<?php require_once SB_PROG_KERNEL_PATH.'/footer.inc.php'; ?>";
        }

        sql_query('INSERT INTO sb_editor (e_text, e_u_id) VALUES(?, ?d)', $html, $_SESSION['sbAuth']->getUserId());

        return sql_insert_id();
    }

    private function Profiler_Wrapper($html, $ident)
    {
        if(!$this->mWrapping || sb_stripos($html, '<?php') === false || sb_stripos($html, '<?php') > 0)
        {
            //Если функция возвращает НЕ php-код или профилирование отключено, то не профилируем
            return $html;
        }

        $html = preg_replace(array('/^<\?php/', '/\?>$/'), '', $html);
        $postfix = uniqid();
        $html = '<?php function call_'.$postfix.'($ident){
            sbProfiler::Plugin();
            '.$html.'
            sbProfiler::Plugin($ident);
        }
        call_'.$postfix.'("'.$ident.'");
        ?>';

        return $html;
    }
}
?>