<?php
/**
 * Вывод облака тегов
 *
 * @param array $res Выборка тегов из базы данных. Порядок полей - идентификатор тега, тег, рейтинг тега, максимальное время последнего добавления тега.
 * @param int $temp_id Идентификатор макета дизайна вывода облака тегов.
 * @param string $pt_page_list Строка вывода номеров страниц (распарсенный макет).
 * @param int $tags_total Общее кол-во тегов.
 * @param string $page Страница для вывода элементов, связанных с тегом.
 * @param string $get_prefix Имя GET-параметра (в ссылке).
 * 
 * @return string Строка вывода обалка тегов.
 */
function fClouds_Show(&$res, $temp_id, $pt_page_list, $tags_total, $page, $get_prefix)
{
	$res_temp = sbQueryCache::getTemplate('sb_clouds_temps', $temp_id);

    if (!$res_temp)
    {
        return '';
    }
    
    list($ct_count, $ct_top, $ct_element, $ct_empty, $ct_delim, $ct_bottom, $ct_font_from, $ct_font_to, $ct_strip_words, $ct_color, $ct_color_percent) = $res_temp[0];
    	 
	// верх вывода 
    $result = str_replace(array('{NUM_LIST}', '{ALL_COUNT}'), array($pt_page_list, $tags_total), $ct_top);
    $tags = array('{ID}', '{TEXT}', '{COUNT}', '{LINK}', '{FONT_SIZE}', '{COLOR}');
    $col = 0;
    
    $min_rating = -1;
    $max_rating = -1;
    $font_range = $ct_font_to - $ct_font_from;
    
    $min_count = -1;
    $max_count = -1;
    $count_range = 0;
    
    $min_time = -1;
    $max_time = -1;
    $color_range = 0;

    $r = hexdec(substr($ct_color, 1, 2));
    $g = hexdec(substr($ct_color, 3, 2));
    $b = hexdec(substr($ct_color, 5, 2));

    if ($ct_strip_words != '')
    {
    	$ct_strip_words = preg_replace('/\s\s+/', ' ', $ct_strip_words);
    	$ct_strip_words = explode(' ', $ct_strip_words);

    	array_walk($ct_strip_words, 'sbProgSpacer');

    	$replace_array = array_fill(0, count($ct_strip_words), ' ');
    }
    else
    {
    	$ct_strip_words = false;
    }

    foreach ($res as $value)
    {
        $values = array();
        $values[] = $value[0];
        
        if ($ct_strip_words !== false)
        {
        	$value[1] = sb_str_replace($ct_strip_words, $replace_array, $value[1], '', true);	
        }
        
        $values[] = $value[1];
        $values[] = $value[2];
        $values[] = (sb_strpos($page, '?') !== false ? $page.'&'.$get_prefix.'='.$value[0] : $page.'?'.$get_prefix.'='.$value[0]);
        
        $font_size = $ct_font_to;
        if ($font_range != 0 && count($res) > 1)
        {
        	// расчет размера шрифта
        	if ($max_rating == -1)
        	{
        		$min_rating = $value[2];
        		$max_rating = $value[2];
        		
        		$num = count($res);
        		for ($i = 1; $i < $num; $i++)
        		{
        			if ($res[$i][2] < $min_rating)
        				$min_rating = $res[$i][2];
        			elseif ($res[$i][2] > $max_rating)
        				$max_rating = $res[$i][2];
        		}
        		
        		$min_count = log($min_rating + 1);
				$max_count = log($max_rating + 1);
			
				$count_range = $max_count - $min_count;
        	}
        	
        	if ($count_range != 0)
        		$font_size = round($ct_font_from + (log($value[2] + 1) - $min_count) * $font_range / $count_range);
        	else
        		$font_size = $ct_font_to;
        }
        
        $values[] = $font_size;
        
        $color = $ct_color;
        if ($ct_color_percent != 0 && count($res) > 1)
        {
        	// расчет цвета
        	if ($max_time == -1)
        	{
        		$min_time = $value[3];
        		$max_time = $value[3];
        		
        		$num = count($res);
        		for ($i = 1; $i < $num; $i++)
        		{
        			if ($res[$i][3] < $min_time)
        				$min_time = $res[$i][3];
        			elseif ($res[$i][3] > $max_time)
        				$max_time = $res[$i][3];
        		}
        		
        		$color_range = $max_time - $min_time;
        	}
        	
        	if ($color_range != 0)
        	{
        		$percent = 50 * ($max_time - $value[3]) / $color_range;
        	
        		$nr = min(255, (255 - $r) * $percent / 100 + $r);
        		$ng = min(255, (255 - $g) * $percent / 100 + $g);
        		$nb = min(255, (255 - $b) * $percent / 100 + $b);
        	
        		$color = '#'.sprintf('%02x', $nr).sprintf('%02x', $ng).sprintf('%02x', $nb);
        	}
        }
        
        $values[] = $color;
        
        if ($col >= $ct_count)
        {
            $result .= $ct_delim;
            $col = 0;
        }

        $result .= str_replace($tags, $values, $ct_element);
        $col++;
    }
    
	while ($col < $ct_count)
    {
        $result .= $ct_empty;
        $col++;   
    }
    
    // низ вывода новостной ленты
    $result .= str_replace(array('{NUM_LIST}', '{ALL_COUNT}'), array($pt_page_list, $tags_total), $ct_bottom);
    
    return $result;
}

/**
 * Инициализация REQUEST-переменных для связывания списков элементов через тематические теги
 *
 * @param string $plugin_ident Идентификатор модуля, тематические теги которого надо инициализировать.
 * @param array $el_ids Массив идентификаторов элементов модуля, тематические теги которых надо инициализировать.
 */
function fClouds_Init_Tags($plugin_ident, $el_ids)
{
	if(!is_array($el_ids))
		return;

	// массив идентификаторов элементов модулей
	static $ids = array();

	if(isset($ids[$plugin_ident]) && is_array($ids[$plugin_ident]))
	{
		$el_ids = array_diff($el_ids, $ids[$plugin_ident]);
		// если все элементы уже обрабатывались ранее, ничего не делаем
		if(count($el_ids) == 0)
			return;
	}

	// заносим обрабатываемые идентификаторы в массив, чтобы не обрабатывать их повторно
	if(!isset($ids[$plugin_ident]))
		$ids[$plugin_ident] = $el_ids;
	else
		$ids[$plugin_ident] = array_merge($el_ids, $ids[$plugin_ident]);
			
	$res_ids = array();
	foreach($el_ids as $value)
	{
		if(is_numeric($value))
		{
			$res_ids[] = $value;
		}
	}

	// если пришли некорретные данные, выходим
	if(count($res_ids) == 0)
		return;

	$res = sql_query('SELECT cl_tag_id, cl_el_id FROM sb_clouds_links WHERE cl_ident=? AND cl_el_id IN (?a)', $plugin_ident, $res_ids);
	if ($res)
	{
		// извлекаем уже инициализированные тематические теги
		$tags = isset($_REQUEST['f_tag_comp']) ? explode(',', $_REQUEST['f_tag_comp']) : array();
		foreach ($res as $value)
		{
			if (!in_array($value[0], $tags))
				$tags[] = $value[0];
		}

		$tags = implode(',', $tags);
		$_REQUEST['f_tag_comp'] = $tags;
	    $_REQUEST['im_tag_comp'] = $tags;
		$_REQUEST['n_tag_comp'] = $tags;

		$res = sql_query('SELECT pm_id FROM sb_plugins_maker');
	    if ($res)
	    {
			foreach ($res as $value)
	    	{
				$_REQUEST['p_'.$value[0].'_tag_comp'] = $tags;
	    	}
		}
		$_REQUEST['ssr_tag_comp'] = $tags;
	}
	elseif ($plugin_ident != 'pl_menu' && !isset($_REQUEST['f_tag_comp']))
	{
		$_REQUEST['f_tag_comp'] = -1;
	    $_REQUEST['im_tag_comp'] = -1;
	    $_REQUEST['n_tag_comp'] = -1;

	    $res = sql_query('SELECT pm_id FROM sb_plugins_maker');
	    if ($res)
	    {
	    	foreach ($res as $value)
	    	{
	    		$_REQUEST['p_'.$value[0].'_tag_comp'] = -1;
	    	}
	    }
		$_REQUEST['ssr_tag_comp'] = -1;
	}
}

?>