<?php
function fFolders_Read()
{
    require_once(SB_CMS_LIB_PATH . '/sbFolders.inc.php');

    $folders = new sbFolders();
    $folders->show();
}

function fFolders_Check_Rights($path)
{
    if ($_SESSION['sbAuth']->isAdmin()) {
        return true;
    }

    $path = rtrim(str_replace('\\', '/', $path), '/');

    if (trim($path) == '') {
        $path = '/';
    }

    // проверяем, есть ли у пользователя права на просмотр директории
    $res = sql_query('SELECT f_ids FROM sb_folders_rights WHERE f_path=? AND f_domain=?', $path, SB_COOKIE_DOMAIN);
    if (!$res) {
        return false;
    }

    $cat_rights = $res[0][0];
    if ($cat_rights == '') {
        return false;
    }

    $cat_rights = explode('^', $cat_rights);
    $cat_groups = array();

    foreach ($cat_rights as $right) {
        if (trim($right) == '') {
            continue;
        }

        if ($right[0] == 'g') {
            $cat_groups[] = intval(substr($right, 1));
        } elseif ($right[0] == 'u') {
            if (intval(substr($right, 1)) == $_SESSION['sbAuth']->getUserId()) {
                return true;
            }
        } else {
            $cat_groups[] = intval($right);
        }
    }

    $groups = $_SESSION['sbAuth']->getUserGroups();

    if (count(array_intersect($groups, $cat_groups)) > 0) {
        return true;
    }

    return false;
}

function fFolders_Add_Folder()
{
    if (!$_SESSION['sbPlugins']->isRightAvailable('pl_filelist', 'folders_edit')) {
        return;
    }

    if (!isset($_GET['par_path']) && !isset($_GET['path'])) {
        return;
    }

    if (isset($_GET['par_path']) && !fFolders_Check_Rights($_GET['par_path'])) {
        sb_show_message(SB_FILELIST_DENY_ERROR_MSG, true, 'warning');
        return;
    }

    if (isset($_GET['path']) && !fFolders_Check_Rights($_GET['path'])) {
        sb_show_message(SB_FILELIST_DENY_ERROR_MSG, true, 'warning');
        return;
    }

    echo '<br><br>
    <script>
    var allowedFoldChars = " QAZWSXEDCRFVTGBYHNMJUIKLOPqwertasdfgzxcvbyhnmjuiklop_1234567890.-[]\\\\/";

    function checkValues()
    {
        var el =  sbGetE("folder_name");
        if (el.value == "")
        {
            alert("' . PL_FOLDERS_ADD_FOLDER_ERROR_NO_NAME . '");
            return false;
        }
        for (i = 0; i < el.value.length; i++)
        {
            if (allowedFoldChars.indexOf(el.value.charAt(i)) == -1)
            {
                alert("' . PL_FOLDERS_ADD_FOLDER_ERROR_INCORRECT_NAME . '");
                return false;
            }
        }

        return true;
    }
    </script>';

    require_once(SB_CMS_LIB_PATH . '/sbLayout.inc.php');
    $value = '';

    if (isset($_GET['par_path'])) {
        // добавление
        $layout = new sbLayout(SB_CMS_MODAL_DIALOG_FILE . '?event=pl_folders_add_folder_submit&par_path=' . urlencode($_GET['par_path']), 'thisDialog', 'post',
            'checkValues()');
    } else {
        if ($_GET['path'] == '/' || $_GET['path'] == '\\') {
            sb_show_message(PL_FOLDERS_FOLDER_RENAME_ROOT_ERROR, true, 'warning');
            return;
        }
        // переименование
        $layout = new sbLayout(SB_CMS_MODAL_DIALOG_FILE . '?event=pl_folders_add_folder_submit&path=' . urlencode($_GET['path']), 'thisDialog', 'post',
            'checkValues()');
        $value = explode('/', str_replace('\\', '/', $_GET['path']));
        if (is_array($value)) {
            $value = $value[count($value) - 1];
        }
    }

    $layout->mTableWidth = '95%';

    $layout->addHeader(PL_FOLDERS_ADD_FOLDER_TITLE);
    $layout->addField(PL_FOLDERS_ADD_FOLDER_NAME, new sbLayoutInput('text', $value, 'folder_name', '', 'style="width:340px;"', true));
    $layout->addButton('submit', isset($_GET['path']) ? KERNEL_RENAME : KERNEL_CREATE);
    $layout->addButton('button', KERNEL_CANCEL, '', '', 'onclick="sbCloseDialog()"');
    $layout->show();
}

function fFolders_Add_Folder_Submit()
{
    if (!$_SESSION['sbPlugins']->isRightAvailable('pl_filelist', 'folders_edit')) {
        return;
    }

    if (isset($_GET['par_path'])) {
        // добавление
        $path = preg_replace('/[^\s0-9a-zA-Z\/\\\\_\-\.\[\]]+/', '', $_GET['par_path']);
        $fold = trim(str_replace('\\', '/', preg_replace('/[^\s0-9a-zA-Z\/\\\\_\-\.\[\]]+/', '', $_POST['folder_name'])), '/');

        // запрещаем выход на более высокий уровень (..)
        $path = preg_replace('/\.\.+/', '', $path);
        $path = preg_replace('/\/\/+/', '', $path);

        $fold = preg_replace('/\.\.+/', '', $fold);
        $fold = preg_replace('/\/\/+/', '', $fold);

        $new_path = str_replace('//', '/', $path . '/' . $fold);

        if (!fFolders_Check_Rights($path)) {
            sb_show_message(SB_FILELIST_DENY_ERROR_MSG, true, 'warning');
            return;
        }

        //Проверка на существование файла с таким же имененм
        $path_tmp = str_replace('//', '/', $path);
        $file_list = $GLOBALS['sbVfs']->scandir($path_tmp);
        $preg_folder_name = sb_str_replace(array('.', '-'), array('\.', '\-'), $fold);

        if (!empty($file_list)) {
            foreach ($file_list as $file) {
                if (preg_match('/^' . $preg_folder_name . '(\..*)?$/', $file) && $GLOBALS['sbVfs']->is_file($path_tmp . '/' . $file)) {
                    sb_show_message(SB_FILELIST_FILE_EXISTS, false, 'warning');
                    fFolders_Add_Folder();
                    return;
                }
            }
        }

        if ($new_path != '' && $GLOBALS['sbVfs']->mkdir($new_path)) {
            $f_res = sql_query('SELECT f_ids FROM sb_folders_rights WHERE f_path=? AND f_domain=?', $path, SB_COOKIE_DOMAIN);
            if ($f_res) {
                // наследуем права папки
                $folds = explode('/', $fold);
                $rights_path = $path;
                $prev_path = $path;
                foreach ($folds as $value) {
                    if (trim($value) != '') {
                        $rights_path .= '/' . $value;
                        $rights_path = str_replace('//', '/', $rights_path);
                        if (!sql_query('SELECT f_ids FROM sb_folders_rights WHERE f_path=? AND f_domain=?', $rights_path, SB_COOKIE_DOMAIN)) {
                            sql_query('INSERT INTO sb_folders_rights (f_path, f_ids, f_domain) VALUES (?, ?, ?)', $rights_path, $f_res[0][0], SB_COOKIE_DOMAIN);
                        }
                        $prev_path .= '/' . $value;
                        $prev_path = str_replace('//', '/', $prev_path);
                    }
                }
            }

            echo '<script>
                sbReturnValue("' . $fold . '");
            </script>';
            // записываем действие в системный журнал
            sb_add_system_message(sprintf(PL_FOLDERS_FOLDER_CREATE_OK, $new_path));
        } else {
            sb_show_message(sprintf(PL_FOLDERS_FOLDER_CREATE_ERROR, $new_path), true, 'warning');
            // записываем действие в системный журнал
            sb_add_system_message(sprintf(PL_FOLDERS_FOLDER_CREATE_ERROR, $new_path), SB_MSG_WARNING);
        }
    } else {
        if ($_GET['path'] == '/' || $_GET['path'] == '\\') {
            sb_show_message(PL_FOLDERS_FOLDER_RENAME_ROOT_ERROR, true, 'warning');
            return;
        }

        // переименование
        $old_path = trim(str_replace('\\', '/', preg_replace('/[^\s0-9a-zA-Z\/\\\\_\-\.\[\]]+/', '', $_GET['path'])), '/');
        $old_path = preg_replace('/\.\.+/', '', $old_path);
        $old_path = preg_replace('/\/\/+/', '', $old_path);

        $fold = preg_replace('/[^\s0-9a-zA-Z_\-\.\[\]]+/', '', $_POST['folder_name']);
        $fold = preg_replace('/\.\.+/', '', $fold);
        $fold = preg_replace('/\/\/+/', '', $fold);

        $new_path = substr($old_path, 0, strrpos($old_path, '/')) . '/' . $fold;
        $old_path = '/' . $old_path;

        //Проверка на существование файла с таким же имененм
        $parent_fold = substr($old_path, 0, strrpos($old_path, '/'));
        $file_list = $GLOBALS['sbVfs']->scandir($parent_fold);
        $preg_folder_name = rtrim(sb_str_replace(array('.', '-'), array('\.', '\-'), $fold), '/');

        if (!empty($file_list)) {
            foreach ($file_list as $file) {
                if (preg_match('/^' . $preg_folder_name . '(\..*)?$/', $file) && $GLOBALS['sbVfs']->is_file($parent_fold . '/' . $file)) {
                    sb_show_message(SB_FILELIST_FILE_EXISTS, false, 'warning');
                    fFolders_Add_Folder();
                    return;
                }
            }
        }

        if (!fFolders_Check_Rights($old_path)) {
            // не удалось переименовать
            sb_show_message(SB_FILELIST_DENY_ERROR_MSG, true, 'warning');
            return;
        }

        if ($GLOBALS['sbVfs']->exists($new_path)) {
            // папка уже существует
            sb_show_message(sprintf(PL_FOLDERS_FOLDER_EXIST_ERROR, $new_path, $old_path), true, 'warning');
            // записываем действие в системный журнал
            sb_add_system_message(sprintf(PL_FOLDERS_FOLDER_EXIST_ERROR, $new_path, $old_path), SB_MSG_WARNING);
        } else {
            if ($GLOBALS['sbVfs']->rename($old_path, $new_path)) {
                // меняем права
                sql_query('UPDATE sb_folders_rights SET f_path = CONCAT(?, SUBSTRING(f_path, ' . (strlen($old_path) + 1) . ')) WHERE f_path LIKE ? AND f_domain=?',
                    $new_path, $old_path . '%', SB_COOKIE_DOMAIN);

                echo '<script>
                    var res = new Object();
                    res.title = "' . $fold . '";
                    res.id = "' . $new_path . '";
                    sbReturnValue(res);
                </script>';
                // записываем действие в системный журнал
                sb_add_system_message(sprintf(PL_FOLDERS_FOLDER_RENAME_OK, $old_path, $new_path));
            } else {
                // не удалось переименовать
                sb_show_message(sprintf(PL_FOLDERS_FOLDER_RENAME_ERROR, $old_path, $new_path), true, 'warning');
                // записываем действие в системный журнал
                sb_add_system_message(sprintf(PL_FOLDERS_FOLDER_RENAME_ERROR, $old_path, $new_path), SB_MSG_WARNING);
            }
        }
    }
}

function fFolders_Delete_Folder()
{
    if (!$_SESSION['sbPlugins']->isRightAvailable('pl_filelist', 'folders_delete')) {
        return;
    }

    $_GET['path'] = urldecode($_GET['path']);
    // удаление
    $path = preg_replace('/[^\s0-9a-zA-Z\/\\\\_\-\.\[\]]+/', '', $_GET['path']);
    // запрещаем выход на более высокий уровень (..)
    $path = preg_replace('/\.\.+/', '', $path);
    $path = preg_replace('/\/\/+/', '', $path);

    if (!fFolders_Check_Rights($path)) {
        return;
    }

    // что-то не так, лучше ничего не делать
    if ($path != $_GET['path']) {
        // записываем действие в системный журнал
        sb_add_system_message(sprintf(PL_FOLDERS_FOLDER_DELETE_ERROR, $_GET['path']), SB_MSG_WARNING);
        return;
    }

    if ($GLOBALS['sbVfs']->delete($path)) {
        echo 'TRUE';
        // удаляем права, связанные с папкой
        sql_query('DELETE FROM sb_folders_rights WHERE f_path LIKE ? AND f_domain=?', $path . '%', SB_COOKIE_DOMAIN);
        // записываем действие в системный журнал
        sb_add_system_message(sprintf(PL_FOLDERS_FOLDER_DELETE_OK, $path));
    } else {
        // записываем действие в системный журнал
        sb_add_system_message(sprintf(PL_FOLDERS_FOLDER_DELETE_ERROR, $path), SB_MSG_WARNING);
    }
}

function fFolders_Paste_Folder()
{
    // копирование и перенос
    if (!$_SESSION['sbPlugins']->isRightAvailable('pl_filelist', 'folders_edit')) {
        return;
    }

    // папка, в которую идет копирование
    $path = preg_replace('/[^\s0-9a-zA-Z\/\\\\_\-\.\[\]]+/', '', urldecode($_GET['path']));
    // запрещаем выход на более высокий уровень (..)
    $path = preg_replace('/\.\.+/', '', $path);
    $path = preg_replace('/\/\/+/', '', $path);

    // папка, которую копируем
    $paste_path = preg_replace('/[^\s0-9a-zA-Z\/\\\\_\-\.\[\]]+/', '', urldecode($_GET['paste_path']));
    // запрещаем выход на более высокий уровень (..)
    $paste_path = preg_replace('/\.\.+/', '', $paste_path);
    $paste_path = preg_replace('/\/\/+/', '', $paste_path);


    if (!fFolders_Check_Rights($path) || !fFolders_Check_Rights($paste_path)) {
        echo sprintf(PL_FOLDERS_FOLDER_NOT_RIGHT, $paste_path, $path);
        return;
    }

    //Проверка на существование файла с таким же имененм
    $fold = pathinfo($paste_path);
    $file_list = $GLOBALS['sbVfs']->scandir($path);
    $preg_folder_name = sb_str_replace(array('.', '-'), array('\.', '\-'), $fold['basename']);

    if (!empty($file_list)) {
        foreach ($file_list as $file) {
            if (preg_match('/^' . $preg_folder_name . '(\..*)?$/', $file) && $GLOBALS['sbVfs']->is_file(ltrim($path, '/') . '/' . $file)) {
                sb_add_system_message(sprintf(PL_FOLDERS_FILE_EXISTS, $paste_path, $path), SB_MSG_WARNING);
                echo sprintf(PL_FOLDERS_FILE_EXISTS, $paste_path, $path);
                return;
            }
        }
    }

    $action = preg_replace('/[^a-z]+/', '', $_GET['action']);

    $rights_path = explode('/', $paste_path);
    $rights_path = $path . '/' . $rights_path[count($rights_path) - 1];

    if ($action == 'copy') {
        // копирование
        if ($GLOBALS['sbVfs']->copy($paste_path, $path)) {
            // копирование прошло успешно
            echo 'TRUE';
            // копируем права доступа
            $res = sql_query('SELECT f_path, f_ids FROM sb_folders_rights WHERE f_path LIKE ? AND f_domain=?', $paste_path . '%', SB_COOKIE_DOMAIN);
            if ($res) {
                foreach ($res as $value) {
                    list($f_path, $f_ids) = $value;

                    $f_path = $rights_path . substr($f_path, strlen($paste_path));
                    sql_query('INSERT INTO sb_folders_rights (f_path, f_ids, f_domain) VALUES (?, ?, ?)', $f_path, $f_ids, SB_COOKIE_DOMAIN);
                }
            }

            // записываем действие в системный журнал
            sb_add_system_message(sprintf(PL_FOLDERS_FOLDER_COPY_OK, $paste_path, $path));
        } else {
            // записываем действие в системный журнал
            sb_add_system_message(sprintf(PL_FOLDERS_FOLDER_COPY_ERROR, $paste_path, $path), SB_MSG_WARNING);
            echo sprintf(PL_FOLDERS_FOLDER_COPY_ERROR, $paste_path, $path);
        }
    } elseif ($action == 'cut') {
        // перенос
        if ($GLOBALS['sbVfs']->copy($paste_path, $path) && $GLOBALS['sbVfs']->delete($paste_path)) {
            // перенос прошел успешно
            echo 'TRUE';
            // меняем права
            sql_query('UPDATE sb_folders_rights SET f_path = CONCAT(?, SUBSTRING(f_path, ' . (strlen($paste_path) + 1) . ')) WHERE f_path LIKE ? AND f_domain=?',
                $rights_path, $paste_path . '%', SB_COOKIE_DOMAIN);
            // записываем действие в системный журнал
            sb_add_system_message(sprintf(PL_FOLDERS_FOLDER_CUT_OK, $paste_path, $path));
        } else {
            // записываем действие в системный журнал
            sb_add_system_message(sprintf(PL_FOLDERS_FOLDER_CUT_ERROR, $paste_path, $path), SB_MSG_WARNING);
            echo sprintf(PL_FOLDERS_FOLDER_CUT_ERROR, $paste_path, $path);
        }
    }
}

function fFolders_Paste_Files()
{
    if (!$_SESSION['sbPlugins']->isRightAvailable('pl_filelist', 'files_cut_copy')) {
        return;
    }

    if ($_GET['old_path'] == $_GET['new_path']) {
        echo '0';
        return;
    }

    $old_path = trim(str_replace('\\', '/', preg_replace('/[^\s0-9a-zA-Z\/\\\\_\-\.\[\]]+/', '', urldecode($_GET['old_path']))), '/');
    // запрещаем выход на более высокий уровень (..)
    $old_path = preg_replace('/\.\.+/', '', $old_path);
    $old_path = preg_replace('/\/\/+/', '', $old_path);

    $new_path = trim(str_replace('\\', '/', preg_replace('/[^\s0-9a-zA-Z\/\\\\_\-\.\[\]]+/', '', urldecode($_GET['new_path']))), '/');
    // запрещаем выход на более высокий уровень (..)
    $new_path = preg_replace('/\.\.+/', '', $new_path);
    $new_path = preg_replace('/\/\/+/', '', $new_path);

    $old_path = '/' . $old_path;
    $new_path = '/' . $new_path;

    if (!fFolders_Check_Rights($old_path) || !fFolders_Check_Rights($new_path)) {
        echo '0';
        return;
    }

    $files = array();
    foreach ($_GET as $key => $value) {
        if ($key != 'action' && $key != 'event' && $key != 'old_path' && $key != 'new_path') {
            $files[] = preg_replace('/[^\s0-9a-zA-Z_\-\.\[\]]+/', '', basename(urldecode($value)));
        }
    }

    $num = 0;

    for ($i = 0; $i < count($files); $i++) {
        $oldfile = str_replace('//', '/', $old_path . '/' . $files[$i]);
        $newfile = str_replace('//', '/', $new_path . '/' . $files[$i]);

        //проверяем наличие одноименной директории
        $pathinfo = pathinfo($newfile);
        $test_dir_name = $pathinfo['dirname'] . '/' . $pathinfo['filename'];
        $test_dir_name = sb_str_replace('//', '/', $test_dir_name);

        if ($GLOBALS['sbVfs']->is_dir($test_dir_name)) {
            sb_add_system_message(sprintf(PL_FILES_PASTE_FILE_IS_DIR, $pathinfo['filename']), SB_MSG_WARNING);
            continue;
        }

        if ($GLOBALS['sbVfs']->copy($oldfile, $newfile, true)) {
            $ok = true;
            if ($_GET['action'] == 'cut') {
                if (!$GLOBALS['sbVfs']->delete($oldfile)) {
                    $ok = false;
                    sb_add_system_message(sprintf(PL_FILES_DELETE_FILE_ERROR, $oldfile), SB_MSG_WARNING);
                }
            }
            if ($ok) {
                $num++;
                sb_add_system_message(sprintf($_GET['action'] == 'cut' ? PL_FILES_PASTE_CUT_FILE_OK : PL_FILES_PASTE_COPY_FILE_OK, $oldfile, $new_path));
            }
        } else {
            sb_add_system_message(sprintf(PL_FILES_PASTE_FILE_ERROR, $oldfile, $new_path), SB_MSG_WARNING);
        }
    }

    echo $num;
}

function fFiles_Rename_File()
{
    if (!$_SESSION['sbPlugins']->isRightAvailable('pl_filelist', 'files_edit')) {
        return;
    }

    if (!isset($_GET['path'])) {
        return;
    }

    echo '<br><br>
    <script>
    var allowedFileChars = " QAZWSXEDCRFVTGBYHNMJUIKLOPqwertasdfgzxcvbyhnmjuiklop_1234567890.-[]";

    function checkValues()
    {
        var el =  sbGetE("file_name");
        if (el.value == "")
        {
            alert("' . PL_FILES_RENAME_FILE_ERROR_NO_NAME . '");
            return false;
        }
        for (i = 0; i < el.value.length; i++)
        {
            if (allowedFileChars.indexOf(el.value.charAt(i)) == -1)
            {
                alert("' . PL_FILES_RENAME_FILE_ERROR_INCORRECT_NAME . '");
                return false;
            }
        }

        return true;
    }
    </script>';

    require_once(SB_CMS_LIB_PATH . '/sbLayout.inc.php');
    // переименование
    $layout = new sbLayout(SB_CMS_MODAL_DIALOG_FILE . '?event=pl_folders_rename_file_submit&path=' . urlencode($_GET['path']), 'thisDialog', 'post',
        'checkValues()');
    $value = @basename($_GET['path']);
    $layout->addHeader(PL_FILES_RENAME_FILE_TITLE);
    $layout->addField(PL_FILES_RENAME_FILE_NAME, new sbLayoutInput('text', $value, 'file_name', '', 'style="width:96%;"', true));
    $layout->addButton('submit', KERNEL_RENAME);
    $layout->addButton('button', KERNEL_CANCEL, '', '', 'onclick="sbCloseDialog()"');
    $layout->show();
}

function fFiles_Rename_File_Submit()
{
    if (!$_SESSION['sbPlugins']->isRightAvailable('pl_filelist', 'files_edit')) {
        return;
    }

    // переименование
    $old_path = str_replace('\\', '/', preg_replace('/[^\s0-9a-zA-Z\/\\\\_\-\.\[\]]+/', '', $_GET['path']));
    $old_path = preg_replace('/\.\.+/', '', $old_path);
    $old_path = preg_replace('/\/\/+/', '', $old_path);

    $file_name = preg_replace('/[^\s0-9a-zA-Z_\-\.\[\]]+/', '', $_POST['file_name']);
    $file_name = preg_replace('/\.\.+/', '', $file_name);
    $file_name = preg_replace('/\/\/+/', '', $file_name);

    $new_path = substr($old_path, 0, strrpos($old_path, '/'));

    if (!fFolders_Check_Rights($new_path)) {
        sb_show_message(SB_FILELIST_DENY_ERROR_MSG, true, 'warning');
        return;
    }

    $new_path .= '/' . $file_name;

    //проверяем на наличие одноименной директории
    $test_dir_name = preg_replace('/\..*$/', '', $new_path);
    if ($GLOBALS['sbVfs']->is_dir($test_dir_name)) {
        sb_show_message(sprintf(PL_FILES_UPLOAD_DIR_EXISTS, preg_replace('/\..*$/', '', $new_path)), false, 'warning');
        fFiles_Rename_File();
        return;
    }

    if ($GLOBALS['sbVfs']->exists($new_path)) {
        // файл уже существует
        sb_show_message(sprintf(PL_FILES_FILE_EXIST_ERROR, $new_path, $old_path), true, 'warning');
        // записываем действие в системный журнал
        sb_add_system_message(sprintf(PL_FILES_FILE_EXIST_ERROR, $new_path, $old_path), SB_MSG_WARNING);
    } else {
        if ($GLOBALS['sbVfs']->rename($old_path, $new_path)) {
            require_once(SB_CMS_LIB_PATH . '/sbFilesList.inc.php');

            $ext = pathinfo($new_path);
            $ext = isset($ext['extension']) ? $ext['extension'] : '';
            $icon = sbFilesList::getFileIcon($ext, $new_path);
            $icon = $GLOBALS['sbSql']->escape($icon, false, false);

            echo '<script>
                var res = new Object();
                res.title = "' . $file_name . '";
                res.id = "' . $new_path . '";
                res.icon = "' . $icon . '";
                sbReturnValue(res);
            </script>';
            // записываем действие в системный журнал
            sb_add_system_message(sprintf(PL_FILES_RENAME_FILE_OK, $old_path, $new_path));
        } else {
            // не удалось переименовать
            sb_show_message(sprintf(PL_FILES_RENAME_FILE_ERROR, $old_path, $new_path), true, 'warning');
            // записываем действие в системный журнал
            sb_add_system_message(sprintf(PL_FILES_RENAME_FILE_ERROR, $old_path, $new_path), SB_MSG_WARNING);
        }
    }
}

function fFiles_Delete_Files()
{
    if (!$_SESSION['sbPlugins']->isRightAvailable('pl_filelist', 'files_delete')) {
        return;
    }

    $path = str_replace('\\', '/', preg_replace('/[^\s0-9a-zA-Z\/\\\\_\-\.\[\]]+/', '', urldecode($_GET['path'])));
    // запрещаем выход на более высокий уровень (..)
    $path = preg_replace('/\.[\.]+\//', '', $path);
    $path = preg_replace('/\/\/+/', '', $path);

    if (!$GLOBALS['sbVfs']->is_dir($path)) {
        $check_path = dirname($path);
    } else {
        $check_path = $path;
    }

    if (!fFolders_Check_Rights($check_path)) {
        return;
    }

    if ($GLOBALS['sbVfs']->delete($path)) {
        echo 'TRUE';
        // записываем действие в системный журнал
        sb_add_system_message(sprintf(PL_FILES_DELETE_FILE_OK, $path));
    } else {
        // записываем действие в системный журнал
        sb_add_system_message(sprintf(PL_FILES_DELETE_FILE_ERROR, $path), SB_MSG_WARNING);
    }
}

function fFiles_Upload_File()
{
    $fld_name = 'sb_file_upload';
    $tmp_fld = 'tmp_file_name';
    $error = true;
    if (isset($_POST['tmp_file_name']) && trim($_POST['tmp_file_name']) != '') {
        //Фай пришел из нормального браузера
        if (!isset($_FILES['sb_file_upload']) || !@is_uploaded_file($_FILES['sb_file_upload']['tmp_name'])) {
            sb_add_system_message(PL_FILES_UPLOAD_FILES_NO_FORM_DATA, SB_MSG_WARNING);
            echo SB_LAYOUT_UPLOAD_ERROR;
            return;
        }
        $error = false;
    } else {
        //Файл пришел из ИЕ :Е
        $GLOBALS['sbVfs']->mLocal = true;
        foreach ($_POST as $key => $val) {
            if (sb_stripos($key, 'tmp_file_name') !== 0) {
                continue;
            }

            $tmp = sb_str_replace('tmp_file_name_', '', $key);
            if (!$GLOBALS['sbVfs']->is_file(SB_CMS_TMP_PATH . '/' . $val) && (!isset($_FILES['sb_file_upload_' . $tmp]) || !@is_uploaded_file($_FILES['sb_file_upload_' . $tmp]['tmp_name']))) {
                continue;
            } elseif (!$GLOBALS['sbVfs']->is_file(SB_CMS_TMP_PATH . '/' . $val)) {
                $fld_name .= '_' . $tmp;
                $tmp_fld .= '_' . $tmp;
                $error = false;
                break;
            }
        }
        $GLOBALS['sbVfs']->mLocal = false;
    }

    if ($error) {
        sb_add_system_message(PL_FILES_UPLOAD_FILES_NO_FORM_DATA, SB_MSG_WARNING);
        echo SB_LAYOUT_UPLOAD_ERROR;
        return;
    }

    if (!$_SESSION['sbPlugins']->isRightAvailable('pl_filelist', 'folders_upload')) {
        return;
    }


    $accept_types = array();
    if (!$_SESSION['sbAuth']->isAdmin()) {
        $accept_types = $_SESSION['sbAuth']->getUploadingExts();
        if (count($accept_types) == 0) {
            sb_add_system_message(PL_FILES_UPLOAD_FILES_NO_ACCEPT_TYPES_ERROR, SB_MSG_WARNING);
            echo SB_LAYOUT_UPLOAD_ERROR;
            return;
        }
    }

    // подключаем библиотеку работы с загрузкой файлов
    require_once(SB_CMS_LIB_PATH . '/sbUploader.inc.php');
    $uploader = new sbUploader();

    // сохраняем файл
    $success = $uploader->upload($fld_name, $accept_types);
    if (!$success) {
        sb_add_system_message(sprintf(PL_FILES_UPLOAD_FILES_ERROR, $_FILES[$fld_name]['name'], $uploader->getError()), SB_MSG_WARNING);
        echo SB_LAYOUT_UPLOAD_ERROR;
        return;
    }

    $GLOBALS['sbVfs']->mLocal = true;
    if (!$GLOBALS['sbVfs']->is_dir(SB_CMS_TMP_PATH)) {
        /**
         * Если темповая директория не существует, то создаем ее
         */
        if (!$GLOBALS['sbVfs']->mkdir(SB_CMS_TMP_PATH)) {
            sb_add_system_message(sprintf(PL_FILES_UPLOAD_FILES_TMP_ERROR, $_FILES[$fld_name]['name']), SB_MSG_WARNING);
            $GLOBALS['sbVfs']->mLocal = false;
            echo SB_LAYOUT_UPLOAD_ERROR;
            return;
        }
    } else {
        // удаляем ранее загруженные файлы
        $files = $GLOBALS['sbVfs']->scandir(SB_CMS_TMP_PATH);
        if ($files) {
            for ($i = 0; $i < count($files); $i++) {
                $filename = $files[$i];
                if (preg_match('/_upload.tmp$/',
                        $filename) && $GLOBALS['sbVfs']->is_file(SB_CMS_TMP_PATH . '/' . $filename) && $GLOBALS['sbVfs']->filectime(SB_CMS_TMP_PATH . '/' . $filename) < time() - 3600
                ) {
                    $GLOBALS['sbVfs']->delete(SB_CMS_TMP_PATH . '/' . $filename);
                }
            }
        }
    }

    $filename = str_replace('\\', '/', preg_replace('/[^\s0-9a-zA-Z\_\-\.\[\]]+/', '', $_POST[$tmp_fld]));
    $filename = preg_replace('/\.\.+/', '', $filename);
    $filename = trim(preg_replace('/\/\/+/', '', $filename));

    if (!$GLOBALS['sbVfs']->move_uploaded_file($_FILES[$fld_name]['tmp_name'], SB_CMS_TMP_PATH . '/' . $filename)) {
        sb_add_system_message(sprintf(PL_FILES_UPLOAD_FILES_ERROR, $_FILES[$fld_name]['name']), SB_MSG_WARNING);
        $GLOBALS['sbVfs']->mLocal = false;
        echo SB_LAYOUT_UPLOAD_ERROR;
        return;
    }

    $GLOBALS['sbVfs']->mLocal = false;
    echo 'TRUE';
}

function fFiles_Upload_Files()
{
    if (!$_SESSION['sbPlugins']->isRightAvailable('pl_filelist', 'folders_upload')) {
        return;
    }

    $path = str_replace('\\', '/', preg_replace('/[^\s0-9a-zA-Z\/\\\\_\-\.\[\]]+/', '', $_GET['path']));
    // запрещаем выход на более высокий уровень (..)
    $path = preg_replace('/\.\.+/', '', $path);
    $path = preg_replace('/\/\/+/', '', $path);

    if (!fFolders_Check_Rights($path)) {
        sb_show_message(SB_FILELIST_DENY_ERROR_MSG, true, 'warning');
        return;
    }

    echo '<script>
    function checkValues()
    {
        var el = sbGetE("file1");
        if (el.value == "")
        {
            alert("' . PL_FILES_UPLOAD_NO_FILES_ERROR . '");
            sbGetE("submit_btn").disabled = true;
            return false;
        }

        var img_chk = sbGetE("file1_img_chk");
        var img_width = sbGetE("spin_file1_img_width");
        var img_height = sbGetE("spin_file1_img_height");
        if (img_chk.checked && img_width.value == "" && img_height.value == "")
        {
            alert("' . PL_FILES_UPLOAD_NO_SIZES_ERROR . '");
            return false;
        }

        return true;
    }

    function uploadComplete()
    {
        sbGetE("submit_btn").disabled = false;
    }

    function checkFile(el)
    {
        var fname = el.value;
        var parts = fname.split(".");
        if (parts.length == 1)
            var fext = "";
        else
            var fext = new String(parts[parts.length-1]).toLowerCase();

        var chk_el = sbGetE("file1_zip_chk");
        if (chk_el)
        {
            if (fext == "zip")
            {
                chk_el.disabled = false;
            }
            else
            {
                chk_el.disabled = true;
                chk_el.checked = false;
            }
        }

        var img_chk = sbGetE("file1_img_chk");
        var w_chk = sbGetE("file1_watermark_chk");

        if (fext == "jpg" || fext == "jpeg" ||' . (function_exists('imagecreatefromgif') ? ' fext == "gif" ||' : '') . ' fext == "png")
        {
            img_chk.disabled = false;
            w_chk.disabled = false;
        }
        else
        {
            img_chk.disabled = true;
            w_chk.disabled = true;
        }

        changeResize(img_chk);
        changeWatermark(w_chk);
    }

    function changeZip(el)
    {
        var img_chk = sbGetE("file1_img_chk");
        var w_chk = sbGetE("file1_watermark_chk");

        if (el.checked)
        {
            img_chk.disabled = false;
            w_chk.disabled = false;
        }
        else
        {
            img_chk.disabled = true;
            w_chk.disabled = true;
        }

        changeResize(img_chk);
        changeWatermark(w_chk);
    }

    function changeResize(el)
    {
        sbGetE("spin_file1_img_width").disabled = !el.checked || el.disabled;
        sbGetE("spin_file1_img_height").disabled = !el.checked || el.disabled;
        sbGetE("spin_file1_img_quality").disabled = !el.checked || el.disabled;
    }

    function changeWatermark(el)
    {
        sbGetE("file1_watermark_position").disabled = !el.checked || el.disabled;
        sbGetE("spin_file1_watermark_opacity").disabled = !el.checked || el.disabled;
        sbGetE("spin_file1_watermark_margin").disabled = !el.checked || el.disabled;

        sbGetE("file1_watermark").disabled = !el.checked || el.disabled;

        sbGetE("file1_copyright").disabled = !el.checked || el.disabled;
        sbGetE("file1_copyright_color").disabled = !el.checked || el.disabled;
        sbGetE("file1_copyright_font").disabled = !el.checked || el.disabled;
        sbGetE("file1_copyright_size").disabled = !el.checked || el.disabled;
    }
    </script><br /><br />';

    $zip = false;
    if (extension_loaded('zip')) {
        $zip = true;
    }

    require_once(SB_CMS_LIB_PATH . '/sbLayout.inc.php');
    $settings = array();
    if (isset($_GET['fld_id']) && trim($_GET['fld_id']) != '' && isset($_SESSION[trim($_GET['fld_id'])])) {
        $settings = $_SESSION[trim($_GET['fld_id'])];
    }

    if (empty($settings)) {
        $settings = array(
            'resize' => sbPlugins::getSetting('sb_files_resize'),
            'img_width' => sbPlugins::getSetting('sb_files_max_resize_width'),
            'img_height' => sbPlugins::getSetting('sb_files_max_resize_height'),
            'img_quality' => sbPlugins::getSetting('sb_files_resize_quality'),
            'watermark' => sbPlugins::getSetting('sb_files_watermark'),
            'watermark_opacity' => sbPlugins::getSetting('sb_files_watermark_opacity'),
            'watermark_position' => sbPlugins::getSetting('sb_files_watermark_position'),
            'watermark_margin' => sbPlugins::getSetting('sb_files_watermark_margin'),
            'watermark_file' => sbPlugins::getSetting('sb_files_watermark_img'),
            'copyright' => sbPlugins::getSetting('sb_files_copyright'),
            'copyright_color' => sbPlugins::getSetting('sb_files_copyright_color'),
            'copyright_font' => sbPlugins::getSetting('sb_files_copyright_font'),
            'copyright_size' => sbPlugins::getSetting('sb_files_copyright_size')
        );
    }

    $layout = new sbLayout(SB_CMS_MODAL_DIALOG_FILE . '?event=pl_folders_upload_files_submit&path=' . urlencode($_GET['path']), 'thisDialog', 'post',
        'checkValues()', '', 'enctype="multipart/form-data"');

    $layout->mTableWidth = '95%';
    $layout->mTitleWidth = '230';

    $layout->addHeader(PL_FILES_UPLOAD_FILES_TITLE);

    $fld = new sbLayoutUpload('pl_folders_upload_file', 'file1', '', 'style="width:290px;" onChange="checkFile(this);"', true);
    $fld->mFileNameChange = 'checkFile';
    $fld->mFileUploadComplete = 'uploadComplete';
    $fld->mSuccessMessage = PL_FILES_UPLOAD_FILE_COMPLETE;
    $fld->mHTML = '<div class="hint_div">' . PL_FILES_UPLOAD_FILES_HINT . '</div>';
    $layout->addField(PL_FILES_UPLOAD_FILE_NAME, $fld);

    $layout->addField(PL_FILES_UPLOAD_FILE_REPLACE, new sbLayoutInput('checkbox', 1, 'file_replace'));

    if ($zip) {
        $layout->addField(PL_FILES_UPLOAD_ZIP_NAME, new sbLayoutInput('checkbox', '1', 'file1_zip_chk', '', 'disabled="disabled" onclick="changeZip(this)"'));
        $layout->addField('', new sbLayoutDelim());
    }
    $layout->addField(PL_FILES_UPLOAD_IMG_NAME, new sbLayoutInput('checkbox', '1', 'file1_img_chk', '',
        'disabled="disabled" onclick="changeResize(this)"' . ($settings['resize'] == 1 ? ' checked="checked"' : '')));
    $layout->addField(PL_FILES_UPLOAD_IMG_WIDTH, new sbLayoutInput('text', $settings['img_width'], 'spin_file1_img_width', '', 'disabled="disabled" size="5"'));
    $layout->addField(PL_FILES_UPLOAD_IMG_HEIGHT,
        new sbLayoutInput('text', $settings['img_height'], 'spin_file1_img_height', '', 'disabled="disabled" size="5"'));
    $layout->addField(PL_FILES_UPLOAD_IMG_QUALITY,
        new sbLayoutInput('text', $settings['img_quality'], 'spin_file1_img_quality', '', 'disabled="disabled" size="5"'));
    $layout->addField('', new sbLayoutDelim());

    $layout->addField(PL_FILES_UPLOAD_IMG_WATERMARK, new sbLayoutInput('checkbox', '1', 'file1_watermark_chk', '',
        'disabled="disabled" onclick="changeWatermark(this)"' . ($settings['watermark'] == 1 ? ' checked="checked"' : '')));

    $fld = new sbLayoutSelect($GLOBALS['sb_watermark_positions'], 'file1_watermark_position', '', 'disabled="disabled"');
    $fld->mSelOptions = array($settings['watermark_position']);
    $layout->addField(PL_FILES_UPLOAD_WATERMARK_POSITION, $fld);
    $layout->addField(PL_FILES_UPLOAD_WATERMARK_OPACITY,
        new sbLayoutInput('text', $settings['watermark_opacity'], 'spin_file1_watermark_opacity', '', 'disabled="disabled" size="5"'));
    $layout->addField(PL_FILES_UPLOAD_WATERMARK_MARGIN,
        new sbLayoutInput('text', $settings['watermark_margin'], 'spin_file1_watermark_margin', '', 'disabled="disabled" size="5"'));
    $layout->addField('', new sbLayoutDelim());

    $fld = new sbLayoutFile($settings['watermark_file'], 'file1_watermark', '', 'disabled="disabled" style="width:450px;"');
    $fld->mFileTypes = 'jpg gif png';
    $layout->addField(PL_FILES_UPLOAD_WATERMARK, $fld);
    $layout->addField('', new sbLayoutDelim());

    $layout->addField(PL_FILES_UPLOAD_COPYRIGHT,
        new sbLayoutInput('text', $settings['copyright'], 'file1_copyright', '', 'disabled="disabled" style="width:480px;"'));
    $fld = new sbLayoutColor($settings['copyright_color'], 'file1_copyright_color', '', 'disabled="disabled"');
    $fld->mDropButton = true;
    $layout->addField(PL_FILES_UPLOAD_COPYRIGHT_COLOR, $fld);

    $fld = new sbLayoutSelect($GLOBALS['sb_watermark_fonts'], 'file1_copyright_font', '', 'disabled="disabled"');
    $fld->mSelOptions = array($settings['copyright_font']);
    $layout->addField(PL_FILES_UPLOAD_COPYRIGHT_FONT, $fld);

    $fld = new sbLayoutSelect($GLOBALS['sb_watermark_sizes'], 'file1_copyright_size', '', 'disabled="disabled"');
    $fld->mSelOptions = array($settings['copyright_size']);
    $layout->addField(PL_FILES_UPLOAD_COPYRIGHT_SIZE, $fld);

    $layout->addButton('submit', KERNEL_SAVE, 'submit_btn', '', 'disabled="disabled"');
    $layout->addButton('button', KERNEL_CANCEL, '', '', 'onclick="sbCloseDialog()"');

    $layout->show();

    echo '<br>';
}

function fFiles_Upload_Files_Submit()
{
    if (!$_SESSION['sbPlugins']->isRightAvailable('pl_filelist', 'folders_upload')) {
        fFiles_Upload_Files();
        return;
    }

    if (!isset($_POST['file1']) || trim($_POST['file1']) == '' || !isset($_POST['file1_tmp']) || trim($_POST['file1_tmp']) == '') {
        sb_show_message(PL_FILES_UPLOAD_NO_FILES_ERROR, false, 'warning');

        fFiles_Upload_Files();
        return;
    }

    $filename = sb_strtolat($_POST['file1']);
    $filename = trim(preg_replace('/[^\s0-9a-zA-Z\_\-\.\[\]]+/', '', str_replace(array(' ', '%20'), array('_', '_'), $filename)));

    $filename_tmp = str_replace('\\', '/', preg_replace('/[^\s0-9a-zA-Z\_\-\.\[\]]+/', '', $_POST['file1_tmp']));
    $filename_tmp = preg_replace('/\.\.+/', '', $filename_tmp);
    $filename_tmp = trim(preg_replace('/\/\/+/', '', $filename_tmp));

    $GLOBALS['sbVfs']->mLocal = true;
    if ($filename_tmp == '' || !$GLOBALS['sbVfs']->exists(SB_CMS_TMP_PATH . '/' . $filename_tmp) || !$GLOBALS['sbVfs']->is_file(SB_CMS_TMP_PATH . '/' . $filename_tmp)) {
        $GLOBALS['sbVfs']->mLocal = false;
        sb_show_message(sprintf(PL_FILES_UPLOAD_FILES_ERROR, $_POST['file1'], ''), false, 'warning');
        sb_add_system_message(sprintf(PL_FILES_UPLOAD_FILES_ERROR, $_POST['file1'], ''), SB_MSG_WARNING);

        fFiles_Upload_Files();
        return;
    }

    $path = str_replace('\\', '/', preg_replace('/[^\s0-9a-zA-Z\/\\\\_\-\.\[\]]+/', '', $_GET['path']));
    // запрещаем выход на более высокий уровень (..)
    $path = preg_replace('/\.\.+/', '', $path);
    $path = preg_replace('/\/\/+/', '', $path);

    $GLOBALS['sbVfs']->mLocal = false;

    if (!fFolders_Check_Rights($path)) {
        $GLOBALS['sbVfs']->delete(SB_CMS_TMP_PATH . '/' . $filename_tmp);
        sb_show_message(SB_FILELIST_DENY_ERROR_MSG, false, 'warning');

        fFiles_Upload_Files();
        return;
    }

    $error = false;
    $zip = false;

    $accept_types = array();
    if (!$_SESSION['sbAuth']->isAdmin()) {
        $accept_types = $_SESSION['sbAuth']->getUploadingExts();
    }

    if (!isset($_POST['file1_zip_chk'])) {
        // если не надо разархивировать
        $file = str_replace('//', '/', $path . '/' . $filename);

        $GLOBALS['sbVfs']->mLocal = true;

        //проверяем на наличие одноименной директории
        $pathinfo = pathinfo($file);

        if ($pathinfo['filename'] != '') {
            $test_dir_name = SB_BASEDIR . '/' . $pathinfo['dirname'] . '/' . $pathinfo['filename'];
            $test_dir_name = sb_str_replace('//', '/', $test_dir_name);

            if ($GLOBALS['sbVfs']->is_dir($test_dir_name)) {
                sb_show_message(sprintf(PL_FILES_UPLOAD_DIR_EXISTS, $pathinfo['filename'], ''), false, 'warning');
                fFiles_Upload_Files();
                return;
            }
        }

        if (!$GLOBALS['sbVfs']->copy(SB_CMS_TMP_PATH . '/' . $filename_tmp, SB_BASEDIR . '/' . $file,
            isset($_POST['file_replace']) && $_POST['file_replace'] == 1)
        ) {
            $GLOBALS['sbVfs']->delete(SB_CMS_TMP_PATH . '/' . $filename_tmp);
            $GLOBALS['sbVfs']->mLocal = false;

            sb_show_message(sprintf(PL_FILES_UPLOAD_FILES_ERROR, $_POST['file1'], ''), false, 'warning');
            sb_add_system_message(sprintf(PL_FILES_UPLOAD_FILES_ERROR, $_POST['file1'], ''), SB_MSG_WARNING);

            fFiles_Upload_Files();
            return;
        }

        $GLOBALS['sbVfs']->mLocal = false;

        sb_add_system_message(sprintf(PL_FILES_UPLOAD_FILES_OK, $file));

        $ext = pathinfo($filename);
        $ext = isset($ext['extension']) ? strtolower($ext['extension']) : '';
        if ($ext == 'gif' || $ext == 'png' || $ext == 'jpg' || $ext == 'jpeg') {
            if (isset($_POST['file1_img_chk'])) {
                // пытаемся сжать изображение
                if (!sb_resize_image($file, $file, $_POST['spin_file1_img_width'], $_POST['spin_file1_img_height'], $_POST['spin_file1_img_quality'])) {
                    sb_add_system_message(sprintf(PL_FILES_UPLOAD_FILES_RESIZE_ERROR, $file), SB_MSG_WARNING);
                    $error = true;
                }
            }

            if (isset($_POST['file1_watermark_chk'])) {
                if (!sb_watermark_image($file, $file, $_POST['file1_watermark_position'], $_POST['spin_file1_watermark_opacity'],
                    $_POST['spin_file1_watermark_margin'], $_POST['file1_watermark'], $_POST['file1_copyright'], $_POST['file1_copyright_color'],
                    $_POST['file1_copyright_font'], $_POST['file1_copyright_size'])
                ) {
                    sb_add_system_message(sprintf(PL_FILES_UPLOAD_FILES_WATERMARK_ERROR, $file), SB_MSG_WARNING);
                    $error = true;
                }
            }
        }
    } else {
        // разархивируем
        $unziped_files = sb_unzip(SB_CMS_TMP_PATH . '/' . $filename_tmp, $path, $accept_types, isset($_POST['file_replace']) && $_POST['file_replace'] == 1);

        if ($unziped_files === false) {
            $GLOBALS['sbVfs']->mLocal = true;
            $GLOBALS['sbVfs']->delete(SB_CMS_TMP_PATH . '/' . $filename_tmp);
            $GLOBALS['sbVfs']->mLocal = false;

            sb_show_message(sprintf(PL_FILES_UPLOAD_FILES_UNZIP_ERROR, $_POST['file1']), false, 'warning');
            sb_add_system_message(sprintf(PL_FILES_UPLOAD_FILES_UNZIP_ERROR, $_POST['file1']), SB_MSG_WARNING);

            fFiles_Upload_Files();
            return;
        }

        if (!$_SESSION['sbAuth']->isAdmin() && is_array($unziped_files) && !empty($unziped_files)) {
            $uid = 'u' . $_SESSION['sbAuth']->getUserId();
            $unziped_folders = array();
            foreach ($unziped_files as $file) {
                $tmp = explode('/', trim(pathinfo($file, PATHINFO_DIRNAME), '/'));
                if (is_array($tmp) && !in_array($tmp[0], $unziped_folders)) {
                    $unziped_folders[] = $tmp[0];
                }
            }
            foreach ($unziped_folders as $dir) {
                fFolders_Set_Rights($path.'/'.$dir, $uid, 1, 1);
            }
        }

        $zip = true;
        sb_add_system_message(sprintf(PL_FILES_UPLOAD_FILES_UNZIP_OK, $_POST['file1'], $path));

        if (isset($_POST['file1_img_chk']) || isset($_POST['file1_watermark_chk'])) {
            foreach ($unziped_files as $value) {

                // пытаемся сжать изображения в архиве
                $ext = pathinfo($value);
                $ext = isset($ext['extension']) ? strtolower($ext['extension']) : '';
                if ($ext != 'gif' && $ext != 'png' && $ext != 'jpg' && $ext != 'jpeg') {
                    continue;
                }

                $value = str_replace('//', '/', $path . '/' . $value);
                if (isset($_POST['file1_img_chk'])) {
                    if (!sb_resize_image($value, $value, $_POST['spin_file1_img_width'], $_POST['spin_file1_img_height'], $_POST['spin_file1_img_quality'])) {
                        sb_add_system_message(sprintf(PL_FILES_UPLOAD_FILES_RESIZE_ERROR, $value), SB_MSG_WARNING);
                        $error = true;
                    }
                }

                if (isset($_POST['file1_watermark_chk'])) {
                    if (!sb_watermark_image($value, $value, $_POST['file1_watermark_position'], $_POST['spin_file1_watermark_opacity'],
                        $_POST['spin_file1_watermark_margin'], $_POST['file1_watermark'], $_POST['file1_copyright'], $_POST['file1_copyright_color'],
                        $_POST['file1_copyright_font'], $_POST['file1_copyright_size'])
                    ) {
                        sb_add_system_message(sprintf(PL_FILES_UPLOAD_FILES_WATERMARK_ERROR, $value), SB_MSG_WARNING);
                        $error = true;
                    }
                }
            }
        }
    }

    $GLOBALS['sbVfs']->mLocal = true;
    $GLOBALS['sbVfs']->delete(SB_CMS_TMP_PATH . '/' . $filename_tmp);
    $GLOBALS['sbVfs']->mLocal = false;

    echo '<script>
             var res = new Object();
             res.error = ' . ($error ? 'true' : 'false') . ';
             res.refresh_tree = ' . ($zip ? 'true' : 'false') . ';
             sbReturnValue(res);
          </script>';
}

function fFolders_Set_Rights($path = '', $rights = '', $subcategs = false, $skipCheckRights = false)
{
    if (!$_SESSION['sbPlugins']->isRightAvailable('pl_filelist', 'folders_rights'))
        return;

    $path = (isset($_GET['path']) && $path == '') ? $_GET['path'] : $path;
    $rights = (isset($_GET['rights']) && $rights == '') ? $_GET['rights'] : $rights;
    $subcategs = (isset($_GET['subcategs']) && $subcategs == false) ? $_GET['subcategs'] : $subcategs;
    $path = str_replace('\\', '/', preg_replace('/[^\s0-9a-zA-Z\/\\\\_\-\.\[\]]+/', '', $path));
    // запрещаем выход на более высокий уровень (..)
    $path = preg_replace('/\.\.+/', '', $path);
    $path = preg_replace('/\/\/+/', '', $path);
    if (!$skipCheckRights && !fFolders_Check_Rights($path)) {
        return;
    }
    $rights = trim(preg_replace('/[^0-9gu\^]+/', '', $rights));
    if ($subcategs) {
        sql_query('DELETE FROM sb_folders_rights WHERE f_path LIKE ? AND f_domain=?', $path.'%', SB_COOKIE_DOMAIN);
    } else {
        sql_query('DELETE FROM sb_folders_rights WHERE f_path=? AND f_domain=?', $path, SB_COOKIE_DOMAIN);
    }
    if ($rights == '') {
        sb_add_system_message(sprintf(PL_FILES_SET_RIGHTS_OK, $path));
        return;
    }
    if (sql_query('INSERT INTO sb_folders_rights (f_path, f_ids, f_domain) VALUES (?, ?, ?)', $path, $rights, SB_COOKIE_DOMAIN)) {
        sb_add_system_message(sprintf(PL_FILES_SET_RIGHTS_OK, $path));
    } else {
        sb_add_system_message(sprintf(PL_FILES_SET_RIGHTS_ERROR, $path), SB_MSG_WARNING);
    }
    if ($subcategs) {
        fFolders_Set_Rights_Sub($path, $rights);
    }
}

function fFolders_Set_Rights_Sub($path, $rights)
{
    $dir = $GLOBALS['sbVfs']->scandir($path);

    foreach ($dir as $value) {
        if ($value == 'cms') {
            continue;
        }

        $new_path = $path . ($path != '/' ? '/' : '') . $value;

        if ($value != '.' && $value != '..' && ($GLOBALS['sbVfs']->is_dir($new_path) || $GLOBALS['sbVfs']->is_link($new_path))) {
            if (sql_query('INSERT INTO sb_folders_rights (f_path, f_ids, f_domain) VALUES (?, ?, ?)', $new_path, $rights, SB_COOKIE_DOMAIN)) {
                sb_add_system_message(sprintf(PL_FILES_SET_RIGHTS_OK, $new_path));
            } else {
                sb_add_system_message(sprintf(PL_FILES_SET_RIGHTS_ERROR, $new_path), SB_MSG_WARNING);
            }

            fFolders_Set_Rights_Sub($new_path, $rights);
        }
    }
}

function fFolders_Get_Folder()
{
    require_once(SB_CMS_LIB_PATH . '/sbFolders.inc.php');

    if (isset($_GET['path']) && $_GET['path'] != '') {
        $path = preg_replace('/[^\s0-9a-zA-Z\/\\\\_\-\.\[\]]+/', '', $_GET['path']);
    } else {
        $path = '/';
    }

    echo '<script>
    function chooseCat()
    {
        var path = sbFoldersTree.getSelectedItemId();

        var res = new Object();
        res.path = path;

        sbReturnValue(res);
    }
    function onResize()
    {
        var list = sbGetE("sb_folders_list");
        var footer = sbGetE("footer");

        if (_isIE)
        {
            list.style.height = Math.max(parseInt(document.body.clientHeight) - 25 - (footer?footer.offsetHeight:0), 0);
            if (sbFoldersTree) sbFoldersTree.allTree.style.height = list.style.height;
        }
        else
        {
            list.style.height = Math.max(parseInt(document.body.clientHeight) - 25 - (footer?footer.offsetHeight:0), 0);
            if (sbFoldersTree) sbFoldersTree.allTree.style.height = list.offsetHeight - 10;
        }
    }
    sbResizeFuncs[sbResizeFuncs.length]=onResize;
    sbAddEvent(window, "load", onResize);
    </script>
    <div id="sb_folders_list" style="text-align: left;"></div>
    <table cellspacing="0" cellpadding="7" width="100%" class="form" id="footer">
    <tr><td class="footer" style="padding-top: 2px;">
        <div class="footer" style="margin: 0;">
        <button onclick="chooseCat();">' . KERNEL_CHOOSE . '</button>&nbsp;&nbsp;&nbsp;
        <button onclick="sbCloseDialog();">' . KERNEL_CANCEL . '</button>
        </div>
    </td></tr></table>';

    $folders = new sbFolders();

    $folders->mFoldersUploadMenu = false;
    $folders->mFoldersRightsMenu = false;
    $folders->mFoldersSelectedPath = $path;

    $folders->init();
}
