<?php
/**
 *Вывод меню
 * 
 * @param $el_id Integer По умолчанию -1;
 * @param $temp_id Int идентификатор макета дизайна меню
 * @param $params Serialized string Сериализованный массив параметров
 * @param $tag_id Int Идентификатор тега на странице
 *  
 * Массив параметров:
 * <code>
 * $params = array(
 *   'ids'              => String,  // идентификаторы разделов меню, разделенные символом ^
 *   'temp_id'          => Int,     // идентификатор макета дизайна меню
 *   'from'             => Int,     // Выводить пункты, начиная от 
 *   'to'               => Int,     // Выводить пункты, до
 *   'count'            => Int,     // Кол-во выводимых уровней
 *   'parent_selection' => Int,     // Установить связь с выводом пунктов верхнего уровня 0/1
 *   'parent_link'      => Int,     // Сохранять выделение у пунктов верхнего уровня 0/1
 *   'show_closed'      => Int,     // выводить закрытые пункты 0/1 
 *   'show_hidden'      => Int,     // Выводить скрытые пункты  0/1
 *   'menu_design'      => Int,     // Использовать макет дизайна меню для вывода разделов 0/1
 *   'query_string'     => Int,     // Сохранять GET-параметры для ссылок 0/1
 * );
 * </code>
 *
 * @return String
 */
function fMenu_Elem_Tree($el_id, $temp_id, $params, $tag_id)
{
    // проверяем кэш
    if ($GLOBALS['sbCache']->check('pl_menu', $tag_id, array($el_id, $temp_id, $params)))
        return;

    // вытаскиваем макет дизайна вывода меню
    $res = sbQueryCache::getTemplate('sb_menu_temps', $temp_id);
	
    if (!$res)
    {
        // макета дизайна нет
        sb_add_system_message(sprintf(KERNEL_PROG_NO_TEMPLATE, $_SERVER['PHP_SELF'], KERNEL_PROG_MENU_PLUGIN), SB_MSG_WARNING);
        $GLOBALS['sbCache']->save('pl_menu', '');
        return;
    }

    list($mt_lang, $mt_levels, $mt_fields_temps, $mt_checked) = $res[0];

    if ($mt_levels != '')
    {
    	$mt_levels = unserialize($mt_levels);
    }

    if (!$mt_levels)
    {
        $GLOBALS['sbCache']->save('pl_menu', '');
        return;
    }

    if ($mt_fields_temps != '')
    {
        $mt_fields_temps = unserialize($mt_fields_temps);
    }
    else
    {
        $mt_fields_temps = array();
    }

    $params = unserialize(stripslashes($params));
    if (!isset($params['show_closed']))
    	$params['show_closed'] = 0;

    if (!isset($params['from']))
    	$params['from'] = 1;

    if (!isset($params['to']))
    	$params['to'] = '';

    if (!isset($params['count']))
    	$params['count'] = 0;

    if (!isset($params['query_string']))
    	$params['query_string'] = 1;

    $cat_ids = explode('^', $params['ids']);

    $page_like_str = '';

    $sel_left = -1;
    $sel_right = -1;
    $sel_cat_id = -1;
    $sel_level = -1;
    $sel_type = -1;

    // проверяем, выбрана страница или раздел
    $get_idents = array();
    foreach (array_keys($_GET) as $key)
    {
    	if (sb_substr($key, -4) == '_cid')
    	{
    		$get_idents[] = sb_substr($key, 0, -4);
    	}
    	elseif (sb_substr($key, -5) == '_scid')
    	{
    		$get_idents[] = sb_substr($key, 0, -5);
    	}
    }

    // определяем дефолтную страницу
    $index_page = sbPlugins::getSetting('sb_directory_index');
    if (trim($index_page) == '')
    {
        $index_page = 'index.php';
    }

    $filename = basename($_SERVER['PHP_SELF']);
    $dirname = trim($GLOBALS['sbSql']->escape(rtrim(str_replace('\\', '/', dirname($_SERVER['PHP_SELF'])), '/')), "'");

    // строим строку SQL-запроса для определения выбранного пункта меню
    $php_self = trim($GLOBALS['sbSql']->escape($_SERVER['PHP_SELF']), "'");

    $get_like_str = '(';
    $get_like_str_idents = '(';

    if (count($get_idents) > 0)
    {
    	// есть идентификаторы разделов, вытаскиваем пункты меню, связанные с этими идентификаторами
    	foreach ($get_idents as $get_ident)
    	{
    		if ($filename != $index_page)
    		{
	    		$get_like_str .= '(m.m_props LIKE "%s:9:\"get_ident\";s:'.sb_strlen($get_ident).':\"'.$get_ident.'\";%" AND
	    			(m_url = '.$GLOBALS['sbSql']->escape($_SERVER['PHP_SELF']).' OR
		             m_url = '.$GLOBALS['sbSql']->escape(trim($_SERVER['PHP_SELF'], '/')).' OR
		             m_url = \'http://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
		             m_url = \'http://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
		             m_url = \'https://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
		             m_url = \'https://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\'
					)) OR ';
    		}
    		else
    		{
    			$get_like_str .= '(m.m_props LIKE "%s:9:\"get_ident\";s:'.sb_strlen($get_ident).':\"'.$get_ident.'\";%" AND
	    			(m_url = '.$GLOBALS['sbSql']->escape($_SERVER['PHP_SELF']).' OR
	                 m_url = '.$GLOBALS['sbSql']->escape(trim($_SERVER['PHP_SELF'], '/')).' OR
	                 m_url = \'http://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
	                 m_url = \'http://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
                     m_url = \'http://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$dirname.'/\' OR
                     m_url = \'http://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$dirname.'/\' OR
                     m_url = \'https://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
                     m_url = \'https://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
                     m_url = \'https://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$dirname.'/\' OR
                     m_url = \'https://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$dirname.'/\' OR
                     m_url = \''.$dirname.'/\'
                    )) OR ';
    		}

    		$get_like_str_idents .= '(m.m_props LIKE "%s:9:\"get_ident\";s:'.sb_strlen($get_ident).':\"'.$get_ident.'\";%") OR ';
    	}
	}

	$demo_statuses = ' AND m.m_status IN ('.sb_get_workflow_demo_statuses().') ';
	$show_str = '';
	if (!isset($params['show_hidden']) || $params['show_hidden'] == 0)
	{
		$show_str = ' AND m.m_show = 1 ';
	}

	$get_like_str .= 'm.m_props LIKE "%s:9:\"get_ident\";s:8:\"pl_pages\";%")
							'.$demo_statuses.'
							AND c.cat_id IN (?a) AND m.m_cat_id=c.cat_id';

	$get_like_str_idents .= 'm.m_props LIKE "%s:9:\"get_ident\";s:8:\"pl_pages\";%")
							'.$demo_statuses.'
							AND c.cat_id IN (?a) AND m.m_cat_id=c.cat_id';

	$res = sql_param_query('SELECT m.m_left, m.m_right, m.m_cat_id, m.m_level, m.m_props FROM sb_menu m, sb_categs c WHERE '.$get_like_str.' ORDER BY c.cat_left, m.m_left', $cat_ids);

	if (!$res)
	{
		$res = sql_param_query('SELECT m.m_left, m.m_right, m.m_cat_id, m.m_level, m.m_props FROM sb_menu m, sb_categs c WHERE '.$get_like_str_idents.' ORDER BY c.cat_left, m.m_left', $cat_ids);
	}

	if ($res)
	{
        require_once SB_CMS_PL_PATH.'/pl_categs/prog/pl_categs.php';

        foreach ($res as $value)
        {
			list($m_left, $m_right, $m_cat_id, $m_level, $m_props) = $value;

        	if ($m_props == '')
				continue;

			$m_props = unserialize($m_props);

        	$cat_params = array();
            $cat_params['ids'] = $m_props['cat_id'];
            $cat_params['from'] = $m_props['from'];
            $cat_params['to'] = $m_props['to'];
            $cat_params['show_closed'] = isset($params['show_closed']) ? $params['show_closed'] : 0;
            $cat_params['show_hidden'] = isset($m_props['show_hidden']) ? $m_props['show_hidden'] : 0;

			$cat_count = fCategs_Count_Menu_Categs($m_props['temp_id'], $cat_params, $m_props['plugin_ident'], $m_props['ident'], $m_props['get_ident']);
			if ($cat_count)
            {
				// есть разделы
				$page_like_str .= 'm_left <= '.$m_left.' AND m_right >= '.$m_right.' AND m_cat_id = '.$m_cat_id.' OR ';

            	if ($sel_left == -1)
            	{
		            $sel_left = $m_left;
				    $sel_right = $m_right;
				    $sel_cat_id = $m_cat_id;
				    $sel_level = $m_level;
				    $sel_type = 'categs';
            	}
            }
        }

        if ($page_like_str != '')
        {
			$page_like_str = '('.sb_substr($page_like_str, 0, -4).')';
        }
    }

    if ($page_like_str == '')
    {
	    if ($filename != $index_page)
	    {
	    	if (isset($GLOBALS['PHP_SELF']) && $GLOBALS['PHP_SELF'] != $_SERVER['PHP_SELF'])
	    	{
	    		$chpu_self = trim($GLOBALS['sbSql']->escape($GLOBALS['PHP_SELF']), "'");
	    		$page_like_str = '(m_url = '.$GLOBALS['sbSql']->escape($_SERVER['PHP_SELF']).' OR
	                      m_url = '.$GLOBALS['sbSql']->escape(trim($_SERVER['PHP_SELF'], '/')).' OR
	                      m_url = \'http://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
	                      m_url = \'http://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
	                      m_url = \'https://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
	                      m_url = \'https://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
	                      m_url = '.$GLOBALS['sbSql']->escape($GLOBALS['PHP_SELF']).' OR
	                      m_url = '.$GLOBALS['sbSql']->escape(trim($GLOBALS['PHP_SELF'], '/')).' OR
	                      m_url = \'http://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$chpu_self.'\' OR
	                      m_url = \'http://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$chpu_self.'\' OR
	                      m_url = \'https://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$chpu_self.'\' OR
	                      m_url = \'https://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$chpu_self.'\'
						';

	    		if (isset($_SERVER['QUERY_STRING']) && trim($_SERVER['QUERY_STRING']) != '')
	    		{
	    			$page_like_str .= ' OR m_url = '.$GLOBALS['sbSql']->escape($_SERVER['PHP_SELF'].'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape(trim($_SERVER['PHP_SELF'], '/').'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape('http://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape('http://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape('https://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape('https://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape($GLOBALS['PHP_SELF'].'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape(trim($GLOBALS['PHP_SELF'], '/').'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape('http://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$chpu_self.'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape('http://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$chpu_self.'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape('https://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$chpu_self.'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape('https://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$chpu_self.'?'.$_SERVER['QUERY_STRING']).'
						';
	    		}

	    		$page_like_str .= ')';
	    	}
	    	else
	    	{
				$page_like_str = '(m_url = '.$GLOBALS['sbSql']->escape($_SERVER['PHP_SELF']).' OR
	                      m_url = '.$GLOBALS['sbSql']->escape(trim($_SERVER['PHP_SELF'], '/')).' OR
	                      m_url = \'http://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
	                      m_url = \'http://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
	                      m_url = \'https://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
	                      m_url = \'https://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\'
						';

				if (isset($_SERVER['QUERY_STRING']) && trim($_SERVER['QUERY_STRING']) != '')
	    		{
	    			$page_like_str .= ' OR m_url = '.$GLOBALS['sbSql']->escape($_SERVER['PHP_SELF'].'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape(trim($_SERVER['PHP_SELF'], '/').'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape('http://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape('http://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape('https://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape('https://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'?'.$_SERVER['QUERY_STRING']).'
	                      ';
	    		}

	    		$page_like_str .= ')';
	    	}
	    }
		else
	    {
			$page_like_str = '(m_url = '.$GLOBALS['sbSql']->escape($_SERVER['PHP_SELF']).' OR
	                      m_url = '.$GLOBALS['sbSql']->escape(trim($_SERVER['PHP_SELF'], '/')).' OR
	                      m_url = \'http://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
	                      m_url = \'http://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
	                      m_url = \'http://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$dirname.'/\' OR
	                      m_url = \'http://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$dirname.'/\' OR
	                      m_url = \'https://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
	                      m_url = \'https://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
	                      m_url = \'https://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$dirname.'/\' OR
	                      m_url = \'https://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$dirname.'/\' OR
	                      m_url = \''.$dirname.'/\'
	                      ';

			if (isset($_SERVER['QUERY_STRING']) && trim($_SERVER['QUERY_STRING']) != '')
	    	{
	    		$page_like_str .= ' OR m_url = '.$GLOBALS['sbSql']->escape($_SERVER['PHP_SELF'].'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape(trim($_SERVER['PHP_SELF'], '/').'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape('http://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape('http://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape('https://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape('https://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape('http://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$dirname.'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape('http://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$dirname.'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape('https://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$dirname.'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape('https://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$dirname.'?'.$_SERVER['QUERY_STRING']).'
	                      OR m_url = '.$GLOBALS['sbSql']->escape($dirname.'/?'.$_SERVER['QUERY_STRING']).'
						';
	    	}

	    	$page_like_str .= ')';
	    }
    }

    // вытаскиваем пользовательские поля меню
    $res = sbQueryCache::query('SELECT pd_fields FROM sb_plugins_data WHERE pd_plugin_ident="pl_menu"');

    $elems_fields = array();
    $elems_fields_select_sql = '';

    $tags = array();

    $where_str = '';
    if ($mt_checked != '')
    {
        $mt_checked = explode(' ', $mt_checked);
    }
    else
    {
        $mt_checked = array();
    }

    // формируем SQL-запрос для пользовательских полей
    if ($res && $res[0][0] != '')
    {
        $elems_fields = unserialize($res[0][0]);
        if ($elems_fields)
        {
            $checked_sql = '';

            foreach ($elems_fields as $value)
            {
                if (isset($value['sql']) && $value['sql'] == 1)
                {
                    $elems_fields_select_sql .= ', user_f_'.$value['id'];

                    if ($value['type'] == 'google_coords' || $value['type'] == 'yandex_coords')
	                {
	                	$tags[] = '{'.$value['tag'].'_LATITUDE}';
	                	$tags[] = '{'.$value['tag'].'_LONGTITUDE}';

	                	if ($value['type'] == 'yandex_coords')
	                	{
	                		$tags[] = '{'.$value['tag'].'_API_KEY}';
	                	}
	                }
	                else
	                {
	                	$tags[] = '{'.$value['tag'].'}';
	                }
                }

                foreach ($mt_checked as $val)
                {
                    if ($value['id'] == $val)
                    {
                        $checked_sql .= 'user_f_'.$value['id'].' = 1 AND ';
                    }
                }
            }

            if ($checked_sql != '')
            {
                $where_str .= '(m_type != "item" OR ('.substr($checked_sql, 0, -5).')) AND ';
            }
        }
    }

    if ($sel_left == -1)
    {
	    $res = sql_param_query('SELECT m_left, m_right, m_level, m_cat_id
	                           FROM sb_menu, sb_categs WHERE '.$where_str . $page_like_str.' AND cat_id IN (?a)
	                           AND m_cat_id=cat_id AND m_level > 0 AND m_type="item"
	                           ORDER BY cat_left, m_left', $cat_ids);
	    if ($res)
	    {
			$page_like_str = '';
			foreach ($res as $value)
	        {
	            list($m_left, $m_right, $m_level, $m_cat_id) = $value;

				$page_like_str .= 'm_left <= '.$m_left.' AND m_right >= '.$m_right.' AND m_cat_id = '.$m_cat_id.' OR ';

	            if ($sel_left == -1)
	            {
	                $sel_left = $m_left;
				    $sel_right = $m_right;
				    $sel_cat_id = $m_cat_id;
				    $sel_level = $m_level;
					$sel_type = 'item';
	            }
			}

			$page_like_str = '('.sb_substr($page_like_str, 0, -4).')';
	    }
    }

    $like_str = $page_like_str;
    if ($params['parent_selection'])
    {
        // если надо сохранять выделение у пунктов верхнего уровня, то формируем новую строку SQL-запроса для
        // определения выбранных пунктов меню
        $res = sql_param_query('SELECT m_left, m_right, m_cat_id FROM sb_menu, sb_categs WHERE '.$where_str.' cat_id IN (?a)
                           AND m_cat_id=cat_id AND m_level > 0 AND '.$like_str, $cat_ids);
        $like_str = '';
        if ($res)
        {
			foreach ($res as $value)
            {
				list($left, $right, $cat_id) = $value;

				$like_str .= 'm_left <= '.$left.' AND m_right >= '.$right.' AND m_cat_id = '.$cat_id.' OR ';
            }

			$like_str = '('.sb_substr($like_str, 0, -4).')';
        }
    }

    if ($like_str == '')
        $like_str = 'NULL';

    $max_level = 1;
    foreach ($mt_levels as $value)
    {
        if (sb_strpos($value['sub'], '{SUB_ITEMS}') !== false || sb_strpos($value['sub_sel'], '{SUB_ITEMS}') !== false)
        {
            $max_level++;
        }
        else
        {
            break;
        }
    }

    if ($max_level == count($mt_levels))
        $max_level = 1000000;

    // определяем с какого по какой уровень необходимо выводить пункты меню
    $from = max(1, intval($params['from']));
    $count = intval($params['count']);
    if ($count > 0 && isset($params['parent_link']) && $params['parent_link'] == 1)
    {
   		$from = min(max($sel_level, $from - 1), $params['to'] != '' ? intval($params['to']) : 1000000);
    	$to = min($max_level + $from, $from + $count, $params['to'] != '' ? intval($params['to']) : 1000000);
    }
    else
    {
		$to = min($max_level + $from, $params['to'] != '' ? intval($params['to']) : 1000000);
	}

    // выводим подменю или меню
    if (isset($params['parent_link']) && $params['parent_link'] == 1)
    {
        // вытаскиваем выделенные пункты меню
        if ($sel_left == -1)
        {
            // если ничего не выбрано, то подменю нет
            $GLOBALS['sbCache']->save('pl_menu', '');
            return;
        }

        if ($count > 0)
        {
			$res = sql_query('SELECT m_left, m_right FROM sb_menu WHERE '.$where_str.' m_cat_id='.$sel_cat_id.'
							AND m_left < '.$sel_left.' AND m_right > '.$sel_right.' ORDER BY m_left DESC LIMIT 1');

			if ($res)
        	{
				list($first_left, $first_right) = $res[0];
        	}
        	else
        	{
				$first_left = $sel_left;
				$first_right = $sel_right;
        	}

			if(!SB_DEMO_SITE)
			{
				$hide_str = '';
				if(!isset($params['show_hidden']) || $params['show_hidden'] == 0)
				{
					$hide_str = ' AND (0 = (SELECT COUNT(*) FROM sb_menu m2 WHERE m2.m_level != 0 AND m2.m_left <= m.m_left AND m2.m_right >= m.m_right AND m2.m_cat_id='.$sel_cat_id.' AND (m2.m_show != 1 OR m2.m_status != 1)))';
				}

				// считаем кол-во пунктов в выбраной ветке
				$res = sql_query('SELECT m.m_level, m.m_left, m.m_right FROM sb_menu m WHERE '.sb_str_replace('m_', 'm.m_', $where_str).' m.m_cat_id='.$sel_cat_id.'
						AND m.m_left >= '.$first_left.' AND m.m_right <= '.$first_right.$hide_str.' GROUP BY (m.m_level)');

				// считаем кол-во уровней у выбранного подпункта
				$res_sub = sql_query('SELECT m.m_level FROM sb_menu m WHERE '.sb_str_replace('m_', 'm.m_', $where_str).' m.m_cat_id='.$sel_cat_id.'
						AND m.m_left > '.$sel_left.' AND m.m_right < '.$sel_right.$hide_str.' GROUP BY (m_level)');
			}
			else
			{
				$hide_str = '';
				if(!isset($params['show_hidden']) || $params['show_hidden'] == 0)
				{
					$hide_str = ' AND (0 = (SELECT COUNT(*) FROM sb_menu m2 WHERE m2.m_level != 0 AND m2.m_left <= m.m_left AND m2.m_right >= m.m_right AND m2.m_cat_id='.$sel_cat_id.' AND (m2.m_show != 1 OR m2.m_status IN ('.sb_get_workflow_demo_statuses(true, false).'))))';
				}

				// считаем кол-во пунктов в выбраной ветке
				$res = sql_query('SELECT m.m_level FROM sb_menu m WHERE '.sb_str_replace('m_', 'm.m_', $where_str).' m.m_cat_id='.$sel_cat_id.'
						AND m.m_left >= '.$first_left.' AND m.m_right <= '.$first_right.$hide_str.' GROUP BY (m.m_level)');

				// считаем кол-во уровней у выбранного подпункта
				$res_sub = sql_query('SELECT m.m_level FROM sb_menu m WHERE '.sb_str_replace('m_', 'm.m_', $where_str).' m.m_cat_id='.$sel_cat_id.'
						AND m.m_left > '.$sel_left.' AND m.m_right < '.$sel_right.$hide_str.' GROUP BY (m_level)');
			}

			$max_root_level = $sel_level;
			if ($res)
        	{
				foreach ($res as $value)
	        	{
					if ($params['to'] != '' && $value[0] > intval($params['to']))
						break;

					$max_root_level = $value[0];
	        	}
        	}

			$max_sub_level = $sel_level;
        	if ($res_sub)
        	{
				foreach ($res_sub as $value)
				{
					if ($params['to'] != '' && $value[0] > intval($params['to']))
						break;

					$max_sub_level = $value[0];
	        	}
        	}

        	$max_root_level = max(0, $max_root_level - $sel_level);
        	$max_sub_level = max(0, $max_sub_level - $sel_level);
        	$sub_count = min($to - $from, $max_root_level);
        	if ($max_sub_level < $count)
        	{
				if ($max_root_level >= $count)
        		{
        			$from = max(1, intval($params['from']) - 1, $from - 1);
        		}
        		else
        		{
        			$from = max(1, intval($params['from']) - 1, $from - ($count - $sub_count));
        		}

        		$to = min($max_level + $from + 1, $from + $count, $params['to'] != '' ? intval($params['to']) : 1000000);
        	}

        	$res = sql_query('SELECT m_left, m_right, m_type FROM sb_menu WHERE '.$where_str.' m_cat_id='.$sel_cat_id.'
								AND m_left <= '.$sel_left.' AND m_right >= '.$sel_right.' AND m_level='.$from);

			if ($res)
            {
                list($sel_left, $sel_right, $sel_type) = $res[0];
            }

            if ($sel_type != 'categs')
            {
            	$from++;
            }
        }
        else
        {
			$from = max($from, $sel_level + ($sel_type == 'categs' ? 0 : 1));
        }

        // вытаскиваем пункты меню
		$res = sql_query('SELECT m_id, m_text, m_url, m_alt, m_target, m_type, m_props, m_level, m_show, m_status, '.$like_str.' AS m_sel'.
						   $elems_fields_select_sql.'
						   FROM sb_menu WHERE '.$where_str.' m_cat_id='.$sel_cat_id.'
						   AND m_left '.($sel_type == 'categs' ? ' >= ' : ' > ').$sel_left.'
						   AND m_right '.($sel_type == 'categs' ? ' <= ' : ' < ').$sel_right.'
						   AND m_level >= '.$from.' AND m_level <= '.$to.'
						   ORDER BY m_left');
    }
	else
    {
        if ($count > 0 && ($to - $from + 1) > $count)
        {
            $to = min($to, $from + $count - 1);
        }

        // вытаскиваем пункты меню
        $res = sql_param_query('SELECT m_id, m_text, m_url, m_alt, m_target, m_type, m_props, m_level, m_show, m_status, '.$like_str.' AS m_sel'.
                           $elems_fields_select_sql.'
                           FROM sb_menu, sb_categs WHERE '.$where_str.' cat_id IN (?a)
                           AND m_cat_id=cat_id
                           AND m_level >= '.$from.' AND m_level <= '.$to.'
                           ORDER BY cat_left, m_left', $cat_ids);
    }

    if (!$res)
    {
        $GLOBALS['sbCache']->save('pl_menu', '');
        return;
    }

    $tags = array_merge($tags, array('{ID}',
                                     '{PARENT_ID}',
                                     '{TEXT}',
                                     '{URL}',
                                     '{ALT}',
                                     '{TARGET}',
                                     '{CLOSED_ICON}',
                                     '{COUNT}',
                                     '{SUB_ITEMS}'
                        ));

    $i = 0;
    $num_sub = 0;
    $sel_id = 0; // идентификатор последнего выбранного пункта меню

    $res_closed = sbQueryCache::query('SELECT COUNT(*) FROM sb_categs WHERE cat_ident="pl_pages" AND cat_closed=1');
    $check_closed = ($res_closed && $res_closed[0][0] > 0);

    $result = fMenu_Parse_Tree($res, $i, 1, '0', $mt_lang, $mt_levels, $mt_fields_temps, $elems_fields, $tags, $num_sub, $params, $check_closed, $sel_id);

	if($sel_id > 0)
	{
		@require_once SB_CMS_PL_PATH.'/pl_clouds/prog/pl_clouds.php';
		fClouds_Init_Tags('pl_menu', array($sel_id));
    }

    $result = preg_replace('/\{[_A-Z0-9'.$GLOBALS['sb_reg_upper_interval'].']+\}/'.SB_PREG_MOD, '', $result);
    $GLOBALS['sbCache']->save('pl_menu', $result);
}

/**
 * Проверка, является ли страница закрытой или нет.
 *
 * @param string $url Адрес страницы.
 *
 * @return bool TRUE, если страница закрыта, FALSE в ином случае.
 */
function fMenu_Check_Closed($url)
{
    static $domains = array();

    $domain = SB_COOKIE_DOMAIN;
    
    /*Если домен является алиасом*/
    if(!array_key_exists($domain, $GLOBALS['sb_domains']))
    {
        $keys = array_keys($GLOBALS['sb_domains']);
        $domain = $keys[0];
    }
    
    $path = '';
	$page = sbPlugins::getSetting('sb_directory_index');
	if (trim($page) == '')
	{
	    $page = 'index.php';
	}

    if ($url != '')
    {
        if (sb_strpos($url, '://') === false)
        {
            $url = 'http://'.$domain.'/'.trim($url, '/');
        }

        $url = parse_url($url);
        if (!$url)
            return false;

        $domain = str_ireplace('www.', '', $url['host']);

        if (isset($url['path']) && $url['path'] != '')
        {
            $path = explode('/', str_replace('\\', '/', $url['path']));

            if (strpos($path[count($path) - 1], '.') !== false)
            {
                $page = array_pop($path);
                $path = trim(implode('/', $path), '/\\');
            }
            else
            {
                $path = trim(implode('/', $path), '/\\');
            }
        }
    }

    $left = -1;
    $right = -1;

    if (!array_key_exists($domain, $domains))
    {
        $res = sql_param_query('SELECT cat_left, cat_right FROM sb_categs WHERE cat_title=? AND cat_ident="pl_pages" AND cat_level=1', strtolower($domain));
        if ($res)
        {
            $domains[$domain] = $res[0];
            $left = $res[0][0];
            $right = $res[0][1];
        }
    }
    else
    {
        $left = $domains[$domain][0];
        $right = $domains[$domain][1];
    }

    if ($left == -1 || $right == -1)
    {
        // ссылка на внешнюю страницу
        return false;
    }

    $res = sql_param_query('SELECT c.cat_closed, r.group_ids
                    FROM sb_categs c LEFT JOIN sb_catrights r ON r.cat_id=c.cat_id, sb_catlinks l, sb_pages p
                    WHERE p.p_filepath=? AND p.p_filename=?
                    AND l.link_el_id=p.p_id AND l.link_cat_id=c.cat_id
                    AND c.cat_ident="pl_pages" AND c.cat_left >= ?d AND c.cat_right <= ?d', $path, $page, $left, $right);

    if ($res && $res[0][0] == 1)
    {
        if (!isset($_SESSION['sbAuth']))
            return true;

        $groups = explode('^', trim($res[0][1], '^'));
        if (count(array_intersect($_SESSION['sbAuth']->getUserGroups(), $groups)) > 0)
            return false;
        else
            return true;
    }
    else
    {
        return false;
    }
}

/**
 * Парсинг вывода пунктов
 *
 * @param array $res Массив выборки пунктов из БД.
 * @param int $i Текущий индекс массива $res.
 * @param int $level Текущий уровень пункта меню.
 * @param int $parent_id Идентификатор родительского пункта.
 * @param string $mt_lang Язык макета дизайна.
 * @param array $mt_levels Массив макетов дизайна пунктов меню.
 * @param array $mt_fields_temps Массив макетов дизайна пользовательских полей пунктов меню.
 * @param array $elems_fields Описания пользовательских полей пунктов меню.
 * @param array $tags Массив тегов.
 * @param int $num_sub Кол-во подпунктов.
 * @param array $params Параметры компонента.
 * @param bool $check_closed Производить проверку на закрытые страницы (TRUE) или нет (FALSE).
 * @param int $sel_id Идентификатор последнего выбранного пункта.
 *
 * @return string Вывод пунктов.
 */
function fMenu_Parse_Tree(&$res, &$i, $level, $parent_id, &$mt_lang, &$mt_levels, &$mt_fields_temps, &$elems_fields, &$tags, &$num_sub, &$params, $check_closed, &$sel_id)
{
	$num = count($res);
    $num_fields = count($res[0]);
    $showed = ($i != 0);

    $levels = isset($mt_levels[$level - 1]) ? $mt_levels[$level - 1] : $mt_levels[count($mt_levels) - 1];
    $fields_temps = isset($mt_fields_temps[$level - 1]) ? $mt_fields_temps[$level - 1] : (isset($mt_fields_temps[count($mt_fields_temps) - 1]) ? $mt_fields_temps[count($mt_fields_temps) - 1] : array());
    $show_sub = (sb_strpos($levels['sub_sel'], '{SUB_ITEMS}') !==false || sb_strpos($levels['sub'], '{SUB_ITEMS}') !== false);

    $check_closed_level = $check_closed;
    if ($check_closed_level && $params['show_closed'] == 1)
    {
        $check_closed_level = (sb_strpos($levels['sub'], '{CLOSED_ICON}') !== false ||
                         sb_strpos($levels['sub_sel'], '{CLOSED_ICON}') !== false ||
                         sb_strpos($levels['item'], '{CLOSED_ICON}') !== false ||
						 sb_strpos($levels['item_sel'], '{CLOSED_ICON}') !== false);
    }

	// верх вывода меню
	$result = '';

	require_once(SB_CMS_LIB_PATH.'/sbLayout.inc.php');
	$dop_tags = array('{ID}', '{PARENT_ID}', '{TEXT}', '{URL}', '{ALT}', '{TARGET}');

	if(SB_DEMO_SITE)
	{
		$demo_statuses = explode(',', sb_get_workflow_demo_statuses());
	}
	else
	{
		$demo_statuses = array(1);
	}

	while ($i < $num)
	{
		$id = $res[$i][0];
        $url = trim($res[$i][2]);
        $closed = false;

		if ($check_closed_level)
		{
			$closed = fMenu_Check_Closed($url);
		}

		if(!isset($params['show_hidden']) || $params['show_hidden'] == 0)
		{
			$show = $res[$i][8] == 1 && is_array($demo_statuses) && in_array($res[$i][9], $demo_statuses) && ($params['show_closed'] == 1 || !$closed);
		}
		else
		{
			$show = is_array($demo_statuses) && in_array($res[$i][9], $demo_statuses) && ($params['show_closed'] == 1 || !$closed);
		}

		$own_i = $i;
		$sel = ($res[$own_i][10] == 1);

        if (!$show)
        {
            $j = $i + 1;
            while ($j < $num && $res[$own_i][7] < $res[$j][7])
            {
            	$i++;
                $j++;
            }

            if (($i + 1) < $num && $res[$own_i][7] > $res[$i + 1][7] && $showed)
            {
				// выход из рекурсии
			if ($result != '')
                    $result = str_replace('{PARENT_ID}', $parent_id, $levels['top']).$result.str_replace('{PARENT_ID}', $parent_id, $levels['bottom']);

				$i++;
				return $result;
            }
			$i++;
			continue;
        }

	$showed = true;

        if ($res[$i][5] == 'delim')
        {
            // разделитель
            $result .= str_replace('{ID}', $id, $levels['delim']);
            if (($i + 1) < $num && $res[$own_i][7] > $res[$i + 1][7])
            {
                // выход из рекурсии
                if ($result != '')
                    $result = str_replace('{PARENT_ID}', $parent_id, $levels['top']).$result.str_replace('{PARENT_ID}', $parent_id, $levels['bottom']);

                $i++;
                return $result;
            }
            $i++;
            continue;
        }
		elseif ($res[$i][5] == 'categs')
        {
			$props = unserialize($res[$i][6]);
			require_once SB_CMS_PL_PATH.'/pl_categs/prog/pl_categs.php';

            if (intval($params['to']) > 0)
            {
                $count = $params['to'] - $res[$i][7];

                if (intval($props['to']) > 0)
                {
					$props['to'] = min($props['to'], $props['from'] + $count);
                }
                else
                {
                    $props['to'] = $props['from'] + $count;
                }
            }

            if (isset($params['menu_design']) && $params['menu_design'] == 1)
            	$props['temp_id'] = -1;

            $cat_params = array();
            $cat_params['ids'] = $props['cat_id'];
            $cat_params['parent_selection'] = isset($params['parent_selection']) ? $params['parent_selection'] : 0;
            $cat_params['parent_link'] = isset($params['parent_link']) && $props['plugin_ident'] != 'pl_pages' ? $params['parent_link'] : 0;
            $cat_params['from'] = $props['from'] + ($props['from'] != 0 && $props['plugin_ident'] == 'pl_pages' ? 1 : 0);
            $cat_params['to'] = $props['to'] + ($props['to'] != 0 && $props['plugin_ident'] == 'pl_pages' ? 1 : 0);
            $cat_params['count'] = max(0, $params['count'] - $level + 1);
            $cat_params['show_closed'] = isset($params['show_closed']) ? $params['show_closed'] : 0;
            $cat_params['show_hidden'] = isset($props['show_hidden']) ? $props['show_hidden'] : 0;
            $cat_params['page'] = ($props['ident'] == 'pl_pages' ? 'pl_pages' : $url);
            $cat_params['menu_temp_id'] = $params['temp_id'];
            $cat_params['query_string'] = isset($params['query_string']) ? $params['query_string'] : 1;

			$cat_res = fCategs_Show_Menu_Categs($props['temp_id'], $cat_params, $props['plugin_ident'], $props['ident'], $props['get_ident'], $num_sub, $level - 1, $sel);

            $result .= str_replace('{TARGET}', $res[$i][4], $cat_res);

            if (($i + 1) < $num && $res[$own_i][7] > $res[$i + 1][7])
            {
                // выход из рекурсии
                if ($result != '')
                    $result = str_replace('{PARENT_ID}', $parent_id, $levels['top']).$result.str_replace('{PARENT_ID}', $parent_id, $levels['bottom']);

                $i++;
				return $result;
            }

            $i++;
            continue;
        }

        $values = array();
        $dop_values = array($id, $parent_id, $res[$i][1], $url, $res[$i][3], $res[$i][4]);

        if ($num_fields > 11)
        {
            for ($j = 11; $j < $num_fields; $j++)
            {
                $values[] = $res[$i][$j];
			}
			$values = sbLayout::parsePluginFields($elems_fields, $values, $fields_temps, $dop_tags, $dop_values, $mt_lang);
        }

        $values[] = $id;         // {ID}
        $values[] = $parent_id;  // {PARENT_ID}
        $values[] = $res[$i][1]; // {TEXT}
        $values[] = $url;        // {URL}
        $values[] = $res[$i][3]; // {ALT}
        $values[] = $res[$i][4]; // {TARGET}
        $values[] = $closed && isset($fields_temps['closed_icon']) ? str_replace($dop_tags, $dop_values, $fields_temps['closed_icon']) : ''; // {CLOSED_ICON}

        $num_sub_sub = 0;
        $subitems = '';
        if (($i + 1) < $num && $res[$i][7] < $res[$i + 1][7])
        {
            // вывод подпунктов
            $i++;
            if ($show_sub)
            {
				$subitems = fMenu_Parse_Tree($res, $i, ($show ? $level + 1 : $level), $id, $mt_lang, $mt_levels, $mt_fields_temps, $elems_fields, $tags, $num_sub_sub, $params, $check_closed, $sel_id);
            }
			else
			{
                while ($i < $num && $res[$own_i][7] < $res[$i][7])
                {
                    $i++;
                }
            }
            $i--;
        }

        $values[] = $num_sub_sub; // {COUNT}
        $values[] = $subitems; // {SUB_ITEMS}

        $num_sub++;

        if ($sel)
        {
            // вывод выбранного пункта
            if ($subitems != '')
            {
                // с подпунктами
                $result .= str_replace($tags, $values, $levels['sub_sel']);
            }
            else
            {
				// без подпунктов
				$result .= str_replace($tags, $values, $levels['item_sel']);
			}

			if ($sel_id <= 0)
				$sel_id = $res[$own_i][0];
		}
		else
        {
			if ($subitems != '')
            {
                // с подпунктами
                $result .= str_replace($tags, $values, $levels['sub']);
            }
			else
            {
                // без подпунктов
                $result .= str_replace($tags, $values, $levels['item']);
            }
        }

        if (($i + 1) < $num && $res[$own_i][7] > $res[$i + 1][7])
        {
            // выход из рекурсии
            if ($result != '')
                $result = str_replace('{PARENT_ID}', $parent_id, $levels['top']).$result.str_replace('{PARENT_ID}', $parent_id, $levels['bottom']);

            $i++;
            return $result;
        }

		$i++;
	}

	if ($result != '')
	{
       $result = str_replace('{PARENT_ID}', $parent_id, $levels['top']).$result.str_replace('{PARENT_ID}', $parent_id, $levels['bottom']);
    }

    return $result;
}

/**
 * Вывод пути по сайту
 * 
 * @param $el_id = -1;
 * @param $temp_id Int; //идентификатор макета дизайна пути по сайту
 * @param $params Serialize Array
 * @param $params['ids'] идентификаторы разделов меню, разделенные символом ^
 * @param $params['temp_id'] идентификатор макета дизайна пути по сайту
 * @param $params['num_items'] количество пунктов в пути
 * @param $params['show_hidden'] выводить скрытые пункты 0/1 
 * @param $params['show_closed'] выводить закрытые пункты 0/1
 * @param $strip_tags Bool //вырезать html-теги
 * 
 * @return string Вывод пути по сайту
 */
function fMenu_Elem_Path($el_id, $temp_id, $params, $tag_id, $strip_tags=false)
{
    if ($GLOBALS['sbCache']->check('pl_menu', $tag_id, array($el_id, $temp_id, $params)))
        return;

    $res = sbQueryCache::getTemplate('sb_menu_path_temps', $temp_id, '', $temp_id);
    if (!$res)
    {
        sb_add_system_message(sprintf(KERNEL_PROG_NO_TEMPLATE, $_SERVER['PHP_SELF'], KERNEL_PROG_MENU_PLUGIN), SB_MSG_WARNING);
        $GLOBALS['sbCache']->save('pl_menu', '');
        return;
    }

    list($mpt_lang, $mpt_checked, $mpt_top, $mpt_item, $mpt_last_item, $mpt_bottom, $mpt_fields_temps) = $res[0];

    if ($mpt_fields_temps != '')
    {
        $mpt_fields_temps = unserialize($mpt_fields_temps);
    }
    else
    {
        $mpt_fields_temps = array();
    }

    $params = unserialize(stripslashes($params));
    if (!isset($params['show_closed']))
    	$params['show_closed'] = 0;

    $cat_ids = explode('^', $params['ids']);

    $left = -1;
    $right = -1;
    $cat_id = -1;

    // проверяем, выбрана страница или раздел
    $get_idents = array();
    foreach (array_keys($_GET) as $key)
    {
    	if (sb_substr($key, -4) == '_cid')
    	{
    		$get_idents[] = sb_substr($key, 0, -4);
    	}
    	elseif (sb_substr($key, -5) == '_scid')
    	{
    		$get_idents[] = sb_substr($key, 0, -5);
    	}
    }

    // определяем дефолтную страницу
    $index_page = sbPlugins::getSetting('sb_directory_index');
    if (trim($index_page) == '')
    {
        $index_page = 'index.php';
    }

    $filename = basename($_SERVER['PHP_SELF']);
    $dirname = trim($GLOBALS['sbSql']->escape(rtrim(str_replace('\\', '/', dirname($_SERVER['PHP_SELF'])), '/')), "'");

    $php_self = trim($GLOBALS['sbSql']->escape($_SERVER['PHP_SELF']), "'");

    $get_like_str = '(';
    $get_like_str_idents = '(';

    if (count($get_idents) > 0)
    {
    	// есть идентификаторы разделов, вытаскиваем пункты меню, связанные с этими идентификаторами
    	foreach ($get_idents as $get_ident)
    	{
    		if ($filename != $index_page)
    		{
				$get_like_str .= '(m.m_props LIKE "%s:9:\"get_ident\";s:'.sb_strlen($get_ident).':\"'.$get_ident.'\";%" AND
					(m_url = '.$GLOBALS['sbSql']->escape($_SERVER['PHP_SELF']).' OR
		             m_url = '.$GLOBALS['sbSql']->escape(trim($_SERVER['PHP_SELF'], '/')).' OR
		             m_url = \'http://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
		             m_url = \'http://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
		             m_url = \'https://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
		             m_url = \'https://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\'
		            )) OR ';
    		}
    		else
    		{
    			$get_like_str .= '(m.m_props LIKE "%s:9:\"get_ident\";s:'.sb_strlen($get_ident).':\"'.$get_ident.'\";%" AND
					(m_url = '.$GLOBALS['sbSql']->escape($_SERVER['PHP_SELF']).' OR
					 m_url = '.$GLOBALS['sbSql']->escape(trim($_SERVER['PHP_SELF'], '/')).' OR
                     m_url = \'http://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
					 m_url = \'http://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
                     m_url = \'http://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$dirname.'/\' OR
                     m_url = \'http://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$dirname.'/\' OR
                     m_url = \'https://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
                     m_url = \'https://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
                     m_url = \'https://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$dirname.'/\' OR
                     m_url = \'https://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$dirname.'/\' OR
                     m_url = \''.$dirname.'/\'
                    )) OR ';
    		}

    		$get_like_str_idents .= '(m.m_props LIKE "%s:9:\"get_ident\";s:'.sb_strlen($get_ident).':\"'.$get_ident.'\";%") OR ';
    	}
	}

	$demo_statuses = ''; // AND m.m_status IN ('.sb_get_workflow_demo_statuses().') ';
	$show_str = '';
	if (!isset($params['show_hidden']) || $params['show_hidden'] == 0)
	{
		$show_str = ' AND m.m_show = 1 ';
	}

	$get_like_str .= 'm.m_props LIKE "%s:9:\"get_ident\";s:8:\"pl_pages\";%" )
							'.$show_str.$demo_statuses.'
							AND c.cat_id IN (?a) AND m.m_cat_id=c.cat_id';

	$get_like_str_idents .= 'm.m_props LIKE "%s:9:\"get_ident\";s:8:\"pl_pages\";%" )
							'.$show_str.$demo_statuses.'
							AND c.cat_id IN (?a) AND m.m_cat_id=c.cat_id';

	$res = sql_param_query('SELECT m.m_left, m.m_right, m.m_cat_id, m.m_props FROM sb_menu m, sb_categs c WHERE '.$get_like_str.' ORDER BY c.cat_left, m.m_left', $cat_ids);

	if (!$res)
	{
		$res = sql_param_query('SELECT m.m_left, m.m_right, m.m_cat_id, m.m_props FROM sb_menu m, sb_categs c WHERE '.$get_like_str_idents.' ORDER BY c.cat_left, m.m_left', $cat_ids);
	}

    if ($res)
    {
        require_once SB_CMS_PL_PATH.'/pl_categs/prog/pl_categs.php';

        foreach ($res as $value)
        {
        	list($m_left, $m_right, $m_cat_id, $m_props) = $value;

        	if ($m_props == '')
        		continue;

        	$m_props = unserialize($m_props);

        	$cat_params = array();
            $cat_params['ids'] = $m_props['cat_id'];
            $cat_params['from'] = $m_props['from'];
            $cat_params['to'] = $m_props['to'];
            $cat_params['show_closed'] = isset($params['show_closed']) ? $params['show_closed'] : 0;
            $cat_params['show_hidden'] = isset($m_props['show_hidden']) ? $m_props['show_hidden'] : 0;

            $cat_count = fCategs_Count_Menu_Categs($m_props['temp_id'], $cat_params, $m_props['plugin_ident'], $m_props['ident'], $m_props['get_ident']);

            if ($cat_count)
            {
            	// есть разделы
            	$left = $m_left;
            	$right = $m_right;
            	$cat_id = $m_cat_id;
            	break;
            }
        }
    }

	// вытаскиваем пользовательские поля меню
    $res = sbQueryCache::query('SELECT pd_fields FROM sb_plugins_data WHERE pd_plugin_ident="pl_menu"');

    $elems_fields = array();
    $elems_fields_select_sql = '';

    $tags = array();

    // формируем SQL-запрос для пользовательских полей
    if ($res && $res[0][0] != '')
    {
        $elems_fields = unserialize($res[0][0]);
        if ($elems_fields)
        {
            foreach ($elems_fields as $value)
            {
                if (isset($value['sql']) && $value['sql'] == 1)
                {
                    $elems_fields_select_sql .= ', user_f_'.$value['id'];

                    if ($value['type'] == 'google_coords' || $value['type'] == 'yandex_coords')
	                {
	                	$tags[] = '{'.$value['tag'].'_LATITUDE}';
	                	$tags[] = '{'.$value['tag'].'_LONGTITUDE}';

	                	if ($value['type'] == 'yandex_coords')
	                	{
	                		$tags[] = '{'.$value['tag'].'_API_KEY}';
	                	}
	                }
	                else
	                {
	                	$tags[] = '{'.$value['tag'].'}';
	                }
                }
            }
        }
    }

    $where_str = '';
    if ($mpt_checked != '')
    {
        $checked_sql = '';

        $mpt_checked = explode(' ', $mpt_checked);
        foreach ($mpt_checked as $value)
        {
            foreach ($elems_fields as $val)
            {
                if ($val['id'] == $value)
                {
                    $checked_sql .= 'user_f_'.$value.' = 1 AND ';
                }
            }
        }

        if ($checked_sql != '')
        {
            $where_str .= '(m_type != "item" OR ('.substr($checked_sql, 0, -5).')) AND ';
        }
    }

    if ($left == -1)
    {
    	if ($filename != $index_page)
	    {
	    	if (isset($GLOBALS['PHP_SELF']) && $GLOBALS['PHP_SELF'] != $_SERVER['PHP_SELF'])
	    	{
	    		$chpu_self = trim($GLOBALS['sbSql']->escape($GLOBALS['PHP_SELF']), "'");
	    		$like_str = '(m_url = '.$GLOBALS['sbSql']->escape($_SERVER['PHP_SELF']).' OR
	                      m_url = '.$GLOBALS['sbSql']->escape(trim($_SERVER['PHP_SELF'], '/')).' OR
	                      m_url = \'http://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
	                      m_url = \'http://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
	                      m_url = \'https://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
	                      m_url = \'https://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
	                      m_url = '.$GLOBALS['sbSql']->escape($GLOBALS['PHP_SELF']).' OR
	                      m_url = '.$GLOBALS['sbSql']->escape(trim($GLOBALS['PHP_SELF'], '/')).' OR
	                      m_url = \'http://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$chpu_self.'\' OR
	                      m_url = \'http://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$chpu_self.'\' OR
	                      m_url = \'https://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$chpu_self.'\' OR
	                      m_url = \'https://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$chpu_self.'\')
						';
	    	}
	    	else
	    	{
				$like_str = '(m_url = '.$GLOBALS['sbSql']->escape($_SERVER['PHP_SELF']).' OR
	                      m_url = '.$GLOBALS['sbSql']->escape(trim($_SERVER['PHP_SELF'], '/')).' OR
	                      m_url = \'http://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
	                      m_url = \'http://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
	                      m_url = \'https://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
	                      m_url = \'https://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\')
						';
	    	}
	    }
	    else
	    {
			$like_str = '(m_url = '.$GLOBALS['sbSql']->escape($_SERVER['PHP_SELF']).' OR
						  m_url = '.$GLOBALS['sbSql']->escape(trim($_SERVER['PHP_SELF'], '/')).' OR
	                      m_url = \'http://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
						  m_url = \'http://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
	                      m_url = \'http://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$dirname.'/\' OR
	                      m_url = \'http://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$dirname.'/\' OR
	                      m_url = \'https://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
	                      m_url = \'https://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$php_self.'\' OR
	                      m_url = \'https://'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$dirname.'/\' OR
	                      m_url = \'https://www.'.sb_str_replace('www-demo.', '', SB_COOKIE_DOMAIN).$dirname.'/\' OR
	                      m_url = \''.$dirname.'/\'
	                      )';
	    }

	    // вытаскиваем выделенные пункты меню
	    $res = sql_param_query('SELECT m_left, m_right, m_cat_id
	                           FROM sb_menu, sb_categs WHERE '.$where_str . $like_str.' AND cat_id IN (?a)
	                           AND m_cat_id=cat_id AND m_level > 0 AND m_type="item"
	                           ORDER BY cat_left, m_left DESC LIMIT 1', $cat_ids);

	    if (!$res)
	    {
	        $GLOBALS['sbCache']->save('pl_menu', '');
	        return;
	    }

		list($left, $right, $cat_id) = $res[0];
	}

	$res = sql_query('SELECT m_id, m_text, m_url, m_alt, m_target, m_type, m_props, m_level, m_show, m_status '.
                           $elems_fields_select_sql.'
                           FROM sb_menu WHERE '.$where_str.' m_cat_id='.$cat_id.'
                           AND m_left <= '.$left.' AND m_right >= '.$right.' AND m_level > 0
						   ORDER BY m_left');

	if (!$res)
    {
		$GLOBALS['sbCache']->save('pl_menu', '');
		return;
    }

    $tags = array_merge($tags, array('{ID}',
                                     '{TEXT}',
                                     '{URL}',
                                     '{ALT}',
                                     '{TARGET}'
                        ));

    $result = '';
    require_once(SB_CMS_LIB_PATH.'/sbLayout.inc.php');
    $dop_tags = array('{ID}', '{TEXT}', '{URL}', '{ALT}', '{TARGET}');
    $num_fields = count($res[0]);

    $res_closed = sql_query('SELECT COUNT(*) FROM sb_categs WHERE cat_ident="pl_pages" AND cat_closed=1');
    $check_closed = ($res_closed && $res_closed[0][0] > 0);

    $num_items = intval($params['num_items']);
    $num = 1;
	$last = true;

	if(SB_DEMO_SITE)
	{
		$demo_statuses = explode(',', sb_get_workflow_demo_statuses());
	}
	else
	{
		$demo_statuses = array(1);
	}

    for ($i = count($res) - 1; $i >= 0; $i--)
    {
		if ($num > $num_items)
			break;

        if ($res[$i][5] == 'delim')
        {
			// разделитель
			continue;
        }

        $show_status = is_array($demo_statuses) && in_array($res[$i][9], $demo_statuses);
    	if(!isset($params['show_hidden']) || $params['show_hidden'] == 0)
		{
			$show = $res[$i][8] == 1 && $show_status;
		}
		else
		{
			$show = $show_status;
		}

		if (!$show_status)
		{
			$last = false;
		}

		if(!$show)
		{
			continue;
		}

		$url = $res[$i][2];

        if ($params['show_closed'] != 1 && $check_closed)
        {
            if (fMenu_Check_Closed($url))
                continue;
        }

        if($res[$i][5] == 'categs')
        {
            $props = unserialize($res[$i][6]);
            require_once SB_CMS_PL_PATH.'/pl_categs/prog/pl_categs.php';

            $cat_params = array();
            $cat_params['ids'] = $props['cat_id'];
            $cat_params['from'] = $props['from'];
            $cat_params['to'] = $props['to'];
            $cat_params['show_closed'] = isset($params['show_closed']) ? $params['show_closed'] : 0;
            $cat_params['show_hidden'] = isset($props['show_hidden']) ? $props['show_hidden'] : 0;
            $cat_params['page'] = $url;
            $cat_params['menu_temp_id'] = $params['temp_id'];

            $cat_res = fCategs_Show_Menu_Path($props['temp_id'], $cat_params, $props['plugin_ident'], $props['ident'], $props['get_ident'], $num, $num_items);

            $result .= str_replace('{TARGET}', $res[$i][4], $cat_res);
        }
        else
        {
			$id = $res[$i][0];

			$values = array();
			if ($num_fields > 10)
            {
                for ($j = 10; $j < $num_fields; $j++)
                {
                    $values[] = $res[$i][$j];
                }

				$dop_values = array($id, $res[$i][1], $url, $res[$i][3], $res[$i][4]);
				$values = sbLayout::parsePluginFields($elems_fields, $values, $mpt_fields_temps, $dop_tags, $dop_values, $mpt_lang, '', '', 0, 0, ($last ? $mpt_last_item : $mpt_item));
            }

            $values[] = $id;         // {ID}
            $values[] = $res[$i][1]; // {TEXT}
            $values[] = $url;        // {URL}
            $values[] = $res[$i][3]; // {ALT}
            $values[] = $res[$i][4]; // {TARGET}

			if ($last)
            {
				@require_once 	SB_CMS_PL_PATH.'/pl_clouds/prog/pl_clouds.php';
				fClouds_Init_Tags('pl_menu', array($id));

				// последний пункт
				$result = str_replace($tags, $values, $mpt_last_item).$result;
            }
            else
        	{
				// не последний пункт
				$result = str_replace($tags, $values, $mpt_item).$result;
            }
        }

        $last = false;
		$num++;
    }

	if ($strip_tags)
		$result = strip_tags($mpt_top.preg_replace('/\{[_A-Z0-9'.$GLOBALS['sb_reg_upper_interval'].']+\}/'.SB_PREG_MOD, '', $result).$mpt_bottom);
	else
		$result = $mpt_top.preg_replace('/\{[_A-Z0-9'.$GLOBALS['sb_reg_upper_interval'].']+\}/'.SB_PREG_MOD, '', $result).$mpt_bottom;

	$GLOBALS['sbCache']->save('pl_menu', $result);
}
?>