<?php
define('SB_SEARCH_PAGE_ERROR', 0);
define('SB_SEARCH_PAGE_OK', 1);

/**
 * Запуск задания индексации из планировщика
 *
 * @param int $ss_id Идентификатор настроек индексации.
 */
function fSearch_Cron_Refresh($ss_id)
{
	@set_time_limit(0);
	// взять из базы параметры поиска
	$query = sql_param_query('SELECT ss_domains, ss_settings FROM sb_search WHERE ss_id = ?d', $ss_id);
	if($query)
	{
		list($domains, $arr_settings) = $query[0];

		$arr_settings = unserialize($arr_settings);
		$domains = explode(',', $domains);
		foreach ($domains as $key => $value)
		{
			$domains[$key] = trim($value);
		}

		// подключаем класс морфологии
		require_once(SB_CMS_LIB_PATH.'/sbSearch.inc.php');
		// создаем класс морфологии
		$GLOBALS['sb_indexed_morfology'] = new sbSearch();

		// поместить в глобальный массив установки
		if (!isset($GLOBALS['sb_search_settings']) || !is_array($GLOBALS['sb_search_settings']))
			$GLOBALS['sb_search_settings'] = array();

		$GLOBALS['sb_search_settings'][$ss_id] = array();
		// домены
		$GLOBALS['sb_search_settings'][$ss_id]['domains'] = $domains;
		// порт
		$GLOBALS['sb_search_settings'][$ss_id]['port'] = isset($arr_settings['port']) && trim($arr_settings['port']) != '' ? $arr_settings['port'] : '80';
		// протокол
		$GLOBALS['sb_search_settings'][$ss_id]['scheme'] = isset($arr_settings['scheme']) && trim($arr_settings['scheme']) != '' ? $arr_settings['scheme'] : 'http';
		// признак индексации метатегов
		$GLOBALS['sb_search_settings'][$ss_id]['meta_tags'] = isset($arr_settings['meta_tags']) ? $arr_settings['meta_tags'] : 0;
		// полный индекс
		$GLOBALS['sb_search_settings'][$ss_id]['full_index'] = isset($arr_settings['full_index']) ? $arr_settings['full_index'] : 0;
		// минимальная длина слова
		$GLOBALS['sb_search_settings'][$ss_id]['min_length'] = $arr_settings['min_length_word'];
		// недопустимые параметры в URL
		$GLOBALS['sb_search_settings'][$ss_id]['words_url'] = explode(' ', $arr_settings['impermissible_words']);
		// список недопустимых слов
		$GLOBALS['sb_search_settings'][$ss_id]['words'] = explode(' ', $arr_settings['words_no_indexing']);
		// список расширений файлов для индексации
		$GLOBALS['sb_search_settings'][$ss_id]['exts'] = explode(' ', $arr_settings['extentions_files']);
		// список запрещенных директорий
		$GLOBALS['sb_search_settings'][$ss_id]['dirs'] = explode("\n", str_replace("\r\n", "\n", $arr_settings['dirs_not_allow']));
		// символы разрешенные для индексации
		$GLOBALS['sb_search_settings'][$ss_id]['chars'] = $arr_settings['characters_allow'];
		// максимальный уровень вложенности ссылок
		$GLOBALS['sb_search_settings'][$ss_id]['max_levels'] = intval($arr_settings['max_levels']);
		// стартовая страница
		$GLOBALS['sb_search_settings'][$ss_id]['indexing_start_page'] = '';
        //индексировать закрытые страницы
        $GLOBALS['sb_search_settings'][$ss_id]['allow_closed_pages'] = isset($arr_settings['allow_closed_pages']) ? $arr_settings['allow_closed_pages'] : 0;

		if (isset($arr_settings['indexing_start_page']) && trim($arr_settings['indexing_start_page']) != '')
		{
			if (sb_stripos($arr_settings['indexing_start_page'], 'http://') !== false || sb_stripos($arr_settings['indexing_start_page'], 'https://') !== false)
			{
				$arr_settings['indexing_start_page'] = sb_str_replace(array('http://', 'https://'), '', $arr_settings['indexing_start_page']);
				$pos = sb_strpos($arr_settings['indexing_start_page'], '/');
				if ($pos !== false)
				{
					$arr_settings['indexing_start_page'] = sb_substr($arr_settings['indexing_start_page'], $pos);
				}
			}

			$GLOBALS['sb_search_settings'][$ss_id]['indexing_start_page'] = trim($arr_settings['indexing_start_page'], '\\/');
		}

		// стереть таблицу sb_search_pages_...
		sql_query('TRUNCATE TABLE sb_search_pages_'.$ss_id);

        $hash = '';
        if($GLOBALS['sb_search_settings'][$ss_id]['allow_closed_pages'] == 1)
        {
            // формируем хэш для идентификации робота
            $hash = md5(uniqid(time()));

            // создать временную таблицу для хранения хэша
            $res = sql_param_query('CREATE TABLE IF NOT EXISTS sb_search_mem(
                hash VARCHAR(255),
                PRIMARY KEY(hash)
            ) ENGINE=MEMORY DEFAULT CHARSET=utf8 COLLATE utf8_bin');

            if ($res)
            {
                $res = sql_param_query('INSERT INTO sb_search_mem SET hash=?s', $hash);
            }
        }

		// вызвать функцию индексации для каждого индексируемого домена
		foreach($domains as $domain)
		{
			sb_add_system_message(sprintf(KERNEL_PROG_PL_SEARCH_SYSTEMLOG_INDEXING_START, $domain));
			$tm_start = microtime(true);

			fSearch_Indexing($ss_id, $domain, $hash);

			$tm = round(microtime(true) - $tm_start, 3);
			$res = sql_query('SELECT COUNT(*) FROM sb_search_pages_'.$ss_id);
			sb_add_system_message(sprintf(KERNEL_PROG_PL_SEARCH_SYSTEMLOG_INDEXING_END, $domain, $res[0][0], $tm));
		}

		// обновить время индексации домена
		sql_param_query('UPDATE sb_search SET ss_date = ?d WHERE ss_id = ?d', time(), $ss_id);

		// уничтожаем глобальные переменные
		unset($GLOBALS['sb_indexed_morfology']);
		unset($GLOBALS['sb_search_settings'][$ss_id]);
		unset($GLOBALS['sb_search_esc']);

        sql_query('DROP TABLE IF EXISTS sb_search_mem');
	}
}

/**
 * Формирует полную ссылку и проверяет ее на корректность
 *
 * @param int $ss_id Идентификатор настроек индексации.
 * @param string $base Протокол, порт и домен.
 * @param string $url Ссылка.
 *
 * @return string Абсолютная ссылка или FALSE, если ссылка не проходит по условиям.
 */
function fSearch_Parse_URL($ss_id, &$base, &$url)
{
	static $index_file = '';
	if ($index_file == '')
	{
		$index_file = trim(sbPlugins::getSetting('sb_directory_index'));
		if ($index_file == '')
		{
			$index_file = 'index.php';
		}
	}

	$domain = '';

	$ar = @parse_url($url);
	if (isset($ar['path']))
	{
		$ar['path'] = str_replace('\\', '/', $ar['path']);
	}

	$ar_base = @parse_url($base);

	// scheme
	$scheme = strtolower(isset($ar['scheme']) ? $ar['scheme'] : $ar_base['scheme']);
	if($scheme != 'http' && $scheme != 'https' && $scheme != 'ftp')
		return false;

	// host
	$domain = sb_strtolower(isset($ar['host']) ? $ar['host'] : $ar_base['host']);
	$domain = str_ireplace('www.', '', $domain);

	$is_domain = in_array($domain, $GLOBALS['sb_search_settings'][$ss_id]['domains']);
	$is_www_domain = in_array('www.'.$domain, $GLOBALS['sb_search_settings'][$ss_id]['domains']);

	// проверить домен на допустимый
	if (!$is_domain && !$is_www_domain)
	{
		$found = false;
		foreach ($GLOBALS['sb_domains'] as $config_domain => $value)
		{
			if (isset($value['pointers']) && is_array($value['pointers']) && in_array($domain, $value['pointers']))
			{
				if (!in_array($config_domain, $GLOBALS['sb_search_settings'][$ss_id]['domains']))
				{
					return false;
				}
				else
				{
					$found = true;
					break;
				}
			}
		}

		if (!$found)
		{
			return false;
		}
	}

	if ($is_www_domain)
	{
		$domain = 'www.'.$domain;
	}

	// port
	$port = isset($ar['port']) ? $ar['port'] : isset($ar_base['port']) ? $ar_base['port'] : '80';

	// path
	$path = '';
    $path_index = '';
	if(isset($ar['path']) && trim($ar['path']) != '')
	{
		$path_info = pathinfo($ar['path']);

		$filename = '';
		$extension = '';
		if (!isset($path_info['extension']))
		{
			$path = trim($ar['path'], '/');
			$path .= '/';
            $path_index = $path.$index_file;

			$filename = $index_file;
			$extension = pathinfo($index_file, PATHINFO_EXTENSION);
		}
		else
		{
			$path_info['dirname'] = str_replace('\\', '/', $path_info['dirname']);
			$path = trim($path_info['dirname'], '/');

			$filename = trim($path_info['basename']);
			$extension = $path_info['extension'];

			if ($filename == $index_file)
            {
				$path .= '/';
                $path_index = $path.$index_file;
            }
			else
				$path .= '/'.$filename;
		}

		// проверить расширение файла
		if(!in_array($extension, $GLOBALS['sb_search_settings'][$ss_id]['exts']))
			return false;

		// проверить на список запрещенных файлов и директорий
		if(!empty($GLOBALS['sb_search_settings'][$ss_id]['dirs']))
        {
            foreach($GLOBALS['sb_search_settings'][$ss_id]['dirs'] as $dir)
            {
                $dir = trim(sb_str_replace('\\', '/', $dir), '/');
                if(preg_match('/\S+\.\S+/', $dir))
                {
                    if(preg_match('/'.preg_quote($dir, '/').'$/', $path) || preg_match('/'.preg_quote($dir, '/').'$/', $path_index))
                    {
                        return false;
                    }
                }
                else
                {
                    if (preg_match('/\/' . preg_quote($dir, '/') . '\//', '/'.$path) || preg_match('/\/' . preg_quote($dir, '/') . '\//', '/'.$path_index))
                    {
                        return false;
                    }
                }
            }
        }
	}

	// query
	if(isset($ar['query']) && trim($ar['query']) != '')
	{
		// проверить слова в строке запроса URL
		$query = explode('&', trim($ar['query']));
        $ar_query = array();

		foreach($query as $pair)
		{
			// разбить каждый элемент по "="
			$pair = explode('=', $pair);
			if(isset($pair[0]))
			{
				if(in_array(sb_strtolower($pair[0]), $GLOBALS['sb_search_settings'][$ss_id]['words_url']))
					return false;

				if(!isset($pair[1]))
					continue;

				if(in_array(sb_strtolower($pair[1]), $GLOBALS['sb_search_settings'][$ss_id]['words_url']))
					return false;

                $ar_query[] = $pair[0].'='.$pair[1];
			}
		}
		$path .= '?'.implode('&', $ar_query);
	}

	$base = $scheme.'://'.$domain;
	if ($port != '80')
	{
		$base .= ':'.$port;
	}
	$base .= '/';

	$url = $base.ltrim($path, '/');

	return true;
}

/**
 * Функция, содержащая основной цикл индексации
 *
 * @param int $ss_id Идентификатор настроек индексации.
 * @param string $domain Индексируемый домен.
 */
function fSearch_Indexing($ss_id, $domain, $hash = '')
{
	$ss_id = intval($ss_id);

	// подключаем класс для загрузки страницы
	require_once(SB_CMS_LIB_PATH.'/sbDownload.inc.php');

	$start_url = $GLOBALS['sb_search_settings'][$ss_id]['scheme'].'://'.$domain;
	if ($GLOBALS['sb_search_settings'][$ss_id]['port'] != '80')
	{
		$start_url .= ':'.$GLOBALS['sb_search_settings'][$ss_id]['port'];
	}
	$start_url .= '/';

	if ($GLOBALS['sb_search_settings'][$ss_id]['indexing_start_page'] != '')
	{
		$start_url .= $GLOBALS['sb_search_settings'][$ss_id]['indexing_start_page'];
	}
	else
	{
		$index_file = trim(sbPlugins::getSetting('sb_directory_index'));
		if ($index_file == '')
		{
			$index_file = 'index.php';
		}

		if (strtolower($index_file) != 'index.php')
		{
			$start_url .= trim($index_file, '\\/');
		}
	}

	// ссылки, которые еще не индексировались
	$to_visit = array();
	$to_visit[$start_url] = array('base' => $start_url, 'level' => 0);

	// ссылки, которые уже проиндексировали
	$visited = array();

	// протокол, домен и порт для относительных ссылок
	$base = $start_url;
	while (($ar = each($to_visit)) != false)
	{
		$url = $ar['key'];
		$base = $ar['value']['base'];
		$level = $ar['value']['level'] + 1;
        unset($ar);

		// удаляем из массива ссылок, которые еще надо обойти
		unset($to_visit[$url]);

		// пауза между загрузкой содержимого
		@usleep(sbPlugins::getSetting('sb_search_delay_microsecond'));

		// загружаем содержимое страницы
		$download = new sbDownload($url);
		$download->mQueryStr = 'sb_search=1'.($GLOBALS['sb_search_settings'][$ss_id]['allow_closed_pages'] == 1 ? '&sb_search_hash='.$hash : '');
		$html_text = $download->download();
		$head = $download->mHeader;

		$canonical = array();
		if (preg_match('/<[\s]*link[\s]+(.*?)rel[\s]*=[\s]*["\']canonical["\'](.*?)>/imsu', $html_text, $canonical) > 0)
		{
			$canonical_href = array();
			if (preg_match('/href[\s]*=[\s]*[\'"](.*?)["\']/imsu', $canonical[1], $canonical_href) > 0)
			{
				$download->mRedirectUrl = trim($canonical_href[1]);
			}
			elseif(preg_match('/href[\s]*=[\s]*[\'"](.*?)["\']/imsu', $canonical[2], $canonical_href) > 0)
			{
				$download->mRedirectUrl = trim($canonical_href[1]);
			}
		}

		if($download->mRedirectUrl != '')
		{
		    $visited[$url] = isset($visited[$url]) ? $visited[$url] : 0;
		    $old_url = $url;

		    $url = str_replace(array('&sb_search=1', '&sb_search_hash='.$hash, '?sb_search=1'), '', $download->mRedirectUrl);

		    $result = fSearch_Parse_URL($ss_id, $new_base, $url);
		    if (!$result || (isset($visited[$url]) && $old_url != $url))
		    {
		        $visited[$url] = isset($visited[$url]) ? $visited[$url] : 0;

		        if (isset($to_visit[$url]))
		            unset($to_visit[$url]);

		        unset($download);

		        continue;
		    }

		    if (isset($to_visit[$url]))
		        unset($to_visit[$url]);
		}

		unset($download);

		$visited[$url] = isset($visited[$url]) ? $visited[$url] : 0;

		$cp = 'UTF-8';
		$tm_page = time();

		$code = SB_SEARCH_PAGE_OK;

		$matches = array();
		if (trim($html_text) == '')
		{
			$code = SB_SEARCH_PAGE_ERROR;
		}
		elseif(trim($head) != '')
		{
			if(preg_match('/HTTP\/\d\.\d\s200\s/si', $head, $matches))
			{
				if(preg_match('/\scharset=(.*?)\s/si', $head, $matches))
				{
					$cp = trim(strtolower($matches[1]));
				}
				if(preg_match('/Last-Modified: ([a-zA-Z]{3},\s+\d{2}\s+[a-zA-Z]{3}\s+\d{4}\s+\d{2}:\d{2}:\d{2})/si', $head, $matches))
				{
					$tm_page = strtotime($matches[1]);
				}
			}
			else
			{
				$code = SB_SEARCH_PAGE_ERROR;
			}
		}

		if ($code == SB_SEARCH_PAGE_ERROR && $visited[$url] == 0)
		{
			$row = array();
			$row['sp_url'] = $url;
			$row['sp_flags'] = $code;

			// записываем ошибку
			sql_param_query('INSERT INTO sb_search_pages_'.$ss_id.' SET ?a', $row);
            $visited[$url] = 1;
			continue;
		}

		if(preg_match('/<meta[^>]+content=(["\']?)[^"\'>]+charset=([^\s;"\'>]+)\\1/si', $html_text, $matches))
		{
			$cp = trim(strtoupper($matches[2]));
		}

		if($cp != 'UTF-8')
		{
			$html_text = iconv($cp, 'UTF-8//IGNORE', $html_text);
		}

		// индексируем только внутри <!--sb_index_start--> и <!--sb_index_end-->
		$text = '<!--sb_index_end-->'.$html_text.'<!--sb_index_start-->';
        $text = preg_replace('/<!--\s*sb_index_end\s*-->\s*(.*?)\s*<!--\s*sb_index_start\s*-->/isu', ' ', $text);
        $text = preg_replace('/<!--\s*sb_index_start\s*-->\s*(.*?)\s*<!--\s*sb_index_end\s*-->/isu', ' ', $text);
        // вырезаем тег title (чтобы два раза не индексировать
		$text = preg_replace('/<title>(.*?)<\/title>/siu', '', $text);
        // вырезаем стили и скрипты
        $text = preg_replace('/<script.*?<\/script>/siu', ' ', $text);
    	$text = preg_replace('/<style.*?<\/style>/siu', ' ', $text);

		// вытащить заголовок
		$title = '';
		$full_title = '';
		if(preg_match('/<title>(.*?)<\/title>/siu', $html_text, $matches))
		{
			$full_title = $matches[1];
			if (sb_strlen($full_title) > 250)
				$title = sb_substr($full_title, 0, 250).' ...';
			else
				$title = $full_title;
		}

		if (trim($text) == ''  && $visited[$url] == 0)
		{
			$row = array();
			$row['sp_url'] = $url;
			$row['sp_flags'] = $code;
			$row['sp_title'] = $title;
			$row['sp_last_modified'] = $tm_page;

			// записываем ошибку
			sql_param_query('INSERT INTO sb_search_pages_'.$ss_id.' SET ?a', $row);
            $visited[$url] = 1;
			continue;
		}

		$meta_keys = '';
		$meta_desc = '';
		// вытащить значения тегов <META>
		if ($GLOBALS['sb_search_settings'][$ss_id]['meta_tags'] == 1)
		{
			// keywords
			if(preg_match('/<meta\s*name=(\'|")?\s*keywords\\1\s*content=(\'|")?(.*?)\\2.*>/siu', $html_text, $matches))
				$meta_keys = $matches[3];

			// description
			if(preg_match('/<meta\s*name=(\'|")?\s*description\\1\s*content=(\'|")?(.*?)\\2.*>/siu', $html_text, $matches))
				$meta_desc = $matches[3];
		}

    	if ($level < $GLOBALS['sb_search_settings'][$ss_id]['max_levels'] || $GLOBALS['sb_search_settings'][$ss_id]['max_levels'] <= 0)
    	{
    		$new_links = fSearch_Get_All_Links($text, $hash);
	    	foreach ($new_links as $link)
	    	{
	    		// сформировать полную ссылку и проверить ее на корректность
	    		$new_base = $base;
				$result = fSearch_Parse_URL($ss_id, $new_base, $link);
				if ($result && !isset($visited[$link]) && !isset($to_visit[$link]))
				{
					$to_visit[$link] = array('base' => $new_base, 'level' => $level);
				}
	    	}
    	}

    	$row = array();
		$row['sp_url'] = $url;
		$row['sp_flags'] = $code;
		$row['sp_title'] = $title;
		$row['sp_last_modified'] = $tm_page;

		$text = $full_title.' '.$meta_keys.' '.$meta_desc.' '.$text;
    	// вырезаем текст, в котором надо индексировать только ссылки
	   	$text = preg_replace('/<!--\s*sb_index_links_start\s*-->(.*?)<!--\s*sb_index_links_end\s*-->/isu', '', $text);
	   	// HTML-теги заменяем на пробелы
	   	$text = preg_replace('/<[^>]*>/u', ' ', $text);
	   	// переводим в нижний регистр
		$text = sb_strtolower($text);
		$text = str_replace(array('ё'), array('е'), $text);
		// убираем escape-последовательности
	   	$text = sb_html_entity_decode($text);
	    // убираем все символы, которые не могут входить в слова
		$text = preg_replace('/[^a-z'.$GLOBALS['sb_reg_lower_interval'].$GLOBALS['sb_search_settings'][$ss_id]['chars'].'\s]/u', ' ', $text);
	    // убираем повторяющиеся пробелы
	    $text = trim(preg_replace('/\s+/u', ' ', $text));

	    if (trim($text) == '' && $visited[$url] == 0)
	    {
	    	// записываем ошибку
			sql_param_query('INSERT INTO sb_search_pages_'.$ss_id.' SET ?a', $row);
            unset($row);
            $visited[$url] = 1;
			continue;
	    }

	    // контрольная сумма страницы
	    $crc = sb_crc($text);

	    $exists = array_keys($visited, $crc);
	    $visited[$url] = $crc;

		if ($exists && count($exists) > 0)
		{
			// страница с таким содержимым уже была по другому УРЛу
			$found = false;
			// новый УРЛ короче уже существующих?
			foreach ($exists as $old_url)
			{
				if (sb_strlen($old_url) <= sb_strlen($url))
				{
					$found = true;
					break;
				}
			}

			if (!$found)
			{
				// если новый УРЛ короче всех существующих, пишем его в базу
				sql_param_query('UPDATE sb_search_pages_'.$ss_id.' SET sp_url=? WHERE sp_url IN (?a)', $url, $exists);
			}
			continue;
		}

		if ($GLOBALS['sb_search_settings'][$ss_id]['full_index'] != 1)
	   	{
	   		$html_text = $meta_desc;
	   	}

		// запаковать текст в ZIP
		if(function_exists('gzcompress'))
			$row['sp_text'] = gzcompress($html_text, 3);
		else
			$row['sp_text'] = $html_text;

		$row['sp_words_ids'] = fSearch_Process_Words($ss_id, $text);

		sql_param_query('INSERT INTO sb_search_pages_'.$ss_id.' SET ?a', $row);
        unset($row);
        unset($text);
        unset($html_text);
	}
}

/**
 * Возвращает массив ссылок из текста
 *
 * @param string $text Анализируемый текст.
 *
 * @return array Массив ссылок.
 */
function fSearch_Get_All_Links($text, $hash = '')
{
	// не берем ссылки между <!--sb_index_not_links_start--> и <!--sb_index_not_links_end-->
	$text = preg_replace('/<!--\s*sb_index_not_links_start\s*-->\s*(.*?)\s*<!--\s*sb_index_not_links_end\s*-->/siu', ' ',$text);
    // вырезаем идентификатор сессии из ссылок
	$text = preg_replace('/([\?&])'.@session_name().'[=](.){32}/', '\$1', $text);
    // удаляем комментарии
    $text = preg_replace('/<!--.*?-->/su', ' ', $text);

    $links = array();
	$matches = array();

    preg_match_all('/<a[^>]+href\s*=\s*(["\']?)([^"\'>]+)\\1/siu', $text, $matches, PREG_SET_ORDER);
    for($i = 0; $i < count($matches); $i++)
    {
    	$url = trim($matches[$i][2]);
    	$url = preg_replace('/#.*/', '', $url);
    	$url = str_replace(array('&sb_search=1', '&sb_search_hash='.$hash, '?sb_search=1'), '', $url);

    	if ($url != '')
        	$links[] = $url;
    }

    preg_match_all('/<frame[^>]+src\s*=\s*(["\']?)([^"\'>]+)\\1/siu', $text, $matches, PREG_SET_ORDER);
    for($i = 0; $i < count($matches); $i++)
    {
    	$url = trim($matches[$i][2]);
    	$url = preg_replace('/#.*/', '', $url);
    	$url = str_replace(array('&sb_search=1', '&sb_search_hash='.$hash, '?sb_search=1'), '', $url);

    	if ($url != '')
        	$links[] = $matches[$i][2];
    }

	preg_match_all('/<iframe[^>]+src\s*=\s*(["\']?)([^"\'>]+)\\1/siu', $text, $matches, PREG_SET_ORDER);
    for($i = 0; $i < count($matches); $i++)
    {
    	$url = trim($matches[$i][2]);
    	$url = preg_replace('/#.*/', '', $url);
    	$url = str_replace(array('&sb_search=1', '&sb_search_hash='.$hash, '?sb_search=1'), '', $url);

    	if ($url != '')
        	$links[] = $matches[$i][2];
    }

    preg_match_all('/<area[^>]+href\s*=\s*(["\']?)([^"\'>]+)\\1/siu', $text, $matches, PREG_SET_ORDER);
    for($i = 0; $i < count($matches); $i++)
    {
    	$url = trim($matches[$i][2]);
    	$url = preg_replace('/#.*/', '', $url);
    	$url = str_replace(array('&sb_search=1', '&sb_search_hash='.$hash, '?sb_search=1'), '', $url);

    	if ($url != '')
        	$links[] = $matches[$i][2];
    }


    return $links;
}

/**
 * Обработка текста, запись слов в базу
 *
 * @param int $ss_id Идентификатор настроек индексации.
 * @param string $text Обрабатываемый текст.
 *
 * @return string Идентификаторы слов в виде строки, разделенные пробелом.
 */
function fSearch_Process_Words($ss_id, $text)
{
    $text = explode(' ', $text);

    $words = array();
	foreach($text as $word)
	{
		$word = trim($word);
		if ($word == '')
			continue;

		// если длина слова меньше Х - пропускаем его
		if(sb_strlen($word) < $GLOBALS['sb_search_settings'][$ss_id]['min_length'])
			continue;

		// проверить на список запрещенных слов
		if(in_array($word, $GLOBALS['sb_search_settings'][$ss_id]['words']))
			continue;

		// привести слово к "базовой" форме
		$base = $GLOBALS['sb_indexed_morfology']->get_base_form($word);
		if(isset($base[0]))
		{
			$word = $base[0];
			// проверить на список запрещенных слов
			if(in_array($word, $GLOBALS['sb_search_settings'][$ss_id]['words']))
				continue;
		}

		// если длина слова больше - пропускаем его
		if(sb_strlen($word) > 50)
			continue;

		// запоминаем слово
		$words[] = $word;
	}

	// заносим все найденные cлова в базу
	$links = array();
	foreach($words as $word)
	{
		$query = sql_param_query('SELECT sw_id FROM sb_search_words WHERE sw_word = ?', $word);
		if($query)
		{
			list($id) = $query[0];
		}
		else
		{
			if(sql_param_query('INSERT INTO sb_search_words SET sw_word = ?', $word))
			{
				$id = sql_insert_id();
			}
			else
			{
				$id = -1;
			}
		}

		if ($id > 0)
		{
			$len = strlen($id);
			if($len < 4)
			{
				$id = str_repeat('0', 4 - $len).$id;
			}

			$links[] = $id;
		}
	}

	return implode(' ', $links);
}
?>