<?php
/**
 * Выводит или возвращает список чекбоксов для выбора пользователей или групп пользователей.
 *
 * @param string $moderates Содержит данные выбранных пользователей или групп пользователей.
 * @param boolean $return Вывести (FALSE) или вернуть (TRUE) HTML-код списка.
 * @param subcats - выводить (TRUE) или нет (FALSE) чекбокс "Применить права доступа к подразделам"
 *
 * @return string Список чекбоксов для выбора пользователей или групп пользователей.
 */
function fUsers_Get_Groups($moderates = '', $return = false, $subcats = true)
{
	$workflow_status = -1;

	if (isset($_GET['workflow_status']) && isset($_GET['plugin_ident']))
	{
		$workflow_status = intval($_GET['workflow_status']);
		$return = true;
	}

	if ($workflow_status >= 0)
	{
		$res = sql_query('SELECT c.cat_id, c.cat_title, u.u_id, u.u_login, u.u_email, u.u_name, c.cat_left
			FROM sb_catlinks l, sb_users u, sb_categs c LEFT JOIN sb_users_rights r ON c.cat_id=r.ur_cat_id
			WHERE c.cat_ident = ? AND
			(r.ur_plugin=? AND CONCAT(?, r.ur_workflow_rights, ?) LIKE ? OR c.cat_closed = 1)
			AND c.cat_id=l.link_cat_id AND l.link_el_id=u.u_id AND u.u_id != ?d
			ORDER BY c.cat_left, u.u_login', 'pl_users', $_GET['plugin_ident'], '|', '|', '%|'.$workflow_status.'|%', $_SESSION['sbAuth']->getUserId());
	}
	elseif ($return)
	{
		$res = sql_query('SELECT DISTINCT c.cat_id, c.cat_title, u.u_id, u.u_login, u.u_email, u.u_name, c.cat_left
			FROM sb_categs c LEFT JOIN (sb_catlinks l JOIN sb_users u ON l.link_el_id=u.u_id)
				ON c.cat_id=l.link_cat_id
			WHERE c.cat_ident=? AND c.cat_level = 1
			ORDER BY c.cat_left, u.u_login', 'pl_users');
	}
	else
	{
		$res = sql_query('SELECT DISTINCT c.cat_id, c.cat_title, u.u_id, u.u_login, u.u_email, u.u_name, c.cat_left
			FROM sb_categs c LEFT JOIN (sb_catlinks l JOIN sb_users u ON l.link_el_id=u.u_id AND u.u_admin != 1)
				ON c.cat_id=l.link_cat_id
			WHERE c.cat_ident=? AND c.cat_closed != 1 AND c.cat_level = 1
			ORDER BY c.cat_left, u.u_login', 'pl_users');
	}

	if (!$res)
	{
		if (!$return)
		{
			sb_show_message(PL_USERS_H_GET_GROUP_NO_USERS.'<br /><br />[ <a href="#" onclick="sbCloseDialog()">'.KERNEL_CLOSE.'</a> ]', true);
		}
		return '';
	}

	$sel_cat_ids = array();
    $sel_u_ids = array();

	if($moderates != '')
	{
		$moderates = explode('^', $moderates);
	    foreach ($moderates as $right)
	    {
			if ($right[0] == 'g')
			{
				$sel_cat_ids[] = intval(substr($right, 1));
			}
			elseif ($right[0] == 'u')
			{
				$sel_u_ids[] = intval(substr($right, 1));
			}
			else
			{
				$sel_cat_ids[] = intval($right);
			}
		}
	}
    elseif (!$return)
    {
    	$cat_rights = '';

    	if (isset($_GET['cat_id']))
    	{
			$res_rights = sql_query('SELECT cat_rights FROM sb_categs WHERE cat_id=?d', $_GET['cat_id']);
	    	if ($res_rights)
	    	{
				list($cat_rights) = $res_rights[0];
	    	}
    	}
    	elseif (isset($_GET['path']))
    	{
	    	$path = str_replace('\\', '/', preg_replace('/[^\s0-9a-zA-Z\/\\\\_\-\.\[\]]+/', '', $_GET['path']));
			// запрещаем выход на более высокий уровень (..)
		    $path = preg_replace('/\.\.+/', '', $path);
		    $path = preg_replace('/\/\/+/', '', $path);

		    $f_res = sql_query('SELECT f_ids FROM sb_folders_rights WHERE f_path=? AND f_domain=?', $path, SB_COOKIE_DOMAIN);
		    if ($f_res)
		    {
		        $cat_rights = $f_res[0][0];
		    }
    	}

    	if (trim($cat_rights) != '')
    	{
	    	$cat_rights = explode('^', $cat_rights);
		    foreach ($cat_rights as $right)
		    {
		        if (trim($right) == '')
		            continue;

		    	if ($right[0] == 'g')
		    	{
		    		$sel_cat_ids[] = intval(substr($right, 1));
		    	}
		    	elseif ($right[0] == 'u')
		    	{
		    		$sel_u_ids[] = intval(substr($right, 1));
		    	}
		    	else
		    	{
		    		$sel_cat_ids[] = intval($right);
		    	}
		    }
    	}
    }

    ob_start();

	echo '<script>';

	if (!$return)
	{
		echo 'function sbResizeGroupsDiv()
	    {
	        var cl = sbGetE("categs_list_td");
	        var cld = sbGetE("categs_list");
	        var clf = sbGetE("categs_list_footer_td");
	        var clp = document.body;

	        cl.style.height = Math.max(0, parseInt(clp.clientHeight) - (clf ? parseInt(clf.offsetHeight) : 0) - 25);
	        cld.style.height = cl.style.height;
	        cl.style.visibility = "visible";
	        clf.style.visibility = "visible";
	    }
	    sbResizeFuncs[sbResizeFuncs.length]=sbResizeGroupsDiv;';
	}

    echo 'function setChecked(el, cat_id)
    {
    	var div_el = sbGetE("gu_" + cat_id);
    	var chk_els = div_el.getElementsByTagName("input");

    	for (var i = 0; i < chk_els.length; i++)
    	{
    		'.($workflow_status == -1 ? 'chk_els[i].disabled = el.checked;' : '').'
    		chk_els[i].checked = el.checked;
    	}
    }
	</script>';

    if (!$return)
    {
		echo '<form action="'.SB_CMS_MODAL_DIALOG_FILE.'?event=pl_users_set_groups" method="post"'.(browser_is_opera() ? '' : ' target="thisDialog"').'>
		<table cellpadding="0" cellspacing="0" width="100%">
		<tr><td valign="top" id="categs_list_td" style="visibility: hidden;height:100%;"><div id="categs_list" style="overflow-y: auto;border-top: 0px; background-color: #EFE5D0; height: 100%;padding: 7px;">';
    }

    $cat_cur_id = -1;

	foreach ($res as $value)
	{
		list($cat_id, $cat_title, $u_id, $u_login, $u_email, $u_name, ) = $value;
		if ($cat_id != $cat_cur_id)
		{
			if ($cat_cur_id != -1)
			{
				echo '</div>';
			}

			$disabled = in_array($cat_id, $sel_cat_ids);

			echo '<table border="0" cellpadding="2" cellspacing="0">
			<tr>
				<td>
					<img src="'.SB_CMS_IMG_URL.'/tree/folderOpen.gif" width="16" align="absmiddle" height="16" border="0" />
				</td>
				<td valign="middle">
					<input type="checkbox" name="groups[]" id="g_'.$cat_id.'" value="'.$cat_id.'"'.($disabled ? ' checked="checked"' : '').' onclick="setChecked(this, '.$cat_id.')" />
				</td>
				<td>
					<label for="g_'.$cat_id.'" class="smalltext"><b>'.$cat_title.'</b></label>
				</td>
			</tr>
			</table>
			<div id="gu_'.$cat_id.'" style="padding-left: 23px;">';

			$cat_cur_id = $cat_id;
		}

		if (!is_null($u_id))
		{
			echo '<table border="0" cellpadding="2" cellspacing="0">
			<tr>
				<td>
					<img src="'.SB_CMS_IMG_URL.'/plugins/pl_users_16.png" width="16" align="absmiddle" height="16" border="0" />
				</td>
				<td valign="middle">
					<input type="checkbox" name="users[]" id="u_'.$u_id.'" value="'.$u_id.'"'.(in_array($u_id, $sel_u_ids) || $disabled ? ' checked="checked"' : '').($disabled ? ' disabled="disabled"' : '').' />
				</td>
				<td>
					<label for="u_'.$u_id.'" class="smalltext">'.($u_login != '' ? $u_login : $u_email).($u_name != '' ? ' ('.$u_name.')' : '').'</label>
				</td>
			</tr>
			</table>';
		}
	}

	echo '</div>';

	if (!$return)
	{
		echo '</div></td></tr>
		<tr><td><div id="categs_list_footer_td" style="visibility:hidden;">
			<table cellspacing="0" cellpadding="7" width="100%" class="form">';
		if($subcats)
		{
			echo '<tr>
			        <th width="300"><label for="subcategs">'.(isset($_GET['folder']) && $_GET['folder'] == 1 ? PL_USERS_H_SUBFOLDERS : PL_USERS_H_SUBCATEGS).':</label></th>
			        <td><input type="checkbox" id="subcategs" name="subcategs" value="1" /></td>
			    </tr>';
		}
		echo '<tr><td class="footer" colspan="2">
		        <div class="footer">
		            <input type="submit" value="'.KERNEL_CHOOSE.'" />&nbsp;&nbsp;&nbsp;
		            <button onclick="sbCloseDialog();">'.KERNEL_CANCEL.'</button>
		        </div>
		    </td></tr></table>
	    </div></td></tr></table></form>';
	}

	if ($return)
	{
		if (isset($_GET['workflow_status']) && isset($_GET['plugin_ident']))
			ob_end_flush();
		else
			return ob_get_clean();
	}
	else
	{
		ob_end_flush();
	}

	return '';
}
/**
 * Выводит список чекбоксов для выбора пользователей или групп пользователей.
 *
 * @return string Список чекбоксов для выбора пользователей или групп пользователей.
 */
function fUsers_Get_Groups_Print()
{
	echo fUsers_Get_Groups($_GET['sel_ids'], false, false);
}

function fUsers_Set_Groups()
{
	$groups = array();
	$users = array();
	$subcategs = 0;

	extract($_POST);

	$rights = '';

	if (count($groups) > 0)
	{
		$rights = 'g'.implode('^g', $groups);
	}

	if (count($users) > 0)
	{
		$rights .= ($rights != '' ? '^u' : 'u').implode('^u', $users);
	}

	$text = '';
	foreach($groups as $value)
	{
		$res = sql_query('SELECT cat_title FROM sb_categs WHERE cat_id = ?d', $value);
		if($res && $res != '')
		{
			$text .= $res[0][0].', ';
		}
	}
	foreach($users as $value)
	{
		$res = sql_query('SELECT u_login FROM sb_users WHERE u_id = ?d', $value);
		if($res && $res != '')
		{
			$text .= $res[0][0].', ';
		}
	}
	if($text != '')
		$text = sb_substr($text,0, -2);

	echo '<script>
    var res = new Object();
    res.ids = "'.$rights.'";
    res.subcategs = '.intval($subcategs).';
    res.text = "'.$text.'";
    sbReturnValue(res);
	</script>';
}
?>