<?php
/**
 * Функция, удаляющая данные голосования, относящиеся к удаляемому элементу
 *
 * @param int $el_id ID Удаляемого элемента.
 * @param string $plugin_id Уникальный идентификатор модуля.
 */
function fVoting_Delete($el_id, $plugin_id)
{
    if(is_array($el_id))
    {
		sql_query('DELETE sb_vote_ips FROM sb_vote_results, sb_vote_ips
      				 WHERE sb_vote_results.vr_plugin=? AND sb_vote_results.vr_el_id IN (?a) AND sb_vote_ips.vi_vr_id=sb_vote_results.vr_id', $plugin_id, $el_id);

		sql_query('DELETE FROM sb_vote_results WHERE vr_plugin=? AND vr_el_id IN (?a)', $plugin_id, $el_id);
    }
    else
    {
    	sql_query('DELETE sb_vote_ips FROM sb_vote_results, sb_vote_ips
      				 WHERE sb_vote_results.vr_plugin=? AND sb_vote_results.vr_el_id = ?d AND sb_vote_ips.vi_vr_id=sb_vote_results.vr_id', $plugin_id, $el_id);

		sql_query('DELETE FROM sb_vote_results WHERE vr_plugin=? AND vr_el_id = ?d', $plugin_id, $el_id);
    }
}

/**
 * Функция возвращает рейтинг, количество баллов, отданных за элемент и число проголосовавших
 *
 * @param int $el_id ID эелемента.
 * @param string $plugin_ident Уникальный идентификатор модуля.
 *
 * @return array Ассоциативный массив, 'rating' - значение рейтинга, 'sum' - кол-во баллов отданных за элемент,
 * 				 'count' - число проголосовавших.
 */
function fVoting_Rating_Votes ($el_id, $plugin_ident)
{
	$res = sql_query('SELECT vote.vr_count, vote.vr_num, (vote.vr_count / vote.vr_num) AS rating
		                FROM sb_vote_results vote
		                WHERE vote.vr_el_id=?d AND vote.vr_plugin=?', $el_id, $plugin_ident);

	$vote_ar = array();

	if ($res)
    {
    	$vote_ar['rating'] = $res[0][2];
    	$vote_ar['sum'] = $res[0][0];
    	$vote_ar['count'] = $res[0][1];

    	return $vote_ar;
    }

    return $vote_ar;
}

/**
 * Возвращает строку вывода рейтинга элемента в списке элементов
 *
 * @param int $el_id ID эелемента.
 * @param string $plugin_ident Уникальный идентификатор модуля.
 *
 * @return string Строка вывода рейтинга элемента.
 */
function fVoting_Rating_Get ($el_id, $plugin_ident)
{
	$res = fVoting_Rating_Votes ($el_id, $plugin_ident);

	return '<br />'.PL_VOTING_H_GET_RATING.': '.(count($res) > 0 ? sprintf('%.2f', $res['rating']): 0);
}

/**
 * Функция, выводящая поля  "Количество голосов за элемент", "Рейтинг", "Кол-во баллов, отданных за элемент"
 *
 * @param object $layout Объект sbLayout, к которому будут добавлены поля.
 * @param int $el_id Идентификатор элемента.
 * @param string $plugin_ident Идентификатор модуля.
 */
function fVoting_Rating_Edit (&$layout, $el_id, $plugin_ident)
{
	$rating = 0;
	$count = 0;

	if (count($_POST) == 0 && $el_id != '')
	{
		$res = fVoting_Rating_Votes ($el_id, $plugin_ident);

	    if(count($res) > 0)
	    {
	    	$sum = $res['sum'];
	        $count = $res['count'];
	        $rating = sprintf('%.2f', ($count != 0 ? $sum / $count : 0 ));
	    }
	    else
	    {
	        $sum = 0;
	    }
	}
	elseif (count($_POST) > 0)
	{
		$sum = isset($_POST['v_sum']) ? intval($_POST['v_sum']) : 0;
		$count = 1;

		if ($el_id != '')
		{
			$res = fVoting_Rating_Votes ($el_id, $plugin_ident);

			if (count($res) > 0)
			{
				$sum = isset($_POST['v_sum']) ? intval($_POST['v_sum']) : $res['sum'];
				$count = $res['count'];
			}
		}

		$rating = sprintf('%.2f', ($count != 0 ? $sum / $count : 0 ));
	}
	else
	{
	    $sum = 0;
	}

    echo '<script>
    function voting_change_rating()
    {
    	var v_sum = sbGetE("spin_v_sum");
        var v_count = sbGetE("v_count");

        if(v_count.innerHTML != "")
        {
        	var count = v_count.innerHTML != "0" ? parseInt(v_count.innerHTML) : 1;
        	var rating = new String(v_sum.value / count);
        	var pos = rating.indexOf(".");
        	if (pos != -1)
        	{
        		rating = rating.substr(0, pos + 1) + Math.round(rating.substr(pos + 1, 2));
        	}
        	else
        	{
        		rating = rating + ".00";
        	}

        	v_count.innerHTML = count;
        	sbGetE("v_rating").innerHTML = rating;
        }
    }
    </script>';

    $layout->addTab(PL_VOTING_H_GET_RATING);
    $layout->addHeader(PL_VOTING_H_GET_RATING);

    $layout->addField(PL_VOTING_H_EDIT_SUM, new sbLayoutInput('text', $sum, 'v_sum', 'spin_v_sum', 'style="width:60px;" onchange="voting_change_rating();"'));

    $fld = new sbLayoutHTML('<div id="v_count">'.$count.'</div>', false);
    $fld->mShowColon = false;

    $layout->addField(PL_VOTING_H_EDIT_COUNT, $fld);

    $fld = new sbLayoutHTML('<div id="v_rating">'.$rating.'</div>', false);
    $fld->mShowColon = false;
    $layout->addField(PL_VOTING_H_EDIT_RATING, $fld);
}

/**
 * Функция, сохраняющая значения полей "Количество голосов за элемент", "Рейтинг", "Кол-во баллов, отданных за элемент"
 *
 * @param int $el_id Идентификатор элемента.
 * @param string $plugin_ident Идентификатор модуля.
 */
function fVoting_Rating_Edit_Submit ($el_id, $plugin_ident)
{
	$sum = isset($_POST['v_sum']) ? $_POST['v_sum'] : 0;
	if ($sum == 0)
		return;

	$res = sql_query('SELECT COUNT(*) FROM sb_vote_results WHERE vr_el_id=?d AND vr_plugin=?', $el_id, $plugin_ident);

	if(!$res || $res[0][0] <= 0)
	{
		sql_query('INSERT INTO sb_vote_results (vr_plugin, vr_el_id, vr_count, vr_num) VALUES (?, ?d, ?d, 1)', $plugin_ident, $el_id, $sum);
		$id = sql_insert_id();

		$ip = $_SESSION['sbAuth']->getUserIP();
		$exists_ip = sql_query('SELECT vi_vr_id FROM sb_vote_ips ips, sb_vote_results r WHERE
										r.vr_id = ips.vi_vr_id AND r.vr_plugin = ? AND ips.vi_ip = ?', $ip, $plugin_ident);

		if($id && !$exists_ip)
		{
			sql_query('INSERT INTO sb_vote_ips (vi_vr_id, vi_ip, vi_count) VALUES (?, ?, 1)', $id, $ip);
		}
	}
	else
    {
		$res = sql_query('UPDATE sb_vote_results SET vr_count=?d WHERE vr_el_id=?d AND vr_plugin=?', $sum, $el_id, $plugin_ident);
	}
}

/**
 * Функция, выводящая поле выбора макет дизайна голосований
 *
 * @param object $layout Объект sbLayout, к которому будут добавлены поля.
 * @param string $field_value Идентификатор выбранного макета дизайна.
 * @param string $field_name Имя поля, в котором будет храниться идентификатор макета дизайна.
 */
function fVoting_Design_Get (&$layout, $field_value, $field_name)
{
	$res = sql_query('SELECT categs.cat_title, svt.vt_id, svt.vt_title FROM sb_categs categs, sb_catlinks links, sb_vote_temps svt WHERE svt.vt_id=links.link_el_id AND categs.cat_id=links.link_cat_id AND categs.cat_ident=? ORDER BY categs.cat_left, svt.vt_title', 'pl_voting_design');
    if ($res)
    {
    	$options = array();
        $old_cat_title = '';
        foreach ($res as $value)
        {
            list($cat_title, $vt_id, $vt_title) = $value;

            if ($old_cat_title != $cat_title)
            {
                $options[uniqid()] = '-'.$cat_title;
                $old_cat_title = $cat_title;
            }
            $options[$vt_id] = $vt_title;
        }
        $fld = new sbLayoutSelect($options, $field_name, '', '');
        $fld->mSelOptions = array($field_value);
    }
    else
    {
        $fld = new sbLayoutLabel('<div class="hint_div">'.PL_VOTING_H_NO_TEMP_MSG.'</div>', '','', false);
    }

    $layout->addField(PL_VOTING_H_DESIGN_GET, $fld);
}
?>